/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.action;

import java.util.List;

import org.eclipse.core.runtime.Assert;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IActionDelegate2;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.NumberArray;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Action zur Anzeige des Tooltip fr Anzeigen der Anlagendarstellung
 * 
 * @author Pittner
 * 
 */
public class ActionAnzeigeToolTip implements IWorkbenchWindowActionDelegate, IActionDelegate2
{

  private IWorkbenchPart _workbenchPart;

  ClientDavInterface _connection = DaVVerbindung.getInstanz().getDaVVerbindung();

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
  }

  @Override
  public void init(IWorkbenchWindow window)
  {
    _workbenchPart = window.getActivePage().getActivePart();
  }

  @Override
  public void run(IAction action)
  {
  }

  @Override
  public void selectionChanged(IAction action, ISelection selection)
  {
  }

  /**
   * @return
   */
  protected IWorkbenchPart getWorkbenchPart()
  {
    return _workbenchPart;
  }

  /**
   * Gibt den {@link GraphicalViewer} des Workbench Parts zurck.
   * 
   * @return der Viewer oder <code>null</code>, wenn es keinen gibt.
   */
  protected GraphicalViewer getViewer()
  {
    return (GraphicalViewer)getWorkbenchPart().getAdapter(GraphicalViewer.class);
  }

  /**
   * 
   * @param workbenchPart
   */
  protected void setWorkbenchPart(IWorkbenchPart workbenchPart)
  {
    _workbenchPart = workbenchPart;
  }

  /**
   * Gibt den Command Stack zurck.
   * 
   * @return der Command Stack, niemals <code>null</code>.
   */
  protected CommandStack getCommandStack()
  {
    CommandStack stack = getViewer().getEditDomain().getCommandStack();
    Assert.isNotNull(stack, "Command Stack nicht verfgbar.");
    return stack;
  }

  @Override
  public void init(IAction action)
  {
    // TODO Auto-generated method stub

  }

  @Override
  public void runWithEvent(IAction action, Event event)
  {
    GraphicalViewer viewer = getViewer();
    Object obj = event.data;
    
    boolean shift = ((event.stateMask & SWT.SHIFT) > 0);
    
//    System.out.println("stateMask " + event.stateMask);
    
    if (viewer != null && obj != null)
    {
      DoModel doModel = null;
      if (obj instanceof DoModel)
      {
        doModel = (DoModel)obj;

        // if (!(doModel instanceof AnzeigeModelTypD || doModel instanceof AnzeigeModelTypD3))
        {
          SystemObject so = doModel.getSystemObject();

          if (so != null)
          {
            if (_connection != null)
            {
              SystemObject de = deZuAnzeigeBestimmen(so);
              
              DataModel dataModel = _connection.getDataModel();
              IPreferenceStore store = Activator.getDefault().getPreferenceStore();
              String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
              if (pidSba != null && pidSba.length() > 0)
              {
                SystemObject object = dataModel.getObject(pidSba);
                if (object != null)
                {
                  AttributeGroup atg = dataModel.getAttributeGroup("atg.sswSbaSchaltbildAnzeige");

                  AnzeigeQuerschnittModelTypAbstract aq = DaVHelper.holeAnzeigeQuerschnittVonAnzeige(_connection,
                      viewer, doModel);
                  Aspect asp = null;

                  if (aq != null)
                  {
                    // System.out.println("ActionAnzeigeToolTip.runWithEvent(): " + aq);

                    if (aq.getKontext() < 2)
                    {
                      asp = dataModel.getAspect("asp.sswSbaBefehle");
                    }
                    else
                    {
                      asp = dataModel.getAspect("asp.sswSbaSonderprogrammSimu");
                    }

                    ResultData result = _connection.getData(object, new DataDescription(atg, asp), 1000L);
                    if (result.hasData())
                    {
                      Data data = result.getData();
                      Array array = data.getItem("Anzeigen").asArray();
                      for (int i = 0; i < array.getLength(); i++)
                      {
                        Data item = array.getItem(i);
                        SystemObject ref = item.getItem("Anzeige").asReferenceValue().getSystemObject();
                        if (so.equals(ref))
                        {
                          String prio = item.getItem("Prioritt").asTextValue().getValueText();
                          String status = item.getItem("Programmstatus").asTextValue().getValueText();
                          String blinken = item.getItem("Schaltbild").getItem("Blinken").asTextValue().getValueText();
                          String takt = item.getItem("Schaltbild").getItem("Blinktakt").asTextValue().getValueText();
                          SystemObject ref1 = item.getItem("Quelle").asReferenceValue().getSystemObject();
                          SystemObject ref2 = item.getItem("Schaltgrund").asReferenceValue().getSystemObject();

                          String s = "Anzeige: " + ref;
                          s += "\nPrioritt: " + prio;
                          s += "\nProgrammstatus: " + status;
                          s += "\nBlinken: " + blinken;
                          s += "\nBlinktakt: " + takt;
                          s += "\nQuelle: " + ref1;
                          s += "\nSchaltgrund: " + ref2;
                          
                          if ((shift) && (aq.getKontext() == 0)) // Shift bettigt und Ist-Zustand
                          {
                            if (de != null)
                            {                           
                              s += "\n\nDE: " + de;

                              // DE-Fehler
                              
                              AttributeGroup atgGloDeFehler = _connection.getDataModel().getAttributeGroup("atg.tlsGloDeFehler");
                              Aspect aspTlsAntwort = _connection.getDataModel().getAspect("asp.tlsAntwort");
                              
                              if ((atgGloDeFehler != null) && (aspTlsAntwort != null))
                              {
                                DataDescription dd1 = new DataDescription(atgGloDeFehler, aspTlsAntwort);
                                ResultData resultData1 = _connection.getData(de, dd1, 0L);
                              
                                if (resultData1 != null)
                                {
                                  if (resultData1.hasData())
                                  {
                                    if (!resultData1.getData().getScaledValue("DEFehlerStatus").getValueText().equals("ok"))
                                    {
                                      s += "\nDE-Fehler: " + resultData1.getData().getScaledValue("DEFehlerStatus").getValueText() + " Herstellercode: " + 
                                      resultData1.getData().getScaledValue("HerstellerDefinierterCode").getValueText();
                                    }
                                  }
                                  else
                                  {
                                    if (resultData1.isNoSourceAvailable())
                                    {
                                      s += "\nDE-Fehler: keine Quelle";
                                    }
                                    else
                                    {
                                      s += "\nDE-Fehler: keine Daten";
                                    }
                                  }
                                }
                              }

                              // defekte Lampen
                              
                              AttributeGroup atgWzgDefekteLampen = _connection.getDataModel().getAttributeGroup("atg.tlsWzgDefekteLampen");
                              
                              if ((atgWzgDefekteLampen != null) && (aspTlsAntwort != null))
                              {
                                DataDescription dd2 = new DataDescription(atgWzgDefekteLampen, aspTlsAntwort);
                                ResultData resultData2 = _connection.getData(de, dd2, 0L);
                                
                                if (resultData2 != null)
                                {
                                  if (resultData2.hasData())
                                  {
                                    NumberArray arrayZustandLampenPaar = resultData2.getData().getScaledArray("ZustandLampenPaar");
                                    
                                    if (arrayZustandLampenPaar.getLength() > 0)
                                    {
                                      StringBuffer stringBuffer = new StringBuffer("\nDefekte Lampen: ");
                                      
                                      int anzahlDefekteLampen = 0;
                                      
                                      for (int j = 0; j < arrayZustandLampenPaar.getLength(); j++)
                                      {
                                        String valueText = arrayZustandLampenPaar.getValue(j).getValueText();
                                        
                                        if (valueText.equals("Haupt- und Nebenlampe defekt"))
                                        {
                                          if (anzahlDefekteLampen > 0)
                                          {
                                            stringBuffer.append(", ");
                                          }
                                          
                                          anzahlDefekteLampen += 2;
                                          
                                          stringBuffer.append("HL" + (j+1) + ", NL" + (j+1));
                                        }
                                        else if (valueText.equals("Hauptlampe defekt, Nebenlampe ok"))
                                        {
                                          if (anzahlDefekteLampen > 0)
                                          {
                                            stringBuffer.append(", ");
                                          }
                                          
                                          anzahlDefekteLampen++;
                                          
                                          stringBuffer.append("HL" + (j+1));
                                        }
                                        else if (valueText.equals("Hauptlampe ok, Nebenlampe defekt"))
                                        {
                                          if (anzahlDefekteLampen > 0)
                                          {
                                            stringBuffer.append(", ");
                                          }
                                          
                                          anzahlDefekteLampen++;
                                          
                                          stringBuffer.append("NL" + (j+1));
                                        }
                                      }
                                      
                                      if (anzahlDefekteLampen > 0)
                                      {
                                        s += stringBuffer.toString();
                                      }
                                    }
                                  }
                                  else
                                  {
                                    if (resultData2.isNoSourceAvailable())
                                    {
                                      s += "\nDefekte Lampen: keine Quelle";
                                    }
                                    else
                                    {
                                      s += "\nDefekte Lampen: keine Daten";
                                    }
                                  }
                                }
                              }
                              

//                              AttributeGroup atgWzgNichtDarstellbareWVZ = _connection.getDataModel().getAttributeGroup("atg.tlsWzgNichtDarstellbareWVZ");
//                              DataDescription dd3 = new DataDescription(atgWzgNichtDarstellbareWVZ, aspTlsAntwort);
//                              ResultData resultData3 = _connection.getData(de, dd3, 0L);
//
//                              s += "\nNicht darstellbare WVZ: " + resultData3.getData();

                              // gestrte Inhalte
                              
                              AttributeGroup atgAnzeigeEigenschaftIst = _connection.getDataModel().getAttributeGroup("atg.anzeigeEigenschaftIst");
                              Aspect aspAqZustand = _connection.getDataModel().getAspect("asp.aqZustand");
                              
                              if ((atgAnzeigeEigenschaftIst != null) && (aspAqZustand != null))
                              {
                                DataDescription dd3 = new DataDescription(atgAnzeigeEigenschaftIst, aspAqZustand);
                                ResultData resultData3 = _connection.getData(so, dd3, 0L);

                                if (resultData3 != null)
                                {
                                  if (resultData3.hasData())
                                  {
                                    ReferenceArray arrayGestoerteInhalte = resultData3.getData().getReferenceArray("GestrteInhalte");
                                    
                                    if ((arrayGestoerteInhalte != null) && (arrayGestoerteInhalte.getLength() > 0))
                                    {
                                      StringBuffer stringBuffer = new StringBuffer("\nGestrte Inhalte: ");
                                      
                                      for (int j = 0; j < arrayGestoerteInhalte.getLength(); j++)
                                      {
                                        SystemObject wvzInhalt = arrayGestoerteInhalte.getReferenceValue(j).getSystemObject();
                                        
                                        AttributeGroup atgWvzInhalt = dataModel.getAttributeGroup("atg.wvzInhalt");
                                        Data data2 = wvzInhalt.getConfigurationData(atgWvzInhalt);
                                        String text = data2.getTextValue("Bildinhalt").getValueText();
                                        
                                        if (j > 0)
                                        {
                                          stringBuffer.append(", ");
                                        }
                                        
                                        stringBuffer.append(text);
                                      }
                                      
                                      s += stringBuffer.toString();
                                    }
                                  }
                                  else
                                  {
                                    if (resultData3.isNoSourceAvailable())
                                    {
                                      s += "\nGestrte Inhalte: keine Quelle";
                                    }
                                    else
                                    {
                                      s += "\nGestrte Inhalte: keine Daten";
                                    }
                                  }
                                }
                              }
                            }
                          }

                          EStructuralFeature feature = AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE;
                          final SetCommand cmd = new SetCommand(doModel, feature, s);
                          getCommandStack().execute(cmd);

                          break;
                        }
                      }
                    }
                  }
                }
              }
            }
          }

        }
        // else{
        // EStructuralFeature feature =
        // AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE;
        // final SetCommand cmd = new SetCommand(doModel, feature, "");
        // getCommandStack().execute(cmd);
        // }
      }
    }
  }

  private SystemObject deZuAnzeigeBestimmen(SystemObject so)
  {
    DataModel dataModel = _connection.getDataModel();
   
    SystemObjectType typAnzeigeAktorTls = dataModel.getType("typ.anzeigeAktorTls");
    
    if (typAnzeigeAktorTls != null)
    {
      List<SystemObject> listeAnzeigeAktorTls = typAnzeigeAktorTls.getElements();
      
      AttributeGroup atgAnzeigeAktor = dataModel.getAttributeGroup("atg.anzeigeAktor");
      AttributeGroup atgAnzeigeAktorTls = dataModel.getAttributeGroup("atg.anzeigeAktorTls");
      
      if ((listeAnzeigeAktorTls != null) && (atgAnzeigeAktor != null) &&(atgAnzeigeAktorTls != null))
      {
        dataModel.getConfigurationData(listeAnzeigeAktorTls, atgAnzeigeAktor);
        dataModel.getConfigurationData(listeAnzeigeAktorTls, atgAnzeigeAktorTls);
        
        for (SystemObject anzeigeAktorTls : listeAnzeigeAktorTls)
        {
          Data dataAnzeigeAktor = anzeigeAktorTls.getConfigurationData(atgAnzeigeAktor);
          
          SystemObject anzeigeTmp = dataAnzeigeAktor.getReferenceValue("AnzeigeReferenz").getSystemObject();
          
          if (anzeigeTmp.equals(so)) // AnzeigeAktor zu Anzeige gefunden
          {
            Data dataAnzeigeAktorTls = anzeigeAktorTls.getConfigurationData(atgAnzeigeAktorTls);
            
            if (dataAnzeigeAktorTls != null)
            {
              ReferenceArray arrayDEs = dataAnzeigeAktorTls.getReferenceArray("ZugehrigeDEs");
              
              if ((arrayDEs != null) && (arrayDEs.getLength() > 0))
              {
                return arrayDEs.getReferenceValue(0).getSystemObject(); 
              }
            }
            
            break;
          }
        }
      }
    }
    
    return null;
  }
}
