/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.commands;

import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PrecisionPoint;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.requests.ReconnectRequest;

import de.bsvrz.buv.plugin.anlagendarstellung.model.LinkElement;

/**
 * 
 * Command for setting the end points. It is executed during the layout
 * 
 * @see LinkElement
 * 
 */
public class ReconnectEndpointCommand extends Command
{

  private Point location;

  private NodeEditPart node;

  private ConnectionEditPart connectionPart;

  private String oldId;

  private String newId;

  private boolean sourceReconnect;

  public ReconnectEndpointCommand(boolean sourceReconnect)
  {
    this.sourceReconnect = sourceReconnect;
  }

  /**
   * 
   * @param location
   *          Mouse location in local coordinates
   * 
   */
  public void setLocation(Point location)
  {
    this.location = location;
  }

  /**
   * 
   * @param node
   *          the node of the endpoint
   * 
   */
  public void setNode(NodeEditPart node)
  {
    this.node = node;
  }

  public void setConnectionEditPart(ConnectionEditPart connection)
  {
    this.connectionPart = connection;
  }

  public void execute()
  {   
    LinkElement link = (LinkElement)connectionPart.getModel();
    if (sourceReconnect)
    {
      oldId = link.getSourceAnchorOffset();
      if (location == null)
      {
        newId = null;
      }
      else
      {
        ReconnectRequest request = new ReconnectRequest(RequestConstants.REQ_RECONNECT_SOURCE);
        PrecisionPoint pLocation = new PrecisionPoint(location);
        Connection connection = (Connection)connectionPart.getFigure();
        connection.getSourceAnchor().getOwner().translateToAbsolute(pLocation);
        request.setLocation(pLocation);
        request.setTargetEditPart(node);
        request.setConnectionEditPart(connectionPart);       
      }
      link.setSourceAnchorOffset(newId);
    }
    else
    {
      oldId = link.getTargetAnchorOffset();
      if (location == null)
      {
        newId = null;
      }
      else
      {
        ReconnectRequest request = new ReconnectRequest(RequestConstants.REQ_RECONNECT_TARGET);
        PrecisionPoint pLocation = new PrecisionPoint(location);
        Connection connection = (Connection)connectionPart.getFigure();
        connection.getTargetAnchor().getOwner().translateToAbsolute(pLocation);
        request.setLocation(pLocation);
        request.setTargetEditPart(node);
        request.setConnectionEditPart(connectionPart);       
      }
      link.setTargetAnchorOffset(newId);
    }

  }

  public void redo()
  {    
    LinkElement link = (LinkElement)connectionPart.getModel();
    if (sourceReconnect)
      link.setSourceAnchorOffset(newId);
    else
      link.setTargetAnchorOffset(newId);
  }

  public void undo()
  {
    LinkElement link = (LinkElement)connectionPart.getModel();
    if (sourceReconnect)
      link.setSourceAnchorOffset(oldId);
    else
      link.setTargetAnchorOffset(oldId);

  }
}
