/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dav;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.semi.DialogSemiGuiManager;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.Fg6Helper;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.KmdParameter;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.Kommandozeilenparameter;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Zentrale Klasse. Dient der Kommunikation mit dem DaV bzgl. der Anlagendarstellung
 * 
 * @author Pittner
 * 
 */
public class DaVKommunikationDobj implements ClientReceiverInterface, MutableSetChangeListener
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  private ClientDavInterface _connection = null;

  /**
   * Singleton-Instanz
   */
  private static DaVKommunikationDobj _instanz = null;

  /**
   * Liefert die Singletoninstanz.
   * 
   * @return Instanz dieser Klasse.
   */
  public static DaVKommunikationDobj getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new DaVKommunikationDobj();
    }
    return _instanz;
  }

  /**
   * Konstruktor, setzt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  private DaVKommunikationDobj()
  {

    this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern gehrt werden.
     */
    IDaVVerbindungZuhoerer zuhoerer = new IDaVVerbindungZuhoerer() {
      public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        _connection = null;
      }

      public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
        // Anmelden, wenn die Verbindung aufgebaut wurde
        _connection = e.getDaVVerbindung();
      }

    };
  }

  /**
   * Abmeldung von den Daten fr die Anlagendarstellung
   */
  public void abmeldenAufDaten(List<SystemObject> list)
  {
    List<SystemObject> listAQ = new ArrayList<SystemObject>();
    List<SystemObject> listAE = new ArrayList<SystemObject>();
    List<SystemObject> listFS = new ArrayList<SystemObject>();
    List<SystemObject> listFS2 = new ArrayList<SystemObject>();
    List<SystemObject> listDE = new ArrayList<SystemObject>();
    List<SystemObject> listSM_DE = new ArrayList<SystemObject>();
    List<SystemObject> listMQ = new ArrayList<SystemObject>();
    List<SystemObject> listUfdFBZ = new ArrayList<SystemObject>();
    List<SystemObject> listUfdHK = new ArrayList<SystemObject>();
    List<SystemObject> listUfdSW = new ArrayList<SystemObject>();
    List<SystemObject> listUfdSbaMs = new ArrayList<SystemObject>();
    List<SystemObject> listUfdMs = new ArrayList<SystemObject>();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");

    for (SystemObject so : list)
    {
      if (so != null)
      {
        if (so.isOfType("typ.anzeigeQuerschnitt"))
        {
          listAQ.add(so);
        }
        else if (so.isOfType("typ.anzeige"))
        {
          listAE.add(so);
        }
        else if (so.isOfType("typ.fahrStreifen"))
        {
          listFS.add(so);
        }
        else if (so.isOfType("typ.de"))
        {
          listDE.add(so);
        }
        else if (so.isOfType("typ.steuerModul"))
        {

          ConfigurationObject co = (ConfigurationObject)so;
          NonMutableSet set = co.getNonMutableSet("Eak");
          if (set != null)
          {
            for (SystemObject so2 : set.getElements())
            {
              ConfigurationObject co2 = (ConfigurationObject)so2;
              NonMutableSet set2 = co2.getNonMutableSet("De");
              if (set2 != null)
              {
                for (SystemObject so3 : set2.getElements())
                {
                  if (so3.isOfType("typ.deVlt"))
                  {
                    Data data = so3.getConfigurationData(atg);
                    int deKanal = data.getItem("DEKanal").asUnscaledValue().intValue();
                    if (deKanal != 255)
                    {
                      if (!listSM_DE.contains(so3))
                        listSM_DE.add(so3);
                    }
                  }
                }
              }
            }
          }

        }
        else if (so.isOfType("typ.messQuerschnittAllgemein"))
        {
          listMQ.add(so);
          ConfigurationObject co = (ConfigurationObject)so;
          NonMutableSet set = co.getNonMutableSet("FahrStreifen");
          listFS2.addAll(set.getElements());
        }
        else if (so.isOfType("typ.ufdsFahrBahnOberFlchenZustand"))
        {
          listUfdFBZ.add(so);
        }
        else if (so.isOfType("typ.ufdsHelligkeit"))
        {
          listUfdHK.add(so);
        }
        else if (so.isOfType("typ.ufdsSichtWeite"))
        {
          listUfdSW.add(so);
        }
        else if (so.isOfType("typ.sswSbaUmfeldDatenMessStelle"))
        {
          listUfdSbaMs.add(so);
        }
        else if (so.isOfType("typ.umfeldDatenMessStelle"))
        {
          listUfdMs.add(so);
        }
        else
        {
          // System.out.println("else: " + so);
        }

      }
      else
      {
        // System.out.println("SO ist null: " + i);
      }

    }

    abmeldeLeseDaV(listAE, "atg.anzeigeEigenschaftIst", "asp.aqZustand");
    abmeldeLeseDaV(listAQ, "atg.anzeigeQuerschnittEigenschaftIst", "asp.aqZustand");

    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    if (pidSba != null && pidSba.length() > 0)
    {
      SystemObject object = _connection.getDataModel().getObject(pidSba);
      if (object != null)
      {
        List<SystemObject> listAnlage = new ArrayList<SystemObject>();
        listAnlage.add(object);
        abmeldeLeseDaV(listAnlage, "atg.sswSbaSchaltbildAnzeige", "asp.sswSbaBefehle");
      }
    }

    abmeldeLeseDaV(listFS, "atg.strfallZustand", "asp.strfallVerfahrenMARZ");
    abmeldeLeseDaV(listFS, "atg.passivierungFs", "asp.zustand");
    abmeldeLeseDaV(listDE, "atg.tlsGloDeFehler", "asp.tlsAntwort");

    abmeldeLeseDaV(listMQ, "atg.verkehrsDatenKurzZeitMq", "asp.analyse");
    abmeldeLeseDaV(listMQ, "atg.strfallZustand", "asp.strfallVerfahrenMARZ");
    abmeldeLeseDaV(listFS2, "atg.verkehrsDatenKurzZeitFs", "asp.analyse");

    abmeldeLeseDaV(listUfdFBZ, "atg.ufdsStufeFahrBahnOberFlchenZustand", "asp.klassifizierung");
    abmeldeLeseDaV(listUfdFBZ, "atg.ufdsFahrBahnOberFlchenZustand", "asp.externeErfassung");
    abmeldeLeseDaV(listUfdHK, "atg.ufdsStufeHelligkeit", "asp.klassifizierung");
    abmeldeLeseDaV(listUfdHK, "atg.ufdsHelligkeit", "asp.externeErfassung");
    abmeldeLeseDaV(listUfdSW, "atg.ufdsStufeSichtWeite", "asp.klassifizierung");
    abmeldeLeseDaV(listUfdSW, "atg.ufdsSichtWeite", "asp.externeErfassung");
    abmeldeLeseDaV(listUfdFBZ, "atg.ufdsStufeFahrBahnOberFlchenZustand", "asp.klassifizierung");
    abmeldeLeseDaV(listUfdSbaMs, "atg.sswSbaUfdmsNsseStufe", "asp.klassifizierung");
    abmeldeLeseDaV(listUfdMs, "atg.ufdmsNsseStufe", "asp.klassifizierung");

    for (String fg6Atg : Fg6Helper.FG6_ATG)
    {
      abmeldeLeseDaV(listSM_DE, fg6Atg, Fg6Helper.FG6_ASP);
    }

    SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaAnzeigeQuerschnitt");

    if (type != null)
    {
      List<String> listSbaAq = new ArrayList<String>();
      for (SystemObject so : type.getElements())
      {
        listSbaAq.add(so.getPid());
      }
      abmeldeLeseDaV(type.getElements(), "atg.sswSbaHandeingriffBetriebsart", "asp.sswSbaHandschaltungVorgabe");
      abmeldeLeseDaV(type.getElements(), "atg.sswSbaHelligkeitProzentErgebnis", "asp.zustand");
    }

  }

  /**
   * Anmeldung auf die Daten vom DAV fr die Sicht Zustandsdarstellung
   */
  public void anmeldenAufDaten(List<SystemObject> list)
  {
    List<SystemObject> listAQ = new ArrayList<SystemObject>();
    List<SystemObject> listAE = new ArrayList<SystemObject>();
    List<SystemObject> listFS = new ArrayList<SystemObject>();
    List<SystemObject> listFS2 = new ArrayList<SystemObject>();
    List<SystemObject> listDE = new ArrayList<SystemObject>();
    List<SystemObject> listSM_DE = new ArrayList<SystemObject>();
    List<SystemObject> listMQ = new ArrayList<SystemObject>();
    List<SystemObject> listUfdFBZ = new ArrayList<SystemObject>();
    List<SystemObject> listUfdHK = new ArrayList<SystemObject>();
    List<SystemObject> listUfdSW = new ArrayList<SystemObject>();
    List<SystemObject> listUfdSbaMs = new ArrayList<SystemObject>();
    List<SystemObject> listUfdMs = new ArrayList<SystemObject>();

    /*
     * ZLT LSAs
     */
    List<SystemObject> listZltLsaSperranlage = new ArrayList<SystemObject>();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.de");

    for (SystemObject so : list)
    {
      if (so != null)
      {
        if (so.isOfType("typ.anzeigeQuerschnitt"))
        {
          listAQ.add(so);
        }
        else if (so.isOfType("typ.anzeige"))
        {
          listAE.add(so);
        }
        else if (so.isOfType("typ.fahrStreifen"))
        {
          listFS.add(so);
        }
        else if (so.isOfType("typ.de"))
        {
          listDE.add(so);
        }
        else if (so.isOfType("typ.steuerModul"))
        {

          ConfigurationObject co = (ConfigurationObject)so;
          NonMutableSet set = co.getNonMutableSet("Eak");
          if (set != null)
          {
            for (SystemObject so2 : set.getElements())
            {
              ConfigurationObject co2 = (ConfigurationObject)so2;
              NonMutableSet set2 = co2.getNonMutableSet("De");
              if (set2 != null)
              {
                for (SystemObject so3 : set2.getElements())
                {
                  if (so3.isOfType("typ.deVlt"))
                  {
                    Data data = so3.getConfigurationData(atg);
                    int deKanal = data.getItem("DEKanal").asUnscaledValue().intValue();
                    if (deKanal != 255)
                    {
                      if (!listSM_DE.contains(so3))
                        listSM_DE.add(so3);
                    }
                  }
                }
              }
            }
          }

        }
        else if (so.isOfType("typ.messQuerschnittAllgemein"))
        {
          listMQ.add(so);
          ConfigurationObject co = (ConfigurationObject)so;
          NonMutableSet set = co.getNonMutableSet("FahrStreifen");
          listFS2.addAll(set.getElements());
        }
        else if (so.isOfType("typ.ufdsFahrBahnOberFlchenZustand"))
        {
          listUfdFBZ.add(so);
        }
        else if (so.isOfType("typ.ufdsHelligkeit"))
        {
          listUfdHK.add(so);
        }
        else if (so.isOfType("typ.ufdsSichtWeite"))
        {
          listUfdSW.add(so);
        }
        else if (so.isOfType("typ.sswSbaUmfeldDatenMessStelle"))
        {
          listUfdSbaMs.add(so);
        }
        else if (so.isOfType("typ.umfeldDatenMessStelle"))
        {
          listUfdMs.add(so);
        }
        else
        {
          // System.out.println("else: " + so);
        }
        
        
        if (so.isOfType("typ.deZltLsaSperranlage"))
        {
          /*
           * ZLT LSA
           */
          listZltLsaSperranlage.add(so);
        }

      }
      else
      {
        // System.out.println("SO ist null: " + i);
      }

    }
    anmeldeLeseDaV(listAE, "atg.anzeigeEigenschaftIst", "asp.aqZustand", ReceiveOptions.delta(), ReceiverRole
        .receiver());
    anmeldeLeseDaV(listAQ, "atg.anzeigeQuerschnittEigenschaftIst", "asp.aqZustand", ReceiveOptions.delta(),
        ReceiverRole.receiver());
    // anmeldeLeseDaV(listAE, "atg.anzeigeEigenschaftSoll", "asp.aqVorgabe", ReceiverRole.receiver());

    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    if (pidSba != null && pidSba.length() > 0)
    {
      SystemObject object = _connection.getDataModel().getObject(pidSba);
      if (object != null)
      {
        List<SystemObject> listAnlage = new ArrayList<SystemObject>();
        listAnlage.add(object);
        anmeldeLeseDaV(listAnlage, "atg.sswSbaSchaltbildAnzeige", "asp.sswSbaBefehle", ReceiveOptions.delta(),
            ReceiverRole.receiver());
      }
    }

    anmeldeLeseDaV(listFS, "atg.strfallZustand", "asp.strfallVerfahrenMARZ", ReceiveOptions.delta(), ReceiverRole
        .receiver());
    anmeldeLeseDaV(listFS, "atg.passivierungFs", "asp.zustand", ReceiveOptions.delta(), ReceiverRole.receiver());
    anmeldeLeseDaV(listDE, "atg.tlsGloDeFehler", "asp.tlsAntwort", ReceiveOptions.delta(), ReceiverRole.receiver());

    anmeldeLeseDaV(listMQ, "atg.verkehrsDatenKurzZeitMq", "asp.analyse", ReceiveOptions.delta(), ReceiverRole
        .receiver());
    anmeldeLeseDaV(listMQ, "atg.strfallZustand", "asp.strfallVerfahrenMARZ", ReceiveOptions.delta(), ReceiverRole
        .receiver());
    anmeldeLeseDaV(listFS2, "atg.verkehrsDatenKurzZeitFs", "asp.analyse", ReceiveOptions.delta(), ReceiverRole
        .receiver());

    anmeldeLeseDaV(listUfdFBZ, "atg.ufdsStufeFahrBahnOberFlchenZustand", "asp.klassifizierung",
        ReceiveOptions.delta(), ReceiverRole.receiver());
    anmeldeLeseDaV(listUfdFBZ, "atg.ufdsFahrBahnOberFlchenZustand", "asp.externeErfassung", ReceiveOptions.delta(),
        ReceiverRole.receiver());
    anmeldeLeseDaV(listUfdHK, "atg.ufdsStufeHelligkeit", "asp.klassifizierung", ReceiveOptions.delta(), ReceiverRole
        .receiver());
    anmeldeLeseDaV(listUfdHK, "atg.ufdsHelligkeit", "asp.externeErfassung", ReceiveOptions.delta(), ReceiverRole
        .receiver());
    anmeldeLeseDaV(listUfdSW, "atg.ufdsStufeSichtWeite", "asp.klassifizierung", ReceiveOptions.delta(), ReceiverRole
        .receiver());
    anmeldeLeseDaV(listUfdSW, "atg.ufdsSichtWeite", "asp.externeErfassung", ReceiveOptions.delta(), ReceiverRole
        .receiver());
    anmeldeLeseDaV(listUfdFBZ, "atg.ufdsStufeFahrBahnOberFlchenZustand", "asp.klassifizierung",
        ReceiveOptions.delta(), ReceiverRole.receiver());
    anmeldeLeseDaV(listUfdSbaMs, "atg.sswSbaUfdmsNsseStufe", "asp.klassifizierung", ReceiveOptions.delta(),
        ReceiverRole.receiver());
    anmeldeLeseDaV(listUfdMs, "atg.ufdmsNsseStufe", "asp.klassifizierung", ReceiveOptions.delta(), ReceiverRole
        .receiver());

    for (String fg6Atg : Fg6Helper.FG6_ATG)
    {
      anmeldeLeseDaV(listSM_DE, fg6Atg, Fg6Helper.FG6_ASP, ReceiveOptions.delta(), ReceiverRole.receiver());
    }

    SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaAnzeigeQuerschnitt");

    if (type != null)
    {
      List<String> listSbaAq = new ArrayList<String>();
      for (SystemObject so : type.getElements())
      {
        listSbaAq.add(so.getPid());
      }
      anmeldeLeseDaV(type.getElements(), "atg.sswSbaHandeingriffBetriebsart", "asp.sswSbaHandschaltungVorgabe",
          ReceiveOptions.delta(), ReceiverRole.receiver());
      anmeldeLeseDaV(type.getElements(), "atg.sswSbaHelligkeitProzentErgebnis", "asp.zustand", ReceiveOptions.delta(),
          ReceiverRole.receiver());
    }

    /*
     * Anmelden auf Daten fr ZLT LSA.
     */
    if (listZltLsaSperranlage.size() > 0) 
    {
      anmeldeLeseDaV(listZltLsaSperranlage, "atg.tlsZltLsaSperranlage", "asp.tlsZustand", ReceiveOptions.delta(),
          ReceiverRole.receiver());
    }
  }

  public void anmeldenAufDatenSm(List<String> pids, String atg, String asp)
  {
    anmeldeStringsLeseDaV(pids, atg, asp, ReceiverRole.receiver());
  }
  
  public void anmeldenAufDatenPicker(List<String> pids, String atg, String asp)
  {
    
    System.out.println("DaVKommunikationDobj.anmeldenAufDatenPicker(!!!)");
    
    anmeldeStringsLeseDaV(pids, atg, asp, ReceiverRole.receiver());
    
    
    KmdParameter kommandoZeilenParameter = Kommandozeilenparameter
    .getInstanz().getKommandoZeilenParameter("-verkehrsModellObjekt");

    if (kommandoZeilenParameter != null)
    {
      
      String pidVerkehrsModellObjekt = kommandoZeilenParameter.getWert();
      
      if (pidVerkehrsModellObjekt != null && pidVerkehrsModellObjekt.length() > 0)
      {
        SystemObject object = _connection.getDataModel().getObject(pidVerkehrsModellObjekt);
        ConfigurationObject ko = (ConfigurationObject)object;
        if (ko != null)
        {
          MutableSet ms1 = ko.getMutableSet("Baustellen");
          
          if (ms1 != null)
          {
            ms1.addChangeListener(this);
          }
          MutableSet ms2 = ko.getMutableSet("Unflle");
          
          if (ms2 != null)
          {
            ms2.addChangeListener(this);
          }
          MutableSet ms3 = ko.getMutableSet("Staus");
          
          if (ms3 != null)
          {
            ms3.addChangeListener(this);
          }
        }
      }
    }
  }

  /**
   * Anmelden auf die Daten die von der Sicht SonderProgramme bentigt werden
   */
  public void anmeldenAufDatenSonder()
  {
    SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaUeSonderprogramm");
    List<SystemObject> list = type.getElements();

    List<String> listSPrg = new ArrayList<String>();

    for (SystemObject so : list)
    {
      DynamicObject dyn = (DynamicObject)so;
      if (dyn.isValid())
        listSPrg.add(so.getPid());
    }

    anmeldeStringsLeseDaV(listSPrg, "atg.sswSbaSonderprogramm", "asp.parameterSoll", ReceiverRole.receiver());

    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    if (pidSba != null && pidSba.length() > 0)
    {
      SystemObject object = _connection.getDataModel().getObject(pidSba);
      if (object != null)
      {
        List<String> listAnlage = new ArrayList<String>();
        listAnlage.add(object.getPid());
        anmeldeStringsLeseDaV(listAnlage, "atg.sswSbaManahmeVorschlagAntwort", "asp.sswSbaAntwort", ReceiverRole
            .receiver());

        ConfigurationObject ko = (ConfigurationObject)object;
        if (ko != null)
        {
          MutableSet ms = ko.getMutableSet("UrsachenEinheiten");

          if (ms != null)
          {
            ms.addChangeListener(this);
          }
        }

        anmeldeStringsLeseDaV(listAnlage, "atg.sswSbaSchaltbildAnzeige", "asp.sswSbaSonderprogrammSimu", ReceiveOptions
            .delta(), ReceiverRole.receiver());
      }
    }

    type = _connection.getDataModel().getType("typ.sswSbaAnzeigeQuerschnitt");
    list = type.getElements();

    List<String> listSbaAq = new ArrayList<String>();
    for (SystemObject so : list)
    {
      listSbaAq.add(so.getPid());
    }
    anmeldeStringsLeseDaV(listSbaAq, "atg.sswSbaHandeingriffBetriebsart", "asp.sswSbaHandschaltungVorgabe",
        ReceiveOptions.delta(), ReceiverRole.receiver());
    anmeldeStringsLeseDaV(listSbaAq, "atg.sswSbaHelligkeitProzentErgebnis", "asp.zustand", ReceiveOptions.delta(),
        ReceiverRole.receiver());

    type = _connection.getDataModel().getType("typ.sswSbaStreckenBeeinflussungsAnlageRichtung");
    list = type.getElements();

    List<String> listSbaRichtung = new ArrayList<String>();
    for (SystemObject so : list)
    {
      listSbaRichtung.add(so.getPid());
    }
    anmeldeStringsLeseDaV(listSbaRichtung, "atg.sswSbaWanderbaustelle", "asp.parameterSoll", ReceiverRole.receiver());
    anmeldeStringsLeseDaV(listSbaRichtung, "atg.sswSbaSonderprogrammHierarchie", "asp.parameterSoll", ReceiverRole
        .receiver());

  }

  /**
   * Setzt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public void setzeDaVVerbindung(ClientDavInterface connnection)
  {
    this._connection = connnection;
  }

  /**
   * Holt die Verbindung zum DaV.
   * 
   * @param connnection
   *          Verbindung zum DaV.
   */
  public ClientDavInterface holeDaVVerbindung()
  {
    return _connection;
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(Long objId, String atgPid, String aspPid)
  {
    if (objId == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objId);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, obj, dd);
  }

  /**
   * Methode die das Objekt objId beim Datenverteiler abmeldet zum Lesen der Attibutgruppe atgPid unter dem Aspekt
   * aspPid.
   * 
   * @param objId
   *          Id des Objekts
   * @param atgPid
   *          Attributgruppe die abgemeldet werden soll
   * @param aspPid
   *          Apekt der abgemeldet werden soll
   */
  private void abmeldeLeseDaV(List objs, String atgPid, String aspPid)
  {
    if (objs == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.unsubscribeReceiver(this, objs, dd);
  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(String objPid, String atgPid, String aspPid, ReceiverRole role)
  {
    if (objPid == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    SystemObject obj = _connection.getDataModel().getObject(objPid);
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, obj, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(List objs, String atgPid, String aspPid, ReceiveOptions options, ReceiverRole role)
  {

    if (objs == null || atgPid == null || aspPid == null || options == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, objs, dd, options, role);

  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeLeseDaV(List objs, String atgPid, String aspPid, ReceiverRole role)
  {

    if (objs == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    _connection.subscribeReceiver(this, objs, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeStringsLeseDaV(List<String> objs, String atgPid, String aspPid, ReceiverRole role)
  {

    if (objs == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    List<SystemObject> sos = new ArrayList<SystemObject>();

    for (String s : objs)
    {
      SystemObject so = _connection.getDataModel().getObject(s);

      if (so != null)
        sos.add(so);
      else
      {
        // System.out.println(s + " : nicht in der Konfiguration vorhanden");
      }
    }

    // if (atgPid.equals("atg.tlsLveErgebnisMeldungVersion0Bis4"))
    // System.out.println("so: " + sos.get(0) + " atg: " + atg + " asp: " + asp);

    _connection.subscribeReceiver(this, sos, dd, ReceiveOptions.normal(), role);

  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void anmeldeStringsLeseDaV(List<String> objs, String atgPid, String aspPid, ReceiveOptions options,
      ReceiverRole role)
  {

    if (objs == null || atgPid == null || aspPid == null || role == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    List<SystemObject> sos = new ArrayList<SystemObject>();

    for (String s : objs)
    {
      SystemObject so = _connection.getDataModel().getObject(s);

      if (so != null)
        sos.add(so);
      else
      {
        // System.out.println(s + " : nicht in der Konfiguration vorhanden");
      }
    }

    // if (atgPid.equals("atg.tlsLveErgebnisMeldungVersion0Bis4"))
    // System.out.println("so: " + sos.get(0) + " atg: " + atg + " asp: " + asp);

    _connection.subscribeReceiver(this, sos, dd, options, role);

  }

  /**
   * Methode die das Objekt mit der PID objPid beim Datenverteiler anmeldet zum Lesen der Attibutgruppe atgPid unter dem
   * Aspekt aspPid.
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   * @param role
   *          Rolle des Empfngers (siehe stauma.dav.clientside.ReceiveOptions)
   */
  private void abmeldeStringsLeseDaV(List<String> objs, String atgPid, String aspPid)
  {

    if (objs == null || atgPid == null || aspPid == null)
      throw new IllegalArgumentException();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(atgPid);
    Aspect asp = _connection.getDataModel().getAspect(aspPid);
    DataDescription dd = new DataDescription(atg, asp);

    List<SystemObject> sos = new ArrayList<SystemObject>();

    for (String s : objs)
    {
      SystemObject so = _connection.getDataModel().getObject(s);

      if (so != null)
        sos.add(so);
      else
      {
        // System.out.println(s + " : nicht in der Konfiguration vorhanden");
      }
    }

    _connection.unsubscribeReceiver(this, sos, dd);
  }

  public void anmeldenAufDatenMQDaten(List<String> pids, String atg, String asp)
  {
    anmeldeStringsLeseDaV(pids, atg, asp, ReceiverRole.receiver());
  }

  /**
   * Verarbeitet smtliche Updates, hier wird entschieden welche Daten weiterverarbeitet werden.
   */
  public void update(ResultData[] results)
  {
    for (int i = 0; i < results.length; i++)
    {
      final ResultData result = results[i];

      if (result != null)
      {
        AttributeGroup atg = result.getDataDescription().getAttributeGroup();

        if (atg.getPid().equals("atg.anzeigeQuerschnittEigenschaftIst")
            || atg.getPid().equals("atg.anzeigeQuerschnittEigenschaftSoll"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.anzeigeEigenschaftIst"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.anzeigeEigenschaftSoll"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.strfallZustand"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.passivierungFs"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.berwachungsStatus"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.verkehrsDatenKurzZeitMq"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.verkehrsDatenKurzZeitFs"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.tlsGloDeFehler"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.tlsLveErgebnisMeldungVersion0Bis4"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.tlsLveBetriebsParameter"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (Arrays.asList(Fg6Helper.FG6_ATG).contains(atg.getPid()))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {

              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.ufdsFahrBahnFeuchte") || atg.getPid().equals("atg.ufdsFahrBahnGltte")
            || atg.getPid().equals("atg.ufdsFahrBahnOberFlchenTemperatur")
            || atg.getPid().equals("atg.ufdsFahrBahnOberFlchenZustand")
            || atg.getPid().equals("atg.ufdsGefrierTemperatur") || atg.getPid().equals("atg.ufdsHelligkeit")
            || atg.getPid().equals("atg.ufdsLuftDruck") || atg.getPid().equals("atg.ufdsLuftTemperatur")
            || atg.getPid().equals("atg.ufdsNiederschlagsArt")
            || atg.getPid().equals("atg.ufdsNiederschlagsIntensitt")
            || atg.getPid().equals("atg.ufdsNiederschlagsMenge") || atg.getPid().equals("atg.ufdsRelativeLuftFeuchte")
            || atg.getPid().equals("atg.ufdsRestSalz") || atg.getPid().equals("atg.ufdsSchneeHhe")
            || atg.getPid().equals("atg.ufdsSichtWeite") || atg.getPid().equals("atg.ufdsTaupunktTemperatur")
            || atg.getPid().equals("atg.ufdsTemperaturInTiefe1") || atg.getPid().equals("atg.ufdsTemperaturInTiefe2")
            || atg.getPid().equals("atg.ufdsTemperaturInTiefe3") || atg.getPid().equals("atg.ufdsWasserFilmDicke")
            || atg.getPid().equals("atg.ufdsWindGeschwindigkeitMittelWert")
            || atg.getPid().equals("atg.ufdsWindGeschwindigkeitSpitzenWert")
            || atg.getPid().equals("atg.ufdsWindRichtung") || atg.getPid().equals("atg.ufdsStufeFahrBahnFeuchte")
            || atg.getPid().equals("atg.ufdsStufeFahrBahnGltte")
            || atg.getPid().equals("atg.ufdsStufeFahrBahnOberFlchenTemperatur")
            || atg.getPid().equals("atg.ufdsStufeFahrBahnOberFlchenZustand")
            || atg.getPid().equals("atg.ufdsStufeGefrierTemperatur") || atg.getPid().equals("atg.ufdsStufeHelligkeit")
            || atg.getPid().equals("atg.ufdsStufeLuftDruck") || atg.getPid().equals("atg.ufdsStufeLuftTemperatur")
            || atg.getPid().equals("atg.ufdsStufeNiederschlagsArt")
            || atg.getPid().equals("atg.ufdsStufeNiederschlagsIntensitt")
            || atg.getPid().equals("atg.ufdsStufeNiederschlagsMenge")
            || atg.getPid().equals("atg.ufdsStufeRelativeLuftFeuchte") || atg.getPid().equals("atg.ufdsStufeRestSalz")
            || atg.getPid().equals("atg.ufdsStufeSchneeHhe") || atg.getPid().equals("atg.ufdsStufeSichtWeite")
            || atg.getPid().equals("atg.ufdsStufeTaupunktTemperatur")
            || atg.getPid().equals("atg.ufdsStufeTemperaturInTiefe1")
            || atg.getPid().equals("atg.ufdsStufeTemperaturInTiefe2")
            || atg.getPid().equals("atg.ufdsStufeTemperaturInTiefe3")
            || atg.getPid().equals("atg.ufdsStufeWasserFilmDicke")
            || atg.getPid().equals("atg.ufdsStufeWindGeschwindigkeitMittelWert")
            || atg.getPid().equals("atg.ufdsStufeWindGeschwindigkeitSpitzenWert")
            || atg.getPid().equals("atg.ufdsStufeWindRichtung") || atg.getPid().equals("atg.ufdmsNsseStufe")
            || atg.getPid().equals("atg.ufdmsTaupunktTemperaturFahrBahn")
            || atg.getPid().equals("atg.ufdmsTaupunktTemperaturLuft")
            || atg.getPid().equals("atg.sswSbaHelligkeitErgebnisTagNacht")
            || atg.getPid().equals("atg.sswSbaUfdmsNsseStufe"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.sswSbaSonderprogramm"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.sswSbaManahmeVorschlagAnforderung"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.sswSbaManahmeVorschlagAntwort"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
            }
          });
        }
        else if (atg.getPid().equals("atg.sswSbaSchaltbildAnzeige"))
        {

          if (result.getObject().getType().getPid().equals("typ.sswSbaStreckenBeeinflussungsAnlage"))
          {
            Display.getDefault().asyncExec(new Runnable() {
              public void run()
              {
                DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
              }
            });
          }
          else if (result.getObject().getType().getPid().equals("typ.sswSbaUeSonderprogramm"))
          {
            Display.getDefault().asyncExec(new Runnable() {
              public void run()
              {
                DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
              }
            });
          }
        }
        else if (atg.getPid().equals("atg.sswSbaHandeingriffBetriebsart"))
        {
          // Notifizierung Listener
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              if (result != null)
              {
                DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
              }
            }
          });
        }
        else if (atg.getPid().equals("atg.sswSbaHelligkeitProzentErgebnis"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              if (result != null)
              {
                DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
              }
            }
          });
        }
        else if (atg.getPid().equals("atg.sswSbaHelligkeitSchaltungManuell"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              if (result != null)
              {
                DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
              }
            }
          });
        }
        else if (atg.getPid().equals("atg.sswSbaWanderbaustelle"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              if (result != null)
              {
                DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
              }
            }
          });
        }
        else if (atg.getPid().equals("atg.sswSbaSonderprogrammHierarchie"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              if (result != null)
              {
                DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
              }
            }
          });
        }
        else if (atg.getPid().equals("atg.tlsZltLsaSperranlage"))
        {
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              if (result != null)
              {
                DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
              }
            }
          });
        }
        else if (atg.getPid().equals("atg.situationsEigenschaften"))
        {
//          System.out.println("SituationsUpdate: " + result);
          
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              if (result != null)
              {
                DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
              }
            }
          });
        }
        else if (atg.getPid().equals("atg.sswSbaZuBearbeitendeAspekte"))
        {
//          System.out.println("SituationsUpdate: " + result);
          
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              if (result != null)
              {
                
//                System.out.println("DaV->atg.sswSbaZuBearbeitendeAspekte: " + result);
                
                DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
              }
            }
          });
        }
        else if (atg.getPid().equals("atg.sswSbaMassnahmenAnfrageGUI"))
        {
//          System.out.println("SituationsUpdate: " + result);
          
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              if (result != null)
              {
                DaVDataUpdateListener.getInstanz().notifyDaVDataUpdateListener(result);
              }
            }
          });
        }
      }
    }
  }

  /**
   * Methode liest fr das Objekt mit der PID objPid die Attibutgruppe atgPid unter dem Aspekt aspPid direkt vom DaV und
   * ruft sofort die Auswertemethode auf (nicht ber den Thread)
   * 
   * @param objPid
   *          Pid des Objekts
   * @param atgPid
   *          Attributgruppe die angemeldet werden soll
   * @param aspPid
   *          Apekt der angemeldet werden soll
   */
  public ResultData leseDatenDirektVonDaV(String objPid, String atgPid, String aspPid)
  {
    ClientDavInterface verb = _connection;

    AttributeGroup atg = verb.getDataModel().getAttributeGroup(atgPid);

    Aspect asp = verb.getDataModel().getAspect(aspPid);

    DataDescription dd = new DataDescription(atg, asp);

    SystemObject obj = verb.getDataModel().getObject(objPid);

    try
    {
      ResultData data = verb.getData(obj, dd, 10000);

      ResultData[] results = new ResultData[] { data };

      if (results.length == 1)
        return results[0];
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }

    return null;
  }

  /**
   * Anmeldung auf die Daten vom DAV fr Sicht UmfelddatenTabelle
   */
  public void anmeldenAufDatenUmfeld(Map<String, String[]> map)
  {
    for (Map.Entry<String, String[]> me : map.entrySet())
    {

      String key = me.getKey();
      if (key.contains("typ.umfeldDatenMessStelle"))
      {
        key = "typ.umfeldDatenMessStelle";
      }
      else if (key.contains("typ.sswSbaUmfeldDatenMessStelle"))
      {
        key = "typ.sswSbaUmfeldDatenMessStelle";
      }

      SystemObjectType type = _connection.getDataModel().getType(key);
      List<SystemObject> sos = type.getElements();
      ArrayList<String> list = new ArrayList<String>();

      for (SystemObject so : sos)
      {
        list.add(so.getPid());
      }

      if (me.getKey().contains("typ.umfeldDatenMessStelle"))
      {
        if (me.getValue()[0] != null && me.getValue()[0].length() > 0)
        {
          anmeldeStringsLeseDaV(list, me.getValue()[0], "asp.analyse", ReceiverRole.receiver());
        }
      }
      else if (me.getKey().contains("typ.sswSbaUmfeldDatenMessStelle"))
      {
        if (me.getValue()[0] != null && me.getValue()[0].length() > 0)
        {
          anmeldeStringsLeseDaV(list, me.getValue()[0], "asp.zustand", ReceiverRole.receiver());
        }
      }
      else
        anmeldeStringsLeseDaV(list, me.getValue()[0], "asp.messWertErsetzung", ReceiverRole.receiver());

      if (me.getValue().length > 3)
      {
        anmeldeStringsLeseDaV(list, me.getValue()[3], "asp.klassifizierung", ReceiverRole.receiver());
      }
    }
  }

  /**
   * Anmeldung auf die Daten vom DAV fr Sicht UmfelddatenTabelle
   */
  public void abmeldenAufDatenUmfeld(Map<String, String[]> map)
  {
    for (Map.Entry<String, String[]> me : map.entrySet())
    {
      String key = me.getKey();
      if (key.contains("typ.umfeldDatenMessStelle"))
      {
        key = "typ.umfeldDatenMessStelle";
      }

      SystemObjectType type = _connection.getDataModel().getType(key);

      List<SystemObject> sos = type.getElements();
      ArrayList<String> list = new ArrayList<String>();

      for (SystemObject so : sos)
      {
        list.add(so.getPid());
      }

      if (me.getKey().contains("typ.umfeldDatenMessStelle"))
        abmeldeStringsLeseDaV(list, me.getValue()[0], "asp.analyse");
      else
        abmeldeStringsLeseDaV(list, me.getValue()[0], "asp.messWertErsetzung");

      if (me.getValue().length > 3)
      {
        abmeldeStringsLeseDaV(list, me.getValue()[3], "asp.klassifizierung");
      }

    }

  }

  /**
   * Bestimmt des Default-Bereich des DaV
   * 
   * @param connection
   * @return
   */
  public ConfigurationArea bestimmeDefaultBereich(ClientDavInterface connection)
  {
    Data data = connection.getDataModel().getConfigurationAuthority().getConfigurationData(
        connection.getDataModel().getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften"));

    if (data != null)
    {
      TextArray array = data.getTextArray("defaultBereich");

      if (array.getLength() == 1)
      {
        return (ConfigurationArea)connection.getDataModel().getObject(array.getText(0));
      }
    }

    return null;
  }

  public boolean isObjektVorhanden(String pid)
  {

    SystemObject object = _connection.getDataModel().getObject(pid);

    if (object != null)
      return true;

    return false;
  }

  @Override
  public void update(MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects)
  {
    System.out.println("DaVKommunikationDobj.updateMenge())" + set.getName());

    if (set.getName().equals("UrsachenEinheiten"))
    {

      if (addedObjects != null)
      {
        for (int i = 0; i < addedObjects.length; i++)
        {

          SystemObject so = addedObjects[i];

          if (so.isOfType("typ.sswSbaUeSonderprogramm"))
          {
            AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaSonderprogramm");
            Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
            DataDescription dd = new DataDescription(atg, asp);

            _connection.subscribeReceiver(this, so, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
          }
        }
      }

      if (removedObjects != null)
      {
        for (int i = 0; i < removedObjects.length; i++)
        {
          SystemObject so = removedObjects[i];

          if (so.isOfType("typ.sswSbaUeSonderprogramm"))
          {

          }
        }
      }
    } 
    else if (set.getName().equals("Baustellen") || set.getName().equals("Unflle") || set.getName().equals("Staus"))
    {
      
      if (addedObjects != null)
      {
        for (int i = 0; i < addedObjects.length; i++)
        {
          
          SystemObject so = addedObjects[i];
                       
          
          if (so.isOfType("typ.situation"))
          {
            AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.situationsEigenschaften");
            Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
            DataDescription dd = new DataDescription(atg, asp);           
//            System.out.println("DaVKommunikationDobj.update(anmelden): " + so.getPid() +  " : " + so.getId());
            _connection.subscribeReceiver(this, so, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
          }
        }
      }
      
      if (removedObjects != null)
      {
        for (int i = 0; i < removedObjects.length; i++)
        {
          SystemObject so = removedObjects[i];
          
          if (so.isOfType("typ.situation"))
          {
            
//            AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.situationsEigenschaften");
//            Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
//            DataDescription dd = new DataDescription(atg, asp);
//            
//            final ResultData result = _connection.getData(so, dd, 1000);
//            
//            System.out.println("-->Alt " + result);            
          }
        }
      }
    } 
  }

  public void abmeldenAufDatenSemiAspekt()
  {
    // TODO Auto-generated method stub
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    if (pidSba != null && pidSba.length() > 0)
    {
      SystemObject object = _connection.getDataModel().getObject(pidSba);
      if (object != null)
      {
        List<String> listAnlage = new ArrayList<String>();
        listAnlage.add(object.getPid());
        
        abmeldeStringsLeseDaV(listAnlage, "atg.sswSbaZuBearbeitendeAspekte", "asp.parameterSoll");
      }
    }
    
  }
  public void anmeldenAufDatenSemiAspekt()
  {
//    System.out.println("DaVKommunikationDobj.anmeldenAufDatenSemiAspekt()");
    
    // TODO Auto-generated method stub
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    if (pidSba != null && pidSba.length() > 0)
    {
      SystemObject object = _connection.getDataModel().getObject(pidSba);
      if (object != null)
      {
        List<String> listAnlage = new ArrayList<String>();
        listAnlage.add(object.getPid());
        
        anmeldeStringsLeseDaV(listAnlage, "atg.sswSbaZuBearbeitendeAspekte", "asp.parameterSoll", ReceiverRole
            .receiver());
      }
    }
    
  }
  
  public void anmeldenAufDatenSemi(List<String> aspekte)
  {
    
//    System.out.println("DaVKommunikationDobj.anmeldenAufDatenSemi(): " + aspekte.size());
    
    // TODO Auto-generated method stub
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    if (pidSba != null && pidSba.length() > 0)
    {
      SystemObject object = _connection.getDataModel().getObject(pidSba);
      if (object != null)
      {
        List<String> listAnlage = new ArrayList<String>();
        listAnlage.add(object.getPid());
        
        AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaMassnahmenAnfrageGUI");
        Collection<Aspect> aspects = atg.getAspects();
        
        List<String> listAllowedAspects = new ArrayList<String>();
        for (Aspect tmp : aspects)
        {
          listAllowedAspects.add(tmp.getPid());
        }
        
        for (String asp : aspekte)
        {          
          if (listAllowedAspects.contains(asp))          
            anmeldeStringsLeseDaV(listAnlage, atg.getPid(), asp, ReceiverRole.receiver());          
        }
        
      }
    }
    
  }
  
  public void abmeldenAufDatenSemi(List<String> aspekte)
  {
    
//    System.out.println("DaVKommunikationDobj.abmeldenAufDatenSemi(): " + aspekte.size());
    
    // TODO Auto-generated method stub
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();
    String pidSba = store.getString(DefsPreference.P_STRECKEN_BEEINFLUSSUNGS_ANLAGE);
    if (pidSba != null && pidSba.length() > 0)
    {
      SystemObject object = _connection.getDataModel().getObject(pidSba);
      if (object != null)
      {
        List<String> listAnlage = new ArrayList<String>();
        listAnlage.add(object.getPid());
        
        AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaMassnahmenAnfrageGUI");
        Collection<Aspect> aspects = atg.getAspects();
        
        List<String> listAllowedAspects = new ArrayList<String>();
        for (Aspect tmp : aspects)
        {
          listAllowedAspects.add(tmp.getPid());
        }
        
        for (String asp : aspekte)
        {          
          if (listAllowedAspects.contains(asp))          
            abmeldeStringsLeseDaV(listAnlage, atg.getPid(), asp);          
        }
        
      }
    }
    
  }

}
