/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dav;

import java.awt.BorderLayout;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;

import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;

/**
 * Zentrale Klasse zur Verwaltung der Images
 * 
 * @author Pittner
 * 
 */
public class ImageFactory
{
	private static ConcurrentHashMap<String, Image> _map = new ConcurrentHashMap<String, Image>();

	private static ConcurrentHashMap<String, Image> _mapGestoert = new ConcurrentHashMap<String, Image>();

	/**
	 * Singleton-Instanz
	 */
	private static ImageFactory _instanz = null;

	/**
	 * Holt ein Image aus den Ressourcen. Es wird aber nur eine Instanz des Images
	 * erzeugt. Die Methode implementiert das Singleton-Pattern.
	 * 
	 * @param pfad
	 * @return das Image
	 */
	public static Image getImage(String pfad)
	{
		return getImage(pfad, false);
	}

	/**
	 * Holt ein Image aus den Ressourcen. Es wird aber nur eine Instanz des Images
	 * erzeugt. Die Methode implementiert das Singleton-Pattern.
	 * 
	 * @param pfad
	 *          Wenn am Pfad eine mit <code>;</code> separierter {@link Double}
	 *          -Wert dran hngt, wird das Bild um diesen Wert gedreht.
	 * @return das Image
	 */
	public static Image getImage(String pfad, boolean gestoert)
	{
		if (pfad != null)
		{
			String pfadOriginal = pfad;

			Image erg = null;
			if (!gestoert)
				erg = (Image) _map.get(pfadOriginal);
			else
				erg = (Image) _mapGestoert.get(pfadOriginal);

			if (erg == null || erg != null && erg.isDisposed())
			{
				Double rotation = null;
				boolean blinken = false;

				if (pfad.contains(";"))
				{
					String[] split = pfad.split(";");
					pfad = split[0];
					if (split.length > 0)
					{
						try
						{
							rotation = Double.parseDouble(split[1]);
						} catch (NumberFormatException nfe)
						{
							// nfe.printStackTrace();
						}

					}

					if (split.length > 1)
					{
						String strBlinken = split[split.length - 1];
						if (strBlinken.contains("="))
						{
							String[] splitBlinken = strBlinken.split("=");
							if ((splitBlinken.length > 1)
									&& (splitBlinken[0].equalsIgnoreCase("Blinken"))
									&& (splitBlinken[1].equalsIgnoreCase("true")))
							{
								blinken = true;
							}
						}
					}
				}

				ImageDescriptor imageDescriptor = null;
				try
				{
					imageDescriptor = Activator.getImageDescriptor(pfad);
				} catch (IllegalArgumentException iae)
				{
					ILog log = Activator.getDefault().getLog();
					log.log(new Status(Status.WARNING, Activator.PLUGIN_ID,
							"ImageDescriptor " + pfad + " nicht verfgbar!", iae));
				}

				if (imageDescriptor != null)
				{
					Image image = imageDescriptor.createImage();
					if (gestoert)
					{
						Image imageTemp = ImageFactory.getImage(
								"icons/raster_rot_37x29.png", false);
						ImageData image02Data = imageTemp.getImageData();
						int blackPixel = image02Data.palette.getPixel(new RGB(0, 0, 0));
						image02Data.transparentPixel = blackPixel;

						Image imageTemp2 = new Image(
								PlatformUI.getWorkbench().getDisplay(), image02Data);
						Image mergedImage = new Image(PlatformUI.getWorkbench()
								.getDisplay(), image.getImageData().width,
								image.getImageData().height);
						GC gcMergedImage = new GC(mergedImage);
						gcMergedImage.drawImage(image, 0, 0);
						gcMergedImage.drawImage(imageTemp2, 0, 0);
						gcMergedImage.dispose();
						gcMergedImage = null;
						image = new Image(PlatformUI.getWorkbench().getDisplay(),
								mergedImage.getImageData());
					}

					if (image != null)
					{
						if (!gestoert)
						{
							_map.put(pfad, image);
						} else
						{
							_mapGestoert.put(pfad, image);
						}

//						System.out.println("=> Rotation=" + rotation + ", Blinken="
//								+ blinken + ", Pfad=" + pfadOriginal);

						if ((rotation != null) || blinken)
						{
							java.awt.Image swingImg = ImageUtils.readImageFromBundle(pfad);
							java.awt.Image modifiedImage = swingImg;

							getInstanz();

							/*
							 * Wichtig: Erst heller machen fr das Blinken, DANN rotieren!
							 */
							if (blinken)
							{
								/*
								 * Helligkeit fr das Blinken anpassen.
								 */
								modifiedImage = ImageUtils.brightenSwingImage(swingImg);
							}

							if (rotation != null)
							{
								/*
								 * Rotation bearbeiten
								 */
								modifiedImage = ImageUtils.rotateImage(modifiedImage, rotation);
							}

							if (modifiedImage != null)
							{
								/*
								 * Sichern des Images unter dem Originalpfad, inklusive Rotation
								 * und Blinkflag.
								 */
								image = ImageUtils.swingToSwtImage(modifiedImage);
								_map.put(pfadOriginal, image);
							}
						}

						return image;
					}

				}

			} else
			{
				return erg;
			}

		} else
		{
			/*
			 * Kein Bild zum Zurckgeben, Pfad nicht verwendbar.
			 */
		}

		return null;
	}

	/**
	 * Liefert die Singletoninstanz.
	 * 
	 * @return Instanz dieser Klasse.
	 */
	public static ImageFactory getInstanz()
	{
		if (_instanz == null)
		{
			_instanz = new ImageFactory();
		}
		return _instanz;
	}

	/**
	 * Konstruktor der Klasse
	 */
	private ImageFactory() {
	}
}
