/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.List;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.util.EList;

import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittLinksFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittRechtsFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeQuerschnittTyp;
import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.dobj.internal.DarstellungExtensionPoint;
import de.bsvrz.buv.plugin.dobj.model.ConfiguratedDoTyp;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link AnzeigeQuerschnittModelTypAbstract} verwendet.
 * 
 * @author Pittner
 * 
 */
public class AnzeigeQuerschnittTyp16A8BEditPart extends AnzeigeQuerschnittTyp16A8EditPart
{
  @Override
  protected List getModelChildren()
  {
    if (getCastedModel().getKomponenten() != null && getCastedModel().getKomponenten().size() > 0)
    {
      return getCastedModel().getKomponenten();
    }
    else
    {
      ConfiguratedDoTyp doTyp1 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypBlinkerEditPart");
      ConfiguratedDoTyp doTyp2 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypTEditPart");
      ConfiguratedDoTyp doTyp3 = DarstellungExtensionPoint.getDoTyp("AnzeigeTypW1EditPart");

      Darstellung darstellung = (Darstellung)getViewer().getProperty(Darstellung.class.toString());
      EList<DoTyp> doTypen = darstellung.getDoTypen();

      boolean vorhanden1 = false;
      boolean vorhanden2 = false;
      boolean vorhanden3 = false;

      for (DoTyp typ : doTypen)
      {
        if (typ instanceof ConfiguratedDoTyp)
        {
          ConfiguratedDoTyp co = (ConfiguratedDoTyp)typ;
          if (co.getId().equals(doTyp1.getId()))
          {
            vorhanden1 = true;
            doTyp1 = co;
          }
          if (co.getId().equals(doTyp2.getId()))
          {
            vorhanden2 = true;
            doTyp2 = co;
          }
          if (co.getId().equals(doTyp3.getId()))
          {
            vorhanden3 = true;
            doTyp3 = co;
          }
        }
      }

      AnzeigeModelTypAbstract t1 = (AnzeigeModelTypAbstract)doTyp2.createModel();
      t1.setLocation(new Point(1, 35));
      getCastedModel().getKomponenten().add(t1);

      AnzeigeModelTypAbstract w1 = (AnzeigeModelTypAbstract)doTyp3.createModel();
      w1.setLocation(new Point(52, 35));
      getCastedModel().getKomponenten().add(w1);

      AnzeigeModelTypAbstract t2 = (AnzeigeModelTypAbstract)doTyp2.createModel();
      t2.setLocation(new Point(164, 35));
      getCastedModel().getKomponenten().add(t2);

      AnzeigeModelTypAbstract bl2 = (AnzeigeModelTypAbstract)doTyp1.createModel();
      bl2.setLocation(new Point(99, 17));
      getCastedModel().getKomponenten().add(bl2);

      AnzeigeModelTypAbstract bl1 = (AnzeigeModelTypAbstract)doTyp1.createModel();
      bl1.setLocation(new Point(18, 17));
      getCastedModel().getKomponenten().add(bl1);

      AnzeigeModelTypAbstract bl3 = (AnzeigeModelTypAbstract)doTyp1.createModel();
      bl3.setLocation(new Point(181, 17));
      getCastedModel().getKomponenten().add(bl3);

      if (!vorhanden1)
      {
        doTypen.add(doTyp1);
      }
      if (!vorhanden2)
      {
        doTypen.add(doTyp2);
      }
      if (!vorhanden3)
      {
        doTypen.add(doTyp3);
      }

      return getCastedModel().getKomponenten();
    }
  }

  @Override
  protected AnzeigeQuerschnittFigure createFigure()
  {

    AnzeigeQuerschnittRechtsFigure rechts = null;
    AnzeigeQuerschnittFigure result = null;

    _contentPane = new AnzeigeQuerschnittLinksFigure(AnzeigeQuerschnittTyp.WVZ_A8_Typ16B, 190);
    _contentPane.setSize(220, 99);
    rechts = new AnzeigeQuerschnittRechtsFigure(getCastedModel().getKontext(), false);
    rechts.setSize(35, 99);

    result = new AnzeigeQuerschnittFigure(_contentPane, rechts);
    result.setSize(250, 99);

    result.switchLinksRechts(getCastedModel().getFahrtrichtung());

    String text = getCastedModel().getBezeichnung();
    if (text != null)
    {
      result.setExtraText(text);
    }

    addToolTipListener(result);

    return result;
  }

}
