/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.swt.graphics.Image;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeStatusBefehle;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionAnzeigeStatus;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionTabelle;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.ILegendeBaustein2;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.util.LegendeHelper;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link AnzeigeModelTypAbstract} verwendet.
 * 
 * @author Pittner
 * 
 */
public class AnzeigeTypAEditPart extends DoMitVerbLinieEditPart implements ILegendeBaustein2
{
  /**
   * Default - Skalierungsfaktor.
   */
  private double scale = 1.0;

  private final Dimension DIM = new Dimension(33, 33);

  private ArrayList<ILegendeBaustein> legendenBausteine;

//  private Image image;

  @Override
  protected AnzeigeElementFigure createFigure()
  {

    AnzeigeModelTypAbstract model = getCastedModel();

    ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
    DataModel dataModel = dav.getDataModel();

    setzteInhalteModell(model, dataModel);

    boolean isFrNord = getCastedModel().getFahrtrichtung() != 1 ? true : false;

    AnzeigeElementFigure aef = new AnzeigeElementFigure("AnzeigeA", AnzeigeElementTyp.WZG_A,
        AnzeigeInhaltTyp.DUNKEL_BI, true, isFrNord);
    aef.setOpaque(true);
    aef.setSize(DIM.width, DIM.height);
    aef.setBorder(new LineBorder(ColorConstants.orange, 2));
    SystemObject so = model.getSystemObject();
    if (so != null)
    {
      AttributeGroup atg = dataModel.getAttributeGroup("atg.anzeige");
      Data cd = so.getConfigurationData(atg);
      String anzeigeTyp = cd.getItem("AnzeigeTyp").asReferenceValue().getSystemObjectPid();
      aef.setPid(anzeigeTyp);
    }

    addToolTipListener(aef);
    
    return aef;
  }

  protected void setzteInhalteModell(AnzeigeModelTypAbstract model, DataModel dataModel)
  {
    AttributeGroup atg = dataModel.getAttributeGroup("atg.anzeige");
    AttributeGroup atg2 = dataModel.getAttributeGroup("atg.wvzInhalt");

    if (model != null)
    {
      SystemObject so = model.getSystemObject();

      if (so != null)
      {
        Data dta = so.getConfigurationData(atg);

        SystemObject soTyp = dta.getReferenceValue("AnzeigeTyp").getSystemObject();

        ConfigurationObject co = (ConfigurationObject)soTyp;
        NonMutableSet set = co.getNonMutableSet("WvzInhalt");
        if (set != null)
        {
          List<SystemObject> inhalte = set.getElements();
          ArrayList<String> list = new ArrayList<String>();
          for (SystemObject inhalt : inhalte)
          {
            dta = inhalt.getConfigurationData(atg2);
            String s = dta.getItem("Bildinhalt").asTextValue().getValueText();
            list.add(s);
          }

          EList<String> elist = new BasicEList<String>(list);
          getCastedModel().setInhalte1(elist);
          getCastedModel().setInhalte2(elist);
        }
      }
    }
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);
    

    if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__BEZEICHNUNG.equals(notification.getFeature()))
    {
      
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__KONTEXT.equals(notification.getFeature()))
    {
      
    }    
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE.equals(notification
        .getFeature()))
    {
      Object newValue = notification.getNewValue();
      Object oldValue = notification.getOldValue();

      int kontext = 0;
      if (getParent() != null)
      {
        Object parentModel = getParent().getModel();
        {
          if (parentModel instanceof AnzeigeQuerschnittModelTypAbstract)
          {
            AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parentModel;
            kontext = aq.getKontext();
          }
        }
      }

      updateAnzeigeInhalt(kontext, newValue, oldValue, AnzeigeElementTyp.WZG_A);
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE.equals(notification
        .getFeature()))
    {
      Object object = notification.getNewValue();
      if (object instanceof String)
      {
        String s = (String)object;       
        getFigure().setToolTip(new Label(s));
      }
    }
    
  }

  /**
   * We don't need to explicitly handle refresh visuals because the times when this needs to be done it is handled by
   * the table e.g. handleNameChange()
   */
  protected void refreshVisuals()
  {
    Point pos = getCastedModel().getLocation();
    Dimension dim = getFigure().getSize();

    if (pos != null && dim != null)
    {
      Rectangle constraint = new Rectangle(pos, dim);

      if (constraint != null)
        ((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), constraint);
      else
      {
        ((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), new Rectangle(pos.x, pos.y, -1, -1));
      }
    }

  }

  @Override
  protected void createEditPolicies()
  {
    super.createEditPolicies();
    NonResizableEditPolicy policy = new NonResizableEditPolicy();
    installEditPolicy(EditPolicy.LAYOUT_ROLE, policy);
  }

  /**
   * @return
   */
  protected AnzeigeModelTypAbstract getCastedModel()
  {
    AnzeigeModelTypAbstract model = null;
    if (getModel() instanceof AnzeigeModelTypAbstract)
    {
      model = (AnzeigeModelTypAbstract)getModel();
    }
    return model;
  }
  
  /** {@inheritDoc} */
  @Override
  public List<ILegendeBaustein> getBausteine() {
    if (legendenBausteine == null) {
      legendenBausteine = new ArrayList<ILegendeBaustein>();
      for (Object listObj : getDefinition())
      {
        legendenBausteine.addAll(LegendeHelper.erzeugeLegendeBausteine(listObj));        
      }
    }
    return legendenBausteine;
  }

  @Override
  public Image getIcon()
  {
    // TODO Auto-generated method stub
    return null;
  }

  @Override
  public String getText()
  {
    // TODO Auto-generated method stub
    return getId();
  }

  @Override
  public String getId()
  {
    // TODO Auto-generated method stub
    return "Anzeige";
  }
  
  @Override
  public List getDefinition()
  {
    List list = new ArrayList();
    Object parent = getParent().getModel();
    if (parent instanceof AnzeigeQuerschnittModelTypAbstract)
    {
      AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parent;
      list.addAll(LegendeHelper.erzeugeAnzeigeLegendeDefinition(aq, getCastedModel()));
    }
    
    return list;
  }

}
