/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypHell;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittRechtsFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.ZusatzElementTyp;
import de.bsvrz.buv.plugin.dobj.editparts.DoModelEditPart;
import de.bsvrz.buv.plugin.dobj.model.DobjPackage;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link AnzeigeModelTypHell} verwendet.
 * 
 * @author Pittner
 * 
 */
public class AnzeigeTypHellEditPart extends DoModelEditPart<AnzeigeModelTypHell, Label> implements
    ClientReceiverInterface
{

  private final Dimension DIM = new Dimension(20, 20);

  @Override
  protected void geheOnline()
  {
    super.geheOnline();
  }

  @Override
  protected void geheOffline()
  {

    super.geheOffline();
  }

  @Override
  protected Label createFigure()
  {
    Label l = new Label();
    l.setOpaque(true);
    l.setSize(DIM.width, DIM.height);

    String path = AnzeigeQuerschnittRechtsFigure.createImagePath(ZusatzElementTyp.HELLIGKEIT,
        ZusatzElementTyp.PROGRAMM_UNDEFINIERT);
    Image image = ImageFactory.getImage(path);

    l.setIcon(image);

    return l;
  }

  @Override
  public void update(ResultData[] results)
  {
    
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);
    if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__BEZEICHNUNG.equals(notification.getFeature()))
    {
      
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__KONTEXT.equals(notification.getFeature()))
    {
      
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DATA_UPDATE.equals(notification
        .getFeature()))
    {      
      ResultData result = (ResultData)notification.getNewValue();
      dataUpdate(result);
    }
    else if (AnlagendarstellungPackage.Literals.ANZEIGE_MODEL_TYP_ABSTRACT__DARSTELLUNG_UPDATE.equals(notification
        .getFeature()))
    {
      Object object = notification.getNewValue();
      if (object instanceof AnzeigeInhaltTyp)
      {
        AnzeigeInhaltTyp inhalt = (AnzeigeInhaltTyp)object;
        if (inhalt != null)
        {
          String path = AnzeigeElementFigure.createImagePath(AnzeigeElementTyp.PROGRAMM, inhalt);

          if (path != null)
          {
            Image image = ImageFactory.getImage(path);
            getFigure().setIcon(image);
          }
        }
      }
    }
    else if (DobjPackage.Literals.DO_MODEL__SYSTEM_OBJECT.equals(notification.getFeature()))
    {
      SystemObject so = (SystemObject)notification.getNewValue();
    }
  }

  @Override
  public void zoomChanged(double zoom)
  {

    if (scale == zoom)
      return;
    else
      scale = zoom;
   
    getFigure().repaint();
  }
  

  /**
   * Default - Skalierungsfaktor.
   */
  private double scale = 1.0;

  /**
   * We don't need to explicitly handle refresh visuals because the times when this needs to be done it is handled by
   * the table e.g. handleNameChange()
   */
  protected void refreshVisuals()
  {
    Point pos = getModel().getLocation();
    Dimension dim = getFigure().getSize();

    Rectangle constraint = new Rectangle(pos, dim);

    if (constraint != null)
      ((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), constraint);
    else
    {
      ((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), new Rectangle(pos.x, pos.y, -1, -1));
    }

  }

  public void dataUpdate(ResultData result)
  {

    final Label figure = getFigure();
    AttributeGroup atg = result.getDataDescription().getAttributeGroup();

    if (atg.getPid().equals("atg.anzeigeEigenschaftIst"))
    {
      Data data = result.getData();
      if (data != null)
      {

        SystemObject so = data.getItem("Eigenschaft").getItem("AnzeigeInhalt").asReferenceValue().getSystemObject();
        ClientDavInterface dav = DaVVerbindung.getInstanz().getDaVVerbindung();
        AttributeGroup atg2 = dav.getDataModel().getAttributeGroup("atg.wvzInhalt");
        Data data2 = so.getConfigurationData(atg2);
        final String inhalt = data2.getItem("Bildinhalt").asTextValue().getValueText();        
        AnzeigeInhaltTyp typ = AnzeigeInhaltTyp.setBildInhalt(inhalt, result.getDataTime());

        if (getModel().getSystemObject().equals(result.getObject()))
        {
          if (getModel().getSystemObject().equals(result.getObject()))
          {
            String imagepath = AnzeigeElementFigure.createImagePath(AnzeigeElementTyp.WZG_A, typ);
            final Image image = ImageFactory.getImage(imagepath);
            if (imagepath != null)
            {
              PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                @Override
                public void run()
                {
                  figure.setBorder(new LineBorder(ColorConstants.green, 2));
                  figure.setIcon(image);
                }
              });
            }
          }
        }
      }
      else
      {
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

          @Override
          public void run()
          {
            String imagepath = AnzeigeElementFigure.createImagePath(AnzeigeElementTyp.WZG_A, AnzeigeInhaltTyp.GESTOERT);
            final Image image = ImageFactory.getImage(imagepath);
            figure.setBorder(new LineBorder(ColorConstants.red, 2));
            figure.setIcon(image);
          }
        });
      }
    }
  }
}
