/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.gef.EditPart;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.dobj.figures.TestFigure;
import de.bsvrz.buv.plugin.dobj.editparts.BitCtrlDoModelEditPart;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateListener;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.objekte.AnzeigeQuerschnitt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmanzeigenglobal.onlinedaten.OdAnzeigeQuerschnittEigenschaftIst;

/**
 * Einfache Beispiel-Implementation fr einen {@link EditPart} eines DAV Darstellungsobjektes, der die BitCtrl Dav
 * Modell API verwendet.
 * 
 * @author Thomas Thierfelder / Christian Hsel
 * 
 */
public class BcTestEditPart extends BitCtrlDoModelEditPart<AnzeigeQuerschnitt, TestFigure> implements
    DatensatzUpdateListener
{

  @Override
  protected void geheOnline()
  {
    super.geheOnline();
    getSystemObjekt().getOdAnzeigeQuerschnittEigenschaftIst().addUpdateListener(
        OdAnzeigeQuerschnittEigenschaftIst.Aspekte.AqZustand, this);
  }

  @Override
  protected void geheOffline()
  {
    super.geheOffline();
    getSystemObjekt().getOdAnzeigeQuerschnittEigenschaftIst().removeUpdateListener(
        OdAnzeigeQuerschnittEigenschaftIst.Aspekte.AqZustand, this);
  }

  @Override
  protected TestFigure createFigure()
  {
    TestFigure result = new TestFigure();    
    return result;
  }

  @Override
  public void datensatzAktualisiert(DatensatzUpdateEvent event)
  {
    final TestFigure f = (TestFigure)getFigure();

    if (event.getDatum().dContainsDaten())
    {
      final OdAnzeigeQuerschnittEigenschaftIst.Daten daten = (OdAnzeigeQuerschnittEigenschaftIst.Daten)event.getDatum();

      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

        @Override
        public void run()
        {
          f.setForegroundColor(ColorConstants.green);           
        }
      });
    }
    else
    {
      PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

        @Override
        public void run()
        {
          f.setForegroundColor(ColorConstants.red);           
        }
      });
    }
  }
}
