package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.RootEditPart;

import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeQuerschnittLinksFigure;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Liefert diverse Hilfsmethoden fr EditParts.
 * 
 * @author STS
 * 
 */
public class EditPartHelper
{
	/**
	 * Berechnet in Abhngigkeit von der Breite des ContentPane die fr den
	 * AQ-Namen zur Verfgung stehende Breite. Die Formel lautet wie folgt:<br>
	 * 
	 * <pre>
	 * Breite AQName = Breite Contentpane - Breite Zeitfeld - Randbreite Hozizontal
	 * </pre>
	 * 
	 * @param widthContentPane
	 *          Breite des Contentpane
	 * @return Breite fr den AQ-Namen.
	 */
	private static int berechneBreiteAQName(int widthContentPane)
	{
		int re = widthContentPane - ADefs.WIDTH_TEXT_ZEIT - ADefs.FRAME_WIDTH_ADD
				- 1;

		return re;
	}

	/**
	 * Positioniert die Texte fr Zeit und AQ-Bezeichnung. Bei Querpositionierung
	 * soll das Zeitfeld immer zur Fahrbahn hin zeigen.
	 * 
	 * @param inRichtung
	 *          Wenn <code>true</code> dann in Richtung, sonst gegen Richtung.
	 * @param positioniereQuer
	 *          Legt Fest ob Quer (<code>true</code>) oder Lngs (
	 *          <code>false</code>) positioniert wird
	 * @param contentPane
	 *          Contentpane auf dem die Darstellung erfolgt.
	 * @param widthContentPane
	 *          Breite des Contentpane, auf dem die Darstellung erfolgt.
	 * @param heightContentPane
	 *          Hhe des Contentpane, auf dem die Darstellung erfolgt.
	 * @param imgPathNormal
	 *          Hintergrundgrafik in normaler (Nord!) Ausrichtung.
	 * @param imgPathGedreht
	 *          Hintergrundgrafik in entgegengesetzter (Sd!) Ausrichtung.
	 */
	public static void positioniereText(boolean inRichtung,
			boolean positioniereQuer, AnzeigeQuerschnittLinksFigure contentPane,
			int widthContentPane, int heightContentPane, String imgPathNormal,
			String imgPathGedreht)
	{
		/*
		 * Legt fest, ob der Text immer im Norden angezeigt werden soll.
		 */
		boolean textImmerImNorden = false;

		if (contentPane != null)
		{
			boolean showBorder = false;

			if (showBorder)
			{
				contentPane.getLabelName().setBorder(
						new LineBorder(ColorConstants.cyan, 1));
				contentPane.getLabelZeit().setBorder(
						new LineBorder(ColorConstants.red, 1));
			}

			if (inRichtung)
			{
				/*
				 * Hintergrund
				 */
				if (imgPathNormal != null)
				{
					contentPane.setImage(ImageFactory.getImage(imgPathNormal));
				}

				if (positioniereQuer)
				{
					/*
					 * Name
					 */
					contentPane.getLabelName().setLabelAlignment(PositionConstants.RIGHT);
					contentPane.setConstraint(contentPane.getLabelName(), new Rectangle(
							ADefs.WIDTH_TEXT_ZEIT + 1, 0,
							berechneBreiteAQName(widthContentPane), ADefs.HEIGHT_TEXT_ZEIT));

					/*
					 * Zeit
					 */
					contentPane.getLabelZeit().setLabelAlignment(PositionConstants.LEFT);
					contentPane.setConstraint(contentPane.getLabelZeit(), new Rectangle(
							0, 0, ADefs.WIDTH_TEXT_ZEIT, ADefs.HEIGHT_TEXT_ZEIT));

				} else
				{
					/*
					 * Name
					 */
					contentPane.getLabelName().setLabelAlignment(PositionConstants.LEFT);
					contentPane.setConstraint(contentPane.getLabelName(), new Rectangle(
							0, ADefs.TEXT_Y_ZEILEN_OFFSET, contentPane.getSize().width
									- ADefs.FRAME_WIDTH_ADD, ADefs.HEIGHT_TEXT_ZEIT));

					/*
					 * Zeit
					 */
					contentPane.getLabelZeit().setLabelAlignment(PositionConstants.LEFT);
					contentPane.setConstraint(contentPane.getLabelZeit(), new Rectangle(
							0, 0, ADefs.WIDTH_TEXT_ZEIT, ADefs.HEIGHT_TEXT_ZEIT));
				}

			} else
			{
				/*
				 * Hintergrund
				 */
				if (imgPathGedreht != null)
				{
					contentPane.setImage(ImageFactory.getImage(imgPathGedreht));
				}

				if (positioniereQuer)
				{
					/*
					 * Name
					 */
					contentPane.getLabelName().setLabelAlignment(PositionConstants.LEFT);

					int y = 0;
					if (!textImmerImNorden)
					{
						y = heightContentPane - ADefs.TEXT_Y_ZEILEN_OFFSET
								- ADefs.FRAME_HEIGHT_ADD;
					}

					contentPane.setConstraint(contentPane.getLabelName(), new Rectangle(
							0, y, berechneBreiteAQName(widthContentPane),
							ADefs.HEIGHT_TEXT_ZEIT));

					/*
					 * Zeit
					 */
					contentPane.getLabelZeit().setLabelAlignment(PositionConstants.RIGHT);

					contentPane.setConstraint(contentPane.getLabelZeit(), new Rectangle(
							widthContentPane - ADefs.WIDTH_TEXT_ZEIT - ADefs.FRAME_WIDTH_ADD,
							y, ADefs.WIDTH_TEXT_ZEIT, ADefs.HEIGHT_TEXT_ZEIT));
				} else
				{
					int yName = 0;
					int yZeit = 1 * ADefs.TEXT_Y_ZEILEN_OFFSET;

					if (!textImmerImNorden)
					{
						yName = heightContentPane - ADefs.FRAME_HEIGHT_ADD
								- (2 * ADefs.TEXT_Y_ZEILEN_OFFSET);

						yZeit = heightContentPane - ADefs.FRAME_HEIGHT_ADD
								- ADefs.TEXT_Y_ZEILEN_OFFSET;
					}

					/*
					 * Name
					 */
					contentPane.getLabelName().setLabelAlignment(PositionConstants.LEFT);
					contentPane.setConstraint(contentPane.getLabelName(), new Rectangle(
							0, yName, widthContentPane - ADefs.FRAME_WIDTH_ADD,
							ADefs.HEIGHT_TEXT_ZEIT));

					/*
					 * Zeit
					 */
					contentPane.getLabelZeit().setLabelAlignment(PositionConstants.LEFT);
					contentPane.setConstraint(contentPane.getLabelZeit(), new Rectangle(
							0, yZeit, ADefs.WIDTH_TEXT_ZEIT, ADefs.HEIGHT_TEXT_ZEIT));
				}
			}
		}
	}

	/**
	 * Bestimmt eine MAP mit den sich die EditParts zu einem Model bestimmen
	 * lassen. Key=Model, Value = EditPart
	 * 
	 * @param gv
	 * @return
	 */
	public static HashMap<DoModelMitVerbLinie, DoMitVerbLinieEditPart> bestimmeModelEditpartZuordnung(
			GraphicalViewer gv)
	{
		HashMap<DoModelMitVerbLinie, DoMitVerbLinieEditPart> mapModelEditpart = new HashMap<DoModelMitVerbLinie, DoMitVerbLinieEditPart>();

		if (gv != null)
		{
			RootEditPart rootEditPart = gv.getRootEditPart();
			if (rootEditPart != null)
			{
				LinkedList<DoMitVerbLinieEditPart> list = bestimmeAlleDoRekursiv(rootEditPart);
				for (DoMitVerbLinieEditPart doMitVerbLinieEditPart : list)
				{
					DoModelMitVerbLinie model = doMitVerbLinieEditPart.getModel();
					mapModelEditpart.put(model, doMitVerbLinieEditPart);
				}
			}
		}

		return mapModelEditpart;
	}

//	/**
//	 * Meldet alle {@link DoMitVerbLinieEditPart}s, die sich in einem
//	 * {@link GraphicalViewer} befinden, vom {@link Taktgeber} ab. Die Bestimmung
//	 * erfolgt rekursiv, ausgehend vom {@link RootEditPart}.
//	 * 
//	 * @param gv
//	 *          {@link GraphicalViewer} dessen {@link DoMitVerbLinieEditPart}s
//	 *          abgemeldet werden.
//	 */
//	public static void alleDoVomTaktgeberAbmelden(GraphicalViewer gv)
//	{
//		// ITaktListener[] aTaktlistenerVorher = Taktgeber.getInstance()
//		// .getAngemeldeteTaktlistener();
//
//		if (gv != null)
//		{
//			RootEditPart rootEditPart = gv.getRootEditPart();
//			if (rootEditPart != null)
//			{
//				LinkedList<DoMitVerbLinieEditPart> lDo = bestimmeAlleDoRekursiv(rootEditPart);
//				for (DoMitVerbLinieEditPart doMitVerbLinieEditPart : lDo)
//				{
//					/*
//					 * Es wird hier explizit die Funktion des Darstellungsobjektes
//					 * aufgerufen, da dieses so in einen definierten Zustand abgemeldet
//					 * gelangt.
//					 */
//					doMitVerbLinieEditPart.abmeldenTaktgeber();
//				}
//
//			}
//		}
//
//		// ITaktListener[] aTaktlistenerNachher = Taktgeber.getInstance()
//		// .getAngemeldeteTaktlistener();
//		// System.out.println("Angemeldete Taktlistener vorher  =>"
//		// + aTaktlistenerVorher.length);
//		// System.out.println("Angemeldete Taktlistener nachher =>"
//		// + aTaktlistenerNachher.length);
//	}

	/**
	 * Bestimmt rekursiv eine Liste aller {@link DoMitVerbLinieEditPart} Elemente
	 * eines {@link EditPart}s.
	 * 
	 * @param ep
	 *          Der zu prfende {@link EditPart}.
	 * @return Liste aller {@link DoMitVerbLinieEditPart}-Objekte des
	 *         {@link EditPart}s.
	 */
	private static LinkedList<DoMitVerbLinieEditPart> bestimmeAlleDoRekursiv(
			EditPart ep)
	{
		LinkedList<DoMitVerbLinieEditPart> lDo = new LinkedList<DoMitVerbLinieEditPart>();

		if (ep != null)
		{
			if (ep instanceof DoMitVerbLinieEditPart)
			{
				DoMitVerbLinieEditPart doMitVerbLinieEditPart = (DoMitVerbLinieEditPart) ep;
				lDo.add(doMitVerbLinieEditPart);
			}

			@SuppressWarnings("unchecked")
			List lChildren = ep.getChildren();
			if ((lChildren != null) && (lChildren.size() > 0))
			{
				for (Object oChild : lChildren)
				{
					if ((oChild != null) && (oChild instanceof EditPart))
					{
						EditPart childEditPart = (EditPart) oChild;
						lDo.addAll(bestimmeAlleDoRekursiv(childEditPart));
					}
				}
			}
		}

		return lDo;
	}

	/**
	 * Bestimmt alle AnzeigenModelle, die in einer Darstellung vorhanden sind.
	 * 
	 * @param contents
	 *          {@link AnsichtEditPart} mit allen Inhalten der Darstellung.
	 * @param kontext
	 *          Kontext (IST, SOLL etc.) der Darstellung.
	 * @return {@link ConcurrentHashMap}, Key=Systemobjekt der Anzeige, Value=
	 *         {@link AnzeigeModelTypAbstract}.
	 */
	public static ConcurrentHashMap<SystemObject, AnzeigeModelTypAbstract> bestimmeAlleAnzeigenModelle(
			AnsichtEditPart contents, int kontext)
	{

		if (contents == null)
			throw new IllegalArgumentException("Parameter ist null: " + contents);

		ConcurrentHashMap<SystemObject, AnzeigeModelTypAbstract> mapAnzeigen = new ConcurrentHashMap<SystemObject, AnzeigeModelTypAbstract>();

		Ansicht ansicht = contents.getModel();

		StilisierteDarstellung darstellung = (StilisierteDarstellung) ansicht
				.getDarstellung();
		EList<Ebene> ebenen = darstellung.getEbenen();
		for (Ebene ebene : ebenen)
		{
			EList<DoModel> doObjekte = ebene.getDoObjekte();
			for (DoModel dom : doObjekte)
			{
				if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
				{
					AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract) dom;

					int aqKontext = aq.getKontext();
					if (kontext == aqKontext)
					{
						EList<AnzeigeModelTypAbstract> anzeigen = aq.getKomponenten();
						for (AnzeigeModelTypAbstract anzeige : anzeigen)
						{
							SystemObject soAnz = anzeige.getSystemObject();
							if (soAnz != null)
							{
								mapAnzeigen.put(soAnz, anzeige);
							}
						}
					}
				} else if (dom instanceof AnzeigeModelTypAbstract)
				{
					/*
					 * Eine einzelne Anzeige auf oberster Ebene! JH
					 */
					AnzeigeModelTypAbstract anzeige = (AnzeigeModelTypAbstract) dom;
					SystemObject soAnzeige = anzeige.getSystemObject();
					if (soAnzeige != null)
					{
						mapAnzeigen.put(soAnzeige, anzeige);
					}
				}
			}
		}

		return mapAnzeigen;
	}

	/**
	 * Bestimmt alle AQ-Modelle einer Zustandsdarstellung. Es werden natrlich nur
	 * die Darstellungsobjekte bercksichtigt, bei denen eine SystemObject fr den
	 * AQ hinterlegt ist.
	 * 
	 * @param contents
	 *          {@link AnsichtEditPart} mit allen Inhalten der
	 *          Zustandsdarstellung.
	 * @param kontext
	 *          Kontext (IST, SOLL etc.) der Darstellung.
	 * @return {@link ConcurrentHashMap}, Key=Systemobjekt des AQs, Value=
	 *         {@link AnzeigeQuerschnittModelTypAbstract}.
	 */
	public static ConcurrentHashMap<SystemObject, AnzeigeQuerschnittModelTypAbstract> bestimmeAlleAQModelle(
			AnsichtEditPart contents, int kontext)
	{

		if (contents == null)
			throw new IllegalArgumentException("Parameter ist null: " + contents);

		ConcurrentHashMap<SystemObject, AnzeigeQuerschnittModelTypAbstract> mapAQs = new ConcurrentHashMap<SystemObject, AnzeigeQuerschnittModelTypAbstract>();

		Ansicht ansicht = contents.getModel();

		StilisierteDarstellung darstellung = (StilisierteDarstellung) ansicht
				.getDarstellung();
		EList<Ebene> ebenen = darstellung.getEbenen();
		for (Ebene ebene : ebenen)
		{
			EList<DoModel> doObjekte = ebene.getDoObjekte();
			for (DoModel dom : doObjekte)
			{
				if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
				{
					AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract) dom;
					SystemObject soAQ = aq.getSystemObject();
					if (soAQ != null)
					{
						mapAQs.put(soAQ, aq);
					}
				}
			}
		}

		return mapAQs;
	}

}
