/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.editparts;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PlatformUI;

import com.bitctrl.lib.eclipse.draw2d.FigureUtilities2;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.DoModelMitVerbLinie;
import de.bsvrz.buv.plugin.anlagendarstellung.MessQuerschnittModel;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.MessQuerschnittFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.LegendeDefinitionVerkehrsLage;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.ILegendeBaustein2;
import de.bsvrz.buv.plugin.anlagendarstellung.model.SituationTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.util.LegendeHelper;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ILegendeBaustein;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.LegendeBaustein;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;

/**
 * Implementierung fr einen {@link DoMitVerbLinieEditPart} der ein Model-Objekt 
 * vom Typ {@link MessQuerschnittModel} verwendet.
 * 
 * @author Pittner
 * 
 */
public class MessQuerschnittEditPart extends DoMitVerbLinieEditPart implements ILegendeBaustein2
{
  private Image image;
  private ArrayList<ILegendeBaustein> legendenBausteine;

  @Override
  protected IFigure createFigure()
  {

    Dimension size = getCastedModel().getSize();

    if (size == null)
    {
      size = new Dimension(51, 4);
      getCastedModel().setSize(size);
    }

    MessQuerschnittFigure fig = new MessQuerschnittFigure("");
    fig.setSize(size);

    addToolTipListener(fig);

    return fig;
  }

  public void dataUpdate(ResultData result)
  {

    final MessQuerschnittModel model = getCastedModel();
    final MessQuerschnittFigure inhalt = (MessQuerschnittFigure)getFigure();/* .getInhalt(); */

    AttributeGroup atg = result.getDataDescription().getAttributeGroup();

    if (atg.getPid().equals("atg.strfallZustand"))
    {
      Data data = result.getData();
      if (data != null)
      {

        String s = data.getItem("Situation").asTextValue().getValueText();
        final SituationTyp situation = SituationTyp.setTyp(s);

        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
          @Override
          public void run()
          {
            if (situation.getType().equals(SituationTyp.STOERUNG.getType()))
            {
              inhalt.setForegroundColor(ColorConstants.black);
              inhalt.setBackgroundColor(ColorConstants.white);
            }
            else if (situation.getType().equals(SituationTyp.KEINE_AUSSAGE.getType()))
            {
              inhalt.setForegroundColor(ColorConstants.black);
              inhalt.setBackgroundColor(ColorConstants.gray);
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_FREI.getType()))
            {
              inhalt.setForegroundColor(ColorConstants.black);
              inhalt.setBackgroundColor(ColorConstants.lightGreen);
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_LEBHAFT.getType()))
            {
              inhalt.setForegroundColor(ColorConstants.black);
              inhalt.setBackgroundColor(ColorConstants.green);
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_DICHT.getType()))
            {
              inhalt.setForegroundColor(ColorConstants.black);
              inhalt.setBackgroundColor(ColorConstants.yellow);
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_STOCKEND.getType()))
            {
              inhalt.setForegroundColor(ColorConstants.black);
              inhalt.setBackgroundColor(ColorConstants.red);
            }
            else if (situation.getType().equals(SituationTyp.VERKEHR_ZAEHFLIESSEND.getType()))
            {
              inhalt.setForegroundColor(ColorConstants.black);
              inhalt.setBackgroundColor(ColorConstants.orange);
            }
            else if (situation.getType().equals(SituationTyp.STAU.getType()))
            {
              inhalt.setForegroundColor(ColorConstants.black);
              inhalt.setBackgroundColor(ColorConstants.red);
            }            
          }
        });

      }
      else
      {
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
          @Override
          public void run()
          {
            inhalt.setForegroundColor(ColorConstants.black);
            inhalt.setBackgroundColor(ColorConstants.gray);
          }
        });
      }
    }
  }

  @Override
  public void notifyChanged(Notification notification)
  {
    super.notifyChanged(notification);

    final Object notifier = notification.getNotifier();
    final int type = notification.getEventType();
    if (notifier instanceof MessQuerschnittModel)
    {
      final int featureID = notification.getFeatureID(DoModel.class);
      switch (type)
      {
        case Notification.SET:
          switch (featureID)
          {
            case AnlagendarstellungPackage.FAHR_STREIFEN_MODEL_ABSTRACT__SIZE:
              refresh();
              break;
            case AnlagendarstellungPackage.FAHR_STREIFEN_MODEL_ABSTRACT__LOCATION:
              refresh();
              break;
          }

          break;
        default:
      }
    }

    if (AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__DATA_UPDATE.equals(notification.getFeature()))
    {
      ResultData result = (ResultData)notification.getNewValue();
      dataUpdate(result);
    }
  }

  /**
   * We don't need to explicitly handle refresh visuals because the times when this needs to be done it is handled by
   * the table e.g. handleNameChange()
   */
  protected void refreshVisuals()
  {
    Point pos = getModel().getLocation();
    Dimension size = getCastedModel().getSize();
    if (pos != null && size != null)
    {
      Rectangle constraint = new Rectangle(pos, size);

      if (constraint != null)
        ((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), constraint);
      else
      {
        ((GraphicalEditPart)getParent()).setLayoutConstraint(this, getFigure(), new Rectangle(pos.x, pos.y, -1, -1));
      }
    }
  }

  @Override
  protected void createEditPolicies()
  {
    super.createEditPolicies();
    installEditPolicy(EditPolicy.LAYOUT_ROLE, new ResizableEditPolicy());
  }

  /**
   * @return
   */
  protected MessQuerschnittModel getCastedModel()
  {
    MessQuerschnittModel model = null;
    if (getModel() instanceof MessQuerschnittModel)
    {
      model = (MessQuerschnittModel)getModel();
    }
    return model;
  }

  /** {@inheritDoc} */
  @Override
  public List<ILegendeBaustein> getBausteine() {
    if (legendenBausteine == null) {
      legendenBausteine = new ArrayList<ILegendeBaustein>();
      for (Object listObj : getDefinition())
      {
        legendenBausteine.addAll(LegendeHelper.erzeugeLegendeBausteine(listObj));           
      }
      
    }

    return legendenBausteine;
  }

  @Override
  public Image getIcon()
  { 
    return null;
  }

  @Override
  public String getText()
  {
    // TODO Auto-generated method stub
    return getModel().getDoTyp().getName();
  }

  @Override
  public String getId()
  {
    // TODO Auto-generated method stub
    return getText();
  }
  
  @Override
  public List getDefinition()
  {
    List list = new ArrayList();
    list.add(LegendeDefinitionVerkehrsLage.class);
    return list;
  }

}
