/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.properties;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import de.bsvrz.buv.plugin.dobj.properties.AbstractSection;

/**
 * Abstrakte Implementierung einer Sektion zur Darstellung von Eigenschaften eines ELements eines Darstellungsobjekttyps
 * innerhab einer {@link TabbedPropertySheetPage}.
 * 
 * @author Pittner
 * @param <T>
 */
public abstract class AbstractDobjSection<T extends EObject> extends AbstractSection<T>
{

  /** berschreibt die Standardbreite, fr den DOT-Editor. */
  @SuppressWarnings("hiding")
  protected static final int STANDARD_LABEL_WIDTH = 100;

  /**
   * markiert, ob nderungen in den Datenfeldern an die Listener weitergeleitet werden sollen.
   */
  private boolean aktiv = true;

  /** aktualisiert die Daten, die in der Sektion dargestellt werden. */
  protected abstract void aktualisiereDaten();

  @Override
  public final void createControls(final Composite parent, final TabbedPropertySheetPage aTabbedPropertySheetPage)
  {
    super.createControls(parent, aTabbedPropertySheetPage);

    final Composite panel = getWidgetFactory().createFlatFormComposite(parent);
    final FormLayout layout = new FormLayout();
    layout.marginHeight = 1;
    layout.marginLeft = 3;
    layout.marginRight = 3;
    panel.setLayout(layout);

    createUI(panel);
    installListeners();
  }

  /**
   * erzeugt die Elemente mit denen die Daten der Sektion dem Benutzer prsentiert werden.
   * 
   * @param panel
   *          das umgebenede Element
   */
  protected abstract void createUI(Composite panel);

  /**
   * installiert die Listener, mit denen Eingaben innerhalb der Sektion weitervermittelt werden.
   */
  protected abstract void installListeners();

  /**
   * markiert, ob nderungen der Daten innerhalb der Sektion aktualisiert werden sollen. ber diese Eigenschaft knnen
   * Listener innerhalb einer Sektion deaktiviert werden, um Rekursionen beim Setzen und Aktualisieren der Daten zu
   * verhindern.
   * 
   * @return den Zustand
   */
  protected boolean isAktiv()
  {
    return aktiv;
  }

  @Override
  public final void refresh()
  {

    try
    {
      aktiv = false;
      aktualisiereDaten();
    }
    finally
    {
      aktiv = true;
    }
  }
}
