/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.properties;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;

import com.bitctrl.lib.eclipse.emf.eclipse.model.EColor;
import com.bitctrl.lib.eclipse.emf.eclipse.model.EclipseFactory;
import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.doeditor.model.DoeditorPackage;
import de.bsvrz.buv.plugin.doeditor.model.PunktForm;

 
/**
 * Sektion zur Darstellung der Zeichenfarbe einer {@link PunktForm} innerhalb einer TabbedPropertySheetPage.
 * 
 * @author Pittner
 * @param <T>
 */
public class ForegroundColorSection extends AbstractDobjSection<PunktForm>
{

  /** Textfeld zur Anzeige der aktuell ausgewhlten Farbe. */
  private Text colorText;

  /** Hintergrundfarbe des Textfeldes zur Darstellung der ausgewhlten Farbe. */
  private Color color;

  /** Textfarbe zur Darstellung der ausgewhlten Farbe. */
  private Color textColor;

  /** Schaltflche zum Start des Farbauswahldialogs. */
  private Button colorButton;

  @Override
  protected void aktualisiereDaten()
  {
    disposeColors();
    final RGB rgb = getElement().getForegroundColor().getRgb();
    final RGB opposite = new RGB(255 - rgb.red, 255 - rgb.green, 255 - rgb.blue);
    color = new Color(Display.getDefault(), rgb);
    textColor = new Color(Display.getDefault(), opposite);
    colorText.setText(color.toString());
    colorText.setBackground(color);
    colorText.setForeground(textColor);
  }

  /** gibt die angelegten Farbressourcen frei. */
  private void disposeColors()
  {
    if (color != null)
    {
      color.dispose();
    }
    if (textColor != null)
    {
      textColor.dispose();
    }
  }

  @Override
  public void createUI(final Composite panel)
  {
    FormData data;

    colorText = getWidgetFactory().createText(panel, "");
    data = new FormData();
    data.left = new FormAttachment(0, AbstractDobjSection.STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(90, 0);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    colorText.setLayoutData(data);
    colorText.setEditable(false);

    final CLabel label = getWidgetFactory().createCLabel(panel, "Farbe:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(colorText, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(colorText, 0, SWT.CENTER);
    label.setLayoutData(data);

    colorButton = getWidgetFactory().createButton(panel, "...", SWT.PUSH);
    data = new FormData();
    data.left = new FormAttachment(colorText, 0);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(colorText, 0, SWT.CENTER);
    colorButton.setLayoutData(data);
  }

  @Override
  public void dispose()
  {
    disposeColors();
    super.dispose();
  }

  @Override
  protected void installListeners()
  {
    colorButton.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(final SelectionEvent e)
      {
        if (isAktiv())
        {
          final ColorDialog dialog = new ColorDialog(getPart().getSite().getShell());
          dialog.setRGB(getElement().getForegroundColor().getRgb());
          final RGB newRGB = dialog.open();
          if (newRGB != null)
          {
            final EColor newColor = EclipseFactory.eINSTANCE.createEColor();
            newColor.setRgb(newRGB);
            final SetCommand cmd = new SetCommand(getElement(), DoeditorPackage.Literals.PUNKT_FORM__FOREGROUND_COLOR,
                newColor);
            getCommandStack().execute(cmd);
          }
        }
      }
    });
  }
}
