/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.properties;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.bitctrl.lib.eclipse.emf.eclipse.model.HorizontalAlignment;
import com.bitctrl.lib.eclipse.emf.eclipse.model.VerticalAlignment;
import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.doeditor.model.DoeditorPackage;
import de.bsvrz.buv.plugin.doeditor.model.TextForm;

/**
 * Sektion zur Darstellung der Ausrichtung des Textes einer {@link TextForm} innerhalb einer TabbedPropertySheetPage.
 * 
 * @author Pittner
 * @param <T>
 */
public class TextAlignmentSection extends AbstractDobjSection<TextForm>
{

  /** Auswahlliste fr die horizontale Ausrichtung eines Textes. */
  private ComboViewer horizontal;

  /** Auswahlliste fr die vertikale Ausrichtung eines Textes. */
  private ComboViewer vertikal;

  @Override
  protected void aktualisiereDaten()
  {
    horizontal.setSelection(new StructuredSelection(getElement().getHorizontalAlignment()));
    vertikal.setSelection(new StructuredSelection(getElement().getVerticalAlignment()));

  }

  /**
   * liefert die ausgewhlte horizontale Ausrichtung.
   * 
   * @return die Ausrichtung
   */
  private HorizontalAlignment ausgewaehltHorizontal()
  {
    final IStructuredSelection auswahl = (IStructuredSelection)horizontal.getSelection();
    if (!auswahl.isEmpty())
    {
      return (HorizontalAlignment)auswahl.getFirstElement();
    }

    return null;
  }

  /**
   * liefert die ausgewhlte vertikale Ausrichtung.
   * 
   * @return die Ausrichtung
   */
  private VerticalAlignment ausgewaehltVertikal()
  {
    final IStructuredSelection auswahl = (IStructuredSelection)vertikal.getSelection();
    if (!auswahl.isEmpty())
    {
      return (VerticalAlignment)auswahl.getFirstElement();
    }

    return null;
  }

  @Override
  public void createUI(final Composite parent)
  {
    final TabbedPropertySheetWidgetFactory factory = getWidgetFactory();
    FormData data;
    CLabel label;

    final CCombo horCombo = factory.createCCombo(parent);
    data = new FormData();
    data.left = new FormAttachment(0, AbstractDobjSection.STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(50, 0);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    horCombo.setLayoutData(data);

    horizontal = new ComboViewer(horCombo);
    horizontal.setContentProvider(new ArrayContentProvider());
    horizontal.setSorter(new ViewerSorter());
    horizontal.setInput(HorizontalAlignment.values());

    label = factory.createCLabel(parent, "Horizontale Ausrichtung:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(horCombo, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(horCombo, 0, SWT.CENTER);
    label.setLayoutData(data);

    final CCombo verCombo = factory.createCCombo(parent);
    data = new FormData();
    data.left = new FormAttachment(horCombo, AbstractDobjSection.STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(horCombo, 0, SWT.CENTER);
    verCombo.setLayoutData(data);

    vertikal = new ComboViewer(verCombo);
    vertikal.setContentProvider(new ArrayContentProvider());
    vertikal.setSorter(new ViewerSorter());
    vertikal.setInput(VerticalAlignment.values());

    label = factory.createCLabel(parent, "Vertikale Ausrichtung:");
    data = new FormData();
    data.left = new FormAttachment(50, ITabbedPropertyConstants.HSPACE);
    data.right = new FormAttachment(verCombo, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(verCombo, 0, SWT.CENTER);
    label.setLayoutData(data);
  }

  @Override
  protected void installListeners()
  {

    horizontal.addSelectionChangedListener(new ISelectionChangedListener() {
      @Override
      public void selectionChanged(final SelectionChangedEvent event)
      {
        if (isAktiv())
        {
          final HorizontalAlignment ausrichtung = ausgewaehltHorizontal();
          if (ausrichtung != null)
          {
            final SetCommand cmd = new SetCommand(getElement(),
                DoeditorPackage.Literals.TEXT_FORM__HORIZONTAL_ALIGNMENT, ausrichtung);
            getCommandStack().execute(cmd);
          }
        }
      }
    });

    vertikal.addSelectionChangedListener(new ISelectionChangedListener() {
      @Override
      public void selectionChanged(final SelectionChangedEvent event)
      {
        if (isAktiv())
        {
          final VerticalAlignment ausrichtung = ausgewaehltVertikal();
          if (ausrichtung != null)
          {
            final SetCommand cmd = new SetCommand(getElement(), DoeditorPackage.Literals.TEXT_FORM__VERTICAL_ALIGNMENT,
                ausrichtung);
            getCommandStack().execute(cmd);
          }
        }
      }
    });

  }
}
