/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.dobj.section;

import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertyConstants;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetWidgetFactory;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnschlussStelleModel;
import de.bsvrz.buv.plugin.dobj.properties.AbstractSection;


/**
 * Sektion zur Darstellung einer Anschlussstelle in der Properties View.
 * 
 * @author Pittner
 * @param <T>
 */
public class AnschlussStelleExtraSection extends AbstractSection<AnschlussStelleModel>
{

  private Text anzeigeName;

  private CCombo kontext;

  @Override
  public void createControls(final Composite parent, final TabbedPropertySheetPage aTabbedPropertySheetPage)
  {
    super.createControls(parent, aTabbedPropertySheetPage);

    final TabbedPropertySheetWidgetFactory factory = getWidgetFactory();
    final Composite container = factory.createFlatFormComposite(parent);

    FormData data;

    anzeigeName = factory.createText(container, "");
    data = new FormData();
    data.left = new FormAttachment(0, STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(0, ITabbedPropertyConstants.VSPACE);
    anzeigeName.setLayoutData(data);
    anzeigeName.setTextLimit(4);

    final CLabel label = factory.createCLabel(container, "AnschlussStelleName:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(anzeigeName, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(anzeigeName, 0, SWT.CENTER);
    label.setLayoutData(data);

    anzeigeName.addModifyListener(new ModifyListener() {

      @Override
      public void modifyText(ModifyEvent e)
      {
        if (!anzeigeName.getText().equals(getElement().getBezeichnung()))
        {
          // XXX: Der Text wurde in der PropertyPage gendert, also
          // versenden wir ein neues Command an das Modell-Objekt
          EStructuralFeature feature = AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__BEZEICHNUNG;

          String text = anzeigeName.getText();

          getElement().setBezeichnung(text);

          if (text != null)
          {
            final SetCommand cmd = new SetCommand(getElement(), feature, text);
            getCommandStack().execute(cmd);
          }

        }
      }
    });

    kontext = factory.createCCombo(container);
    kontext.add("Zufahrt Links", 0);
    kontext.add("Zufahrt Rechts", 1);
    kontext.add("Abfahrt Links", 2);
    kontext.add("Abfahrt Rechts", 3);
    data = new FormData();
    data.left = new FormAttachment(0, STANDARD_LABEL_WIDTH);
    data.right = new FormAttachment(100, 0);
    data.top = new FormAttachment(anzeigeName, ITabbedPropertyConstants.VSPACE);
    kontext.setLayoutData(data);

    final CLabel label2 = factory.createCLabel(container, "Kontext:");
    data = new FormData();
    data.left = new FormAttachment(0, 0);
    data.right = new FormAttachment(kontext, -ITabbedPropertyConstants.HSPACE);
    data.top = new FormAttachment(kontext, 0, SWT.CENTER);
    label2.setLayoutData(data);

    kontext.addSelectionListener(new SelectionAdapter() {

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        if (!kontext.getSelection().equals(getElement().getKontext()))
        {
          // XXX: Der Text wurde in der PropertyPage gendert, also
          // versenden wir ein neues Command an das Modell-Objekt
          EStructuralFeature feature = AnlagendarstellungPackage.Literals.FAHR_STREIFEN_MODEL_ABSTRACT__KONTEXT;

          int i = kontext.getSelectionIndex();
          getElement().setKontext(i);

          final SetCommand cmd = new SetCommand(getElement(), feature, i);
          getCommandStack().execute(cmd);
        }

      }

    });
  }

  @Override
  public void refresh()
  {
    super.refresh();
    String name = getElement().getBezeichnung();
    if (name != null)
    {
      anzeigeName.setText(name);
      anzeigeName.setSelection(name.length());
    }
    else
    {
      anzeigeName.setText("");
    }

    int i = getElement().getKontext();
    kontext.select(i);
  }
}
