/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.editor;

import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtensionRegistry;
import org.eclipse.core.runtime.Platform;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.common.util.EList;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.commands.CommandStackListener;
import org.eclipse.gef.ui.actions.ActionBarContributor;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.DeleteAction;
import org.eclipse.gef.ui.actions.PrintAction;
import org.eclipse.gef.ui.actions.RedoAction;
import org.eclipse.gef.ui.actions.SelectAllAction;
import org.eclipse.gef.ui.actions.UndoAction;
import org.eclipse.gef.ui.actions.UpdateAction;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.gef.ui.parts.SelectionSynchronizer;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.ActionDescriptor;
import org.eclipse.ui.internal.PluginAction;
import org.eclipse.ui.internal.registry.IWorkbenchRegistryConstants;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.legende.DoModelLegendeComposite;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.DarstellungsSpalte;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.rw.bitctrl.eclipse.legende.ICustomLegende;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.NetzCacheExtended;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.StreckenAbschnittCache;

/**
 * Wrapper-Klasse fr den Gebrauch eines {@link GraphicalViewer} in einer View
 * 
 * @author Pittner
 * 
 */
public abstract class GraphicalEditorToView extends ViewPart implements CommandStackListener, ISelectionListener
{

  private static class ActionIDList extends ArrayList
  {
    public boolean add(Object o)
    {
      if (o instanceof IAction)
      {
        try
        {
          IAction action = (IAction)o;
          o = action.getId();
          throw new IllegalArgumentException("Action IDs should be added to lists, not the action: " + action); //$NON-NLS-1$
        }
        catch (IllegalArgumentException exc)
        {
          exc.printStackTrace();
        }
      }
      return super.add(o);
    }
  }
 
  private DefaultEditDomainViewPart editDomain;

  protected GraphicalViewer graphicalViewer;

  private ActionRegistry actionRegistry;

  private SelectionSynchronizer synchronizer;

  private List selectionActions = new ActionIDList();

  private List stackActions = new ActionIDList();

  private List propertyActions = new ActionIDList();

  private NetzCacheExtended _netzCacheExtended;

  private StreckenAbschnittCache _streckenCache;

  private boolean _bAnmeldungPicker;

  private PropertyChangeListener _pclCache;
  
  
  public void restoreFromPrefs()
  {
    IPreferenceStore preferenceStore = getPreferenceStore();
    if (preferenceStore != null)
    {
      AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();
      Ansicht model = contents.getModel();
      StilisierteDarstellung darstellung = (StilisierteDarstellung)model.getDarstellung();
      EList<DarstellungsSpalte> spalten = darstellung.getSpalten();
  
      SortedMap<Integer, DarstellungsSpalte> map = new TreeMap<Integer, DarstellungsSpalte>();
  
      for (DarstellungsSpalte ds : spalten)
      {
        String key = getViewSite().getId() + "" + getViewSite().getSecondaryId();
        key += ds.getName().replaceAll(" ", "");
        String s = preferenceStore.getString(key);
        
//        System.out.println("-> get Prefs: " + key + " -> " + s);
        
        if (s != null && s.contains(";"))
        {
          String[] split = s.split(";");
          if (split.length > 1)
          {
            Integer index = new Integer(split[0]);
            Boolean visible = new Boolean(split[1]);
            ds.setVisible(visible);
            map.put(index, ds);
          }
        }
      }
  
      for (Map.Entry<Integer, DarstellungsSpalte> me : map.entrySet())
      {
        spalten.move(me.getKey(), me.getValue());
      }
    }
  }
  
  public void einstellungenSichern()
  {    
    IPreferenceStore preferenceStore = getPreferenceStore();
    if (preferenceStore != null)
    {
      AnsichtEditPart contents = (AnsichtEditPart)graphicalViewer.getContents();
      Ansicht model = contents.getModel();
      StilisierteDarstellung darstellung = (StilisierteDarstellung)model.getDarstellung();
      EList<DarstellungsSpalte> spalten = darstellung.getSpalten();
      for (int i = 0; i < spalten.size(); i++)
      {
        DarstellungsSpalte ds = spalten.get(i);
        String value = i + ";" + ds.isVisible();
        String key = getViewSite().getId() + "" + getViewSite().getSecondaryId();
        key += ds.getName().replaceAll(" ", "");
        
//        System.out.println("put Prefs: " + key + " -> " + value);
        
        preferenceStore.putValue(key, value);
      }
    }
  }

  private IPreferenceStore getPreferenceStore()
  {
    return Activator.getDefault().getPreferenceStore();
  }

  /**
   * Constructs the editor part
   */
  public GraphicalEditorToView()
  {
    addPartPropertyListener(new IPropertyChangeListener() {
      @Override
      public void propertyChange(PropertyChangeEvent event)
      {
        String property = event.getProperty();
        if (property.equals("NAME_PROP"))
        {
          Object value = event.getNewValue();
          if (value instanceof String)
          {
            String partName = (String)value;
            setPartName(partName);
          }
        }
      }
    });
    
    _netzCacheExtended = StartupPlugin.getDefault().getDefaultNetzCacheExtended();
    _streckenCache = StartupPlugin.getDefault().getDefaultStreckenAbschnittCache();
    
  }

  /**
   * When the command stack changes, the actions interested in the command stack are updated.
   * 
   * @param event
   *          the change event
   */
  public void commandStackChanged(EventObject event)
  {
    updateActions(stackActions);
  }

  /**
   * Called to configure the graphical viewer before it receives its contents. This is where the root editpart should be
   * configured. Subclasses should extend or override this method as needed.
   */
  protected void configureGraphicalViewer()
  {
    getGraphicalViewer().getControl().setBackground(ColorConstants.listBackground);
  }

  /**
   * Creates actions for this editor. Subclasses should override this method to create and register actions with the
   * {@link ActionRegistry}.
   */
  protected void createActions()
  {
    ActionRegistry registry = getActionRegistry();
    IAction action;

    action = new UndoAction(this);
    registry.registerAction(action);
    getStackActions().add(action.getId());

    action = new RedoAction(this);
    registry.registerAction(action);
    getStackActions().add(action.getId());

    action = new SelectAllAction(this);
    registry.registerAction(action);

    action = new DeleteAction((IWorkbenchPart)this);
    registry.registerAction(action);
    getSelectionActions().add(action.getId());

    registry.registerAction(new PrintAction(this));
  }

  /**
   * Creates the GraphicalViewer on the specified <code>Composite</code>.
   * 
   * @param parent
   *          the parent composite
   */
  protected void createGraphicalViewer(Composite parent)
  {
    GraphicalViewer viewer = new ScrollingGraphicalViewer();
    viewer.createControl(parent);
    setGraphicalViewer(viewer);
    configureGraphicalViewer();
    hookGraphicalViewer();
    initializeGraphicalViewer();
  }

  /**
   * Realizes the Editor by creating it's Control.
   * <P>
   * WARNING: This method may or may not be called by the workbench prior to {@link #dispose()}.
   * 
   * @param parent
   *          the parent composite
   */
  public void createPartControl(Composite parent)
  {   
    createGraphicalViewer(parent);    
  }
  
  

  
  
  /**
   * @see org.eclipse.ui.IWorkbenchPart#dispose()
   */
  protected void anmeldenSituationen(ClientDavInterface connection)
  {
    
    final List<String> list2 = new ArrayList<String>();
        
    if (connection != null){
      SystemObjectType type = connection.getDataModel().getType("typ.situation");
      if (type != null){
        List<SystemObject> elements = type.getElements();
        for (SystemObject so : elements)
        {
          list2.add(so.getPid());
        }
      }
      final AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.situationsEigenschaften");
      final Aspect asp = connection.getDataModel().getAspect("asp.parameterSoll");
      
      
      if (_netzCacheExtended.isInitialisiert() && _streckenCache.isInitialisiert()){
        DaVKommunikationDobj.getInstanz().anmeldenAufDatenPicker(list2, atg.getPid(), asp.getPid());
        _bAnmeldungPicker = true;
      }

      _pclCache = new PropertyChangeListener() {
        
        @Override
        public void propertyChange(java.beans.PropertyChangeEvent evt)
        {
          if (evt != null){
            String name = evt.getPropertyName();
                        
            if (name.equals(NetzCacheExtended.PROP_STATE)){
              if (_netzCacheExtended.isInitialisiert() && _streckenCache.isInitialisiert() && !_bAnmeldungPicker){
                DaVKommunikationDobj.getInstanz().anmeldenAufDatenPicker(list2, atg.getPid(), asp.getPid()); 
                _bAnmeldungPicker = true;
              }
             }
          }
        }
      };
      
      _netzCacheExtended.addPropertyChangeListener(_pclCache);
      _streckenCache.addPropertyChangeListener(_pclCache);     
    }    
  }
  

  /**
   * @see org.eclipse.ui.IWorkbenchPart#dispose()
   */
  public void dispose()
  {
    getSite().getWorkbenchWindow().getSelectionService().removeSelectionListener(this);
    getEditDomain().setActiveTool(null);
    getActionRegistry().dispose();
    super.dispose();
  }

  /**
   * Does nothing be default. This method should be overridden if {@link #isSaveAsAllowed()} has been overridden to
   * return <code>true</code>.
   * 
   * @see org.eclipse.ui.ISaveablePart#doSaveAs()
   */
  public void doSaveAs()
  {
    throw new RuntimeException("doSaveAs must be overridden"); //$NON-NLS-1$
  }

  /**
   * @see org.eclipse.ui.part.WorkbenchPart#firePropertyChange(int)
   */
  protected void firePropertyChange(int property)
  {
    super.firePropertyChange(property);
    updateActions(propertyActions);
  }

  /**
   * Lazily creates and returns the action registry.
   * 
   * @return the action registry
   */
  protected ActionRegistry getActionRegistry()
  {
    if (actionRegistry == null)
      actionRegistry = new ActionRegistry();
    return actionRegistry;
  }

  /**
   * Returns the adapter for the specified key.
   * 
   * <P>
   * <EM>IMPORTANT</EM> certain requests, such as the property sheet, may be made before or after
   * {@link #createPartControl(Composite)} is called. The order is unspecified by the Workbench.
   * 
   * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
   */
  public Object getAdapter(Class type)
  {
    if (type == GraphicalViewer.class)
      return getGraphicalViewer();
    if (type == ActionRegistry.class)
      return getActionRegistry();
    if (type == EditPart.class && getGraphicalViewer() != null)
      return getGraphicalViewer().getRootEditPart();
    if (type == IFigure.class && getGraphicalViewer() != null)
      return ((GraphicalEditPart)getGraphicalViewer().getRootEditPart()).getFigure();   
    return super.getAdapter(type);
  }

  /**
   * Returns the command stack.
   * 
   * @return the command stack
   */
  protected CommandStack getCommandStack()
  {    
    return null;
  }

  /**
   * Returns the edit domain.
   * 
   * @return the edit domain
   */
  protected DefaultEditDomainViewPart getEditDomain()
  {
    return editDomain;
  }

  /**
   * Returns the graphical viewer.
   * 
   * @return the graphical viewer
   */
  protected GraphicalViewer getGraphicalViewer()
  {
    return graphicalViewer;
  }

  /**
   * Returns the list of {@link IAction IActions} dependant on property changes in the Editor. These actions should
   * implement the {@link UpdateAction} interface so that they can be updated in response to property changes. An
   * example is the "Save" action.
   * 
   * @return the list of property-dependant actions
   */
  protected List getPropertyActions()
  {
    return propertyActions;
  }

  /**
   * Returns the list of <em>IDs</em> of Actions that are dependant on changes in the workbench's
   * {@link ISelectionService}. The associated Actions can be found in the action registry. Such actions should
   * implement the {@link UpdateAction} interface so that they can be updated in response to selection changes.
   * 
   * @see #updateActions(List)
   * @return the list of selection-dependant action IDs
   */
  protected List getSelectionActions()
  {
    return selectionActions;
  }

  /**
   * Returns the selection syncronizer object. The synchronizer can be used to sync the selection of 2 or more
   * EditPartViewers.
   * 
   * @return the syncrhonizer
   */
  protected SelectionSynchronizer getSelectionSynchronizer()
  {
    if (synchronizer == null)
      synchronizer = new SelectionSynchronizer();
    return synchronizer;
  }

  /**
   * Returns the list of <em>IDs</em> of Actions that are dependant on the CommmandStack's state. The associated Actions
   * can be found in the action registry. These actions should implement the {@link UpdateAction} interface so that they
   * can be updated in response to command stack changes. An example is the "undo" action.
   * 
   * @return the list of stack-dependant action IDs
   */
  protected List getStackActions()
  {
    return stackActions;
  }

  /**
   * Hooks the GraphicalViewer to the rest of the Editor. By default, the viewer is added to the SelectionSynchronizer,
   * which can be used to keep 2 or more EditPartViewers in sync. The viewer is also registered as the
   * ISelectionProvider for the Editor's PartSite.
   */
  protected void hookGraphicalViewer()
  {
    getSelectionSynchronizer().addViewer(getGraphicalViewer());
    getSite().setSelectionProvider(getGraphicalViewer());
  }

  /**
   * Sets the site and input for this editor then creates and initializes the actions. Subclasses may extend this
   * method, but should always call <code>super.init(site, input)
   * </code>.
   * 
   * @see org.eclipse.ui.IEditorPart#init(IEditorSite, IEditorInput)
   */
  public void init(IEditorSite site, IEditorInput input) throws PartInitException
  {
    setSite(site);
    initializeActionRegistry();
  }

  /**
   * Initializes the ActionRegistry. This registry may be used by {@link ActionBarContributor ActionBarContributors}
   * and/or {@link ContextMenuProvider ContextMenuProviders}.
   * <P>
   * This method may be called on Editor creation, or lazily the first time {@link #getActionRegistry()} is called.
   */
  protected void initializeActionRegistry()
  {
    createActions();
    updateActions(propertyActions);
    updateActions(stackActions);
  }

  /**
   * Override to set the contents of the GraphicalViewer after it has been created.
   * 
   * @see #createGraphicalViewer(Composite)
   */
  protected abstract void initializeGraphicalViewer();

  /**
   * Returns <code>true</code> if the command stack is dirty
   * 
   * @see org.eclipse.ui.ISaveablePart#isDirty()
   */
  public boolean isDirty()
  {
    return getCommandStack().isDirty();
  }

  /**
   * Returns <code>false</code> by default. Subclasses must return <code>true</code> to allow {@link #doSaveAs()} to be
   * called.
   * 
   * @see org.eclipse.ui.ISaveablePart#isSaveAsAllowed()
   */
  public boolean isSaveAsAllowed()
  {
    return false;
  }

  /**
   * @see org.eclipse.ui.ISelectionListener#selectionChanged(IWorkbenchPart, ISelection)
   */
  public void selectionChanged(IWorkbenchPart part, ISelection selection)
  {
    // If not the active editor, ignore selection changed.
    if (this.equals(getSite().getPage().getActiveEditor()))
      updateActions(selectionActions);
  }

  /**
   * Sets the ActionRegistry for this EditorPart.
   * 
   * @param registry
   *          the registry
   */
  protected void setActionRegistry(ActionRegistry registry)
  {
    actionRegistry = registry;
  }

  /**
   * Sets the EditDomain for this EditorPart.
   * 
   * @param ed
   *          the domain
   */
  protected void setEditDomain(DefaultEditDomainViewPart ed)
  {
    this.editDomain = ed;
  }

  /**
   * @see org.eclipse.ui.IWorkbenchPart#setFocus()
   */
  public void setFocus()
  {
    getGraphicalViewer().getControl().setFocus();
  }

  /**
   * Sets the graphicalViewer for this EditorPart.
   * 
   * @param viewer
   *          the graphical viewer
   */
  protected void setGraphicalViewer(GraphicalViewer viewer)
  {
    // getEditDomain().addViewer(viewer);
    this.graphicalViewer = viewer;
  }

  /**
   * A convenience method for updating a set of actions defined by the given List of action IDs. The actions are found
   * by looking up the ID in the {@link #getActionRegistry() action registry}. If the corresponding action is an
   * {@link UpdateAction}, it will have its <code>update()</code> method called.
   * 
   * @param actionIds
   *          the list of IDs to update
   */
  protected void updateActions(List actionIds)
  {
    ActionRegistry registry = getActionRegistry();
    Iterator iter = actionIds.iterator();
    while (iter.hasNext())
    {
      IAction action = registry.getAction(iter.next());
      if (action instanceof UpdateAction)
        ((UpdateAction)action).update();
    }
  } 
  
  protected class Legende implements ICustomLegende {   

    public Legende()
    {
      // TODO Auto-generated constructor stub
    }
    
    @Override
    public void createControl(final Composite parent) {
      createScroll(parent, getGraphicalViewer());
    }

    @Override
    public Control getControl() {      
      if (getGraphicalViewer() != null)
        return getGraphicalViewer().getControl();
      return null;
    }
  }
  
  protected void createScroll(Composite parent, GraphicalViewer graphicalViewer)
  {
//    if (_scroll == null)
    {
      ScrolledComposite scroll = new ScrolledComposite(parent,
          SWT.V_SCROLL | SWT.H_SCROLL);
      scroll.setExpandHorizontal(true);
      scroll.setExpandVertical(true);
      scroll.setLayout(new GridLayout());
      
      
      //legendeComposite = new AnlagenStatusLegendeComposite(scroll, SWT.DOUBLE_BUFFERED);
      DoModelLegendeComposite composite = new DoModelLegendeComposite(scroll, SWT.NONE, (AnsichtEditPart)graphicalViewer.getContents());
      
      scroll.setMinSize(composite.computeSize(
          SWT.DEFAULT, SWT.DEFAULT));
      scroll.setContent(composite);      
    }
    
  }
  
  /**
   * Liest die Aktionen unter Bercksichtigung des aktuellen WorkbenchWindow ein.<br>
   * Die Aktionen werden so erzeugt, dass die Actions am SelectionProvider des aktuellen EclipseWindows hngen. <br>
   * Evtl. muss noch das aktuelle Workbenchwindow beim speichern der Aktionen bercksichtigt werden. Dies ist fr ein
   * Window aber NICHT ntig.
   */
  @SuppressWarnings("restriction")
  public IAction holeAction(String id)
  {
    IExtensionRegistry reg = Platform.getExtensionRegistry();
    IConfigurationElement[] aConfigurationElementsFor = reg.getConfigurationElementsFor("org.eclipse.ui.actionSets");
    IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    for (IConfigurationElement confEle : aConfigurationElementsFor)
    {
      String contributor = confEle.getContributor().getName();
//      if (Activator.PLUGIN_ID.equals(contributor))
      {
        IConfigurationElement[] iceAction = confEle.getChildren("action");
        for (IConfigurationElement element : iceAction)
        {
          String att = element.getAttribute("id");
          if (att.equals(id))
          {
            boolean pullDownStyle = false;
            String style = element.getAttribute(IWorkbenchRegistryConstants.ATT_STYLE);
            if (style != null)
            {
              pullDownStyle = style.equals(ActionDescriptor.STYLE_PULLDOWN);
            }
            else
            {
              String pulldown = element.getAttribute(ActionDescriptor.STYLE_PULLDOWN);
              pullDownStyle = pulldown != null && pulldown.equals("true");
            }
            
            ActionDescriptor desc = null;
            if (pullDownStyle)
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH_PULLDOWN, window);
            }
            else
            {
              desc = new ActionDescriptor(element, ActionDescriptor.T_WORKBENCH, window);
            }
            PluginAction action = desc.getAction();
            return action;

          }
        }
        
      }
    }
    return null;
  }
}
