/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;

import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;

/**
 * Figure, welche das Image einer Streckenstation reprsentiert.               
 * 
 * @author Pittner
 * 
 */
public class StreckenstationAnzeigeElementFigure extends Figure
{

  private Label _labelKilometer;

  private StreckenstationAnzeigeElementFigure _instanz;

  private Label _labelName;

  private AnzeigeElementFigure _sst;

  private static Font fFett = new Font(null, "Arial", 8, SWT.BOLD);

  private static Font fNorm = new Font(null, "Arial", 8, SWT.NORMAL);

  public StreckenstationAnzeigeElementFigure()
  {
    _instanz = this;    
    setOpaque(true);

    setLayoutManager(new XYLayout());

    Label l = new Label();
    add(l);
    setConstraint(l, new Rectangle(0, 0, -1, -1));
  }

  public StreckenstationAnzeigeElementFigure(String bezeichnung, String kilometer)
  {
    int x = 0;
    int y = 0;

    _instanz = this;

    setOpaque(false);

    setLayoutManager(new XYLayout());

    // Name der Streckenstation
    _labelName = new Label(bezeichnung);
    _labelName.setFont(fFett);
    _labelName.setLabelAlignment(GridData.END);

    add(_labelName);
    setConstraint(_labelName, new Rectangle(x, y, -1, -1));
    y += 12;

    // Kilometer
    _labelKilometer = new Label(kilometer);
    _labelKilometer.setFont(fNorm);
    _labelKilometer.setLabelAlignment(GridData.BEGINNING);

    add(_labelKilometer);
    setConstraint(_labelKilometer, new Rectangle(x, y, -1, -1));
    y += 12;

    _sst = new AnzeigeElementFigure("", AnzeigeElementTyp.STRECKEN_STATION, AnzeigeInhaltTyp.DUNKEL, true, true);
    add(_sst);
    setConstraint(_sst, new Rectangle(x, y, -1, -1));
  }

  public AnzeigeElementFigure getSst()
  {
    return _sst;
  }

  public void setSst(AnzeigeElementFigure sst)
  {
    _sst = sst;
  }

  public Label getLabelName()
  {
    return _labelName;
  }

  public void setLabelName(Label l)
  {
    _labelName = l;
  }

  public Label getLabelKilometer()
  {
    return _labelKilometer;
  }

  public void setLabelKilometer(Label l)
  {
    _labelKilometer = l;
  }
}
