/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.model;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.geometry.Rectangle;

import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ZustandsDarstellungIstView;

/**
 * Model Object of a Anzeige, wird aus Kompatibilittsgrunden bentigt
 * 
 * @deprecated  
 * @author Pittner
 * 
 */
public class AnzeigeElement extends PropertyAwareObject
{

  private String pid;

  private AnzeigeElementTyp _typ;

  private AnzeigeInhaltTyp _inhalt;

  private AnzeigeInhaltTyp _inhaltZRA = AnzeigeInhaltTyp.PROGRAMM_UNDEFINIERT;

  private AnzeigeInhaltTyp _inhaltZufluss = AnzeigeInhaltTyp.PROGRAMM_UNDEFINIERT;

  private AnzeigeInhaltTyp _inhaltLSA = AnzeigeInhaltTyp.PROGRAMM_UNDEFINIERT;;

  private AnzeigeInhaltTyp _inhaltBetriebsArt = AnzeigeInhaltTyp.PROGRAMM_UNDEFINIERT;

  private StatusTyp _status;

  private String _viewId;

  private int _x;

  private int _y;

  private boolean _aktiv = true;

  private boolean _gendert = false;

  private AnzeigeInhaltTyp _inhaltInitial;

  private VisuAnzeigeKapsel _visuAnzeigeKapsel = null;

  private List<AnzeigeInhaltTyp> _listeInhaltGestoert = null;

  public AnzeigeElement()
  {
    super();
  }

  public AnzeigeElement(AnzeigeElementTyp typ)
  {
    super();
    this._typ = typ;
    this._listeInhaltGestoert = new ArrayList<AnzeigeInhaltTyp>();
  }

  public AnzeigeElement(String name, AnzeigeElementTyp typ, AnzeigeInhaltTyp inhalt, int x, int y)
  {
    super();
    this.pid = name;
    this._typ = typ;
    this._inhalt = inhalt;
    this._inhaltInitial = inhalt;
    this._x = x;
    this._y = y;
    this._viewId = ZustandsDarstellungIstView.ID;
    _status = StatusTyp.UNDEFINERT;
    _listeInhaltGestoert = new ArrayList<AnzeigeInhaltTyp>();
  }

  public AnzeigeElement(String name, AnzeigeElementTyp typ, AnzeigeInhaltTyp inhalt, int x, int y, String viewId)
  {
    super();
    this.pid = name;
    this._typ = typ;
    this._inhalt = inhalt;
    this._inhaltInitial = inhalt;
    this._x = x;
    this._y = y;
    this._viewId = viewId;
    _status = StatusTyp.UNDEFINERT;
    _listeInhaltGestoert = new ArrayList<AnzeigeInhaltTyp>();
  }

  public AnzeigeElement(String name, AnzeigeElementTyp typ, AnzeigeInhaltTyp inhalt, int x, int y, boolean aktiv)
  {
    super();
    this.pid = name;
    this._typ = typ;
    this._inhalt = inhalt;
    this._inhaltInitial = inhalt;
    this._x = x;
    this._y = y;
    _status = StatusTyp.UNDEFINERT;
    _aktiv = aktiv;
    _listeInhaltGestoert = new ArrayList<AnzeigeInhaltTyp>();
  }

  /**
   * Returns the persistent position and size of the node. The edit part handling this node has the task to translate it
   * into a constraint that enables the draw2D system to place the figure of the node. Therefore, the coordinate system
   * of the rectangle is known only to the edit part (but in fact, in this example it is always the local coordinate
   * system of the figure of the node).
   */
  public Rectangle getConstraint()
  {
    return (Rectangle)getProperty(CONSTRAINT);
  }

  /**
   * If modified, sets name and fires off event notification
   * 
   * @param pid
   *          The name to set.
   */
  public void statusChange(String status)
  {

    AnzeigeElement old = this;
    StatusTyp neu = _status.setTyp(status);

    if (neu != null)
    {
      firePropertyChange(STATUS, old, neu);
      _status = neu;
    }
    else
      System.err.println(pid + " : Status konnte nicht gesetzt werden ->" + status);
  }

  /**
   * If modified, sets name and fires off event notification
   * 
   * @param pid
   *          The name to set.
   */
  public void inhaltChange(String inhalt)
  {
    // TODO dies funktioniert irgendwie nicht richtig
    AnzeigeElement old = this;
    AnzeigeInhaltTyp neu = null;

    if (_inhalt != null)
      neu = _inhalt.setTyp(inhalt);

    if (neu != null)
    {
      // keine nderung
      if (neu.equals(AnzeigeInhaltTyp.INAKTIV))
      {
        if (!_aktiv)
          firePropertyChange(INHALT, old, AnzeigeInhaltTyp.INAKTIV);
        else
          firePropertyChange(INHALT, old, AnzeigeInhaltTyp.DUNKEL);
      }
      else
      {
        firePropertyChange(INHALT, old, neu);
        _inhalt = neu;
      }
    }
    else
    {
      System.err.println(pid + " : Inhalt konnte nicht gesetzt werden ->" + inhalt);
    }
  }

  /**
   * If modified, sets name and fires off event notification
   * 
   * @param pid
   *          The name to set.
   */
  public void inhaltChange(String inhalt, String atg)
  {
    AnzeigeElement old = this;
    AnzeigeInhaltTyp neu = null;

    if (_inhalt != null)
      neu = _inhalt.setTyp(inhalt);

    if (neu != null)
    {

      if (atg.equals("atg.zuflussRegelungsAnlageBefehl"))
      {
        _inhaltZRA = neu;
      }
      else if (atg.equals("atg.zraZuflussBefehl"))
      {
        _inhaltZufluss = neu;
      }
      else if (atg.equals("atg.zraLichtSignalAnlageBefehl"))
      {
        _inhaltLSA = neu;
      }
      else if (atg.equals("atg.tlsZrBetriebsArt"))
      {
        _inhaltBetriebsArt = neu;
      }
      else
      {
        // System.out.println("else: " + atg);
      }

      if (_inhaltBetriebsArt.equals(AnzeigeInhaltTyp.PROGRAMM_NORMAL_BETRIEB)
          || _inhaltBetriebsArt.equals(AnzeigeInhaltTyp.PROGRAMM_BLIND_BETRIEB))
      {
        if (_inhaltZRA.equals(AnzeigeInhaltTyp.PROGRAMM_AUTOMATIK)
            && _inhaltZufluss.equals(AnzeigeInhaltTyp.PROGRAMM_DEAKTIVIERT_NEIN)
            && _inhaltLSA.equals(AnzeigeInhaltTyp.PROGRAMM_AUTOMATIK))
        {
          if (_inhaltBetriebsArt.equals(AnzeigeInhaltTyp.PROGRAMM_NORMAL_BETRIEB))
            firePropertyChange(INHALT, old, AnzeigeInhaltTyp.PROGRAMM_AUTOMATIK);
          else
            firePropertyChange(INHALT, old, AnzeigeInhaltTyp.PROGRAMM_AUTOMATIK_BLIND);
        }
        else if (_inhaltZRA.equals(AnzeigeInhaltTyp.PROGRAMM_UNDEFINIERT)
            || _inhaltZufluss.equals(AnzeigeInhaltTyp.PROGRAMM_UNDEFINIERT)
            || _inhaltLSA.equals(AnzeigeInhaltTyp.PROGRAMM_UNDEFINIERT))
        {
          firePropertyChange(INHALT, old, AnzeigeInhaltTyp.PROGRAMM_UNDEFINIERT);
        }
        else
        {
          System.out.println("4");
          if (_inhaltBetriebsArt.equals(AnzeigeInhaltTyp.PROGRAMM_NORMAL_BETRIEB))
            firePropertyChange(INHALT, old, AnzeigeInhaltTyp.PROGRAMM_DEAKTIVIERT);
          else
            firePropertyChange(INHALT, old, AnzeigeInhaltTyp.PROGRAMM_DEAKTIVIERT_BLIND);
        }
      }
      else
      {
        if (_inhaltBetriebsArt != null)
          firePropertyChange(INHALT, old, _inhaltBetriebsArt);
        else
          firePropertyChange(INHALT, old, AnzeigeInhaltTyp.PROGRAMM_UNDEFINIERT);
      }

      _inhalt = neu;
    }
    else
    {
      System.err.println(pid + " : Inhalt konnte nicht gesetzt werden ->" + inhalt);
    }
  }

  /**
   * If modified, sets name and fires off event notification
   * 
   * @param pid
   *          The name to set.
   */
  public void inhaltChangeDwiSta(VisuAnzeigeKapsel inhalt)
  {

    AnzeigeElement old = this;

    if (inhalt != null)
    {
      firePropertyChange(INHALT, old, inhalt);
    }
    else
    {
      System.err.println(pid + " : Inhalt konnte nicht gesetzt werden -> kapsel ist null");
    }

  }

  /**
   * If modified, sets name and fires off event notification
   * 
   * @param pid
   *          The name to set.
   */
  public void showAuswahlFigure()
  {

    firePropertyChange(AUSWAHL, this, "");

  }

  /**
   * @return Returns the name.
   */
  public String getPid()
  {
    return pid;
  }

  /**
   * @param name
   *          The name to set.
   */
  public void setPid(String name)
  {
    this.pid = name;
  }

  /**
   * @return Returns the name.
   */
  public StatusTyp getStatus()
  {
    return _status;
  }

  /**
   * @param pid
   *          The name to set.
   */
  public void setStatus(StatusTyp status)
  {
    this._status = status;
  }

  public AnzeigeInhaltTyp getAnzeigeInhalt()
  {
    return _inhalt;
  }

  public void setAnzeigeInhalt(AnzeigeInhaltTyp inhalt)
  {
    _inhalt = inhalt;
  }

  public int getX()
  {
    return _x;
  }

  public void setX(int x)
  {
    _x = x;
  }

  public int getY()
  {
    return _y;
  }

  public void setY(int y)
  {
    _y = y;
  }

  public AnzeigeElementTyp getTyp()
  {
    return _typ;
  }

  public void setTyp(AnzeigeElementTyp typ)
  {
    _typ = typ;
  }

  public boolean isAktiv()
  {
    return _aktiv;
  }

  public void setAktiv(boolean aktiv)
  {
    _aktiv = aktiv;
  }

  public Object getViewId()
  {    
    return _viewId;
  }

  public void setAnzeigeInhaltInitial(AnzeigeInhaltTyp typ)
  {   
    _inhaltInitial = typ;
  }

  public AnzeigeInhaltTyp getAnzeigeInhaltInitial()
  {
    return _inhaltInitial;
  }

  public VisuAnzeigeKapsel getVisuAnzeigeKapsel()
  {
    return _visuAnzeigeKapsel;
  }

  public void setVisuAnzeigeKapsel(VisuAnzeigeKapsel visuAnzeigeKapsel)
  {
    _visuAnzeigeKapsel = visuAnzeigeKapsel;
  }

  public List<AnzeigeInhaltTyp> getListeInhaltGestoert()
  {
    return _listeInhaltGestoert;
  }

  public void setListeInhaltGestoert(List<AnzeigeInhaltTyp> listeInhaltGestoert)
  {
    _listeInhaltGestoert = listeInhaltGestoert;
  }

}
