/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.preferences;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.TreeMap;
import java.util.TreeSet;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.views.UmfeldDatenTabelleView;

/**
 * Liefert die {@link PreferencePage} zum Durchfhren der Umfeldateneinstellungen
 * 
 * @author Pittner
 * 
 */
public class UmfeldDatenTypenPreference extends PreferencePage implements IWorkbenchPreferencePage
{
  /**
   * ID der {@link PreferencePage}
   */
  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.pageUmfelddatenTypen";

  /**
   * Flag zum Debuggen
   */
  protected static final boolean m_debug = false;

  private final static Map<String, String[]> _mapTypAtg = new HashMap<String, String[]>() {
    {
      put("typ.ufdsFahrBahnFeuchte", new String[] { "atg.ufdsFahrBahnFeuchte", "FahrBahnFeuchte", "FBF" });
      put("typ.ufdsFahrBahnGltte", new String[] { "atg.ufdsFahrBahnGltte", "FahrBahnGltte", "FBG" });
      put("typ.ufdsFahrBahnOberFlchenTemperatur", new String[] { "atg.ufdsFahrBahnOberFlchenTemperatur",
          "FahrBahnOberFlchenTemperatur", "OFT" });
      put("typ.ufdsFahrBahnOberFlchenZustand", new String[] { "atg.ufdsFahrBahnOberFlchenZustand",
          "FahrBahnOberFlchenZustand", "OFZ" });
      put("typ.ufdsGefrierTemperatur", new String[] { "atg.ufdsGefrierTemperatur", "GefrierTemperatur", "GT" });
      // Stufe
      put("typ.ufdsHelligkeit", new String[] { "atg.ufdsHelligkeit", "Helligkeit", "H", "atg.ufdsStufeHelligkeit" });
      put("typ.ufdsLuftDruck", new String[] { "atg.ufdsLuftDruck", "LuftDruck", "LD" });
      put("typ.ufdsLuftTemperatur", new String[] { "atg.ufdsLuftTemperatur", "LuftTemperatur", "LT" });
      put("typ.ufdsNiederschlagsArt", new String[] { "atg.ufdsNiederschlagsArt", "NiederschlagsArt", "NA" });
      put("typ.ufdsNiederschlagsIntensitt", new String[] { "atg.ufdsNiederschlagsIntensitt",
          "NiederschlagsIntensitt", "NI", "atg.ufdsStufeNiederschlagsIntensitt" });
      put("typ.ufdsNiederschlagsMenge", new String[] { "atg.ufdsNiederschlagsMenge", "NiederschlagsMenge", "NM" });
      put("typ.ufdsRelativeLuftFeuchte", new String[] { "atg.ufdsRelativeLuftFeuchte", "RelativeLuftFeuchte", "RLF" });
      put("typ.ufdsRestSalz", new String[] { "atg.ufdsRestSalz", "RestSalz", "RS" });
      put("typ.ufdsSchneeHhe", new String[] { "atg.ufdsSchneeHhe", "SchneeHhe", "SH" });
      // Stufe
      put("typ.ufdsSichtWeite", new String[] { "atg.ufdsSichtWeite", "SichtWeite", "SW", "atg.ufdsStufeSichtWeite" });
      put("typ.ufdsTaupunktTemperatur", new String[] { "atg.ufdsTaupunktTemperatur", "TaupunktTemperatur", "T" });
      put("typ.ufdsTemperaturInTiefe1", new String[] { "atg.ufdsTemperaturInTiefe1", "TemperaturInTiefe1", "TT1" });
      put("typ.ufdsTemperaturInTiefe2", new String[] { "atg.ufdsTemperaturInTiefe2", "TemperaturInTiefe2", "TT2" });
      put("typ.ufdsTemperaturInTiefe3", new String[] { "atg.ufdsTemperaturInTiefe3", "TemperaturInTiefe3", "TT3" });
      // Stufe
      put("typ.ufdsWasserFilmDicke", new String[] { "atg.ufdsWasserFilmDicke", "WasserFilmDicke", "WFD",
          "atg.ufdsStufeWasserFilmDicke" });
      put("typ.ufdsWindGeschwindigkeitMittelWert", new String[] { "atg.ufdsWindGeschwindigkeitMittelWert",
          "WindGeschwindigkeitMittelWert", "WGM" });
      put("typ.ufdsWindGeschwindigkeitSpitzenWert", new String[] { "atg.ufdsWindGeschwindigkeitSpitzenWert",
          "WindGeschwindigkeitSpitzenWert", "WGS" });
      put("typ.ufdsWindRichtung", new String[] { "atg.ufdsWindRichtung", "WindRichtung", "WR" });
      // Stufe
      put("typ.umfeldDatenMessStelle1", new String[] { "", "NsseStufe", "NS", "atg.ufdmsNsseStufe" });
      put("typ.umfeldDatenMessStelle2", new String[] { "atg.ufdmsTaupunktTemperaturFahrBahn",
          "TaupunktTemperaturFahrBahn", "TFB" });
      put("typ.umfeldDatenMessStelle3", new String[] { "atg.ufdmsTaupunktTemperaturLuft", "TaupunktTemperaturLuft",
          "TL" });
      put("typ.sswSbaUmfeldDatenMessStelle1", new String[] { "atg.sswSbaHelligkeitErgebnisTagNacht",
          "SswSbaErgebnisTagNacht", "SSW HTN" });
      // Stufe
      put("typ.sswSbaUmfeldDatenMessStelle2", new String[] { "", "SswSbaNsseStufe", "SSW NS",
          "atg.sswSbaUfdmsNsseStufe" });
    }
  };

  public static Map<String, String[]> getMapTypAtg()
  {
    return _mapTypAtg;
  }

  /**
   * Label zu den UmfeldddatenTypen
   */
  private Label m_lbKategorien;

  /**
   * Tabelle der UmfeldddatenTypen
   */
  private Table m_tableKategorien;

  /**
   * Selektierte UmfeldddatenTypen
   */
  private TableItem m_selKategorie;

  /**
   * Button fr die Einsortierung nach oben
   */
  private Button m_btnUp;

  /**
   * Button fr die Einsortierung nach unten
   */
  private Button m_btnDown;

  /**
   * Speicher der Prferenzen
   */
  private IPreferenceStore m_preferenceStore;

  private Button _btnStufeHell;

  /**
   * Konstruktor, parameterlos.
   */
  public UmfeldDatenTypenPreference()
  {
  }

  /**
   * Konstruktor, setzt den Titel der Umfelddateneinstellungen zu den UmfeldddatenTypen.
   * 
   * @param title
   *          Titel
   */
  public UmfeldDatenTypenPreference(String title)
  {
    super(title);
  }

  /**
   * Konstruktor, setzt den Titel und das Bild der Umfelddateneinstellungen zu den UmfeldddatenTypen.
   * 
   * @param title
   *          Titel
   * @param image
   *          Bild
   */
  public UmfeldDatenTypenPreference(String title, ImageDescriptor image)
  {
    super(title, image);
  }

  @Override
  protected Control createContents(Composite parent)
  {
    Composite top = new Composite(parent, SWT.NONE);
    
	PlatformUI.getWorkbench().getHelpSystem().setHelp(top,
			Activator.PLUGIN_ID + ".umfelddatenTypen");

    // 1.Teil des Fensters: Einstellungen der UmfeldddatenTypen
    Label lbinfo = new Label(top, SWT.NONE);
    lbinfo
        .setText("Diese Umfelddateneinstellungen ermglichen die Aktivierung und Deaktivierung bestimmter Umfelddatentypen in der Tabelle der Umfelddaten.\n");

    m_lbKategorien = new Label(top, SWT.NONE);
    m_lbKategorien.setText("Umfelddatentypen:");

    Composite comp = new Composite(top, SWT.FILL);
    GridLayout gridLayout = new GridLayout(2, false);
    comp.setLayout(gridLayout);

    // Tabelle der Navigatorkategorien erzeugen
    erzeugeTabelle(comp);

    // zustzliche Buttons erzeugen
    erzeugeButtonRechts(comp);
    erzeugeRadioUnten(comp);
    erzeugeButtonUnten(comp);

    // Daten setzen
    setzeDaten();

    // Einstellungen laden
    ladeEinstellungen();

    /*
     * Layout
     */
    GridLayout glTop = new GridLayout(1, false);
    top.setLayout(glTop);

    GridData gdTree = new GridData();
    gdTree.grabExcessHorizontalSpace = true;
    gdTree.horizontalAlignment = GridData.FILL;
    gdTree.grabExcessVerticalSpace = true;
    gdTree.verticalAlignment = GridData.FILL;
    top.setLayoutData(gdTree);

    return null;
  }

  /**
   * Initialisierungen fr die Tabelle der Umfelddatentypen
   * 
   * @param parent
   *          Elternteil zu dem diese Komponenten hinzugefgt werden sollen
   */
  private void erzeugeTabelle(Composite parent)
  {

    GridData data = new GridData(SWT.NONE, SWT.FILL, true, true);
    data.heightHint = 300;
    data.widthHint = 500;

    m_tableKategorien = new Table(parent, SWT.CHECK | SWT.BORDER);
    m_tableKategorien.setLayoutData(data);

    m_tableKategorien.addListener(SWT.Selection, new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        // Selektion merken
        TableItem item = (TableItem)event.item;
        m_selKategorie = item;

        // entsprechende Up-Down-Buttons de-/aktivieren
        TableItem[] tia = m_tableKategorien.getItems();

        if (tia[0].equals(m_selKategorie))
          m_btnUp.setEnabled(false);
        else
          m_btnUp.setEnabled(true);

        if (tia[tia.length - 1].equals(m_selKategorie))
          m_btnDown.setEnabled(false);
        else
          m_btnDown.setEnabled(true);

      }

    });

  }

  /**
   * Initialisierungen fr die zustzlichen rechten Buttons
   * 
   * @param parent
   *          Elternteil zu dem diese Komponenten hinzugefgt werden sollen
   */
  private void erzeugeButtonRechts(Composite parent)
  {
    // Buttons fr die Positionierung
    Composite compRechts = new Composite(parent, SWT.NONE);
    RowLayout rowLayout = new RowLayout(SWT.VERTICAL);
    rowLayout.pack = false;
    compRechts.setLayout(rowLayout);

    GridData data = new GridData(SWT.FILL, SWT.FILL, true, true);
    data.verticalAlignment = SWT.BEGINNING;
    compRechts.setLayoutData(data);

    m_btnUp = new Button(compRechts, SWT.ARROW | SWT.UP);
    m_btnUp.setEnabled(false);
    m_btnUp.addSelectionListener(new SelectionAdapter() {
      // Selektiertes Element nach oben schieben
      public void widgetSelected(SelectionEvent event)
      {
        if (m_selKategorie != null)
        {
          TableItem[] tia = m_tableKategorien.getItems();

          for (int i = 0; i < tia.length; i++)
          {
            if (tia[i].equals(m_selKategorie) && (i != 0))
            {
              String selText = m_selKategorie.getText();
              boolean selCheck = m_selKategorie.getChecked();
              Object selData = m_selKategorie.getData();

              tia[i].setText(tia[i - 1].getText());
              tia[i].setChecked(tia[i - 1].getChecked());
              tia[i].setData(tia[i - 1].getData());

              tia[i - 1].setText(selText);
              tia[i - 1].setChecked(selCheck);
              tia[i - 1].setData(selData);

              m_tableKategorien.setSelection(i - 1);
              m_selKategorie = tia[i - 1];
              break;
            }
          }

        }

      }
    });
    m_btnDown = new Button(compRechts, SWT.ARROW | SWT.DOWN);
    m_btnDown.setEnabled(false);
    m_btnDown.addSelectionListener(new SelectionAdapter() {
      // Selektiertes Element nach unten schieben
      public void widgetSelected(SelectionEvent event)
      {
        if (m_selKategorie != null)
        {
          TableItem[] tia = m_tableKategorien.getItems();

          for (int i = 0; i < tia.length; i++)
          {
            if (tia[i].equals(m_selKategorie) && (i != tia.length - 1))
            {
              String selText = m_selKategorie.getText();
              boolean selCheck = m_selKategorie.getChecked();
              Object selData = m_selKategorie.getData();

              tia[i].setText(tia[i + 1].getText());
              tia[i].setChecked(tia[i + 1].getChecked());
              tia[i].setData(tia[i + 1].getData());

              tia[i + 1].setText(selText);
              tia[i + 1].setChecked(selCheck);
              tia[i + 1].setData(selData);

              m_tableKategorien.setSelection(i + 1);
              m_selKategorie = tia[i + 1];
              break;
            }
          }
        }
      }
    });

  }

  /**
   * Initialisierungen fr die zustzlichen Buttons unten
   * 
   * @param parent
   *          Elternteil zu dem diese Komponenten hinzugefgt werden sollen
   */
  private void erzeugeRadioUnten(Composite parent)
  {
    // Buttons fr die Aktivierung und Deaktivierung
    Composite compUnten = new Composite(parent, SWT.NONE);
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    compUnten.setLayout(rowLayout);

    GridData data = new GridData(SWT.FILL, SWT.FILL, true, true);
    data.verticalAlignment = SWT.BEGINNING;
    data.horizontalSpan = 2;
    compUnten.setLayoutData(data);

    _btnStufeHell = new Button(compUnten, SWT.CHECK);
    _btnStufeHell.setText("Stufe Helligkeit");
    _btnStufeHell.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        String[] ss = null;

        boolean sel = _btnStufeHell.getSelection();

        if (sel)
          ss = new String[] { "atg.ufdsHelligkeit", "Helligkeit", "H", "atg.ufdsStufeHelligkeit" };
        else
        {
          ss = new String[] { "atg.ufdsHelligkeit", "Helligkeit", "H" };
        }

        _mapTypAtg.put("typ.ufdsHelligkeit", ss);
      }
    });

  }

  /**
   * Initialisierungen fr die zustzlichen Buttons unten
   * 
   * @param parent
   *          Elternteil zu dem diese Komponenten hinzugefgt werden sollen
   */
  private void erzeugeButtonUnten(Composite parent)
  {
    // Buttons fr die Aktivierung und Deaktivierung
    Composite compUnten = new Composite(parent, SWT.NONE);
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    compUnten.setLayout(rowLayout);

    GridData data = new GridData(SWT.FILL, SWT.FILL, true, true);
    data.verticalAlignment = SWT.BEGINNING;
    compUnten.setLayoutData(data);

    Button btnAktivAll = new Button(compUnten, SWT.PUSH);
    btnAktivAll.setText("Alle aktivieren");
    btnAktivAll.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        TableItem[] tia = m_tableKategorien.getItems();

        for (int i = 0; i < tia.length; i++)
        {
          tia[i].setChecked(true);
        }

      }
    });
    Button btnDeaktivAll = new Button(compUnten, SWT.PUSH);
    btnDeaktivAll.setText(" Alle deaktivieren ");
    btnDeaktivAll.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(SelectionEvent event)
      {
        TableItem[] tia = m_tableKategorien.getItems();

        for (int i = 0; i < tia.length; i++)
        {
          tia[i].setChecked(false);
        }
      }
    });

  }

  /**
   * Setzt die Daten
   */
  private void setzeDaten()
  {
    TreeSet<String> treeSet = new TreeSet<String>(_mapTypAtg.keySet());

    for (Iterator<String> iterator = treeSet.iterator(); iterator.hasNext();)
    {
      String key = (String)iterator.next();

      String[] ss = _mapTypAtg.get(key);

      TableItem item = new TableItem(m_tableKategorien, SWT.NONE);
      item.setText(ss[1]);
      item.setChecked(false);
      item.setData(Activator.PLUGIN_ID + ":" + key);
    }
  }

  /**
   * Ldt die Einstellungen
   */
  private void ladeEinstellungen()
  {
    assert m_preferenceStore != null;

    TableItem[] tia = m_tableKategorien.getItems();
    TreeMap<Integer, String> sortList = new TreeMap<Integer, String>();
    LinkedList<String> deaktivList = new LinkedList<String>();

    // Sortierung merken
    for (int i = 0; i < tia.length; i++)
    {
      int position = m_preferenceStore.getInt(tia[i].getData().toString());

      if (position != 0)
        sortList.put(position - 1, tia[i].getText());
      else
        deaktivList.add(tia[i].getText());
    }

    int index = 0;

    // die ausgewhlten und positionierten UmfeldddatenTypen
    for (Map.Entry<Integer, String> me : sortList.entrySet())
    {
      String titel = me.getValue();
      tia[index].setText(titel);
      tia[index].setChecked(true);

      String s = null;
      for (Map.Entry<String, String[]> me2 : _mapTypAtg.entrySet())
      {
        String key = me2.getKey();
        String[] ss = me2.getValue();
        if (ss[1].equals(titel))
        {
          s = key;
          break;
        }
      }
      if (s != null)
      {
        tia[index].setData(Activator.PLUGIN_ID + ":" + s);
      }

      index++;
    }

    // die nicht ausgewhlten Umfelddatentypen
    for (Iterator<String> iterator = deaktivList.iterator(); iterator.hasNext();)
    {
      String titel = (String)iterator.next();
      tia[index].setText(titel);
      tia[index].setChecked(false);

      String s = null;
      for (Map.Entry<String, String[]> me2 : _mapTypAtg.entrySet())
      {
        String key = me2.getKey();
        String[] ss = me2.getValue();
        if (ss[1].equals(titel))
        {
          s = key;
          break;
        }
      }
      if (s != null)
      {
        tia[index].setData(Activator.PLUGIN_ID + ":" + s);
      }

      index++;
    }

    boolean stufeHelligkeit = m_preferenceStore.getBoolean(Activator.PLUGIN_ID + ":" + "stufeHelligkeit");

    String[] ss = null;

    if (stufeHelligkeit)
      ss = new String[] { "atg.ufdsHelligkeit", "Helligkeit", "H", "atg.ufdsStufeHelligkeit" };
    else
    {
      ss = new String[] { "atg.ufdsHelligkeit", "Helligkeit", "H" };
    }
    _mapTypAtg.put("typ.ufdsHelligkeit", ss);

    _btnStufeHell.setSelection(stufeHelligkeit);

  }

  /**
   * Speichert die Einstellungen
   * 
   */
  private void sichernEinstellungen()
  {
    // Speichern, ob das Element angezeigt werden soll
    TableItem[] tia = m_tableKategorien.getItems();

    for (int i = 0; i < tia.length; i++)
    {
      if (tia[i].getChecked())
      {
        m_preferenceStore.setValue(tia[i].getData().toString(), i + 1);
      }
      else
      {
        m_preferenceStore.setValue(tia[i].getData().toString(), 0);
      }
    }

    m_preferenceStore.setValue(Activator.PLUGIN_ID + ":" + "stufeHelligkeit", _btnStufeHell.getSelection());

    // Umfelddatentabelle aktualisieren
    IWorkbench workbench = Activator.getDefault().getWorkbench();
    try
    {
      IWorkbenchPage activePage = workbench.getActiveWorkbenchWindow().getActivePage();
      IViewReference viewRef = activePage.findViewReference(UmfeldDatenTabelleView.ID);
      if (viewRef != null)
      {
        IViewPart view = viewRef.getView(false);
        ((UmfeldDatenTabelleView)view).erzeugeInhaltTabelle();
      }
    }
    catch (Exception e)
    {

    }

  }

  @Override
  protected void performApply()
  {
    sichernEinstellungen();
  }

  @Override
  public boolean performOk()
  {
    performApply();
    return super.performOk();
  }

  @Override
  public void performHelp()
  {
    String helpContextId = Activator.PLUGIN_ID + ".umfelddatenTypen";
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(helpContextId);
  }

  @Override
  public void init(IWorkbench workbench)
  {
    // Hole den Prferenzen Speicher
    m_preferenceStore = Activator.getDefault().getPreferenceStore();
    setPreferenceStore(m_preferenceStore);

  }

  @Override
  protected void performDefaults()
  {

    ladeEinstellungen();
    super.performDefaults();

  }

}
