/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.anlagendarstellung.programmtyp;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.Vector;
import java.util.Map.Entry;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeElementFigure;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.AnzeigeInhaltAuswahlDialog;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeElementTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Darstellung des Fensters zur Versorgung der Parameter
 */
public class CompositeParameterDefinitionen extends Composite implements PropertyChangeListener
{
	
	/**
	 * Aktuelles Composite
	 */
  private Composite _composite;

  /**
   * Aktuelle Parameter
   */
    private Data _aktuelleDaten;

  /**
   * Composite fr die Buttons
   */
	private Composite _compositeButtons;

	/**
	 * Button Hinzufgen
	 */
	private Button _buttonHinzufuegen;

	/**
	 * Button Lschen
	 */
	private Button _buttonLoeschen;

	/**
	 * Tabellenmodell
	 */
	private TableModel[] _tableModelAktuell;

	/**
	 * Anzeigetyp
	 */
	protected SystemObject _soAnzeigeTyp = null;

  private ClientDavInterface _connection = null;

  protected int _height = 35;

  private CTabFolder _tabFolder;
  
  private TableViewer _tableViewerAktuell;

  private int _indexStufe;

  private int _indexPrg;

  private boolean _isSpurSperrung;
  
  /**
   * Mapping der logischen AnzeigeTypen in der GUI zu dem DaV-Typen
   */
  private static Map<String, AnzeigeElementTyp> _mapAnzeigeElemente = new HashMap<String, AnzeigeElementTyp>() {
    {
//    put("sba.anzeigeTyp.A2.TypA", AnzeigeElementTyp.WZG_A);
//    put("sba.anzeigeTyp.A2.TypB", AnzeigeElementTyp.WZG_B);
//    put("sba.anzeigeTyp.A2.TypBAG", AnzeigeElementTyp.WZG_BAG);
//    put("sba.anzeigeTyp.A2.TypBlinker", AnzeigeElementTyp.BLINKER_B);
//    put("sba.anzeigeTyp.A2.TypC", AnzeigeElementTyp.WZG_C);
//    put("sba.anzeigeTyp.A2.TypD", AnzeigeElementTyp.FREITEXT);      
      
      put("sba.anzeigeTyp.A8.sba_stgt.TypA", AnzeigeElementTyp.WZG_A);
      put("sba.anzeigeTyp.A8.sba_stgt.TypB", AnzeigeElementTyp.WZG_B);
      put("sba.anzeigeTyp.A8.sba_stgt.TypBAQ", AnzeigeElementTyp.PR_T2);
      put("sba.anzeigeTyp.A8.sba_stgt.Blinker", AnzeigeElementTyp.BLINKER_B);
      put("sba.anzeigeTyp.A8.sba_stgt.TypC", AnzeigeElementTyp.WZG_C2);
      put("sba.anzeigeTyp.A8.sba_stgt.TypD", AnzeigeElementTyp.FREITEXT);
      put("sba.anzeigeTyp.A8.sba_stgt.TypD2", AnzeigeElementTyp.FREITEXT2);
      put("sba.anzeigeTyp.A8.sba_stgt.TypTAQ", AnzeigeElementTyp.PR_T);
      put("sba.anzeigeTyp.A8.sba_stgt.TypWAQ", AnzeigeElementTyp.PR_W);
      
//      put("sba.anzeigeTyp.A81.sba_stgt.TypA", AnzeigeElementTyp.WZG_A);
//      put("sba.anzeigeTyp.A81.sba_stgt.TypB", AnzeigeElementTyp.WZG_B);
//      put("sba.anzeigeTyp.A81.sba_stgt.TypBAQ", AnzeigeElementTyp.PR_T2);
//      put("sba.anzeigeTyp.A81.sba_stgt.Blinker", AnzeigeElementTyp.BLINKER_B);
//      put("sba.anzeigeTyp.A81.sba_stgt.TypC", AnzeigeElementTyp.WZG_C2);
//      put("sba.anzeigeTyp.A81.sba_stgt.TypD", AnzeigeElementTyp.FREITEXT);
//      put("sba.anzeigeTyp.A81.sba_stgt.TypD2", AnzeigeElementTyp.FREITEXT2);
//      put("sba.anzeigeTyp.A81.sba_stgt.TypTAQ", AnzeigeElementTyp.PR_T);
//      put("sba.anzeigeTyp.A81.sba_stgt.TypWAQ", AnzeigeElementTyp.PR_W);
      

    }
  };

  /**
   * Mapping der DaV-Typen zu dem DaV-Sba-Typen
   */
  private static Map<String, String> _mapAnzeigeTypen = new HashMap<String, String>() {
    {
//      put("anzeigeTyp.uz.a2.TypA.1", "sba.anzeigeTyp.A2.TypA");
//      put("anzeigeTyp.uz.a2.TypA.1.links", "sba.anzeigeTyp.A2.TypA");
//      put("anzeigeTyp.uz.a2.TypA.2", "sba.anzeigeTyp.A2.TypA");
//      put("anzeigeTyp.uz.a2.TypA.2.links", "sba.anzeigeTyp.A2.TypA");
//      put("anzeigeTyp.uz.a2.TypA.3", "sba.anzeigeTyp.A2.TypA");
//      put("anzeigeTyp.uz.a2.TypA.4", "sba.anzeigeTyp.A2.TypA");        
//      put("anzeigeTyp.uz.a2.TypB.1", "sba.anzeigeTyp.A2.TypB");
//      put("anzeigeTyp.uz.a2.TypB.2", "sba.anzeigeTyp.A2.TypB");      
//      put("anzeigeTyp.uz.a2.TypBAG.1", "sba.anzeigeTyp.A2.TypBAG");    
//      put("anzeigeTyp.uz.a2.TypC.1", "sba.anzeigeTyp.A2.TypC");
      
      put("anzeigeTyp.uz.A8.sba_stgt.TypA.1", "sba.anzeigeTyp.A8.sba_stgt.TypA");
      put("anzeigeTyp.uz.A8.sba_stgt.TypA.1.links", "sba.anzeigeTyp.A8.sba_stgt.TypA");
      put("anzeigeTyp.uz.A8.sba_stgt.TypA.1.rechts", "sba.anzeigeTyp.A8.sba_stgt.TypA");
      put("anzeigeTyp.uz.A8.sba_stgt.TypA.2", "sba.anzeigeTyp.A8.sba_stgt.TypA");
      put("anzeigeTyp.uz.A8.sba_stgt.TypA.2.rechts", "sba.anzeigeTyp.A8.sba_stgt.TypA");
      put("anzeigeTyp.uz.A8.sba_stgt.TypB.1", "sba.anzeigeTyp.A8.sba_stgt.TypB");
      put("anzeigeTyp.uz.A8.sba_stgt.TypBAQ.1", "sba.anzeigeTyp.A8.sba_stgt.TypBAQ");
      put("anzeigeTyp.uz.A8.sba_stgt.TypBlinker.1", "sba.anzeigeTyp.A8.sba_stgt.Blinker");
      put("anzeigeTyp.uz.A8.sba_stgt.TypC.1", "sba.anzeigeTyp.A8.sba_stgt.TypC");
      put("anzeigeTyp.uz.A8.sba_stgt.TypD.1", "sba.anzeigeTyp.A8.sba_stgt.TypD");
      put("anzeigeTyp.uz.A8.sba_stgt.TypD.2", "sba.anzeigeTyp.A8.sba_stgt.TypD2");
      put("anzeigeTyp.uz.A8.sba_stgt.TypTAQ.1", "sba.anzeigeTyp.A8.sba_stgt.TypTAQ");
      put("anzeigeTyp.uz.A8.sba_stgt.TypTAQ.2", "sba.anzeigeTyp.A8.sba_stgt.TypTAQ");
      put("anzeigeTyp.uz.A8.sba_stgt.TypTAQ.3", "sba.anzeigeTyp.A8.sba_stgt.TypTAQ");
      put("anzeigeTyp.uz.A8.sba_stgt.TypWAQ.1", "sba.anzeigeTyp.A8.sba_stgt.TypWAQ");
      
//      put("anzeigeTyp.uz.A81.sba_stgt.TypA.1", "sba.anzeigeTyp.A81.sba_stgt.TypA");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypA.1.links", "sba.anzeigeTyp.A81.sba_stgt.TypA");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypA.1.rechts", "sba.anzeigeTyp.A81.sba_stgt.TypA");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypA.2", "sba.anzeigeTyp.A81.sba_stgt.TypA");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypA.2.rechts", "sba.anzeigeTyp.A81.sba_stgt.TypA");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypB.1", "sba.anzeigeTyp.A81.sba_stgt.TypB");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypBAQ.1", "sba.anzeigeTyp.A81.sba_stgt.TypBAQ");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypBlinker.1", "sba.anzeigeTyp.A81.sba_stgt.Blinker");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypC.1", "sba.anzeigeTyp.A81.sba_stgt.TypC");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypD.1", "sba.anzeigeTyp.A81.sba_stgt.TypD");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypD.2", "sba.anzeigeTyp.A81.sba_stgt.TypD2");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypTAQ.1", "sba.anzeigeTyp.A81.sba_stgt.TypTAQ");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypTAQ.2", "sba.anzeigeTyp.A81.sba_stgt.TypTAQ");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypTAQ.3", "sba.anzeigeTyp.A81.sba_stgt.TypTAQ");
//      put("anzeigeTyp.uz.A81.sba_stgt.TypWAQ.1", "sba.anzeigeTyp.A81.sba_stgt.TypWAQ");
      
    }
  };
  
  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil des Fenstern
   * @param aktuell Aktuelle Parameter
   * @param nameTreppenFunktion Name der Treppenfunktion
   */
  CompositeParameterDefinitionen (Composite parent, int style, ResultData aktuell, int indexPrg, int indexStufe)
  {
    super (parent, style);
    
    _indexPrg = indexPrg;
    _indexStufe = indexStufe;
    
    _aktuelleDaten = aktuell.getData().createModifiableCopy();
        
    _connection  = DaVVerbindung.getInstanz().getDaVVerbindung();
    
    initialize();
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
    
    Array array = _aktuelleDaten.getItem("ProgrammTypen").asArray();
    
    _isSpurSperrung = false;
    
    if (_indexPrg < array.getLength())
      _isSpurSperrung = !_aktuelleDaten.getItem("ProgrammTypen").asArray().getItem(_indexPrg).getItem("Spursperrung").asTextValue().getValueText().equals("Ja");
    else{
      array.setLength(array.getLength()+1);
    }
        
    _tableViewerAktuell = erzeugeGruppeRegeln();    
    _tableModelAktuell = createModel(_tableViewerAktuell.getTable());     
    
    if ( _tableModelAktuell != null )
      _tableViewerAktuell.setInput(_tableModelAktuell); 
    
    createButtons();
    
    updateData(_aktuelleDaten, _tableModelAktuell, _tableViewerAktuell);
    
    _tabFolder.setSelection(0);
    _tabFolder.redraw();
    _tabFolder.layout(true);
   
  }
  
  

  /**
   * Methode bernimmt die Daten und zeigt diese in der Tabelle an
   */
  private void updateData (Data parameter, TableModel[] tableModel, TableViewer tableViewer)
  {
        
    int anzahl = getAnzahlParameter(parameter);

  	_tableModelAktuell = erzeugeTabellenModell (anzahl, tableModel, tableViewer);
  	  	
  	for (int j=0; j<anzahl; ++j)
  	{
      TableModel tm = _tableModelAktuell [j];    	
    	Data aktuell = getArrayParameter(parameter).getItem(j);
    	
    	/*
    	 * Lschen
    	 */
    	tm.setSelectedLoeschen("false");
    	    	
    	
    	/*
    	 * AnzeigeTyp
    	 */    	
    	Combo cmX = (Combo) tm.getAnzeigeTyp().getChildren()[0];
    	String pidTyp = aktuell.getItem("AnzeigeTyp").asReferenceValue().getSystemObject().getPid(); 
    	
    	/*
    	 * WVZ Inhalt
    	 */    	
  	  Label imX = (Label) tm.getWvzInhalt().getChildren()[0];  	  
  	  SystemObject so = aktuell.getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().getSystemObject();
  	  if (so != null && pidTyp != null){
  	    WVZInhalt x = new WVZInhalt(so); 
  	    setzeBildInhalte(cmX, imX, x, pidTyp);  	    
  	  }
  	  
  	  /*
  	   * Blinktakt
  	   */    	
  	  Combo imY = (Combo) tm.getBlinkTakt().getChildren()[0];  	  
  	  String blinktakt = aktuell.getItem("WVZInhalt").getItem("Blinktakt").asTextValue().getText();
  	  
  	  
  	  if (blinktakt != null){  	     
//  	    if (blinktakt.contains(","))
//  	      blinktakt = blinktakt.replace(",", ".");
  	    
  	    List<String> list = Arrays.asList(imY.getItems());
        int idx = list.indexOf(blinktakt);
        
//        System.out.println(blinktakt + " -> " + idx);
        
        imY.select(idx);
        imY.setData(pidTyp);  	    
  	  }
    }
  }

  /**
   * Methode erzeugt ein Tabellenmodell einer bestimmten Gre. Dabei werden bereits vorhandene
   * Composite in der Tabelle weiter verwendet
   * @param anzahlParameter Gre des erzeugten Tabellenmodells
   */
  private TableModel[] erzeugeTabellenModell (int anzahlParameter, TableModel[] tableModel, TableViewer tableViewer)
  {
  	/*
  	 * Hat sich die Gre gendert
  	 */
  	if (tableModel.length != anzahlParameter)
  	{
  		/*
  		 * Tabelle erweitern
  		 */
  		if (tableModel.length < anzahlParameter)
  		{
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  
  		  for (int i=0; i<tableModel.length; ++i)
  		  	v.add(tableModel [i]);
  		  
  		  for (int i=tableModel.length; i<anzahlParameter; ++i)
  		  	v.add (erzeugeZeileInTabelle(tableViewer.getTable()));
  		  
  		  tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i){
  		  	tableModel [i] = v.get(i);
  		  }

  		}
  		/*
  		 * Tabelle reduzieren
  		 */
  		else if (tableModel.length > anzahlParameter)
  		{
  			/*
  			 * Nicht mehr bentigte Elemente lschen
  			 */
  			
  			for (int i = tableModel.length - 1; i > anzahlParameter - 1; i--)
  			{
  				tableModel [i].getWvzInhalt().dispose();
  				tableModel [i].getAnzeigeTyp().dispose();  	
  				tableModel [i].getBlinkTakt().dispose();   
  				tableModel [i].getLoeschen().dispose();
  			}
  				
  		  Vector<TableModel> v = new Vector<TableModel>();
  		  for (int i=0; i<anzahlParameter; ++i)
  		  	v.add(tableModel [i]);

  		  tableModel = new TableModel [anzahlParameter];
  		  for (int i=0; i<v.size(); ++i)
  		  	tableModel [i] = v.get(i);
  		}
  		
  		tableViewer.setInput(tableModel);
  		tableViewer.getTable().layout(true);
  	}
  	
  	
  	return tableModel.clone();
  	
  	
  }
  
  /**
   * Methode erzeugt die Gruppe mit den Regeln
   */
  private TableViewer erzeugeGruppeRegeln()
  {
    
    GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;
    gridDataTabFolder.heightHint = 300;

    _tabFolder = new CTabFolder(_composite, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridDataTabFolder);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {        
        CTabItem itemSel = _tabFolder.getSelection();        
        Composite cSel = (Composite)itemSel.getControl();        
        Control[] children = cSel.getChildren();
                
        if (children.length == 0){
            _tableViewerAktuell = erzeugeTabelle(cSel);
            _tableModelAktuell = createModel(_tableViewerAktuell.getTable());     
            
            if ( _tableModelAktuell != null )
              _tableViewerAktuell.setInput(_tableModelAktuell);
            
            itemSel.setData(_tableViewerAktuell);
                                    
            cSel.redraw();
            cSel.layout(true);
        }else{
          _tableViewerAktuell = (TableViewer)itemSel.getData();
          _tableModelAktuell = (TableModel[])_tableViewerAktuell.getInput();               
        }
        
                
        updateData(_aktuelleDaten, _tableModelAktuell, _tableViewerAktuell);
        
        _tabFolder.redraw();
        _tabFolder.layout(true);
        
      }
    });
        

    erzeugeTabItems();
    
    
    CTabItem item0 = _tabFolder.getItem(0);
    
    Control c = item0.getControl();
    if (c instanceof Composite)
    {
      Composite c1 = (Composite)c;
      TableViewer tableViewer = erzeugeTabelle(c1);
      item0.setData(tableViewer);
      return tableViewer;     
    }
    return null;
  }

  private void erzeugeTabItems()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.verticalAlignment = GridData.FILL;
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.FILL;
    gridData11.grabExcessHorizontalSpace = true;
    gridData11.grabExcessVerticalSpace = true;
    gridData11.verticalAlignment = GridData.FILL;

    Composite c1 = new Composite( _tabFolder, SWT.NONE );
    Composite c2 = new Composite( _tabFolder, SWT.NONE );
    c1.setLayout( new GridLayout() );
    c1.setLayoutData( gridData1 );    
    c2.setLayout( new GridLayout() );
    c2.setLayoutData( gridData11 );  
   
    
    
    if (!_isSpurSperrung){
      CTabItem tabItem1 = new CTabItem( _tabFolder, SWT.NONE );
      tabItem1.setText( "gesperrt" );
      tabItem1.setControl( c1 );      
    }
    CTabItem tabItem2 = new CTabItem( _tabFolder, SWT.NONE );
    tabItem2.setText( "nicht gesperrt" );
    tabItem2.setControl( c2 );          
  }  
  
  /**
   * Methode erzeugt das Composite mit den Buttons
   */
  private void createButtons()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    
    _compositeButtons = new Composite( _composite, SWT.NONE );
    _compositeButtons.setLayoutData( gridData3 );
    _compositeButtons.setLayout( rowLayout );

    _buttonHinzufuegen = new Button( _compositeButtons, SWT.NONE );
    _buttonHinzufuegen.setText( "Definition hinzufgen" );
    _buttonHinzufuegen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelHinzufuegen(_tableModelAktuell, _tableViewerAktuell);
      }
    } );

    _buttonLoeschen = new Button( _compositeButtons, SWT.NONE );
    _buttonLoeschen.setText( "Selektierte Definition loeschen" );
    _buttonLoeschen.addMouseListener( new MouseAdapter()
    {
      @Override
      public void mouseDown( MouseEvent e )
      {
        super.mouseDown( e );
        aktionRegelLoeschen(_tableModelAktuell, _tableViewerAktuell);
      }

    } );
    _buttonLoeschen.setEnabled(false);

  }

  /**
   * Methode prft ob Regeln zum Lschen ausgewhlt sind
   */
  private void pruefeObLoeschenAngewaehlt (TableModel[] tableModel)
  {
  	boolean angewaehlt = false;
  	
		for (TableModel tm : tableModel)
			if (tm.getSelectedLoeschen().equals("true"))
				angewaehlt = true;
		
		_buttonLoeschen.setEnabled(angewaehlt);
  }
  
  /**
   * Methode zum Lschen der Regeln
   */
	private void aktionRegelLoeschen(TableModel[] tableModel, TableViewer tableViewer)
  {
		Vector<ParameterDaten> stufen = leseAktuelleEinstellungen(tableModel).getParameterDaten();
		
    Vector<ParameterDaten> loeschen = new Vector<ParameterDaten>();
    
    for (int i=0; i<tableModel.length; ++i)
    	if (!tableModel [i].getSelectedLoeschen().equals("false"))
    		loeschen.add(stufen.get(i));

    for (ParameterDaten r : loeschen)
    {
    	stufen.remove(r);
    }
    
    setParameterDatenInData(stufen);
    
    updateData(_aktuelleDaten, tableModel, tableViewer);
  }

	/**
	 * Aktion Regel hinzufgen
	 */
	private void aktionRegelHinzufuegen(TableModel[] tableModel, TableViewer tableViewer)
  {
//	  System.out.println("CompositeParameterEntfernungsAngaben.aktionRegelHinzufuegen()");
	  
		Vector<ParameterDaten> stufen = leseAktuelleEinstellungen(tableModel).getParameterDaten();
		setParameterDatenInData(stufen);

		SystemObject soWVZInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, AnzeigeInhaltTyp.DUNKEL_BI.getType());
		 
		
		WVZInhalt dunkel = new WVZInhalt(soWVZInhalt);
		for (Map.Entry<String, AnzeigeElementTyp> me : _mapAnzeigeElemente.entrySet())
    {
      if(me.getValue().equals(AnzeigeElementTyp.WZG_A)){
          stufen.add(new ParameterDaten(me.getKey(), dunkel, "Undefiniert"));
          break;
      }
    }
		 
		setParameterDatenInData(stufen);
		updateData(_aktuelleDaten, tableModel, tableViewer);
  }

	/**
	 * Methode setzt einen WVZ Inhalt an einem Label
	 * @param label Label
	 * @param inhalt dazustellender WVZ-Inhalt
	 * @param anzeigeTyp Anzeigetyp
	 */
	private void setzeBildInhalte (Combo combo, Label label, WVZInhalt inhalt, String pidTyp)
	{
	  if (inhalt != null && inhalt.getAnzeigenInhalt() != null && pidTyp != null)
	  {
      String tWertX = inhalt.getAnzeigenInhalt().getPid();
      label.setData( tWertX );

      SystemObject ai = inhalt.getAnzeigenInhalt();
      Data cd = ai.getConfigurationData(ai.getDataModel().getAttributeGroup("atg.wvzInhalt"));
      if (cd != null){
        String s = cd.getItem("Bildinhalt").asTextValue().getValueText();
        
        AnzeigeInhaltTyp aitypX = AnzeigeInhaltTyp.setBildInhalt(s, System.currentTimeMillis());
        
        AnzeigeElementTyp anzeigeTyp = _mapAnzeigeElemente.get(pidTyp);
        
        Image image = createImage(anzeigeTyp, aitypX);
        if (image != null){
          label.setImage(image);
          Rectangle bounds = image.getBounds();                   
          _height  = Math.max(_height, bounds.height + 2);  
        }
        
        String name = _connection.getDataModel().getObject(pidTyp).getName();
        
        List<String> list = Arrays.asList(combo.getItems());
        int idx = list.indexOf(name);
        
        combo.select(idx);
        combo.setData(pidTyp);
        
      }          
	  }
	}

	
	/**
	 * Methode erzeugt das Composite mit der Tabelle
	 */
  private TableViewer erzeugeTabelle (Composite parent)
  {
    FillLayout fillLayout = new FillLayout();
    fillLayout.marginHeight = 5;
    fillLayout.marginWidth = 5;
    fillLayout.type = org.eclipse.swt.SWT.HORIZONTAL;
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    Group groupRegeln = new Group(parent, SWT.NONE);
    groupRegeln.setText("Definition Programmtyp");
    
    groupRegeln.setLayout(fillLayout);
    groupRegeln.setLayoutData(gridData2);
        
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.type = org.eclipse.swt.SWT.VERTICAL;
    
    Composite compositeTabelle = new Composite(groupRegeln, SWT.None);
    compositeTabelle.setLayout(fillLayout1);
    
    TableViewer tableViewer = new TableViewer(compositeTabelle, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    tableViewer.setContentProvider(new TableContentProvider());

    TableViewerColumn column0 = new TableViewerColumn(tableViewer, SWT.NONE);
    column0.getColumn().setWidth(60);
    column0.getColumn().setText("Lschen");
    column0.setLabelProvider(new MyCellLabelProvider());
    
    TableViewerColumn column21 = new TableViewerColumn(tableViewer, SWT.NONE);
    column21.getColumn().setWidth(200);
    column21.getColumn().setText("AnzeigeTyp");
    column21.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column3 = new TableViewerColumn(tableViewer, SWT.NONE);
    column3.getColumn().setWidth(120);
    column3.getColumn().setText("WZG Inhalt");
    column3.setLabelProvider(new MyCellLabelProvider());
    
    TableViewerColumn column4 = new TableViewerColumn(tableViewer, SWT.NONE);
    column4.getColumn().setWidth(120);
    column4.getColumn().setText("Blinktakt");
    column4.setLabelProvider(new MyCellLabelProvider());

    // resize the row height using a MeasureItem listener    
    Listener listener = new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:                               
            event.height = _height;
            break;
          default:
        }
      }
    };
    
    tableViewer.getTable().addListener(SWT.MeasureItem, listener);
    tableViewer.getTable().setLinesVisible(true);
    tableViewer.getTable().setHeaderVisible(true);
    
    return tableViewer;
  }
  
  /**
   * Klasse realsiert den TableContentProvider fr die Tabelle
   * @author ObertM
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
  	/**
  	 * Tabelviewer
  	 */
    private TableViewer tableViewer;

    /**
     * Methode liefert die Elemente der Tabelle
     */
    public Object[] getElements(Object inputElement)
    {
			Object[] re = null;

			if (inputElement instanceof TableModel [])
			{
        re = (TableModel[]) inputElement;
			}
			
			return re;
    }

    /**
     * Dispose
     */
    public void dispose()
    {
    }

    /**
     * Methode wird aufgerufen wenn sich der Inhalt der Tabelle gendert hat
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.deleteObserver(this);
        }
      }
      
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
            tm.addObserver(this);
        }
      }
    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();
      }
    }
  }

  /**
   * Erzeugt ein TabellenModell fr den Inhalt der Tabellen
   * @param table
   * @return TabellenModell
   */
  private TableModel[] createModel (Table table)
  {
    int tabLength = getAnzahlParameter(_aktuelleDaten);
    int elIndex   = 0;
    
    TableModel[] elements = new TableModel [tabLength];
    
    if (elements.length == 0)
    	return elements;

    for (int x=0; x<getAnzahlParameter(_aktuelleDaten); ++x)
    {
	    elements[elIndex] = erzeugeZeileInTabelle(table);
	    elIndex++;
    }
    
    return elements;
  }
  
  /**
   * Holt die gemappten AnzeigeTypen(typ.anzeigeTypen) zu einem SswSbaANzeigeTyp (typ.sswSbaAnzeigeTyp)
   * @param map
   * @param value
   * @return  die AnzeigeTypen
   */
  public static Set<String> getAnzeigeTypenVonSswSbaAnzeigeTyp(Map<String, String> map, String value) {
    Set<String> keys = new HashSet<String>();
    for (Entry<String, String> entry : map.entrySet()) {
        if (value.equals(entry.getValue())) {
            keys.add(entry.getKey());
        }
    }
    return keys;
}

  /**
   * Methode erzeugt ein Tabellenmodell fr eine Zeile in der Tabelle
   * @param table Tabelle 
   * @return Tabellenmodell fr eine Zeile
   */
  private TableModel erzeugeZeileInTabelle (Table table)
  {
    /*
     * Layout festlegen
     */  	
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.grabExcessVerticalSpace = true;

    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.verticalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;

    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.CENTER;
    gd2.verticalAlignment = GridData.CENTER;
    gd2.grabExcessHorizontalSpace = true;
    gd2.grabExcessVerticalSpace = true;
    
    GridData gd21 = new GridData();
    gd21.horizontalAlignment = GridData.CENTER;
    gd21.verticalAlignment = GridData.CENTER;
    gd21.grabExcessHorizontalSpace = true;
    gd21.grabExcessVerticalSpace = true;

    GridData gd3 = new GridData();
    gd3.horizontalAlignment = GridData.CENTER;
    gd3.verticalAlignment = GridData.CENTER;
    gd3.grabExcessHorizontalSpace = true;
    gd3.grabExcessVerticalSpace = true;

    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    /*
     * Button Lschen
     */
    Composite comp0 = new Composite( table, SWT.NONE );
    comp0.setBackground( table.getBackground() );
    comp0.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp0.setLayoutData( gd1 );
    comp0.setLayout( new GridLayout() );
    Button b = new Button( comp0, SWT.CHECK );
    
    b.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				pruefeObLoeschenAngewaehlt(_tableModelAktuell);
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
    
    /*
     * Combo SswSba AnzeigeTypen
     */
    Composite comp21 = new Composite( table, SWT.NONE );
    comp21.setBackground( table.getBackground() );
    comp21.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp21.setLayoutData( gd21 );
    comp21.setLayout( new GridLayout() );
    
    ComboViewer viewer = new ComboViewer( comp21, SWT.BORDER );
    viewer.setContentProvider(ArrayContentProvider.getInstance());
    viewer.setLabelProvider(new LabelProvider() {
      @Override
      public String getText(Object element) {
        if (element instanceof SystemObject) {
          SystemObject person = (SystemObject) element;
          return person.getName();
        }else if (element instanceof String){
          String s = (String) element;
          return s;
        }
        return super.getText(element);
      }
    });

    
    List<Object> list = new ArrayList<Object>();
    list.add("undefiniert");
    
    SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaAnzeigeTyp");
    if (type != null){
      List<SystemObject> types = type.getElements();
      for (SystemObject so : types)
      {
        list.add(so);         
      }
    }
       
    
    viewer.setInput(list.toArray(new Object[list.size()]));
    viewer.setSelection(new StructuredSelection(list.get(0)));
    viewer.addSelectionChangedListener(new ISelectionChangedListener() {
      
      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        ISelection selection = event.getSelection();
        
        Object source = event.getSource();
        
        Combo comboSource = null;
        if (source instanceof ComboViewer)
        {
          ComboViewer cv = (ComboViewer)source;
          comboSource = cv.getCombo();          
        }
        
        if (selection instanceof StructuredSelection  )
        {
          StructuredSelection ss = (StructuredSelection)selection;
          Object firstElement = ss.getFirstElement();
          if (firstElement instanceof SystemObject)
          {
            SystemObject so = (SystemObject)firstElement;            
            
            if (_mapAnzeigeElemente.containsKey(so.getPid())){
              AnzeigeElementTyp anzeigeTyp = _mapAnzeigeElemente.get(so.getPid());              
              _soAnzeigeTyp = so;              
                            
              
              AnzeigeInhaltTyp inhaltTyp = AnzeigeInhaltTyp.DUNKEL_BI;
              
              if (anzeigeTyp.equals(AnzeigeElementTyp.PR_T) 
                  || anzeigeTyp.equals(AnzeigeElementTyp.PR_T2)
                  || anzeigeTyp.equals(AnzeigeElementTyp.PR_W)
                  || anzeigeTyp.equals(AnzeigeElementTyp.PR_W1)
                  || anzeigeTyp.equals(AnzeigeElementTyp.PR_W11)
                  || anzeigeTyp.equals(AnzeigeElementTyp.PR_W2)
                  ){
                inhaltTyp = AnzeigeInhaltTyp.PR_TLS_PRISMA_SEITE_1;
              }
              
              Image image = createImage(anzeigeTyp, inhaltTyp);
                                           
              for (int i = 0; i < _tableModelAktuell.length; i++)
              {
                TableModel tableModel = _tableModelAktuell[i];
                Composite compAnzeigeTyp = tableModel.getAnzeigeTyp();
                Control control = compAnzeigeTyp.getChildren()[0];
                if (comboSource != null && comboSource.equals(control)){
                  Control control2 = tableModel.getWvzInhalt().getChildren()[0];
                  if (control2 instanceof Label)                
                  {
                    Label label = (Label)control2;
                    label.setImage(image);
                    Rectangle bounds = image.getBounds();                   
                    _height  = Math.max(_height, bounds.height + 2);                                    
                    _tableViewerAktuell.setInput(_tableModelAktuell);                    
                  }                    
                 comboSource.setData(so.getPid());                  
                }                
              } 
              
              SystemObject soWVZInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhaltTyp.getType());
              
              if (soWVZInhalt != null){
                Vector<ParameterDaten> stufen = leseAktuelleEinstellungen(_tableModelAktuell).getParameterDaten();
                for (ParameterDaten stufe : stufen)
                {               
                  if (_soAnzeigeTyp.getPid().equals(stufe.getAnzeigeTyp())){                  
                    if (soWVZInhalt != null)
                      stufe.setWvzInhalt(new WVZInhalt(soWVZInhalt));
                    break;
                  }
                }                
                setParameterDatenInData(stufen);                
              }
              
            }else{
              
            }
            
          }else if (firstElement instanceof String){
            String s = (String) firstElement;
//            _soAnzeigeTyp = null;
          }
        }
        
      }
    });
                 
//    String toolTip21 = "Tooltip fr Combo";        
//    comp21.setToolTipText(toolTip21);
//    viewer.getCombo().setToolTipText(toolTip21);
    viewer.getCombo().setLayoutData( gdTableItem );
    
    /*
     * WVZ Inhalt
     */
    Composite comp3 = new Composite( table, SWT.NONE );
    comp3.setBackground( table.getBackground() );
    comp3.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp3.setLayoutData( gd3 );
    comp3.setLayout( new FillLayout() );
    
    Label lbImageX = new Label( comp3, SWT.CENTER );
    lbImageX.addMouseListener( new MouseAdapter()
    {
			@Override
      public void mouseDown( MouseEvent e )
      {
			  Combo combo = null;
			  
			  for (TableModel tm : _tableModelAktuell)
        {
			    Control control = tm.getWvzInhalt().getChildren()[0];
          if (control.equals(e.getSource())){
            Label label = (Label)control;
            combo = (Combo)tm.getAnzeigeTyp().getChildren()[0];
            String pid = (String)combo.getData();              
            _soAnzeigeTyp = _connection.getDataModel().getObject(pid);
            break;            
          }
        }
			  
        AnzeigeInhaltAuswahlDialog dialog = new AnzeigeInhaltAuswahlDialog(PlatformUI.getWorkbench().getDisplay()
            .getActiveShell());         
        
        ArrayList<String> list = new ArrayList<String>();
        if (_soAnzeigeTyp != null){   
          if (_mapAnzeigeTypen.containsValue(_soAnzeigeTyp.getPid())){
            Set<String> typen = getAnzeigeTypenVonSswSbaAnzeigeTyp(_mapAnzeigeTypen, _soAnzeigeTyp.getPid());
            for (String pidTyp : typen)
            {
              SystemObject anzeigeTyp = _connection.getDataModel().getObject(pidTyp);
              AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.wvzInhalt");                                          
              ConfigurationObject co = (ConfigurationObject)anzeigeTyp;
              if (co != null){
                NonMutableSet set = co.getNonMutableSet("WvzInhalt");
                if (set != null)
                {
                  List<SystemObject> inhalte = set.getElements();
                  for (SystemObject inhalt : inhalte)
                  {
                    Data dta = inhalt.getConfigurationData(atg);
                    String s = dta.getItem("Bildinhalt").asTextValue().getValueText();
                    list.add(s);
                  }              
                }            
              }              
            }
          }          
          EList<String> elist = new BasicEList<String>(list);          
          AnzeigeElementTyp anzeigeTyp = _mapAnzeigeElemente.get(_soAnzeigeTyp.getPid());
          
          AnzeigeInhaltTyp inhalt = dialog.oeffne(anzeigeTyp, elist);          
          
          if (inhalt != null)
          {            
            SystemObject soWVZInhalt = DaVHelper.holeWVZInhaltVonBildInhalt(_connection, inhalt.getType());
            
            if (soWVZInhalt != null){
              Vector<ParameterDaten> stufen = leseAktuelleEinstellungen(_tableModelAktuell).getParameterDaten();
              for (ParameterDaten stufe : stufen)
              {               
                if (_soAnzeigeTyp.getPid().equals(stufe.getAnzeigeTyp())){                  
                  if (soWVZInhalt != null)
                    stufe.setWvzInhalt(new WVZInhalt(soWVZInhalt));
                  break;
                }
              }
              
              setParameterDatenInData(stufen);
              
              final Label im = (Label)e.getSource();
              im.setData( soWVZInhalt.getPid() );
              Image image = createImage(anzeigeTyp, inhalt);
              if (image != null)
                im.setImage(image);
            }            
          }           
        }        
      }         
    });
    
    
    /*
     * Combo SswSba AnzeigeTypen
     */
    Composite comp31 = new Composite( table, SWT.NONE );
    comp31.setBackground( table.getBackground() );
    comp31.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp31.setLayoutData( gd21 );
    comp31.setLayout( new GridLayout() );
    
    GridData gridDataGrp = new GridData();
    gridDataGrp.horizontalAlignment = GridData.FILL;
    gridDataGrp.grabExcessHorizontalSpace = true;
    gridDataGrp.verticalAlignment = GridData.FILL;
    
    ComboViewer viewer2 = new ComboViewer( comp31, SWT.BORDER );
    viewer2.setContentProvider(ArrayContentProvider.getInstance());
    viewer2.setLabelProvider(new LabelProvider() {
      @Override
      public String getText(Object element) {
        if (element instanceof String){
          String s = (String) element;
          return s;
        }
        return super.getText(element);
      }
    });

    
    List<Object> list2 = new ArrayList<Object>();
    list2.add("Undefiniert");
    list2.add("1,000 s");
    list2.add("1,200 s");
    list2.add("1,400 s");
    list2.add("1,600 s");
    list2.add("1,800 s");
    list2.add("2,000 s");
    list2.add("2,200 s");
    list2.add("2,400 s");
    list2.add("2,600 s");
    list2.add("2,800 s");
    list2.add("3,000 s");    
      
    
    viewer2.setInput(list2.toArray(new Object[list2.size()]));
    viewer2.setSelection(new StructuredSelection(list2.get(0)));
    viewer2.addSelectionChangedListener(new ISelectionChangedListener() {
      
      @Override
      public void selectionChanged(SelectionChangedEvent event)
      {
        ISelection selection = event.getSelection();
        
        Object source = event.getSource();                                 
        Combo comboSource = null;
        if (source instanceof ComboViewer)
        {
          ComboViewer cv = (ComboViewer)source;
          comboSource = cv.getCombo();          
        }        
        
        for (TableModel tm : _tableModelAktuell)
        {
          Control control = tm.getBlinkTakt().getChildren()[0];
          
          if (control.equals(comboSource)){            
            Combo comboAnzeige = (Combo)tm.getAnzeigeTyp().getChildren()[0];
            String pid = (String)comboAnzeige.getData();              
            _soAnzeigeTyp = _connection.getDataModel().getObject(pid);
            break;            
          }
        }
        
        
        if (selection instanceof StructuredSelection  )
        {
          StructuredSelection ss = (StructuredSelection)selection;
          Object firstElement = ss.getFirstElement();
          if (firstElement instanceof String){
            String blinkTakt = (String) firstElement;
            
            Vector<ParameterDaten> stufen = leseAktuelleEinstellungen(_tableModelAktuell).getParameterDaten();
            int cnt = 0;
            for (ParameterDaten stufe : stufen)
            {
              if (_soAnzeigeTyp.getPid().equals(stufe.getAnzeigeTyp())){           
                if (blinkTakt != null){                  
                  stufe.setBlinkTakt(blinkTakt);
                }
                break;
              }              
              cnt++;
            }            
            setParameterDatenInData(stufen);                        
          }
        }        
      }
    });
                 
//    String toolTip22 = "Tooltip fr Combo2";        
//    comp31.setToolTipText(toolTip22);
//    viewer2.getCombo().setToolTipText(toolTip22);
    viewer2.getCombo().setLayoutData( gdTableItem );
 
    return new TableModel ( comp0, comp21, comp3, comp31);
  }
  
  /**
   * Methode liefert die Anzahl der Parameter
   * @return Anzahl der Parameter
   */
  public int getAnzahlParameter(Data parameter)
  {
    Array array = getArrayParameter(parameter);
    
    return array.getLength();
  }
  
  /**
   * Methode liefert die Anzahl der Parameter
   * @return Anzahl der Parameter
   */
  public Array getArrayParameter(Data parameter)
  {    
    String bezStufe = "";
    switch(_indexStufe){
      case 2:
        bezStufe = "Kernbereich";
        break;
      case 3:
        bezStufe = "Nachbereich";
        break;
      case 1:
        bezStufe = "Vorbereich1";
        break;
      case 0:
        bezStufe = "Vorbereich2";
        break;
      default:  
    }
    
    String bezBereich = "";
    int selIndex = _tabFolder.getSelectionIndex();
    if (selIndex < 0)
      selIndex = 0;
    switch(selIndex){
      case 0:
        bezBereich = "gesperrt";
        if (_isSpurSperrung)
          bezBereich = "nichtGesperrt";
        break;
      case 1:
        bezBereich = "nichtGesperrt";
        break;      
      default:  
    }
    
    Array array = parameter.getItem("ProgrammTypen").asArray();
      
    int index = 0;
    
    if (_indexPrg < array.getLength()){
      index = _indexPrg;
    }else{
      index = array.getLength()-1;
    }
    
    return array.getItem(index).getItem(bezStufe).getItem(bezBereich).asArray();
  }
  
  /**
   * Renderer fr die Tabelle
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider 
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();
      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getLoeschen();
          break;
                  
        case 1:
          comp = model.getAnzeigeTyp();
          break;
          
        case 2:
          comp = model.getWvzInhalt();
          break;
        
        case 3:
          comp = model.getBlinkTakt();
          break;
          
        default:
      }
      
      TableEditor editor = new TableEditor(item.getParent());
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);

    }
  }
		
	private SonderProgrammTypDaten leseAktuelleEinstellungen (TableModel[] tableModel)
	{
		SonderProgrammTypDaten typ = new SonderProgrammTypDaten();
		
		for (int j=0; j<tableModel.length; ++j)
		{
      TableModel tm = tableModel [j];
                 

      /*
       * WVZ Inhalt
       */
      WVZInhalt x = new WVZInhalt(tm.getSelectedWVZInhalt());
      String sel = tm.getSelectedAnzeigeTyp();
      
      typ.addParameterDaten(sel, x);
		}

		return typ;
  }

	/**
	 * Methode bernimmt die Daten eines anderen Folders
	 * @param tvAnzeige Tableviewer der zu bernhmenden Tabelle
	 */
	public void setTvAnzeige (TableModel[] tableModel, TableViewer tableViewer)
	{
    Vector<ParameterDaten> stufen = new Vector<ParameterDaten>();

    for (int i=0; i<tableModel.length; ++i)
    {
      /*
       * WVZ Inhalt
       */
      WVZInhalt x = new WVZInhalt(tableModel[i].getSelectedWVZInhalt());
      String anzeigeTyp = tableModel [i].getSelectedAnzeigeTyp();
      String blinkTakt = tableModel [i].getSelectedBlinkTakt();
      
  	  stufen.add (new ParameterDaten (anzeigeTyp, x, blinkTakt));
    }
    
    setParameterDatenInData(stufen);
    updateData(_aktuelleDaten, tableModel, tableViewer);
  }
	
	private void setParameterDatenInData(Vector<ParameterDaten> stufen){
	  
	  Array array = getArrayParameter(_aktuelleDaten);
	  array.setLength(stufen.size());
	  
	  for (int i = 0; i < stufen.size(); i++)
    {
	    ParameterDaten stufe = stufen.get(i);
	    String pid1 = stufe.getWvzInhalt().getPid();
	    String pid2 = stufe.getAnzeigeTyp();
	    String blinkTakt = stufe.getBlinkTakt();
	    
	    SystemObject so1 = _connection.getDataModel().getObject(pid1);
	    SystemObject so2 = _connection.getDataModel().getObject(pid2);
	    
      array.getItem(i).getItem("AnzeigeTyp").asReferenceValue().setSystemObject(so2);
      array.getItem(i).getItem("WVZInhalt").getItem("AnzeigeInhalt").asReferenceValue().setSystemObject(so1);
      
      array.getItem(i).getItem("WVZInhalt").getItem("Blinken").asTextValue().setText("Nein");
      array.getItem(i).getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText("Undefiniert");
      
//      System.out.println("setze Blinktakt in Data: " + blinkTakt);
      
      if (blinkTakt != null && !blinkTakt.equals("Undefiniert")){
//        if (blinkTakt.contains(".")){
//          blinkTakt = blinkTakt.replace(".", ",");
          array.getItem(i).getItem("WVZInhalt").getItem("Blinken").asTextValue().setText("Ja");
          array.getItem(i).getItem("WVZInhalt").getItem("Blinktakt").asTextValue().setText(blinkTakt);
//        }
      }      
      array.getItem(i).getItem("WVZInhalt").getItem("DynamischerWechseltext").getItem("TextAusrichtung").asTextValue().setText("Undefiniert");
    }	  	   
	}
	

  private Image createImage(AnzeigeElementTyp typ, AnzeigeInhaltTyp inhalt)
  {
    String path = AnzeigeElementFigure.createImagePath(typ, inhalt);
    if (path != null){
      final Image image = ImageFactory.getImage(path);
 
          if (image != null && !image.isDisposed())
          {
            return image;  
          }
    }
    return null;
  }

  public Data getAktuelleDaten()
  {
    return _aktuelleDaten;
  }

  public void setAktuelleDaten(Data aktuelleDaten)
  {
    _aktuelleDaten = aktuelleDaten;
    
    Array array = _aktuelleDaten.getItem("ProgrammTypen").asArray();
    
    int index = _indexPrg;
    if (_indexPrg < array.getLength()){
      
    }else{
      index = array.getLength()-1;
    }
    
    _isSpurSperrung = !array.getItem(index).getItem("Spursperrung").asTextValue().getValueText().equals("Ja");
    
    updateData(_aktuelleDaten, _tableModelAktuell, _tableViewerAktuell);
    
    if (_isSpurSperrung)
      aktionSpurSperrung();
  }

  @Override
  public void propertyChange(PropertyChangeEvent evt)
  {
    if (evt.getPropertyName().equals(CompositeParameterBereiche.PROP_SPUR_SPERRUNG)){
      CompositeParameterBereiche compStufen = (CompositeParameterBereiche)evt.getSource();
      _isSpurSperrung = !(Boolean)evt.getNewValue();
      aktionSpurSperrung();
    }
  }

  /**
   * 
   */
  private void aktionSpurSperrung()
  {
    
//    System.out.println("CompositeParameterDefinitionen.aktionSpurSperrung(): " +  _tabFolder.getItemCount() + " -> " +  _isSpurSperrung);
    
//    if (_isSpurSperrung)
//    {
//      CTabItem item = _tabFolder.getItem(0);
//      disposeTabItem(item); 
//            
//      CTabItem item0 = _tabFolder.getItem(0);
//      Control c = item0.getControl();
//      if (c instanceof Composite)
//      {       
//        createAndUpdateTableModel();
//      }
//    }
//    else
    {
      CTabItem item3 = null;
      if (_tabFolder.getItemCount() > 2){
        item3 = _tabFolder.getItem(2);
      }
      CTabItem item2 = null;
      if (_tabFolder.getItemCount() > 1){
        item2 = _tabFolder.getItem(1);
      }
      CTabItem item1 = null;
      if (_tabFolder.getItemCount() > 0){
        item1 = _tabFolder.getItem(0);
      }
//      CTabItem item1 = _tabFolder.getItem(0);
      if (item3 != null)
        disposeTabItem(item3);
      if (item2 != null)
        disposeTabItem(item2);
      if (item1 != null)
        disposeTabItem(item1); 
     
      erzeugeTabItems(); 
      
      CTabItem item0 = _tabFolder.getItem(0);
      Control c = item0.getControl();
      if (c instanceof Composite)
      {
        Composite c1 = (Composite)c;
        _tableViewerAktuell = erzeugeTabelle(c1);
        item0.setData(_tableViewerAktuell);
        createAndUpdateTableModel();
      }      
    }
    _tableViewerAktuell.refresh();      
    _tabFolder.setSelection(0);
    _tabFolder.redraw();
    _tabFolder.layout(true);
  }

  /**
   * 
   */
  private void createAndUpdateTableModel()
  {
    _tableModelAktuell = createModel(_tableViewerAktuell.getTable());        
    if ( _tableModelAktuell != null ){
      _tableViewerAktuell.setInput(_tableModelAktuell);
        updateData(_aktuelleDaten, _tableModelAktuell, _tableViewerAktuell);
    }
  }

  /**
   * @param item1
   */
  private void disposeTabItem(CTabItem item1)
  {
    Composite cSel = (Composite)item1.getControl();
    if (((Composite)cSel).getChildren().length > 0)
    {
      Control control = cSel.getChildren()[0];
      if (control.isVisible())
      {
        control.setVisible(false);
      }
    }
    item1.dispose();
  }

  public CTabFolder getTabFolder()
  {
    return _tabFolder;
  }

  public void setTabFolder(CTabFolder tabFolder)
  {
    _tabFolder = tabFolder;
  }

  public TableViewer getTableViewerAktuell()
  {
    return _tableViewerAktuell;
  }

  public void setTableViewerAktuell(TableViewer tableViewerAktuell)
  {
    _tableViewerAktuell = tableViewerAktuell;
  }
}
