/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/

package de.bsvrz.buv.plugin.anlagendarstellung.programmtyp;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.SystemObject;


/**
 * @author Pittner
 *
 */
public class SonderProgrammTypDaten
{
  /**
   * Bezeichnung fr den SonderProgrammTyp.
   */
  private String m_bezeichnung;
  
  /**
   * Vektor mit ParameterDaten fr die Sonderprogrammtyp
   */
  private Vector<ParameterDaten> _daten = new Vector<ParameterDaten>();

  /**
   * Konstruktor der Klasse
   * @param bezeichnung Bezeichnung der Treppenfunktion
   */
  public SonderProgrammTypDaten (String bezeichnung)
  {
    m_bezeichnung = bezeichnung;
  }

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public SonderProgrammTypDaten()
  {
  }

  /**
   * Methode liefert die Variable m_bezeichnung zurck
   * @return Variable m_bezeichnung
   */
  public String getBezeichnung()
  {
    return m_bezeichnung;
  }

  /**
   * Methode setzt die Variable m_bezeichnung
   * @param bezeichnung Wert auf den die Variable m_bezeichnung gesetzt werden soll
   */
  public void setBezeichnung( String bezeichnung )
  {
    m_bezeichnung = bezeichnung;
  }

  /**
   * Methode liefert die Variable m_stufen zurck
   * @return Variable m_stufen
   */
  public Vector<ParameterDaten> getParameterDaten()
  {
    return _daten;
  }

  /**
   * Methode setzt die Variable m_stufen
   * @param stufen Wert auf den die Variable _daten gesetzt werden soll
   */
  public void setParameterDaten( Vector<ParameterDaten> stufen )
  {
    _daten = stufen;
  }

  
  /**
   * Methode zum Ausgeben der Klasse als String
   */
  @Override
  public String toString ()
  {
    String s = "";
    
    s += "Bezeichnung: " + getBezeichnung() + "\n";
    s += "Daten:\n";
    
    int x = 1;
    for (ParameterDaten st : getParameterDaten())
    {
      s += "Parameter " + x + ": " + st.toString();
      x++;
    }
    
    return s;
  }

  /**
   * Methode liefert die Stufe die zu einem WVZ-Inhalt gehrt  
   * @param wvzInhalt WVZ-Inhalt dessen Stufe gesucht wird
   * @return Stufe sofern vorhanden, sonst null
   */
  public ParameterDaten getParameterDaten( SystemObject wvzInhalt )
  {
    synchronized (_daten)
    {
      for (ParameterDaten st : _daten)
      {
        if (st.getWvzInhalt() == wvzInhalt )
          return st;
      }
    } 
    
    return null;
  }

  /**
   * Methode fgt eine Stufe der Treppenfunktion hinzu
   * @param von unterer Schwellwert der Stufe
   * @param bis oberer Schwellwert der Stufe
   * @param wvzInhalt dazugehrender WVZ-Inhalt
   */
  public void addParameterDaten(String anzeigeTyp, WVZInhalt wvzInhalt )
  {
    ParameterDaten daten = new ParameterDaten ();
    daten.setAnzeigeTyp(anzeigeTyp);
    daten.setWvzInhalt( wvzInhalt );

    synchronized (_daten)
    {
      _daten.add( daten );
    }
  }
}
