/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/

package de.bsvrz.buv.plugin.anlagendarstellung.programmtyp;

import java.util.Vector;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse realisiert den WVZ-Inhalt einer Anzeige
 * @author Pittner
 */
public class WVZInhalt
{
  /**
   * Objekt mit dem Anzeigeninhalt
   */
  private SystemObject m_anzeigenInhalt;
  
  /**
   * Blinken
   */
  private boolean m_blinken = false;
  
  /**
   * Blinktakt
   */
  private float m_blinkTakt = 0;
  
  /**
   * Grafikdarstellungen
   */
  private Vector<SystemObject> m_grafikDarstellungen = new Vector<SystemObject>();
  
  /**
   * Wechseltext
   */
  private SystemObject m_wechselText;
  
  /**
   * Dynamischer Wechseltext
   */
  private DynamischerWechselText m_dynamischerWechselText = new DynamischerWechselText ();

  /**
   * PID des WVZ-Inhalts
   */
  private String m_pid;

  /**
   * Parameterloser Konstruktor der Klasse
   */  
  public WVZInhalt ()
  {
    
  }
  
  /**
   * Konstruktor der Klasse. Dieser Konstruktor liefer einen WVZ-Inhalt mit dem 
   * Anzeigeinhalt der bergebenen PID
   * @param connection Verbindung zum DaV
   * @param pid Pid des Anzeigeinhalts
   */
  public WVZInhalt (ClientDavConnection connection, String pid)
  {
    m_pid = pid;
    
    if (connection != null)
    {
      DataModel dataModel = connection.getDataModel();
      
      ConfigurationObject co = (ConfigurationObject) dataModel.getObject (pid);
      
      if (co == null)
      	System.err.println("Kein Konfigurationsobjekt fr Pid " + pid + " gefunden !");
      
      m_anzeigenInhalt = (SystemObject) co;
    }
  }

  /**
   * Konstruktor der Klasse. Dieser Konstruktor liefer einen WVZ-Inhalt mit dem 
   * Anzeigeinhalt der bergebenen PID
   * @param connection Verbindung zum DaV
   * @param pid Pid des Anzeigeinhalts
   */
  public WVZInhalt (SystemObject so)
  {
    m_pid = so.getPid();
    
    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    if (connection != null)
    {
      m_anzeigenInhalt = so;
    }
  }

  /**
   * Konstruktor der Klasse
   * @param pid Pid des WVZ-Inhalts
   */
  public WVZInhalt(String pid) 
  {
  	if (pid == null)
  		return;
  	
  	ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
	
    DataModel dataModel = connection.getDataModel();
    
    ConfigurationObject co = (ConfigurationObject) dataModel.getObject (pid);
    m_anzeigenInhalt = (SystemObject) co;
  }

/**
   * Methode liefert die Variable m_anzeigenInhalt zurck
   * @return Variable m_anzeigenInhalt
   */
  public SystemObject getAnzeigenInhalt()
  {
    return m_anzeigenInhalt;
  }

  /**
   * Methode setzt die Variable m_anzeigenInhalt
   * @param anzeigenInhalt Wert auf den die Variable m_anzeigenInhalt gesetzt werden soll
   */
  public void setAnzeigenInhalt( SystemObject anzeigenInhalt )
  {
    m_anzeigenInhalt = anzeigenInhalt;
    
    if (anzeigenInhalt != null)
      m_pid = anzeigenInhalt.getPid();
  }

  /**
   * Methode liefert die Variable m_blinken zurck
   * @return Variable m_blinken
   */
  public boolean isBlinken()
  {
    return m_blinken;
  }

  /**
   * Methode setzt die Variable m_blinken
   * @param blinken Wert auf den die Variable m_blinken gesetzt werden soll
   */
  public void setBlinken( boolean blinken )
  {
    m_blinken = blinken;
  }

  /**
   * Methode liefert die Variable m_blinkTakt zurck
   * @return Variable m_blinkTakt
   */
  public float getBlinkTakt()
  {
    return m_blinkTakt;
  }

  /**
   * Methode setzt die Variable m_blinkTakt
   * @param blinkTakt Wert auf den die Variable m_blinkTakt gesetzt werden soll
   */
  public void setBlinkTakt( float blinkTakt )
  {
    m_blinkTakt = blinkTakt;
  }

  /**
   * Methode liefert die Variable m_grafikDarstellungen zurck
   * @return Variable m_grafikDarstellungen
   */
  public Vector<SystemObject> getGrafikDarstellungen()
  {
    Vector<SystemObject> v = new Vector<SystemObject>();
    
    synchronized (m_grafikDarstellungen)
    {
      for (SystemObject o : m_grafikDarstellungen)
        v.add( o );
    }
    return v;
  }

  /**
   * Methode setzt die Variable m_grafikDarstellungen
   * @param grafikDarstellungen Wert auf den die Variable m_grafikDarstellungen gesetzt werden soll
   */
  public void setGrafikDarstellungen( Vector<SystemObject> grafikDarstellungen )
  {
    synchronized (m_grafikDarstellungen)
    {
      m_grafikDarstellungen = grafikDarstellungen;
    }
  }

  /**
   * Methode liefert die Variable m_wechselText zurck
   * @return Variable m_wechselText
   */
  public SystemObject getWechselText()
  {
    return m_wechselText;
  }

  /**
   * Methode setzt die Variable m_wechselText
   * @param wechselText Wert auf den die Variable m_wechselText gesetzt werden soll
   */
  public void setWechselText( SystemObject wechselText )
  {
    m_wechselText = wechselText;
  }

  /**
   * Methode liefert die Variable m_dynamischerWechselText zurck
   * @return Variable m_dynamischerWechselText
   */
  public DynamischerWechselText getDynamischerWechselText()
  {
    return m_dynamischerWechselText;
  }

  /**
   * Methode setzt die Variable m_dynamischerWechselText
   * @param dynamischerWechselText Wert auf den die Variable m_dynamischerWechselText gesetzt werden soll
   */
  public void setDynamischerWechselText( DynamischerWechselText dynamischerWechselText )
  {
    m_dynamischerWechselText = dynamischerWechselText;
  }
  
  /**
   * Methode zum Auswerten der Attributgruppe des DaV die den WVZ-Inhalt darstellt.
   * @param data Datensatz vom DaV
   */
  public void auswerteAttributgruppe (Data data)
  {
    m_anzeigenInhalt = data.getReferenceValue( "AnzeigeInhalt" ).getSystemObject();
    
    setBlinken( data.getItem( "Blinken" ).asTextValue().getText().equalsIgnoreCase( "Ja" ) );
    
    m_blinkTakt = data.getItem( "Blinktakt" ).asUnscaledValue().floatValue();
    
    synchronized (m_grafikDarstellungen)
    {
      m_grafikDarstellungen.clear();
      
      ReferenceArray grafik = data.getReferenceArray( "GrafikDarstellungen" );
      int anzahl = grafik.getLength();
      for (int x = 0; x < anzahl; ++x)
      {
        SystemObject o = grafik.getSystemObject( x );
        
        m_grafikDarstellungen.add( o );
      }
    }
    
    m_wechselText = data.getReferenceValue( "Wechseltext" ).getSystemObject();
    
    Data dynWechselText = data.getItem( "DynamischerWechseltext" );
    m_dynamischerWechselText.auswerteAttributGruppe( dynWechselText );
  }

  /**
   * Methode zum Ausgeben des WVZInhalts als String
   */
  public String toString ()
  {
    String s = "";
    
    if (getAnzeigenInhalt() != null)
      s += "Anzeigeninhalt: " + getAnzeigenInhalt().getName() + "\n";
    else
    {
      s += "Anzeigeninhalt: " + "null\n";
    
      s += "Blinken:        " + isBlinken() + "\n";
      s += "Blinktakt:      " + getBlinkTakt() + "\n";
      s += "Grafikdarstellung:" + "\n";
      for (SystemObject o : getGrafikDarstellungen())
        s += "-" + o.getPid() + "\n";
      s += "Wechseltext:    " + getWechselText() + "\n";
      s += m_dynamischerWechselText.toString();
    }
    
    return s;
  }
  
  /**
   * Methode prft ob zwei Objekte vom Typ WVZInhalt inhaltlich gleich sind.<br>
   * Dabei wird wie folgt vorgegangen:<br>
   * 1. Prfen ob der Anzeigeinhalt versorgt ist, wenn ja werden nur die zum Anzeigeinhalt
   * gehrenden Attribute (Inhalt + Blinken) verglichen.<br>
   * 2. Prfen ob ein Grafikdarstellung versorgt ist, wenn ja wird nur die Grafikdarstellung 
   * verglichen<br>
   * 3. Prfen ob ein Wechseltext versorgt ist, wenn ja wird nur der Wechseltext verglichen<br>
   * 4. Prfen ob ein dyn. Wechseltext versorgt ist, wenn ja wird nur der dyn. Wechseltext verglichen<br>
   * 5. Wenn kein sinnvollers Schaltbild bei dem aktuellen und dem zu vergleichenden WVZInhalt vorliegt,
   * wird true zurckgeliefert.<br>
   * @param vgl Objekt mit dem verglichen werden soll
   * @return wenn Inhalt entsprechend obiger Kriterien identisch ist dann true, sonst false
   */
  public boolean isInhaltlichGleich (WVZInhalt vgl)
  {
    if (vgl == null)
      return false;
    
    // 1. Prfen ob Anzeigeninhalt als SystemObjekt vorgegeben wurde
    
    if (getAnzeigenInhalt() != null || vgl.getAnzeigenInhalt() != null)
    {
      if (getAnzeigenInhalt() != vgl.getAnzeigenInhalt())
        return false;
      
//      if (isBlinken() != vgl.isBlinken())
//        return false;
//      
//      if (getBlinkTakt() != vgl.getBlinkTakt())
//        return false;
      
      return true;
    }
    
    // 2. Prfen der Grafikdarstellung
    
    if (getGrafikDarstellungen().size() > 0 || vgl.getGrafikDarstellungen().size() > 0)
    {
      Vector<SystemObject> gd1 = getGrafikDarstellungen();
      Vector<SystemObject> gd2 = vgl.getGrafikDarstellungen();

      if (gd1.size() != gd2.size())
        return false;
      
      for (int i = 0; i<gd1.size(); ++i)
      {
        if (gd1.get( i ) != gd2.get( i ))
          return false;
      }
      
      return true;
    }
    
    // 3. Prfen des Wechseltextes
    
    if (getWechselText() != null || vgl.getWechselText() != null)
    {
      if (getWechselText() != vgl.getWechselText())
        return false;
      
      return true;
    }
    
    // 4. Dynamischer Wechseltext
    
    if ((getDynamischerWechselText().getTextDefinition().size() > 0) ||
        (vgl.getDynamischerWechselText().getTextDefinition().size() > 0))
    {
      DynamischerWechselText wt1 = getDynamischerWechselText();
      DynamischerWechselText wt2 = vgl.getDynamischerWechselText();
      
      if (!wt1.isInhaltlichGleich(wt2))
        return false;
      
      return true;
    }
      
    return true;
  }

  /**
   * Methode liefert Informationen darber, ob es sich bei dem WVZ-Inhalt um den
   * Inhalt beliebig handelt.
   * @return WVZ-Inhalt = "beliebig"
   */
  public boolean isBeliebig()
  {
//    return this.isInhaltlichGleich( SpezialWVZInhalte._beliebig );
    return false;
  }
  
  /**
   * Methode liefert Informationen darber, ob es sich bei dem WVZ-Inhalt um den
   * Inhalt "Vorigen Inhalt beibehalten" handelt.
   * @return WVZ-Inhalt = "Vorigen Inhalt beibehalten"
   */
  public boolean isBeibehalten()
  {
//    return this.isInhaltlichGleich( SpezialWVZInhalte._beibehalten );
    return false;
  }

  /**
   * Methode liefert die Variable m_pid zurck
   * @return Variable m_pid
   */
  public String getPid()
  {
  	if (m_pid == null)
  		if (getAnzeigenInhalt() != null)
  		  return getAnzeigenInhalt().getPid();
  	
    return m_pid;
  }

  /**
   * Methode setzt die Variable m_pid
   * @param pid Wert auf den die Variable m_pid gesetzt werden soll
   */
  public void setPid( String pid )
  {
    m_pid = pid;
  }

  /**
   * Methode speicher den WVZ-Inhalt in eine Attributliste vom Typ 'atl.sswSbaWVZInhalt'.
   * ber die Verbindung zum DaV wird das Systemobjekt Anzeigeinhalt bestimmt.
   * @param connection Verbindung zum DaV
   * @param data Attributgruppe in die die Daten geschrieben werden sollen
   */
  public void erzeugeAttributgruppe( ClientDavConnection connection, Data data )
  {
    SystemObject inhalt = getAnzeigenInhalt();
    if (inhalt != null)
    {
      DataModel dataModel = connection.getDataModel();
      ConfigurationObject co = (ConfigurationObject) dataModel.getObject (inhalt.getPid());
      if (co != null)
      {
        data.getReferenceValue( "AnzeigeInhalt" ).setSystemObject( co );
        
        if (isBlinken())
          data.getTextValue( "Blinken" ).setText( "Ja" );
        else
          data.getTextValue( "Blinken" ).setText( "Nein" );
        
        data.getUnscaledValue( "Blinktakt" ).set( getBlinkTakt() );
        
        Array grafikDarstellung = data.getArray( "GrafikDarstellungen" );
        grafikDarstellung.setLength( 0 );
        
        data.getReferenceValue( "Wechseltext" ).setSystemObject( null );
        
        Data dynamischerWechseltext = data.getItem( "DynamischerWechseltext" );
        
        dynamischerWechseltext.getReferenceValue( "Font" ).setSystemObject( null );
        dynamischerWechseltext.getUnscaledValue( "TextAusrichtung" ).set( -1 );
        
        Array textDefinition = dynamischerWechseltext.getArray( "Textdefinition" );
        textDefinition.setLength( 0 );
      }
    }
  }

  /**
   * Methode speicher den WVZ-Inhalt in eine Attributliste vom Typ 'atl.sswSbaWVZInhalt'
   * @param data Attributgruppe in die die Daten geschrieben werden sollen
   */
  public void erzeugeAttributgruppe( Data data )
  {
    data.getReferenceValue( "AnzeigeInhalt" ).setSystemObject( getAnzeigenInhalt() );
    
    if (isBlinken())
      data.getTextValue( "Blinken" ).setText( "Ja" );
    else
      data.getTextValue( "Blinken" ).setText( "Nein" );
    
    data.getUnscaledValue( "Blinktakt" ).set( getBlinkTakt() );
    
    Array grafikDarstellung = data.getArray( "GrafikDarstellungen" );
    grafikDarstellung.setLength( 0 );
    
    data.getReferenceValue( "Wechseltext" ).setSystemObject( null );
    
    Data dynamischerWechseltext = data.getItem( "DynamischerWechseltext" );
    
    dynamischerWechseltext.getReferenceValue( "Font" ).setSystemObject( null );
    dynamischerWechseltext.getUnscaledValue( "TextAusrichtung" ).set( -1 );
    
    Array textDefinition = dynamischerWechseltext.getArray( "Textdefinition" );
    textDefinition.setLength( 0 );
  }

  /**
   * Methode liefert eine Kopie der Instanz zurck
   * @return Kopie der Instanz
   */
  public WVZInhalt getCopy()
  {
    WVZInhalt copy = new WVZInhalt();
    
    copy.setAnzeigenInhalt( getAnzeigenInhalt() );
    copy.setBlinken( isBlinken() );
    copy.setBlinkTakt( getBlinkTakt() );
    copy.setGrafikDarstellungen( getGrafikDarstellungen() );
    copy.setWechselText( getWechselText() );
    copy.setDynamischerWechselText( getDynamischerWechselText() );
    
    return copy;
  }

  /**
   * Methode prft ob es sich bei dem WVZ-Inhalt umd "Dunkel" handelt
   * @return Information es sich bei dem WVZ-Inhalt umd "Dunkel" handelt
   */
  public boolean isDunkel()
  {
//    if (getAnzeigenInhalt() != null)
//      return this.isInhaltlichGleich( SpezialWVZInhalte._dunkel );
      
    return false;
  }
}
