/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.anlagendarstellung.semi;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm.SonderProgramm;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ZustandsDarstellungSonderProgramm;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Singleton-Klasse zum Verwalten der Dialoge {@link DialogSemi} zum Schalten
 * bzw. Nicht schalten von UrsachenEinheiten.
 * 
 * @author Pittner
 * 
 */
public class DialogSemiGuiManager implements IDaVDataUpdateListener
{
	/**
	 * Instanz des Singletons
	 */
	private static DialogSemiGuiManager _instance = null;
	
	/**
	 * Instanz des einzigen Dialogs
	 */
	private DialogSemi _dialog = null;
	
	/**
	 * Liste der gltigen Aspekte
	 */
	private List<String> _listAspekte = new ArrayList<String>();
	
	private boolean _aspekteAngemeldet = false;
	
	/**
	 * Map mit denm Anfragen
	 */
	private Map<String, MassnahmenAnfrageGuiData> _mapData = new HashMap<String, MassnahmenAnfrageGuiData>();


	/**
	 * Konstruktor, meldet sich auf die internen GUI-Daten an
	 */
	private DialogSemiGuiManager()
	{	  	 
	  DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);
	}

	/**
	 * Liefert die Instanz des Singletons
	 * @return
	 */
	public static DialogSemiGuiManager getInstance()
	{
		if (_instance == null)
		{
			synchronized (DialogSemiGuiManager.class)
			{
				if (_instance == null)
				{
					_instance = new DialogSemiGuiManager();
				}
			}
		}

		return _instance;
	}

	/**
	 * Setzt die Instanz des Dialogs auf null, damit dieser entfernt werden kann
	 */
	public void removeDialogSchalten()
	{
	  _dialog = null;
	}

	/**
	 * ffnet einen Dialog zum Schalten der UrsachenEinheiten. Bevor der
	 * Dialog jedoch geffnet wird wird geprft, ob bereits ein Dialog existiert.
	 * Wenn nicht wird ein neuer Dialog erzeugt.
	 * 
	 * @param map
	 *         Die Map mit den Anfragen
	 */
	public void openDialogSchalten(final Map<String, MassnahmenAnfrageGuiData> map)
	{
		if (map != null)
		{
		  DialogSemi dialogSchalten = null;
			if (_dialog != null)
			{
				dialogSchalten = _dialog;
			}
			
			if (dialogSchalten == null)
			{
				dialogSchalten = createNewDialogWeiterschalten();
				_dialog = dialogSchalten;

				final DialogSemi fDialogSchalten = dialogSchalten;
				Runnable runNeuenDlgOeffnen = new Runnable()
				{
					@Override
					public void run()
					{
						fDialogSchalten.oeffne(map);
					}
				};
				PlatformUI.getWorkbench().getDisplay().asyncExec(runNeuenDlgOeffnen);

			} else
			{
			  dialogSchalten.setTableInput(map);
				final DialogSemi fDialogSchalten = dialogSchalten;
				Runnable runExistierendenDlgInVordergrundBringen = new Runnable()
				{
					@Override
					public void run()
					{
						fDialogSchalten.getParent().forceActive();
					}
				};
				PlatformUI.getWorkbench().getDisplay().asyncExec(
						runExistierendenDlgInVordergrundBringen);
			}

		}
	}

	/**
	 * Erzeugt einen neuen Dialog. Synchronisiert an Hand des UI-Threads. Kann von
	 * jedem anderen Thread aus aufgerufen werden.
	 * 
	 * @return
	 */
	private DialogSemi createNewDialogWeiterschalten()
	{
		Runnable run = new Runnable()
		{
			@Override
			public void run()
			{
				Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
						.getShell();

				DialogSemi dlgNeu = new DialogSemi(shell,
						DialogSemiGuiManager.this);

				_dialog = dlgNeu;
			}
		};

		PlatformUI.getWorkbench().getDisplay().syncExec(run);
		return _dialog;
	}

	
  @Override
  public void daVDataUpdate(DaVDataUpdateEvent e)
  {
    ResultData rd = e.getResult();
    if (rd != null && rd.hasData()){
      processDaVData(rd);
    }    
  }

  /**
   * Verarbeitet die DaV-Daten
   * @param rd
   */
  /**
   * @param rd
   */
  private void processDaVData(ResultData rd)
  {
    DataDescription dd = rd.getDataDescription();
    Data data = rd.getData();
    AttributeGroup atg = dd.getAttributeGroup();
    Aspect asp = dd.getAspect();
    if (atg.getPid().equals("atg.sswSbaZuBearbeitendeAspekte")){
      Array array = data.getItem("Zuordnungen").asArray();
      
      _listAspekte.clear();
      
      for (int i = 0; i < array.getLength(); i++)
      {
        Data item = array.getItem(i);
        Array array3 = item.getItem("AspekteEingang").asArray();
        for (int j = 0; j < array3.getLength(); j++)
        {
          Data item3 = array3.getItem(j);
          SystemObject so = item3.asReferenceValue().getSystemObject();
          if (so != null){
            if (!_listAspekte.contains(so.getPid()))
              _listAspekte.add(so.getPid());
          }
        }
      }        
      if (_listAspekte.size() > 0 && !_aspekteAngemeldet){
        DaVKommunikationDobj.getInstanz().anmeldenAufDatenSemi(_listAspekte);
        _aspekteAngemeldet = true;
      }
    }
    else if (atg.getPid().equals("atg.sswSbaMassnahmenAnfrageGUI")){
      
      SystemObject ursachenEinheit = data.getItem("Ursacheneinheit").asReferenceValue().getSystemObject();        
      boolean spontan = data.getItem("spontan").asTextValue().getValueText().equals("Ja");
      boolean loeschen = data.getItem("lschen").asTextValue().getValueText().equals("Ja");
      
      long nummer = data.getItem("BenutzerInfo").getItem("nummer").asUnscaledValue().longValue();
      String absenderAlgorithmus = data.getItem("BenutzerInfo").getItem("absenderAlgorithmus").asTextValue().getValueText();
      SystemObject absenderApplikation = data.getItem("BenutzerInfo").getItem("absenderApplikation").asReferenceValue().getSystemObject();
      SystemObject empfaenger = data.getItem("BenutzerInfo").getItem("empfnger").asReferenceValue().getSystemObject();
      int alternative = data.getItem("Alternative").asUnscaledValue().intValue();
      Array array = data.getItem("EinzelMassnahmeAlternative").asArray();
      
      List<SonderProgramm> listeSchaltbilder = new ArrayList<SonderProgramm>();
      for (int i = 0; i < array.getLength(); i++)
      {
        Data item = array.getItem(i);
        SonderProgramm sonderProgramm = new SonderProgramm();
        ZustandsDarstellungSonderProgramm.schreibeDataSchaltbild(sonderProgramm, item);
        listeSchaltbilder.add(sonderProgramm);
      }
      
      SonderProgramm[] schaltBilder = listeSchaltbilder.toArray(new SonderProgramm[listeSchaltbilder.size()]);
      
      // Interne Datenstruktur wird erzeugt
      MassnahmenAnfrageGuiData daten = new MassnahmenAnfrageGuiData(ursachenEinheit, 
          spontan, 
          loeschen, 
          nummer, 
          absenderAlgorithmus, 
          absenderApplikation, 
          empfaenger, 
          alternative, 
          asp, 
          schaltBilder);
      
      boolean entfernt = false;
      if (!_mapData.containsKey(ursachenEinheit.getPid())){

      }else{
        entfernt = true;
        _mapData.remove(ursachenEinheit.getPid());
      }
      
      boolean geloescht = false;
      if (schaltBilder.length > 0)          
        _mapData.put(ursachenEinheit.getPid(), daten);
      else{
        geloescht = true;
      }
      
      if (_mapData != null && !geloescht || geloescht && entfernt)
        openDialogSchalten(_mapData);        
    }
  }

  /**
   * Anmelden auf die DaV-Daten
   */
  public void anmeldenAufDaten()
  {
    DaVKommunikationDobj.getInstanz().anmeldenAufDatenSemiAspekt();    
  }
  
  /**
   * Abmelden der DaV-Daten
   */
  public void abmeldenAufDaten()
  {
    DaVKommunikationDobj.getInstanz().abmeldenAufDatenSemiAspekt();
    if (_listAspekte != null && _listAspekte.size() > 0)
      DaVKommunikationDobj.getInstanz().abmeldenAufDatenSemi(_listAspekte);    
  }
}
