/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm;

import java.util.Formatter;
import java.util.SortedMap;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.views.ZustandsDarstellungSonderProgramm;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Die Klasse realisert die Darstellung und Bearbeitung einer Sperrung des Streckenbereichs eines Sonderprogramms. Diese
 * wird in einer Registerkarte der Composites {@code CompositeSperrungBereiche} verwendet.
 * 
 * @author Pittner
 * 
 */
public class CompositeSperrungDarstellung extends Composite
{
  /**
   * Die Verbindung zum Datenverteiler
   */
  private ClientDavInterface _connection;

  /**
   * Die aktuelle Sperrung
   */
  private int _indexPrg;

  /**
   * Label
   */
  private Label label31;

  private Label label32;

  private Label labelStartKmSperrung;

  private Label labelStopKmSperrung;

  /**
   * Text
   */
  private Text textStartKmSperrung;

  private Text textStopKmSperrung;

  /**
   * Group
   */
  private Group groupSperrung;

  /**
   * Composite
   */
  private Composite compositeSperrung;

  /**
   * Die SecondaryId der View, damit man weis in welchem Kontext man sich befindet
   */
  private String _secId;

  /**
   * Der Datensatz des Sonderprogramms
   */
  private SonderProgramm _sonderProgrammDatensatz;

  /**
   * Konstruktor
   * 
   * @param parent
   *          das Elternobjekt
   * @param style
   *          der Stil wie das {@code Composite} dargestellt werden soll
   * @param so
   *          das Systemobjekt
   * @param sonderProgrammDatensatz
   *          der Datensatz
   * @param secId
   *          die SecId der View
   */
  public CompositeSperrungDarstellung(Composite parent, int style, int indexPrg,
      SonderProgramm sonderProgrammDatensatz, String secId)
  {

    super(parent, style);
    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
    _sonderProgrammDatensatz = sonderProgrammDatensatz;
    _indexPrg = indexPrg;
    _secId = secId;

    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = false;
    gridData1.verticalAlignment = GridData.CENTER;

    setLayoutData(gridData1);
    setLayout(new GridLayout());

    createInhaltTab();
  }

  /**
   * 
   */
  private void createInhaltTab()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = false;
    gridData1.verticalAlignment = GridData.CENTER;
    createComposite1122(this);
  }

  private void createComposite1122(Composite parent)
  {

    GridData gridData11 = new GridData();
    gridData11.verticalAlignment = GridData.CENTER;
    gridData11.horizontalAlignment = GridData.FILL;
    GridData gridData12 = new GridData();
    gridData12.grabExcessHorizontalSpace = true;
    gridData12.verticalAlignment = GridData.CENTER;
    gridData12.horizontalAlignment = GridData.FILL;
    GridData gridData13 = new GridData();
    gridData13.grabExcessHorizontalSpace = true;
    gridData13.verticalAlignment = GridData.CENTER;
    gridData13.horizontalAlignment = GridData.FILL;

    GridData gridData18 = new GridData();
    gridData18.horizontalAlignment = GridData.FILL;
    gridData18.grabExcessHorizontalSpace = true;
    gridData18.grabExcessVerticalSpace = false;
    gridData18.verticalAlignment = GridData.CENTER;
    GridData gridData20 = new GridData();
    gridData20.horizontalAlignment = GridData.FILL;
    gridData20.grabExcessHorizontalSpace = true;
    gridData20.grabExcessVerticalSpace = false;
    gridData20.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout13 = new GridLayout();
    gridLayout13.numColumns = 4;
    gridLayout13.makeColumnsEqualWidth = false;
    GridLayout gridLayout19 = new GridLayout();
    gridLayout19.numColumns = 8;
    gridLayout19.makeColumnsEqualWidth = false;
    gridLayout19.horizontalSpacing = 1;
    compositeSperrung = new Composite(parent, SWT.NONE);
    compositeSperrung.setLayoutData(gridData18);
    compositeSperrung.setLayout(gridLayout13);
    label31 = new Label(compositeSperrung, SWT.NONE);
    label31.setLayoutData(gridData11);
    label31.setText("startKm");
    textStartKmSperrung = new Text(compositeSperrung, SWT.BORDER | SWT.RIGHT);
    textStartKmSperrung.setTextLimit(7);
    textStartKmSperrung.setText("000,000");
    textStartKmSperrung.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent event)
      {
        switch (event.keyCode)
        {
          case SWT.CR:
            if (!DaVHelper.validiereTextInput(_secId, textStartKmSperrung.getText(), textStopKmSperrung.getText()))
            {
              event.doit = false;
            }
            break;
          case SWT.ESC:            
            break;
        }
      }
    });

    Button btnBereichVon = new Button(compositeSperrung, SWT.NONE);
    btnBereichVon.setText("ber AQ");
    btnBereichVon.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionStartKm();
      }
    });

    labelStartKmSperrung = new Label(compositeSperrung, SWT.NONE);
    labelStartKmSperrung.setLayoutData(gridData12);
    labelStartKmSperrung.setText("keine Ausw.");

    label32 = new Label(compositeSperrung, SWT.NONE);
    label32.setLayoutData(gridData11);
    label32.setText("endeKm ");

    textStopKmSperrung = new Text(compositeSperrung, SWT.BORDER | SWT.RIGHT);
    textStopKmSperrung.setTextLimit(7);
    textStopKmSperrung.setText("000,000");
    textStopKmSperrung.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent event)
      {
        switch (event.keyCode)
        {
          case SWT.CR:
            if (!DaVHelper.validiereTextInput(_secId, textStartKmSperrung.getText(), textStartKmSperrung.getText()))
            {
              event.doit = false;
            }
            break;
          case SWT.ESC:
            // System.out.println(SWT.ESC);
            break;
        }
      }
    });

    Button btnBereichBis = new Button(compositeSperrung, SWT.NONE);
    btnBereichBis.setText("ber AQ");
    btnBereichBis.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionStopKm();

      }
    });

    labelStopKmSperrung = new Label(compositeSperrung, SWT.NONE);
    labelStopKmSperrung.setLayoutData(gridData12);
    labelStopKmSperrung.setText("keine Ausw.");

    GridData gridData19 = new GridData();
    gridData19.horizontalAlignment = GridData.FILL;
    gridData19.grabExcessVerticalSpace = false;
    gridData19.verticalAlignment = GridData.FILL;
    gridData19.horizontalSpan = 3;
    GridLayout gridLayout14 = new GridLayout();
    gridLayout14.numColumns = 2;
    gridLayout14.makeColumnsEqualWidth = false;
    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.FILL;
    gridData21.grabExcessHorizontalSpace = true;
    gridData21.verticalAlignment = GridData.CENTER;
    groupSperrung = new Group(parent, SWT.NONE);
    groupSperrung.setText("Fahrstreifen");
    groupSperrung.setLayoutData(gridData20);
    groupSperrung.setLayout(gridLayout19);

    for (int i = 0; i < 8; i++)
    {
      Button btn = new Button(groupSperrung, SWT.CHECK);

      String ss = "";
      String tt = "";

      switch (i)
      {
        case 0:
          ss = "HF";
          tt = ("HFS");
          break;
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
          ss = i + "F";
          tt = i + "FS";
          break;
        case 7:
          ss = "TSS";
          tt = "STNDS";
          break;
        default:
      }
      ;

      btn.setText(ss);
      btn.setToolTipText(tt);
      // initialisiereControls();
    }

  }

  public Label getLabelStartKmSperrung()
  {
    return labelStartKmSperrung;
  }

  public Label getLabelStopKmSperrung()
  {
    return labelStopKmSperrung;
  }

  /**
   * @return
   */
  public Group getGroupSperrung()
  {
    return groupSperrung;
  }

  /**
   * @return
   */
  public int getIndexPrg()
  {
    return _indexPrg;
  }

  /**
   * @return
   */
  public int setIndexPrg(int indexPrg)
  {
    return _indexPrg = indexPrg;
  }

  public SonderProgramm getSonderProgrammDatensatz()
  {
    return _sonderProgrammDatensatz;
  }

  public void setSonderProgrammDatensatz(SonderProgramm sonderProgrammDatensatz)
  {
    _sonderProgrammDatensatz = sonderProgrammDatensatz;
  }

  public Text getTextStopKmSperrung()
  {
    return textStopKmSperrung;
  }

  public Text getTextStartKmSperrung()
  {
    return textStartKmSperrung;
  }

  /**
   * 
   */
  public void initialisiereControls()
  {
    IPreferenceStore store = Activator.getDefault().getPreferenceStore();

    String pref1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_NO;
    String pref2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_NO;

    if (_secId.equals("SW"))
    {
      pref1 = DefsPreference.P_KILOMETER_ANFANG_DARSTELLUNG_SONDER_SW;
      pref2 = DefsPreference.P_KILOMETER_ENDE_DARSTELLUNG_SONDER_SW;
    }

    double d3 = store.getDouble(pref1);
    double d4 = store.getDouble(pref2);

    boolean absteigend = DaVHelper.ermittleKilometrierungAufsteigendAbsteigend(_secId);

    Double von = new Double(0);
    SortedMap<String, SystemObject> listeKilometrierungAq = DaVHelper.holeListeKilometrierungAq(_secId, absteigend, d3,
        d4);
    if (listeKilometrierungAq != null && !listeKilometrierungAq.isEmpty())
    {
      String fkey = listeKilometrierungAq.firstKey();
      String[] split = fkey.split("_");
      if (split.length > 0)
        von = new Double(split[0]);
    }

    String s = new Formatter().format("%1$7.3f", von.doubleValue()).toString();
    textStartKmSperrung.setText(s);

    Double bis = new Double(0);
    if (listeKilometrierungAq != null && !listeKilometrierungAq.isEmpty())
    {
      String lkey = listeKilometrierungAq.lastKey();
      String[] split = lkey.split("_");
      if (split.length > 0)
        bis = new Double(split[0]);
    }

    s = new Formatter().format("%1$7.3f", bis.doubleValue()).toString();
    textStopKmSperrung.setText(s);

    ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
    if (connection != null)
    {
      ZustandsDarstellungSonderProgramm.markiereLabelAq(connection, _secId, textStartKmSperrung, labelStartKmSperrung);
      ZustandsDarstellungSonderProgramm.markiereLabelAq(connection, _secId, textStopKmSperrung, labelStopKmSperrung);
    }

    Control[] children = groupSperrung.getChildren();
    for (int i = 0; i < children.length; i++)
    {
      Control control = children[i];
      if (control instanceof Button)
      {
        Button btn = (Button)control;
        btn.setSelection(false);

      }
    }
  }

  /**
   * 
   */
  private void aktionStartKm()
  {
    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    IViewReference ref = page.findViewReference(ZustandsDarstellungSonderProgramm.ID, _secId);
    if (ref != null)
    {
      IViewPart part = ref.getView(true);
      if (part instanceof ZustandsDarstellungSonderProgramm)
      {
        ZustandsDarstellungSonderProgramm view = (ZustandsDarstellungSonderProgramm)part;
        Double d1 = null;
        Double d2 = null;

        String s1 = view.getTextStartKm().getText().trim();
        String s2 = view.getTextStopKm().getText().trim();
        String s11 = s1.trim();
        String s21 = s2.trim();
        if (s1.contains(","))
          s11 = s11.replace(",", ".");
        if (s2.contains(","))
          s21 = s21.replace(",", ".");
        try
        {
          d1 = new Double(s11);
        }
        catch (NumberFormatException e)
        {
        }

        try
        {
          d2 = new Double(s21);
        }
        catch (NumberFormatException e)
        {

        }
        SystemObject so = DaVHelper.aktionListeAuswahlAq(_connection, _secId, d1, d2);
        if (_connection != null && so != null)
        {

          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaBetriebsKilometer");
          Data cd = so.getConfigurationData(atg);
          double dv = cd.getItem("Wert").asUnscaledValue().doubleValue();
          if (dv > 0.0)
            dv /= 1000.0;

          if (so.getPid().equals("sba.aq.A8.6130.HFB.W"))
          {
            dv = 208.43;          
          }

          String s = new Formatter().format("%1$7.3f", dv).toString();

          if (DaVHelper.validiereTextInput(_secId, s, textStopKmSperrung.getText(), d1, d2))
          {
            textStartKmSperrung.setText(s);
            labelStartKmSperrung.setText(so.getName());
            labelStartKmSperrung.setToolTipText(so.getName());
          }
        }
      }
    }

  }

  /**
   * 
   */
  private void aktionStopKm()
  {

    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    IViewReference ref = page.findViewReference(ZustandsDarstellungSonderProgramm.ID, _secId);
    if (ref != null)
    {
      IViewPart part = ref.getView(true);
      if (part instanceof ZustandsDarstellungSonderProgramm)
      {
        ZustandsDarstellungSonderProgramm view = (ZustandsDarstellungSonderProgramm)part;
        Double d1 = null;
        Double d2 = null;

        String s1 = view.getTextStartKm().getText().trim();
        String s2 = view.getTextStopKm().getText().trim();
        String s11 = s1.trim();
        String s21 = s2.trim();
        if (s1.contains(","))
          s11 = s11.replace(",", ".");
        if (s2.contains(","))
          s21 = s21.replace(",", ".");
        try
        {
          d1 = new Double(s11);
        }
        catch (NumberFormatException e)
        {
        }

        try
        {
          d2 = new Double(s21);
        }
        catch (NumberFormatException e)
        {

        }

        SystemObject so = DaVHelper.aktionListeAuswahlAq(_connection, _secId, d1, d2);
        if (_connection != null && so != null)
        {
          AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.sswSbaBetriebsKilometer");
          Data cd = so.getConfigurationData(atg);
          double dv = cd.getItem("Wert").asUnscaledValue().doubleValue();
          if (dv > 0.0)
            dv /= 1000.0;

          if (so.getPid().equals("sba.aq.A8.6130.HFB.W"))
          {
            dv = 208.43;           
          }

          String s = new Formatter().format("%1$7.3f", dv).toString();

          if (DaVHelper.validiereTextInput(_secId, textStartKmSperrung.getText(), s, d1, d2))
          {
            textStopKmSperrung.setText(s);
            labelStopKmSperrung.setText(so.getName());
            labelStopKmSperrung.setToolTipText(so.getName());
          }
        }
      }
    }
  }

}
