/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.sonderprogramm;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.print.attribute.standard.Fidelity;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.parts.ScrollingGraphicalViewer;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.viewers.IFontProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ITableFontProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeNode;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;

import com.bitctrl.lib.eclipse.emf.gef.commands.SetCommand;
import com.bitctrl.lib.eclipse.emf.viewers.EmfLabelProvider;

import de.bsvrz.buv.plugin.anlagendarstellung.AnlagendarstellungPackage;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.ImageFactory;
import de.bsvrz.buv.plugin.anlagendarstellung.einstellungen.SbaAnsichtProvider;
import de.bsvrz.buv.plugin.anlagendarstellung.figures.fonts.VisuAnzeigeKapsel;
import de.bsvrz.buv.plugin.anlagendarstellung.model.AnzeigeInhaltTyp;
import de.bsvrz.buv.plugin.anlagendarstellung.preferences.DefsPreference;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DaVHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.util.DoModelHelper;
import de.bsvrz.buv.plugin.anlagendarstellung.views.AuswahlSonderProgrammView;
import de.bsvrz.buv.plugin.anlagendarstellung.views.SbaScalableRootEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.editparts.DarstellungEditPartFactory;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Die Klasse realisert die Darstellung und Bearbeitung des ProgrammHierarchie-Parameters von Sonderprogrammen. 
 * Diese wird in einer Registerkarte der Sicht {@code AuswahlSonderProgrammView} verwendet.
 * 
 * @author Pittner
 * 
 */
public class ProgrammHierarchieViewParameterComposite extends Composite
{
  /**
   * Die Verbindung zum Datenverteiler
   */
  private ClientDavInterface _connection;

  /**
   * Der Baum
   */
  private Tree _tree;

  /**
   * Composites
   */
  private Composite _composite12;

  private Composite _compositeButton1;

  /**
   * Schaltflaechen
   */
  private Button _buttonHinzufuegen;

  private Button _buttonLoeschen;

  private Button _buttonHoch;

  private Button _buttonRunter;

  private Button _buttonKopieren;

  private MenuManager _kontextMenuMgr;

  private ActionRegistry _actionRegistry;

  private Button _buttonHinzufuegen2;

  private Button _buttonUmbenennen;

  private SystemObject _systemObject;

  private TreeViewer _tvFilter3;

  private Map<String, ProgrammHierarchie> _mapHierarchie = null;

  /**
   * Attributgruppe des DAV, die bercksichtigt wird
   */
  private static final String _attributgruppe = "atg.sswSbaSonderprogrammHierarchie";
  private static final String _attributgruppe2 = "atg.sswSbaSonderprogramm";

  /**
   * Aspekt des DAV, der bercksichtigt wird
   */
  private static final String _aspekt = "asp.parameterSoll";

  /**
   * Konstruktor
   * 
   * @param parent
   *          das Elternobjekt
   * @param style
   *          der Stil wie das {@code Composite} dargestellt werden soll
   * @param so
   *          die Ursacheneinheit
   */
  public ProgrammHierarchieViewParameterComposite(Composite parent, int style, SystemObject so)
  {
    super(parent, style);
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    
    this.setLayout(new FillLayout());
    this.setLayoutData(gridData3);
    
    _connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
    _systemObject = so;
    createInhaltTab(this, so);

  }

  /**
   * Initialisiert den Inhalt des {@code TabItem}
   * 
   * @param tabItemComposite
   *          das Eltern Objekt
   * @param so
   *          die Ursacheneinheit
   */
  private void createInhaltTab(Composite tabItemComposite, SystemObject so)
  {
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = true;
    gridData3.verticalAlignment = GridData.FILL;
    _composite12 = new Composite(tabItemComposite, SWT.NONE);
    _composite12.setLayout(new GridLayout(2, false));
    _composite12.setLayoutData(gridData3);

    createGroupTree();
    createCompositeButton();
        
    aktualisiereBaum(so);
  }



  /**
   * This method initializes _tree
   */
  private void createGroupTree()
  {
    Group grp = new Group(_composite12, SWT.NONE);
    grp.setText("Men");

    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessVerticalSpace = true;
    
    grp.setLayout(new GridLayout());
    grp.setLayoutData(gd);

    _tree = new Tree(grp, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
    _tree.setHeaderVisible(false);
    _tree.setLinesVisible(false);
    _tree.setLayoutData(gd);

    _tree.addListener(SWT.Selection, new Listener() {

      @Override
      public void handleEvent(Event event)
      {
        // TODO Auto-generated method stub
        Widget widget = event.item;
        if (widget instanceof TreeItem)
        {
          updateButtons();
        }
      }
    });   
    
    _tvFilter3 = new TreeViewer(_tree);
  
  _tvFilter3.setContentProvider(new ProgrammContentProvider3());    
  _tvFilter3.setLabelProvider(new ProgrammLabelProvider3());    
  _tvFilter3.addSelectionChangedListener(new ISelectionChangedListener() {
    public void selectionChanged(SelectionChangedEvent event)
    {
      
    }
  });
    
  }

  /**
   * This method initializes _compositeButton1
   * 
   */
  private void createCompositeButton()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.type = org.eclipse.swt.SWT.VERTICAL;
    rowLayout.justify = false;
    rowLayout.wrap = true;
    rowLayout.pack = false;
    rowLayout.marginTop = 5;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.END;
    gridData.grabExcessHorizontalSpace = false;
    gridData.verticalAlignment = GridData.BEGINNING;
    _compositeButton1 = new Composite(_composite12, SWT.NONE);
    _compositeButton1.setLayout(rowLayout);
    _compositeButton1.setLayoutData(gridData);
    _buttonHinzufuegen2 = new Button(_compositeButton1, SWT.NONE);
    _buttonHinzufuegen2.setText("Hierarchie hinzufgen");
    _buttonHinzufuegen2.setEnabled(true);
    _buttonHinzufuegen2.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHinzufuegenHierarchie();
      }
    });
    _buttonHinzufuegen = new Button(_compositeButton1, SWT.NONE);
    _buttonHinzufuegen.setText("Programm hinzufgen");
    _buttonHinzufuegen.setEnabled(false);
    _buttonHinzufuegen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHinzufuegenProgramm();
      }
    });
    _buttonLoeschen = new Button(_compositeButton1, SWT.NONE);
    _buttonLoeschen.setText("Lschen");
    _buttonLoeschen.setEnabled(false);
    _buttonLoeschen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionLoeschen();
      }
    });
    _buttonHoch = new Button(_compositeButton1, SWT.NONE);
    _buttonHoch.setText("Hoch");
    _buttonHoch.setEnabled(false);
    _buttonHoch.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHoch();
        updateButtons();
      }
    });
    _buttonRunter = new Button(_compositeButton1, SWT.NONE);
    _buttonRunter.setText("Runter");
    _buttonRunter.setEnabled(false);
    _buttonRunter.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionRunter();
        updateButtons();
      }
    });
    _buttonKopieren = new Button(_compositeButton1, SWT.NONE);
    _buttonKopieren.setText("Kopieren");
    _buttonKopieren.setEnabled(false);
    _buttonKopieren.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionKopieren();
      }
    });
    
    _buttonUmbenennen = new Button(_compositeButton1, SWT.NONE);
    _buttonUmbenennen.setText("Umbenennen");
    _buttonUmbenennen.setEnabled(false);
    _buttonUmbenennen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);        
        aktionUmbenennen();
      }
    });
  }
  
  /**
   * Der Label-Provider des Trees
   * 
   * @author Pittner
   * 
   */
  private class ProgrammLabelProvider3 implements ILabelProvider, IFontProvider
  {   
    
    public void addListener(ILabelProviderListener listener)
    {
    }
    
    public void dispose()
    {
    }
    
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }
    
    public void removeListener(ILabelProviderListener listener)
    {
    }

    @Override
    public Image getImage(Object element)
    {
      // TODO Auto-generated method stub
      return null;
    }

    @Override
    public String getText(Object element)
    {
      // TODO Auto-generated method stub
      if (element instanceof Programm)
      {           
        return ((Programm)element).toString();
      }
      else if (element instanceof ProgrammHierarchie)
      {
        return ((ProgrammHierarchie)element).toString();        
      }         
      return null;
    }

    @Override
    public Font getFont(Object element)
    {
      if (element instanceof ProgrammHierarchie)
      {
        FontData fontData = Display.getCurrent().getSystemFont().getFontData()[0];
        String name = fontData.getName();
        return JFaceResources.getFontRegistry().getBold(name);        
      }
      
      return null;
    }
  }
  
  
  /**
   * Der Content-Provider
   * 
   * @author Pittner
   * 
   */
  private class ProgrammContentProvider3 implements ITreeContentProvider
  {
    
    public Object[] getChildren(Object parentElement)
    {
      
      if (parentElement instanceof List)
        return ((List<?>)parentElement).toArray();
      
      if (parentElement instanceof ProgrammHierarchie)
      {
        List<Object> list = new ArrayList<Object>();
        
        list.addAll(((ProgrammHierarchie)parentElement).getProgramme());
        
        Collection<String> menues = ((ProgrammHierarchie)parentElement).getUntermenues();
        for (String name : menues)
        {
          if (_mapHierarchie.containsKey(name)){
            list.add(_mapHierarchie.get(name));
          }   
        }
        return list.toArray();
      }
      return new Object[0];
    }
    
    public Object getParent(Object element)
    {
      
      if (element instanceof Programm){                
        return ((Programm)element).getParent();
      }
      
      if (element instanceof ProgrammHierarchie){              
        String parent = ((ProgrammHierarchie)element).getParentString();      
        
        if (_mapHierarchie.containsKey(parent)){
          return _mapHierarchie.get(parent);
        }
      }      
      return null;
    }
    
    public boolean hasChildren(Object element)
    {
      if (element instanceof List)
        return ((List<?>)element).size() > 0;
      if (element instanceof ProgrammHierarchie){
        boolean b1 = ((ProgrammHierarchie)element).getProgramme().size() > 0; 
        boolean b2 = ((ProgrammHierarchie)element).getUntermenues().size() > 0;        
        return b1 | b2;
      }
      return false;
    }
    
    public Object[] getElements(Object input)
    {
      return getChildren(input);
    }
    
    public void dispose()
    {
    }
    
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }
  }

  /**
   * Aktualisert die Darstellung im {@code TreeViewer} mit dem Daten des uebergebenen {@code SystemObject}
   * 
   * @param die
   *          Ursacheneinheit
   */
  protected void aktualisiereBaum(SystemObject so)
  {
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(_attributgruppe);
    Aspect asp = _connection.getDataModel().getAspect(_aspekt);
    ResultData result = _connection.getData(so, new DataDescription(atg, asp), 1500);
 
    if (result.hasData())
    {
      if (result != null && result.hasData())
      {                
        if (_tvFilter3 != null && !_tvFilter3.getTree().isDisposed())
        {
          List<ProgrammHierarchie> list = getListeProgrammHierarchie(so);                              
          initMapHierarchie(list);          
          erzeugeUndSetInput(list);
        }        
        _tvFilter3.expandAll();
      }
    }
  }

  private void erzeugeUndSetInput(Collection<ProgrammHierarchie> list)
  {
    List<ProgrammHierarchie> input = new ArrayList<ProgrammHierarchie>();
    for (ProgrammHierarchie ph : list)
    {
      if (ph.getParentString() == null)
        input.add(ph);
    }          
    _tvFilter3.setInput(input);
  }

  private void initMapHierarchie(Collection<ProgrammHierarchie> list)
  {
    if (_mapHierarchie == null){
      _mapHierarchie = new HashMap<String, ProgrammHierarchie>();
    }          
    _mapHierarchie.clear();
    for (ProgrammHierarchie ph : list)
    {
      _mapHierarchie .put(ph.bezeichnung, ph);
    }
  }
  
  /**
   * @param s
   * @return
   */
  public TreeItem find (String s) {
        
    int itemCount = _tree.getItemCount ();
    for (int i = 0; i < itemCount; i++) {
      TreeItem item = _tree.getItem (i);
      TreeItem ti = findRecursively (item, s);
      if (ti != null) {
        return ti;
      }
    }
    
    return null;
  }
  
  /**
   * @param item
   * @param searchString
   * @return
   */
  public TreeItem findRecursively (TreeItem item, String searchString) {
    String contents = item.getText();
    if ((contents.toUpperCase ().indexOf (searchString.toUpperCase ())) != -1) {
      return item;
    }
    
    /* check child items */
    int childCount = item.getItemCount ();
    for (int i = 0; i < childCount; i++) {
      TreeItem child = item.getItem (i);
      TreeItem ti = findRecursively (child, searchString);
      if (ti != null)
        return ti;
    }

    return null;
  }
  
  /**
   * Liefert die Liste der Sonderprogramme der aktuellen Instanz der SonderprogrammView
   * 
   * @return
   */
  public List<ProgrammHierarchie> getListeProgrammHierarchie(SystemObject systemObject)
  {   
    ClientDavInterface connection = DaVKommunikationDobj.getInstanz().holeDaVVerbindung();
    
    if (connection != null)
    {
      AttributeGroup atg1 = connection.getDataModel().getAttributeGroup(_attributgruppe);
      Aspect asp = connection.getDataModel().getAspect(_aspekt);
          
      DataDescription dd = new DataDescription(atg1, asp);          
      ResultData rd = connection.getData(systemObject, dd, 1000);
      if (rd != null && rd.hasData()){
        Data data = rd.getData();   
        Data dataNeu = data.createModifiableCopy();
        List<ProgrammHierarchie> ret = new ArrayList<ProgrammHierarchie>();
        bestimmeProgrammHierarchie(connection, null, dataNeu, ret);
        return ret;       
      }
    }
    return null;
  }
  
  /**
   * @param connection
   * @param parent
   * @param data
   * @param ret
   */
  void bestimmeProgrammHierarchie(ClientDavInterface connection, ProgrammHierarchie parent, Data data, List<ProgrammHierarchie> ret){
     
    AttributeGroup atg = connection.getDataModel().getAttributeGroup(_attributgruppe2);
    Aspect asp = connection.getDataModel().getAspect(_aspekt);
    
    Array array = data.getItem("Men").asArray();             
    for (int i = 0; i < array.getLength(); i++)
    {
      String name = array.getItem(i).getItem("Name").asTextValue().getValueText();
      String[] menues = array.getItem(i).getItem("UnterMen").asTextArray().getTextArray();
      SystemObject[] sps = array.getItem(i).getItem("Sonderprogramme").asReferenceArray().getSystemObjectArray();
                    
      List<ResultData> list = new ArrayList<ResultData>();
      for (SystemObject sp : sps)
      {         
        if (sp.isValid()){
          ResultData rd2 = connection.getData(sp, new DataDescription(atg, asp), 1000);
          list.add(rd2);                                                         
        } 
      }            
      String parentString = getParentString(connection, data, name);           
      ProgrammHierarchie programmHierarchie = new ProgrammHierarchie(parentString, name, menues, list);      
      ret.add(programmHierarchie);  
    }       
  }
  
  /**
   * @param connection
   * @param data
   * @param name
   * @return
   */
  private String getParentString(ClientDavInterface connection, Data data, String name)
  {    
    
    AttributeGroup atg = connection.getDataModel().getAttributeGroup(_attributgruppe2);
    Aspect asp = connection.getDataModel().getAspect(_aspekt);
    
    Array array = data.getItem("Men").asArray();
    for (int i = 0; i < array.getLength(); i++)
    {               
      
      String parentName = array.getItem(i).getItem("Name").asTextValue().getValueText();
      String[] parentMenues = array.getItem(i).getItem("UnterMen").asTextArray().getTextArray();
      SystemObject[] sps = array.getItem(i).getItem("Sonderprogramme").asReferenceArray().getSystemObjectArray();
      
      List<ResultData> list = new ArrayList<ResultData>();
      for (SystemObject sp : sps)
      {         
        if (sp.isValid()){
          ResultData rd2 = connection.getData(sp, new DataDescription(atg, asp), 1000);
          list.add(rd2);                                                         
        } 
      }      
      for (String string : parentMenues)
      {
        if (string.equals(name)){
          return parentName;             
        }
      }
    }
    return null;
  }
  

  /**
   * Aktivieren bzw. Deaktivieren der Buttons anhanf der Selektion im Baum
   */
  private void updateButtons()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      String text = ti.getText();
      // Das Parent ist null, wenn es sich um eine ProgrammHierarchie handelt
      TreeItem parentItem = ti.getParentItem();
      if (parentItem == null)
      {
        _buttonHinzufuegen.setEnabled(true);
        _buttonLoeschen.setEnabled(true);
        _buttonHoch.setEnabled(false);
        _buttonRunter.setEnabled(false);
        _buttonKopieren.setEnabled(true);        
        _buttonUmbenennen.setEnabled(true);        
      }
      else
      {
        _buttonHinzufuegen.setEnabled(false);
        _buttonLoeschen.setEnabled(true);
        int cnt = parentItem.getItemCount();
        int cntNeu = cnt;
        if (cnt > 0)
          cntNeu -= 1;
        if (cnt == 1)
        {
          _buttonHoch.setEnabled(false);
          _buttonRunter.setEnabled(false);
        } 
        else
        {          
          _buttonHoch.setEnabled(false);
          _buttonRunter.setEnabled(false);
          
          if (!isFirstItem(ti))
            _buttonHoch.setEnabled(true);
          if (!isLastItem(ti))
            _buttonRunter.setEnabled(true);
        }

        _buttonKopieren.setEnabled(false);
        _buttonUmbenennen.setEnabled(false);        
      }
    }
    else
    {
//      _buttonHinzufuegen2.setEnabled(true);
      _buttonHinzufuegen.setEnabled(false);
      _buttonLoeschen.setEnabled(false);
      _buttonHoch.setEnabled(false);
      _buttonRunter.setEnabled(false);
      _buttonKopieren.setEnabled(false);
      _buttonUmbenennen.setEnabled(false);        
    }
  }
  
  /**
   * Erzeugt eine neue Alternative und fgt diese am Ende der Alternativen ein
   */
  protected void aktionHinzufuegenHierarchie()
  {
    InputDialog inputDialog = new InputDialog(this.getShell(), "Hierarchie anlegen", "Bitte Namen eingeben", "", null);
    int open = inputDialog.open();
    if (open == IStatus.OK){
      String nameNeu = inputDialog.getValue();      
      ProgrammHierarchie phNeu = new ProgrammHierarchie(null, nameNeu, new String[0], new ArrayList<ResultData>());                      
      
      int size = 0;
      for (Map.Entry<String, ProgrammHierarchie> me : _mapHierarchie.entrySet())
      {
        ProgrammHierarchie ph = me.getValue();            
        if (ph.getParentString() == null){
          size++;
        }
      }
      
      if (size > 0){
        size--;          
      }
      
      _tvFilter3.insert(_tvFilter3.getInput(), phNeu, size);        
      _mapHierarchie.put(phNeu.bezeichnung, phNeu);
    }
  }
  
  /**
   * Erzeugt eine neue Alternative und fgt diese am Ende der Alternativen ein
   */
  protected void aktionHinzufuegenProgramm()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem selectedItem = selection[0];
      String text = selectedItem.getText();
      {
        
        List<Object> contents = new ArrayList<Object>();
        
        final String REMOVE_RELATIV_ZU = "<Referenz lschen>";
        contents.add(0, REMOVE_RELATIV_ZU);

        SystemObjectType type = _connection.getDataModel().getType("typ.sswSbaUeSonderprogramm");
        if (type != null)
          contents.addAll(type.getElements());
        
        final ElementListSelectionDialog dialog = new ElementListSelectionDialog(
            this.getShell(), new EmfLabelProvider());
        dialog.setTitle("SystemObjekt auswhlen");
        dialog.setMessage("Bitte whlen Sie eine anderes SystemObjekt zur Auswahl.");
        dialog.setMultipleSelection(true);
        dialog.setElements(contents.toArray());

        if (text != null)
        {
          dialog.setInitialElementSelections(Collections.singletonList(text));
        }

        if (dialog.open() == Window.OK)
        {
          Object value = null;
          if (REMOVE_RELATIV_ZU.equals(dialog.getFirstResult()))
          {
            value = null;
          }
          else
          {
            AttributeGroup atg = _connection.getDataModel().getAttributeGroup(_attributgruppe);
            AttributeGroup atg2 = _connection.getDataModel().getAttributeGroup(_attributgruppe2);
            Aspect asp = _connection.getDataModel().getAspect(_aspekt);  
            TreeItem parent = selectedItem;  
            
            Object[] result = dialog.getResult();
            for (Object object : result)
            {              
              if (_connection != null)
              {
                Data data = _connection.createData(atg).createModifiableCopy();
                Array array = data.getItem("Men").asArray();
                array.setLength(1);
                Data item0 = array.getItem(0);
                Array array2 = item0.getItem("Sonderprogramme").asArray();
                int cnt = parent.getItemCount();
                
                if (cnt == 0)
                  cnt = 1;
                array2.setLength(cnt);
                if (cnt > 0)
                  cnt--;
                
                
                Data item1 = array2.getItem(cnt);
                
                if (object instanceof SystemObject)
                {
                  SystemObject so = (SystemObject)object;
                  item1.asReferenceValue().setSystemObject(so);                
                }
                
                if (_mapHierarchie.containsKey(parent.getText())){                  
                  ProgrammHierarchie ph = _mapHierarchie.get(parent.getText());
                  SystemObject so = item1.asReferenceValue().getSystemObject();
                  if (so.isValid()){
                    ResultData result2 = _connection.getData(so, new DataDescription(atg2, asp), 1000);
                    if (result2 != null){
                      Programm prg = new Programm(ph, result2);
                      ph.getProgramme().add(prg);
                      int size = ph.getProgramme().size();
                      if (size > 0)
                        size--;
                      _tvFilter3.insert(ph, prg, size);
                    }
                  }
                }else{
                  System.out.println("Kein Eintrag in Map: " + parent);
                }
              }              
            }
          }
        }                
      }
    }
  }

  /**
   * Loescht die selektierte Alternative
   */
  protected void aktionLoeschen()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      TreeItem parent = ti.getParentItem();
      
      
      Object data = ti.getData();
      if (data instanceof ProgrammHierarchie)
      {
        ProgrammHierarchie ph = (ProgrammHierarchie)data;
        if (_mapHierarchie.containsKey(ph.bezeichnung)){
          _mapHierarchie.remove(ph.bezeichnung);
        }      
      }
      if (data instanceof Programm)
      {
        Programm prg = (Programm)data;
        Object data2 = parent.getData();
        if (data2 instanceof ProgrammHierarchie)
        {
          ProgrammHierarchie ph = (ProgrammHierarchie)data2;
          Collection<Programm> prgs = ph.getProgramme();
          if (prgs.contains(prg)){
            prgs.remove(prg);
          }
        }          
      }
      
      if (ti != null && !ti.isDisposed())
        ti.dispose();      
    }
  }
  
  /**
   * Verschiebt die Alternative um eine Position nach oben
   */
  protected void aktionHoch()
  {
    // TODO Auto-generated method stub
    TreeItem ti = _tree.getSelection()[0];
    TreeItem parentItem = ti.getParentItem();
    TreeItem[] items = parentItem.getItems();
    int positionPrg = 0;
    int positionPh = 0;
    for (int i = 0; i < items.length; i++)
    {
      TreeItem treeItem = items[i];
      
      if (treeItem.equals(ti)){
        break;
      }    
      
      Object data = treeItem.getData();
      if (data instanceof Programm)
      {
        Programm prg = (Programm)data;         
        positionPrg++;
      }
      if (data instanceof ProgrammHierarchie)
      {
        ProgrammHierarchie ph = (ProgrammHierarchie)data;
        positionPh++;
      }              
    }
    
    
    ISelection selection = _tvFilter3.getSelection();
    if (selection instanceof StructuredSelection)
    {
      StructuredSelection ss = (StructuredSelection)selection;
      Object firstElement = ss.getFirstElement();
      if (firstElement instanceof ProgrammHierarchie)
      {       
        ProgrammHierarchie ph = (ProgrammHierarchie)firstElement;
        String parentString = ph.getParentString();
        ProgrammHierarchie parent = _mapHierarchie.get(parentString);
        List<String> list = new ArrayList<String>(parent.getUntermenues());  
        positionPh = list.indexOf(ph.bezeichnung);
        
        if (positionPh > 0){          
          _tvFilter3.remove(parent, positionPh);
          _tvFilter3.insert(parent, ph, positionPh-1);
          
          Collections.swap(list, positionPh-1, positionPh);                       
          parent.setUntermenues(list);
          _mapHierarchie.put(parentString, parent);
        }        
      }
      else if (firstElement instanceof Programm)
      {
        Programm prg = (Programm)firstElement;
        Object object = prg.getParent();
        if (object instanceof ProgrammHierarchie)
        {
          ProgrammHierarchie parent = (ProgrammHierarchie)object;
          if (positionPrg > 0){          
            _tvFilter3.remove(parent, positionPrg);
            _tvFilter3.insert(parent, prg, positionPrg-1);
            
            List<Programm> list = new ArrayList<Programm>(parent.getProgramme());
            Collections.swap(list, positionPrg-1, positionPrg); 
            parent.setProgramme(list);
            _mapHierarchie.put(parent.bezeichnung, parent);              
          }           
        }        
      }      
    }
  }
  
  /**
   * Verschiebt die Alternative um eine Position nach unten
   */
  protected void aktionRunter()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem ti = selection[0];
      TreeItem parent = ti.getParentItem();
      
      TreeItem[] items = null;
      if (parent != null)
       items = parent.getItems();       

      Map<Data, String> map = new HashMap<Data, String>();

      int index = -1;

      List<Data> list = new ArrayList<Data>();
      for (int i = 0; i < items.length; i++)
      {
        TreeItem item = items[i];
        Data dta = (Data)item.getData();
        String text = item.getText();
        map.put(dta, text);

        if (item.equals(ti))
        {
          index = i;
        }

        list.add(dta);
      }
      
      if (index >= 0)
        Collections.swap(list, index + 1, index);
      
     
      for (int i = 0; i < list.size(); i++)
      {
        Data dta = list.get(i);
        String text = map.get(dta);
        TreeItem item = items[i];        
        item.setData(dta);
        item.setText(text);
      }
      if (index > 0){
        _tree.redraw();
        _tree.select(parent.getItem(index + 1));
      }
    }

  }

  /**
   * Kopiert die selektierte Alternative mit ihren Daten und fgt diese am Ende der Alternativen ein
   */
  protected void aktionKopieren()
  {
    TreeItem[] selection = _tree.getSelection();
    if (selection != null && selection.length > 0)
    {
      TreeItem selectedItem = selection[0];
      String text = selectedItem.getText();

      TreeItem treeItemNeu = new TreeItem(_tree, SWT.NONE);
      treeItemNeu.setText("Kopie von " + text);
      
      
      TreeItem[] items = selectedItem.getItems();
      for (TreeItem subItem : items)
      {
        
        TreeItem subTreeItemNeu = new TreeItem(treeItemNeu, SWT.NONE);
        subTreeItemNeu.setData(subItem.getData());
        subTreeItemNeu.setText(subItem.getText());
      }
      
      _tree.setSelection(treeItemNeu);     
 
      Event event = new Event();
      event.item = treeItemNeu;
      _tree.notifyListeners(SWT.Selection, event);
    }
  }
  
  
  private boolean isFirstItem(TreeItem item){
    TreeItem parent = item.getParentItem();
    if (parent != null){
      TreeItem[] items = parent.getItems();
      for (int i = 0; i < items.length; i++)
      {
        TreeItem treeItem = items[i];
        if (i == 0 && treeItem.equals(item))
          return true;
      }
    }    
    return false;    
  }
  
  private boolean isLastItem(TreeItem item){    
    TreeItem parent = item.getParentItem();
    if (parent != null){
      TreeItem[] items = parent.getItems();
      for (int i = 0; i < items.length; i++)
      {
        TreeItem treeItem = items[i];
        if (i == (items.length-1) && treeItem.getText().equals(item.getText()))
          return true;
      }
    }    
    return false;    
  }
  

  /**
   * Holt das Baum-Objekt
   * 
   * @return
   */
  public Tree getTree()
  {
    return _tree;
  }
  
  
//  /**
//   * Liefert das KontextMenue zum Baum
//   */
//  private void erzeugeKontextMenue()
//  {  
//    _kontextMenuMgr = new MenuManager("#EinzelmassnahmenMenu");
//    erzeugeAktionen();
//    
//    addtoKontextMenue((Action)_actionRegistry.getAction("Hinzufuegen"));
//    addtoKontextMenue((Action)_actionRegistry.getAction("Loeschen"));
//    addtoKontextMenue((Action)_actionRegistry.getAction("Hoch"));
//    addtoKontextMenue((Action)_actionRegistry.getAction("Runter"));
//    addtoKontextMenue((Action)_actionRegistry.getAction("Kopieren"));
//    
//    // Men erzeugen - Viewer wird dabei dem Men bekannt gemacht
//    Menu menu = _kontextMenuMgr.createContextMenu(_tree);
//    // Das Men dem Baum bekannt machen
//    _tree.setMenu( menu );        
//  }
//  
//  /**
//   * Aktion im Kontextmen setzen
//   * 
//   * @param action  Aktion die im Kontextmen erscheinen soll
//   */
//  public void addtoKontextMenue(Action action)
//  {
//    
//    if ( action != null )
//      _kontextMenuMgr.add(action);
//  }
//  
//  /**
//   * Erzeugt die Aktionen zum Verwalten der Hierarchieobjekte
//   */
//  private void erzeugeAktionen()
//  {
//    _actionRegistry = new ActionRegistry();
//
//    Action action1 = new Action() {
//      public void run()
//      {
//        aktionHinzufuegenProgramm();
//      }
//    };
//    action1.setId("Hinzufuegen");
//    action1.setText("Hinzufgen");
//    action1.setToolTipText("Erstellt eine neue Alternative");
//    _actionRegistry.registerAction(action1);
//    
//    Action action2 = new Action() {
//      public void run()
//      {
//        aktionLoeschen();
//      }
//    };
//    action2.setId("Loeschen");
//    action2.setText("Lschen");
//    action2.setToolTipText("Lscht eine bestehende Alternative");
//    _actionRegistry.registerAction(action2);
//    
//    Action action3 = new Action() {
//      public void run()
//      {
//        aktionHoch();
//      }
//    };
//    action3.setId("Hoch");
//    action3.setText("Hoch");
//    action3.setToolTipText("Verschiebt die Alternative um eine Position nach oben");
//    _actionRegistry.registerAction(action3);
//    
//    Action action4 = new Action() {
//      public void run()
//      {
//        aktionRunter();
//      }
//    };
//    action4.setId("Runter");
//    action4.setText("Runter");
//    action4.setToolTipText("Verschiebt die Alternative um eine Position nach unten");
//    _actionRegistry.registerAction(action4);
//    
//    Action action5 = new Action() {
//      public void run()
//      {
//        aktionKopieren();
//      }
//    };
//    action5.setId("Kopieren");
//    action5.setText("Kopieren");
//    action5.setToolTipText("Kopiert die Alternative und fgt diese am Ende der Alternativen ein");
//    _actionRegistry.registerAction(action5);
//  }
//
  private void aktionUmbenennen()
  {
    if (_tree.getSelection().length > 0){
      TreeItem treeItem = _tree.getSelection()[0];
      String text = treeItem.getText();
      InputDialog inputDialog = new InputDialog(this.getShell(), "ProgrammHierarchie umbenennen", "Bitte Namen eingeben", text, null);
      int open = inputDialog.open();
      if (open == IStatus.OK){
        String nameNeu = inputDialog.getValue();          
        treeItem.setText(nameNeu);
        _tree.setSelection(treeItem);
      }          
    }
  }

  public TreeViewer getTvFilter3()
  {
    return _tvFilter3;
  }

  public Map<String, ProgrammHierarchie> getMapHierarchie()
  {
    return _mapHierarchie;
  }

}
