/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.util;

import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;

import com.bitctrl.lib.eclipse.draw2d.FixedSizeFigure;

import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.AnzeigeQuerschnittModelTypAbstract;
import de.bsvrz.buv.plugin.anlagendarstellung.StreckenStationModel;
import de.bsvrz.buv.plugin.anlagendarstellung.VerhaltenDoTyp;
import de.bsvrz.buv.plugin.darstellung.editparts.AnsichtEditPart;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.editparts.DoModelEditPart;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Helfer Klasse zum Zugriff auf den DarstellungsObjekte
 * 
 * @author Pittner
 * 
 */
public class DoModelHelper
{
  public enum ENUM_DARSTELLUNG_KONTEXT {
    IST_ZUSTAND, SOLL_ZUSTAND, HANDSCHALTUNG, SIMULATION, TRICHTER, PARAMETER
  };

  public static AnzeigeModelTypAbstract holeAnzeigeModelTypAbstractVonSystemObject(AnsichtEditPart contents,
      SystemObject objekt, ENUM_DARSTELLUNG_KONTEXT kontext)
  {
    if (contents == null || objekt == null)
      throw new IllegalArgumentException("Uebergabe-Parameter ist null: " + contents + " - " + objekt);

    Ansicht ansicht = contents.getModel();

    StilisierteDarstellung darstellung = (StilisierteDarstellung)ansicht.getDarstellung();

    EList<Ebene> ebenen = darstellung.getEbenen();
    for (Ebene ebene : ebenen)
    {

      EList<DoModel> doObjekte = ebene.getDoObjekte();
      for (DoModel dom : doObjekte)
      {
        if (dom instanceof AnzeigeQuerschnittModelTypAbstract)
        {

          AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)dom;
          int aqKontext = aq.getKontext();

          int i = getDarstellungKontext(kontext);

          if (i == aqKontext)
          {

            EList<AnzeigeModelTypAbstract> anzeigen = aq.getKomponenten();
            for (AnzeigeModelTypAbstract anzeige : anzeigen)
            {

              SystemObject so = anzeige.getSystemObject();
              if (so != null)
              {
                if (so.equals(objekt))
                {
                  return anzeige;
                }
              }
            }
          }
        }
      }

    }
    return null;
  }

  public static void holeDoppelKlickAktionAusViewerSelection(ISelection selection, Data data)
  {
//    System.out.println("DoModelHelper.holeDoppelKlickAktionAusViewerSelection(): " + data);
    
    IStructuredSelection ss = (IStructuredSelection)selection;
    if (ss != null && !ss.isEmpty())
    {
      Object obj = ss.getFirstElement();

      if (obj instanceof DoModelEditPart)
      {
        DoModelEditPart part = (DoModelEditPart)obj;
        DoModel model = (DoModel)part.getModel();
        DoTyp doTyp = model.getDoTyp();
        if (doTyp instanceof VerhaltenDoTyp)
        {
          VerhaltenDoTyp coDoTyp = (VerhaltenDoTyp)doTyp;

          int kontext = -1;

          if (model instanceof AnzeigeQuerschnittModelTypAbstract)
          {
            AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)model;
            kontext = aq.getKontext();
          }
          else if (model instanceof AnzeigeModelTypAbstract)
          {
            EObject parent = model.eContainer();
            if (parent instanceof AnzeigeQuerschnittModelTypAbstract)
            {
              AnzeigeQuerschnittModelTypAbstract aq = (AnzeigeQuerschnittModelTypAbstract)parent;
              kontext = aq.getKontext();
            }
          }
          else if (model instanceof StreckenStationModel)
          {
            StreckenStationModel sst = (StreckenStationModel)model;
            kontext = sst.getKontext();
          }

          String id = null;

          switch (kontext)
          {
            case 0:
              id = coDoTyp.getDoppelKlickIdIstZustand();
              break;
            case 1:
              id = coDoTyp.getDoppelKlickIdSollZustand();
              break;
            case 2:
              id = coDoTyp.getDoppelKlickIdHandSchaltung();
              break;
            case 3:
              id = coDoTyp.getDoppelKlickIdSimulation();
              break;
            case 4:
              id = coDoTyp.getDoppelKlickIdTrichter();
              break;
            case 5:
              id = coDoTyp.getDoppelKlickIdParameter();
              break;

            default:
              break;
          }

          if (id != null)
          {
            if (kontext != 5)
            {
              ExtensionPointHelper.leseActionsUndFuehreAus(id);
            }
            else
            {
              ExtensionPointHelper.leseActionsUndFuehreAusMitEvent(id, data);
            }
          }
        }
      }
    }
  }

  public static int getDarstellungKontext(ENUM_DARSTELLUNG_KONTEXT kontext)
  {
    int i = -1;
    if (kontext.equals(ENUM_DARSTELLUNG_KONTEXT.IST_ZUSTAND))
      i = 0;
    else if (kontext.equals(ENUM_DARSTELLUNG_KONTEXT.SOLL_ZUSTAND))
      i = 1;
    if (kontext.equals(ENUM_DARSTELLUNG_KONTEXT.HANDSCHALTUNG))
      i = 2;
    if (kontext.equals(ENUM_DARSTELLUNG_KONTEXT.SIMULATION))
      i = 3;
    if (kontext.equals(ENUM_DARSTELLUNG_KONTEXT.TRICHTER))
      i = 4;
    if (kontext.equals(ENUM_DARSTELLUNG_KONTEXT.PARAMETER))
      i = 5;
    return i;
  }

  public static AnzeigeModelTypAbstract holeAnzeigeModelTypAbstractVonMausPosition(GraphicalViewer viewer,
      DoModelEditPart part)
  {

    FixedSizeFigure figure = (FixedSizeFigure)part.getFigure();
    AnzeigeQuerschnittModelTypAbstract model = (AnzeigeQuerschnittModelTypAbstract)part.getModel();

    org.eclipse.draw2d.geometry.Point location = figure.getLocation();

    org.eclipse.draw2d.geometry.Point copy = location.getCopy();
    figure.translateToAbsolute(copy);

    Point pt = Display.getDefault().getCursorLocation();
    Point pt2 = viewer.getControl().toControl(pt);
    org.eclipse.draw2d.geometry.Point point = new org.eclipse.draw2d.geometry.Point(pt2);
    int fr = model.getFahrtrichtung();
    if (fr != 0)
    {
      point = new org.eclipse.draw2d.geometry.Point(point.x - 35, point.y);
    }

    EList<AnzeigeModelTypAbstract> komponenten = model.getKomponenten();
    for (AnzeigeModelTypAbstract az : komponenten)
    {
      org.eclipse.draw2d.geometry.Point loc = az.getLocation();

      Dimension d = null;
      if (az.getDoTyp().getName().equals("Anzeige TypA2"))
      {
        d = new Dimension(33, 33);
      }
      else if (az.getDoTyp().getName().equals("Anzeige TypB2"))
      {
        d = new Dimension(41, 33);
      }
      else if (az.getDoTyp().getName().equals("Anzeige TypC2"))
      {
        d = new Dimension(41, 17);
      }

      if (d != null)
      {
        org.eclipse.draw2d.geometry.Point tlt = new org.eclipse.draw2d.geometry.Point(copy.x + loc.x, copy.y + loc.y);
        Rectangle rect = new Rectangle(tlt, d);

        if (rect.contains(point))
        {
          return az;
        }
      }
    }
    return null;
  }
}
