/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.Vector;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.anlagendarstellung.dav.DaVKommunikationDobj;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Sicht zur Anzeige von Steuermodul-Daten.
 * 
 * @author Pittner
 * 
 */ 
public class TabelleMqView extends ViewPart
{
  public final static SimpleDateFormat TIME_FORMAT = new SimpleDateFormat("HH:mm:ss");

  public final static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("dd.MM.yy");

  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.views.tabelleMqView";

  private String _pidMq;

  public final static String UNDEF = "undefiniert";

  public final static String NO_DATA = "Keine Daten/keine Quelle";

  public final int WIDTH_FIRST_COL = 20;

  public final int WIDTH_OTHER_COL = 70;
 
  public final int COL_WIDTH_VAL = 62;

  public final int COL_WIDTH_ATT = 78;

  private DataReceiver _dataReceiver = null;

  private TableViewer _tv;

  // Attributnamen attAq[0] ... MQ, attAq[1] ... MQ Prognose, attAq[2] FS (HFS, 1FS ...)
  // Alle Arrays mssen die gleiche Lnge haben - ggfs mit null padden
  private String[][] _attAq = {
      { "Datum", "Zeit", "QKfz", "QPkw", "QLkw", "VKfz", "VPkw", "VLkw", "QB", "ALkw", "KB" },
      { "Datum", "Zeit", "QKfzP", "QPkwP", "QLkwP", "VKfzP", "VPkwP", "VLkwP", "QBP", null, null },
      { "Datum", "Zeit", "qKfz", "qPkw", "qLkw", "vPkw", "vLkw", "b", "sKfz", "aLkw", "kB" }, };

  @Override
  public void init(IViewSite site) throws PartInitException
  {
    String secondaryId = site.getSecondaryId();
    String[] components = null;

    if (secondaryId != null)
    {
      components = secondaryId.split("#");
      if (components != null && components.length > 0)
      {
        _pidMq = components[0];
      }
    }

    String partName = "MQ";
    if (_pidMq != null && !"".equals(_pidMq))
    {
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      if (connection != null)
      {
        SystemObject so = connection.getDataModel().getObject(_pidMq);
        partName += ' ' + so.getNameOrPidOrId();
      }
      else
      {
        partName += ' ' + _pidMq;
      }
    }

    setPartName(partName);

    if (components != null && components.length > 1)
    {
      String contDesc = "";
      for (int i = 1; i < components.length; i += 1)
        if (contDesc.isEmpty())
          contDesc += components[i];
        else
          contDesc += " " + components[i];
      setContentDescription(contDesc);
    }

    super.init(site);
  }

  @Override
  public void createPartControl(Composite parent)
  {

    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout());

    _tv = createTable(composite);
  }

  private TableViewer createTable(Composite parent)
  {

    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;
    gd.verticalAlignment = GridData.FILL;

    TableViewer tv = new TableViewer(parent, SWT.H_SCROLL | SWT.V_SCROLL);
    ColumnViewerToolTipSupport.enableFor(tv, ToolTip.NO_RECREATE);

    tv.setContentProvider(new ArrayContentProvider());

    Table table = tv.getTable();

    // Erste Spalte ist Dummy mit Breite 0, da die erste Spalte den
    // Style nicht korrekt verarbeitet
    TableViewerColumn tc = new TableViewerColumn(tv, SWT.RIGHT);
    tc.setLabelProvider(new TabelleMqLabelProvider(0));
    TableColumn col = tc.getColumn();
    col.setWidth(0);
    col.setResizable(false);
    col.setMoveable(false);

    tc = new TableViewerColumn(tv, SWT.RIGHT);
    tc.setLabelProvider(new TabelleMqLabelProvider(1));
    col = tc.getColumn();
    col.setWidth(COL_WIDTH_ATT);
    col.setMoveable(false);

    tc = new TableViewerColumn(tv, SWT.RIGHT);
    tc.setLabelProvider(new TabelleMqLabelProvider(2));
    col = tc.getColumn();
    col.setWidth(COL_WIDTH_VAL);
    col.setMoveable(false);
    col.setText("MQ");

    tc = new TableViewerColumn(tv, SWT.RIGHT);
    tc.setLabelProvider(new TabelleMqLabelProvider(3));
    col = tc.getColumn();
    col.setWidth(COL_WIDTH_ATT);
    col.setMoveable(false);

    tc = new TableViewerColumn(tv, SWT.RIGHT);
    tc.setLabelProvider(new TabelleMqLabelProvider(4));
    col = tc.getColumn();
    col.setWidth(COL_WIDTH_VAL);
    col.setMoveable(false);
    col.setText("Prognose");

    if (_pidMq != null && !"".equals(_pidMq))
    {
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();

      if (connection != null)
      {
        List<String> listeMq = new ArrayList<String>();
        List<String> listeFs = new ArrayList<String>();

        DataModel dataModel = connection.getDataModel();
        SystemObject mq = dataModel.getObject(_pidMq);
        listeMq.add(_pidMq);
        if (mq instanceof ConfigurationObject)
        {
          NonMutableSet set = ((ConfigurationObject)mq).getNonMutableSet("FahrStreifen");
          if (set != null)
          {
            Vector<String> listLage = new Vector<String>();
            Vector<String> listPid = new Vector<String>();

            for (SystemObject fs : set.getElements())
            {
              AttributeGroup atg = dataModel.getAttributeGroup("atg.fahrStreifen");
              Data data = fs.getConfigurationData(atg);
              if (data != null)
              {
                String lage = data.getItem("Lage").asTextValue().getText();
                if (lage != null && !"".equals(lage))
                {
                  int index = data.getItem("Lage").asUnscaledValue().intValue();
                  
                  if (lage.length() == 1 && Character.isDigit(lage.charAt(0)))
                  {
                    lage += "FS";
                  }

                  if (listLage.size() <= index)
                  {
                    listLage.setSize(index + 1);
                  }
                  if (listPid.size() <= index)
                  {
                    listPid.setSize(index + 1);
                  }
                  listLage.set(index, lage);
                  listPid.set(index, fs.getPid());
                  listeFs.add(fs.getPid());     
                }
              }
            }

            if (!listeFs.isEmpty())
            {

              int indexAbs = 5;
              int indexRel = 0;
              tc = new TableViewerColumn(tv, SWT.RIGHT);
              tc.setLabelProvider(new TabelleMqLabelProvider(indexAbs));
              col = tc.getColumn();
              col.setWidth(COL_WIDTH_ATT);
              col.setMoveable(false);

              for (int i = 0; i < listLage.size(); i += 1)
              {

                String lage = listLage.get(i);
                if (lage != null)
                {
                  tc = new TableViewerColumn(tv, SWT.RIGHT);
                  tc.setLabelProvider(new TabelleMqLabelProvider(++indexAbs));
                  col = tc.getColumn();
                  col.setWidth(COL_WIDTH_VAL);
                  col.setMoveable(false);
                  if (lage.equals("6FS"))
                  {
                    col.setText("TSS");
                  }
                  else
                  {
                    col.setText(lage);
                  }
                  String pid = listPid.get(i);
                  if (pid != null)
                  {
                    table.setData(pid, new Integer(indexRel++));
                  }
                }
              }
            }
          }
        }
        _dataReceiver = new DataReceiver(listeMq, listeFs);
      }
    }
    
    table.setLayoutData(gd);
    table.setLinesVisible(true);
    table.setHeaderVisible(true);

    int colCount = table.getColumnCount();
    int rowCount = _attAq[0].length;

    String[][] model = new String[rowCount][colCount];
    for (int row = 0; row < rowCount; row += 1)
    {
      model[row][1] = _attAq[0][row];
      model[row][3] = _attAq[1][row];
      if (colCount > 5)
        model[row][5] = _attAq[2][row];
    }

    tv.setInput(model);
    table.setData(model);

    return tv;
  }

  @Override
  public void setFocus()
  {
  }

  @Override
  public void dispose()
  {
    if (_dataReceiver != null)
    {
      _dataReceiver.dispose();
      _dataReceiver = null;
    }
    super.dispose();
  }

  private class TabelleMqLabelProvider extends CellLabelProvider
  {
    private int _spaltenIndex;

    TabelleMqLabelProvider(int spaltenIndex)
    {
      _spaltenIndex = spaltenIndex;
    }

    public String getText(Object element)
    {
      return ((String[])element)[_spaltenIndex];
    };

    @Override
    public String getToolTipText(Object element)
    {

      return ((String[])element)[_spaltenIndex];
    };

    @Override
    public boolean useNativeToolTip(Object object)
    {      
      return false;
    }

//     @Override
    public Color getBackground(Object element)
    {
      if (_spaltenIndex == 1 || _spaltenIndex == 3 || _spaltenIndex == 5)
        return ColorConstants.lightGray;
      else
        return ColorConstants.white;
    };

    public void update(ViewerCell cell)
    {
      Object element = cell.getElement();
      cell.setText(getText(element));
      cell.setBackground(getBackground(element));
    }
  }

  private class DataReceiver implements IDaVDataUpdateListener
  {

    private Set<String> _setPid = new HashSet<String>();

    public DataReceiver(List<String> davAnmeldungMq, List<String> davAnmeldungFs)
    {
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      if (connection != null)
      {
        if (davAnmeldungMq.size() > 0)
        {
          DaVKommunikationDobj.getInstanz().anmeldenAufDatenMQDaten(davAnmeldungMq, "atg.verkehrsDatenKurzZeitMq",
              "asp.analyse");

          DaVKommunikationDobj.getInstanz().anmeldenAufDatenMQDaten(davAnmeldungMq,
              "atg.verkehrsDatenKurzZeitTrendExtraPolationMq", "asp.prognoseNormal");

          _setPid.addAll(davAnmeldungMq);

          if (davAnmeldungFs.size() > 0)
          {
            DaVKommunikationDobj.getInstanz().anmeldenAufDatenMQDaten(davAnmeldungFs, "atg.verkehrsDatenKurzZeitFs",
                "asp.analyse");
            _setPid.addAll(davAnmeldungFs);
          }

          DaVDataUpdateListener.getInstanz().addDaVDataUpdateListener(this);
        }
      }
    }

    public void dispose()
    {
      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
      if (connection != null)
      {
        if (!_setPid.isEmpty())
        {
          DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
        }
      }
    }

    @Override
    public void daVDataUpdate(DaVDataUpdateEvent e)
    {

      ResultData result = e.getResult();

      Aspect asp = result.getDataDescription().getAspect();
      String pidAsp = asp.getPid();
      if (!("asp.analyse".equals(pidAsp) || "asp.prognoseNormal".equals(pidAsp)))
      {
        return;
      }

      AttributeGroup atg = result.getDataDescription().getAttributeGroup();
      String pidAtg = atg.getPid();
      if ("asp.analyse".equals(pidAsp)
          && !("atg.verkehrsDatenKurzZeitMq".equals(pidAtg) || "atg.verkehrsDatenKurzZeitFs".equals(pidAtg)))
      {
        return;
      }

      if ("asp.prognoseNormal".equals(pidAsp) && !"atg.verkehrsDatenKurzZeitTrendExtraPolationMq".equals(pidAtg))
      {
        return;
      }

      SystemObject so = result.getObject();
      if (!_setPid.contains(so.getPid()))
      {
        return;
      }

//      System.out
//          .println("Daten empfangen fr MQ " + _pidMq + "\n\t Objekt " + so.getPid() + "\n\t " + result.getData());

      int indexVal = 0;
      int indexAtt = 0;
      int indexBase = 0;
      Table table = _tv.getTable();
      String[][] model = (String[][])table.getData();
      Data data = result.getData();

      if (pidAtg.equals("atg.verkehrsDatenKurzZeitMq"))
      {
        indexAtt = 1;
        indexVal = 2;
      }
      else if (pidAtg.equals("atg.verkehrsDatenKurzZeitTrendExtraPolationMq"))
      {
        indexAtt = 3;
        indexVal = 4;
        indexBase = 1;
      }
      else if (pidAtg.equals("atg.verkehrsDatenKurzZeitFs"))
      {
        indexAtt = 5;
        indexBase = 2;
        Integer val = (Integer)table.getData(so.getPid());
        if (val != null)
          indexVal = (indexAtt + 1) + val.intValue();
      }

      if (indexVal > 0)
      {

        for (int row = 0; row < table.getItemCount(); row += 1)
        {
          if (row == 0)
          {
            // Datum
            model[row][indexVal] = DATE_FORMAT.format(new Date(result.getDataTime()));
          }
          else if (row == 1)
          {
            // Zeit
            model[row][indexVal] = TIME_FORMAT.format(new Date(result.getDataTime()));
          }
          else
          {
            String att = TabelleMqView.this._attAq[indexBase][row];
            String value = null;
            String unit = null;
            if (att != null)
            {
              if (data != null)
              {
                try
                {
                  value = data.getItem(att).getItem("Wert").asTextValue().getValueText();                   
                  unit = data.getItem(att).getItem("Wert").asTextValue().getSuffixText();
                }
                catch (RuntimeException ex)
                {
                  ex.printStackTrace(System.err);
                }
              }
              else
                value = NO_DATA;
            }
            model[row][indexVal] = value;
            if (unit != null && !unit.equals(""))
            {
              model[row][indexAtt] = TabelleMqView.this._attAq[indexBase][row] + " [" + unit + "]";
            }
          }
        }
        _tv.refresh();
      }
    }
  }
}
