/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.anlagendarstellung.views;

import org.eclipse.draw2d.FigureCanvas;
import org.eclipse.draw2d.LightweightSystem;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.parts.ScrollableThumbnail;
import org.eclipse.draw2d.parts.Thumbnail;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.anlagendarstellung.Activator;
import de.bsvrz.buv.plugin.anlagendarstellung.editor.GraphicalEditorToView;

/**
 * View-Klasse, welche die Outline-Sicht zur Zustandsdarstellung darstellt
 * 
 * @author Pittner
 * 
 */
public class ZustandsDarstellungOutlineView extends ViewPart
{
  public static String ID = "de.bsvrz.buv.plugin.anlagendarstellung.views.zustandsdarstellungOutlineView";

  /** the control of the overview */
  private FigureCanvas _overview;

  private LightweightSystem _lws;

  @Override
  public void createPartControl(Composite parent)
  {
    _overview = new FigureCanvas(parent, SWT.NONE);

    // Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_overview, Activator.PLUGIN_ID + ".outline");

    ISelectionService selectionService = getSite().getWorkbenchWindow().getSelectionService();
    selectionService.addSelectionListener(new ISelectionListener() {
      public void selectionChanged(IWorkbenchPart part, ISelection selection)
      {
        if (!(selection instanceof IStructuredSelection))
          return;

        if (part instanceof ZustandsDarstellungIstView || part instanceof ZustandsDarstellungSollView)
        {
          GraphicalEditorToView view = (GraphicalEditorToView)part;
          GraphicalViewer viewer = (GraphicalViewer)view.getAdapter(GraphicalViewer.class);
          RootEditPart rootEditPart = viewer.getRootEditPart();
          setOutlineContent(rootEditPart);
        }
      }
    });
  }

  public void setOutlineContent(RootEditPart rootEditPart)
  {
    if (!_overview.isDisposed())
    {
      if (_lws == null)
      {
        _lws = new LightweightSystem(_overview);
      }

      if (rootEditPart instanceof ScalableRootEditPart)
      {
        ScalableRootEditPart root = (ScalableRootEditPart)rootEditPart;
        Thumbnail thumbnail = new ScrollableThumbnail((Viewport)root.getFigure());
        thumbnail.setBorder(new MarginBorder(3));
        thumbnail.setSource(root.getLayer(LayerConstants.PRINTABLE_LAYERS));
        _lws.setContents(thumbnail);
      }
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.ui.part.IPage#getControl()
   */
  public Control getControl()
  {
    return _overview;
  }

  /*
   * (non-Javadoc)
   * 
   * @see org.eclipse.ui.part.IPage#setFocus()
   */
  public void setFocus()
  {
    if (getControl() != null)
      getControl().setFocus();
  }

  @Override
  public void dispose()
  {
    super.dispose();
  }
}
