/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.actions;

import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.WorkbenchException;

import de.bsvrz.buv.plugin.baueditor.sts.BauEditorPlugin;

/**
 * Abstrakte Superklasse fr Aktionen zum ffnen von Ansichten.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: AbstrakteAnsichtOeffnenAktion.java,v 1.2 2014/11/14 07:29:41 Pittner Exp $
 */
public abstract class AbstrakteAnsichtOeffnenAktion extends
		AbstrakteOeffnenAktion {

	/** Die ID dieser Aktion. */
	private final String id;

	/**
	 * Der Konstruktor.
	 * 
	 * @param text
	 *            der Text dieser Aktion
	 * @param id
	 *            die ID der zu ffnenden Ansicht
	 */
	public AbstrakteAnsichtOeffnenAktion(final String text, final String id) {
		super(text);
		this.id = id;
		setToolTipText("ffnet die Ansicht '" + text + "'");
		setId(id);
	}

	@Override
	public void runWithEvent(final Event event) {
		if (isEnabled()) {
			final IWorkbenchWindow dw = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow();
			final IWorkbenchPage page = dw.getActivePage();

			try {
				page.showView(id);
			} catch (final WorkbenchException e) {
				BauEditorPlugin.getDefault().getLogger().error(
						"Ansicht '" + getText()
								+ "' konnte nicht geffnet werden.", e);
			}
		}
	}
}
