/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.actions;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Event;
import org.eclipse.ui.IWorkbenchPart;

import de.bsvrz.buv.plugin.baueditor.sts.BauEditorPlugin;
import de.bsvrz.buv.plugin.baueditor.sts.EngstellenVerwaltungPluginFunktion;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;

/**
 * Abstrakte Aktion, welche eine vorhandenes Verkehrsmodellnetz voraussetzen.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: VerkehrsModellNetzAktion.java,v 1.2 2014/11/14 07:29:41 Pittner Exp $
 */
public abstract class VerkehrsModellNetzAktion extends
		EngstellenVerwaltungPluginAktion {

	/**
	 * Der Konstruktor.
	 * 
	 * @param name
	 *            der Name
	 * @param imageDescriptor
	 *            der ImageDescriptor
	 * @param funktion
	 *            die Funktion
	 */
	public VerkehrsModellNetzAktion(final String name,
			final ImageDescriptor imageDescriptor,
			final EngstellenVerwaltungPluginFunktion funktion) {
		super(name, imageDescriptor, funktion);
	}

	@Override
	public IStatus runWithEvent(final Event event, final Object element,
			final IWorkbenchPart targetPart) {

		if (!isEnabled()) {
			return Status.CANCEL_STATUS;
		}

		if (!BauEditorPlugin.getDefault().getEngstellenCache()
						.isInitialisiert()) {

			MessageDialog
					.openInformation(
							getShell(),
							"Information",
							"Aktion kann nicht ausgefhrt werden, da das Straennetz noch nicht initialisiert wurde. "
									+ "Um das zur Engstellenverwaltung bentigte Straennetz zu initialisieren, ffnen Sie bitte "
									+ "die Perspektive 'Engstellenverwaltung'. "
									+ "\n\nBeachten Sie, die Initialisierung des Straennetzes kann einige Minute in Anspruch nehmen.");

			return Status.CANCEL_STATUS;
		}

		if (StartupPlugin.getDefault().getDefaultNetz() == null) {

			MessageDialog
					.openInformation(
							getShell(),
							"Information",
							"Aktion kann nicht ausgefhrt werden, da kein globales Verkehrsmodellnetz gesetzt ist bzw. dieses fehlerhaft ist.");

			return Status.CANCEL_STATUS;
		}
		return Status.OK_STATUS;
	}
}
