/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.graph.figures;

import java.util.List;

import org.eclipse.core.runtime.ListenerList;
import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.ConnectionRouter;
import org.eclipse.draw2d.FanRouter;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Layer;
import org.eclipse.draw2d.LayeredPane;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.MouseEvent;
import org.eclipse.draw2d.MouseListener;
import org.eclipse.draw2d.PolygonDecoration;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.ISelectionListener;

import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.InneresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenKnoten;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.StreckenAbschnitt;

/**
 * Figure zur Darstellung von Straenknoten im Segmente Bereich des
 * Engstelleneditors.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: StrassenKnotenFigure.java,v 1.2 2014/11/14 07:29:40 Pittner Exp $
 */
public class StrassenKnotenFigure extends LayeredPane {

	/** Die Liste der Listener, welche auf nderungen der Selektionen hren. */
	private final ListenerList listeners = new ListenerList(
			ListenerList.IDENTITY);

	/**
	 * Der Konstruktor.
	 * 
	 * @param knoten
	 *            der Straenknoten
	 * @param streckenAbschnitt
	 *            der Streckenabschnitt, auf welchem sich der Straenknoten
	 *            befindet
	 */
	public StrassenKnotenFigure(final StrassenKnoten knoten,
			final StreckenAbschnitt streckenAbschnitt) {

		final Color color = ColorConstants.black;
		setBorder(new LineBorder(color, SWT.LINE_SOLID));

		final Layer figures = new Layer();
		final BorderLayout figuresLayout = new BorderLayout();
		figures.setLayoutManager(figuresLayout);

		final ConnectionLayer connections = new ConnectionLayer();
		final ConnectionRouter router = new FanRouter();
		connections.setConnectionRouter(router);

		// Erzeuge Top- und Bottom-Figure, welche die Connections der inneren
		// Straensegmente halten.
		final Figure topFigure = new Figure();
		figures.add(topFigure, BorderLayout.TOP);

		final Figure bottomFigure = new Figure();
		figures.add(bottomFigure, BorderLayout.BOTTOM);

		// Erzeuge Connections fr die inneren Straensegmente.
		final List<InneresStrassenSegment> innereStrassenSegmente = streckenAbschnitt
				.getInnereStrassenSegmente(knoten);
		for (final InneresStrassenSegment inneresSegment : innereStrassenSegmente) {

			final PolylineConnection connection = new PolylineConnection();
			final boolean alternativesISS = !streckenAbschnitt
					.getStrassenSegmente().contains(inneresSegment);
			final Color connectionColor = alternativesISS ? ColorConstants.red
					: ColorConstants.black;
			connection.addMouseListener(new MouseListener() {
				public void mouseDoubleClicked(final MouseEvent me) {
					// do nothing
				}

				public void mousePressed(final MouseEvent me) {
					// do nothing
				}

				public void mouseReleased(final MouseEvent me) {
					if (alternativesISS) {
						streckenAbschnitt
								.setInneresStrassenSegment(inneresSegment);
					}
					for (final Object listener : listeners.getListeners()) {
						if (listener != null
								&& listener instanceof ISelectionListener) {
							((ISelectionListener) listener).selectionChanged(
									null, new StructuredSelection(
											inneresSegment));
						}
					}
				}
			});
			connection.setForegroundColor(connectionColor);
			connection.setLineWidth(1);
			connection.setToolTip(TooltipUtil.erzeugeTooltip(inneresSegment
					.getPid()));
			connection.setSourceAnchor(new ChopboxAnchor(topFigure));
			connection.setTargetAnchor(new ChopboxAnchor(bottomFigure));

			final PolygonDecoration decoration = new PolygonDecoration();
			decoration.setTemplate(PolygonDecoration.TRIANGLE_TIP);
			connection.setTargetDecoration(decoration);
			connections.add(connection);
		}

		// HACK: Wenn keine Connection hinzugefgt wird, funktioniert der
		// Layout-Algorithmus beim StrassenKnotenFigureMitLabel nichts
		if (innereStrassenSegmente.isEmpty()) {
			connections.add(new PolylineConnection());
		}

		add(figures);
		add(connections);
	}

	/**
	 * Fgt dem Figure einen {@link ISelectionListener} hinzu.
	 * 
	 * @param listener
	 *            der {@link ISelectionListener}
	 */
	public void addSelectionListener(final ISelectionListener listener) {
		listeners.add(listener);
	}

	/**
	 * Entfernt dem Figure einen {@link ISelectionListener}.
	 * 
	 * @param listener
	 *            der {@link ISelectionListener}
	 */
	public void removeSelectionListener(final ISelectionListener listener) {
		listeners.remove(listener);
	}
}
