/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.bitctrl.util.monitor.IMonitor;
import com.bitctrl.util.monitor.SubMonitor;

import de.bsvrz.buv.plugin.baueditor.sts.BauEditorException;
import de.bsvrz.buv.plugin.baueditor.sts.BauEditorPlugin;
import de.bsvrz.buv.plugin.baueditor.sts.Konstanten;
import de.bsvrz.buv.plugin.baueditor.sts.util.listener.EngstellenChangedListener;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.BaustellenWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.EngstellenWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.SituationsWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.StauWrapper;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.UnfallWrapper;
import de.bsvrz.buv.plugin.ereigniskal.EreignisKalenderListener;
import de.bsvrz.buv.plugin.ereigniskal.EreignisKalenderVerwaltung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.DynamicObjectType.NameChangeListener;
import de.bsvrz.dav.daf.main.config.InvalidationListener;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.bitctrl.modell.AenderbareMenge;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateListener;
import de.bsvrz.sys.funclib.bitctrl.modell.MengenEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.MengenListener;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.att.Feld;
import de.bsvrz.sys.funclib.bitctrl.modell.att.Zeitstempel;
import de.bsvrz.sys.funclib.bitctrl.modell.tmereigniskalenderglobal.attribute.AtlZusaetzlicheAttribute;
import de.bsvrz.sys.funclib.bitctrl.modell.tmereigniskalenderglobal.konfigurationsdaten.KdEreignisEigenschaften;
import de.bsvrz.sys.funclib.bitctrl.modell.tmereigniskalenderglobal.konfigurationsdaten.KdEreignisEigenschaften.Daten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmereigniskalenderglobal.objekte.Ereignis;
import de.bsvrz.sys.funclib.bitctrl.modell.tmereigniskalenderglobal.objekte.EreignisTyp;
import de.bsvrz.sys.funclib.bitctrl.modell.tmtmcglobal.attribute.AttTmcRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Baustelle;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Situation;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Stau;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Strasse;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Unfall;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.VerkehrsModellNetz;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.onlinedaten.OdStauVerlauf;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.AbstractCache;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.StreckenAbschnittCache;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * <p>
 * Diese Hilfsklasse cacht smtliche fr dieses Plugin bentigten Informationen
 * bzgl. Baustellen und Unfllen.
 * </p>
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: EngstellenCache.java,v 1.2 2014/11/14 07:29:43 Pittner Exp $
 */
public class EngstellenCache extends AbstractCache implements MengenListener,
		NameChangeListener, DatensatzUpdateListener, InvalidationListener {

	/**
	 * Lock-Object.
	 */
	private static Object lock = new Object();

	/**
	 * Der erweiterte Netz-Cache, von welchem dieser Cache abhngig ist.
	 */
	private StreckenAbschnittCache streckenAbschnittCache;

	/**
	 * Flag, welches angibt, ob auf DatensatzUpdateListener reagiert werden
	 * soll.
	 */
	private boolean listenerAktiv = true;

	/**
	 * Liste von Listener, welche auf nderungen der Engstellen warten.
	 */
	private List<EngstellenChangedListener> listeners;

	/**
	 * Map, welche den entsprechenden BaustellenWrapper zu Baustellen speichert.
	 */
	private Map<Baustelle, BaustellenWrapper> baustellenWrapperMap;

	/**
	 * Map, welche den entsprechenden BaustellenWrapper zu Baustellen speichert.
	 */
	private Map<Unfall, UnfallWrapper> unfallWrapperMap;

	/**
	 * Map, welche den entsprechenden StauWrapper zu Baustellen speichert.
	 */
	private Map<Stau, StauWrapper> stauWrapperMap;

	/**
	 * Map, welche zu einer (positiv- bzw. negativ gerichteteten) Strae eine
	 * Liste aller auf der Strae liegenden Staus speichert.
	 */
	private Map<Strasse, List<StauWrapper>> stausAufStrasseMap;

	/**
	 * Map, welche zu einer (positiv- bzw. negativ gerichteteten) Strae eine
	 * Liste aller auf der Strae liegenden Baustellen speichert.
	 */
	private Map<Strasse, List<BaustellenWrapper>> baustellenAufStrasseMap;

	/**
	 * Map, welche zu einer Strae eine Liste aller auf der Strae liegenden
	 * Unflle speichert.
	 */
	private Map<Strasse, List<UnfallWrapper>> unfaelleAufStrasseMap;

	/**
	 * Map, welche zu einer (positiv- bzw. negativ gerichteteten) Strae eine
	 * Liste aller auf der Strae liegenden Baustellen speichert.
	 */
	private Map<AttTmcRichtung, Map<Strasse, List<BaustellenWrapper>>> baustellenAufStrassenRichtungMap;

	/**
	 * Map, welche zu einer (positiv- bzw. negativ gerichteteten) Strae eine
	 * Liste aller auf der Strae liegenden Unflle speichert.
	 */
	private Map<AttTmcRichtung, Map<Strasse, List<UnfallWrapper>>> unfaelleAufStrassenRichtungMap;

	/**
	 * Map, welche zu einer (positiv- bzw. negativ gerichteteten) Strae eine
	 * Liste aller auf der Strae liegenden Staus speichert.
	 */
	private Map<AttTmcRichtung, Map<Strasse, List<StauWrapper>>> stausAufStrassenRichtungMap;

	/**
	 * Map, welcher zu einer Engstelle das dazugehrige Ereignis speichert.
	 */
	private Map<Situation, Ereignis> ereignisseMap;

	/**
	 * ThreadPool, der die Threads zur Bearbeitung der Ergebnisse der
	 * Parameteranmeldung bereit stellt.
	 */
	private final ExecutorService threadPool = Executors.newFixedThreadPool(30);

	/**
	 * Listener, welcher ber nderungen im Ereigniskalender informiert wird.
	 * Abhngig von entfernten oder neu hinzugefgten Ereignissen wird die Map
	 * mit den gespeicherten Ereignissen aktualisiert.
	 */
	private final EreignisKalenderListener ereignisKalenderListener = new EreignisKalenderListener() {

		@Override
		public void ereignisseEntfernt(
				final Collection<Ereignis> entfernteEreignisse) {

			if (entfernteEreignisse != null && !entfernteEreignisse.isEmpty()) {
				new Thread(new EreignisseEntferntRunner(entfernteEreignisse))
						.start();
			}
		}

		class EreignisseEntferntRunner implements Runnable {

			private final Collection<Ereignis> entfernteEreignisse;

			public EreignisseEntferntRunner(
					final Collection<Ereignis> entfernteEreignisse) {
				this.entfernteEreignisse = entfernteEreignisse;
			}

			@Override
			public void run() {
				synchronized (lock) {

					final ObjektFactory objektFactory = BauEditorPlugin
							.getDefault().getObjektFactory();
					for (final Ereignis ereignis : entfernteEreignisse) {
						final KdEreignisEigenschaften kdEreignisEigenschaften = ereignis
								.getKdEreignisEigenschaften();
						if (kdEreignisEigenschaften == null) {
							continue;
						}
						final Daten datum = kdEreignisEigenschaften.getDatum();
						if (datum == null) {
							continue;
						}
						final Feld<AtlZusaetzlicheAttribute> zusaetzlicheAttribute = datum
								.getZusaetzlicheAttribute();
						if (zusaetzlicheAttribute == null
								|| zusaetzlicheAttribute.isEmpty()) {
							continue;
						}
						for (final AtlZusaetzlicheAttribute attribut : zusaetzlicheAttribute) {
							if (!Konstanten.EREIGNIS_EIGENSCHAFTEN_REFERENZ_AUSLOESENDE_SITUATION
									.equals(attribut.getAttributname())) {
								continue;
							}
							final String engstellenPid = attribut
									.getAttributwert();
							SystemObjekt systemObjekt = null;
							if (engstellenPid != null && objektFactory != null
									&& objektFactory.getDav() != null
									&& objektFactory.isVerbunden()) {
								systemObjekt = objektFactory
										.getModellobjekt(engstellenPid);
							}
							if (systemObjekt != null
									&& systemObjekt instanceof Situation) {
								ereignisseMap.remove(systemObjekt);

								SituationsWrapper wrapper = null;
								if (systemObjekt instanceof Baustelle) {
									wrapper = getBaustellenWrapper((Baustelle) systemObjekt);
								} else if (systemObjekt instanceof Unfall) {
									wrapper = getUnfallWrapper((Unfall) systemObjekt);
								} else if (systemObjekt instanceof Stau) {
									wrapper = getStauWrapper((Stau) systemObjekt);
								}
								if (wrapper != null) {
									wrapper.setEreignis(null);
								}
							}
						}
					}
				}
			}
		}

		@Override
		public void ereignisseAngelegt(
				final Collection<Ereignis> hinzugefuegteEreignisse) {

			if (hinzugefuegteEreignisse != null
					&& !hinzugefuegteEreignisse.isEmpty()) {
				new Thread(new EreignisseHinzugefuegtRunner(
						hinzugefuegteEreignisse)).start();
			}
		}

		class EreignisseHinzugefuegtRunner implements Runnable {

			private final Collection<Ereignis> hinzugefuegteEreignisse;

			public EreignisseHinzugefuegtRunner(
					final Collection<Ereignis> hinzugefuegteEreignisse) {
				this.hinzugefuegteEreignisse = hinzugefuegteEreignisse;
			}

			@Override
			public void run() {
				synchronized (lock) {

					final ObjektFactory objektFactory = BauEditorPlugin
							.getDefault().getObjektFactory();
					for (final Ereignis ereignis : hinzugefuegteEreignisse) {
						final Feld<AtlZusaetzlicheAttribute> zusaetzlicheAttribute = ereignis
								.getKdEreignisEigenschaften().getDatum()
								.getZusaetzlicheAttribute();
						if (zusaetzlicheAttribute == null
								|| zusaetzlicheAttribute.isEmpty()) {
							continue;
						}
						for (final AtlZusaetzlicheAttribute attribut : zusaetzlicheAttribute) {
							if (!Konstanten.EREIGNIS_EIGENSCHAFTEN_REFERENZ_AUSLOESENDE_SITUATION
									.equals(attribut.getAttributname())) {
								continue;
							}
							final String engstellenPid = attribut
									.getAttributwert();
							SystemObjekt systemObjekt = null;
							if (engstellenPid != null && objektFactory != null
									&& objektFactory.isVerbunden()) {
								systemObjekt = objektFactory
										.getModellobjekt(engstellenPid);
							}
							if (systemObjekt != null
									&& systemObjekt instanceof Situation) {
								ereignisseMap.put((Situation) systemObjekt,
										ereignis);

								SituationsWrapper wrapper = null;
								if (systemObjekt instanceof Baustelle) {
									wrapper = getBaustellenWrapper((Baustelle) systemObjekt);
								} else if (systemObjekt instanceof Unfall) {
									wrapper = getUnfallWrapper((Unfall) systemObjekt);
								} else if (systemObjekt instanceof Stau) {
									wrapper = getStauWrapper((Stau) systemObjekt);
								}
								if (wrapper != null) {
									wrapper.setEreignis(ereignis);
								}
							}
						}
					}
				}
			}
		}

		@Override
		public void ereignisTypenEntfernt(final Collection<EreignisTyp> liste) {
			// tut nix
		}

		@Override
		public void ereignisTypenAngelegt(final Collection<EreignisTyp> liste) {
			// tut nix
		}

		@Override
		public void ereignisTypAktualisiert(final EreignisTyp typ) {
			// tut nix
		}
	};

	/**
	 * Der Konstruktor.
	 * 
	 * @param streckenAbschnittCache
	 *            der Streckenabschnitt-Cache, von welchem dieser Cache abhngig
	 *            ist
	 */
	public EngstellenCache(final StreckenAbschnittCache streckenAbschnittCache) {
		super(streckenAbschnittCache);
	}

	/**
	 * Initialisiert den Cache.
	 * 
	 * @param cache
	 *            der Streckenabschnitt-Cache, von welchem dieser Cache abhngig
	 *            ist
	 */
	protected void preInit(final StreckenAbschnittCache cache) {
		streckenAbschnittCache = cache;
	}

	/**
	 * Liefert den Streckenabschnitt-Cache zurck, von welchem dieser Cache
	 * abhngig ist.
	 * 
	 * @return den Streckenabschnitt-Cache, von welchem dieser Cache abhngig
	 *         ist
	 */
	protected StreckenAbschnittCache getStreckenAbschnittCache() {
		return streckenAbschnittCache;
	}

	/**
	 * Fgt eine EngstellenListener dem EngstellenCache hinzu.
	 * 
	 * @param listener
	 *            der EngstellenListener, nicht <code>null</code>
	 */
	public void addEngstellenListener(final EngstellenChangedListener listener) {
		if (listener == null) {
			throw new IllegalArgumentException(
					"bergebener Listener darf nicht null sein.");
		}
		if (listeners == null) {
			listeners = new ArrayList<EngstellenChangedListener>();
		}
		if (!listeners.contains(listener)) {
			listeners.add(listener);
		}
	}

	/**
	 * Entfernt den gegebenen EngstellenListener vom EngstellenCache.
	 * 
	 * @param listener
	 *            der EngstellenListener, nicht <code>null</code>
	 */
	public void removeEngstellenListener(
			final EngstellenChangedListener listener) {
		if (listeners != null) {
			listeners.remove(listener);
			if (listeners.isEmpty()) {
				listeners = null;
			}
		}
	}

	@Override
	protected boolean doInit(final IMonitor monitor) {
		final SubMonitor subMonitor = SubMonitor.convert(monitor);

		subMonitor.beginTask("Lade Engstellen", 200000);

		if (ermittelEreignisse(subMonitor.newChild(50000))) {
			if (erzeugeBaustellenWrapper(subMonitor.newChild(50000))) {
				if (erzeugeUnfallWrapper(subMonitor.newChild(50000))) {
					if (erzeugeStauWrapper(subMonitor.newChild(50000))) {
						if (ordneBaustellen(subMonitor.newChild(25000))) {
							if (ordneUnfaelle(subMonitor.newChild(25000))) {
								if (ordneStaus(subMonitor.newChild(25000))) {
									registerListener();
									return true;
								}
							}
						}
					}
				}
			}
		}
		return false;
	}

	/**
	 * Registriert die NameChangeListener fr die Typen Baustelle und Unfall.
	 */
	private void registerListener() {

		final VerkehrsModellNetz netz = streckenAbschnittCache.getNetz();
		if (netz == null) {
			return;
		}

		final ObjektFactory objektFactory = BauEditorPlugin.getDefault()
				.getObjektFactory();

		if (objektFactory != null && objektFactory.getDav() != null
				&& objektFactory.isVerbunden()) {

			final ClientDavInterface dav = objektFactory.getDav();
			final DataModel dataModel = dav.getDataModel();

			// Baustelle
			final SystemObjectType baustellenType = dataModel
					.getType(Baustelle.PID);
			if (baustellenType != null
					&& baustellenType instanceof DynamicObjectType) {
				((DynamicObjectType) baustellenType)
						.addNameChangeListener(this);
			}
			// Unfall
			final SystemObjectType unfallType = dataModel.getType(Unfall.PID);
			if (unfallType != null && unfallType instanceof DynamicObjectType) {
				((DynamicObjectType) unfallType).addNameChangeListener(this);
			}
		}

		// Fge UpdateListener hinzu
		listenerAktiv = false;
		final AenderbareMenge<Baustelle> baustellen = netz.getBaustellen();
		if (baustellen != null) {
			baustellen.addMengenListener(this);
			for (final Baustelle baustelle : baustellen) {
				addUpdateListener(baustelle);
				addInvalidationListener(baustelle);
			}
		}
		final AenderbareMenge<Unfall> unfaelle = netz.getUnfaelle();
		if (unfaelle != null) {
			unfaelle.addMengenListener(this);
			for (final Unfall unfall : unfaelle) {
				addUpdateListener(unfall);
				addInvalidationListener(unfall);
			}
		}
		final AenderbareMenge<Stau> staus = netz.getStaus();
		if (staus != null) {
			staus.addMengenListener(this);
			for (final Stau stau : staus) {
				addUpdateListener(stau);
				addInvalidationListener(stau);
			}
		}
		listenerAktiv = true;

		// Fge EreignisKalenderListener hinzu
		EreignisKalenderVerwaltung.getInstanz().addEreignisKalenderListener(
				ereignisKalenderListener);
	}

	/**
	 * Erzeugt fr smtliche Baustellen einen entsprechenden Wrapper.
	 * 
	 * @param monitor
	 *            der Monitor
	 * 
	 * @return <code>true</code>, wenn diese Methode komplett ausgefhrt wurde,
	 *         ansonsten bei vorzeitigem Abbruch <code>false</code>
	 */
	private boolean erzeugeBaustellenWrapper(final IMonitor monitor) {

		synchronized (lock) {

			if (baustellenWrapperMap == null) {
				baustellenWrapperMap = new HashMap<Baustelle, BaustellenWrapper>();

				final int totalWork = 100000;
				final String taskName = "Lade Baustellen";
				monitor.setTaskName(taskName);
				monitor.beginTask(taskName, totalWork);

				final AenderbareMenge<Baustelle> baustellen = streckenAbschnittCache
						.getNetz().getBaustellen();
				final int anzahlBaustellen = baustellen.size();
				if (anzahlBaustellen == 0) {
					return true;
				}
				final int work = totalWork / anzahlBaustellen;

				int i = 0;
				for (final Baustelle baustelle : baustellen) {
					try {
						monitor.subTask(new StringBuffer(
								"Initialisiere Baustelle ").append(i)
								.append(" von ").append(anzahlBaustellen)
								.toString());

						final BaustellenWrapper wrapper = erzeugeBaustellenWrapper(baustelle);
						if (wrapper != null) {
							baustellenWrapperMap.put(baustelle, wrapper);
						}

					} catch (final RuntimeException e) {
						if (baustellenWrapperMap.get(baustelle) != null) {
							baustellenWrapperMap.remove(baustelle);
						}
						Debug.getLogger().warning(
								"Es gabe Probleme beim Laden einer Baustelle.",
								e);
					} finally {
						monitor.worked(work);
						if (monitor.isCanceled()) {
							monitor.done();
							return false;
						}
						i++;
					}
				}
			}
		}
		return true;
	}

	/**
	 * Erzeugt den BaustellenWrapper zu einer gegebenen Baustelle.
	 * 
	 * <p>
	 * Diese Methode berprft ob fr smtliche Parameterdatenstze Nutzdaten
	 * vorliegen. Zudem wird berprft, ob es sich um eine gltige (valide)
	 * Baustelle handelt.
	 * </p>
	 * 
	 * @param baustelle
	 *            die Baustelle
	 * 
	 * @return der BaustellenWrapper, oder <code>null</code>, wenn die Baustelle
	 *         ungltig (invalid) ist oder fr einen Parameterdatensatz keine
	 *         Nutzdaten vorliegen
	 */
	private BaustellenWrapper erzeugeBaustellenWrapper(final Baustelle baustelle) {

		// berprfe Gltigkeit
		if (!baustelle.getSystemObject().isValid()) {
			BauEditorPlugin
					.getDefault()
					.getLogger()
					.warning(
							"Ungltige (Invalide) Baustelle in Menge: "
									+ baustelle.getPid());
			return null;
		}

		// berprfe Parameterdatensatz 'PdBaustellenEigenschaften'
		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdBaustellenEigenschaften.Daten pdEigenschaften = baustelle
				.getPdBaustellenEigenschaften().getDatum();
		if (pdEigenschaften == null
				|| !pdEigenschaften.dContainsDaten()
				|| pdEigenschaften.dGetDatenStatus() != OnlineDatum.Status.DATEN) {
			BauEditorPlugin
					.getDefault()
					.getLogger()
					.warning(
							"Parameterdatensatz 'PdBaustellenEigenschaften' enthielt keine Nutzdaten bei Baustelle: "
									+ baustelle.getPid());
			return null;
		}

		// berprfe Parameterdatensatz 'PdBaustellenVerantwortlicher'
		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdBaustellenVerantwortlicher.Daten pdVerantwortlicher = baustelle
				.getPdBaustellenVerantwortlicher().getDatum();
		if (pdVerantwortlicher == null
				|| !pdVerantwortlicher.dContainsDaten()
				|| pdVerantwortlicher.dGetDatenStatus() != OnlineDatum.Status.DATEN) {
			BauEditorPlugin
					.getDefault()
					.getLogger()
					.warning(
							"Parameterdatensatz 'PdBaustellenVerantwortlicher' enthielt keine Nutzdaten bei Baustelle: "
									+ baustelle.getPid());
			return null;
		}

		// berprfe Parameterdatensatz 'PdBaustellenEigenschaftenErweitert'
		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdBaustellenEigenschaftenErweitert.Daten pdEigenschaftenErweitert = baustelle
				.getPdBaustellenEigenschaftenErweitert().getDatum();
		if (pdEigenschaftenErweitert == null
				|| !pdEigenschaftenErweitert.dContainsDaten()
				|| pdEigenschaftenErweitert.dGetDatenStatus() != OnlineDatum.Status.DATEN) {
			BauEditorPlugin
					.getDefault()
					.getLogger()
					.warning(
							"Parameterdatensatz 'PdBaustellenEigenschaftenErweitert' enthielt keine Nutzdaten bei Baustelle: "
									+ baustelle.getPid());
			return null;
		}

		// berprfe Parameterdatensatz 'PdSituationsEigenschaften'
		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdSituationsEigenschaften.Daten pdSituationsEigenschaften = baustelle
				.getPdSituationsEigenschaften().getDatum();
		if (pdSituationsEigenschaften == null
				|| !pdSituationsEigenschaften.dContainsDaten()
				|| pdSituationsEigenschaften.dGetDatenStatus() != OnlineDatum.Status.DATEN) {
			BauEditorPlugin
					.getDefault()
					.getLogger()
					.warning(
							"Parameterdatensatz 'PdSituationsEigenschaften' enthielt keine Nutzdaten bei Baustelle: "
									+ baustelle.getPid());
			return null;
		}

		try {
			return new BaustellenWrapper(baustelle);
		} catch (final BauEditorException e) {
			return null;
		}
	}

	/**
	 * Erzeugt fr smtliche Unflle einen entsprechenden Wrapper.
	 * 
	 * @param monitor
	 *            der Monitor
	 * 
	 * @return <code>true</code>, wenn diese Methode komplett ausgefhrt wurde,
	 *         ansonsten bei vorzeitigem Abbruch <code>false</code>
	 */
	private boolean erzeugeUnfallWrapper(final IMonitor monitor) {
		synchronized (lock) {

			if (unfallWrapperMap == null) {
				unfallWrapperMap = new HashMap<Unfall, UnfallWrapper>();

				final int totalWork = 100000;
				final String taskName = "Lade Unflle";
				monitor.setTaskName(taskName);
				monitor.beginTask(taskName, totalWork);

				final AenderbareMenge<Unfall> unfaelle = streckenAbschnittCache
						.getNetz().getUnfaelle();
				final int anzahlUnfaelle = unfaelle.size();
				if (anzahlUnfaelle == 0) {
					return true;
				}
				final int work = totalWork / anzahlUnfaelle;

				int i = 0;
				for (final Unfall unfall : unfaelle) {
					try {
						monitor.subTask(new StringBuffer(
								"Initialisiere Unfall ").append(i)
								.append(" von ").append(anzahlUnfaelle)
								.toString());

						final UnfallWrapper wrapper = erzeugeUnfallWrapper(unfall);
						if (wrapper != null) {
							unfallWrapperMap.put(unfall, wrapper);
						}

					} catch (final RuntimeException e) {
						if (unfallWrapperMap.get(unfall) != null) {
							unfallWrapperMap.remove(unfall);
						}
						Debug.getLogger()
								.warning(
										"Es gabe Probleme beim Laden eines Unfalls.",
										e);
					} finally {
						monitor.worked(work);
						if (monitor.isCanceled()) {
							monitor.done();
							return false;
						}
						i++;
					}
				}
			}
		}
		return true;
	}

	/**
	 * Erzeugt fr smtliche Staus einen entsprechenden Wrapper.
	 * 
	 * @param monitor
	 *            der Monitor
	 * 
	 * @return <code>true</code>, wenn diese Methode komplett ausgefhrt wurde,
	 *         ansonsten bei vorzeitigem Abbruch <code>false</code>
	 */
	private boolean erzeugeStauWrapper(final IMonitor monitor) {
		synchronized (lock) {

			if (stauWrapperMap == null) {
				stauWrapperMap = new HashMap<Stau, StauWrapper>();

				final int totalWork = 100000;
				final String taskName = "Lade Staus";
				monitor.setTaskName(taskName);
				monitor.beginTask(taskName, totalWork);

				final AenderbareMenge<Stau> staus = streckenAbschnittCache
						.getNetz().getStaus();
				final int anzahlStaus = staus.size();
				if (anzahlStaus == 0) {
					return true;
				}
				final int work = totalWork / anzahlStaus;

				int i = 0;
				for (final Stau stau : staus) {
					try {
						monitor.subTask(new StringBuffer("Initialisiere Stau")
								.append(i).append(" von ").append(anzahlStaus)
								.toString());

						final StauWrapper wrapper = erzeugeStauWrapper(stau);
						if (wrapper != null) {
							stauWrapperMap.put(stau, wrapper);
						}

					} catch (final RuntimeException e) {
						if (stauWrapperMap.get(stau) != null) {
							stauWrapperMap.remove(stau);
						}
						Debug.getLogger().warning(
								"Es gabe Probleme beim Laden eines Staus.", e);
					} finally {
						monitor.worked(work);
						if (monitor.isCanceled()) {
							monitor.done();
							return false;
						}
						i++;
					}
				}
			}
		}
		return true;
	}

	/**
	 * Erzeugt den UnfallWrapper zu einem gegebenen Unfall.
	 * 
	 * <p>
	 * Diese Methode berprft ob fr smtliche Parameterdatenstze Nutzdaten
	 * vorliegen. Zudem wird berprft, ob es sich um einen gltigen (valide)
	 * Unfall handelt.
	 * </p>
	 * 
	 * @param unfall
	 *            der Unfall
	 * 
	 * @return der UnfallWrapper, oder <code>null</code>, wenn der Unfall
	 *         ungltig (invalid) ist oder fr einen Parameterdatensatz keine
	 *         Nutzdaten vorliegen
	 */
	private UnfallWrapper erzeugeUnfallWrapper(final Unfall unfall) {

		// berprfe Gltigkeit
		if (!unfall.getSystemObject().isValid()) {
			BauEditorPlugin
					.getDefault()
					.getLogger()
					.warning(
							"Ungltiger (Invalider) Unfall in der Menge: "
									+ unfall.getPid());
			return null;
		}

		// berprfe Parameterdatensatz 'PdUnfallEigenschaften'
		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdUnfallEigenschaften.Daten pdEigenschaften = unfall
				.getPdUnfallEigenschaften().getDatum();
		if (pdEigenschaften == null
				|| !pdEigenschaften.dContainsDaten()
				|| pdEigenschaften.dGetDatenStatus() != OnlineDatum.Status.DATEN) {
			BauEditorPlugin
					.getDefault()
					.getLogger()
					.warning(
							"Parameterdatensatz 'PdUnfallEigenschaften' enthielt keine Nutzdaten bei Unfall: "
									+ unfall.getPid());
			return null;
		}

		// berprfe Parameterdatensatz 'PdUnfallEigenschaftenErweitert'
		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdUnfallEigenschaftenErweitert.Daten pdEigenschaftenErweitert = unfall
				.getPdUnfallEigenschaftenErweitert().getDatum();
		if (pdEigenschaftenErweitert == null
				|| !pdEigenschaftenErweitert.dContainsDaten()
				|| pdEigenschaftenErweitert.dGetDatenStatus() != OnlineDatum.Status.DATEN) {
			BauEditorPlugin
					.getDefault()
					.getLogger()
					.warning(
							"Parameterdatensatz 'PdUnfallEigenschaftenErweitert' enthielt keine Nutzdaten bei Unfall: "
									+ unfall.getPid());
			return null;
		}

		// berprfe Parameterdatensatz 'PdSituationsEigenschaften'
		// final
		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdSituationsEigenschaften.Daten pdSituationsEigenschaften = unfall
				.getPdSituationsEigenschaften().getDatum();
		if (pdSituationsEigenschaften == null
				|| !pdSituationsEigenschaften.dContainsDaten()
				|| pdSituationsEigenschaften.dGetDatenStatus() != OnlineDatum.Status.DATEN) {
			BauEditorPlugin
					.getDefault()
					.getLogger()
					.warning(
							"Parameterdatensatz 'PdSituationsEigenschaften' enthielt keine Nutzdaten bei Unfall: "
									+ unfall.getPid());
			return null;
		}

		try {
			return new UnfallWrapper(unfall);
		} catch (final BauEditorException e) {
			return null;
		}
	}

	/**
	 * Erzeugt den StauWrapper zu einem gegebenen Stau.
	 * 
	 * <p>
	 * Diese Methode berprft ob fr smtliche Parameterdatenstze Nutzdaten
	 * vorliegen. Zudem wird berprft, ob es sich um einen gltigen (valide)
	 * Stau handelt.
	 * </p>
	 * 
	 * @param stau
	 *            der Stau
	 * 
	 * @return der StauWrapper, oder <code>null</code>, wenn der Stau ungltig
	 *         (invalid) ist oder fr einen Parameterdatensatz keine Nutzdaten
	 *         vorliegen
	 */
	private StauWrapper erzeugeStauWrapper(final Stau stau) {

		// berprfe Gltigkeit
		if (!stau.getSystemObject().isValid()) {
			BauEditorPlugin
					.getDefault()
					.getLogger()
					.warning(
							"Ungltiger (Invalider) Stau in der Menge: "
									+ stau.getPid());
			return null;
		}

		// berprfe Parameterdatensatz 'PdSituationsEigenschaften'
		// final
		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdSituationsEigenschaften.Daten pdSituationsEigenschaften = stau
				.getPdSituationsEigenschaften().getDatum();
		if (pdSituationsEigenschaften == null
				|| !pdSituationsEigenschaften.dContainsDaten()
				|| pdSituationsEigenschaften.dGetDatenStatus() != OnlineDatum.Status.DATEN) {
			BauEditorPlugin
					.getDefault()
					.getLogger()
					.warning(
							"Parameterdatensatz 'PdSituationsEigenschaften' enthielt keine Nutzdaten bei Stau: "
									+ stau.getPid());
			return null;
		}

		try {
			return new StauWrapper(stau);
		} catch (final BauEditorException e) {
			return null;
		}
	}

	/**
	 * Ordnet die BaustellenWrapper den Straen und Richtungen zu.
	 * 
	 * @param monitor
	 *            der Monitor
	 * 
	 * @return <code>true</code>, wenn diese Methode komplett ausgefhrt wurde,
	 *         ansonsten bei vorzeitigem Abbruch <code>false</code>
	 */
	private boolean ordneBaustellen(final IMonitor monitor) {
		synchronized (lock) {

			baustellenAufStrasseMap = new HashMap<Strasse, List<BaustellenWrapper>>();
			baustellenAufStrassenRichtungMap = new HashMap<AttTmcRichtung, Map<Strasse, List<BaustellenWrapper>>>();
			for (final AttTmcRichtung r : AttTmcRichtung.getZustaende()) {
				baustellenAufStrassenRichtungMap.put(r,
						new HashMap<Strasse, List<BaustellenWrapper>>());
			}

			final int totalWork = 100000;
			final String taskName = "Ordne Baustellen";
			monitor.setTaskName(taskName);
			monitor.beginTask(taskName, totalWork);

			int i = 0;
			final int anzahlBaustellen = baustellenWrapperMap.values().size();
			if (anzahlBaustellen == 0) {
				return true;
			}
			final int work = totalWork / anzahlBaustellen;

			for (final BaustellenWrapper wrapper : baustellenWrapperMap
					.values()) {
				monitor.subTask("Analysiere Baustelle: " + wrapper.getName()
						+ " (" + i + "/" + anzahlBaustellen + ")");

				final Strasse strasse = wrapper.getStrasse();
				final AttTmcRichtung richtung = wrapper.getRichtung();

				if (strasse != null) {

					List<BaustellenWrapper> baustellenAufStrasse = baustellenAufStrasseMap
							.get(strasse);
					if (baustellenAufStrasse == null) {
						baustellenAufStrasse = new ArrayList<BaustellenWrapper>();
						baustellenAufStrasseMap.put(strasse,
								baustellenAufStrasse);
					}
					if (!baustellenAufStrasse.contains(wrapper)) {
						baustellenAufStrasse.add(wrapper);
					}

					if (richtung != null) {
						List<BaustellenWrapper> baustellenAufStrassenRichtung = baustellenAufStrassenRichtungMap
								.get(richtung).get(strasse);
						if (baustellenAufStrassenRichtung == null) {
							baustellenAufStrassenRichtung = new ArrayList<BaustellenWrapper>();
							baustellenAufStrassenRichtungMap.get(richtung).put(
									strasse, baustellenAufStrassenRichtung);
						}
						if (!baustellenAufStrassenRichtung.contains(wrapper)) {
							baustellenAufStrassenRichtung.add(wrapper);
						}
					}
				}

				monitor.worked(work);
				if (monitor.isCanceled()) {
					monitor.done();
					return false;
				}
				i++;
			}
		}
		return true;
	}

	/**
	 * Ordnet die UnfallWrapper den Straen und Richtungen zu.
	 * 
	 * @param monitor
	 *            der Monitor
	 * 
	 * @return <code>true</code>, wenn diese Methode komplett ausgefhrt wurde,
	 *         ansonsten bei vorzeitigem Abbruch <code>false</code>
	 */
	private boolean ordneUnfaelle(final IMonitor monitor) {
		synchronized (lock) {

			unfaelleAufStrasseMap = new HashMap<Strasse, List<UnfallWrapper>>();
			unfaelleAufStrassenRichtungMap = new HashMap<AttTmcRichtung, Map<Strasse, List<UnfallWrapper>>>();
			for (final AttTmcRichtung r : AttTmcRichtung.getZustaende()) {
				unfaelleAufStrassenRichtungMap.put(r,
						new HashMap<Strasse, List<UnfallWrapper>>());
			}

			final int totalWork = 100000;
			final String taskName = "Ordne Unflle";
			monitor.setTaskName(taskName);
			monitor.beginTask(taskName, totalWork);

			int i = 0;
			final int anzahlUnfaelle = unfallWrapperMap.values().size();
			if (anzahlUnfaelle == 0) {
				return true;
			}
			final int work = totalWork / anzahlUnfaelle;

			for (final UnfallWrapper wrapper : unfallWrapperMap.values()) {
				monitor.subTask("Analysiere Unfall: " + wrapper.getName()
						+ " (" + i + "/" + anzahlUnfaelle + ")");

				final Strasse strasse = wrapper.getStrasse();
				final AttTmcRichtung richtung = wrapper.getRichtung();

				if (strasse != null) {

					List<UnfallWrapper> unfaelleAufStrasse = unfaelleAufStrasseMap
							.get(strasse);
					if (unfaelleAufStrasse == null) {
						unfaelleAufStrasse = new ArrayList<UnfallWrapper>();
						unfaelleAufStrasseMap.put(strasse, unfaelleAufStrasse);
					}
					if (!unfaelleAufStrasse.contains(wrapper)) {
						unfaelleAufStrasse.add(wrapper);
					}

					if (richtung != null) {
						List<UnfallWrapper> unfaelleAufStrassenRichtung = unfaelleAufStrassenRichtungMap
								.get(richtung).get(strasse);
						if (unfaelleAufStrassenRichtung == null) {
							unfaelleAufStrassenRichtung = new ArrayList<UnfallWrapper>();
							unfaelleAufStrassenRichtungMap.get(richtung).put(
									strasse, unfaelleAufStrassenRichtung);
						}
						if (!unfaelleAufStrassenRichtung.contains(wrapper)) {
							unfaelleAufStrassenRichtung.add(wrapper);
						}
					}
				}

				monitor.worked(work);
				if (monitor.isCanceled()) {
					monitor.done();
					return false;
				}
				i++;
			}
		}
		return true;
	}

	/**
	 * Ordnet die StauWrapper den Straen und Richtungen zu.
	 * 
	 * @param monitor
	 *            der Monitor
	 * 
	 * @return <code>true</code>, wenn diese Methode komplett ausgefhrt wurde,
	 *         ansonsten bei vorzeitigem Abbruch <code>false</code>
	 */
	private boolean ordneStaus(final IMonitor monitor) {
		synchronized (lock) {

			stausAufStrasseMap = new HashMap<Strasse, List<StauWrapper>>();
			stausAufStrassenRichtungMap = new HashMap<AttTmcRichtung, Map<Strasse, List<StauWrapper>>>();
			for (final AttTmcRichtung r : AttTmcRichtung.getZustaende()) {
				stausAufStrassenRichtungMap.put(r,
						new HashMap<Strasse, List<StauWrapper>>());
			}

			final int totalWork = 100000;
			final String taskName = "Ordne Staus";
			monitor.setTaskName(taskName);
			monitor.beginTask(taskName, totalWork);

			int i = 0;
			final int anzahlStaus = stauWrapperMap.values().size();
			if (anzahlStaus == 0) {
				return true;
			}
			final int work = totalWork / anzahlStaus;

			for (final StauWrapper wrapper : stauWrapperMap.values()) {
				monitor.subTask("Analysiere Stau: " + wrapper.getName() + " ("
						+ i + "/" + anzahlStaus + ")");

				final Strasse strasse = wrapper.getStrasse();
				final AttTmcRichtung richtung = wrapper.getRichtung();

				if (strasse != null) {

					List<StauWrapper> stausAufStrasse = stausAufStrasseMap
							.get(strasse);
					if (stausAufStrasse == null) {
						stausAufStrasse = new ArrayList<StauWrapper>();
						stausAufStrasseMap.put(strasse, stausAufStrasse);
					}
					if (!stausAufStrasse.contains(wrapper)) {
						stausAufStrasse.add(wrapper);
					}

					if (richtung != null) {
						List<StauWrapper> stausAufStrassenRichtung = stausAufStrassenRichtungMap
								.get(richtung).get(strasse);
						if (stausAufStrassenRichtung == null) {
							stausAufStrassenRichtung = new ArrayList<StauWrapper>();
							stausAufStrassenRichtungMap.get(richtung).put(
									strasse, stausAufStrassenRichtung);
						}
						if (!stausAufStrassenRichtung.contains(wrapper)) {
							stausAufStrassenRichtung.add(wrapper);
						}
					}
				}

				monitor.worked(work);
				if (monitor.isCanceled()) {
					monitor.done();
					return false;
				}
				i++;
			}
		}
		return true;
	}

	/**
	 * Ermittelt die zu den existierenden Engstellen ggf. asoziierte Ereignisse.
	 * 
	 * @param monitor
	 *            der Monitor
	 * 
	 * @return <code>true</code>, wenn diese Methode komplett ausgefhrt wurde,
	 *         ansonsten bei vorzeitigem Abbruch <code>false</code>
	 */
	private boolean ermittelEreignisse(final IMonitor monitor) {

		synchronized (lock) {

			ereignisseMap = new HashMap<Situation, Ereignis>();

			final int totalWork = 100000;
			final String taskName = "Finde Ereignisse";
			monitor.setTaskName(taskName);
			monitor.beginTask(taskName, totalWork);

			// (1) ermittel smtliche Ereignisse
			final AenderbareMenge<Ereignis> ereignisse = BauEditorPlugin
					.getDefault().getAOE().getEreignisse();

			int i = -1;
			final int anzahlEreignisse = ereignisse.size();
			if (anzahlEreignisse == 0) {
				return true;
			}
			final int work = totalWork / anzahlEreignisse;

			// (2) analysiere smtliche Ereignisse dahingehend, ob als
			// zustzliches Attribut auf eine Engstelle referenziert wird
			for (final Ereignis ereignis : ereignisse) {
				i++;
				monitor.worked(work);
				if (monitor.isCanceled()) {
					monitor.done();
					return false;
				}

				monitor.subTask("Analysiere Ereignis " + i + " von "
						+ anzahlEreignisse);

				final KdEreignisEigenschaften kdEreignisEigenschaften = ereignis
						.getKdEreignisEigenschaften();
				if (kdEreignisEigenschaften == null) {
					continue;
				}
				final Daten datum = kdEreignisEigenschaften.getDatum();
				if (datum == null) {
					continue;
				}
				final Feld<AtlZusaetzlicheAttribute> attribute = datum
						.getZusaetzlicheAttribute();
				if (attribute == null || attribute.isEmpty()) {
					continue;
				}

				for (final AtlZusaetzlicheAttribute attribut : attribute) {

					if (!Konstanten.EREIGNIS_EIGENSCHAFTEN_REFERENZ_AUSLOESENDE_SITUATION
							.equals(attribut.getAttributname())) {
						continue;
					}

					final String wert = attribut.getAttributwert();
					if (wert == null || "".equals(wert)) {
						continue;
					}

					final ObjektFactory objektFactory = BauEditorPlugin
							.getDefault().getObjektFactory();
					if (objektFactory != null && objektFactory.isVerbunden()) {
						final SystemObjekt systemObjekt = objektFactory
								.getModellobjekt(wert);
						if (systemObjekt != null
								&& systemObjekt instanceof Situation) {
							ereignisseMap.put((Situation) systemObjekt,
									ereignis);
						}
					}
				}
			}
		}
		return true;
	}

	/**
	 * Liefert eine Collection smtlicher StauWrapper zurck.
	 * 
	 * @return eine Collection smtlicher StauWrapper zurck, niemals
	 *         <code>null</code>
	 */
	public Collection<StauWrapper> getStauWrapper() {
		ensureInit();
		if (stauWrapperMap != null) {
			return stauWrapperMap.values();
		}
		return new ArrayList<StauWrapper>();
	}

	/**
	 * Liefert eine Collection smtlicher BaustellenWrapper zurck.
	 * 
	 * @return eine Collection smtlicher BaustellenWrapper zurck, niemals
	 *         <code>null</code>
	 */
	public Collection<BaustellenWrapper> getBaustellenWrapper() {
		ensureInit();
		if (baustellenWrapperMap != null) {
			return baustellenWrapperMap.values();
		}
		return new ArrayList<BaustellenWrapper>();
	}

	/**
	 * Liefert den zu der Baustelle passenden BaustellenWrapper zurck.
	 * 
	 * @param baustelle
	 *            die Baustelle, darf nicht <code>null</code> sein
	 * 
	 * @return den zu der Baustelle passenden BaustellenWrapper zurck
	 */
	public BaustellenWrapper getBaustellenWrapper(final Baustelle baustelle) {
		// ensureInit();
		if (baustelle == null) {
			throw new IllegalArgumentException(
					"bergebene Baustelle darf nicht null sein.");
		}
		return baustellenWrapperMap.get(baustelle);
	}

	/**
	 * Liefert eine Liste aller BaustellenWrapper zurck, welche auf der
	 * gegebenen Strae liegen.
	 * 
	 * @param strasse
	 *            die Strae, darf nicht <code>null</code> sein
	 * 
	 * @return eine Liste aller BaustellenWrapper, welche auf der gegebenen
	 *         Strae liegen, niemals <code>null</code>
	 */
	public List<BaustellenWrapper> getBaustellenWrapper(final Strasse strasse) {
		ensureInit();
		if (strasse == null) {
			throw new IllegalArgumentException(
					"bergebene Strae darf nicht null sein.");
		}
		final List<BaustellenWrapper> ergebnis = baustellenAufStrasseMap
				.get(strasse);
		if (ergebnis == null) {
			return new ArrayList<BaustellenWrapper>();
		}
		return ergebnis;
	}

	/**
	 * Liefert eine Liste aller StauWrapper zurck, welche auf der gegebenen
	 * Strae liegen.
	 * 
	 * @param strasse
	 *            die Strae, darf nicht <code>null</code> sein
	 * 
	 * @return eine Liste aller StauWrapper, welche auf der gegebenen Strae
	 *         liegen, niemals <code>null</code>
	 */
	public List<StauWrapper> getStauWrapper(final Strasse strasse) {
		ensureInit();
		if (strasse == null) {
			throw new IllegalArgumentException(
					"bergebene Strae darf nicht null sein.");
		}
		final List<StauWrapper> ergebnis = stausAufStrasseMap.get(strasse);
		if (ergebnis == null) {
			return new ArrayList<StauWrapper>();
		}
		return ergebnis;
	}

	/**
	 * Liefert eine Liste aller BaustellenWrapper zurck, welche auf der
	 * gegebenen Straen(richtung) liegen.
	 * 
	 * @param strasse
	 *            die Strae, darf nicht <code>null</code> sein
	 * @param richtung
	 *            die Richtung
	 * 
	 * @return eine Liste aller BaustellenWrapper, welche auf der gegebenen
	 *         Straen(richtung) liegen, niemals <code>null</code>
	 */
	public List<BaustellenWrapper> getBaustellenWrapper(final Strasse strasse,
			final AttTmcRichtung richtung) {
		ensureInit();
		if (strasse == null) {
			throw new IllegalArgumentException(
					"bergebene Strae darf nicht null sein.");
		}
		final List<BaustellenWrapper> ergebnis = baustellenAufStrassenRichtungMap
				.get(richtung).get(strasse);
		if (ergebnis == null) {
			return new ArrayList<BaustellenWrapper>();
		}
		return ergebnis;
	}

	/**
	 * Liefert eine Collection smtlicher UnfallWrapper zurck.
	 * 
	 * @return eine Collection smtlicher UnfallWrapper zurck, niemals
	 *         <code>null</code>
	 */
	public Collection<UnfallWrapper> getUnfallWrapper() {
		ensureInit();
		if (unfallWrapperMap != null) {
			return unfallWrapperMap.values();
		}
		return new ArrayList<UnfallWrapper>();
	}

	/**
	 * Liefert den zu der Unfall passenden UnfallWrapper zurck.
	 * 
	 * @param unfall
	 *            der Unfall, darf nicht <code>null</code> sein
	 * 
	 * @return den zu der Unfall passenden UnfallWrapper zurck
	 */
	public UnfallWrapper getUnfallWrapper(final Unfall unfall) {
		// ensureInit();
		if (unfall == null) {
			throw new IllegalArgumentException(
					"bergebener Unfall darf nicht null sein.");
		}
		return unfallWrapperMap.get(unfall);
	}

	/**
	 * Liefert den zum Stau passenden StauWrapper zurck.
	 * 
	 * @param stau
	 *            der Unfall, darf nicht <code>null</code> sein
	 * 
	 * @return den zu der Unfall passenden UnfallWrapper zurck
	 */
	public StauWrapper getStauWrapper(final Stau stau) {
		// ensureInit();
		if (stau == null) {
			throw new IllegalArgumentException(
					"bergebener Stau darf nicht null sein.");
		}
		return stauWrapperMap.get(stau);
	}

	/**
	 * Liefert eine Liste aller UnfallWrapper zurck, welche auf der gegebenen
	 * Strae liegen.
	 * 
	 * @param strasse
	 *            die Strae, darf nicht <code>null</code> sein
	 * 
	 * @return eine Liste aller UnfallWrapper, welche auf der gegebenen Strae
	 *         liegen, niemals <code>null</code>
	 */
	public List<UnfallWrapper> getUnfallWrapper(final Strasse strasse) {
		ensureInit();
		if (strasse == null) {
			throw new IllegalArgumentException(
					"bergebene Strae darf nicht null sein.");
		}
		final List<UnfallWrapper> ergebnis = unfaelleAufStrasseMap.get(strasse);
		if (ergebnis == null) {
			return new ArrayList<UnfallWrapper>();
		}
		return ergebnis;
	}

	/**
	 * Liefert eine Liste aller UnfallWrapper zurck, welche auf der gegebenen
	 * Straen(richtung) liegen.
	 * 
	 * @param strasse
	 *            die Strae, darf nicht <code>null</code> sein
	 * @param richtung
	 *            die Richtung
	 * 
	 * @return eine Liste aller UnfallWrapper, welche auf der gegebenen
	 *         Straen(richtung) liegen, niemals <code>null</code>
	 */
	public List<UnfallWrapper> getUnfallWrapper(final Strasse strasse,
			final AttTmcRichtung richtung) {
		ensureInit();
		if (strasse == null) {
			throw new IllegalArgumentException(
					"bergebene Strae darf nicht null sein.");
		}
		final List<UnfallWrapper> ergebnis = unfaelleAufStrassenRichtungMap
				.get(richtung).get(strasse);
		if (ergebnis == null) {
			return new ArrayList<UnfallWrapper>();
		}
		return ergebnis;
	}

	/**
	 * Liefert eine Liste aller StauWrapper zurck, welche auf der gegebenen
	 * Straen(richtung) liegen.
	 * 
	 * @param strasse
	 *            die Strae, darf nicht <code>null</code> sein
	 * @param richtung
	 *            die Richtung
	 * 
	 * @return eine Liste aller StauWrapper, welche auf der gegebenen
	 *         Straen(richtung) liegen, niemals <code>null</code>
	 */
	public List<StauWrapper> getStauWrapper(final Strasse strasse,
			final AttTmcRichtung richtung) {
		ensureInit();
		if (strasse == null) {
			throw new IllegalArgumentException(
					"bergebene Strae darf nicht null sein.");
		}
		final List<StauWrapper> ergebnis = stausAufStrassenRichtungMap.get(
				richtung).get(strasse);
		if (ergebnis == null) {
			return new ArrayList<StauWrapper>();
		}
		return ergebnis;
	}

	/**
	 * Liefert das zu einer Engstelle dazugehrige Ereignis zurck oder
	 * <code>null</code> falls kein solches Ereignis existiert.
	 * 
	 * @param situation
	 *            die Engstelle, darf <code>null</code> sein
	 * 
	 * @return das zu einer Engstelle dazugehrige Ereignis, kann
	 *         <code>null</code> sein
	 */
	public Ereignis getEreignis(final Situation situation) {
		// kein Aufruf von ensureInit(), da in der Initialisierungsphase beim
		// Erzeugen der Baustellen- und Unfall-Wrapper bereits auf diese Methode
		// zugegriffen werden muss
		if (situation == null || ereignisseMap == null) {
			return null;
		}
		return ereignisseMap.get(situation);
	}

	/**
	 * Entfernt smtliche registrierten Listener sowie smtliche gecachten
	 * Referenzen auf Engstellen.
	 * 
	 * @param entferneGlobaleListener
	 *            <code>true</code>, wenn auch smtliche globalen Listener
	 *            entfernt werden sollen
	 */
	public void dispose(final boolean entferneGlobaleListener) {

		EreignisKalenderVerwaltung.getInstanz().removeEreignisKalenderListener(
				ereignisKalenderListener);

		if (streckenAbschnittCache != null) {
			final VerkehrsModellNetz verkehrsModellNetz = streckenAbschnittCache
					.getNetz();

			// Entferne UpdateListener
			if (verkehrsModellNetz != null) {
				final AenderbareMenge<Baustelle> baustellen = verkehrsModellNetz
						.getBaustellen();
				if (baustellen != null) {
					for (final Baustelle baustelle : baustellen) {
						removeUpdateListener(baustelle);
						removeInvalidationListener(baustelle);
					}
				}
				final AenderbareMenge<Unfall> unfaelle = verkehrsModellNetz
						.getUnfaelle();
				if (unfaelle != null) {
					for (final Unfall unfall : unfaelle) {
						removeUpdateListener(unfall);
						removeInvalidationListener(unfall);
					}
				}

				final AenderbareMenge<Stau> staus = verkehrsModellNetz
						.getStaus();
				if (staus != null) {
					for (final Stau stau : staus) {
						removeUpdateListener(stau);
						removeInvalidationListener(stau);
					}
				}
			}

			if (entferneGlobaleListener) {

				// Entferne MengenListener
				if (verkehrsModellNetz != null) {
					verkehrsModellNetz.getBaustellen().removeMengenListener(
							this);
					verkehrsModellNetz.getUnfaelle().removeMengenListener(this);
					verkehrsModellNetz.getStaus().removeMengenListener(this);
				}

				// Entferne NameChangeListener fr die Typen Baustelle und
				// Unfall
				final ObjektFactory objektFactory = BauEditorPlugin
						.getDefault().getObjektFactory();
				if (objektFactory != null && objektFactory.getDav() != null) {

					final ClientDavInterface dav = objektFactory.getDav();
					if (dav != null) {
						final DataModel dataModel = dav.getDataModel();
						if (dataModel != null) {

							// Baustelle
							final SystemObjectType baustellenType = dataModel
									.getType(Baustelle.PID);
							if (baustellenType != null) {
								if (baustellenType != null
										&& baustellenType instanceof DynamicObjectType) {
									((DynamicObjectType) baustellenType)
											.removeNameChangeListener(this);
								}
							}

							// Unfall
							final SystemObjectType unfallType = dataModel
									.getType(Unfall.PID);
							if (unfallType != null) {
								if (unfallType != null
										&& unfallType instanceof DynamicObjectType) {
									((DynamicObjectType) unfallType)
											.removeNameChangeListener(this);
								}
							}
						}
					}
				}
			}
		}
		synchronized (lock) {
			ereignisseMap = null;
			baustellenWrapperMap = null;
			baustellenAufStrasseMap = null;
			baustellenAufStrassenRichtungMap = null;
			unfallWrapperMap = null;
			unfaelleAufStrasseMap = null;
			unfaelleAufStrassenRichtungMap = null;
			stauWrapperMap = null;
			stausAufStrasseMap = null;
			stausAufStrassenRichtungMap = null;
		}
	}

	@Override
	public void mengeAktualisiert(final MengenEvent e) {

		final Collection<? extends SystemObjekt> entfernt = e.getEntfernt();
		final Collection<? extends SystemObjekt> hinzugefuegt = e
				.getHinzugefuegt();
		threadPool.execute(new MengeAktualisiertRunner(entfernt, hinzugefuegt));
	}

	/**
	 * Runnable zur Behandlung einer Mengennderung.
	 * 
	 * @author BitCtrl Systems GmbH, Andreas Meissner
	 * @version $Id: EngstellenCache.java,v 1.2 2014/11/14 07:29:43 Pittner Exp $
	 */
	private class MengeAktualisiertRunner implements Runnable {

		/** Die entfernten Engstellen. */
		private final Collection<? extends SystemObjekt> entfernt;

		/** Die neu hinzugefgten Engstellen. */
		private final Collection<? extends SystemObjekt> hinzugefuegt;

		/**
		 * Der Konstruktor.
		 * 
		 * @param entfernt
		 *            die entfernten Engstellen
		 * @param hinzugefuegt
		 *            die neu hinzugefgten Engstellen
		 */
		public MengeAktualisiertRunner(
				final Collection<? extends SystemObjekt> entfernt,
				final Collection<? extends SystemObjekt> hinzugefuegt) {
			this.entfernt = entfernt;
			this.hinzugefuegt = hinzugefuegt;
		}

		public void run() {

			final List<EngstellenWrapper> entfernteEngstellen = new ArrayList<EngstellenWrapper>();
			final List<EngstellenWrapper> hinzugefuegteEngstellen = new ArrayList<EngstellenWrapper>();

			synchronized (lock) {

				// Ermittel entfernte Engstellen
				for (final SystemObjekt obj : entfernt) {

					if (obj instanceof Baustelle) {
						final Baustelle baustelle = (Baustelle) obj;
						removeUpdateListener(baustelle);
						removeInvalidationListener(baustelle);

						final BaustellenWrapper wrapper = getBaustellenWrapper(baustelle);
						if (wrapper != null
								&& !entfernteEngstellen.contains(wrapper)) {
							entfernteEngstellen.add(wrapper);
							baustellenWrapperMap.remove(baustelle);
							baustellenAufStrasseMap.get(wrapper.getStrasse())
									.remove(wrapper);
							baustellenAufStrassenRichtungMap
									.get(wrapper.getRichtung())
									.get(wrapper.getStrasse()).remove(wrapper);
						}

					} else if (obj instanceof Unfall) {
						final Unfall unfall = (Unfall) obj;
						removeUpdateListener(unfall);
						removeInvalidationListener(unfall);

						final UnfallWrapper wrapper = getUnfallWrapper(unfall);
						if (wrapper != null
								&& !entfernteEngstellen.contains(wrapper)) {
							entfernteEngstellen.add(wrapper);
							unfallWrapperMap.remove(unfall);
							unfaelleAufStrasseMap.get(wrapper.getStrasse())
									.remove(wrapper);
							unfaelleAufStrassenRichtungMap
									.get(wrapper.getRichtung())
									.get(wrapper.getStrasse()).remove(wrapper);
						}
					} else if (obj instanceof Stau) {
						final Stau stau = (Stau) obj;
						removeUpdateListener(stau);
						removeInvalidationListener(stau);

						final StauWrapper wrapper = getStauWrapper(stau);
						if (wrapper != null
								&& !entfernteEngstellen.contains(wrapper)) {
							entfernteEngstellen.add(wrapper);
							stauWrapperMap.remove(stau);
							stausAufStrasseMap.get(wrapper.getStrasse())
									.remove(wrapper);
							stausAufStrassenRichtungMap
									.get(wrapper.getRichtung())
									.get(wrapper.getStrasse()).remove(wrapper);
						}
					}

				}

				// Ermittel hinzugefgte Engstellen
				for (final SystemObjekt obj : hinzugefuegt) {

					if (obj instanceof Baustelle) {

						try {

							final Baustelle baustelle = (Baustelle) obj;
							removeUpdateListener(baustelle);
							addUpdateListener(baustelle);
							removeInvalidationListener(baustelle);
							addInvalidationListener(baustelle);

							final BaustellenWrapper wrapper = erzeugeBaustellenWrapper(baustelle);
							if (wrapper != null
									&& !baustellenWrapperMap
											.containsKey(baustelle)
									&& !hinzugefuegteEngstellen
											.contains(wrapper)) {
								hinzugefuegteEngstellen.add(wrapper);
								addBaustelle(baustelle, wrapper);
							}

						} catch (final RuntimeException e) {
							BauEditorPlugin
									.getDefault()
									.getLogger()
									.warning(
											"Es gabe Probleme bei der Behandlung einer neu hinzugefgten Baustelle.",
											e);
						}

					} else if (obj instanceof Unfall) {

						try {
							final Unfall unfall = (Unfall) obj;
							removeUpdateListener(unfall);
							addUpdateListener(unfall);
							removeInvalidationListener(unfall);
							addInvalidationListener(unfall);

							final UnfallWrapper wrapper = erzeugeUnfallWrapper(unfall);
							if (wrapper != null
									&& !unfallWrapperMap.containsKey(unfall)
									&& !hinzugefuegteEngstellen
											.contains(wrapper)) {
								hinzugefuegteEngstellen.add(wrapper);
								addUnfall(unfall, wrapper);
							}

						} catch (final RuntimeException e) {
							BauEditorPlugin
									.getDefault()
									.getLogger()
									.warning(
											"Es gabe Probleme bei der Behandlung eines neu hinzugefgten Unfalls.",
											e);
						}
					} else if (obj instanceof Stau) {

						try {
							final Stau stau = (Stau) obj;
							removeUpdateListener(stau);
							addUpdateListener(stau);
							removeInvalidationListener(stau);
							addInvalidationListener(stau);

							final StauWrapper wrapper = erzeugeStauWrapper(stau);
							if (wrapper != null
									&& !stauWrapperMap.containsKey(stau)
									&& !hinzugefuegteEngstellen
											.contains(wrapper)) {
								hinzugefuegteEngstellen.add(wrapper);
								addStau(stau, wrapper);
							}

						} catch (final RuntimeException e) {
							BauEditorPlugin
									.getDefault()
									.getLogger()
									.warning(
											"Es gabe Probleme bei der Behandlung eines neu hinzugefgten Staus.",
											e);
						}
					}

				}

				// Benachrichtige Listener
				if (listeners != null
						&& !listeners.isEmpty()
						&& (!entfernteEngstellen.isEmpty() || !hinzugefuegteEngstellen
								.isEmpty())) {
					final List<EngstellenChangedListener> tempList = new ArrayList<EngstellenChangedListener>(
							listeners);
					for (final EngstellenChangedListener listener : tempList) {
						if (!entfernteEngstellen.isEmpty()) {
							listener.engstellenRemoved(EngstellenCache.this,
									entfernteEngstellen);
						}
						if (!hinzugefuegteEngstellen.isEmpty()) {
							listener.engstellenAdded(EngstellenCache.this,
									hinzugefuegteEngstellen);
						}
					}
				}
			}
		}
	}

	@Override
	public void nameChanged(final DynamicObject dynobj) {
		final ObjektFactory objektFactory = BauEditorPlugin.getDefault()
				.getObjektFactory();
		if (objektFactory != null && objektFactory.isVerbunden()) {
			final SystemObjekt sysObj = objektFactory.getModellobjekt(dynobj
					.getPid());
			if (sysObj != null) {
				threadPool
						.execute(new DatensatzAktualisiertRunner(sysObj, null));
			}
		}
	}

	@Override
	public void datensatzAktualisiert(final DatensatzUpdateEvent event) {

		if (!listenerAktiv) {
			return;
		}

		final SystemObjekt systemObjekt = event.getObjekt();
		final Zeitstempel zeitstempel = event.getDatum().dGetZeitstempel();
		if (systemObjekt != null) {
			threadPool.execute(new DatensatzAktualisiertRunner(systemObjekt,
					zeitstempel));
		}
	}

	/**
	 * Runnable zur Behandlung einer Datensatznderung.
	 * 
	 * @author BitCtrl Systems GmbH, Andreas Meissner
	 * @version $Id: EngstellenCache.java,v 1.2 2014/11/14 07:29:43 Pittner Exp $
	 */
	private class DatensatzAktualisiertRunner implements Runnable {

		/** Das Systemobjekt dessen Daten sich gendert haben. */
		private final SystemObjekt systemObjekt;

		/** Der Zeitstempel des genderten Datums. */
		private final Zeitstempel zeitStempel;

		/**
		 * Der Konstruktor.
		 * 
		 * @param systemObjekt
		 *            das Systemobjekt dessen Daten sich gendert haben.
		 * @param zeitStempel
		 *            der Zeitstempel des genderten Datums
		 */
		public DatensatzAktualisiertRunner(final SystemObjekt systemObjekt,
				final Zeitstempel zeitStempel) {
			this.systemObjekt = systemObjekt;
			this.zeitStempel = zeitStempel;
		}

		public void run() {

			EngstellenWrapper alterWrapper = null;
			EngstellenWrapper neuerWrapper = null;

			synchronized (lock) {

				if (systemObjekt instanceof Baustelle) {
					final Baustelle baustelle = (Baustelle) systemObjekt;
					alterWrapper = getBaustellenWrapper(baustelle);
					if (zeitStempel != null
							&& alterWrapper != null
							&& alterWrapper.getZeitStempel() != null
							&& zeitStempel.getTime() <= alterWrapper
									.getZeitStempel().getTime()) {
						return;
					}
					neuerWrapper = changeBaustelle(baustelle,
							(BaustellenWrapper) alterWrapper);
				} else if (systemObjekt instanceof Unfall) {
					final Unfall unfall = (Unfall) systemObjekt;
					alterWrapper = getUnfallWrapper(unfall);
					if (zeitStempel != null
							&& alterWrapper != null
							&& alterWrapper.getZeitStempel() != null
							&& zeitStempel.getTime() <= alterWrapper
									.getZeitStempel().getTime()) {
						return;
					}
					neuerWrapper = changeUnfall(unfall,
							(UnfallWrapper) alterWrapper);
				} else if (systemObjekt instanceof Stau) {
					final Stau stau = (Stau) systemObjekt;
					alterWrapper = getStauWrapper(stau);
					if (zeitStempel != null
							&& alterWrapper != null
							&& alterWrapper.getZeitStempel() != null
							&& zeitStempel.getTime() <= alterWrapper
									.getZeitStempel().getTime()) {
						return;
					}
					neuerWrapper = changeStau(stau, (StauWrapper) alterWrapper);
				}

				// Benachrichtige Listener
				if (neuerWrapper != null && listeners != null
						&& !listeners.isEmpty()) {
					final List<EngstellenChangedListener> tempList = new ArrayList<EngstellenChangedListener>(
							listeners);
					for (final EngstellenChangedListener listener : tempList) {
						listener.engstelleChanged(EngstellenCache.this,
								neuerWrapper, alterWrapper);
					}
				}
			}
		}
	}

	@Override
	public void invalidObject(final DynamicObject dynamicObject) {
		if (!listenerAktiv) {
			return;
		}

		final ObjektFactory objektFactory = BauEditorPlugin.getDefault()
				.getObjektFactory();
		if (objektFactory != null && objektFactory.isVerbunden()) {
			final SystemObjekt sysObj = objektFactory
					.getModellobjekt(dynamicObject);
			if (sysObj != null && sysObj instanceof Situation) {
				threadPool.execute(new MeldungInvalidiertRunner(
						(Situation) sysObj));
			}
		}
	}

	/**
	 * Runnable zur Behandlung einer Invalidierung.
	 * 
	 * @author BitCtrl Systems GmbH, Andreas Meissner
	 * @version $Id: EngstellenCache.java,v 1.2 2014/11/14 07:29:43 Pittner Exp $
	 */
	private class MeldungInvalidiertRunner implements Runnable {

		/** Die Engstelle. */
		private final Situation engstelle;

		/**
		 * Der Konstruktor.
		 * 
		 * @param engstelle
		 *            die Engstelle
		 */
		public MeldungInvalidiertRunner(final Situation engstelle) {
			this.engstelle = engstelle;
		}

		public void run() {

			final List<EngstellenWrapper> entfernteRdsMeldungen = new ArrayList<EngstellenWrapper>();

			synchronized (lock) {

				removeUpdateListener(engstelle);
				removeInvalidationListener(engstelle);

				EngstellenWrapper wrapper = null;
				if (engstelle instanceof Baustelle) {
					wrapper = getBaustellenWrapper((Baustelle) engstelle);
				} else if (engstelle instanceof Unfall) {
					wrapper = getUnfallWrapper((Unfall) engstelle);
				} else if (engstelle instanceof Stau) {
					wrapper = getStauWrapper((Stau) engstelle);
				}

				if (wrapper != null) {
					entfernteRdsMeldungen.add(wrapper);
					if (engstelle instanceof Baustelle) {
						baustellenWrapperMap.remove(engstelle);
					} else if (engstelle instanceof Unfall) {
						unfallWrapperMap.remove(engstelle);
					} else if (engstelle instanceof Stau) {
						stauWrapperMap.remove(engstelle);
					}
				}

				// Benachrichtige Listener
				if (listeners != null && !listeners.isEmpty()
						&& !entfernteRdsMeldungen.isEmpty()) {
					final List<EngstellenChangedListener> tempList = new ArrayList<EngstellenChangedListener>(
							listeners);
					for (final EngstellenChangedListener listener : tempList) {
						listener.engstellenRemoved(EngstellenCache.this,
								entfernteRdsMeldungen);
					}
				}
			}
		}
	}

	/**
	 * ndert den BaustellenWrapper in den Maps.
	 * 
	 * @param baustelle
	 *            die Baustelle
	 * @param alterWrapper
	 *            der alte Wrapper
	 * 
	 * @return der neue BaustellenWrapper
	 */
	private BaustellenWrapper changeBaustelle(final Baustelle baustelle,
			final BaustellenWrapper alterWrapper) {
		final BaustellenWrapper neuerWrapper = erzeugeBaustellenWrapper(baustelle);
		if (neuerWrapper != null) {
			baustellenWrapperMap.remove(baustelle);
			if (alterWrapper != null) {
				baustellenAufStrasseMap.get(alterWrapper.getStrasse()).remove(
						alterWrapper);
				baustellenAufStrassenRichtungMap
						.get(alterWrapper.getRichtung())
						.get(alterWrapper.getStrasse()).remove(alterWrapper);
			}
			addBaustelle(baustelle, neuerWrapper);
		}
		return neuerWrapper;
	}

	/**
	 * Fgt die Baustelle den Maps hinzu.
	 * 
	 * @param baustelle
	 *            die Baustelle
	 * @param wrapper
	 *            der dazugehrige Wrapper
	 */
	private void addBaustelle(final Baustelle baustelle,
			final BaustellenWrapper wrapper) {
		baustellenWrapperMap.put(baustelle, wrapper);
		List<BaustellenWrapper> liste1 = baustellenAufStrasseMap.get(wrapper
				.getStrasse());
		if (liste1 == null) {
			liste1 = new ArrayList<BaustellenWrapper>();
			baustellenAufStrasseMap.put(wrapper.getStrasse(), liste1);
		}
		liste1.add(wrapper);

		List<BaustellenWrapper> liste2 = baustellenAufStrassenRichtungMap.get(
				wrapper.getRichtung()).get(wrapper.getStrasse());
		if (liste2 == null) {
			liste2 = new ArrayList<BaustellenWrapper>();
			baustellenAufStrassenRichtungMap.get(wrapper.getRichtung()).put(
					wrapper.getStrasse(), liste2);
		}
		liste2.add(wrapper);
	}

	/**
	 * ndert den UnfallWrapper in den Maps.
	 * 
	 * @param unfall
	 *            der Unfall
	 * @param alterWrapper
	 *            der alte Wrapper
	 * 
	 * @return der neue UnfallWrapper
	 */
	private UnfallWrapper changeUnfall(final Unfall unfall,
			final UnfallWrapper alterWrapper) {
		final UnfallWrapper neuerWrapper = erzeugeUnfallWrapper(unfall);
		if (neuerWrapper != null) {
			unfallWrapperMap.remove(unfall);
			if (alterWrapper != null) {
				unfaelleAufStrasseMap.get(alterWrapper.getStrasse()).remove(
						alterWrapper);
				unfaelleAufStrassenRichtungMap.get(alterWrapper.getRichtung())
						.get(alterWrapper.getStrasse()).remove(alterWrapper);
			}
			addUnfall(unfall, neuerWrapper);
		}
		return neuerWrapper;
	}

	/**
	 * ndert den Stau Wrapper in den Maps.
	 * 
	 * @param stau
	 *            der Stau
	 * @param alterWrapper
	 *            der alte Wrapper
	 * 
	 * @return der neue StauWrapper
	 */
	private StauWrapper changeStau(final Stau stau,
			final StauWrapper alterWrapper) {
		final StauWrapper neuerWrapper = erzeugeStauWrapper(stau);
		if (neuerWrapper != null) {
			stauWrapperMap.remove(stau);
			if (alterWrapper != null) {
				stausAufStrasseMap.get(alterWrapper.getStrasse()).remove(
						alterWrapper);
				stausAufStrassenRichtungMap.get(alterWrapper.getRichtung())
						.get(alterWrapper.getStrasse()).remove(alterWrapper);
			}
			addStau(stau, neuerWrapper);
		}
		return neuerWrapper;
	}

	/**
	 * Fgt den Unfall den Maps hinzu.
	 * 
	 * @param unfall
	 *            der Unfalls
	 * @param wrapper
	 *            der dazugehrige Wrapper
	 */
	private void addUnfall(final Unfall unfall, final UnfallWrapper wrapper) {
		unfallWrapperMap.put(unfall, wrapper);
		List<UnfallWrapper> liste1 = unfaelleAufStrasseMap.get(wrapper
				.getStrasse());
		if (liste1 == null) {
			liste1 = new ArrayList<UnfallWrapper>();
			unfaelleAufStrasseMap.put(wrapper.getStrasse(), liste1);
		}
		liste1.add(wrapper);

		List<UnfallWrapper> liste2 = unfaelleAufStrassenRichtungMap.get(
				wrapper.getRichtung()).get(wrapper.getStrasse());
		if (liste2 == null) {
			liste2 = new ArrayList<UnfallWrapper>();
			unfaelleAufStrassenRichtungMap.get(wrapper.getRichtung()).put(
					wrapper.getStrasse(), liste2);
		}
		liste2.add(wrapper);
	}

	/**
	 * Fgt den Stau den Maps hinzu.
	 * 
	 * @param stau
	 *            der Stau
	 * @param wrapper
	 *            der dazugehrige Wrapper
	 */
	private void addStau(final Stau stau, final StauWrapper wrapper) {
		stauWrapperMap.put(stau, wrapper);
		List<StauWrapper> liste1 = stausAufStrasseMap.get(wrapper.getStrasse());
		if (liste1 == null) {
			liste1 = new ArrayList<StauWrapper>();
			stausAufStrasseMap.put(wrapper.getStrasse(), liste1);
		}
		liste1.add(wrapper);

		List<StauWrapper> liste2 = stausAufStrassenRichtungMap.get(
				wrapper.getRichtung()).get(wrapper.getStrasse());
		if (liste2 == null) {
			liste2 = new ArrayList<StauWrapper>();
			stausAufStrassenRichtungMap.get(wrapper.getRichtung()).put(
					wrapper.getStrasse(), liste2);
		}
		liste2.add(wrapper);
	}

	/**
	 * Fgt die entsprechenden UpdateListener einer Engstelle hinzu.
	 * 
	 * @param situation
	 *            die Engstelle
	 */
	private void addUpdateListener(final Situation situation) {
		if (situation instanceof Baustelle) {
			final Baustelle baustelle = (Baustelle) situation;
			baustelle.getPdSituationsEigenschaften().addUpdateListener(this);
			baustelle.getPdBaustellenEigenschaften().addUpdateListener(this);
			baustelle.getPdBaustellenEigenschaftenErweitert()
					.addUpdateListener(this);
			baustelle.getPdBaustellenVerantwortlicher().addUpdateListener(this);
		} else if (situation instanceof Unfall) {
			final Unfall unfall = (Unfall) situation;
			unfall.getPdSituationsEigenschaften().addUpdateListener(this);
			unfall.getPdUnfallEigenschaften().addUpdateListener(this);
			unfall.getPdUnfallEigenschaftenErweitert().addUpdateListener(this);
		} else if (situation instanceof Stau) {
			final Stau stau = (Stau) situation;
			stau.getPdSituationsEigenschaften().addUpdateListener(this);
			stau.getOdStauVerlauf().addUpdateListener(
					OdStauVerlauf.Aspekte.PrognoseNormal, this);
		}
	}

	/**
	 * Entfernt die entsprechenden UpdateListener von einer Engstelle.
	 * 
	 * @param situation
	 *            die Engstelle
	 */
	private void removeUpdateListener(final Situation situation) {
		if (situation instanceof Baustelle) {
			final Baustelle baustelle = (Baustelle) situation;
			baustelle.getPdSituationsEigenschaften().removeUpdateListener(this);
			baustelle.getPdBaustellenEigenschaften().removeUpdateListener(this);
			baustelle.getPdBaustellenEigenschaftenErweitert()
					.removeUpdateListener(this);
			baustelle.getPdBaustellenVerantwortlicher().removeUpdateListener(
					this);
		} else if (situation instanceof Unfall) {
			final Unfall unfall = (Unfall) situation;
			unfall.getPdSituationsEigenschaften().removeUpdateListener(this);
			unfall.getPdUnfallEigenschaften().removeUpdateListener(this);
			unfall.getPdUnfallEigenschaftenErweitert().removeUpdateListener(
					this);
		} else if (situation instanceof Stau) {
			final Stau stau = (Stau) situation;
			stau.getPdSituationsEigenschaften().removeUpdateListener(this);
			stau.getOdStauVerlauf().removeUpdateListener(
					OdStauVerlauf.Aspekte.PrognoseNormal, this);
		}
	}

	/**
	 * Fgt den InvalidationListener zu einer Engstelle hinzu.
	 * 
	 * @param situation
	 *            die Engstelle
	 */
	private void addInvalidationListener(final Situation situation) {
		((DynamicObject) situation.getSystemObject())
				.addListenerForInvalidation(this);
	}

	/**
	 * Entfernt den InvalidationListener von einer Engstelle.
	 * 
	 * @param situation
	 *            die Engstelle
	 */
	private void removeInvalidationListener(final Situation situation) {
		((DynamicObject) situation.getSystemObject())
				.removeListenerForInvalidation(this);
	}

	/**
	 * Entfernt smtliche nicht-globalen Listener und fhrt die Initialisierung
	 * dieses Caches erneut durch.
	 */
	public void reInit() {
		dispose(false);
		redoInit();
	}
}
