/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.util.wrapper;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.concurrent.CancellationException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import de.bsvrz.buv.plugin.baueditor.sts.BauEditorException;
import de.bsvrz.buv.plugin.baueditor.sts.BauEditorPlugin;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.sys.funclib.bitctrl.modell.AnmeldeException;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensendeException;
import de.bsvrz.sys.funclib.bitctrl.modell.att.Feld;
import de.bsvrz.sys.funclib.bitctrl.modell.att.Zeitstempel;
import de.bsvrz.sys.funclib.bitctrl.modell.impl.AbstractOnlineDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.impl.AbstractParameterDatensatz;
import de.bsvrz.sys.funclib.bitctrl.modell.tmereigniskalenderglobal.objekte.Ereignis;
import de.bsvrz.sys.funclib.bitctrl.modell.tmereigniskalenderglobal.objekte.EreignisTyp;
import de.bsvrz.sys.funclib.bitctrl.modell.tmtmcglobal.attribute.AttTmcRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.konfigurationsdaten.KdAeusseresStrassenSegment.Daten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.AeusseresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.InneresStrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Situation;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Strasse;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenKnoten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdSituationsEigenschaften;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.NetzCacheExtended;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.StreckenAbschnitt;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.StreckenAbschnittCache;

/**
 * Abstrakte Wrapper-Klasse fr Situationen.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: SituationsWrapper.java,v 1.2 2014/11/14 07:29:42 Pittner Exp $
 * 
 * @see Situation
 */
public abstract class SituationsWrapper {

	/** Der PropertyChangeSupport. */
	private final PropertyChangeSupport changeSupport = new PropertyChangeSupport(
			this);

	/** Der erweiterte Netz-Cache. */
	private final NetzCacheExtended netzCache;

	/** Der Streckenabschnitts-Cache. */
	private final StreckenAbschnittCache streckenCache;

	/** Das in diesem Wrapper gekapselte Situations-SystemObjekt. */
	private Situation situation;

	/** Das zu der Engstelle dazugehrige Ereignis, kann <code>null</code> sein. */
	private Ereignis ereignis;

	/**
	 * Der Ereignistyp, des zu der Engstelle dazugehrige Ereignisses, kann
	 * <code>null</code> sein.
	 */
	private EreignisTyp ereignisTyp;

	/** Das Dirty-Flag. */
	private boolean dirty;

	/**
	 * Flag, welches angibt, ob es sich bei der Engstelle um ein neu anzulegende
	 * Engstelle handelt.
	 */
	private boolean neuesElement;

	/** Der aktuellste Zeitstempel der in diesem Wrapper geladenen Daten. */
	private Zeitstempel zeitStempel;

	/**
	 * Flag, welches angibt, ob die Engstelle mit einem dazugehrigen Ereignis
	 * verknpft ist bzw. verknpft werden soll.
	 */
	private boolean verknuepfeEreignis;

	/** Der Name der Engstelle. */
	private String name;

	/** Die optionale Kurzinfo. */
	private String kurzInfo;

	/** Die optionale Beschreibung. */
	private String beschreibung;

	/** Die Strae auf der diese Engstelle liegt. */
	private Strasse strasse;

	/** Die Straenrichtung dieser Engstelle. */
	private AttTmcRichtung richtung;

	/**
	 * Flag, welches angibt, ob fr die aktuell ausgewhlte Strae eine
	 * Richtungsauswahl mglich ist.
	 */
	private boolean richtungsAuswahlMoeglich;

	/** Der Startstraenknoten dieser Engstelle. */
	private StrassenKnoten startKnoten;

	/** Der Endstraenknoten dieser Engstelle. */
	private StrassenKnoten endKnoten;

	/** Der zugrundeliegende Streckenabschnitt dieser Engstelle. */
	private StreckenAbschnitt streckenAbschnitt;

	/** Die Startzeit dieser Engstelle (auf Sekunden abgeschnitten). */
	private Date startZeit;

	/** Die Endzeit dieser Engstelle (auf Sekunden abgeschnitten). */
	private Date endZeit;

	/** Die Dauer dieser Engstelle. */
	private Long dauer;

	/** Der PropertyChangeListener zum Setzen des Dirty-Flags. */
	private final PropertyChangeListener propertyChangeListener = new PropertyChangeListener() {
		public void propertyChange(final PropertyChangeEvent evt) {
			setDirty(true);
		}
	};

	/**
	 * Der Konstruktor.
	 * 
	 * @param strasse
	 *            die Strae, auf welcher die neue Engstelle liegen soll, darf
	 *            <code>null</code> sein
	 * @param richtung
	 *            die Fahrtrichtung, auf welcher die neue Engstelle liegen soll,
	 *            darf <code>null</code> sein
	 * @param segment
	 *            das Straensegment, kann <code>null</code> sein
	 */
	public SituationsWrapper(final Strasse strasse,
			final AttTmcRichtung richtung, final StrassenSegment segment) {

		netzCache = StartupPlugin.getDefault().getDefaultNetzCacheExtended();
		streckenCache = StartupPlugin.getDefault()
				.getDefaultStreckenAbschnittCache();

		dirty = true;
		neuesElement = true;
		verknuepfeEreignis = false;
		ereignis = null;
		ereignisTyp = null;
		name = "";
		zeitStempel = null;

		setDefaultInfo();
		setDefaultZeiten();
		setDefaultSituation(strasse, richtung, segment);

		fuegeStreckenAbschnittListenerHinzu();
	}

	/**
	 * Setzt die Standardwerte fr die Kurzinfo und die Beschreibung.
	 */
	private void setDefaultInfo() {
		kurzInfo = "";
		beschreibung = "";
	}

	/**
	 * Setzt die Standardwerte fr die Zeiten und Dauer.
	 */
	private void setDefaultZeiten() {
		startZeit = new Date((System.currentTimeMillis() / 1000) * 1000);
		dauer = Long.valueOf(0);
		endZeit = new Date(startZeit.getTime());
	}

	/**
	 * Setzt die Standardwerte fr die Lage dieser Engstelle.
	 * 
	 * @param neueStrasse
	 *            die Strae
	 * @param neueRichtung
	 *            die Richtung
	 * @param neuesSegment
	 *            das Straensegment
	 */
	private void setDefaultSituation(final Strasse neueStrasse,
			final AttTmcRichtung neueRichtung,
			final StrassenSegment neuesSegment) {

		strasse = neueStrasse;
		if (strasse == null) {
			final List<Strasse> strassen = netzCache.getStrassen();
			if (!strassen.isEmpty()) {
				strasse = strassen.get(0);
			}
		}

		// Setze und prfe Richtung.
		richtung = neueRichtung;
		if (richtung == null
				|| richtung == AttTmcRichtung.ZUSTAND_0_OHNE_RICHTUNG) {
			richtung = AttTmcRichtung.ZUSTAND_1_POSITIV;
		}
		richtungsAuswahlMoeglich = true;
		List<StrassenKnoten> strKnoten = netzCache.getStrassenKnoten(strasse,
				richtung);
		if (strKnoten == null || strKnoten.isEmpty()) {
			richtung = (richtung == AttTmcRichtung.ZUSTAND_1_POSITIV) ? AttTmcRichtung.ZUSTAND_1N_NEGATIV
					: AttTmcRichtung.ZUSTAND_1_POSITIV;
			richtungsAuswahlMoeglich = false;
		}
		if (richtungsAuswahlMoeglich) {
			strKnoten = netzCache
					.getStrassenKnoten(
							strasse,
							(richtung == AttTmcRichtung.ZUSTAND_1_POSITIV) ? AttTmcRichtung.ZUSTAND_1N_NEGATIV
									: AttTmcRichtung.ZUSTAND_1_POSITIV);
			if (strKnoten == null || strKnoten.isEmpty()) {
				richtungsAuswahlMoeglich = false;
			}
		}

		// Ermittel Streckenabschnitt.
		if (neuesSegment != null) {

			if (neuesSegment instanceof InneresStrassenSegment) {
				final InneresStrassenSegment iss = (InneresStrassenSegment) neuesSegment;
				startKnoten = netzCache.getNetzCache().getStrassenKnoten(iss);
				endKnoten = startKnoten;

			} else if (neuesSegment instanceof AeusseresStrassenSegment) {
				final AeusseresStrassenSegment ass = (AeusseresStrassenSegment) neuesSegment;
				final Daten datum = ass.getKdAeusseresStrassenSegment()
						.getDatum();
				if (datum != null) {
					startKnoten = datum.getVonKnoten();
					endKnoten = datum.getNachKnoten();
				}
			}
		}

		if (startKnoten == null || endKnoten == null) {
			final StreckenAbschnitt standardStreckenAbschnitt = streckenCache
					.getStandardStreckenAbschnitt(strasse, richtung);
			final List<StrassenKnoten> knoten = standardStreckenAbschnitt
					.getStrassenKnoten();
			if (knoten != null && knoten.size() >= 2) {
				startKnoten = knoten.get(0);
				endKnoten = knoten.get(1);
			}
		}

		setStreckenAbschnitt(false);
		if (streckenAbschnitt != null) {
			if (neuesSegment != null
					&& neuesSegment instanceof InneresStrassenSegment) {
				streckenAbschnitt
						.setInneresStrassenSegment((InneresStrassenSegment) neuesSegment);
			}
			streckenAbschnitt.setStartOffset(0d);
			streckenAbschnitt.setEndOffset(streckenAbschnitt.getMaxEndOffset());
		}
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param situation
	 *            die Engstelle, darf nicht <code>null</code> sein
	 * 
	 * @throws BauEditorException
	 *             wird geworfen, wenn die gegebene Situation nicht geladen
	 *             werden konnte
	 */
	public SituationsWrapper(final Situation situation)
			throws BauEditorException {

		if (situation == null) {
			throw new IllegalArgumentException(
					"bergebene Engstelle darf nicht null sein.");
		}

		netzCache = StartupPlugin.getDefault().getDefaultNetzCacheExtended();
		streckenCache = StartupPlugin.getDefault()
				.getDefaultStreckenAbschnittCache();
		this.situation = situation;
		dirty = false;
		neuesElement = false;
		name = situation.getName();

		ereignis = BauEditorPlugin.getDefault().getEngstellenCache()
				.getEreignis(situation);
		ereignisTyp = null;
		if (ereignis != null
				&& ereignis.getKdEreignisEigenschaften().getDatum() != null) {
			ereignisTyp = ereignis.getKdEreignisEigenschaften().getDatum()
					.getEreignisTypReferenz();
		}
		verknuepfeEreignis = ereignis != null;

		// Lese Konfigurationsdatensatz Info
		final de.bsvrz.sys.funclib.bitctrl.modell.metamodellglobal.konfigurationsdaten.KdInfo.Daten kdInfo = situation
				.getKdInfo().getDatum();
		if (kdInfo != null) {
			kurzInfo = kdInfo.getKurzinfo();
			if (kurzInfo == null) {
				kurzInfo = "";
			}
			beschreibung = kdInfo.getBeschreibung();
			if (beschreibung == null) {
				beschreibung = "";
			}
		} else {
			setDefaultInfo();
		}

		// Lese Parameterdatensatz Eigenschaften
		final PdSituationsEigenschaften pdSituationsEigenschaften = situation
				.getPdSituationsEigenschaften();
		final de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.parameter.PdSituationsEigenschaften.Daten pdEigenschaften = pdSituationsEigenschaften
				.getDatum();
		setZeitStempel(pdEigenschaften.dGetZeitstempel());

		// Ermittel Zeiten.
		if (pdEigenschaften != null && pdEigenschaften.getStartZeit() != null
				&& pdEigenschaften.getDauer() != null) {

			startZeit = new Date(
					(pdEigenschaften.getStartZeit().getTime() / 1000) * 1000);
			dauer = (pdEigenschaften.getDauer().getTime() / 1000) * 1000;
			endZeit = new Date(startZeit.getTime() + dauer);
			aktualisiereDauer();

		} else {
			setDefaultZeiten();
		}

		// Ermittel Straen- und Richtungsinformationen
		if (pdEigenschaften != null
				&& pdEigenschaften.getStrassenSegment() != null
				&& !pdEigenschaften.getStrassenSegment().isEmpty()) {

			final Feld<StrassenSegment> pdStrassenSegmente = pdEigenschaften
					.getStrassenSegment();

			final List<StrassenSegment> strassenSegmente = new ArrayList<StrassenSegment>();
			final List<AeusseresStrassenSegment> aeussereStrassenSegmente = new ArrayList<AeusseresStrassenSegment>();
			final List<InneresStrassenSegment> innereStrassenSegmente = new ArrayList<InneresStrassenSegment>();

			strasse = null;
			richtung = null;
			boolean richtungGeladen = false;
			for (final StrassenSegment segment : pdStrassenSegmente) {
				if (!strassenSegmente.contains(segment)) {
					strassenSegmente.add(segment);
				}
				if (segment instanceof AeusseresStrassenSegment
						&& !aeussereStrassenSegmente.contains(segment)) {
					final AeusseresStrassenSegment ass = (AeusseresStrassenSegment) segment;
					aeussereStrassenSegmente.add(ass);
					if (!richtungGeladen) {
						richtung = ass.getKdAeusseresStrassenSegment()
								.getDatum().getTmcRichtung();
						strasse = segment.getKdStrassenSegment().getDatum()
								.getGehoertZuStrasse();
						richtungGeladen = true;
					}
				}

				if (segment instanceof InneresStrassenSegment
						&& !innereStrassenSegmente.contains(segment)) {
					final InneresStrassenSegment iss = (InneresStrassenSegment) segment;
					innereStrassenSegmente.add(iss);
					if (!richtungGeladen) {

						strasse = iss.getKdStrassenSegment().getDatum()
								.getGehoertZuStrasse();

						AeusseresStrassenSegment ass = iss
								.getKdInneresStrassenSegment().getDatum()
								.getNachStrassenSegment();
						if (ass == null
								|| (strasse != null && !strasse.equals(ass
										.getKdStrassenSegment().getDatum()
										.getGehoertZuStrasse()))) {
							ass = iss.getKdInneresStrassenSegment().getDatum()
									.getVonStrassenSegment();
						}

						if (ass != null
								&& (strasse == null || strasse.equals(ass
										.getKdStrassenSegment().getDatum()
										.getGehoertZuStrasse()))) {
							richtung = netzCache.getRichtung(iss);
							if (richtung != null) {
								richtungGeladen = true;
							}
							if (strasse == null) {
								strasse = ass.getKdStrassenSegment().getDatum()
										.getGehoertZuStrasse();
							}
						}
					}
				}
			}
			richtungsAuswahlMoeglich = !netzCache.getStrassenKnoten(strasse,
					AttTmcRichtung.ZUSTAND_1_POSITIV).isEmpty()
					&& !netzCache.getStrassenKnoten(strasse,
							AttTmcRichtung.ZUSTAND_1N_NEGATIV).isEmpty();

			startKnoten = ermittelStartStrassenKnoten(strassenSegmente);
			endKnoten = ermittelEndStrassenKnoten(strassenSegmente);

			if (strasse == null || richtung == null
					|| strassenSegmente.isEmpty() || startKnoten == null
					|| endKnoten == null) {
				richtung = AttTmcRichtung.ZUSTAND_1_POSITIV;

				String errorMsg = null;
				if (strasse == null) {
					errorMsg = "Keine Strae zugeordnet!";
				} else if (richtung == null) {
					errorMsg = "Keine Richtung zugeordnet!";
				} else if (strassenSegmente.isEmpty()) {
					errorMsg = "Menge der Strassensegmente ist leer!";
				} else if (startKnoten == null) {
					errorMsg = "Der Startkknoten ist leer!";
				} else if (endKnoten == null) {
					errorMsg = "Der Endknoten ist leer!";
				}

				BauEditorPlugin
						.getDefault()
						.getLogger()
						.warning(
								"Die folgende Engstelle konnte nicht geladen werden: "
										+ this.situation.getName() + " ("
										+ this.situation.getPid() + "):"
										+ errorMsg);
				throw new BauEditorException(errorMsg);
			}

			setStreckenAbschnitt(true);

			if (streckenAbschnitt != null) {

				for (final AeusseresStrassenSegment ass : aeussereStrassenSegmente) {
					setAlternativesAeuesseresStrassenSegment(ass);
				}
				for (final InneresStrassenSegment iss : innereStrassenSegmente) {
					streckenAbschnitt.setInneresStrassenSegment(iss);
				}

				if (pdEigenschaften.getStartOffset() != null) {
					final double startOffset = pdEigenschaften.getStartOffset()
							.doubleValue() / 1000;
					if (strassenSegmente != null
							&& !strassenSegmente.isEmpty()
							&& strassenSegmente.get(0) instanceof InneresStrassenSegment
							&& streckenAbschnitt.getStartASS() != null) {
						streckenAbschnitt.setStartOffset(startOffset
								+ streckenAbschnitt.getLaenge(streckenAbschnitt
										.getStartASS()));
					} else {
						streckenAbschnitt.setStartOffset(startOffset);
					}
				} else {
					streckenAbschnitt.setStartOffset(0d);
				}
				if (pdEigenschaften.getEndOffset() != null) {
					final double endOffset = pdEigenschaften.getEndOffset()
							.doubleValue() / 1000;
					if (strassenSegmente != null
							&& !strassenSegmente.isEmpty()
							&& strassenSegmente
									.get(strassenSegmente.size() - 1) instanceof InneresStrassenSegment
							&& streckenAbschnitt.getEndASS() != null) {
						streckenAbschnitt.setEndOffset(endOffset
								+ streckenAbschnitt.getLaenge(streckenAbschnitt
										.getEndASS()));
					} else {
						streckenAbschnitt.setEndOffset(endOffset);
					}
				} else {
					streckenAbschnitt.setEndOffset(streckenAbschnitt
							.getMaxEndOffset());
				}
			}

		} else {
			setDefaultSituation(null, null, null);
		}

		fuegeStreckenAbschnittListenerHinzu();
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param wrapper
	 *            ein anderer SituationsWrapper
	 * @param istNeuesElement
	 *            <code>true</code>, wenn es sich bei dem neu erstellten
	 *            SituationWrapper, um eine neue Engstelle handelt. false, wenn
	 *            ein Klon erstellt werden soll.
	 * 
	 */
	public SituationsWrapper(final SituationsWrapper wrapper,
			final boolean istNeuesElement) {

		netzCache = StartupPlugin.getDefault().getDefaultNetzCacheExtended();
		streckenCache = StartupPlugin.getDefault()
				.getDefaultStreckenAbschnittCache();

		if (istNeuesElement) {
			situation = null;
			neuesElement = true;
			ereignis = null;
			dirty = true;
			zeitStempel = null;
		} else {
			situation = wrapper.situation;
			neuesElement = wrapper.neuesElement;
			ereignis = wrapper.ereignis;
			dirty = false;
			zeitStempel = wrapper.zeitStempel;
		}

		ereignisTyp = wrapper.ereignisTyp;
		verknuepfeEreignis = wrapper.verknuepfeEreignis;

		name = new String(wrapper.name);
		kurzInfo = new String(wrapper.kurzInfo);
		beschreibung = new String(wrapper.beschreibung);

		strasse = wrapper.strasse;
		richtung = wrapper.richtung;
		richtungsAuswahlMoeglich = wrapper.richtungsAuswahlMoeglich;
		startKnoten = wrapper.startKnoten;
		endKnoten = wrapper.endKnoten;
		streckenAbschnitt = wrapper.streckenAbschnitt.clone();

		startZeit = new Date(wrapper.startZeit.getTime());
		endZeit = new Date(wrapper.endZeit.getTime());
		dauer = wrapper.dauer;

		fuegeStreckenAbschnittListenerHinzu();
	}

	/**
	 * Fgt dem aktuellen Streckenabschnitt entsprechende PropertyChangeListener
	 * fr die Offsets hinzu.
	 */
	protected void fuegeStreckenAbschnittListenerHinzu() {
		if (streckenAbschnitt != null) {
			streckenAbschnitt.addPropertyChangeListener("startOffset",
					propertyChangeListener);
			streckenAbschnitt.addPropertyChangeListener("endOffset",
					propertyChangeListener);
		}
	}

	/**
	 * Liefert das in diesem Objekt gekapselte Situationsobjekt zurck.
	 * 
	 * @return das in diesem Objekt gekapselte Situationsobjekt
	 */
	public Situation getSituation() {
		return situation;
	}

	/**
	 * Liefert das mit dieser Engstelle verknpfte Ereignis zurck.
	 * 
	 * @return das mit dieser Engstelle verknpfte Ereignis, kann
	 *         <code>null</code> sein
	 */
	public Ereignis getEreignis() {
		return ereignis;
	}

	/**
	 * Setzt das mit dieser Engstelle verknpfte Ereignis.
	 * 
	 * @param neuerWert
	 *            das neue Ereignis
	 */
	public void setEreignis(final Ereignis neuerWert) {
		if (neuerWert == null && ereignis == null) {
			return;
		}
		if ((neuerWert != null && ereignis == null)
				|| (neuerWert == null && ereignis != null)
				|| (!neuerWert.getPid().equals(ereignis.getPid()))) {
			final Ereignis alterWert = ereignis;
			ereignis = neuerWert;
			firePropertyChange("ereignis", alterWert, neuerWert);
			setVerknuepfeEreignis(neuerWert != null);
		}
	}

	/**
	 * Setzt den EreignisTyp, des mit dieser Engstelle verknpften Ereignisses.
	 * 
	 * @param neuerWert
	 *            der neue EreignisType
	 */
	public void setEreignisTyp(final EreignisTyp neuerWert) {
		if (neuerWert == null && ereignisTyp == null) {
			return;
		}
		if ((neuerWert != null && ereignisTyp == null)
				|| (neuerWert == null && ereignisTyp != null)
				|| (!neuerWert.getPid().equals(ereignisTyp.getPid()))) {
			final EreignisTyp alterWert = ereignisTyp;
			ereignisTyp = neuerWert;
			firePropertyChange("ereignisTyp", alterWert, neuerWert);
		}
	}

	/**
	 * Liefert den EreignisTyp, des mit dieser Engstelle verknpften Ereignisses
	 * zurck.
	 * 
	 * @return den EreignisTyp, des mit dieser Engstelle verknpften
	 *         Ereignisses, kann <code>null</code> sein
	 */
	public EreignisTyp getEreignisTyp() {
		return ereignisTyp;
	}

	/**
	 * Liefert den aktuellen Dirty-State zurck.
	 * 
	 * @return den aktuellen Dirty-State
	 */
	public boolean isDirty() {
		return dirty;
	}

	/**
	 * Setzt den Dirty-State dieses Wrappers.
	 * 
	 * @param neuerWert
	 *            der neue Dirty-State
	 */
	public void setDirty(final boolean neuerWert) {
		if (dirty != neuerWert) {
			final boolean alterWert = dirty;
			dirty = neuerWert;
			firePropertyChange("dirty", alterWert, neuerWert);
		}
	}

	/**
	 * Liefert <code>true</code> zurck, wenn es sich bei diesem Wrapper, um
	 * eine neu anzulegende Engstelle handelt, ansonsten <code>false</code>.
	 * 
	 * @return <code>true</code>, wenn es sich bei diesem Wrapper, um eine neu
	 *         anzulegende Engstelle handelt, ansonsten <code>false</code>
	 */
	public boolean isNeuesElement() {
		return neuesElement;
	}

	/**
	 * Liefert <code>true</code> zurck, wenn die Engstelle mit einem Ereignis
	 * verknpft ist bzw. werden soll.
	 * 
	 * @return <code>true</code>, wenn die Engstelle mit einem Ereignis
	 *         verknpft ist bzw. werden soll
	 */
	public boolean isVerknuepfeEreignis() {
		return verknuepfeEreignis;
	}

	/**
	 * Definiert, ob die Engstelle mit einem Ereignis verknpft werden soll.
	 * 
	 * @param neuerWert
	 *            der neue Wert
	 */
	public void setVerknuepfeEreignis(final boolean neuerWert) {
		if (verknuepfeEreignis != neuerWert) {
			final boolean alterWert = verknuepfeEreignis;
			verknuepfeEreignis = neuerWert;
			firePropertyChange("verknuepfeEreignis", alterWert, neuerWert);
		}
	}

	/**
	 * Liefert den Namen dieser Engstelle zurck.
	 * 
	 * @return den Namen dieser Engstelle zurck
	 */
	public String getName() {
		return name;
	}

	/**
	 * Setzt den Namen dieser Engstelle.
	 * 
	 * @param neuerWert
	 *            der neue Name
	 */
	public void setName(final String neuerWert) {
		final String alterWert = name;
		name = neuerWert;
		firePropertyChange("name", alterWert, neuerWert);
	}

	/**
	 * Liefert die Kurzinfo dieser Engstelle zurck.
	 * 
	 * @return die Kurzinfo dieser Engstelle zurck
	 */
	public String getKurzInfo() {
		return kurzInfo;
	}

	/**
	 * Setzt die Kurzinfo dieser Engstelle.
	 * 
	 * @param neuerWert
	 *            die neue Kurzinfo
	 */
	public void setKurzInfo(final String neuerWert) {
		final String alterWert = kurzInfo;
		kurzInfo = neuerWert;
		firePropertyChange("kurzInfo", alterWert, neuerWert);
	}

	/**
	 * Liefert die Beschreibung dieser Engstelle zurck.
	 * 
	 * @return die Beschreibung dieser Engstelle zurck
	 */
	public String getBeschreibung() {
		return beschreibung;
	}

	/**
	 * Setzt die Beschreibung dieser Engstelle.
	 * 
	 * @param neuerWert
	 *            die neue Beschreibung
	 */
	public void setBeschreibung(final String neuerWert) {
		final String alterWert = beschreibung;
		beschreibung = neuerWert;
		firePropertyChange("beschreibung", alterWert, neuerWert);
	}

	/**
	 * Liefert die Startzeit dieser Engstelle zurck.
	 * 
	 * @return die Startzeit dieser Engstelle zurck
	 */
	public Date getStartZeit() {
		return startZeit;
	}

	/**
	 * Setzt die Startzeit dieser Engstelle.
	 * 
	 * <p>
	 * Beachte: Diese Methode schneidet Millisekunden ab.
	 * </p>
	 * 
	 * @param neuerWert
	 *            die neue Startzeit
	 */
	public void setStartZeit(final Date neuerWert) {
		setStartZeit(neuerWert, true);
	}

	/**
	 * Setzt die Startzeit dieser Engstelle.
	 * 
	 * @param neuerWert
	 *            die neue Startzeit
	 * @param aktualisiereDauer
	 *            <code>true</code>, wenn die Dauer aktualisiert werden soll,
	 *            ansonsten <code>false</code>
	 */
	private void setStartZeit(final Date neuerWert,
			final boolean aktualisiereDauer) {
		Date derNeueWert = neuerWert;
		if (derNeueWert == null) {
			derNeueWert = new Date();
		}
		derNeueWert = new Date((derNeueWert.getTime() / 1000) * 1000);
		final Date alterWert = startZeit;
		startZeit = derNeueWert;
		firePropertyChange("startZeit", alterWert, derNeueWert);
		if (endZeit.before(startZeit)) {
			setEndZeit(startZeit, aktualisiereDauer);
		} else if (aktualisiereDauer) {
			aktualisiereDauer();
		}
	}

	/**
	 * Liefert die Endzeit dieser Engstelle zurck.
	 * 
	 * @return die Endzeit dieser Engstelle zurck
	 */
	public Date getEndZeit() {
		return endZeit;
	}

	/**
	 * Setzt die Endzeit dieser Engstelle.
	 * 
	 * <p>
	 * Beachte: Diese Methode schneidet Millisekunden ab.
	 * </p>
	 * 
	 * @param neuerWert
	 *            die neue Endzeit
	 */
	public void setEndZeit(final Date neuerWert) {
		setEndZeit(neuerWert, true);
	}

	/**
	 * Setzt die Endzeit dieser Engstelle.
	 * 
	 * @param neuerWert
	 *            die neue Endzeit
	 * @param aktualisiereDauer
	 *            <code>true</code>, wenn die Dauer aktualisiert werden soll,
	 *            ansonsten <code>false</code>
	 */
	private void setEndZeit(final Date neuerWert,
			final boolean aktualisiereDauer) {
		Date derNeueWert = neuerWert;
		if (derNeueWert == null) {
			derNeueWert = new Date();
		}
		derNeueWert = new Date((derNeueWert.getTime() / 1000) * 1000);
		final Date alterWert = endZeit;
		endZeit = derNeueWert;
		firePropertyChange("endZeit", alterWert, derNeueWert);
		if (startZeit.after(endZeit)) {
			setStartZeit(endZeit, aktualisiereDauer);
		} else if (aktualisiereDauer) {
			aktualisiereDauer();
		}
	}

	/**
	 * Aktualisiert die Dauer abhngig von der Start- und Endzeit.
	 */
	private void aktualisiereDauer() {
		final Long alterWert = dauer;
		final long dauerInMillis = endZeit.getTime() - startZeit.getTime();
		dauer = dauerInMillis;
		firePropertyChange("dauer", alterWert, dauer);
	}

	/**
	 * Setzt die Dauer dieser Engstelle.
	 * 
	 * @param neuerWert
	 *            die neue Dauer
	 */
	public void setDauer(final Long neuerWert) {
		final Long alterWert = dauer;
		dauer = neuerWert;
		firePropertyChange("dauer", alterWert, neuerWert);
		setEndZeit(new Date(startZeit.getTime() + dauer), false);
	}

	/**
	 * Liefert die Dauer dieser Engstelle zurck.
	 * 
	 * @return die Dauer dieser Engstelle zurck
	 */
	public Long getDauer() {
		return dauer;
	}

	/**
	 * Liefert die Strae zurck, auf welcher diese Engstelle liegt.
	 * 
	 * @return die Strae, auf welcher diese Engstelle liegt
	 */
	public Strasse getStrasse() {
		return strasse;
	}

	/**
	 * Setzt die Strae dieser Engstelle.
	 * 
	 * @param strasse
	 *            die neue Strae
	 */
	public void setStrasse(final Strasse strasse) {
		final Strasse alterWert = this.strasse;
		this.strasse = strasse;
		firePropertyChange("strasse", alterWert, strasse);

		// ndere die Straenrichtung, falls es fr die neue gesetzte Strae in
		// der bisherigen Richtung keine Straensegmente gibt
		final AttTmcRichtung entgegengesetzteRichtung = (richtung == AttTmcRichtung.ZUSTAND_1_POSITIV) ? AttTmcRichtung.ZUSTAND_1N_NEGATIV
				: AttTmcRichtung.ZUSTAND_1_POSITIV;
		if (netzCache.getStrassenKnoten(strasse, richtung).isEmpty()) {
			setRichtung(entgegengesetzteRichtung);
			setRichtungsAuswahlMoeglich(false);
		} else {
			setRichtungsAuswahlMoeglich(!netzCache.getStrassenKnoten(strasse,
					entgegengesetzteRichtung).isEmpty());
		}
	}

	/**
	 * Liefert die Straenrichtung dieser Engstelle zurck.
	 * 
	 * @return die Straenrichtung dieser Engstelle
	 */
	public AttTmcRichtung getRichtung() {
		return richtung;
	}

	/**
	 * Setzt die Straenrichtung dieser Engstelle.
	 * 
	 * @param richtung
	 *            die neue Richtung
	 */
	public void setRichtung(final AttTmcRichtung richtung) {
		// es darf die Richtung nur gendert werden, wenn es fr diese Richtung
		// auch Straensegmente gibt
		if (netzCache.getStrassenKnoten(strasse, richtung).isEmpty()) {
			return;
		}
		final AttTmcRichtung alterWert = this.richtung;
		this.richtung = richtung;
		firePropertyChange("richtung", alterWert, richtung);
	}

	/**
	 * Liefert <code>true</code> zurck, wenn fr die aktuell gesetzte Strae
	 * eine Richtungsauswahl mglich ist (also es fr die positive und negative
	 * Straenrichtung uere Straensegmente gibt).
	 * 
	 * @return <code>true</code>, wenn fr die aktuell gesetzte Strae eine
	 *         Richtungsauswahl mglich ist, ansonsten false
	 */
	public boolean isRichtungsAuswahlMoeglich() {
		return richtungsAuswahlMoeglich;
	}

	/**
	 * Setzt das Flag, welches angibt, ob eine Richtungsauswahl fr die aktuell
	 * gesetzte Strae mglich ist.
	 * 
	 * @param neuerWert
	 *            <code>true</code>, wenn bei der aktuell gesetzten Strae eine
	 *            Richtungsauswahl mglich, ansonsten <code>false</code>
	 */
	protected void setRichtungsAuswahlMoeglich(final boolean neuerWert) {
		if (richtungsAuswahlMoeglich != neuerWert) {
			final boolean alterWert = richtungsAuswahlMoeglich;
			richtungsAuswahlMoeglich = neuerWert;
			firePropertyChange("richtungsAuswahlMoeglich", alterWert, neuerWert);
		}
	}

	/**
	 * Liefert den Startstraenknoten dieser Engstelle zurck.
	 * 
	 * @return den Startstraenknoten dieser Engstelle
	 */
	public StrassenKnoten getStartKnoten() {
		return startKnoten;
	}

	/**
	 * Setzt den Startstraenknoten dieser Engstelle.
	 * 
	 * @param startKnoten
	 *            der neue Startstraenknoten
	 */
	public void setStartKnoten(final StrassenKnoten startKnoten) {
		setStartKnoten(startKnoten, true);
	}

	/**
	 * Setzt den Startstraenknoten dieser Engstelle.
	 * 
	 * @param neuerStartKnoten
	 *            der neue Startstraenknoten
	 * @param fireEvent
	 *            <code>true</code>, wenn das dazugehrige PropertyChangeEvent
	 *            gefeuert werden soll
	 */
	private void setStartKnoten(final StrassenKnoten neuerStartKnoten,
			final boolean fireEvent) {

		if (neuerStartKnoten != null && startKnoten == neuerStartKnoten) {
			return;
		}

		// Setze neuen Startknoten. Wenn dieser null ist, nimm den ersten in der
		// Liste mglicher Startknoten.
		StrassenKnoten neuerWert = neuerStartKnoten;
		if (neuerWert == null) {
			if (strasse != null && richtung != null) {
				final List<StrassenKnoten> startStrassenKnoten = streckenCache
						.getStartStrassenKnoten(strasse, richtung);
				if (startStrassenKnoten != null
						&& !startStrassenKnoten.isEmpty()) {
					neuerWert = startStrassenKnoten.get(0);
				}
			}
		}
		final StrassenKnoten alterWert = startKnoten;
		startKnoten = neuerWert;

		// berprfe, ob der Endknoten noch gltig (ein Nachfolgeknoten des
		// Startknotens) ist.
		final StrassenKnoten alterEndKnoten = endKnoten;
		StrassenKnoten neuerEndKnoten = null;
		final List<StrassenKnoten> nachfolgerKnoten = streckenCache
				.getNachfolgerStrassenKnoten(strasse, richtung,
						neuerStartKnoten);
		nachfolgerKnoten.add(0, neuerStartKnoten);
		if (endKnoten != null && !nachfolgerKnoten.contains(endKnoten)
				&& !nachfolgerKnoten.isEmpty()) {
			setEndKnoten(nachfolgerKnoten.get(0), false);
			neuerEndKnoten = endKnoten;
		}

		if (fireEvent) {
			firePropertyChange("startKnoten", alterWert, neuerWert);
		}
		if (neuerEndKnoten != null) {
			firePropertyChange("endKnoten", alterEndKnoten, neuerEndKnoten);
		}

		// Abschlieend aktualisiere den Streckenabschnitt.
		setStreckenAbschnitt(true);
	}

	/**
	 * Liefert den Endstraenknoten dieser Engstelle zurck.
	 * 
	 * @return den Endstraenknoten dieser Engstelle
	 */
	public StrassenKnoten getEndKnoten() {
		return endKnoten;
	}

	/**
	 * Setzt den Endstraenknoten dieser Engstelle.
	 * 
	 * @param endKnoten
	 *            der neue Endstraenknoten
	 */
	public void setEndKnoten(final StrassenKnoten endKnoten) {
		setEndKnoten(endKnoten, true);
	}

	/**
	 * Setzt den Endstraenknoten dieser Engstelle.
	 * 
	 * @param neuerEndKnoten
	 *            der neue Endstraenknoten
	 * @param fireEvent
	 *            <code>true</code>, wenn das dazugehrige PropertyChangeEvent
	 *            gefeuert werden soll
	 */
	private void setEndKnoten(final StrassenKnoten neuerEndKnoten,
			final boolean fireEvent) {

		if (neuerEndKnoten != null && endKnoten == neuerEndKnoten) {
			return;
		}

		// Setze neuen Endknoten. Wenn dieser null ist, nimm den ersten in der
		// Liste mglicher Endknoten.
		StrassenKnoten neuerWert = neuerEndKnoten;
		if (neuerWert == null) {
			if (strasse != null && richtung != null && startKnoten != null) {
				final List<StrassenKnoten> endStrassenKnoten = streckenCache
						.getNachfolgerStrassenKnoten(strasse, richtung,
								startKnoten);
				if (endStrassenKnoten != null && !endStrassenKnoten.isEmpty()) {
					neuerWert = endStrassenKnoten.get(0);
				}
			}
		}
		final StrassenKnoten alterWert = endKnoten;
		endKnoten = neuerWert;

		// berprfe, ob der Startknoten noch gltig (ein Vorgngerknoten des
		// neuen Endknotens) ist.
		final StrassenKnoten alterStartKnoten = startKnoten;
		StrassenKnoten neuerStartKnoten = null;
		final List<StrassenKnoten> vorgaengerKnoten = streckenCache
				.getVorgaengerStrassenKnoten(strasse, richtung, neuerEndKnoten);
		vorgaengerKnoten.add(neuerEndKnoten);
		if (neuerEndKnoten != null && !vorgaengerKnoten.contains(startKnoten)
				&& !vorgaengerKnoten.isEmpty()) {
			setStartKnoten(vorgaengerKnoten.get(0), false);
			neuerStartKnoten = startKnoten;
		}

		if (fireEvent) {
			firePropertyChange("endKnoten", alterWert, neuerWert);
		}

		if (neuerStartKnoten != null) {
			firePropertyChange("startKnoten", alterStartKnoten,
					neuerStartKnoten);
		}

		// Abschlieend aktualisiere den Streckenabschnitt.
		setStreckenAbschnitt(true);
	}

	/**
	 * Setzt ein alternatives ueres Straensegment.
	 * 
	 * @param alternativesSegment
	 *            das alternative uere Straensegment
	 */
	public void setAlternativesAeuesseresStrassenSegment(
			final AeusseresStrassenSegment alternativesSegment) {

		if (streckenAbschnitt.getAeussereStrassenSegmente().contains(
				alternativesSegment)) {
			return;
		}

		boolean bestehendesSegmentGefunden = false;
		final Daten alternativesDatum = alternativesSegment
				.getKdAeusseresStrassenSegment().getDatum();
		final List<StrassenSegment> strassenSegmente = new ArrayList<StrassenSegment>(
				getStreckenAbschnitt().getStrassenSegmente());
		int index = 0;
		for (final StrassenSegment segment : strassenSegmente) {
			if (segment instanceof AeusseresStrassenSegment) {
				final AeusseresStrassenSegment aeusseresSegment = (AeusseresStrassenSegment) segment;
				final Daten aktuellesDatum = aeusseresSegment
						.getKdAeusseresStrassenSegment().getDatum();

				if (alternativesDatum.getVonKnoten().getPid()
						.equals(aktuellesDatum.getVonKnoten().getPid())
						&& alternativesDatum
								.getNachKnoten()
								.getPid()
								.equals(aktuellesDatum.getNachKnoten().getPid())) {
					bestehendesSegmentGefunden = true;
					break;
				}
			}
			index++;
		}
		if (!bestehendesSegmentGefunden) {
			return;
		}
		strassenSegmente.set(index, alternativesSegment);

		// Finde Streckenabschnitt, welcher die Segmente enthlt
		final StreckenAbschnitt alterWert = streckenAbschnitt;
		List<StreckenAbschnitt> streckenAbschnitte = streckenCache
				.getStreckenAbschnitte(strasse, richtung, strassenSegmente);

		// Wenn kein Streckenabschnitt gefunden werden konnte, versuche es
		// erneut ohne die Nachfolger- und Vorgnger-Straensegmente
		final List<StrassenSegment> benachbarteSegmente = new ArrayList<StrassenSegment>();
		for (int i = 1; i <= 3; i++) {
			addBenachbartesSegment(benachbarteSegmente, strassenSegmente, index
					+ i);
			addBenachbartesSegment(benachbarteSegmente, strassenSegmente, index
					- i);
		}
		for (final StrassenSegment nachbarSegment : benachbarteSegmente) {
			strassenSegmente.remove(nachbarSegment);
			streckenAbschnitte = streckenCache.getStreckenAbschnitte(strasse,
					richtung, strassenSegmente);
			if (!streckenAbschnitte.isEmpty()) {
				break;
			}
		}

		// Setze den neuen Streckenabschnitt
		if (!streckenAbschnitte.isEmpty()) {
			final StreckenAbschnitt neuerWert = streckenAbschnitte.get(0)
					.clone();
			if (!neuerWert.getStrassenKnoten().contains(startKnoten)
					|| !neuerWert.getStrassenKnoten().contains(endKnoten)) {
				return;
			}
			neuerWert.setEndKnoten(endKnoten);
			neuerWert.setStartKnoten(startKnoten);

			if (!alterWert.getStartASS().getPid()
					.equals(neuerWert.getStartASS().getPid())) {
				neuerWert.setStandardStartBetriebsKilometerBlockNummer();
				neuerWert.setStartBetriebsKilometerBereich();
				neuerWert.setMaxStartOffset();
				neuerWert.setStartOffset(alterWert.getStartOffset());
			}
			if (!alterWert.getEndASS().getPid()
					.equals(neuerWert.getEndASS().getPid())) {
				neuerWert.setStandardEndBetriebsKilometerBlockNummer();
				neuerWert.setEndBetriebsKilometerBereich();
				neuerWert.setMaxEndOffset();
				neuerWert.setEndOffset(alterWert.getEndOffset());
			}
			neuerWert.setAnzahlFahrstreifen();

			streckenAbschnitt = neuerWert;
			firePropertyChange("streckenAbschnitt", alterWert,
					streckenAbschnitt);

			fuegeStreckenAbschnittListenerHinzu();
		}
	}

	/**
	 * Hilfsmethode zum sukzessiven Erstellen einer Liste von Straensegmenten,
	 * welche ggf. entfernt werden mssen, um den passenden Streckenabschnitt
	 * beim Setzen eines alternativen ueren Straensegments zu finden.
	 * 
	 * @param benachbarteSegmente
	 *            die Ergebnisliste der benachbarten Straensegmente (Collecting
	 *            Parameter Pattern)
	 * @param strassenSegmente
	 *            die Liste aller Straensegmente
	 * @param index
	 *            der Index des Segments, welches zu der Liste der benachbarten
	 *            Straensegmente hinzugefgt werden soll
	 */
	private void addBenachbartesSegment(
			final List<StrassenSegment> benachbarteSegmente,
			final List<StrassenSegment> strassenSegmente, final int index) {
		if (index >= 0 && index < strassenSegmente.size()) {
			benachbarteSegmente.add(strassenSegmente.get(index));
		}
	}

	/**
	 * Liefert den Streckenabschnitt dieser Engstelle zurck.
	 * 
	 * @return den Streckenabschnitt dieser Engstelle
	 */
	public StreckenAbschnitt getStreckenAbschnitt() {
		return streckenAbschnitt;
	}

	/**
	 * Setzt den Streckenabschnitt dieser Engstelle.
	 * 
	 * @param aktualisiereListener
	 *            <code>true</code>, wenn die entsprechenden
	 *            PropertyChangeListener fr die Offsets aktualisiert werden
	 *            sollen, ansonsten <code>false</code>
	 */
	private void setStreckenAbschnitt(final boolean aktualisiereListener) {
		StreckenAbschnitt neuerWert = null;
		if (strasse != null && richtung != null) {
			if (startKnoten != null && endKnoten != null) {
				neuerWert = streckenCache.getStandardStreckenAbschnitt(strasse,
						richtung, startKnoten, endKnoten);
			}
			if (neuerWert == null) {
				neuerWert = streckenCache.getStandardStreckenAbschnitt(strasse,
						richtung);
			}
		}
		final StreckenAbschnitt alterWert = streckenAbschnitt;
		streckenAbschnitt = neuerWert;
		firePropertyChange("streckenAbschnitt", alterWert, neuerWert);
		if (aktualisiereListener) {
			fuegeStreckenAbschnittListenerHinzu();
		}
	}

	/**
	 * Ermittelt fr eine gegebene Liste von ueren Straensegmenten den
	 * Startstraenknoten.
	 * 
	 * @param segmente
	 *            die Liste der ueren Straensegmente
	 * 
	 * @return den fr eine gegebene Liste von ueren Straensegmenten
	 *         ermittelten Startstraenknoten
	 */
	private StrassenKnoten ermittelStartStrassenKnoten(
			final List<StrassenSegment> segmente) {
		if (segmente == null || segmente.isEmpty()) {
			return null;
		}

		final StrassenSegment segment = segmente.get(0);
		if (segment instanceof InneresStrassenSegment && segmente.size() == 1) {
			return netzCache.getNetzCache().getStrassenKnoten(
					(InneresStrassenSegment) segment);
		}

		AeusseresStrassenSegment ass = null;
		if (segment instanceof InneresStrassenSegment) {
			ass = ((InneresStrassenSegment) segment)
					.getKdInneresStrassenSegment().getDatum()
					.getVonStrassenSegment();
		}
		if (segment instanceof AeusseresStrassenSegment) {
			ass = (AeusseresStrassenSegment) segment;
		}
		if (ass == null) {
			return null;
		}
		return ass.getKdAeusseresStrassenSegment().getDatum().getVonKnoten();
	}

	/**
	 * Ermittelt fr eine gegebene Liste von ueren Straensegmenten den
	 * Endstraenknoten.
	 * 
	 * @param segmente
	 *            die Liste der ueren Straensegmente
	 * 
	 * @return den fr eine gegebene Liste von ueren Straensegmenten
	 *         ermittelten Endstraenknoten
	 */
	private StrassenKnoten ermittelEndStrassenKnoten(
			final List<StrassenSegment> segmente) {
		if (segmente == null || segmente.isEmpty()) {
			return null;
		}
		final StrassenSegment segment = segmente.get(segmente.size() - 1);

		if (segment instanceof InneresStrassenSegment) {
			return netzCache.getNetzCache().getStrassenKnoten(
					(InneresStrassenSegment) segment);
		}

		if (segment instanceof AeusseresStrassenSegment) {
			return ((AeusseresStrassenSegment) segment)
					.getKdAeusseresStrassenSegment().getDatum().getNachKnoten();
		}
		return null;
	}

	/**
	 * Setzt den Zeitstempel fr diese Engstelle.
	 * 
	 * @param neuerWert
	 *            der neue Zeitstempel
	 */
	protected void setZeitStempel(final Zeitstempel neuerWert) {
		if (neuerWert == null) {
			return;
		}
		if (zeitStempel == null || zeitStempel.getTime() < neuerWert.getTime()) {
			zeitStempel = neuerWert;
		}
	}

	/**
	 * Liefert den aktuellste Zeitstempel der diesem Wrapper zugrundeliegenden
	 * Daten zurck.
	 * 
	 * @return der aktuellste Zeitstempel der diesem Wrapper zugrundeliegenden
	 *         Daten
	 */
	public Zeitstempel getZeitStempel() {
		return zeitStempel;
	}

	/**
	 * Sendet das gegebene Datum fr den ParameterDatenSatz.
	 * 
	 * @param datenSatz
	 *            der Parameterdatensatz
	 * @param datum
	 *            das Datum
	 * 
	 * @return ein Status-Objekt, welches ber den Erfolg dieser Methode
	 *         Auskunft gibt
	 */
	@SuppressWarnings("unchecked")
	protected IStatus sendeDaten(final AbstractParameterDatensatz datenSatz,
			final AbstractOnlineDatum datum) {

		try {

			datenSatz.anmeldenSender();

			final int timePause = 100;
			int timeElapsed = 0;
			boolean sendeDatum = false;
			while (timeElapsed < 10000) {
				final de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatensatz.Status statusSendesteuerung = datenSatz
						.getStatusSendesteuerung();
				if (statusSendesteuerung == de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatensatz.Status.START) {
					sendeDatum = true;
					break;
				}
				Thread.sleep(timePause);
				timeElapsed += timePause;
			}

			if (sendeDatum) {
				datenSatz.sendeDatum(datum);
			} else {
				datenSatz.abmeldenSender();
				throw new CancellationException(
						"TimeOut beim Senden des folgenden Datensatz: "
								+ datenSatz.getPid());
			}

			datenSatz.abmeldenSender();

		} catch (final AnmeldeException e) {
			BauEditorPlugin.getDefault().getLogger()
					.error(e.getLocalizedMessage(), e);
			return new Status(IStatus.ERROR, BauEditorPlugin.PLUGIN_ID,
					IStatus.OK, e.getLocalizedMessage(), e);
		} catch (final CancellationException e) {
			BauEditorPlugin.getDefault().getLogger()
					.error(e.getLocalizedMessage(), e);
			return new Status(IStatus.ERROR, BauEditorPlugin.PLUGIN_ID,
					IStatus.OK, e.getLocalizedMessage(), e);
		} catch (final DatensendeException e) {
			BauEditorPlugin.getDefault().getLogger()
					.error(e.getLocalizedMessage(), e);
			return new Status(IStatus.ERROR, BauEditorPlugin.PLUGIN_ID,
					IStatus.OK, e.getLocalizedMessage(), e);
		} catch (final InterruptedException e) {
			BauEditorPlugin.getDefault().getLogger()
					.error(e.getLocalizedMessage(), e);
			return new Status(IStatus.ERROR, BauEditorPlugin.PLUGIN_ID,
					IStatus.OK, e.getLocalizedMessage(), e);
		}

		return Status.OK_STATUS;
	}

	/**
	 * Fgt diesem Wrapper einen neuen PropertyChangeListener hinzu.
	 * 
	 * @param propertyName
	 *            der Name der Property
	 * @param listener
	 *            der Listener
	 * 
	 * @see PropertyChangeSupport#addPropertyChangeListener(String,
	 *      PropertyChangeListener)
	 */
	public void addPropertyChangeListener(final String propertyName,
			final PropertyChangeListener listener) {
		if (!Arrays.asList(
				changeSupport.getPropertyChangeListeners(propertyName))
				.contains(listener)) {
			changeSupport.addPropertyChangeListener(propertyName, listener);
		}
	}

	/**
	 * Entfernt den gegebenen PropertyChangeListener von diesem Wrapper.
	 * 
	 * @param propertyName
	 *            der Name der Property
	 * @param listener
	 *            der Listener
	 * 
	 * @see PropertyChangeSupport#removePropertyChangeListener(String,
	 *      PropertyChangeListener)
	 */
	public void removePropertyChangeListener(final String propertyName,
			final PropertyChangeListener listener) {
		changeSupport.removePropertyChangeListener(propertyName, listener);
	}

	/**
	 * Informiert alle registrierten PropertyChangeListener ber eine nderung.
	 * 
	 * @param propertyName
	 *            der Name der Property
	 * @param oldValue
	 *            der alter Wert
	 * @param newValue
	 *            der neue Wert
	 * 
	 * @see PropertyChangeSupport#firePropertyChange(String, Object, Object)
	 */
	protected void firePropertyChange(final String propertyName,
			final Object oldValue, final Object newValue) {
		changeSupport.firePropertyChange(propertyName, oldValue, newValue);
		if (!"dirty".equals(propertyName)
				&& !"ereignisTyp".equals(propertyName) && oldValue != null
				&& !oldValue.equals(newValue)) {
			setDirty(true);
		}
	}

	/** {@inheritDoc} */
	@Override
	public String toString() {
		return name;
	}

	/**
	 * Liefert einen Info-String zurck.
	 * 
	 * @return einen Info-String
	 */
	public String getInfo() {
		final StringBuffer buf = new StringBuffer();

		buf.append("\nName: ");
		buf.append(name);
		buf.append("\nStrasse: ");
		buf.append(strasse);
		buf.append("\nRichtung: ");
		buf.append(richtung);
		buf.append("\nStartknoten: ");
		buf.append(startKnoten);
		buf.append("\nEndknoten: ");
		buf.append(endKnoten);
		buf.append("\nKurzinfo: ");
		buf.append(kurzInfo);
		buf.append("\nBeschreibung: ");
		buf.append(beschreibung);
		buf.append("\nZeitspanne: ");
		buf.append(startZeit);
		buf.append(" bis ");
		buf.append(endZeit);
		buf.append(")");
		if (dauer == 0) {
			buf.append("\nUnbekannte Endzeit");
		} else {
			buf.append("\nBekannte Endzeit");
		}
		if (streckenAbschnitt != null) {
			buf.append("\nStart-Offset: ");
			buf.append(streckenAbschnitt.getStartOffset());
			buf.append("\nMax-Start-Offset: ");
			buf.append(streckenAbschnitt.getMaxStartOffset());
			buf.append("\nEnd-Offset: ");
			buf.append(streckenAbschnitt.getEndOffset());
			buf.append("\nMax-End-Offset: ");
			buf.append(streckenAbschnitt.getMaxEndOffset());
			buf.append("\nAnzahl Fahrstreifen: ");
			buf.append(streckenAbschnitt.getAnzahlFahrstreifen());
		}
		return buf.toString();
	}

	/** {@inheritDoc} */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result;
		if (situation != null) {
			result += situation.getPid().hashCode();
		}
		return result;
	}

	/** {@inheritDoc} */
	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final SituationsWrapper other = (SituationsWrapper) obj;
		if (situation == null || other.situation == null) {
			return false;
		}
		return situation.getPid().equals(other.situation.getPid());
	}
}
