/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.util.wrapper;

import de.bsvrz.buv.plugin.baueditor.sts.BauEditorException;
import de.bsvrz.sys.funclib.bitctrl.modell.tmtmcglobal.attribute.AttTmcRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Strasse;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.StrassenSegment;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Unfall;

/**
 * Wrapper, welcher fr ein Unfall-SystemObjekt smtliche Daten vorhlt.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: UnfallWrapper.java,v 1.2 2014/11/14 07:29:42 Pittner Exp $
 * 
 * @see Unfall
 */
public class UnfallWrapper extends EngstellenWrapper {

	/**
	 * Der Konstruktor.
	 * 
	 * @param strasse
	 *            die Strae, kann <code>null</code> sein
	 * @param richtung
	 *            die Richtung, kann <code>null</code> sein
	 * @param segment
	 *            das Straensegment, kann <code>null</code> sein
	 */
	public UnfallWrapper(final Strasse strasse, final AttTmcRichtung richtung,
			final StrassenSegment segment) {
		super(strasse, richtung, segment);
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param unfall
	 *            ein Unfall-SystemObjekt
	 * 
	 * @throws BauEditorException
	 *             wird geworfen, wenn der gegebene Unfall nicht geladen werden
	 *             konnte
	 */
	public UnfallWrapper(final Unfall unfall) throws BauEditorException {
		super(unfall);
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param wrapper
	 *            ein anderer UnfallWrapper
	 */
	public UnfallWrapper(final UnfallWrapper wrapper) {
		this(wrapper, false);
	}

	/**
	 * Der Konstruktor.
	 * 
	 * @param wrapper
	 *            ein anderer UnfallWrapper
	 * @param neuesElement
	 *            <code>true</code>, wenn es sich bei dem neu erstellten
	 *            UnfallWrapper, um einen neuen Unfall handelt. false, wenn ein
	 *            Klon erstellt werden soll.
	 */
	public UnfallWrapper(final UnfallWrapper wrapper, final boolean neuesElement) {
		super(wrapper, neuesElement);
	}

	@Override
	public String toString() {
		return getName() + " (Unfall)";
	}
}
