/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.views;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.forms.widgets.Form;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.progress.UIJob;

import de.bsvrz.buv.plugin.baueditor.sts.BauEditorEventListener;
import de.bsvrz.buv.plugin.baueditor.sts.BauEditorPlugin;
import de.bsvrz.buv.plugin.baueditor.sts.Zeichenketten;
import de.bsvrz.buv.plugin.baueditor.sts.util.EngstellenCache;
import de.bsvrz.buv.plugin.baueditor.sts.util.listener.EngstellenChangedListener;
import de.bsvrz.buv.plugin.baueditor.sts.util.wrapper.EngstellenWrapper;
import de.bsvrz.buv.plugin.selektion.SelektionsListener;
import de.bsvrz.buv.plugin.selektion.SelektionsTransfer;
import de.bsvrz.buv.rw.basislib.seiteneinstellungen.druck.vorschau.IDruckvorschau;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Baustelle;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Unfall;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.AbstractCache;

/**
 * Abstrakte Superklasse smtlicher ViewParts dieses Plug-ins. Behandelt die
 * Anzeige eines Fehler-Textes, wenn das Plug-in nicht ausfhrbar ist.
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: AbstractBauEditorViewPart.java 28516 2011-02-15 12:41:19Z
 *          peuker $
 */
public abstract class AbstractBauEditorViewPart extends ViewPart implements
		EngstellenChangedListener, IDruckvorschau, SelektionsListener,
		IViewPartWithCache, BauEditorEventListener {

	/**
	 * Der CacheListener, welcher ber den Status der Caches informiert.
	 */
	private CacheListener cacheListener;

	/** Der Engstellen-Cache. */
	private EngstellenCache cache;

	/**
	 * PartListener, welcher beim Schlieen der View das Laden der Einstellungen
	 * initiiert.
	 */
	private final IPartListener partListener = new IPartListener() {

		@Override
		public void partOpened(final IWorkbenchPart part) {
			// tut nix
		}

		@Override
		public void partDeactivated(final IWorkbenchPart part) {
			// tut nix
		}

		@Override
		public void partClosed(final IWorkbenchPart part) {
			if (part == AbstractBauEditorViewPart.this) {
				speicherEinstellungen();
			}
		}

		@Override
		public void partBroughtToTop(final IWorkbenchPart part) {
			// tut nix
		}

		@Override
		public void partActivated(final IWorkbenchPart part) {
			// tut nix
		}
	};

	@Override
	public void init(final IViewSite site, final IMemento mem)
			throws PartInitException {
		super.init(site, mem);
		getSite().getPage().addPartListener(partListener);
		ladeEinstellungen();
	}

	/**
	 * Erzeugt das PartControl der Ansicht.
	 * 
	 * @param parent
	 *            das Parent-Composite
	 */
	protected abstract void erzeugePartControl(Composite parent);

	@Override
	public final void createPartControl(final Composite parent) {
		if (StartupPlugin.getDefault().getDefaultNetz() != null) {
			erzeugePartControl(parent);
			BauEditorPlugin.getDefault().addEventListener(this);
			if (BauEditorPlugin.getDefault().istOnline()) {
				goOnline();
			} else {
				goOffline();
			}
		} else {
			final FormToolkit toolkit = new FormToolkit(parent.getDisplay());
			final Form form = toolkit.createForm(parent);
			form.setMessage(
					"Das mit dem Startparameter \"-verkehrsModellObjekt\" referenzierte Objekt\n vom Typ typ.verkehrsModellNetz konnte nicht gefunden werden.",
					IMessageProvider.ERROR);
		}

	}

	/**
	 * Initialisiert die Ansicht im Online-Modus.
	 */
	private void goOnline() {
		if (getCache() != null) {
			if (!getCache().isInitialisiert()) {
				setContentDescription(Zeichenketten.LADE_ENGSTELLEN + "...");
				preInitInUIThread();
			} else {
				initInUIThread();
				if (!BauEditorPlugin.getDefault().isAusfuehrbar()) {
					setEnablement(false);
				}
			}
			cacheListener = new CacheListener(this);
			getCache().addPropertyChangeListener(AbstractCache.PROP_STATE,
					cacheListener);
		}
	}

	/**
	 * Initialisiert die Ansicht im Offline-Modus.
	 */
	private void goOffline() {

		// Leere und deaktiviere Ansicht
		clearViewer();
		setEnablement(false);

		// Setze Content-Description
		final String fehlerMeldung = (BauEditorPlugin.getDefault()
				.getObjektFactory() == null) ? "Diese Ansicht kann nicht angezeigt werden (ObjektFactory fehlt)!"
				: "Diese Ansicht ist im Offline-Modus nicht verfgbar!";
		setContentDescription(fehlerMeldung);

		// Entferne Men- und Symbolleisteneintrge
		final IActionBars actionBars = getViewSite().getActionBars();
		actionBars.getMenuManager().removeAll();
		actionBars.getToolBarManager().removeAll();
		actionBars.updateActionBars();
	}

	/**
	 * Entfernt den gesamten Inhalt aus dem Viewer.
	 */
	protected void clearViewer() {
		// tut nix, kann berschrieben werden
	}

	/**
	 * Aktiviert bzw. deaktivert diese Ansicht.
	 * 
	 * @param enabled
	 *            <code>true</code>, wenn diese Ansicht aktiviert sein soll,
	 *            ansonsten <code>false</code>
	 */
	protected abstract void setEnablement(boolean enabled);

	/**
	 * Speichert die Einstellungen dieser Ansicht.
	 */
	protected abstract void speicherEinstellungen();

	/**
	 * Ldt die Einstellungen dieser Ansicht.
	 */
	protected abstract void ladeEinstellungen();

	/**
	 * Liefert den mglicherweise in der Selektion enthaltenen
	 * {@link EngstellenWrapper} zurck.
	 * 
	 * @param selection
	 *            die Selektion
	 * 
	 * @return den mglicherweise in der Selektion enthaltenen
	 *         {@link EngstellenWrapper}
	 */
	protected EngstellenWrapper getEngstellenWrapper(final ISelection selection) {
		if (selection != null && selection instanceof IStructuredSelection
				&& !selection.isEmpty()) {

			final Object first = ((IStructuredSelection) selection)
					.getFirstElement();

			if (first instanceof EngstellenWrapper) {
				return (EngstellenWrapper) first;
			}

			SystemObjekt systemObjekt = (SystemObjekt) Platform
					.getAdapterManager().getAdapter(first, SystemObjekt.class);

			if (systemObjekt == null) {
				final SystemObject systemObject = (SystemObject) Platform
						.getAdapterManager().getAdapter(first,
								SystemObject.class);
				if (systemObject != null) {
					systemObjekt = BauEditorPlugin.getDefault()
							.getObjektFactory().getModellobjekt(systemObject);
				}
			}

			if (systemObjekt != null) {
				EngstellenWrapper ergebnis = null;
				if (systemObjekt instanceof Baustelle) {
					ergebnis = getCache().getBaustellenWrapper(
							(Baustelle) systemObjekt);
				} else if (systemObjekt instanceof Unfall) {
					ergebnis = getCache().getUnfallWrapper(
							(Unfall) systemObjekt);
				}
				return ergebnis;
			}
		}
		return null;
	}

	/**
	 * Liefert den Engstellen-Cache zurck, oder <code>null</code>, falls kein
	 * Engstellen-Cache erzeugt werden konnte.
	 * 
	 * @return den Engstellen-Cache
	 */
	protected EngstellenCache getCache() {
		if (BauEditorPlugin.getDefault().istOnline() && cache == null) {
			cache = BauEditorPlugin.getDefault().getEngstellenCache();
		}
		return cache;
	}

	@Override
	public boolean aktuelleSelektionBeobachten() {
		return true;
	}

	@Override
	public boolean definierteSelektionenBeobachten(final String... namen) {
		return false;
	}

	@Override
	public boolean vorigeSelektionBeobachten() {
		return false;
	}

	@Override
	public void dispose() {
		getSite().getPage().removePartListener(partListener);
		if (cache != null) {
			cache.removeEngstellenListener(this);
		}
		SelektionsTransfer.getInstanz().removeSelektionsListener(this);
		BauEditorPlugin.getDefault().removeEventListener(this);
		if (cache != null) {
			cache.removePropertyChangeListener(AbstractCache.PROP_STATE,
					cacheListener);
		}
		super.dispose();
	}

	@Override
	public void switchOnline(final boolean online) {
		if (online) {
			goOnline();
		} else {
			goOffline();
		}
	}

	/**
	 * Ein Listener, welcher ber den aktuellen Status von Caches informiert.
	 * 
	 * @author BitCtrl Systems GmbH, Andreas Meissner
	 * @version $Id: AbstractBauEditorViewPart.java 28516 2011-02-15 12:41:19Z
	 *          peuker $
	 */
	private class CacheListener implements PropertyChangeListener {

		/**
		 * Der ViewPart, welche ber nderungen des Caches benachrichtigt werden
		 * soll.
		 */
		private final IViewPartWithCache viewPart;

		/**
		 * Der Konstruktor.
		 * 
		 * @param viewPart
		 *            der ViewPart, welche ber nderungen des Caches
		 *            benachrichtigt werden soll, darf nicht <code>null</code>
		 *            sein
		 */
		public CacheListener(final IViewPartWithCache viewPart) {
			Assert.isNotNull(viewPart);
			this.viewPart = viewPart;
		}

		@Override
		public void propertyChange(final PropertyChangeEvent evt) {
			final boolean initialisiert = AbstractCache.PROP_STATE_INITIALIZED
					.equals(evt.getNewValue());
			final boolean started = AbstractCache.PROP_STATE_STARTED.equals(evt
					.getNewValue());

			if (initialisiert || started) {
				final UIJob uiJob = new UIJob("") {
					@Override
					public IStatus runInUIThread(final IProgressMonitor monitor) {
						if (started) {
							viewPart.preInitInUIThread();
						} else if (initialisiert) {
							viewPart.initInUIThread();
						}
						return Status.OK_STATUS;
					}
				};
				uiJob.schedule();
			}
		}
	}
}
