/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/*
 * Rahmenwerk-Plug-in "Baustellen- und Unfalleingabe"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.baueditor.sts.views.helper;

import java.util.HashMap;
import java.util.Map;

import de.bsvrz.buv.plugin.baueditor.sts.BauEditorPlugin;
import de.bsvrz.buv.plugin.baueditor.sts.BildDateiPfade;
import de.bsvrz.buv.rw.bitctrl.eclipse.databinding.RichtungTextComputedValue;
import de.bsvrz.sys.funclib.bitctrl.modell.tmtmcglobal.attribute.AttTmcRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.tmverkehrglobal.objekte.Strasse;

/**
 * Hilfsklasse fr Richtungsknoten der Ansicht "Engstellen-Navigator".
 * 
 * @author BitCtrl Systems GmbH, Andreas Meissner
 * @version $Id: RichtungHelper.java,v 1.2 2014/11/14 07:29:42 Pittner Exp $
 */
public final class RichtungHelper extends Helper {

	/** Das Richtung-Objekt, welches durch diesen Helper gekapselt wird. */
	private final AttTmcRichtung richtung;

	/**
	 * Map, welche zu jeder Strae (und Richtung) den dazugehrigen
	 * RichtungHelper speichert.
	 */
	private static Map<StrassenHelper, Map<AttTmcRichtung, RichtungHelper>> map;

	/**
	 * Liefert ein Richtungsobjekt fr die Ansicht "Engstellen-Navigator"
	 * zurck.
	 * 
	 * @param istBaustelle
	 *            <code>true</code>, wenn es sich um einen Elternknoten einer
	 *            Baustelle handelt, ansonsten false
	 * @param strasse
	 *            die Strasse
	 * @param richtung
	 *            die Richtung
	 * 
	 * @return ein Richtungsobjekt fr die Ansicht "Engstellen-Navigator"
	 */
	public static RichtungHelper getInstanz(final boolean istBaustelle,
			final Strasse strasse, final AttTmcRichtung richtung) {

		Object grandParent = UnfallHelper.getInstanz(null);
		if (istBaustelle) {
			grandParent = BaustellenHelper.getInstanz(null);
		}
		return getInstanz(StrassenHelper.getInstanz(grandParent, strasse),
				richtung);
	}

	/**
	 * Liefert ein Richtungsobjekt fr die Ansicht "Engstellen-Navigator"
	 * zurck.
	 * 
	 * @param parent
	 *            das Parent-Objekt
	 * @param richtung
	 *            die Richtung
	 * 
	 * @return ein Richtungsobjekt fr die Ansicht "Engstellen-Navigator"
	 */
	public static RichtungHelper getInstanz(final StrassenHelper parent,
			final AttTmcRichtung richtung) {
		if (map == null) {
			map = new HashMap<StrassenHelper, Map<AttTmcRichtung, RichtungHelper>>();
		}
		Map<AttTmcRichtung, RichtungHelper> helperMap = map.get(parent);
		if (helperMap == null) {
			helperMap = new HashMap<AttTmcRichtung, RichtungHelper>();
			map.put(parent, helperMap);
		}

		RichtungHelper helper = helperMap.get(richtung);
		if (helper == null) {
			helper = new RichtungHelper(parent, richtung);
			helperMap.put(richtung, helper);
		}
		return helper;
	}

	/**
	 * Der private Konstruktor.
	 * 
	 * @param parent
	 *            das Parent-Objekt
	 * @param richtung
	 *            die Richtung, welche von diesem Helper gekapselt wird
	 */
	private RichtungHelper(final StrassenHelper parent,
			final AttTmcRichtung richtung) {
		super(parent);
		this.richtung = richtung;
	}

	/**
	 * Liefert die Strae dieses Helpers zurck.
	 * 
	 * @return die Strae
	 */
	public Strasse getStrasse() {
		return ((StrassenHelper) getParent()).getStrasse();
	}

	/**
	 * Liefert die Richtung dieses Helpers zurck.
	 * 
	 * @return die Richtung
	 */
	public AttTmcRichtung getRichtung() {
		return richtung;
	}

	@Override
	public String getImagePath() {
		if (richtung == AttTmcRichtung.ZUSTAND_1_POSITIV) {
			return BildDateiPfade.RICHTUNG_POSITIV;
		}
		if (richtung == AttTmcRichtung.ZUSTAND_1N_NEGATIV) {
			return BildDateiPfade.RICHTUNG_NEGATIV;
		}
		return null;
	}

	@Override
	public String toString() {
		final String richtungText = RichtungTextComputedValue.getRichtungText(
				BauEditorPlugin.getDefault().getNetzCache(), getStrasse(),
				richtung);
		if (richtung != null) {
			return richtungText;
		}
		return "unbekannt";
	}
}
