/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.plugin.pua.basis.extension.ExtensionVerwaltung;
import de.bsvrz.buv.plugin.pua.basis.extension.SkriptInfo;
import de.bsvrz.buv.plugin.pua.basis.pua.ZugriffPua;
import de.bsvrz.buv.plugin.pua.basis.pua.daten.ProtokollInformationen;
import de.bsvrz.buv.plugin.pua.basis.puaDataUpdateListener.PuADataUpdateListener;
import de.bsvrz.buv.plugin.pua.basis.views.ProtokollierungView.ZeitGruppenZeile;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.pua.prot.client.ProtocolResultStream;
import de.bsvrz.pua.prot.client.dataobject.ProtocolResult;
import de.bsvrz.pua.prot.util.ProcessingParameter;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;
import de.bsvrz.sys.funclib.losb.util.Tuple;

/**
 * Klasse realisiert einen Thread zur Entkopplung des Zugriffs auf die PUA
 * 
 * @author Pittner
 * 
 */
public class AnfrageThread implements Runnable
{

  private ClientDavInterface _connection;

//  private int _selSize = -1;

  private TableViewer _tvBetriebsMeldungen2;

//  private List<SystemObject> _input;

  private List<ZeitGruppenZeile> _listeZeitGruppenZeilen;

  private ZugriffPua _zugriffPua;

  private Integer _numberInstances;

  private ArchiveDataKind[] _dataKinds;

  private boolean _speichern;
  
  private boolean _oeffentlich;
  
  private String _informationsText;
  
  private boolean _aenderungsProtokoll; 
  
  private String _ersteller;

  private boolean _sofort;

  private int _simVar;

  private ProtokollInformationen _protokollInformationen;

  private boolean _mitProtokollInformationen = false;
  
  private Map<String, String> _bindungen = new HashMap<String, String>();

//  private String _skript = null;
  
  private SkriptInfo _skriptInfo;

  /**
   * Konstruktor
   * 
   * @param connection
   * @param tableViewer
   * @param liste
   * @param dataKinds
   * @param speichern
   * @param oeffentlich
   * @param informationsText
   * @param ersteller
   * @param sofort
   * @param simuVariante
   * @param secViewId
   */
//  public AnfrageThread(ClientDavInterface connection, TableViewer tableViewer, List<ZeitGruppenZeile> liste,
//      ArchiveDataKind[] dataKinds, boolean speichern, boolean oeffentlich, String informationsText, String ersteller, boolean sofort, int simuVariante, Integer secViewId)
//  {
//    _connection = connection;
//    _tvBetriebsMeldungen2 = tableViewer;
//    _listeZeitGruppenZeilen = liste;
//    _zugriffPua = ZugriffPua.getInstanz(DaVKommunikation.getInstanz().holeDaVVerbindung());
//    _numberInstances = secViewId;
//    _dataKinds = dataKinds;
//    _speichern = speichern;
//    _oeffentlich = oeffentlich;
//    _informationsText = informationsText;
//    _ersteller = ersteller;
//    _sofort = sofort;
//    _simVar = simuVariante;
//
//    Thread t = new Thread(this);
//    t.start();
//  }

  /**
   * Konstruktor
   * 
   * @param connection
   * @param tableViewer
   * @param liste
   * @param dataKinds
   * @param speichern
   * @param oeffentlich
   * @param informationsText
   * @param ersteller
   * @param sofort
   * @param simuVariante
   * @param secViewId
   * @param skript
   */
  public AnfrageThread(ClientDavInterface connection, TableViewer tableViewer, List<ZeitGruppenZeile> liste,
      ArchiveDataKind[] dataKinds, boolean speichern, boolean oeffentlich, String informationsText, 
      boolean aenderungsProtokoll, String ersteller, boolean sofort, int simuVariante, Map<String, String> bindungen,
      Integer secViewId, SkriptInfo skriptInfo)
  {
    _connection = connection;
    _tvBetriebsMeldungen2 = tableViewer;
    _listeZeitGruppenZeilen = liste;
    _zugriffPua = ZugriffPua.getInstanz(DaVKommunikation.getInstanz().holeDaVVerbindung());
    _numberInstances = secViewId;
    _dataKinds = dataKinds;
    _speichern = speichern;
    _oeffentlich = oeffentlich;
    _informationsText = informationsText;
    _aenderungsProtokoll = aenderungsProtokoll;
    _ersteller = ersteller;
    _sofort = sofort;
    _simVar = simuVariante;
    _bindungen.putAll(bindungen);
    _skriptInfo = skriptInfo;

    Thread t = new Thread(this);
    t.start();
  }

  /**
   * Konstruktor
   * 
   * @param protokollInformationen
   * @param secViewId
   */
  public AnfrageThread(ProtokollInformationen protokollInformationen, Integer secViewId)
  {
    // TODO Auto-generated constructor stub

    _protokollInformationen = protokollInformationen;

    if (_protokollInformationen != null)
    {
    	String informationsText = _protokollInformationen.getInformationsText();
    	int beginIndex = informationsText.indexOf('[');
    	if (beginIndex == 0) {
    		int endIndex = informationsText.indexOf(']');
    		String verborgenerText = informationsText.substring(beginIndex + 1, endIndex);
    		String[] zerlegung = verborgenerText.split(":");
    		if (zerlegung.length > 0) {
    			String aktionsId = zerlegung[0];
    			_skriptInfo = ExtensionVerwaltung.getInstanz().getSkriptInfo(aktionsId);
    		}
    	}
//      _skript = _protokollInformationen.getSkriptName();
    }

    _zugriffPua = ZugriffPua.getInstanz(DaVKommunikation.getInstanz().holeDaVVerbindung());
    _numberInstances = secViewId;
    _mitProtokollInformationen = true;

    Thread t = new Thread(this);
    t.start();
  }

  /**
   * Erstellt die Protokollstruktur
   * 
   */
  private void aktionProtokollierungUndAuswertung()
  {
	String skriptPid = _skriptInfo.getPid();  
	  
    if (skriptPid != null && skriptPid.length() > 0)
    {
      if (!_mitProtokollInformationen)
      {
        _protokollInformationen = new ProtokollInformationen();

        Set<ArchiveDataKind> arch = new HashSet<ArchiveDataKind>();
        
        for (int i = 0; i < _dataKinds.length; i++)
        {
          arch.add(_dataKinds[i]);
        }
        
        _protokollInformationen.setArchivDatenArt(arch);

        Hashtable<String, String> bindungen = new Hashtable<String, String>();
//        bindungen.put("?1", "asp.analyse");
        bindungen.putAll(_bindungen);
        _protokollInformationen.setBindungen(bindungen);
        
        for (String key: bindungen.keySet()) {
        	System.out.println("Bindung " + key + ":" + bindungen.get(key)); 
        }
        if (bindungen.isEmpty()) {
        	System.out.println("Keine Bindung");
        }

        final List<SystemObject> objekte = new ArrayList<SystemObject>();

        Display.getDefault().asyncExec(new Runnable() {
          public void run()
          {
            Object input = _tvBetriebsMeldungen2.getInput();

            if (input instanceof ArrayList)
            {
              List<SystemObject> list = (ArrayList<SystemObject>)input;

              if (list != null && list.size() > 0)
              {
                for (SystemObject so : list)
                {
                  synchronized (objekte)
                  {
                    objekte.add(so);
                  }
                }
              }
            }
            synchronized (objekte)
            {
              objekte.notify();
            }
         }
        });
        
        synchronized (objekte)
        {
            try
            {
              objekte.wait();
            }
            catch (InterruptedException e)
            {
              // TODO Auto-generated catch block
              e.printStackTrace();
            }
        }
	      if (objekte.size() > 0)
	      {
	        SystemObject hauptObjekt = _skriptInfo.getObjektProvider().getHauptObjekt(objekte);
	
	        _protokollInformationen.setHauptObjekt(hauptObjekt);
	        
//	        HashSet<SystemObject> objekte = new HashSet<SystemObject>();
//	        objekte.add(hauptObjekt);
//	        _protokollInformationen.setObjekte(objekte);
	
	        Map<String, String[]> pseudoObjekte = _skriptInfo.getObjektProvider().getPseudoObjekte(objekte);
	        
//	        Hashtable<String, String[]> pseudoObjekte = new Hashtable<String, String[]>();
//	
//	        String[] pids = new String[_input.size()];
//	
//	        for (int i = 0; i < _input.size(); i++)
//	        {
//	          pids[i] = _input.get(i).getPid();
//	        }
//	
//	        pseudoObjekte.put("mq0", pids);
	        _protokollInformationen.setPseudoObjekte(pseudoObjekte);
	        
	        _protokollInformationen.setProtokollSpeichern(_speichern);
	
	        _protokollInformationen.setOeffentlich(_oeffentlich);
	        
	        _protokollInformationen.setInformationsText(_informationsText);
	        
	        _protokollInformationen.setAenderungsProtokoll(_aenderungsProtokoll);
	        
	        _protokollInformationen.setErsteller(_ersteller);
	        
	        _protokollInformationen.setSimlationsVariante((short)_simVar);
	
	        SystemObject skript = _connection.getDataModel().getObject(skriptPid);
	        if (skript != null)
	          _protokollInformationen.setSkript(skript);
	
	        List<Tuple<Long, Long>> periodList = new ArrayList<Tuple<Long, Long>>();
	
	        for (ZeitGruppenZeile zgz : _listeZeitGruppenZeilen)
	        {
	          long start = zgz.getDatenStrukturZeitGruppe().getStartzeitpunkt();
	          long stop = zgz.getDatenStrukturZeitGruppe().getEndzeitpunkt();
	          periodList.add(new Tuple<Long, Long>(start, stop));
	        }
	
	        _protokollInformationen.setIntervalle(periodList);
	        
	        if (_skriptInfo.getObjektProvider().isSpeichernSelektierteObjekte()) {
	        	String infoTextBereinigt = _protokollInformationen.getInformationsTextBereinigt();
	        	String infoText = _protokollInformationen.getInformationsText();
	        	String verborgenerText = "";
	        	int index = infoText.indexOf('[');
	        	if (index == 0) {
	        		index = infoText.indexOf(']');
	        		if (index > 0) {
	        			verborgenerText = infoText.substring(1, index);
	        		}
	        	}	
	        	String[] zerlegung = verborgenerText.split(":");
	        	infoText = "[";
	        	for (int i = 0; i < zerlegung.length; i += 1) {
	        		infoText += zerlegung[i];
	        		infoText += ":";
	        	}
	        	int maxIndex = objekte.size() - 1;
	        	for (int i = 0; i <= maxIndex; i += 1)
	        	{
	        		infoText += objekte.get(i).getPid();
	        		if (i < maxIndex) {
	        			infoText += ",";	
	        		}
	        	}	
	        	infoText += "]";
	        	infoText += infoTextBereinigt;
	        	_protokollInformationen.setInformationsText(infoText);
	        }		
	
	        try
	        {
	          final ProcessingParameter processingParameter = _protokollInformationen.getProcessingParameter();
	
	          ProtocolResultStream stream = null;
	          if (_sofort)
	          {
	            stream = _zugriffPua.erstelleProtokollMitSofortigerAusgabe(processingParameter);
	          }
	          else
	          {
	            stream = _zugriffPua.erstelleProtokollOhneAusgabe(processingParameter);
	          }
	
	          if (stream != null)
	          {
	            verarbeiteDatenVonPuA(stream, _protokollInformationen, true);
	          }
	          else
	            System.err.println("Protokoll nicht verfgbar !");
	
	        }
	        catch (FailureException e)
	        {
	          e.printStackTrace();
	        }
	
	      }
	      else
	        System.err.println("Fehler: " + objekte.size());
//      }

      }
      else
      {
        try
        {
          final ProcessingParameter processingParameter = _protokollInformationen.getProcessingParameter();

          ProtocolResultStream stream = _zugriffPua.erstelleProtokollMitSofortigerAusgabe(processingParameter);

          if (stream != null)
          {
            verarbeiteDatenVonPuA(stream, _protokollInformationen, true);
          }
          else
            System.err.println("Protokoll nicht verfgbar !");

        }
        catch (FailureException e)
        {
          e.printStackTrace();
        }

      }

    }
    else
    {
      System.err.println("Kein Skript angegeben!");
    }

  }

  /**
   * Methode arbeitet den Daten-Stream der PUA ab und informiert den Update-Listener
   * 
   * @param stream
   * @param info
   * @param ausgabeAufGui
   */
  private void verarbeiteDatenVonPuA(ProtocolResultStream stream, ProtokollInformationen info, boolean ausgabeAufGui)
  {
    // TODO Auto-generated method stub
    while (stream.hasNext())
    {

      try
      {
        if (stream != null)
        {
          final ProtocolResult res = stream.take();
          final ProtokollInformationen finfo = info;
          final Long protokollId = stream.getProtocolId();
          Display.getDefault().asyncExec(new Runnable() {
            public void run()
            {
              PuADataUpdateListener.getInstanz().notifyPuADataUpdateListener(res, finfo, _numberInstances, protokollId);
            }
          });
        }

      }
      catch (InterruptedException e)
      {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }

    if (!stream.isComplete()) // ggf Fehler ausgeben:
    {
      if (stream.wasAborted()) {
          System.out.println("Stream abgebrochen: " + stream);
      }
      else {
    	  System.err.println("Fr " + stream + " ist ein Fehler aufgetreten: " + stream.getErrorMsg());
      }
    }
  }

  @Override
  public void run()
  {
    // TODO Auto-generated method stub
    aktionProtokollierungUndAuswertung();
    System.out.println("AnfrageThread beendet: " + this);
  }

}
