/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.extension;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Platform;

import de.bsvrz.buv.plugin.pua.basis.expose.INachbereitung;
import de.bsvrz.buv.plugin.pua.basis.expose.IObjektProvider;
import de.bsvrz.buv.plugin.pua.basis.expose.IProjektInfo;


public class SkriptInfo {
	
	private String _pid;
	private String _name;
	private String _aktionsId;
	private String _typ;
	private String _beschreibung;
	private String _autor;
	private String _status;
	private String _datum;
	private String _skriptDatei;
	private boolean _erstellen;
	private boolean _aktuell;
	private boolean _nachgeliefert;
	private boolean _nachgefordertAktuell;
	private boolean _nachgefordertNachgeliefert;
	private boolean _aenderungsProtokoll;
	private boolean _ganglinien;
	private boolean _keinNoChange;
	private boolean _ausfilternDE255;
	private String _umsetungsModulFilterText;
	private IObjektProvider _objektProvider;
		
	private List<Bindung> _bindungen = new ArrayList<Bindung>();
	
	public SkriptInfo (String pid,
		String name,
		String aktionsId,
		String typ,
		String beschreibung,
		String autor,
		String status,
		String datum,
		String skriptDatei,
		boolean erstellen,
		boolean aktuell,
		boolean nachgeliefert,
		boolean nachgefordertAktuell,
		boolean nachgefordertNachgeliefert,
		boolean aenderungsProtokoll,
		boolean ganglinien,
		boolean keinNoChange,
		boolean ausfilternDE255,
		String umsetungsModulFilterText,
		IObjektProvider objektProvider,
		Collection<Bindung> bindungen) {
		
		_pid = pid;
		_name = name;
		_aktionsId = aktionsId;
		_typ = typ;
		_beschreibung = beschreibung;
		_autor = autor;
		_status = status;
		_datum = datum;
		_skriptDatei = skriptDatei;
		_erstellen = erstellen;
		_aktuell = aktuell;
		_nachgeliefert = nachgeliefert;
		_nachgefordertAktuell = nachgefordertAktuell;
		_nachgefordertNachgeliefert = nachgefordertNachgeliefert;
		_aenderungsProtokoll = aenderungsProtokoll;
		_ganglinien = ganglinien;
		_keinNoChange = keinNoChange;
		_ausfilternDE255 = ausfilternDE255;
		_umsetungsModulFilterText = umsetungsModulFilterText;
		_objektProvider = objektProvider;
		_bindungen.addAll(bindungen);
	}

	public String getPid() {
		return _pid;
	}
	
	public String getName() {
		return _name;
	}

	public String getAktionsId() {
		return _aktionsId;
	}

	public String getTyp() {
		return _typ;
	}

	public String getBeschreibung() {
		return _beschreibung;
	}

	public String getAutor() {
		return _autor;
	}

	public String getStatus() {
		return _status;
	}

	public String getDatum() {
		return _datum;
	}

	public String getSkriptDatei() {
		return _skriptDatei;
	}

	public boolean isErstellen() {
		return _erstellen;
	}
	
	public boolean isAktuell() {
		return _aktuell;
	}
	
	public boolean isNachgeliefert() {
		return _nachgeliefert;
	}
	
	public boolean isNachgefordertAktuell() {
		return _nachgefordertAktuell;
	}
	
	public boolean isNachgefordertNachgeliefert() {
		return _nachgefordertNachgeliefert;
	}
	
	public boolean isAenderungsProtokoll() {
		return _aenderungsProtokoll;
	}
	
	public boolean isGanglinien() {
		return _ganglinien;
	}
	
	public boolean isKeinNoChange() {
		return _keinNoChange;
	}
	
	public boolean isAusfilternDE255() {
		return _ausfilternDE255;
	}
	
	public String getUmsetungsModulFilterText() {
		return _umsetungsModulFilterText;
	}
	
	public IObjektProvider getObjektProvider() {
		return _objektProvider;
	}
	
	public List<Bindung> getBindungen() {
		return Collections.unmodifiableList(_bindungen);
	}
	
	private IConfigurationElement getConfigurationElement() {
		IConfigurationElement[] configurationElementsFor = Platform.getExtensionRegistry().getConfigurationElementsFor(
		"de.bsvrz.buv.plugin.pua.basis.skriptInfos");
	  
		IConfigurationElement result = null;
		
		for (IConfigurationElement ce : configurationElementsFor) {
			  
			if("skriptInfos".equals(ce.getName())) {
				  
				for (IConfigurationElement ch : ce.getChildren()) {
					  
//					if("skriptInfo".equals(ch.getName()) && ch.getAttribute("Pid").equals(getPid())) {
					if("skriptInfo".equals(ch.getName()) && ch.getAttribute("Aktions-Id").equals(getAktionsId())) {
						result = ch;
					}	  
				}
			}
		}
		return result;
	}
	
	public IProjektInfo createProjektInfo() {
		IProjektInfo projektInfo = null;
		IConfigurationElement ce = getConfigurationElement();
		
		try {
			projektInfo = (IProjektInfo)ce.createExecutableExtension("Projektinfo-Klasse");
		}
		catch (CoreException e ) {
			// tue nichts
		}
		
		return projektInfo;
	}
	
	public INachbereitung createNachbereitung() {
		INachbereitung nachbereitung = null;
		IConfigurationElement ce = getConfigurationElement();
		try {
			nachbereitung = (INachbereitung)ce.createExecutableExtension("Nachbereitung-Klasse");
		}
		catch (CoreException e ) {
			// tue nichts
		}
		
		return nachbereitung;
	}
}
