/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *
 * Copyright (c) 2011 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.buv.plugin.pua.basis.pua.daten;

import java.text.FieldPosition;
import java.text.SimpleDateFormat;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Set;

import de.bsvrz.pua.prot.util.ProcessingParameter;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.losb.util.Tuple;
import de.bsvrz.sys.funclib.losb.exceptions.FailureException;

/**
 * Klasse fr das Speichern der ProtokollInformationen (keine ProtokollDaten).
 * 
 * @version $Revision: 1.5 $ / $Date: 2012/12/21 09:56:28 $ / ($Author: Pittner $)
 * 
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 * 
 */
public class ProtokollInformationen
{
  private long m_id;

  private Set<ArchiveDataKind> m_archivDatenArt;

  private Map<String, String> m_bindungen;

  private Boolean m_aenderungsProtokoll;

  private SystemObject m_hauptObjekt;

  private HashSet<SystemObject> m_objekte;

  private List<Tuple<Long, Long>> m_intervalle;

  private Map<String, String[]> m_pseudoObjekte;

  private Boolean m_protokollSpeichern;

  private SystemObject m_skript;

  private short m_simlationsVariante;
  
  private boolean m_oeffentlich = true;

  private String m_informationsText = "";

  private String m_ersteller = "";

  /**
   * Wird von {@link #msToDate(StringBuffer, long)} verwendet.
   */
  private static final GregorianCalendar gc = new GregorianCalendar();

  /**
   * Format fr Datumsausgabe festlegen.
   */
  protected static final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

  public ProtokollInformationen()
  {
  }

  public ProtokollInformationen(Tuple<Long, ProcessingParameter> protokoll)
  {
    m_id = protokoll.first;

    ProcessingParameter parameter = protokoll.last;
    m_archivDatenArt = parameter.getArchiveDataKind();
    m_bindungen = parameter.getAspBindings();
    m_aenderungsProtokoll = parameter.getDeltaProtocol();
    m_hauptObjekt = parameter.getMainObject();
//    m_objekte = parameter.objects;
    m_intervalle = parameter.getPeriodList();
    m_pseudoObjekte = parameter.getPseudoObjects();
    m_protokollSpeichern = parameter.getSaveProtocol();
    m_skript = parameter.getScript();
    m_simlationsVariante = parameter.getSimVar();
    setOeffentlich(parameter.isPublic());
    setInformationsText(parameter.getInfoText());
    setErsteller(parameter.getCreatorName());
  }

  /**
   * @return the archivDatenArt
   */
  public Set<ArchiveDataKind> getArchivDatenArt()
  {
    return m_archivDatenArt;
  }

  /**
   * Setzen der ArchivDatenArt fr eine Protokollanforderung.
   * 
   * @param archivDatenArt
   */
  public void setArchivDatenArt(Set<ArchiveDataKind> archivDatenArt)
  {
    m_archivDatenArt = archivDatenArt;
  }

  /**
   * @return the aenderungsProtokoll
   */
  public boolean isAenderungsProtokoll()
  {
    if (m_aenderungsProtokoll == null)
      return false;

    return m_aenderungsProtokoll;
  }

  /**
   * @param aenderungsProtokoll
   */
  public void setAenderungsProtokoll(boolean aenderungsProtokoll)
  {
    m_aenderungsProtokoll = aenderungsProtokoll;
  }

  /**
   * @return the hauptObjekt
   */
  public SystemObject getHauptObjekt()
  {
    return m_hauptObjekt;
  }

  /**
   * @return the hauptObjekt
   */
  public void setHauptObjekt(SystemObject hauptObject)
  {
    m_hauptObjekt = hauptObject;
  }

  /**
   * @return the objekte
   */
  public HashSet<SystemObject> getObjekte()
  {
    return m_objekte;
  }

  /**
   * @return the objekte
   */
  public void setObjekte(HashSet<SystemObject> objekte)
  {
    m_objekte = objekte;
  }

  /**
   * @return the intervalle
   */
  public List<Tuple<Long, Long>> getIntervalle()
  {
    return m_intervalle;
  }

  /**
   * @return the intervalle
   */
  public void setIntervalle(List<Tuple<Long, Long>> intervalle)
  {
    m_intervalle = intervalle;
  }

  /**
   * @return the protokollSpeichern
   */
  public boolean isProtokollSpeichern()
  {
    return m_protokollSpeichern;
  }

  /**
   * @return the protokollSpeichern
   */
  public void setProtokollSpeichern(boolean protokollSpeichern)
  {
    m_protokollSpeichern = protokollSpeichern;
  }

  /**
   * @return the skript
   */
  public SystemObject getSkript()
  {
    return m_skript;
  }

  /**
   * Methode liefert den Namen des Skriptes
   * 
   * @return Name
   */
  public String getSkriptName()
  {
    if (m_skript != null)
      return m_skript.getName();

    return null;
  }

  /**
   * @return the skript
   */
  public void setSkript(SystemObject skript)
  {
    m_skript = skript;
  }

  /**
   * @return the simlationsVariante
   */
  public int getSimlationsVariante()
  {
    return m_simlationsVariante;
  }

  /**
   * @return the simlationsVariante
   */
  public void setSimlationsVariante(short simlationsVariante)
  {
    m_simlationsVariante = simlationsVariante;
  }

  public boolean isOeffentlich()
  {
    return m_oeffentlich;
  }
  
  public void setOeffentlich(boolean oeffentlich)
  {
    m_oeffentlich = oeffentlich;
  }

  public String getInformationsText()
  {
    return m_informationsText;
  }
  
  public String getInformationsTextBereinigt() {
		String infoText = m_informationsText == null ? "" : m_informationsText;  
	  	int index = infoText.indexOf('[');
	  	if (index == 0) {
	  		index = infoText.indexOf(']');
	  		if (index > 0) {
	  			return infoText.substring(index + 1);
	  		}
	  	}
	  	return infoText;
	  }
  
  public void setInformationsText(String informationsText)
  {
    m_informationsText = (informationsText == null) ? "" : informationsText;
  }

  public String getErsteller()
  {
    return m_ersteller;
  }
  
  public void setErsteller(String ersteller)
  {
    m_ersteller = (ersteller == null) ? "" : ersteller;
  }
  
  /**
   * @return the id
   */
  public long getId()
  {
    return m_id;
  }

  /**
   * @return the bindungen
   */
  public Map<String, String> getBindungen()
  {
    return m_bindungen;
  }

  /**
   * @return the bindungen
   */
  public void setBindungen(Hashtable<String, String> bindungen)
  {
    m_bindungen = bindungen;
  }

  /**
   * @return the pseudoObjekte
   */
  public Map<String, String[]> getPseudoObjekte()
  {
    return m_pseudoObjekte;
  }

  /**
   * @return the pseudoObjekte
   */
  public void setPseudoObjekte(Map<String, String[]> pseudoObjekte)
  {
    m_pseudoObjekte = pseudoObjekte;
  }

  /*
   * (non-Javadoc)
   * 
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString()
  {
    StringBuffer sb = new StringBuffer();

    sb.append("Id: " + m_id + " - ");

    sb.append("Archivdatenart: ");
    for (ArchiveDataKind adk : m_archivDatenArt)
      sb.append(adk + ", ");

    sb.append("Bindungen: ");
    for (String key : m_bindungen.keySet())
      sb.append(key + "->" + m_bindungen.get(key) + ", ");

    sb.append("Aenderungsprotokoll: " + ((isAenderungsProtokoll()) ? "ja, " : "nein, "));

    sb.append("Hauptobjekt: " + m_hauptObjekt.getPid() + ", ");

    sb.append("Objekte: ");
    for (SystemObject object : m_objekte)
      if (object != null)
        sb.append(object.getPid() + ", ");

    sb.append("Intervalle : ");
    for (Tuple<Long, Long> period : m_intervalle)
    {
      msToDate(sb, period.first);
      sb.append(" - ");
      msToDate(sb, period.last);
      sb.append(", ");
    }

    sb.append("Pseudoobjektdefinitionen: ");
    for (String alias : m_pseudoObjekte.keySet())
    {
      // sb.append(alias + "->" + pseudoObjects.get(alias) + ", ");
      sb.append(alias + "->");

      String[] temp = m_pseudoObjekte.get(alias);

      for (int i = 0; i < temp.length; i++)
      {
        if (i > 0)
          sb.append("-");

        sb.append(temp[i]);
      }

      sb.append(", ");
    }

    sb.append("Speichern: " + ((m_protokollSpeichern) ? "ja, " : "nein, "));

    sb.append("Skript: " + m_skript.getNameOrPidOrId() + ", ");

    sb.append("Simulationsvariante: " + m_simlationsVariante + ", ");

    sb.append("Informationstext: " + getInformationsText() + ", ");
    
    sb.append("Ersteller: " + getErsteller() + ", ");
    
    sb.append("Oeffentlich: " + (isOeffentlich() ? "ja" : "nein"));
    
    return sb.toString();
  }

  /**
   * Berechnet das Datum aus Zeitangabe in Millisekunden. Das Datum wird folgendermaen formatiert: DD.MM.JJJJ
   * HH:MM:SS,sss
   * 
   * @param sb
   *          Stringbuffer, in den das Datum geschrieben wird.
   * @param timeInMillis
   *          Zeitangabe in Millisekunden.
   */
  public static void msToDate(StringBuffer sb, long timeInMillis)
  {
    synchronized (gc)
    {
      gc.clear();
      gc.setTimeInMillis(timeInMillis);
      sdf.format(gc.getTime(), sb, new FieldPosition(0));
    }
  }

  /**
   * Berechnet das Datum aus Zeitangabe in Millisekunden. Das Datum wird folgendermaen formatiert: DD.MM.JJJJ
   * HH:MM:SS,sss
   * 
   * @param timeInMillis
   *          Zeitangabe in Millisekunden.
   * @return String mit formatiertem Datum.
   */
  public static String msToDate(long timeInMillis)
  {
    StringBuffer sb = new StringBuffer();
    msToDate(sb, timeInMillis);
    return sb.toString();
  }

  /**
   * @return
   * @throws FailureException
   */
  public ProcessingParameter getProcessingParameter() throws FailureException
  {
    ProcessingParameter pp = new ProcessingParameter(getSkript(), getHauptObjekt(), (short)getSimlationsVariante());
    pp.setArchiveDataKind(getArchivDatenArt());
    pp.setAspBindings(getBindungen());
    pp.setDeltaProtocol(isAenderungsProtokoll());
//    pp.objects = getObjekte();
    pp.setPeriodList(getIntervalle());
    pp.setPseudoObjects(getPseudoObjekte());
    pp.setSaveProtocol(isProtokollSpeichern());
    pp.setIsPublic(isOeffentlich());
    pp.setInfoText(getInformationsText());
    pp.setCreatorName(getErsteller());
    
    return pp;
  }

  /**
   * Methode zum Setzen der ID (fr Testzwecke)
   * 
   * @param id
   *          ID
   */
  public void setId(long id)
  {
    m_id = id;
  }
}
