/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.pua.basis.views;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.pua.basis.Activator;
import de.bsvrz.buv.plugin.pua.basis.DaVKommunikation;
import de.bsvrz.buv.plugin.pua.basis.daVDataUpdateListener.DaVDataUpdateEvent;
import de.bsvrz.buv.plugin.pua.basis.daVDataUpdateListener.DaVDataUpdateListener;
import de.bsvrz.buv.plugin.pua.basis.daVDataUpdateListener.IDaVDataUpdateListener;
import de.bsvrz.buv.plugin.pua.basis.dav.AnfrageDaten;
import de.bsvrz.buv.rw.basislib.kalender.DatumZeit;
import de.bsvrz.buv.rw.basislib.kalender.KalenderZeitFilterDatenStruktur;
import de.bsvrz.buv.rw.basislib.kalender.KalenderZeitFilterDatenStrukturZeitGruppe;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
import de.bsvrz.buv.rw.basislib.login.UrlasserDialog.UrlasserDialogInfo;
import de.bsvrz.buv.rw.basislib.objektzusammenstellung.ObjektSpeicher;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKindCombination;
import de.bsvrz.dav.daf.main.archive.ArchiveOrder;
import de.bsvrz.dav.daf.main.archive.ArchiveQueryPriority;
import de.bsvrz.dav.daf.main.archive.ArchiveRequestOption;
import de.bsvrz.dav.daf.main.archive.TimingType;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.NonMutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Klasse realisiert einen View zur Archivanfrage der Rckmeldungen auf DE-Basis
 * 
 * @author ObertM
 * 
 */
public class ArchivAuswertungSicht extends ViewPart implements IDaVDataUpdateListener
{
  /**
   * ID des Views
   */
  public static final String ID = "de.bsvrz.buv.plugin.pua.viewAuswertungRueckmeldungen";

  /**
   * Pid der Objekte
   */
  private String _pidObjekte = "";

  /**
   * Memento
   */
  private IMemento _memento;

  private boolean _mitObjektAuswahl = true;

  private Composite composite = null;

  private Composite compositeBottom;

  private Button buttonUebernehemen;

  private Button buttonAbbrechen;

  private Button buttonHilfe;

  protected String _programm;

  protected Combo combo;

  private ViewerFilter _searchFilter;

  private static Integer _numberInstances = 0;

  private boolean _sperreFrNeueDaten = false;

  private Composite _composite;

  private KalenderZeitFilterDatenStruktur _DatenStruktur;

  private LinkedList<ArchivAuswertungSicht.ZeitGruppenZeile> _listeZeitGruppenZeilen;

  private Group groupFilterDefinitionen11;

  private Label label0;

  private Label label1;

  private Composite composite1;

  private Group groupFilterDefinitionen12;

  private ScrolledComposite _verschiebbaresCompositeZeitgruppen;

  private Composite _compositeZeitGruppenInhalt;

  private Composite composite2;

  private Button button;

  private Composite composite3;

  private Button button2;

  private Button button3;

  private Button button1;

  private TableViewer _tvMoeglicheObjekte;

  private TableViewer _tvAusgewaehlteObjekte;

  private Composite composite41 = null;

  private Composite composite42 = null;

  private ClientDavInterface _connection;

  private Composite composite43 = null;

  private Composite composite44 = null;

  private Text text = null;

  private Button button4 = null;

  private Button button5 = null;

  private Button button51;

  private Text _text;

  private Text _text2;

  private Composite composite45 = null;

  private String _beschreibung;

  private static final String HELP_ID = Activator.PLUGIN_ID + ".sichtauswertungen";

  private String _atg = null;

  private String _asp = null;

  // private TableSort _ts1;
  //	
  // private TableSort _ts2;

  @Override
  public void createPartControl(Composite parent)
  {
    IViewSite site = getViewSite();
    if (site != null)
    {
      String secondaryId = site.getSecondaryId();
      String[] split = secondaryId.split(";");

      _atg = split[0];
      _asp = split[1];

      if (_atg != null && _asp != null)
      {
        if (_atg.equals("atg.anzeigeEigenschaftIst"))
        {
          setPartName("Auswertungen Rckmeldungen");
          _pidObjekte = "typ.anzeigeQuerschnitt";
        }
        else if (_atg.equals("atg.anzeigeEigenschaftSoll"))
        {
          setPartName("Auswertungen Befehle");
          _pidObjekte = "typ.anzeigeQuerschnitt";
        }
        else if (_atg.equals("atg.sbaUeSchaltbild") && _asp.equals("asp.sbaHandschaltung"))
        {
          setPartName("Auswertungen Handschaltungen");
          _pidObjekte = "typ.sbaAnzeigeQuerschnitt";
        }
        else if (_atg.equals("atg.sbaUeSchaltbild") && _asp.equals("asp.sbaSonderprogramm"))
        {
          setPartName("Auswertungen Sonderprogramme");
          _pidObjekte = "typ.streckenBeeinflussungsAnlageBereich";
        }
      }
    }

    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    composite = new Composite(parent, SWT.NONE);
    composite.setLayoutData(gridData);
    composite.setLayout(new FillLayout());

    PlatformUI.getWorkbench().getHelpSystem().setHelp(composite, HELP_ID);

    createCompositeTop();

    createCompositeBottom();

    if (_connection != null)
    {
      List<SystemObject> l = new ArrayList<SystemObject>();

      SystemObjectType type = _connection.getDataModel().getType(_pidObjekte);
      List<SystemObject> kanaele = type.getElements();

      for (SystemObject o : kanaele)
        l.add(o);

      _tvMoeglicheObjekte.setInput(l);
    }
  }

  @Override
  public void setFocus()
  {
    if (composite != null)
      composite.setFocus();
  }

  /**
   * This method initializes compositeBottom
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    compositeBottom = new Composite(_composite, SWT.NONE);
    compositeBottom.setLayoutData(gridData3);
    compositeBottom.setLayout(rowLayout);

    buttonUebernehemen = new Button(compositeBottom, SWT.NONE);
    buttonUebernehemen.setText("Anfordern");
    buttonUebernehemen.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
//        UrlasserDialog uld = new UrlasserDialog(PlatformUI.getWorkbench().getDisplay().getActiveShell());
//        UrlasserDialogInfo info = uld.anzeigen(new ArrayList<ResultData>());
//        if (info.getEingabeOk())
//        {
          aktionAnfordern();
//        }
      }
    });

    buttonAbbrechen = new Button(compositeBottom, SWT.NONE);
    buttonAbbrechen.setText("Abbrechen");
    buttonAbbrechen.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        dispose();
      }

    });
    buttonHilfe = new Button(compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        aktionHilfe();
      }
    });
  }

  protected void aktionAnfordern()
  {
    IWorkbenchWindow dw = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
    IWorkbenchPage page = dw.getActivePage();
    try
    {
      ArchiveQueryPriority archiveQueryPriority = ArchiveQueryPriority.HIGH;
      TimingType timingType = TimingType.DATA_TIME;
      boolean startRelative = false;
      ArchiveDataKindCombination archiveDataKindCombination = new ArchiveDataKindCombination(ArchiveDataKind.ONLINE);
      ArchiveOrder archiveOrder = ArchiveOrder.BY_INDEX;
      ArchiveRequestOption archiveRequestOption = ArchiveRequestOption.NORMAL;

      List<SystemObject> objekte = null;

      if (_mitObjektAuswahl)
      {
        objekte = (List<SystemObject>)_tvAusgewaehlteObjekte.getInput();
      }
      else
      {
        if (_connection != null)
        {
          objekte = _connection.getDataModel().getType(_pidObjekte).getElements();
        }
      }

      if (objekte != null && objekte.size() > 0)
      {
        LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe> zeitgruppen = new LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe>();

        for (ZeitGruppenZeile zgz : _listeZeitGruppenZeilen)
        {
          KalenderZeitFilterDatenStrukturZeitGruppe dszg = zgz.getDatenStrukturZeitGruppe();
          zeitgruppen.add(dszg);
        }

        try
        {
          List<AnfrageDaten> liste = new ArrayList<AnfrageDaten>();

          List<SystemObject> p = new ArrayList<SystemObject>();

          if (_atg.equals("atg.anzeigeEigenschaftIst") || _atg.equals("atg.anzeigeEigenschaftSoll")){
          
            for (SystemObject so : objekte)
            {
              ConfigurationObject co = (ConfigurationObject)so;
              NonMutableSet set = co.getNonMutableSet("Anzeigen");
              List<SystemObject> anzeigen = set.getElements();
              for (SystemObject anzeige : anzeigen)
              {
                p.add(anzeige);
              }
            }
          }
          else if (_atg.equals("atg.sbaUeSchaltbild") && _asp.equals("asp.sbaHandschaltung")){            
            
            for (SystemObject so : objekte)
            {              
              p.add(so);
            }
          }
//          else if (_atg.equals("atg.sbaUeSchaltbild") && _asp.equals("asp.sbaSonderprogramm")){            
//            
//            for (SystemObject so : objekte)
//            {              
//              p.add(so);
//            }
//          }

          if (p.size() > 0)
          {

            AttributeGroup atg = _connection.getDataModel().getAttributeGroup(_atg);
            Aspect asp = _connection.getDataModel().getAspect(_asp);
            DataDescription dd = new DataDescription(atg, asp);

            AnfrageDaten anfrageDaten = new AnfrageDaten(p, objekte, null, zeitgruppen, dd, archiveQueryPriority, timingType,
                startRelative, archiveDataKindCombination, archiveOrder, archiveRequestOption);

            liste.add(anfrageDaten);

          }

          if (liste.size() > 0)
          {
            page.showView(ArchivErgebnisAuswertungSicht.ID, _numberInstances.toString(), IWorkbenchPage.VIEW_ACTIVATE);

            IViewReference ref = page.findViewReference(ArchivErgebnisAuswertungSicht.ID, _numberInstances.toString());
            ArchivErgebnisAuswertungSicht view = (ArchivErgebnisAuswertungSicht)ref.getView(true);
            view.setAnfrageDaten(liste);

            _numberInstances++;

            // new ArchiveRequest().startRequest(liste);
          }
          else
          {
            MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
            mb.setText("Fehler Anfrage");

            String s = "Es muss mindestens ein Anfragetyp ausgewhlt werden!";

            mb.setMessage(s);
            mb.open();
            return;
          }
        }
        catch (IllegalArgumentException ie)
        {
          // TODO: handle exception
          ie.printStackTrace();
        }
      }
    }
    catch (PartInitException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * de.bsvrz.buv.plugin.knotenbeeinflussung.daVDataUpdateListener.IDaVDataUpdateListener#daVDataUpdate(de.bsvrz.buv
   * .plugin.knotenbeeinflussung.daVDataUpdateListener.DaVDataUpdateEvent)
   */
  @Override
  public void daVDataUpdate(DaVDataUpdateEvent e)
  {
    if (e != null)
    {
      if (!_sperreFrNeueDaten)
      {
        // verarbeiteDavDaten(e);
      }
      else
      {
        // System.out.println("Neue Daten -> kein Update, da Sperre fr
        // neue Daten aktiv");
      }
    }
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void createCompositeTop()
  {

    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    _composite = new Composite(composite, SWT.NONE);
    _composite.setLayout(gridLayout);

    if (_mitObjektAuswahl)
      createGroupAuswahlObjekte();

    _DatenStruktur = new KalenderZeitFilterDatenStruktur();
    _listeZeitGruppenZeilen = new LinkedList<ZeitGruppenZeile>();

    createGroupZeitgruppen();

    _connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
  }

  /**
   * This method initializes groupFilterDefinitionen11
   * 
   */
  private void createGroupAuswahlObjekte()
  {
    RowLayout rowLayout3 = new RowLayout();
    rowLayout3.type = org.eclipse.swt.SWT.VERTICAL;
    rowLayout3.justify = true;
    rowLayout3.wrap = false;
    rowLayout3.pack = false;
    rowLayout3.spacing = 25;
    rowLayout3.fill = false;
    GridData gridData22 = new GridData();
    gridData22.horizontalAlignment = GridData.CENTER;
    gridData22.grabExcessVerticalSpace = true;
    gridData22.grabExcessHorizontalSpace = false;
    gridData22.verticalAlignment = GridData.FILL;
    GridData gridData10 = new GridData();
    gridData10.horizontalSpan = 2;
    GridData gridData5 = new GridData();
    gridData5.horizontalAlignment = GridData.END;
    gridData5.grabExcessHorizontalSpace = true;
    gridData5.verticalAlignment = GridData.CENTER;
    GridData gridData4 = new GridData();
    gridData4.horizontalAlignment = GridData.BEGINNING;
    gridData4.verticalAlignment = GridData.CENTER;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = false;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.CENTER;
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.BEGINNING;
    gridData11.verticalAlignment = GridData.CENTER;
    GridData gridData211 = new GridData();
    gridData211.horizontalAlignment = GridData.BEGINNING;
    gridData211.verticalAlignment = GridData.CENTER;
    groupFilterDefinitionen11 = new Group(_composite, SWT.NONE);
    groupFilterDefinitionen11.setText("Auswahl Objekte");
    groupFilterDefinitionen11.setLayout(new FillLayout());
    groupFilterDefinitionen11.setLayoutData(gridData2);

    GridData gridData41 = new GridData();
    gridData41.horizontalAlignment = GridData.FILL;
    gridData41.grabExcessHorizontalSpace = true;
    gridData41.grabExcessVerticalSpace = false;
    gridData41.verticalAlignment = GridData.FILL;
    gridData41.heightHint = 100;

    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.FILL;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.grabExcessVerticalSpace = false;
    gridData3.verticalAlignment = GridData.FILL;
    gridData3.heightHint = 100;

    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.CENTER;
    gridData21.verticalAlignment = GridData.CENTER;
    GridData gridData12 = new GridData();
    gridData12.horizontalAlignment = GridData.CENTER;
    gridData12.grabExcessHorizontalSpace = false;
    gridData12.verticalAlignment = GridData.CENTER;

    // GridData gridData3 = new GridData();
    // gridData3.widthHint = 50;
    // gridData3.heightHint = 50;

    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 4;
    gridLayout.marginHeight = 5;
    gridLayout.verticalSpacing = 5;
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.marginWidth = 5;
    composite = new Composite(groupFilterDefinitionen11, SWT.NONE);
    label0 = new Label(composite, SWT.NONE);
    label0.setText("mgliche Objekte");
    label0.setLayoutData(gridData12);
    composite.setLayout(gridLayout);
    Label filler4 = new Label(composite, SWT.NONE);
    label1 = new Label(composite, SWT.NONE);
    label1.setText("vorhandene Objekte");
    label1.setLayoutData(gridData21);
    Label filler3 = new Label(composite, SWT.NONE);
    createComposite44();
    Label filler1 = new Label(composite, SWT.NONE);
    createComposite43();
    Label filler2 = new Label(composite, SWT.NONE);
    createComposite41();
    erzeugeTabelleMoeglicheObjekte();

    composite1 = new Composite(composite, SWT.NONE);
    composite1.setLayoutData(gridData22);
    createComposite2();
    composite1.setLayout(rowLayout3);
    createComposite3();

    createComposite42();
    erzeugeTabelleAngewaehlteObjekte();
  }

  /**
   * Liest die Datenstruktur ein
   * 
   */
  private void leseDatenstruktur()
  {
    // Zeitgruppen setzen
    List<KalenderZeitFilterDatenStrukturZeitGruppe> zeitGruppen = _DatenStruktur.getListeZeitGruppen();
    // Wenn die ZeitGruppenListe nicht leer ist, dann auslesen und
    // darstellen

    if (zeitGruppen.size() > 0)
    {
      Iterator<KalenderZeitFilterDatenStrukturZeitGruppe> it = zeitGruppen.iterator();
      while (it.hasNext())
      {
        KalenderZeitFilterDatenStrukturZeitGruppe zg = it.next();
        generiereZeitgruppeMitVorgabe(zg);
      }
    }
    else
    // Sonst nur eine Leerzeile darstellen.
    {
      // System.out.println("hier!!!");
      Date d = new Date();
      generiereZeitgruppeMitVorgabe(d.getTime(), d.getTime() + 1);

      // generiereZeitgruppeLeer();
    }

  }

  /**
   * This method initializes groupFilterDefinitionen12
   * 
   */
  private void createGroupZeitgruppen()
  {
    FillLayout fillLayout = new FillLayout();
    fillLayout.type = org.eclipse.swt.SWT.HORIZONTAL;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    groupFilterDefinitionen12 = new Group(_composite, SWT.NONE);
    groupFilterDefinitionen12.setText("Zeitbereiche");
    groupFilterDefinitionen12.setLayout(fillLayout);
    groupFilterDefinitionen12.setLayoutData(gridData2);

    _verschiebbaresCompositeZeitgruppen = new ScrolledComposite(groupFilterDefinitionen12, SWT.H_SCROLL | SWT.V_SCROLL);
    _verschiebbaresCompositeZeitgruppen.getVerticalBar().setIncrement(20);

    _compositeZeitGruppenInhalt = new Composite(_verschiebbaresCompositeZeitgruppen, SWT.NONE);
    _compositeZeitGruppenInhalt.setLayout(new FillLayout());
    GridLayout glGroupZeitgruppen = new GridLayout();
    glGroupZeitgruppen.numColumns = 1;
    glGroupZeitgruppen.makeColumnsEqualWidth = false;

    _verschiebbaresCompositeZeitgruppen.setContent(_compositeZeitGruppenInhalt);
    _verschiebbaresCompositeZeitgruppen.setMinSize(500, 1000);
    _verschiebbaresCompositeZeitgruppen.setExpandHorizontal(true);
    _verschiebbaresCompositeZeitgruppen.setExpandVertical(true);

    _compositeZeitGruppenInhalt.setLayout(glGroupZeitgruppen);

    leseDatenstruktur();

  }

  private void generiereZeitgruppeMitVorgabe(long lStartZeitPunkt, long lEndZeitPunkt)
  {
    generiereZeitgruppeLeer();

    _listeZeitGruppenZeilen.get(0).setDatenStrukturZeitGruppe(
        new KalenderZeitFilterDatenStrukturZeitGruppe(lStartZeitPunkt, lEndZeitPunkt));
  }

  /**
   * Generiert eine Zeitgruppe mit Zeitvorgabe in Form einer Instanz der Klasse
   * KalenderZeitFilterDatensStrukturZeitGruppe
   * 
   * @param zg
   *          Vorgabe von Start- und Endzeitpunkt
   */
  private void generiereZeitgruppeMitVorgabe(KalenderZeitFilterDatenStrukturZeitGruppe zg)
  {
    _DatenStruktur.fuegeZeitGruppeHinzu(zg);
    ZeitGruppenZeile zgz = new ZeitGruppenZeile(_compositeZeitGruppenInhalt, zg);
    zgz.generiereComposite();
    _listeZeitGruppenZeilen.add(zgz);
    _compositeZeitGruppenInhalt.layout();
  }

  /**
   * This method initializes composite2
   * 
   */
  private void createComposite2()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    composite2 = new Composite(composite1, SWT.NONE);
    composite2.setLayout(rowLayout);
    button = new Button(composite2, SWT.NONE);
    button.setText(">");
    button.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        IStructuredSelection selection = (IStructuredSelection)_tvMoeglicheObjekte.getSelection();
        List<SystemObject> x = new ArrayList<SystemObject>();
        Object firstElement = selection.getFirstElement();
        if (firstElement instanceof SystemObject)
        {
          SystemObject fe = (SystemObject)firstElement;
          // System.out.println("fe: " + fe.getPid());

          Iterator it = selection.iterator();
          while (it.hasNext())
          {
            SystemObject so = (SystemObject)it.next();
            // System.out.println("add: " + so.getPid());
            x.add(so);
          }
        }
        else
        {
          // System.out.println(firstElement.getClass().getSimpleName());
        }

        // _tvBetriebsMeldungen1.remove(x.toArray());
        List<SystemObject> y = new ArrayList<SystemObject>();
        List<SystemObject> input1 = (List<SystemObject>)_tvMoeglicheObjekte.getInput();
        if (input1 != null)
        {
          y.addAll(input1);
          y.removeAll(x);
        }
        _tvMoeglicheObjekte.getTable().clearAll();
        _tvMoeglicheObjekte.setInput(y);

        List<SystemObject> input = (List<SystemObject>)_tvAusgewaehlteObjekte.getInput();
        if (input != null)
        {
          x.addAll(input);
        }
        _tvAusgewaehlteObjekte.getTable().clearAll();
        _tvAusgewaehlteObjekte.setInput(x);
        _text.setText(_tvMoeglicheObjekte.getTable().getSelectionCount() + " / "
            + _tvMoeglicheObjekte.getTable().getItemCount());
        _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
            + _tvAusgewaehlteObjekte.getTable().getItemCount());
        composite43.layout(true);
        composite44.layout(true);

      }

    });
    button1 = new Button(composite2, SWT.NONE);
    button1.setText(">>");
    button1.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {

        List<SystemObject> x = new ArrayList<SystemObject>();

        List<SystemObject> y = new ArrayList<SystemObject>();
        y.addAll((List<SystemObject>)_tvMoeglicheObjekte.getInput());

        List<SystemObject> input1 = new ArrayList<SystemObject>();

        if (_tvMoeglicheObjekte.getFilters().length > 0)
        {
          // System.out.println("Filter aktiv: " +
          // _tvBetriebsMeldungen1.getTable().getItemCount());
          TableItem[] items = _tvMoeglicheObjekte.getTable().getItems();
          for (TableItem ti : items)
          {
            // System.out.println(ti.getData().getClass().getSimpleName());
            SystemObject so = (SystemObject)ti.getData();
            // System.out.println(so.toString());
            input1.add(so);
          }
        }
        else
          input1 = (List<SystemObject>)_tvMoeglicheObjekte.getInput();

        List<SystemObject> input2 = (List<SystemObject>)_tvAusgewaehlteObjekte.getInput();
        if (input1 != null)
        {
          x.addAll(input1);
        }
        if (input2 != null)
        {
          x.addAll(input2);
        }

        System.out.println(">> " + x.size());

        // _tvBetriebsMeldungen1.getTable().clearAll();
        // _tvBetriebsMeldungen2.getTable().clearAll();
        if (_tvMoeglicheObjekte.getFilters().length > 0)
        {

          y.removeAll(input1);
          _tvMoeglicheObjekte.setInput(y);

        }
        else
          _tvMoeglicheObjekte.setInput(new ArrayList<SystemObject>());

        _tvAusgewaehlteObjekte.setInput(x);

        _text.setText(_tvMoeglicheObjekte.getTable().getSelectionCount() + " / "
            + _tvMoeglicheObjekte.getTable().getItemCount());
        _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
            + _tvAusgewaehlteObjekte.getTable().getItemCount());
        composite43.layout(true);
        composite44.layout(true);

      }

    });
  }

  /**
   * This method initializes composite3
   * 
   */
  private void createComposite3()
  {
    RowLayout rowLayout1 = new RowLayout();
    rowLayout1.pack = false;
    composite3 = new Composite(composite1, SWT.NONE);
    composite3.setLayout(rowLayout1);
    button2 = new Button(composite3, SWT.NONE);
    button2.setText("<<");
    button2.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {

        List<SystemObject> x = new ArrayList<SystemObject>();
        List<SystemObject> input1 = (List<SystemObject>)_tvMoeglicheObjekte.getInput();
        List<SystemObject> input2 = (List<SystemObject>)_tvAusgewaehlteObjekte.getInput();
        if (input1 != null)
        {
          x.addAll(input1);
        }
        if (input2 != null)
        {
          x.addAll(input2);
        }

        System.out.println("<< " + x.size());

        // _tvBetriebsMeldungen1.getTable().clearAll();
        // _tvBetriebsMeldungen2.getTable().clearAll();
        _tvAusgewaehlteObjekte.setInput(new ArrayList<SystemObject>());
        _tvMoeglicheObjekte.setInput(x);

        _text.setText(_tvMoeglicheObjekte.getTable().getSelectionCount() + " / "
            + _tvMoeglicheObjekte.getTable().getItemCount());
        _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
            + _tvAusgewaehlteObjekte.getTable().getItemCount());
        composite43.layout(true);
        composite44.layout(true);

      }

    });
    button3 = new Button(composite3, SWT.NONE);
    button3.setText("<");
    button3.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        IStructuredSelection selection = (IStructuredSelection)_tvAusgewaehlteObjekte.getSelection();
        List<SystemObject> x = new ArrayList<SystemObject>();
        Object firstElement = selection.getFirstElement();
        if (firstElement instanceof SystemObject)
        {
          SystemObject fe = (SystemObject)firstElement;
          // System.out.println("fe: " + fe.getPid());

          Iterator it = selection.iterator();
          while (it.hasNext())
          {
            SystemObject so = (SystemObject)it.next();
            // System.out.println("add: " + so.getPid());
            x.add(so);
          }
        }
        else
        {
          // System.out.println(firstElement.getClass().getSimpleName());
        }

        // _tvBetriebsMeldungen2.remove(x.toArray());

        List<SystemObject> y = new ArrayList<SystemObject>();
        List<SystemObject> input1 = (List<SystemObject>)_tvAusgewaehlteObjekte.getInput();
        if (input1 != null)
        {
          y.addAll(input1);
          y.removeAll(x);
        }
        _tvAusgewaehlteObjekte.getTable().clearAll();
        _tvAusgewaehlteObjekte.setInput(y);

        List<SystemObject> input = (List<SystemObject>)_tvMoeglicheObjekte.getInput();

        if (input != null)
        {
          x.addAll(input);

        }
        _tvMoeglicheObjekte.getTable().clearAll();
        _tvMoeglicheObjekte.setInput(x);
        _text.setText(_tvMoeglicheObjekte.getTable().getSelectionCount() + " / "
            + _tvMoeglicheObjekte.getTable().getItemCount());
        _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
            + _tvAusgewaehlteObjekte.getTable().getItemCount());
        composite43.layout(true);
        composite44.layout(true);

      }

    });

  }

  /**
   * Bildet die Struktur einer ZeitgruppenZeile
   * 
   * @author HofmannJ
   * 
   */
  public class ZeitGruppenZeile
  {
    private Composite _parent = null;

    private KalenderZeitFilterDatenStrukturZeitGruppe _daten = null;

    private Label _labelVon;

    private Label _labelBis;

    private Label _labelInfo;

    private Button _schaltflaecheHinzufuegen = null;

    private Button _schaltflaecheLoeschen = null;

    /**
     * Diese Composite enthlt die Darstellung eines Zeitgruppen-Eintrags
     */
    private Composite _returnComposite;

    public DatumZeit _dzVon;

    public DatumZeit _dzBis;

    /**
     * Konstruktor
     * 
     * @param parent
     *          Eltern-Composite, in dass die Zeitgruppenzeile eingeblendet werden soll.
     * @param Daten
     *          Von-Bis.
     */
    public ZeitGruppenZeile(Composite parent, KalenderZeitFilterDatenStrukturZeitGruppe Daten)
    {
      this._parent = parent;
      this._daten = Daten;
    }

    /**
     * Sichert die erfassten Zeiten in daten
     * 
     */
    public void sichereErfassteZeiten()
    {
      // _daten.setStartzeitpunkt(_kzVon.getZeit());
      // _daten.setEndzeitpunkt(_kzBis.getZeit());

      // _daten.setStartzeitpunkt(_dzVon.get)

      _daten.setStartzeitpunkt(_dzVon.getDatum().getTime());
      _daten.setEndzeitpunkt(_dzBis.getDatum().getTime());

    }

    /**
     * Liefert die {@link KalenderZeitFilterDatenStrukturZeitGruppe} dieser {@link ZeitGruppenZeile}.
     * 
     * @return {@link KalenderZeitFilterDatenStrukturZeitGruppe}
     */
    public KalenderZeitFilterDatenStrukturZeitGruppe getDatenStrukturZeitGruppe()
    {
      return this._daten;
    }

    /**
     * Setzt die {@link KalenderZeitFilterDatenStrukturZeitGruppe} dieser {@link ZeitGruppenZeile}
     * 
     * @param datenStrukturZeitGruppe
     *          {@link KalenderZeitFilterDatenStrukturZeitGruppe}
     */
    public void setDatenStrukturZeitGruppe(KalenderZeitFilterDatenStrukturZeitGruppe datenStrukturZeitGruppe)
    {
      this._daten = datenStrukturZeitGruppe;
      _dzVon.setDatum(new Date(_daten.getStartzeitpunkt()));
      _dzBis.setDatum(new Date(_daten.getEndzeitpunkt()));
    }

    /**
     * Liefert true zurck, wenn der Startzeitpunkt VOR dem Endzeitpunkt liegt. Andernfalls wird false geliefert.
     * 
     * @return true, wenn der Startzeitpunkt VOR dem Endzeitpunkt liegt.
     */
    private boolean pruefePlausibilitaet()
    {
      boolean returnWert = true;
      if (_daten.getStartzeitpunkt() > _daten.getEndzeitpunkt())
      {
        returnWert = false;
      }

      return returnWert;
    }

    /**
     * Dispose-Methode fr das {@link Composite} dieser {@link ZeitGruppenZeile}
     */
    public void dispose()
    {
      _returnComposite.dispose();
    }

    /**
     * Lscht diese {@link ZeitGruppenZeile}.
     */
    private void aktionZeitgruppenzeileLoeschen()
    {
      loescheZeitGruppenZeile(this);
    }

    /**
     * Hngt eine Zeitgruppenzeile an
     */
    private void aktionZeitgruppenzeileAnhaengen()
    {
      // generiereZeitgruppeLeer();
      einfuegenZeitGruppenZeileNachPosition(this);
    }

    /**
     * Generiert das Composuite
     * 
     * @return Ein Composite
     */
    public Composite generiereComposite()
    {
      _returnComposite = new Composite(_parent, SWT.BORDER);

      RowLayout rl = new RowLayout(SWT.HORIZONTAL);
      rl.pack = false;
      _returnComposite.setLayout(rl);

      _labelVon = new Label(_returnComposite, SWT.NONE);
      _labelVon.setText("von: ");

      // _kzVon = new KalenderZeit(_returnComposite, SWT.NONE, true, true,
      // true);
      // _kzVon.setZeit(_daten.get_startzeitpunkt());

      _dzVon = new DatumZeit(_returnComposite, SWT.NONE, DatumZeit.Eingabetyp.datumuhr, false, false);
      _dzVon.setToolTipText("Datum + Zeit, mit Auswhler.");

      // _dzVon = new DatumZeit(_returnComposite, SWT.NONE,
      // DatumZeit.Eingabetyp.uhrMS, false, false);
      // _dzVon = new DatumZeit(_returnComposite, SWT.NONE,
      // DatumZeit.Eingabetyp.uhrEinfachMS, false, false);
      _dzVon.setDatum(new Date(_daten.getStartzeitpunkt()));
      _dzVon.setLayout(new GridLayout());

      // CDateTime _cdt = new CDateTime(_returnComposite, CDT.DROP_DOWN |
      // CDT.DATE_MEDIUM | CDT.TIME_MEDIUM |
      // CDT.CLOCK_24_HOUR);

      _labelBis = new Label(_returnComposite, SWT.NONE);
      _labelBis.setText("bis: ");

      // _kzBis = new KalenderZeit(_returnComposite, SWT.NONE, true, true,
      // true);
      // _kzBis.setZeit(_daten.get_endzeitpunkt());

      _dzBis = new DatumZeit(_returnComposite, SWT.NONE, DatumZeit.Eingabetyp.datumuhr, false, false);
      _dzBis.setToolTipText("Datum + Zeit, mit Auswhler.");

      // _dzBis = new DatumZeit(_returnComposite, SWT.NONE,
      // DatumZeit.Eingabetyp.uhrMS, false, false);
      // _dzBis = new DatumZeit(_returnComposite, SWT.NONE,
      // DatumZeit.Eingabetyp.uhrEinfachMS, false, false);

      _dzBis.setDatum(new Date(_daten.getStartzeitpunkt()));
      _dzBis.setLayout(new GridLayout());

      _labelInfo = new Label(_returnComposite, SWT.NONE);

      _dzVon.hinzufuegenSelektionsZuhoerer(new SelectionListener() {

        private long _startzeitpunkt = -1;

        public void widgetDefaultSelected(SelectionEvent e)
        {
          gemeinsam(e);
        }

        public void widgetSelected(SelectionEvent e)
        {
          gemeinsam(e);
        }

        private void gemeinsam(SelectionEvent e)
        {
          if (e.data instanceof Date)
          {
            Date datum = (Date)e.data;
            _startzeitpunkt = datum.getTime();
            _daten.setStartzeitpunkt(_startzeitpunkt);

            if (!pruefePlausibilitaet())
            {
              Color color = new Color(Display.getDefault(), 255, 0, 0);
              _labelInfo.setForeground(color);
              _labelInfo.setText("Startzeitpunkt liegt vor dem Endzeitpunkt!");
              _compositeZeitGruppenInhalt.layout();
            }
            else
            {
              _labelInfo.setText("");
              _compositeZeitGruppenInhalt.layout();
            }

            // fireEvent();
          }
        }

      });

      _dzBis.hinzufuegenSelektionsZuhoerer(new SelectionListener() {

        private long _endzeitpunkt = -1;

        public void widgetDefaultSelected(SelectionEvent e)
        {
          gemeinsam(e);
        }

        public void widgetSelected(SelectionEvent e)
        {
          gemeinsam(e);
        }

        private void gemeinsam(SelectionEvent e)
        {
          if (e.data instanceof Date)
          {
            Date datum = (Date)e.data;
            _endzeitpunkt = datum.getTime();
            _daten.setEndzeitpunkt(_endzeitpunkt);

            if (!pruefePlausibilitaet())
            {
              Color color = new Color(Display.getDefault(), 255, 0, 0);
              _labelInfo.setForeground(color);
              _labelInfo.setText("Startzeitpunkt liegt vor dem Endzeitpunkt!");
              _compositeZeitGruppenInhalt.layout();
            }
            else
            {
              _labelInfo.setText("");
              _compositeZeitGruppenInhalt.layout();
            }

            // fireEvent();
          }
        }

      });

      _schaltflaecheHinzufuegen = new Button(_returnComposite, SWT.NONE);
      _schaltflaecheHinzufuegen.setText("+");
      _schaltflaecheLoeschen = new Button(_returnComposite, SWT.NONE);
      _schaltflaecheLoeschen.setText("-");

      _schaltflaecheHinzufuegen.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
        public void mouseDown(org.eclipse.swt.events.MouseEvent e)
        {
          aktionZeitgruppenzeileAnhaengen();
        }
      });

      _schaltflaecheLoeschen.addMouseListener(new org.eclipse.swt.events.MouseAdapter() {
        public void mouseDown(org.eclipse.swt.events.MouseEvent e)
        {
          aktionZeitgruppenzeileLoeschen();
        }
      });

      GridData gridData = new GridData();
      gridData.horizontalAlignment = GridData.FILL;
      gridData.grabExcessHorizontalSpace = false;
      gridData.grabExcessVerticalSpace = false;
      gridData.verticalAlignment = GridData.CENTER;
      _returnComposite.setLayoutData(gridData);

      GridLayout gridLayout = new GridLayout();
      gridLayout.numColumns = 7;
      _returnComposite.setLayout(gridLayout);

      return _returnComposite;
    }

  }

  private void loescheZeitGruppenZeile(ZeitGruppenZeile zgz)
  {
    if (_listeZeitGruppenZeilen.contains(zgz) && _listeZeitGruppenZeilen.size() > 1)
    {
      _listeZeitGruppenZeilen.remove(zgz);
      zgz.dispose();
      _compositeZeitGruppenInhalt.layout();
    }
  }

  private void einfuegenZeitGruppenZeileNachPosition(ZeitGruppenZeile zg)
  {
    generiereZeitgruppeLeer();

    LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe> _dsAlt = new LinkedList<KalenderZeitFilterDatenStrukturZeitGruppe>();

    int idx = _listeZeitGruppenZeilen.indexOf(zg) + 1;

    if (idx < _listeZeitGruppenZeilen.size())
    {
      if (idx > -1)
      {
        for (int i = idx; i < (_listeZeitGruppenZeilen.size() - 1); i++)
        {
          _dsAlt.add(_listeZeitGruppenZeilen.get(i).getDatenStrukturZeitGruppe());
        }
      }

      int i = idx + 1;
      Iterator<KalenderZeitFilterDatenStrukturZeitGruppe> it = _dsAlt.iterator();
      while (it.hasNext())
      {
        KalenderZeitFilterDatenStrukturZeitGruppe datenStrukturZeitGruppe = it.next();
        _listeZeitGruppenZeilen.get(i).setDatenStrukturZeitGruppe(datenStrukturZeitGruppe);
        i++;
      }

      /*
       * Neue Zeile mit Nullstellung erzeugen
       */
      GregorianCalendar gcNullStellung = new GregorianCalendar();
      gcNullStellung.set(Calendar.HOUR_OF_DAY, 0);
      gcNullStellung.set(Calendar.MINUTE, 0);
      gcNullStellung.set(Calendar.SECOND, 0);
      gcNullStellung.set(Calendar.MILLISECOND, 0);

      _listeZeitGruppenZeilen.get(idx).setDatenStrukturZeitGruppe(
          new KalenderZeitFilterDatenStrukturZeitGruppe(gcNullStellung.getTimeInMillis(), gcNullStellung
              .getTimeInMillis()));
    }

  }

  /**
   * Generiert eine leere Zeitgruppe
   * 
   */
  private void generiereZeitgruppeLeer()
  {
    Date d = new Date();
    long nu = d.getTime();

    KalenderZeitFilterDatenStrukturZeitGruppe kz = new KalenderZeitFilterDatenStrukturZeitGruppe(nu, nu);
    _DatenStruktur.fuegeZeitGruppeHinzu(kz);
    ZeitGruppenZeile zgz = new ZeitGruppenZeile(_compositeZeitGruppenInhalt, kz);
    zgz.generiereComposite();

    _listeZeitGruppenZeilen.add(zgz);
    _compositeZeitGruppenInhalt.layout();
  }

  @Override
  public void dispose()
  {
    // TODO Auto-generated method stub
    super.dispose();
    ObjektSpeicher.getInstanz().getSystemObjekte().clear();
    DaVDataUpdateListener.getInstanz().removeDaVDataUpdateListener(this);
    IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
    if (page != null)
    {

      _listeZeitGruppenZeilen = null;
      _DatenStruktur = null;
      page.hideView(this);
    }
  }

  private void erzeugeTabelleMoeglicheObjekte()
  {
    _tvMoeglicheObjekte = new TableViewer(composite41, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL
        | SWT.V_SCROLL);
    _tvMoeglicheObjekte.setData("name", "viewer1");
    _tvMoeglicheObjekte.setComparator(new ViewerComparator() {
      @Override
      public int compare(Viewer viewer, Object e1, Object e2)
      {
        if (e1 instanceof SystemObject && e2 instanceof SystemObject)
        {
          return ((SystemObject)e1).getName().compareToIgnoreCase(((SystemObject)e2).getName());
        }

        throw new IllegalArgumentException("Not comparable: " + e1 + " " + e2);
      }
    });

    // _ts1 = new TableSort (_tvMoeglicheObjekte);

    final TableColumn tc0 = new TableColumn(_tvMoeglicheObjekte.getTable(), SWT.LEFT);
    tc0.setText("Name (Pid)");
    tc0.setWidth(300);
    // tc0.addListener(SWT.Selection, new Listener()
    // {
    // public void handleEvent(Event event)
    // {
    // _ts1.aendereSortierung();
    // _ts1.sortiere (0, TableSort.SORT.STRING);
    // }
    // });
    _tvMoeglicheObjekte.getTable().setHeaderVisible(true);

    _tvMoeglicheObjekte.getTable().setLinesVisible(true);
    _tvMoeglicheObjekte.setContentProvider(new CP());
    _tvMoeglicheObjekte.setLabelProvider(new LP());
    _tvMoeglicheObjekte.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof SystemObject)
        {
          _text.setText(_tvMoeglicheObjekte.getTable().getSelectionCount() + " / "
              + _tvMoeglicheObjekte.getTable().getItemCount());
          composite44.layout(true);
        }
      }
    });
  }

  private void erzeugeTabelleAngewaehlteObjekte()
  {
    _tvAusgewaehlteObjekte = new TableViewer(composite42, SWT.BORDER | SWT.FULL_SELECTION | SWT.MULTI | SWT.H_SCROLL
        | SWT.V_SCROLL);
    _tvAusgewaehlteObjekte.setData("name", "viewer2");
    _tvAusgewaehlteObjekte.setComparator(new ViewerComparator() {
      @Override
      public int compare(Viewer viewer, Object e1, Object e2)
      {
        if (e1 instanceof SystemObject && e2 instanceof SystemObject)
        {
          return ((SystemObject)e1).getName().compareToIgnoreCase(((SystemObject)e2).getName());
        }

        throw new IllegalArgumentException("Not comparable: " + e1 + " " + e2);
      }
    });

    // _ts2 = new TableSort (_tvAusgewaehlteObjekte);

    final TableColumn tc0 = new TableColumn(_tvAusgewaehlteObjekte.getTable(), SWT.LEFT);
    tc0.setText("Name (Pid)");
    tc0.setWidth(300);
    // tc0.addListener(SWT.Selection, new Listener()
    // {
    // public void handleEvent(Event event)
    // {
    // _ts2.aendereSortierung();
    // _ts2.sortiere (0, TableSort.SORT.STRING);
    // }
    // });
    _tvAusgewaehlteObjekte.getTable().setHeaderVisible(true);

    _tvAusgewaehlteObjekte.getTable().setLinesVisible(true);
    _tvAusgewaehlteObjekte.setContentProvider(new CP());
    _tvAusgewaehlteObjekte.setLabelProvider(new LP());
    _tvAusgewaehlteObjekte.addSelectionChangedListener(new ISelectionChangedListener() {
      public void selectionChanged(SelectionChangedEvent event)
      {
        IStructuredSelection selection = (IStructuredSelection)event.getSelection();
        Object firstElement = selection.getFirstElement();

        if (firstElement instanceof SystemObject)
        {
          _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
              + _tvAusgewaehlteObjekte.getTable().getItemCount());
          composite43.layout(true);
        }
      }
    });
  }

  private class CP implements IStructuredContentProvider
  {
    public Object[] getElements(Object inputElement)
    {
      return ((List)inputElement).toArray();
    }

    public void dispose()
    {
    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {

    }

  }

  private class LP implements ITableLabelProvider
  {

    public Image getColumnImage(Object element, int columnIndex)
    {
      return null;
    }

    public String getColumnText(Object object, int columnIndex)
    {
      String returnString = null;

      switch (columnIndex)
      {
        case 0:
//          returnString = ((SystemObject)object).getName();
          returnString = object.toString();
          break;
        // case 1:
        // returnString = ((SystemObject)object).getName();
        // break;
        default:
          returnString = "";

      }

      return returnString;
    }

    public void addListener(ILabelProviderListener listener)
    {
    }

    public void dispose()
    {
    }

    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  /**
   * This method initializes composite41
   * 
   */
  private void createComposite41()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = false;
    gridData.verticalAlignment = GridData.FILL;
    gridData.heightHint = 200;
    composite41 = new Composite(composite, SWT.NONE);
    composite41.setLayout(new FillLayout());
    composite41.setLayoutData(gridData);
  }

  /**
   * This method initializes composite42
   * 
   */
  private void createComposite42()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = false;
    gridData.verticalAlignment = GridData.FILL;
    gridData.heightHint = 200;

    composite42 = new Composite(composite, SWT.NONE);
    composite42.setLayout(new FillLayout());
    composite42.setLayoutData(gridData);
  }

  /**
   * This method initializes composite43
   * 
   */
  private void createComposite43()
  {
    GridData gridData15 = new GridData();
    gridData15.horizontalAlignment = GridData.FILL;
    gridData15.grabExcessHorizontalSpace = true;
    gridData15.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout4 = new GridLayout();
    gridLayout4.numColumns = 3;
    gridLayout4.marginHeight = 0;
    gridLayout4.marginWidth = 0;
    // GridLayout gridLayout2 = new GridLayout();
    // gridLayout2.numColumns = 2;
    GridData gridData9 = new GridData();
    gridData9.grabExcessHorizontalSpace = true;
    gridData9.verticalAlignment = GridData.CENTER;
    gridData9.horizontalAlignment = GridData.FILL;
    composite43 = new Composite(composite, SWT.NONE);
    composite43.setLayoutData(gridData9);
    createComposite45();
    composite43.setLayout(gridLayout4);
    // label3 = new Label(composite43, SWT.NONE);
    // label3.setText("Referenz");
    // text2 = new Text(composite43, SWT.BORDER);
    // text2.setLayoutData(gridData15);
    // text2.setEnabled(false);
    // label2 = new Label(composite43, SWT.NONE);
    // label2.setText("0/0");
    _text2 = new Text(composite43, SWT.BORDER);
    _text2.setEditable(false);
    _text2.setText("0 / 0");
    button51 = new Button(composite43, SWT.BUTTON5);
    button51.setText("X");
    button51.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        _tvAusgewaehlteObjekte.setSelection(null);

        _text2.setText(_tvAusgewaehlteObjekte.getTable().getSelectionCount() + " / "
            + _tvAusgewaehlteObjekte.getTable().getItemCount());
        composite43.layout(true);
      }

    });
    // button51.setEnabled(false);
  }

  /**
   * This method initializes composite44
   * 
   */
  private void createComposite44()
  {
    GridData gridData14 = new GridData();
    gridData14.horizontalAlignment = GridData.BEGINNING;
    gridData14.grabExcessVerticalSpace = false;
    gridData14.grabExcessHorizontalSpace = false;
    gridData14.verticalAlignment = GridData.CENTER;
    GridData gridData13 = new GridData();
    gridData13.horizontalAlignment = GridData.FILL;
    gridData13.grabExcessHorizontalSpace = true;
    gridData13.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout2 = new GridLayout();
    gridLayout2.numColumns = 4;
    gridLayout2.marginWidth = 0;
    gridLayout2.marginHeight = 0;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;
    composite44 = new Composite(composite, SWT.NONE);
    composite44.setLayoutData(gridData1);
    composite44.setLayout(gridLayout2);
    text = new Text(composite44, SWT.BORDER);
    text.setLayoutData(gridData13);
    button4 = new Button(composite44, SWT.NONE);
    button4.setText("Filtern");
    button4.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        _searchFilter = new ViewerFilter() {
          @Override
          public boolean select(Viewer viewer, Object parentElement, Object element)
          {
            return ((SystemObject)element).toString().contains(text.getText());
          }
        };
        _tvMoeglicheObjekte.setFilters(new ViewerFilter[] { _searchFilter });
        _text.setText(_tvMoeglicheObjekte.getTable().getSelectionCount() + " / "
            + _tvMoeglicheObjekte.getTable().getItemCount());
        composite44.layout(true);
      }
    });
    _text = new Text(composite44, SWT.BORDER);
    _text.setEditable(false);
    _text.setText("0 / 0");
    // label = new Label(composite44, SWT.NONE);
    // label.setText("0/0");
    // text.setLayoutData(gridData14);
    button5 = new Button(composite44, SWT.NONE);
    button5.setText("X");
    button5.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        // if (_searchFilter != null)
        // _tvBetriebsMeldungen1.removeFilter(_searchFilter);
        _tvMoeglicheObjekte.setSelection(null);

        _text.setText(_tvMoeglicheObjekte.getTable().getSelectionCount() + " / "
            + _tvMoeglicheObjekte.getTable().getItemCount());
        composite44.layout(true);
      }

    });
  }

  /**
   * This method initializes composite45
   * 
   */
  private void createComposite45()
  {
    GridData gridData16 = new GridData();
    gridData16.horizontalAlignment = GridData.FILL;
    gridData16.grabExcessHorizontalSpace = true;
    gridData16.verticalAlignment = GridData.CENTER;
    composite45 = new Composite(composite43, SWT.NONE);
    composite45.setLayout(new GridLayout());
    composite45.setLayoutData(gridData16);
  }

  /**
   * Hier werden die Daten der Spalten des "original" TableItem[] in einen MyComperatorArray umkopiert und darauf hin
   * wird dieser mittels der Array.sort(Object[] o) Methode sortiert. Da sich der MyComperator die ursprngliche
   * Position in der Tabelle gemerkt hat, kann man nun ganz einfach die entsprechenden Werte im Original Array tauschen
   * und schon ist man fertig.
   * 
   * @param tableItems
   */
  protected final void sortTable(Table table, TableItem[] tableItems, int colPos, BaseComapator[][] comparables,
      boolean sortFlag)
  {

    // SortFlag bei allen richtig setzen ... leider kann ich bei inneren
    // keine static Variablen anlegen ansonsten wre das hier unntig...
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      comparables[colPos][i]._sortFlag = sortFlag;
    }

    Arrays.sort(comparables[colPos]);

    String[] tmp;
    String[] textBuffer = null;
    textBuffer = new String[comparables.length];
    for (int i = 0; i < comparables[colPos].length; i++)
    {
      int rowNum = comparables[colPos][i]._oldRowNum;
      tmp = comparables[colPos][i]._textBuffer;
      tableItems[rowNum].setText(comparables[colPos][rowNum]._textBuffer);
      tableItems[i].setText(tmp);
    }

  }

  /**
   * Basisklasse fr alle weiteren Comparatoren...
   * 
   */
  class BaseComapator implements Comparable
  {

    String[] _textBuffer;

    int _oldRowNum;

    int _colPos;

    boolean _sortFlag;

    public BaseComapator(String[] textBuffer, int oldRowNum, int colPos)
    {
      _textBuffer = textBuffer;
      _oldRowNum = oldRowNum;
      _colPos = colPos;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(Object o)
    {

      return 0;
    }
  }

  /**
   * 
   * @author Administrator Eigener Comparator der das Vergleichen der einzelnen Tabellenzeilen vornimmt.
   */
  class StringComparator extends BaseComapator implements Comparable
  {

    /**
     * Konstruktor ...
     * 
     * @param textBuffer
     *          : Der Text der Zeile
     * @param rowNum
     *          : Die Zeilennr der Tabellenzeile
     * @param sortFlag
     *          : Aufsteigend false, Absteigend true
     * @param colPos
     *          : Die Spalte nach der Sortiert werden soll
     */

    public StringComparator(String[] textBuffer, int rowNum, int colPos)
    {
      super(textBuffer, rowNum, colPos);
    }

    public final int compareTo(Object arg0)
    {
      int ret = _textBuffer[_colPos].compareTo(((StringComparator)arg0)._textBuffer[_colPos]);
      return _sortFlag ? ret : ret * -1;
    }

  }

  public Text getText()
  {
    return _text;
  }

  private void aktionHilfe()
  {
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(HELP_ID);
  }
}
