/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8;

import java.util.ArrayList;
import java.util.List;
import java.util.TreeMap;

import org.eclipse.birt.chart.device.IDeviceRenderer;
import org.eclipse.birt.chart.model.ChartWithAxes;
import org.eclipse.birt.chart.model.attribute.Location;
import org.eclipse.birt.chart.model.component.Axis;
import org.eclipse.birt.chart.model.data.NumberDataElement;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Transform;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import com.bitctrl.lib.eclipse.birt.chart.ChartCanvas;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.Activateable;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenZugEintrag;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.SystemObjektContainer;

/**
 * Canvas, das zustzlich zum Birt-Chart noch weitere Objekte drber malt.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilCanvas.java,v 1.3 2014/04/17 15:51:18 Pittner Exp $
 */
public class StreckenprofilCanvas extends ChartCanvas {

	/**
	 * Die Minimal erlaubt Hhe fr die Darstellung eines Streckenprofils in
	 * diesem {@link StreckenprofilCanvas}. Bei allem was kleiner ist, wird die
	 * Gre auf diesen Minimalwert verndert.
	 */
	public static final int MIN_HOEHE = 500;

	/**
	 * Die Minimal erlaubt Breite fr die Darstellung eines Streckenprofils in
	 * diesem {@link StreckenprofilCanvas}.Bei allem was kleiner ist, wird die
	 * Gre auf diesen Minimalwert verndert.
	 */
	public static final int MIN_BREITE = 700;

	/**
	 * Die Maximal erlaubt Breite fr die Darstellung eines Streckenprofils in
	 * diesem {@link StreckenprofilCanvas}.Bei allem was grer ist, wird die
	 * Gre auf diesen Maximalwert verndert.
	 */
	public static final int MAX_BREITE = 32768;

	/**
	 * {@link FontData} mit der verwendeten Schrift fr die Beschriftung der
	 * Messquerschnitte und der Straenknoten.
	 */
	private static final FontData STANDARD_SCHRIFTART = new FontData("Sans",
			10, SWT.NORMAL);

	/**
	 * Map, die alle {@link Activateable} nach ihrer Position innerhalb des
	 * Streckenprofils sortiert und Mq's mit gleicher Position in einer Liste
	 * speichert.
	 */
	private final TreeMap<Double, List<Activateable>> activatableLokalisationsMap = new TreeMap<Double, List<Activateable>>();

	/**
	 * Die Momentan maximal ermittelte Breite der MQ Beschriftung.
	 */
	private int maxWortbreite;

	/**
	 * Bestimmt die Breite des {@link StreckenprofilCanvas} in Pixel.
	 */
	private float canvasBreite = MIN_BREITE;

	/**
	 * Flag, das anzeigt, ob die Namen der MQ's und Streckenknoten unterhalb des
	 * Charts angezeigt werden sollen. Default ist <code>true</code>.
	 */
	private boolean zeichneKnotenUndMQNamen = true;

	/**
	 * Erzeugt ein neues {@link StreckenprofilCanvas}, mit dem zustzlich die
	 * Beschriftung der MQ in einem BirtChart realisiert wird.
	 * 
	 * @param parent
	 *            das Parent {@link Composite}.
	 * @param style
	 *            das SWT - Style
	 */
	public StreckenprofilCanvas(final Composite parent, final int style) {
		super(parent, style);
		setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WHITE));
		addPaintListener(new PaintListener() {

			@Override
			public void paintControl(final PaintEvent e) {
				BusyIndicator.showWhile(getDisplay(), new Runnable() {
					public void run() {
						if (null != chart && zeichneKnotenUndMQNamen) {
							final GC gc = e.gc;
							gc.setClipping(getClientArea());
							ermittleMaximaleWortLaenge(gc);
							maleMqNamen(gc);
						}
					}
				});
			}
		});
	}

	/**
	 * Ermittelt die maximale Wortlnge fr die Beschriftung der Messqerschnitte
	 * und der Straenknoten.
	 * 
	 * @param gc
	 *            der {@link GC}.
	 */
	private void ermittleMaximaleWortLaenge(final GC gc) {

		final Font schriftart = new Font(gc.getDevice(), STANDARD_SCHRIFTART);
		gc.setFont(schriftart);

		for (final Double key : activatableLokalisationsMap.keySet()) {
			// maxWortbreite = 0;
			for (final Activateable activatable : activatableLokalisationsMap
					.get(key)) {
				if (activatable instanceof SystemObjektContainer<?>) {
					final SystemObjektContainer<?> sysObjC = (SystemObjektContainer<?>) activatable;
					final String name = sysObjC.getModellObjekt().getName();
					maxWortbreite = Math.max(maxWortbreite,
							gc.textExtent(name).x);
				}
			}
		}
		schriftart.dispose();
	}

	/**
	 * Malt die Namen der Messquerschnitt und Streckenknoten unter die X-Achse.
	 * 
	 * @param gc
	 *            der GraphicsContext auf den gemalt wird.
	 */
	private void maleMqNamen(final GC gc) {

		final Location ende = getXAchseEnd();
		final Location start = getXAchseStart();
		final int statischerOffsetY = 40;
		if (start != null && ende != null) {

			// gc.setAdvanced(true);
			final Font schriftart = new Font(gc.getDevice(),
					STANDARD_SCHRIFTART);
			gc.setFont(schriftart);
			gc.setForeground(gc.getDevice().getSystemColor(SWT.COLOR_BLACK));

			for (final Double key : activatableLokalisationsMap.keySet()) {
				final Double xPosition = calculateX(key);
				if (xPosition != null) {
					final StringBuilder text = new StringBuilder();
					// maxWortbreite = 0;
					for (final Activateable activatable : activatableLokalisationsMap
							.get(key)) {
						if (activatable instanceof SystemObjektContainer<?>) {
							final SystemObjektContainer<?> sysObjC = (SystemObjektContainer<?>) activatable;
							final String name = sysObjC.getModellObjekt()
									.getName();
							text.append(name + "\n");
						}
					}

					// gc.drawText(text.toString(), (int) xPosition -
					// maxWortbreite
					// / 2, (int) start.getY() + statischerOffsetY,
					// SWT.DRAW_TRANSPARENT | SWT.DRAW_DELIMITER);

					// Text drehen
					final Transform transe = new Transform(gc.getDevice());

					transe.translate(xPosition.floatValue(),
							(float) start.getY() + statischerOffsetY);
					transe.rotate(45);
					gc.setTransform(transe);
					gc.drawText(text.toString(), 0, 0, SWT.DRAW_TRANSPARENT
							| SWT.DRAW_DELIMITER);
					gc.setTransform(null);
					transe.dispose();
				}
			}
			schriftart.dispose();
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Control#getSize()
	 */
	@Override
	public Point getSize() {

		final Point p = super.getSize();
		if (canvasBreite <= MIN_BREITE) {
			canvasBreite = MIN_BREITE;
		}
		p.y = Math.max(p.y, MIN_HOEHE);

		p.x = (int) canvasBreite;

		if (zeichneKnotenUndMQNamen) {
			p.y -= (maxWortbreite / 1.4);
		}

		return p;
	}

	/**
	 * Liefert die Startposition der X-Achse.
	 * 
	 * @return die {@link Location} des Startpunktes der X-Achse.
	 */
	public Location getXAchseStart() {
		Location result = null;
		final IDeviceRenderer renderer = getRender();
		if (renderer instanceof StreckenprofilDeviceRenderer) {
			result = ((StreckenprofilDeviceRenderer) renderer).getxAchseStart();
		}
		return result;
	}

	/**
	 * Liefert die Endeposition der X-Achse.
	 * 
	 * @return die {@link Location} des Endpunkte der X-Achse.
	 */
	public Location getXAchseEnd() {
		Location result = null;
		final IDeviceRenderer renderer = getRender();
		if (renderer instanceof StreckenprofilDeviceRenderer) {
			result = ((StreckenprofilDeviceRenderer) renderer).getxAchseEnde();
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.widgets.Control#redraw()
	 */
	@Override
	public void redraw() {
		final IDeviceRenderer renderer = getRender();
		if (renderer instanceof StreckenprofilDeviceRenderer) {
			// nach einem erneuten Malen, mu die X-Achse neu bestimmt werden.
			((StreckenprofilDeviceRenderer) renderer).resetXAchse();
		}
		super.redraw();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.bitctrl.lib.eclipse.birt.chart.ChartCanvas#getRendererDeviceId()
	 */
	@Override
	protected String getRendererDeviceId() {
		return "de.bsvrz.buv.plugin.streckenprofil.StreckenprofilDeviceRenderer";
	}

	/**
	 * Fllt eine Menge von {@link Activateable}en innerhalb des
	 * {@link StreckenprofilCanvas}, aus der dann die Namen der
	 * {@link Activateable} unterhalb der X-Achse gemalt werden.
	 * 
	 * @param neueActivatables
	 *            die Menge der {@link Activateable}, deren Namen im dem
	 *            Birt-Chart angezeigt werden sollen.
	 */
	public void fuelleStreckenObjektListe(
			final List<? extends Activateable> neueActivatables) {
		activatableLokalisationsMap.clear();
		for (final Activateable activatable : neueActivatables) {
			if (activatable instanceof StreckenZugEintrag) {
				final StreckenZugEintrag mq = (StreckenZugEintrag) activatable;
				final Double naechstkleinererNachbar = activatableLokalisationsMap
						.lowerKey(mq.getWegBisher());
				final Double naechstgroessererNachbar = activatableLokalisationsMap
						.higherKey(mq.getWegBisher());
				if (activatableLokalisationsMap.containsKey(mq.getWegBisher())) {
					activatableLokalisationsMap.get(mq.getWegBisher()).add(
							activatable);
				} else if ((naechstkleinererNachbar != null && mq
						.getWegBisher() - 100 <= naechstkleinererNachbar)) {
					activatableLokalisationsMap.get(naechstkleinererNachbar)
							.add(activatable);

				} else if ((naechstgroessererNachbar != null && mq
						.getWegBisher() + 100 >= naechstgroessererNachbar)) {
					activatableLokalisationsMap.get(naechstgroessererNachbar)
							.add(activatable);
				} else {
					final List<Activateable> mql = new ArrayList<Activateable>();
					mql.add(activatable);
					activatableLokalisationsMap.put(mq.getWegBisher(), mql);
				}
			}
		}
		regenerateChart();
	}

	/**
	 * Errechnet die Position in X - Richtung.
	 * 
	 * @param wertX
	 *            die relative Position innerhalb des Streckenprofils.
	 * @return die absolute X- Position innerhalb des
	 *         {@link org.eclipse.swt.widgets.Canvas}
	 */
	protected Double calculateX(final Double wertX) {
		final Axis xAchse = ((ChartWithAxes) getChart()).getPrimaryBaseAxes()[0];

		if (xAchse.getScale().getMin() == null
				|| xAchse.getScale().getMax() == null) {
			return null;
		}
		final double min = ((NumberDataElement) xAchse.getScale().getMin())
				.getValue();
		final double max = ((NumberDataElement) xAchse.getScale().getMax())
				.getValue();

		if (wertX < min || wertX > max) {
			// auerhalb der X-Achsen breite
			return null;
		}

		final Double range = max - min;
		Double relativeWertX = 0d;
		if (wertX > min) {
			relativeWertX = (wertX - min) / range;
		}

		final Double pixelBreiteXAchse = getXAchseEnd().getX()
				- getXAchseStart().getX();

		final Double result = (pixelBreiteXAchse * relativeWertX)
				+ getXAchseStart().getX();
		return result;
	}

	/**
	 * Liefert die aktuelle Breite des {@link ChartCanvas} in Pixel.
	 * 
	 * @return the zoomFaktor
	 */
	public final float getCanvasBreite() {
		return canvasBreite;
	}

	/**
	 * Setzt die Breite des {@link ChartCanvas} in Pixel und veranlasst, dass es
	 * neu gemalt wird.
	 * 
	 * @param neueBreite
	 *            die neue Breite des {@link StreckenprofilCanvas}.
	 */
	public final void setCanvasBreite(final float neueBreite) {
		canvasBreite = neueBreite;
		regenerateChart();
	}

	/**
	 * Liefert die momentan maximale Breite einer Bezeichnung am Fue des
	 * Diagramms in Pixeln.
	 * 
	 * @return the maxWortbreite
	 */
	public final int getMaxWortbreite() {
		return maxWortbreite;
	}

	/**
	 * Setzt das Flag {@link StreckenprofilCanvas#zeichneKnotenUndMQNamen}.
	 * 
	 * @return the zeichneKnotenUndMQNamen
	 */
	public final boolean isZeichneKnotenUndMQNamen() {
		return zeichneKnotenUndMQNamen;
	}

	/**
	 * Liest das Flag {@link StreckenprofilCanvas#zeichneKnotenUndMQNamen}.
	 * 
	 * @param zeichneKnotenUndMQNamen
	 *            the zeichneKnotenUndMQNamen to set
	 */
	public final void setZeichneKnotenUndMQNamen(
			final boolean zeichneKnotenUndMQNamen) {
		this.zeichneKnotenUndMQNamen = zeichneKnotenUndMQNamen;
	}
}
