/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.actions;

import org.eclipse.emf.ecore.util.EcoreUtil;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.StreckenprofilEinstellungen;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.rw.bitctrl.eclipse.modell.emf.EMFModellSaveAction;

/**
 * Action zum Speichern eines {@link StreckenprofilEditorModel}.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilSaveAction.java,v 1.3 2014/04/17 15:51:11 Pittner Exp $
 */
public class StreckenprofilSaveAction extends
		EMFModellSaveAction<StreckenprofilEditorModel> {

	/**
	 * Das Streckenprofil, das gespeichert werden soll.
	 */
	private final StreckenprofilEditorModel model;

	/**
	 * Flag, das anzeigt, ob das Speichern erfolgreich war.
	 */
	private boolean erfolgreichGespeichert;

	/**
	 * Erzeugt eine neue Aktion zum Speichern eines Streckenprofils.
	 * 
	 * @param model
	 *            das {@link StreckenprofilEditorModel}, das gespeichert werden
	 *            soll.
	 */
	public StreckenprofilSaveAction(final StreckenprofilEditorModel model) {
		super(StreckenprofilEditorModel.class);
		this.model = model;
	}

	@Override
	protected StreckenprofilEinstellungen createModelEinstellungen() {
		return StreckenprofilEinstellungen.getInstance();
	}

	@Override
	protected StreckenprofilEditorModel getObject(final String newName) {
		if (model != null && model.getStreckenZug() != null) {
			model.getStreckenZug().setName(newName);
		}
		return (StreckenprofilEditorModel) EcoreUtil.copy(model);
	}

	@Override
	protected String getCurrentName() {
		return model.getStreckenZug().getName();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.bsvrz.buv.rw.bitctrl.eclipse.modell.ModellSaveAction#saveHook(java
	 * .lang.String)
	 */
	@Override
	protected void saveHook(final String newName) {
		super.saveHook(newName);
		erfolgreichGespeichert = true;
	}

	/**
	 * Zeigt an, ob des Speichern erfolgreich war.
	 * 
	 * @return the erfolgreichGespeichert
	 */
	public final boolean isErfolgreichGespeichert() {
		return erfolgreichGespeichert;
	}

}
