/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.diagram;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import com.swarco.sts.chart.control.TAxis;
import com.swarco.sts.chart.control.TChart;
import com.swarco.sts.chart.control.TSeries;
import com.swarco.sts.chart.control.event.TActionEvent;
import com.swarco.sts.chart.control.label.defaults.TPdfCapableLabel;
import com.swarco.sts.chart.control.legend.interfaces.TSeriesLegendAgent;
import com.swarco.sts.chart.control.utilities.CHighResGraphics2D;
import com.swarco.sts.chart.control.utilities.CUtilities;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.daten.StreckenProfilKonfigurationData.BezeichnungGeschwindigkeitBeschraenkung;

public class StreckenProfilGeschwindigkeitBeschraenkungLegende extends TSeriesLegendAgent implements IStreckenProfilLegende
{
  private final static int DATENREIHE_LEER = 0;
  private final static int DATENREIHE_SICHTBAR = 1;
  private final static int DATENREIHE_NICHT_SICHTBAR = 2;

  private JPanel legendeAnzeige = null;
  private DatenreiheDarstellungInformation datenreiheDarstellungInformation = null;
  private TPdfCapableLabel beschriftungBezeichner = null;
  private Dimension bezeichnerAusgangsgroesse = null;
  private TPdfCapableLabel beschriftungFormelzeichen = null;
  private Dimension formelzeichenAusgangsgroesse = null;
  private TPdfCapableLabel beschriftungWert = null;
  private Dimension wertAusgangsgroesse = null;
  private TPdfCapableLabel beschriftungEinheitenzeichen = null;
  private Dimension einheitenzeichenAusgangsgroesse = null;
  private Rectangle sichtbarerRahmen = new Rectangle(0, 0, 0, 0);
  private Vector datenreiheListe = null;
  private BufferedImage renderedImage = null;
  
  public StreckenProfilGeschwindigkeitBeschraenkungLegende(final StreckenProfilZuDiagrammZwischenschicht2 zwischenschicht, Font schriftart, int anzahlStellenGanzzahl, int anzahlStellenNachkomma)
  {
    datenreiheDarstellungInformation = new DatenreiheDarstellungInformation(schriftart);
    
    legendeAnzeige = new JPanel();
    legendeAnzeige.setOpaque(false);
    legendeAnzeige.setBorder(BorderFactory.createEmptyBorder(1, 0, 0, 0));
    legendeAnzeige.setLayout(new BorderLayout(0, 0));
    legendeAnzeige.add(datenreiheDarstellungInformation, BorderLayout.WEST);

    JPanel anzeige = new JPanel();
    anzeige.setOpaque(false);
    anzeige.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 0));

    beschriftungBezeichner = new TPdfCapableLabel();
    beschriftungBezeichner.setOpaque(false);
    beschriftungBezeichner.setFont(schriftart);
    anzeige.add(beschriftungBezeichner);
    
    beschriftungFormelzeichen = new TPdfCapableLabel();
    beschriftungFormelzeichen.setOpaque(false);
    beschriftungFormelzeichen.setFont(schriftart);
    anzeige.add(beschriftungFormelzeichen);

    datenreiheListe = new Vector();
    
    String muster = "";
    if(anzahlStellenGanzzahl > 0)
    {
      for(int i = 0; i < anzahlStellenGanzzahl; i++)
      {
        muster += "8";
      }
    }
    if(anzahlStellenNachkomma > 0)
    {
      muster += ",";
      for(int i = 0; i < anzahlStellenNachkomma; i++)
      {
        muster += "8";
      }
    }
    beschriftungWert = new TPdfCapableLabel(muster);
    beschriftungWert.setOpaque(false);
    beschriftungWert.setFont(schriftart);     
    beschriftungWert.setHorizontalAlignment(JLabel.RIGHT);
    Dimension bevorzugteGroesse = (Dimension)beschriftungWert.getPreferredSize().clone();
    beschriftungWert.setPreferredSize(bevorzugteGroesse);
    beschriftungWert.setText("-");
    anzeige.add(beschriftungWert);

    JPanel luecke = new JPanel();
    luecke.setOpaque(false);
    luecke.setPreferredSize(new Dimension(4, 1));
    anzeige.add(luecke);
    
    beschriftungEinheitenzeichen = new TPdfCapableLabel();
    beschriftungEinheitenzeichen.setOpaque(false);
    beschriftungEinheitenzeichen.setFont(schriftart);
    anzeige.add(beschriftungEinheitenzeichen);

    legendeAnzeige.add(anzeige, BorderLayout.CENTER);
    legendeAnzeige.addMouseListener(new MouseAdapter()
    {
      public void mouseClicked(MouseEvent mausEreignis)
      {
        if(SwingUtilities.isLeftMouseButton(mausEreignis))
        {
          if(datenreiheDarstellungInformation.getZustand() != StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_LEER)
          {
            boolean sichtbar = false;
            for(int i = 0; i < datenreiheListe.size(); i++)
            {
              TSeries datenreihe = (TSeries)datenreiheListe.elementAt(i);
              if(i == 0)
              {
                sichtbar = datenreihe.isVisible();
                if(!sichtbar)
                {
                  zwischenschicht.entfernePseudoDatenreihe();
                }
              }
              datenreihe.setVisible(!sichtbar);
            }
          }
        }
      }
    });

    wertAusgangsgroesse = (Dimension)beschriftungWert.getPreferredSize().clone();
    wertAusgangsgroesse.width += 3;
  }
  
  public void attachToSeries(TSeries series)
  {
    datenreiheListe.add(series);
    attached();
  }

  public void detachFromSeries(TSeries series)
  {
    detached();
    datenreiheListe.remove(series);
  }
  
  public Rectangle getSichtbarerRahmen()
  {
    sichtbarerRahmen.width = beschriftungEinheitenzeichen.getBounds().x + beschriftungEinheitenzeichen.getBounds().width - sichtbarerRahmen.x;
    sichtbarerRahmen.height = legendeAnzeige.getSize().height;
    
    return (sichtbarerRahmen);
  }
  
  public Dimension getBevorzugteBezeichnerGroesse()
  {
    return ((Dimension)bezeichnerAusgangsgroesse.clone());
  }

  public void setBevorzugteBezeichnerGroesse(Dimension preferredIdentifierSize)
  {
    beschriftungBezeichner.setPreferredSize(preferredIdentifierSize);
  }

  public Dimension getBevorzugteFormelzeichenGroesse()
  {
    return ((Dimension)formelzeichenAusgangsgroesse.clone());
  }
  
  public void setBevorzugteFormelzeichenGroesse(Dimension preferredQuantitySize)
  {
    beschriftungFormelzeichen.setPreferredSize(preferredQuantitySize);
  }
  
  public Dimension getBevorzugteWertGroesse()
  {
    return ((Dimension)wertAusgangsgroesse.clone());
  }

  public void setBevorzugteWertGroesse(Dimension preferredValueSize)
  {
    beschriftungWert.setPreferredSize(preferredValueSize);
  }

  public Dimension getBevorzugteEinheitenzeichenGroesse()
  {
    return ((Dimension)einheitenzeichenAusgangsgroesse.clone());
  }

  public void setBevorzugteEinheitenzeichenGroesse(Dimension preferredUnitSize)
  {
    beschriftungEinheitenzeichen.setPreferredSize(preferredUnitSize);
  }

  public void setDatenreiheTyp(int typ)
  {
  }
  
  public void attached()
  {
    boolean hatDaten = false;
    boolean sichtbar = false;
    Color basisfarbe = null;
    String beschriftung = null;
    String formelzeichen = null;
    String einheitenzeichen = null;
    TAxis yAchse = null;
    for(int i = 0; i < datenreiheListe.size(); i++)
    {
      TSeries datenreihe = (TSeries)datenreiheListe.elementAt(i);
      if(i == 0)
      {
        sichtbar = datenreihe.isVisible();
        basisfarbe = datenreihe.getBaseColor();
//        beschriftung = datenreihe.getCaption();        
        beschriftung = BezeichnungGeschwindigkeitBeschraenkung.getBezeichnungGeschwindigkeitBeschraenkungEnum();
        
        formelzeichen = datenreihe.getQuantitySymbol();
        einheitenzeichen = datenreihe.getUnitSymbol();
        yAchse = datenreihe.getAxis();
      }
      if(datenreihe.getValues()[0].length > 0)
      {
        hatDaten = true;
        break;
      }
    }
    if(!hatDaten)
    {
      datenreiheDarstellungInformation.setZustand(StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_LEER);
    }
    else if(sichtbar)
    {
      datenreiheDarstellungInformation.setZustand(StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_SICHTBAR);
    }
    else if(!sichtbar)
    {
      datenreiheDarstellungInformation.setZustand(StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_NICHT_SICHTBAR);
    }
    datenreiheDarstellungInformation.setFarbe(basisfarbe);

    beschriftungBezeichner.setText(beschriftung + "  ");
    bezeichnerAusgangsgroesse = (Dimension)beschriftungBezeichner.getPreferredSize().clone();
    beschriftungFormelzeichen.setText(formelzeichen + ":");
    formelzeichenAusgangsgroesse = (Dimension)beschriftungFormelzeichen.getPreferredSize().clone();
    beschriftungWert.setText("-");
    beschriftungEinheitenzeichen.setText(einheitenzeichen);
    einheitenzeichenAusgangsgroesse = (Dimension)beschriftungEinheitenzeichen.getPreferredSize().clone();
    
    if(yAchse != null)
    {
      datenreiheDarstellungInformation.setAchsenZuordnungText(yAchse.getAxisExt());
    }
    
    legendeAnzeige.repaint();    
  }

  public void detached()
  {
  }

  public void actionPerformed(TActionEvent actionEvent)
  {
    boolean hatDaten = false;
    boolean sichtbar = false;
    Color basisfarbe = null;
    for(int i = 0; i < datenreiheListe.size(); i++)
    {
      TSeries datenreihe = (TSeries)datenreiheListe.elementAt(i);
      if(i == 0)
      {
        sichtbar = datenreihe.isVisible();
        basisfarbe = datenreihe.getBaseColor();
      }
      
      if(datenreihe != null && datenreihe.getValues() != null && datenreihe.getValues().length > 0 && datenreihe.getValues()[0].length > 0)
      {
        hatDaten = true;
        break;
      }
    }
    if(actionEvent.isAction(TSeries.SERIES_VISIBILITY))
    {
      if(!hatDaten)
      {
        datenreiheDarstellungInformation.setZustand(StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_LEER);
      }
      else if(sichtbar)
      {
        datenreiheDarstellungInformation.setZustand(StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_SICHTBAR);
      }
      else if(!sichtbar)
      {
        datenreiheDarstellungInformation.setZustand(StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_NICHT_SICHTBAR);
      }
      legendeAnzeige.repaint();
    }
    else if(actionEvent.isAction(TSeries.SERIES_BASE_COLOR))
    {
      datenreiheDarstellungInformation.setFarbe(basisfarbe);
      legendeAnzeige.repaint();
    }
    else if(actionEvent.isAction(TSeries.SERIES_EMPTY))
    {
      boolean empty = ((Boolean)actionEvent.getValue()).booleanValue();
      if(empty)
      {
        datenreiheDarstellungInformation.setZustand(StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_LEER);
      }
      else
      {
        if(sichtbar)
        {
          datenreiheDarstellungInformation.setZustand(StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_SICHTBAR);
        }
        else if(!sichtbar)
        {
          datenreiheDarstellungInformation.setZustand(StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_NICHT_SICHTBAR);
        }
      }
      legendeAnzeige.repaint();
    }
    else if(actionEvent.isAction(TSeries.SERIES_CUTTING_VALUES))
    {
      long minimumWert = Long.MAX_VALUE;
      long maximumWert = Long.MIN_VALUE;
      for(int i = 0; i < datenreiheListe.size(); i++)
      {
        TSeries datenreihe = (TSeries)datenreiheListe.elementAt(i);
        boolean hatLueckenwert = datenreihe.hasBreakValue();
        double lueckenwert = Double.NaN;
        if(hatLueckenwert)
        {
          lueckenwert = datenreihe.getBreakValue();
        }
        double informationWert = datenreihe.getCuttingValue();
        if(!Double.isNaN(informationWert))
        {
          if(!hatLueckenwert || lueckenwert != informationWert)
          {
            minimumWert = Math.min(minimumWert, (long)informationWert);
            maximumWert = Math.max(maximumWert, (long)informationWert);
          }
        }
      }
   
      if(minimumWert < Long.MAX_VALUE && maximumWert > Long.MIN_VALUE)
      {
        if(minimumWert == maximumWert)
        {
          beschriftungWert.setText("" + minimumWert);
        }
        else
        {
          beschriftungWert.setText(minimumWert + "-" + maximumWert);
        }
      }
      else
      {
        beschriftungWert.setText("-");
      }
    }
    else if(actionEvent.isAction(TSeries.SERIES_FOOTER))
    {
      datenreiheDarstellungInformation.setAchsenZuordnungText((String)actionEvent.getValue());
      legendeAnzeige.repaint();
    }
  }

  public JComponent getLegendeAnzeige()
  {
    return (legendeAnzeige);
  }
  
  private class DatenreiheDarstellungInformation extends JComponent
  { 
    private final Dimension KASTEN_BEVORZUGTE_GROESSE = new Dimension(16, 16);
    private final Dimension ACHSEN_ZUORDNUNG_GROESSE = new Dimension(16, 16);
    private final float BREITE_ZEICHENSTIFT = 1.0f;
    private final Stroke ZEICHENSTIFT = new BasicStroke(BREITE_ZEICHENSTIFT);
    private Color farbe = null;
    private int zustand = -1;
    private String achsenZuordnungText = "";
    private int erweiterteHoehe = 0;
    private Font kleineSchriftart = null;
    private Font kleineSchriftartFett = null;

    public DatenreiheDarstellungInformation(Font schriftart)
    {
      setOpaque(false);
      kleineSchriftart = schriftart.deriveFont(schriftart.getSize2D() * 0.8f);
      kleineSchriftartFett = schriftart.deriveFont(schriftart.getSize2D() * 0.8f);
      kleineSchriftartFett = kleineSchriftartFett.deriveFont(Font.BOLD);
      setFont(kleineSchriftartFett);
      int erweiterteBreite = getFontMetrics(kleineSchriftartFett).stringWidth("*88");
      erweiterteHoehe = getFontMetrics(kleineSchriftartFett).getAscent();
      KASTEN_BEVORZUGTE_GROESSE.width += (2 + erweiterteBreite);
    }

    public void setFarbe(Color farbe)
    {
      this.farbe = farbe;
    }

    public int getZustand()
    {
      return (zustand);
    }

    public void setZustand(int zustand)
    {
      this.zustand = zustand;
    }

    public void setAchsenZuordnungText(String achsenZuordnungText)
    {
      this.achsenZuordnungText = achsenZuordnungText;
    }

    public Dimension getPreferredSize()
    {
      return (KASTEN_BEVORZUGTE_GROESSE);
    }

    protected void paintComponent(Graphics g)
    {
      super.paintComponent(g);
      
      Graphics2D g2 = (Graphics2D)g;
      int width = getWidth();
      int height = getHeight();
      if(TChart.isBugfixGraphicsRenderLabel() && width > 0 && height > 0)
      {
        boolean highResGraphics = (g2 instanceof CHighResGraphics2D);
        int factor = 1;
        if(highResGraphics)
        {
          factor = 5;
        }
        // TODO raho label bugfix
        // System.out.println("Bugfix render #" + factor + ": Q");
        if(renderedImage == null || (width * factor) != renderedImage.getWidth() || (height * factor) != renderedImage.getHeight())
        {
          if(renderedImage != null)
          {
            renderedImage.flush();
            renderedImage = null;
          }
          renderedImage = new BufferedImage(width * factor, height * factor, BufferedImage.TYPE_INT_ARGB);
        }
        Graphics2D ig2 = renderedImage.createGraphics();

        Color background = Color.WHITE;
        Component parent = this;
        while(parent != null)
        {
          if(parent.isOpaque())
          {
            background = parent.getBackground();
            break;
          }
          parent = parent.getParent();
        }
        parent = null;

        if(factor > 1)
        {
          ig2.scale(factor * TPdfCapableLabel.PDF_CORRECTION_VALUE, factor * TPdfCapableLabel.PDF_CORRECTION_VALUE);
        }
        ig2.setColor(background);
        ig2.fillRect(0, 0, width * factor, height * factor);
        ig2.setColor(g2.getColor());
        ig2.setFont(g2.getFont());
        ig2.setRenderingHints(g2.getRenderingHints());
        ig2.setStroke(g2.getStroke());
        ig2.setPaint(g2.getPaint());

        renderComponent(ig2);
        ig2.dispose();

        if(factor > 1)
        {
          double scaleFactor = (1.0 / factor);
          g2.scale(scaleFactor, scaleFactor);
          g2.drawImage(renderedImage, 0, 0, null);
          g2.scale(1.0 / scaleFactor, 1.0 / scaleFactor);
          renderedImage.flush();
          renderedImage = null;
        }
        else
        {
          g2.drawImage(renderedImage, 0, 0, null);
        }
      }
      else
      {
        renderComponent(g2);
      }
    }
    
    private void renderComponent(Graphics2D g2)
    {
      Color originalFarbe = g2.getColor();
      Stroke originalZeichenstift = g2.getStroke();
      g2.setColor(farbe);
      g2.setStroke(ZEICHENSTIFT);
      int xAbstand = getWidth() - ACHSEN_ZUORDNUNG_GROESSE.width;
      int yAbstand = (getHeight() - ACHSEN_ZUORDNUNG_GROESSE.height) / 2;
      int xLinks = (int)(xAbstand + (BREITE_ZEICHENSTIFT / 2));
      int xRechts = (int)(xAbstand + ACHSEN_ZUORDNUNG_GROESSE.width - (BREITE_ZEICHENSTIFT / 2));
      int yOben = (int)(yAbstand + (BREITE_ZEICHENSTIFT / 2));
      int yUnten = (int)(yAbstand + ACHSEN_ZUORDNUNG_GROESSE.height - (BREITE_ZEICHENSTIFT / 2));
      int breite = (int)(ACHSEN_ZUORDNUNG_GROESSE.width - BREITE_ZEICHENSTIFT);
      int hoehe = (int)(ACHSEN_ZUORDNUNG_GROESSE.height - BREITE_ZEICHENSTIFT);
      
      sichtbarerRahmen.x = xLinks;
      sichtbarerRahmen.y = yOben;
      if(zustand == StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_LEER)
      {
        g2.drawRect(xLinks, yOben, breite, hoehe);
        g2.drawLine(xLinks, yOben, xRechts, yUnten);
        g2.drawLine(xLinks, yUnten, xRechts, yOben);
      }
      else if(zustand == StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_SICHTBAR)
      {
        g2.fillRect(xLinks, yOben, breite + 1, hoehe + 1);
      }
      else if(zustand == StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_NICHT_SICHTBAR)
      {
        g2.drawRect(xLinks, yOben, breite, hoehe);
      }
      
      int erweiterteBreite = getFontMetrics(getFont()).stringWidth(achsenZuordnungText);
      CUtilities.setToHighestQuality(g2, true, true);
      g2.setColor(Color.BLACK);
      g2.drawString(achsenZuordnungText, getWidth() - ACHSEN_ZUORDNUNG_GROESSE.width - erweiterteBreite, erweiterteHoehe);
      
      if(zustand != StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_LEER)
      {
        if(zustand == StreckenProfilGeschwindigkeitBeschraenkungLegende.DATENREIHE_NICHT_SICHTBAR)
        {
          g2.setColor(Color.BLACK);
        }
        else
        {
          double deltaEWeiss = CUtilities.getDeltaE(farbe, Color.WHITE);
          double deltaESchwarz = CUtilities.getDeltaE(farbe, Color.BLACK);
          if(deltaEWeiss > deltaESchwarz)
          {
            g2.setColor(Color.WHITE);
          }
          else
          {
            g2.setColor(Color.BLACK);
          }
        }
        
        String datenreiheTypText = "Q";
        g2.setFont(kleineSchriftart);
        int tBreite = g2.getFontMetrics(g2.getFont()).stringWidth(datenreiheTypText);
        int tHoehe = g2.getFontMetrics(g2.getFont()).getAscent();
        int tLinks = xLinks + (breite - tBreite) / 2;
        int tUnten = yOben + hoehe - (hoehe - tHoehe) / 2 - 1;
        g2.drawString(datenreiheTypText, tLinks, tUnten);
        g2.setFont(kleineSchriftartFett);
      }

      g2.setColor(originalFarbe);
      g2.setStroke(originalZeichenstift);
    }
  }
}
