/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.diagram;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Rectangle;
import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JPanel;
import com.swarco.sts.chart.control.TAxis;
import com.swarco.sts.chart.control.TChart;
import com.swarco.sts.chart.control.event.TActionEvent;
import com.swarco.sts.chart.control.event.TActionListener;
import com.swarco.sts.chart.control.label.defaults.TPdfCapableLabel;

public class StreckenProfilKmLegende implements TActionListener
{
  private KmAnzeige kmAnzeige = null;
  private TPdfCapableLabel kmText = null;
  private int erweiterteBreite = -1;
  
  public StreckenProfilKmLegende(Font font)
  {
    kmText = new TPdfCapableLabel();
    kmText.setFont(font);
    kmAnzeige = new KmAnzeige();
    kmAnzeige.setOpaque(false);
    kmAnzeige.setBorder(BorderFactory.createEmptyBorder(1, 0, 0, 0));
    kmAnzeige.setLayout(null);
    kmAnzeige.add(kmText);
  }

  public void actionPerformed(TActionEvent actionEvent)
  {
    if(actionEvent.isAction(TAxis.AXIS_CUTTING_VALUE))
    {
      double value = ((double[])actionEvent.getValue())[0];
      if(Double.isNaN(value))
      {
        kmText.setText("");
      }
      else
      {
        long km = (long)value;
        long kmWert = km / 1000;
        long mWert = km % 1000;
        String mWertText = "" + mWert;
        while(mWertText.length() < 3)
        {
          mWertText = "0" + mWertText;
        }
        kmText.setText("Kilometer: " + kmWert + "." + mWertText); 
      }
    }
  }
  
  public JComponent getKmAnzeige()
  {
    return (kmAnzeige);
  }
  
  private class KmAnzeige extends JPanel
  {
    public Dimension getPreferredSize()
    {
      Dimension ret = (Dimension)kmText.getPreferredSize().clone();
      if(erweiterteBreite < 0)
      {
        Font sFont = kmText.getFont().deriveFont(kmText.getFont().getSize2D() * 0.8f);
        sFont = sFont.deriveFont(Font.BOLD);
        erweiterteBreite = getFontMetrics(sFont).stringWidth("*88");
      }
      ret.width += erweiterteBreite;

      return (ret);
    }

    protected void paintChildren(Graphics g)
    {
      super.paintChildren(g);
      
      if(TChart.isBugfixGraphicsRenderLabel())
      {
        try
        {
          if(getParent().getComponentCount() >= 2)
          {
            JComponent predecessor = (JComponent)getParent().getComponent(getParent().getComponentCount() - 2);
            IStreckenProfilLegende legend = (IStreckenProfilLegende)predecessor.getClientProperty("StreckenProfilLegende");
            if(legend != null)
            {
              Rectangle legendRect = legend.getSichtbarerRahmen();
              if(kmText.getText().length() > 0)
              {
                g.drawLine(legendRect.x, 4, legendRect.x + 42, 4);
              }
            }
          }
        }
        catch(Throwable t)
        {
          t.printStackTrace();
        }
      }
    }

    protected void paintComponent(Graphics g)
    {
      super.paintComponent(g);

      try
      {
        if(getParent().getComponentCount() >= 2)
        {
          JComponent predecessor = (JComponent)getParent().getComponent(getParent().getComponentCount() - 2);
          IStreckenProfilLegende legend = (IStreckenProfilLegende)predecessor.getClientProperty("StreckenProfilLegende");
          if(legend != null)
          {
            Rectangle legendRect = legend.getSichtbarerRahmen();
            if(kmText.getText().length() > 0)
            {
              g.drawLine(legendRect.x, 4, legendRect.x + 42, 4);
            }
            kmText.setBounds(legendRect.x, 4, Math.max(legendRect.width, kmText.getPreferredSize().width), getHeight());
          }
        }
      }
      catch(Throwable t)
      {
        t.printStackTrace();
      }
    }
  }
}
