/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor;

import org.eclipse.birt.chart.model.attribute.LineStyle;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;

import com.bitctrl.lib.eclipse.resources.WidgetColorRegistry;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.LinienEigenschaften;

/**
 * Stellt eine Linie entsprechend der Konfiguration in der
 * {@link LinePropertiesTable} dar.
 * 
 * @author BitCtrl Systems GmbH, Enrico Schnepel
 * @version $Id: ShowLineCanvas.java,v 1.3 2014/04/17 15:51:08 Pittner Exp $
 */
public final class ShowLineCanvas extends Canvas {

	/**
	 * Erzeugt eines neues {@link ShowLineCanvas}.
	 * 
	 * @param parent
	 *            das parent {@link Composite}.
	 * @param line
	 *            die {@link LinienEigenschaften}, die es darzustellen gilt.
	 */
	public ShowLineCanvas(final Composite parent, final LinienEigenschaften line) {
		super(parent, SWT.BORDER);
		addPaintListener(new PaintListener() {
			public void paintControl(final PaintEvent event) {
				final Rectangle bounds = getClientArea();
				paint(event.gc, bounds, line, WidgetColorRegistry
						.getInstance(parent));
			}
		});
	}

	/**
	 * Neu malen des {@link ShowLineCanvas}.
	 * 
	 * @param gc
	 *            der {@link GC}.
	 * @param bounds
	 *            die Grenzen des Widgets.
	 * @param line
	 *            die {@link LinienEigenschaften}, die es darzustellen gilt.
	 * @param widgetColorRegistry
	 *            {@link WidgetColorRegistry} zum registrieren der
	 *            {@link org.eclipse.swt.graphics.Color} Objekte.
	 */
	public static void paint(final GC gc, final Rectangle bounds,
			final LinienEigenschaften line,
			final WidgetColorRegistry widgetColorRegistry) {
		int ls = SWT.LINE_SOLID;
		if (line != null) {
			switch (line.getLineStyle().getValue()) {
			case LineStyle.DOTTED:
				ls = SWT.LINE_DOT;
				break;
			case LineStyle.DASHED:
				ls = SWT.LINE_DASH;
				break;
			case LineStyle.DASH_DOTTED:
				ls = SWT.LINE_DASHDOT;
				break;
			default:
			}
			gc.setAntialias(SWT.ON);
			gc.setLineStyle(ls);
			gc.setLineWidth((line.getLineThickness().getValue()
					* gc.getDevice().getDPI().x / 72) + 1);
			gc.setBackground(widgetColorRegistry
					.getColor(new RGB(192, 192, 192)));
			gc.setForeground(widgetColorRegistry.getColor(line.getRgb()));

		}
		gc.fillRectangle(bounds.x, bounds.y, bounds.width, bounds.height);
		gc.drawLine(bounds.x, bounds.y + bounds.height * 2 / 3, bounds.x
				+ bounds.width - 1, bounds.y + bounds.height / 3);

	}
}
