/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.pages;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.ui.progress.UIJob;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.simulation.SingleJobOfTypeSchedulingRule;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.Activateable;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.ActivateableContainer;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.ActivationState;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AeussererStreckenAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.AnzeigeQuerschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.InnererStreckenAbschnitt;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenZug;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilPackage;

/**
 * Contentprovider fr die Baumansicht eines Streckenzuges.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenZugTreeContentProvider.java,v 1.5 2014/04/17 15:51:13 Pittner Exp $
 */
public class StreckenZugTreeContentProvider extends EContentAdapter implements
		ITreeContentProvider {

	/**
	 * Der Viewer, der den {@link StreckenZugTreeContentProvider} verwendet.
	 */
	private Viewer lokalerViewer;

	/**
	 * Der aktuell von den Provider berwachte {@link StreckenZug}.
	 */
	private StreckenZug aktuellerStreckenZug;

	/**
	 * Flag, das anzeigt, ob der {@link Viewer} gerade aktualisisert wird.
	 */
	private boolean refreshView;

	/**
	 * Falg, das anzeigt, ob whrend der Aktualisierung des {@link Viewer}s ein
	 * Event zur erneuten Aktualisierung gekommen ist.
	 */
	private boolean refreshEventWhileRefreshingView;

	@Override
	public Object[] getChildren(final Object parentElement) {
		final List<Object> result = new ArrayList<Object>();
		if (parentElement instanceof ActivateableContainer<?>) {
			result.addAll(((ActivateableContainer<?>) parentElement)
					.getActivateables());
			
			if (parentElement instanceof InnererStreckenAbschnitt)
      {
        InnererStreckenAbschnitt iss = (InnererStreckenAbschnitt)parentElement;
        EList<AnzeigeQuerschnitt> aqs = iss.getAnzeigeQuerschnitt();
//        for (AnzeigeQuerschnitt aq : aqs)
//        {
//          aq.setState(ActivationState.ACTIVE);
//        }
        result.addAll(aqs);        
      }
			else if (parentElement instanceof AeussererStreckenAbschnitt)
			{
			  AeussererStreckenAbschnitt ass = (AeussererStreckenAbschnitt)parentElement;
			  EList<AnzeigeQuerschnitt> aqs = ass.getAnzeigeQuerschnitt();
//        for (AnzeigeQuerschnitt aq : aqs)
//        {
//          aq.setState(ActivationState.ACTIVE);
//        }
			  result.addAll(aqs);        			          
			}			
		}
		return result.toArray();
	}

	@Override
	public Object getParent(final Object element) {
		Object result = null;
		if (element instanceof Activateable) {
			result = ((Activateable) element).getActivateableContainer();
		}
		return result;
	}

	@Override
	public boolean hasChildren(final Object element) {
		boolean result = false;
		if (getChildren(element).length > 0) {
			result = true;
		}
		return result;
	}

	@Override
	public Object[] getElements(final Object inputElement) {
		return getChildren(inputElement);
	}

	@Override
	public void dispose() {
		if (aktuellerStreckenZug != null) {
			aktuellerStreckenZug.eAdapters().remove(this);
		}

	}

	@Override
	public void inputChanged(final Viewer viewer, final Object oldInput,
			final Object newInput) {
		lokalerViewer = viewer;
		if (newInput instanceof StreckenZug) {
			if (oldInput instanceof StreckenZug) {
				((StreckenZug) oldInput).eAdapters().remove(this);
			}
			aktuellerStreckenZug = ((StreckenZug) newInput);
			aktuellerStreckenZug.eAdapters().add(this);
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.emf.ecore.util.EContentAdapter#notifyChanged(org.eclipse
	 * .emf.common.notify.Notification)
	 */
	@Override
	public void notifyChanged(final Notification notification) {
		super.notifyChanged(notification);
		final Object feature = notification.getFeature();
		if (StreckenprofilPackage.Literals.ACTIVATEABLE_CONTAINER__ACTIVATEABLES
				.equals(feature)
				|| StreckenprofilPackage.Literals.STRECKEN_ZUG_EINTRAG__WEG_BISHER
						.equals(feature)) {
			switch (notification.getEventType()) {
			case Notification.ADD:
			case Notification.ADD_MANY:
			case Notification.REMOVE:
			case Notification.REMOVE_MANY:
			case Notification.SET:
				if (refreshView) {
					refreshEventWhileRefreshingView = true;
				} else {
					refreshView = true;
					final UIJob job = new UIJob(
							"Aktualisierung Streckenzug - Tabellenansicht...") {

						@Override
						public IStatus runInUIThread(
								final IProgressMonitor monitor) {

							do {
								refreshEventWhileRefreshingView = false;
								lokalerViewer.refresh();
							} while (refreshEventWhileRefreshingView);
							refreshView = false;
							return Status.OK_STATUS;
						}
					};
					job.setRule(new SingleJobOfTypeSchedulingRule());
					job.schedule();
				}
				break;
			default:
				// nix
			}
		}
	}
}
