/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.simulation;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.sys.funclib.bitctrl.modell.AnmeldeException;
import de.bsvrz.sys.funclib.bitctrl.modell.Aspekt;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensendeException;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.attribute.AttSimulationsZustand;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.objekte.OfflineSimulation;
import de.bsvrz.sys.funclib.bitctrl.modell.tmvewsimulationglobal.onlinedaten.OdSimulationsSteuerungOffline;

/**
 * {@link Job} zum fortsetzen einer {@link OfflineSimulation} aus dem Pause
 * Modus in den Start Modus.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: SimulationFortsetzenJob.java,v 1.3 2014/04/17 15:51:14 Pittner Exp $
 */
public class SimulationFortsetzenJob extends Job {

	/**
	 * Die {@link OfflineSimulation}, die fortgesetzt werden soll.
	 */
	private final OfflineSimulation simulation;

	/**
	 * Die Simulationsgeschwindigkeit. Default 1.
	 */
	private double simulationsGeschwindigkeit = 1;

	/**
	 * Erzeugt einen neuen {@link Job} zum fortsetzen einer
	 * {@link OfflineSimulation} aus dem Pause Modus in den Start Modus.
	 * 
	 * @param simulation
	 *            die {@link OfflineSimulation}, die in den Modus Start versetzt
	 *            werden soll.
	 */
	public SimulationFortsetzenJob(final OfflineSimulation simulation) {
		super("Simulation " + " '" + simulation.getName() + "' "
				+ " fortsetzen");
		Assert.isNotNull(simulation, "Ungltige Simulationsobjekt bergeben.");
		this.simulation = simulation;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.
	 * IProgressMonitor)
	 */
	@Override
	protected IStatus run(final IProgressMonitor monitor) {
		monitor.beginTask("Fortsetzen Streckenprofil-Offline-Simulation...",
				IProgressMonitor.UNKNOWN);
		IStatus status = Status.OK_STATUS;

		try {
			final Aspekt asp = OdSimulationsSteuerungOffline.Aspekte.Vorgabe;
			final OdSimulationsSteuerungOffline simSteuerung = simulation
					.getOdSimulationsSteuerungOffline();
			simSteuerung.anmeldenSender(asp);
			final OdSimulationsSteuerungOffline.Daten datum = simSteuerung
					.createDatum();

			if (AttSimulationsZustand.ZUSTAND_5_PAUSE.equals(simSteuerung
					.getDatum(OdSimulationsSteuerungOffline.Aspekte.Zustand)
					.getSimulationsZustand())) {
				// Von Pause nach Start
				datum
						.setSimulationsZustand(AttSimulationsZustand.ZUSTAND_2_START);
				datum.setSimulationsgeschwindigkeit(simulationsGeschwindigkeit);
				simSteuerung.sendeDatum(asp, datum);

			}
			simSteuerung.abmeldenSender(asp);
			status = Status.OK_STATUS;
		} catch (final AnmeldeException e1) {
			Activator
					.getDefault()
					.getLogger()
					.warning(
							"Anmeldung als Sender zum fortsetzen einer Simulation fehlgeschlagen.",
							e1);
		} catch (final DatensendeException e) {
			Activator
					.getDefault()
					.getLogger()
					.warning(
							"Versenden des Kommandos zum fortsetzen einer Simulation fehlgeschlagen.",
							e);
		}
		monitor.done();
		return status;
	}

	/**
	 * Liefert die Simulationsgeschwindigkeit, die der {@link Job} an die
	 * Simulationsverwaltung senden wird oder bereits gesendet hat.
	 * 
	 * @return the simulationsGeschwindigkeit
	 */
	public final double getSimulationsGeschwindigkeit() {
		return simulationsGeschwindigkeit;
	}

	/**
	 * Setzt die Simulationsgeschwindigkeit, die der Simulationsverwaltung
	 * mitgeteilt werden soll.
	 * 
	 * @param simulationsGeschwindigkeit
	 *            the simulationsGeschwindigkeit to set
	 */
	public final void setSimulationsGeschwindigkeit(
			final double simulationsGeschwindigkeit) {
		this.simulationsGeschwindigkeit = simulationsGeschwindigkeit;
	}

}
