/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.adapter;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;

/**
 * Adapter fr die Registrierung fr Ereignisse, die an Attributen auftreten,
 * die innerhalb des EMF Modells, als "derived" (abhngig) markiert sind.
 * 
 * @author BitCtrl Systems GmbH, Schnepel / Hoesel
 * @version $Id: DerivedAttributeAdapter.java,v 1.3 2014/04/17 15:51:19 Pittner Exp $
 */
public class DerivedAttributeAdapter extends AbstractDependantAttributeAdapter {

	/**
	 * Lister der {@link EStructuralFeature}s fr die dieser
	 * {@link DerivedAttributeAdapter} bereits registriert ist.
	 */
	private final List<EStructuralFeature> derivedFeatures = new ArrayList<EStructuralFeature>();

	/**
	 * Erzeugt einen neuen {@link DerivedAttributeAdapter}.
	 * 
	 * @param source
	 *            Das Wurzelobjekt einer Objekthierarchie.
	 * @param derivedFeature
	 *            das {@link EStructuralFeature} eines Attributes, das als
	 *            "derived" im EMF Modell markiert ist.
	 * @param navigationFeature
	 *            das {@link EStructuralFeature} an dem einem {@link EObject},
	 *            das wiederum andere {@link EObject}s als Kinder hat, und auf
	 *            dessen nderungen reagiert werden soll.
	 * @param dependantFeature
	 *            das {@link EStructuralFeature} eines Kindobjektes, auf das bei
	 *            nderungen reagiert werden soll.
	 * @param localFeature
	 *            das {@link EStructuralFeature} auf das an dem Wurzelobjekt der
	 *            Objekthierarchie reagiert wird.
	 */
	public DerivedAttributeAdapter(final EObject source,
			final EStructuralFeature derivedFeature,
			final EStructuralFeature navigationFeature,
			final EStructuralFeature dependantFeature,
			final EStructuralFeature localFeature) {
		this(source, derivedFeature);
		addNavigatedDependency(navigationFeature, dependantFeature);
		addLocalDependency(localFeature);
	}

	/**
	 * Erzeugt einen neuen {@link DerivedAttributeAdapter}.
	 * 
	 * @param source
	 *            Das Wurzelobjekt einer Objekthierarchie.
	 * @param derivedFeature
	 *            das {@link EStructuralFeature} eines Attributes, das als
	 *            "derived" im EMF Modell markiert ist.
	 * @param navigationFeature
	 *            das {@link EStructuralFeature} an dem einem {@link EObject},
	 *            das wiederum andere {@link EObject}s als Kinder hat, und auf
	 *            dessen nderungen reagiert werden soll.
	 * @param dependantFeature
	 *            das {@link EStructuralFeature} eines Kindobjektes, auf das bei
	 *            nderungen reagiert werden soll.
	 */
	public DerivedAttributeAdapter(final EObject source,
			final EStructuralFeature derivedFeature,
			final EStructuralFeature navigationFeature,
			final EStructuralFeature dependantFeature) {
		this(source, derivedFeature);
		addNavigatedDependency(navigationFeature, dependantFeature);
	}

	/**
	 * Erzeugt einen neuen {@link DerivedAttributeAdapter}.
	 * 
	 * @param source
	 *            Das Wurzelobjekt einer Objekthierarchie.
	 * @param derivedFeature
	 *            das {@link EStructuralFeature} eines Attributes, das als
	 *            "derived" im EMF Modell markiert ist.
	 * @param localFeature
	 *            das {@link EStructuralFeature} auf das an dem Wurzelobjekt der
	 *            Objekthierarchie reagiert wird.
	 */
	public DerivedAttributeAdapter(final EObject source,
			final EStructuralFeature derivedFeature,
			final EStructuralFeature localFeature) {
		this(source, derivedFeature);
		addLocalDependency(localFeature);
	}

	/**
	 * Erzeugt einen neuen {@link DerivedAttributeAdapter}.
	 * 
	 * @param source
	 *            Das Wurzelobjekt einer Objekthierarchie.
	 * @param derivedFeature
	 *            das {@link EStructuralFeature} eines Attributes, das als
	 *            "derived" im EMF Modell markiert ist.
	 */
	public DerivedAttributeAdapter(final EObject source,
			final EStructuralFeature derivedFeature) {
		super(source);
		addDerivedFeature(derivedFeature);
	}

	/**
	 * Registriert den Adapter fr ein {@link EStructuralFeature}.
	 * 
	 * @param derivedFeature
	 *            das {@link EStructuralFeature}.
	 */
	public void addDerivedFeature(final EStructuralFeature derivedFeature) {
		if (0 <= getSource().eClass().getFeatureID(derivedFeature)) {
			derivedFeatures.add(derivedFeature);
		}
	}

	@Override
	protected void notifyNavigationTargetChange(final Notification notification) {
		notifyChange();
	}

	@Override
	protected void notifyDependantChange(final Notification notification) {
		notifyChange();
	}

	/**
	 * Feuert eine neue {@link Notification}.
	 */
	private void notifyChange() {
		final InternalEObject source = getSource();
		if (source.eNotificationRequired()) {
			for (final EStructuralFeature derivedFeature : derivedFeatures) {
				source.eNotify(new ENotificationImpl(source, Notification.SET,
						derivedFeature, null, source.eGet(derivedFeature, true,
								true)));
			}
		}
	}
}
