/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
/* Segment 13 Bedienung und Visualisierung, Plug-In Streckenprofil 
 * Copyright (C) 2009-2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 * 
 * Contact Information: 
 * BitCtrl Systems GmbH 
 * Weienfelser Strae 67 
 * 04229 Leipzig 
 * Phone: +49 341-490670 
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.streckenprofil.sts.a8.views;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Timer;
import java.util.TimerTask;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.events.MenuListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.FilteredTree;
import org.eclipse.ui.forms.widgets.Form;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.StreckenprofilEinstellungen;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditor;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.editor.StreckenprofilEditorInput;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.bsvrz.buv.rw.bitctrl.eclipse.Messages;
import de.bsvrz.buv.rw.bitctrl.eclipse.PatternOrSubstringFilter;
import de.bsvrz.buv.rw.bitctrl.eclipse.modell.ModellEinstellungen;
import de.bsvrz.buv.rw.bitctrl.eclipse.viewers.EinstellungenContentProvider;
import de.bsvrz.buv.rw.bitctrl.startup.StartupPlugin;
import de.bsvrz.sys.funclib.bitctrl.daf.DavProvider;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.AbstractCache;
import de.bsvrz.sys.funclib.bitctrl.modell.util.cache.NetzCache;

/**
 * Die Ansicht Streckenprofil Navigator.
 * 
 * @author BitCtrl Systems GmbH, christian
 * @version $Id: StreckenprofilNavigator.java,v 1.3 2014/04/17 15:51:20 Pittner Exp $
 */
public class StreckenprofilNavigator extends ViewPart implements
		PropertyChangeListener {

	/**
	 * Die Zeitspanne in ms, in der die Benutzereinstellungen des
	 * Streckenprofils aktualisiert werden.
	 */
	private static final int DEFAULT_STRECKENPROFILNAVIGATOR_REFRESH = 10000;

	/**
	 * Zyklische Task, die die Nutzereinstellungen fr das Streckenprofil
	 * zyklisch aktualisiert.
	 */
	private final TimerTask zyklischesBenutzerparameterLesen = new TimerTask() {

		@Override
		public void run() {
			final Job j = new Job("Streckenprofil-Navigator aktualisieren") {

				@Override
				protected IStatus run(final IProgressMonitor monitor) {
					monitor.beginTask("Streckenprofil-Navigator aktualisieren",
							IProgressMonitor.UNKNOWN);
					Display.getDefault().syncExec(new Runnable() {

						@Override
						public void run() {
							if (updateAktiv) {
								refreshView();
							}
						}
					});

					monitor.done();
					return Status.OK_STATUS;
				}
			};
			j.schedule();
		}
	};

	/**
	 * Der TreeViewer.
	 */
	private FilteredTree filteredTree;

	/**
	 * Das {@link Form}, in dem die Baumansicht dargestellt wird.
	 */
	private Form form;

	/**
	 * Der {@link NetzCache}, der initialiseriert sein mu, damit die
	 * gespeicherten Streckenprofile angezeigt werden knnen.
	 */
	private NetzCache netzCache;

	/**
	 * {@link Timer}, der zyklisch eine Aktualisierung der Benutzereinstellungen
	 * des {@link StreckenprofilNavigator}s bewirkt.
	 */
	private Timer streckenprofilNavigatorAktualisierenTimer;

	private boolean updateAktiv = true;

	@Override
	public void createPartControl(final Composite parent) {
		final FormToolkit toolkit = new FormToolkit(parent.getDisplay());
		form = toolkit.createForm(parent);
		toolkit.decorateFormHeading(form);
		final Label headLabel = toolkit.createLabel(form.getHead(),
				Messages.SystemObjectAuswahl_Description, SWT.WRAP);
		headLabel.setBackground(null);
		form.setHeadClient(headLabel);
		final Composite formBody = form.getBody();
		formBody.setLayout(new GridLayout());

		filteredTree = new FilteredTree(formBody, SWT.BORDER,
				new PatternOrSubstringFilter(), true);
		toolkit.adapt(filteredTree);
		filteredTree.getViewer().setContentProvider(
				new EinstellungenContentProvider());
		filteredTree.setInitialText(Messages.SystemObjectAuswahl_DefaultFilter);
		filteredTree.getViewer().setSorter(new ViewerSorter());

		filteredTree.getViewer().addDoubleClickListener(
				new IDoubleClickListener() {
					public void doubleClick(final DoubleClickEvent event) {
						handleDoubleClick(event);
					}
				});
		filteredTree.getViewer().setLabelProvider(
				new StreckenprofilLabelProvider());

		getSite().setSelectionProvider(filteredTree.getViewer());

		setupPopupMenu(parent);
		if (!Activator.getDefault().getObjektFactory().isVerbunden()) {
			form.setMessage(
					"Datenverteilerverbindung noch nicht initialisiert !",
					IMessageProvider.ERROR);
			form.setEnabled(false);
		} else if (!netzCache.isInitialisiert()) {
			form.setMessage(
					"Bitte warten, das Streckennetz wird initialisiert ....",
					IMessageProvider.INFORMATION);
			form.setEnabled(false);
			form.setBusy(true);
		} else {

			filteredTree.getViewer().setInput(
					StreckenprofilEinstellungen.getInstance());
			StreckenprofilEinstellungen.getInstance()
					.addPropertyChangeListener(
							ModellEinstellungen.PROP_EINSTELLUNG_CHANGED, this);
		}

		// Online-Hilfe hinzufgen
		PlatformUI
				.getWorkbench()
				.getHelpSystem()
				.setHelp(
						formBody,
						Activator.PLUGIN_ID + "."
								+ StreckenprofilNavigator.class.getSimpleName());

		streckenprofilNavigatorAktualisierenTimer = new Timer(
				"Zyklisches Aktualisieren der Benutzereinstellungen des StreckenprofilNavigators");
		streckenprofilNavigatorAktualisierenTimer.scheduleAtFixedRate(
				zyklischesBenutzerparameterLesen,
				DEFAULT_STRECKENPROFILNAVIGATOR_REFRESH,
				DEFAULT_STRECKENPROFILNAVIGATOR_REFRESH);

	}

	/**
	 * Wird aufgerufen, sobald ein Doppelklick auf ein Element ausgefhrt wird.
	 * 
	 * @param event
	 *            das {@link DoubleClickEvent} - Objekt.
	 */
	protected void handleDoubleClick(final DoubleClickEvent event) {
		final ISelection s = event.getSelection();
		if (s instanceof TreeSelection) {
			final Object selectedObj = ((TreeSelection) s).getFirstElement();
			final TreePath[] treePaths = ((TreeSelection) s)
					.getPathsFor(selectedObj);

			if (selectedObj instanceof StreckenprofilEditorModel
					&& treePaths != null && treePaths.length > 0) {
				try {
					final Object parentObj = treePaths[0].getFirstSegment();

					final StreckenprofilEditorInput input = new StreckenprofilEditorInput();
					input.setModel((StreckenprofilEditorModel) selectedObj);
					if (parentObj instanceof EinstellungsArt) {
						input.setEinstellungsArt((EinstellungsArt) parentObj);
					}
					getSite().getPage().openEditor(input,
							StreckenprofilEditor.EDITOR_ID);
				} catch (final PartInitException e) {
					e.printStackTrace();
				}
			}
		}

	}

	@Override
	public void setFocus() {
		filteredTree.setFocus();
	}

	/**
	 * Erzeugt ein Kontextmenu in der Ansicht.
	 * 
	 * @param parent
	 *            das Parent Composite.
	 * @return das Kontextmenu.
	 */
	protected MenuManager setupPopupMenu(final Composite parent) {
		final MenuManager menuManager = new MenuManager("#PopupMenu");
		final Menu menu = menuManager.createContextMenu(parent);

		// XXX: Quickfix fr das Problem, das der Viewer nach der zyklischen
		// Aktualisierung seine Selektion verliert, weil der ContentProvider
		// immer neue Objekte liefert, anstatt die alten zu cachen.
		menu.addMenuListener(new MenuListener() {

			@Override
			public void menuShown(final MenuEvent e) {
				updateAktiv = false;
			}

			@Override
			public void menuHidden(final MenuEvent e) {
				updateAktiv = true;
			}
		});
		filteredTree.getViewer().getControl().setMenu(menu);
		getViewSite()
				.registerContextMenu(menuManager, filteredTree.getViewer());
		return menuManager;
	}

	@Override
	public void dispose() {
		if (streckenprofilNavigatorAktualisierenTimer != null) {
			streckenprofilNavigatorAktualisierenTimer.cancel();
		}
		StreckenprofilEinstellungen.getInstance().removePropertyChangeListener(
				this);
		super.dispose();
	}

	/**
	 * Ldt die Benutzereinstellungen neu.
	 */
	public void refreshView() {
		if (filteredTree != null && !filteredTree.isDisposed()) {
			filteredTree.getViewer().refresh(
					StreckenprofilEinstellungen.getInstance(), true);
		}
	}

	@Override
	public void propertyChange(final PropertyChangeEvent arg0) {
		if (AbstractCache.PROP_STATE.equals(arg0.getPropertyName())) {
			if (netzCache.isInitialisiert() && form != null
					&& !form.isDisposed()) {
				// seid ihr (Caches) alle da ? - JA
				form.getDisplay().asyncExec(new Runnable() {

					@Override
					public void run() {
						form.setEnabled(true);
						form.setBusy(false);
						form.setMessage(null, IMessageProvider.NONE);
						if (filteredTree != null) {
							StreckenprofilEinstellungen
									.getInstance()
									.addPropertyChangeListener(
											ModellEinstellungen.PROP_EINSTELLUNG_CHANGED,
											StreckenprofilNavigator.this);
							filteredTree.getViewer().setInput(
									StreckenprofilEinstellungen.getInstance());
						}
					}
				});

			}
		} else if (ModellEinstellungen.PROP_EINSTELLUNG_CHANGED.equals(arg0
				.getPropertyName())) {
			refreshView();
		} else if (DavProvider.PROP_VERBUNDEN.equals(arg0.getPropertyName())) {
			if (Boolean.TRUE.equals(arg0.getNewValue())) {
				form.setMessage(null);
			} else {
				form.setMessage(
						"Datenverteilerverbindung noch nicht initialisiert !",
						IMessageProvider.ERROR);
			}
			refreshView();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.ViewPart#init(org.eclipse.ui.IViewSite)
	 */
	@Override
	public void init(final IViewSite site) throws PartInitException {
		super.init(site);
		final StartupPlugin startupPlugin = StartupPlugin.getDefault();
		netzCache = startupPlugin.getDefaultNetzCache();
		netzCache.addPropertyChangeListener(AbstractCache.PROP_STATE, this);
	}

}
