/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.bsvrz.buv.plugin.streckenprofil.sts.a8.wizard;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWizard;

import de.bsvrz.buv.plugin.streckenprofil.sts.a8.Activator;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.model.StreckenprofilEditorModel;
import de.bsvrz.buv.plugin.streckenprofil.sts.a8.wizardpages.StreckenprofilDateiExportwizardPage;

/**
 * Wizard fr den Export von {@link StreckenprofilEditorModel}.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id: StreckenprofilDateiExportWizard.java,v 1.3 2014/04/17 15:51:20 Pittner Exp $
 */
public class StreckenprofilDateiExportWizard extends Wizard implements
		IWorkbenchWizard {

	private StreckenprofilDateiExportwizardPage exportPage;
	private StreckenprofilEditorModel streckenprofil;

	public StreckenprofilDateiExportWizard() {
		setWindowTitle("Streckenprofil export");
	}

	public StreckenprofilDateiExportWizard(final StreckenprofilEditorModel model) {
		this();
		streckenprofil = model;
	}

	@Override
	public void addPages() {
		if (streckenprofil != null) {
			exportPage = new StreckenprofilDateiExportwizardPage(streckenprofil);
		} else {
			exportPage = new StreckenprofilDateiExportwizardPage();
		}
		addPage(exportPage);
	}

	@Override
	public boolean performFinish() {
		boolean result = true;
		final StreckenprofilEditorModel zuSpeichernStreckenprofil = exportPage
				.getStreckenprofil();
		final String fileName = exportPage.getDestination();

		if ((fileName == null) || fileName.trim().isEmpty()) {
			final String meldung = "Es wurde keine gltiger Dateiname als Ausgabeziel angegeben!";
			Activator
					.getDefault()
					.getLog()
					.log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, meldung));
			MessageDialog.openError(getShell(),
					"Streckenprofil kann nicht exportiert werden", meldung);
			result = false;
		} else if (!(zuSpeichernStreckenprofil instanceof StreckenprofilEditorModel)) {
			final String meldung = "Es wurden keine Streckenprofil zum Export ausgewhlt!";
			Activator
					.getDefault()
					.getLog()
					.log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, meldung));
			MessageDialog.openError(getShell(),
					"Streckenprofil kann nicht exportiert werden", meldung);
			result = false;
		} else {
			final Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
			final Map<String, Object> m = reg.getExtensionToFactoryMap();
			Object object = m.get(Activator.FILE_EXTENSION);
			if (null == object) {
				object = new XMIResourceFactoryImpl();
				m.put(Activator.FILE_EXTENSION, object);
			}

			try {
				final File feile = new File(fileName);
				final FileOutputStream os = new FileOutputStream(feile);
				final ResourceSet resSet = new ResourceSetImpl();
				final Resource resource = resSet.createResource(URI
						.createURI("bitctrl://export."
								+ Activator.FILE_EXTENSION));
				resource.getContents().add(streckenprofil);
				resource.save(os, null);
				os.close();
			} catch (final IOException ex) {
				Activator
						.getDefault()
						.getLog()
						.log(new Status(IStatus.ERROR, Activator.PLUGIN_ID, ex
								.getLocalizedMessage(), ex));
				MessageDialog.openError(getShell(),
						"Streckenprofil kann nicht exportiert werden",
						ex.getLocalizedMessage());
				result = false;
			}
		}
		return result;
	}

	@Override
	public void init(final IWorkbench workbench,
			final IStructuredSelection selection) {
		final Object obj = selection.getFirstElement();
		if (obj instanceof StreckenprofilEditorModel) {
			streckenprofil = (StreckenprofilEditorModel) obj;
		}

	}
}
