/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.erweiterterstaualgorithmus;

import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Darstellung des Fensters zur Parameternderung der Algorithmus spezifischen Parametern
 * @author ObertM
 */
public class CompositeParameterGesamt extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Algorithmus spezifische Parameter
   */
  private ResultData _resultDataAlg;

  /**
   * Hauptcomposite
   */
  private Composite _composite;

	/**
	 * Compoiste mit den Algorithmus spezifischen Parametern
	 */
	private CompositeParameterAlgorithmus[] _compositeAlg;

	/**
	 * Gruppe mit den Parametern
	 */
	private Group _groupParameter;

	/**
	 * mgliche Anzeigequerschnitte
	 */
	private List<String> _anzeigeQuerschnitte;

  /**
   * mgliche Ursacheneinheiten	
   */
	private List<String> _ursachenEinheiten;

	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
	 * @param moeglicheUrsachenEinheiten  mgliche Ursacheneinheiten
	 * @param anzeigeQuerschnitte mgliche Anzeigequerschnitte
	 */
  CompositeParameterGesamt(Composite parent, int style, ResultData resultDataAlg, List<String> anzeigeQuerschnitte, List<String> ursachenEinheiten)
  {
    super(parent, style);
    
    this._resultDataAlg = resultDataAlg;
    
    _anzeigeQuerschnitte = anzeigeQuerschnitte;
    
    _ursachenEinheiten = ursachenEinheiten;
    
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

  	_compositeAlg = new CompositeParameterAlgorithmus [1];

    updateData ();
  }

  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   */
  public synchronized void update (ResultData resultDataAlg)
  {
    this._resultDataAlg = resultDataAlg;

    updateData ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultDataAlg != null)
    {
    	updataDataAlgorithmus ();
    }
  }
  
  /**
   * Updatefunktion fr die Algorithmus spezifischen Parameter
   */
  private void updataDataAlgorithmus()
  {
    Composite c = _composite;
    Control[] children = c.getChildren();
    CompositeParameterAlgorithmus compPar = null;

    Data data = null;
    if (_resultDataAlg.hasData())
      data = _resultDataAlg.getData();
    
    if ( children.length == 0 )
    {
      compPar = new CompositeParameterAlgorithmus ( c, SWT.None, _resultDataAlg, data, _anzeigeQuerschnitte, _ursachenEinheiten );
      _compositeAlg [0] = compPar;
      
      compPar.layout( true );
      c.layout( true );
      
      compPar.getTvParameter().refresh();
    }
    else
    {
      compPar = (CompositeParameterAlgorithmus) children[0];
      compPar.update ( _resultDataAlg, data );
      
      compPar.layout( true );
      c.layout( true );

      compPar.getTvParameter().refresh();
    }
  }

  /**
   * Methode liefert die einzelnen Composite fr die Algorithmus spezifischen Parameter zurck
   * @return Composites
   */
  public CompositeParameterAlgorithmus[] getCompositeAlg()
  {
  	return _compositeAlg;
  }

  /**
   * Dispose
   */
  @Override
  public void dispose ()
  {
     Composite c = _composite;
     if (c.isDisposed())
    	 return;
     
     Control[] children = c.getChildren();

     if ( children.length != 0 )
     {
      CompositeParameterAlgorithmus compPar = (CompositeParameterAlgorithmus) children[0];
      if (!compPar.isDisposed())
        compPar.dispose ();
    }
  	
  	super.dispose();
  }


}
