/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.glaettewarnung;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Spinner;

import de.bsvrz.buv.plugin.ursacheneinheit.stau.CompositeParameterStauBelegProFahrstreifen;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;

/**
 * Darstellung des Fensters zur Parameternderung der Algorithmus spezifischen Parametern
 * @author ObertM
 */
public class CompositeParameterGesamt extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Algorithmus spezifische Parameter
   */
  private ResultData _resultDataAlg;

  /**
   * Hauptcomposite
   */
  private Composite _composite;

  /**
   * Composite mit dem Spinner fr die Anzahl der Sensoren
   */
  private Composite _compositeSpinnerSensoren = null;

  /**
   * Composite mit den Reitern fr die Sensor spezifischen Parameter
   */
  private CTabFolder _tabFolder = null;

  /**
   * Liste mit den unterschiedlichen Parametergruppen
   */
	private ArrayList<String> _list;

	/**
	 * Composite mit den Algorithmus spezifischen Parametern
	 */
	private CompositeParameterAlgorithmus[] _compositeAlg;

	/**
	 * Anzahl der zu versorgenden Sensoren
	 */
	private int _anzahlSensoren;

	/**
	 * Spinner Anzahl Sensoren
	 */
	private Spinner _spinnerAnzahlSensoren;

	private Group groupSensoren;

	private List<String> _listeMoeglicherUFD;

	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
	 * @param listeMoeglicherUFD Liste mit den mglichen Umfelddatenmessstellen
	 */
  CompositeParameterGesamt (Composite parent, int style, ResultData resultDataAlg, List<String> listeMoeglicherUFD)
  {
    super(parent, style);
    
    this._resultDataAlg = resultDataAlg;
    
    this._listeMoeglicherUFD = listeMoeglicherUFD;
    
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    _list = new ArrayList<String>();

    createSpinnerAnzahlSensoren();

    createCompositeTab();

    int anzahl = bestimmeAnzahlSensoren();
    erzeugeBenoetigteReiter(anzahl);
    
    updateData ();
  }

  /**
   * Methode bestimmt die Anzahl der Sensoren aus dem Ergebnis des DaV
   * @return Anzahl Sensoren
   */
  private int bestimmeAnzahlSensoren ()
  {
    int anzahl = 0;
    if (_resultDataAlg.hasData())
    {
    	Data data = _resultDataAlg.getData();
    	anzahl = data.getArray("Sensor").getLength();
    }
    
    return anzahl;
  }
  
  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   */
  public synchronized void update (ResultData resultDataAlg)
  {
    this._resultDataAlg = resultDataAlg;
    
    int anzahl = bestimmeAnzahlSensoren();
    erzeugeBenoetigteReiter(anzahl);

    updateData ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultDataAlg != null)
    	updataDataAlgorithmus ();
  }

  /**
   * Methode legt die bentigte Anzahl der Reiter an
   * @param anzahl Anzahl bentigter Reiter (= Anzahl Sensoren)
   */
  private void erzeugeBenoetigteReiter (int anzahl)
  {
  	if (anzahl == _anzahlSensoren)
  		return;
  	
  	/*
  	 * Beim ersten Anlegen der Tabelle wird Reiter 0 markiert, bei nderung der Anzahl der
  	 * Reiter ber den Spinner wird der letzten Reiter markiert.
  	 */
  	int indexAnwahl = 0;
  	if (_compositeAlg != null)
  		indexAnwahl = anzahl - 1;
  	
		if (_spinnerAnzahlSensoren != null)
			_spinnerAnzahlSensoren.setSelection(anzahl);
		
		/*
		 * Namen der Reiter anpassen
		 */
		_list.clear();
    for (int i=0; i<anzahl; ++i)
      _list.add ("Sensor " + (i+1));
    
    /*
     * Bisherige Composite temp. speichern
     */
		Vector<CompositeParameterAlgorithmus> tmp = new Vector<CompositeParameterAlgorithmus>();
		if (_compositeAlg != null)
			for (CompositeParameterAlgorithmus cpa : _compositeAlg)
				tmp.add(cpa);

		/*
		 * Nicht bentigte Reiter entfernen
		 */
	  CTabItem[] items = _tabFolder.getItems();
	  for (int i=items.length - 1; i >= anzahl; i--)
		  items [i].dispose();

		/*
		 * Feld mit den Composites erweitern bzw. reduzieren
		 */
  	_compositeAlg = new CompositeParameterAlgorithmus [_list.size()];
  	for (int i=0; i<tmp.size() && i<anzahl; ++i)
  		_compositeAlg [i] = tmp.get(i);
		
		/*
		 * Reiter hinzufgen
		 */
		if (anzahl > _anzahlSensoren)
		{
		  for (int i=_anzahlSensoren; i<anzahl; ++i)
		  {
		  	erzeugeTabItem(_list.get(i));
		  	updataDataAlgorithmus (i);
		  }
		}
    
		_anzahlSensoren = anzahl;

	  if (indexAnwahl >= 0)
  	  _tabFolder.setSelection( indexAnwahl );

    _tabFolder.redraw();
  }

  /**
   * Updatefunktion fr die Algorithmus spezifischen Parameter
   */
  private void updataDataAlgorithmus()
  {
    for (int index = 0; index < _tabFolder.getItems().length; ++index)
    	updataDataAlgorithmus(index);
  }

  /**
   * Updatefunktion fr die Algorithmus spezifischen Parameter
   */
  private void updataDataAlgorithmus (int indexReiter)
  {
    Data data = null;
    if (_resultDataAlg.hasData())
    {
      data = _resultDataAlg.getData();

			CTabItem[] items = _tabFolder.getItems();
	    CTabItem item = items [indexReiter];
	    
	    if (item != null )
	    {
	      Composite c = (Composite) item.getControl();
	      Control[] children = c.getChildren();
	      CompositeParameterAlgorithmus compPar = null;
	
	      Data aktuell;
	
	   	  Array a = data.getArray("Sensor");
				if (a.getLength() > indexReiter)
	        aktuell = a.getItem(indexReiter);
	      else
	        aktuell = null;
	      
	      if ( children.length == 0 )
	      {
	        compPar = new CompositeParameterAlgorithmus ( c, SWT.None, _resultDataAlg, aktuell, _listeMoeglicherUFD );
	        _compositeAlg [indexReiter] = compPar;
	        
	        compPar.layout( true );
	        c.layout( true );
	        
	        compPar.getTvParameter().refresh();
	      }
	      else
	      {
	        compPar = (CompositeParameterAlgorithmus) children[0];
	        compPar.update ( _resultDataAlg, aktuell );
	        
//	        compPar.layout( true );
//	        c.layout( true );
//	
//	        compPar.getTvParameter().refresh();
	      }
	    }
    }
  }

  /**
   * Methode erteugt die Darstellung der Auswahl der Sensoren
   * 
   */
  private void createSpinnerAnzahlSensoren()
  {
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = false;

    groupSensoren = new Group(_composite, SWT.NONE);
    groupSensoren.setText("Anzahl Sensoren");
    groupSensoren.setLayout(new GridLayout(1, false));
    groupSensoren.setLayoutData(gridData2);

    _compositeSpinnerSensoren = new Composite(groupSensoren, SWT.NONE);
    _compositeSpinnerSensoren.setLayout(new GridLayout(1, false));

  	_spinnerAnzahlSensoren = new Spinner(_compositeSpinnerSensoren, SWT.BORDER);
  	_spinnerAnzahlSensoren.setSelection(_anzahlSensoren);
  	
  	_spinnerAnzahlSensoren.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				if (e.getSource() instanceof Spinner)
				{
					Spinner sp = (Spinner) e.getSource();
					int selection = sp.getSelection();
					
					erzeugeBenoetigteReiter (selection);
				}
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
  }

  /**
   * Methode zum Anlegen des Composite mit den Reitern
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;
    gridDataTabFolder.heightHint = 350;

//    Group groupParameter = new Group(_composite, SWT.NONE);
//    groupParameter.setText("Parameter");
//    groupParameter.setLayout(new GridLayout(1, false));
//    groupParameter.setLayoutData(gridDataGroup);
  
//    _tabFolder = new CTabFolder(groupParameter, SWT.NONE);
    _tabFolder = new CTabFolder(groupSensoren, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridDataTabFolder);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder)e.getSource();        
        CTabItem itemSel = folder.getSelection();
        
        Composite cSel = (Composite)itemSel.getControl();
        
        if (((Composite)cSel).getChildren().length > 0)
        {
        	CompositeParameterAlgorithmus compParSel = (CompositeParameterAlgorithmus)cSel.getChildren()[0];
        
          TableViewer tvParameter = compParSel.getTvParameter();
          tvParameter.refresh();
        }
      }
    });
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<String> list )
  {
  	for (int i=0; i<list.size(); ++i)
  		erzeugeTabItem(list.get(i));

  	_tabFolder.setSelection( 0 );
    _tabFolder.redraw();
    _tabFolder.layout(true);
  }

  private void erzeugeTabItem (String name)
  { 	
    String so = name;
    
    CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

    String tabName = so;
    tabItem.setText( tabName );
    tabItem.setData( tabName, so );

    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;

    Composite c1 = new Composite( _tabFolder, SWT.NONE );
    c1.setLayout( new GridLayout() );
    c1.setLayoutData( gridData2 );

    tabItem.setControl( c1 );
  }
  /**
   * Methode liefert die einzelnen Composite fr die Algorithmus spezifischen Parameter zurck
   * @return Composites
   */
  public CompositeParameterAlgorithmus[] getCompositeAlg ()
  {
  	return _compositeAlg;
  }

  /**
   * Methode zum Aktualisieren der Folder
   */
  public void refreshTabelle ()
  {
    /*
     * Loop ber die Fahrstreifen	
     */
  
  	for (String s : _list)
  	{
      /*
       * Reiter ermitten
       */

  		CTabItem[] items = _tabFolder.getItems();
      CTabItem item = null;
      
      for (CTabItem ti : items)
      {
        String pid = (String) ti.getData( ti.getText() );
        if ( pid.equals(s) )
       	 item = ti;
      }

      if ( item != null )
      {
        Composite c = (Composite) item.getControl();
        Control[] children = c.getChildren();
        CompositeParameterAlgorithmus compPar = null;

        if ( children.length != 0 )
        {
          compPar = (CompositeParameterAlgorithmus) children[0];
          compPar.layout( true );
          c.layout( true );

          compPar.getTvParameter().refresh();
        }
      }
    }
  }
}
