/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.lkwueberholverbotsicht;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;

import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Darstellung des Fensters zur Parameternderung mit den Messquerschnitten und den Algorithmus spezifischen
 * Parametern
 * @author ObertM
 */
public class CompositeParameterGesamt extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Messquerschnitte
   */
  private ResultData _resultDataMQ;

  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Umfelddatenmesstellen
   */
  private ResultData _resultDataUFD;

  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Algorithmus spezifische Parameter
   */
  private ResultData _resultDataAlg;

  /**
   * Liste mit den mglichen Messquerschnitte
   */
  private List<String> _listeMoeglicherMQ;

  /**
   * Liste mit den mglichen Umfelddatenmessstellen
   */
  private List<String> _listeMoeglicherUFD;

  /**
   * Hauptcomposite
   */
  private Composite _composite;

  /**
   * Composite mit den Reitern
   */
  private CTabFolder _tabFolder = null;

  /**
   * Liste mit den unterschiedlichen Parametergruppen
   */
	private ArrayList<String> _list;

	/**
	 * Compoiste mit den Algorithmus spezifischen Parametern
	 */
	private CompositeParameterAlgorithmus[] _compositeAlg;

	/**
	 * Merker ob die Versorgung der Messquerschnitte angeboten wird oder nicht
	 */
	private boolean _auswahlMQ;

	/**
	 * Instanz zur Auswahl der Messquerschnitte
	 */
	private AuswahlObjekte _auswahlObjekteMQ;

	/**
	 * Instanz zur Auswahl der Umfelddatenmessstellen
	 */
	private AuswahlObjekte _auswahlObjekteUFD;

	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataUFD Ergebnis Umfelddatenmesstellen 
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
	 * @param listMoeglicheMQ Liste mit den mglichen Messquerschnitten
	 * @param listeMoeglicherUFD Liste mit den mglichen Umfelddatenmessstellen
	 * @param auswahlMQ Merker ob die Versorgung der Messquerschnitte angeboten wird oder nicht
	 */
  CompositeParameterGesamt(Composite parent, int style, ResultData resultDataMq, ResultData resultDataUFD, ResultData resultDataAlg, List<String> listMoeglicheMQ, List<String> listeMoeglicherUFD, boolean auswahlMQ)
  {
    super(parent, style);
    
    this._resultDataMQ  = resultDataMq;
    this._resultDataUFD = resultDataUFD;
    this._resultDataAlg = resultDataAlg;
    
    this._auswahlMQ          = auswahlMQ;
    
    this._listeMoeglicherMQ  = listMoeglicheMQ;
    
    this._listeMoeglicherUFD = listeMoeglicherUFD;
    
    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    if (_auswahlMQ)
    {
    	_auswahlObjekteMQ  = new AuswahlObjekte(_composite, "Messquerschnitte");
    	_auswahlObjekteUFD = new AuswahlObjekte(_composite, "Umfelddatenmessstellen");
    }
    
    createCompositeTab();
    
    /*
     * Reiter anlegen fr die unterschiedliche Anzahl Fahrstreifen
     */
    _list = new ArrayList<String>();
    _list.add ("1 Fahrstreifen");
    _list.add ("2 Fahrstreifen");
    _list.add ("3 Fahrstreifen");
    _list.add ("4 Fahrstreifen");
    
  	_compositeAlg = new CompositeParameterAlgorithmus [_list.size()];

    erzeugeTabItems (_list);

    updateData ();
  }

  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   * @param resultData 
   */
  public synchronized void update (ResultData resultDataMq, ResultData resultDataUFD, ResultData resultDataAlg)
  {
    this._resultDataMQ  = resultDataMq;
    this._resultDataUFD = resultDataUFD;
    this._resultDataAlg = resultDataAlg;

    updateData ();
  }
  
  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
   * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   */
  public synchronized void updateDataAlg (ResultData resultDataAlg)
  {
    this._resultDataAlg = resultDataAlg;

    updataDataAlgorithmus ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultDataAlg != null)
    {
    	updataDataAlgorithmus ();
    }

    if (_resultDataMQ != null)
    {
    	if (_auswahlMQ)
        updateDataMessQuerschnitt();
    }
    
    if (_resultDataUFD != null)
    {
    	if (_auswahlMQ)
        updateDataUFD();
    }
  }
  
  /**
   * Updatefunktion fr die Algorithmus spezifischen Parameter
   */
  private void updataDataAlgorithmus()
  {
    /*
     * Loop ber die Fahrstreifen	
     */
  
  	int index = 0;
  	for (String s : _list)
  	{
      /*
       * Reiter emrmitten
       */

  		CTabItem[] items = _tabFolder.getItems();
      CTabItem item = null;
      
       for (CTabItem ti : items)
       {
         String pid = (String) ti.getData( ti.getText() );
         if ( pid.equals(s) )
        	 item = ti;
       }

       if ( item != null )
       {
          Composite c = (Composite) item.getControl();
          Control[] children = c.getChildren();
          CompositeParameterAlgorithmus compPar = null;

          Data data = null;
          if (_resultDataAlg.hasData())
          {
            String kennung = s.replace(" ", "");
            data = _resultDataAlg.getData().getItem(kennung);
          }
          
          if ( children.length == 0 )
          {
            compPar = new CompositeParameterAlgorithmus ( c, SWT.None, _resultDataAlg, data );
            _compositeAlg [index++] = compPar;
            
            compPar.layout( true );
            c.layout( true );
            
            compPar.getTvParameter().refresh();
          }
          else
          {
            compPar = (CompositeParameterAlgorithmus) children[0];
            compPar.update ( _resultDataAlg, data );
            
            compPar.layout( true );
            c.layout( true );

            compPar.getTvParameter().refresh();
          }
        }
      }
  }

  /**
   * Updatefunktion ber die Messquerschnitte
   */
	private void updateDataMessQuerschnitt()
  {
		List<String> angewaehlteObjekte = new ArrayList<String>();
		
    if ( _resultDataMQ != null)
    {
      if ( _resultDataMQ.hasData() )
      {
      	Data data = _resultDataMQ.getData();
      	
        Array arrayMQ = data.getArray( "MessQuerschnitt" );
        for (int k = 0; k < arrayMQ.getLength(); k++)
        {
          String text = "";
          ReferenceValue rV = arrayMQ.getReferenceValue( k );
          if ( rV != null )
            text = Tools.getName(rV.getSystemObject());
          
          angewaehlteObjekte.add(text);
        }
      }
    }
    
    _auswahlObjekteMQ.setAngewaehlteObjekte(_listeMoeglicherMQ, angewaehlteObjekte);
  }

  /**
   * Updatefunktion ber die Umfelddaten
   */
	private void updateDataUFD()
  {
		List<String> angewaehlteObjekte = new ArrayList<String>();
		
    if ( _resultDataUFD != null)
    {
      if ( _resultDataUFD.hasData() )
      {
      	Data data = _resultDataUFD.getData();
      	
        Array arrayUFD = data.getArray( "UmfeldDatenSensor" );
        for (int k = 0; k < arrayUFD.getLength(); k++)
        {
          String text = "";
          ReferenceValue rV = arrayUFD.getReferenceValue( k );
          if ( rV != null )
            text = Tools.getName(rV.getSystemObject());
          
          angewaehlteObjekte.add(text);
        }
      }
    }
    
    _auswahlObjekteUFD.setAngewaehlteObjekte(_listeMoeglicherUFD, angewaehlteObjekte);
  }

  /**
   * Methode zum Anlegen des Composite fr die Tabelle
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;

    Group groupParameter = new Group(_composite, SWT.NONE);
    groupParameter.setText("Parameter");
    groupParameter.setLayout(new GridLayout(1, false));
    groupParameter.setLayoutData(gridDataGroup);
  
    _tabFolder = new CTabFolder(groupParameter, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridDataTabFolder);
    _tabFolder.addSelectionListener(new SelectionListener() {

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder)e.getSource();        
        CTabItem itemSel = folder.getSelection();
        
        Composite cSel = (Composite)itemSel.getControl();
        
        if (((Composite)cSel).getChildren().length > 0)
        {
          CompositeParameterAlgorithmus compParSel = (CompositeParameterAlgorithmus)cSel.getChildren()[0];
        
          TableViewer tvParameter = compParSel.getTvParameter();
          tvParameter.refresh();
        }
      }
    });
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<String> list )
  {
  	for (int i=0; i<list.size(); ++i)
  	{
      String so = list.get(i);
     
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

      String tabName = so;
      tabItem.setText( tabName );
      tabItem.setData( tabName, so );

      GridData gridData2 = new GridData();
      gridData2.horizontalAlignment = GridData.FILL;
      gridData2.grabExcessHorizontalSpace = true;
      gridData2.grabExcessVerticalSpace = true;
      gridData2.verticalAlignment = GridData.FILL;

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData2 );

      tabItem.setControl( c1 );

    }
    _tabFolder.setSelection( 0 );
    
  }

  /**
   * Methode liefert die Liste der angewhlten Messquerschnitte
   * @return Liste der angewhlten Messquerschnitte
   */
  public org.eclipse.swt.widgets.List getZugeordneteMQ()
  {
    return _auswahlObjekteMQ.getZugeordneteObjekte();
  }

  /**
   * Methode liefert die Liste der angewhlten Umfelddatenmessstellen
   * @return Liste der angewhlten Messquerschnitte
   */
  public org.eclipse.swt.widgets.List getZugeordneteUFD()
  {
    return _auswahlObjekteUFD.getZugeordneteObjekte();
  }

  /**
   * Methode liefert die einzelnen Composite fr die Algorithmus spezifischen Parameter zurck
   * @return Composites
   */
  public CompositeParameterAlgorithmus[] getCompositeAlg()
  {
  	return _compositeAlg;
  }
  
  /**
   * Dispose
   */
  @Override
  public void dispose ()
  {
		CTabItem[] items = _tabFolder.getItems();
    
     for (CTabItem item : items)
     {
    	 if (item.isDisposed())
    		 continue;
    	 
       Composite c = (Composite) item.getControl();
       if (c.isDisposed())
      	 continue;
       
       Control[] children = c.getChildren();

       if ( children.length != 0 )
       {
        CompositeParameterAlgorithmus compPar = (CompositeParameterAlgorithmus) children[0];
        if (!compPar.isDisposed())
          compPar.dispose ();
      }
    }
  	
  	super.dispose();
  }

}
