/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.sbaueparam;

import java.math.RoundingMode;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.buv.plugin.ursacheneinheit.harmobelastungunruhe120.DaVKommunikation;
import de.bsvrz.buv.plugin.ursacheneinheit.zeitdauer.MyCompositeZeitDauer;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.dav.daf.main.config.IntegerAttributeType;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Darstellung des Fensters fr die Bearbeitung der Parameter "Ursacheneinheit"
 * @author ObertM
 *
 */
public class SbaUrsachenEinheitCompositeParameter extends Composite
{ 
	/**
	 * Text fr den Label fr das Eingabeelement "StatusBercksichtigung"
	 */
	public final static String _itemStatusBeruecksichtigen = "Status bercksichtigen";

	/**
	 * Text fr den Label fr das Eingabeelement "AspektVerwendung"
	 */
	public final static String _itemAspektVerwendung = "Aspekt Verwendung";

	/**
	 * Text fr den Label fr das Eingabeelement "Semiautomatik"
	 */
	public final static String _itemSemiAutomatik = "Semiautomatik";

	/**
	 * Text fr den Label fr das Eingabeelement "Bearbeitung"
	 */
	public final static String _itemBearbeitung = "Bearbeitung";

	/**
	 * Text fr den Label fr das Eingabeelement "Mindeststandzeit tMinEin"
	 */
	public final static String _itemTMinEin = "Mindeststandzeit/Ein";

	/**
	 * Text fr den Label fr das Eingabeelement "Mindeststandzeit tMinAus"
	 */
	public final static String _itemTMinAus = "Mindeststandzeit/Aus";

	/**
	 * Text fr den Label fr das Eingabeelement "Mindeststandzeit tMinStr"
	 */
	public final static String _itemTMinStoer = "Mindeststandzeit/Str";

	/**
	 * Text fr den Aspekt "undefiniert"
	 */
	public final static String _undefiniert = " undefiniert";
	
	/**
	 * Mgliche Werte fr die ComboBox "Status bercksichtigen"
	 */
  private static final String [] _status = new String []
  {
  	"automatik",
  	"immerWahr",
  	"immerFalsch",
  };
	
  /**
   * Mgliche Werte fr die ComboBoxen mit "Ja/Nein" Auswahl
   */
  private static final String [] _jaNein = new String []
  {
    "Ja",
    "Nein",
  };

  /**
   * Mgliche Werte fr die ComboBoxen "ApektVerwendung"
   */
	private String[] _aspekte;

  private Composite _composite;

  private Composite _composite1;

  private Group _groupFilterDef;

  private TableViewer _tvAnzeige;

  private ResultData _resultData;

	private Composite comp8;

	private Composite comp9;

  
  
  SbaUrsachenEinheitCompositeParameter(Composite parent, int style, ResultData result, String pid)
  {
    super(parent, style);
    this._resultData = result;
    initialize();
  }

  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
  	/*
  	 * Bestimmen der mglichen Aspekte
  	 */
    ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    List<SystemObject> elements = connection.getDataModel().getType( "typ.aspekt" ).getElements();

  	_aspekte = new String [elements.size() + 1];
  	_aspekte[0] = _undefiniert;
  	for (int i=0; i<elements.size(); ++i)
  		_aspekte [i + 1] = elements.get(i).getName();
  		
  	
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    
    _composite = new Composite(this, SWT.NONE);
    _composite.setLayout(new GridLayout(1, false));
    _composite.setLayoutData(gridData1);
 
    //Initialisierungen der Arrays
    List<TableModel[]> inputList = new ArrayList<TableModel[]>();
  
    createGroupDefinition();
    
    Table table = _tvAnzeige.getTable();
    TableModel[] input = createModel1(table);
    
    if ( input != null )
    {
      _tvAnzeige.setInput(input); 
      inputList.add( input );
    }    
    updateData(inputList);
  }

  private void updateData( List<TableModel[]> modelList )
  {
    Data item = _resultData.getData();
    
    if (item != null)
    {
      /*
       *  alle TableModelle durchlaufen
       */
    	
      for (int i = 0; i < modelList.size(); i++)
      {
        for (TableModel tm : modelList.get( i ))
        {
          Label lb = (Label) tm.getAttribut().getChildren()[0];
          
          Control control = tm.getWert().getChildren()[0];

          /*
           * Comboboxen
           */
          if (control instanceof Combo)
          {
            Combo cb = (Combo)control;
            
            /*
             * Item "StatusBercksichtigung"
             */
            if (lb.getText().equals(_itemStatusBeruecksichtigen))
            {
            	cb.setItems(_status);
            	String tWert = item.getItem("StatusBercksichtigung").asTextValue().getValueText();
            	
            	for (int k=0; k<_status.length; ++k)
            		if (tWert.equals(_status [k]))
            			cb.select(k);
            }
            
            /*
             * Item "AspektVerwendung"
             */
            if (lb.getText().equals(_itemAspektVerwendung))
            {
              cb.setItems (_aspekte);
              SystemObject so = item.getReferenceValue("AspektVerwendung").getSystemObject();

              if (so == null)
              {
              	cb.select(0);
              }
              else
              {
            	  for (int k=0; k<_aspekte.length; ++k)
            		  if (so.getName().equals(_aspekte [k]))
            			  cb.select(k);
              }
            }

            /*
             * Item "Semiautomatik"
             */
            if (lb.getText().equals(_itemSemiAutomatik))
            {
            	cb.setItems(_jaNein);
            	String tWert = item.getItem("Semiautomatik").asTextValue().getValueText();
            	
            	for (int k=0; k<_jaNein.length; ++k)
            		if (tWert.equals(_jaNein [k]))
            			cb.select(k);
            }

            /*
             * Item "Bearbeitung"
             */
            if (lb.getText().equals(_itemBearbeitung))
            {
            	cb.setItems(_jaNein);
            	String tWert = item.getItem("Bearbeitung").asTextValue().getValueText();
            	
            	for (int k=0; k<_jaNein.length; ++k)
            		if (tWert.equals(_jaNein [k]))
            			cb.select(k);
            }
          }
          else if (control instanceof MyCompositeZeitDauer)
          {
            MyCompositeZeitDauer czd = (MyCompositeZeitDauer)control;
            
            long l1 = item.getItem( "Mindeststandzeit" ).getItem( "tMinEin" ).asTimeValue().getMillis();                       
            long l2 = item.getItem( "Mindeststandzeit" ).getItem( "tMinAus" ).asTimeValue().getMillis();                       
            long l3 = item.getItem( "Mindeststandzeit" ).getItem( "tMinStr" ).asTimeValue().getMillis(); 
            
            if (lb.getText().equals(_itemTMinEin))                
              czd.setZeitDauerInBezugAufAnfangsZeitpunkt(l1);
            
            else if (lb.getText().equals(_itemTMinAus))
              czd.setZeitDauerInBezugAufAnfangsZeitpunkt(l2);
            
            else if (lb.getText().equals(_itemTMinStoer))
              czd.setZeitDauerInBezugAufAnfangsZeitpunkt(l3);              
          }
          else if (control instanceof Spinner)
          {
//            Spinner sp = (Spinner)control;
////              setSpinnerMinMax( attG, "", "", sp );
//            setSpinnerTooltip( sp, lb );
//            
//            // Wert ermitteln und umwandeln
//            String tWert = item.getItem( "Prioritt" ).asTextValue().getValueText();                       
//            if (tWert.equals("undefiniert"))
//              tWert = "-1";
//            Double dWert = new Double( tWert );
//            long lWert = Math.round( dWert );
//            sp.setSelection( (int) lWert );
            
          }                   
        }
      }     
    }
  }

  /**
   * Minimum- und Maximum-Werte und Digits eines Spinners mit den Daten aus dem DAV setzen
   * 
   * @param attG
   *          AttributeGroup zum ResultData
   *          (result.getDataDescription().getAttributeGroup())
   * @param group
   *          String der Gruppe, zu dem das Item gehrt, oder null falls keine
   *          Gruppe existiert
   * @param item
   *          String des Items zu dessen Attribut die Werte ermittelt werden
   *          sollen
   * @param spinner
   *          Spinner bei dem die Werte gesetzt werden sollen
   * 
   */
  private void setSpinnerMinMax(AttributeGroup attG, String group, String item, Spinner spinner)
  {
    //Attributliste zu der Gruppe ermitteln
    IntegerAttributeType iat1 = null;
    if ( group != null )
    {
      AttributeListDefinition ald = ( AttributeListDefinition )attG.getAttribute(group).getAttributeType();
      iat1 = (IntegerAttributeType)ald.getAttribute(item).getAttributeType();
    }
    else
      iat1 = (IntegerAttributeType)attG.getAttribute(item).getAttributeType();
    
    IntegerValueRange range1 = iat1.getRange();
    
    double rangeFactor = range1.getConversionFactor();
    long rangeMin = range1.getMinimum();
    long rangeMax = range1.getMaximum(); 
    
    //Digits ermitteln (log10(0.0001) = -4; log10(0.01) = -2)
    Double digits = - Math.log10( rangeFactor );
    spinner.setDigits(digits.intValue());  
    spinner.setData( rangeFactor );
    
    //Min- Max-Integer-Werte fr den Spinner setzen
    int iMin = (int) rangeMin;
    int iMax = Integer.MAX_VALUE;
    if ( rangeMax < (long)iMax )
      iMax = (int) rangeMax;
    
    spinner.setMinimum( iMin );
    spinner.setMaximum( iMax );    
    
//    System.out.println(item + " ==> Min: " + range1.getMinimum() + "->"+ iMin 
//        + " Max: " + range1.getMaximum() +"->"+ iMax + " Digits: " + rangeFactor + "->" + spinner.getDigits());
       
  }

  /**
   * Setzen des Tooltips beim Spinner und dem zugehrigen Label
   * 
   * @param spinner Spinner
   * @param label   Label
   */
  private void setSpinnerTooltip(Spinner spinner, Label label)
  {
    int digits = spinner.getDigits();
    Double rangeFactor = (Double)spinner.getData();
    if ( rangeFactor == null )
    {
      rangeFactor = 1/Math.pow( 10, digits );
      spinner.setData(rangeFactor);
    }
    double minSpin = (double)spinner.getMinimum() * rangeFactor;
    double maxSpin = (double)spinner.getMaximum() * rangeFactor;
    
    NumberFormat f1 = NumberFormat.getInstance();
    f1.setRoundingMode(RoundingMode.HALF_UP);
    f1.setMaximumFractionDigits(digits);
    f1.setMinimumFractionDigits(digits);
    f1.setGroupingUsed(true);
    
    String tt = "Wertebereich: " + f1.format(minSpin) + " bis " + f1.format(maxSpin) ;
    spinner.setToolTipText(tt);
    if ( label != null )
      label.setToolTipText(tt);

//    System.out.println("Tooltip: Max: " + spinner.getMaximum() + " Digits: " + spinner.getDigits() + " Faktor: " + spinner.getData());
       
  }
  
  /**
   * This method initializes createGroupDefinition
   * 
   */
  private void createGroupDefinition()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.marginHeight = 5;
    fillLayout1.marginWidth = 5;
    fillLayout1.type = org.eclipse.swt.SWT.HORIZONTAL;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    //Gre der 3.Gruppe fest setzen, damit diese grer wird
    _groupFilterDef = new Group(_composite, SWT.NONE);
    _groupFilterDef.setText("SbaUrsachenEinheit");
    _groupFilterDef.setLayout(fillLayout1);
    _groupFilterDef.setLayoutData(gridData2);

    erzeugeTabelle();

  }

  private void erzeugeTabelle()
  {
    FillLayout fillLayout1 = new FillLayout();
    fillLayout1.type = org.eclipse.swt.SWT.VERTICAL;
    _composite1 = new Composite(_groupFilterDef, SWT.None);
    _composite1.setLayout(fillLayout1);
    _tvAnzeige = new TableViewer(_composite1, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvAnzeige.setContentProvider(new TableContentProvider());
   
    TableViewerColumn column1 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column1.getColumn().setWidth(225);
    column1.getColumn().setText("Attribut");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvAnzeige, SWT.NONE);
    column2.getColumn().setWidth(150);
    column2.getColumn().setText("Wert");
    column2.setLabelProvider(new MyCellLabelProvider());

    // resize the row height using a MeasureItem listener
    Listener listener = new Listener() {
      @Override
      public void handleEvent(Event event)
      {
        switch (event.type)
        {
          case SWT.MeasureItem:
            event.height = 30;
            break;

          default:
        }

      }

    };
    _tvAnzeige.getTable().addListener(SWT.MeasureItem, listener);
    _tvAnzeige.getTable().setLinesVisible(true);
    _tvAnzeige.getTable().setHeaderVisible(true);
  }


  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
    private TableViewer tableViewer;

    public Object[] getElements(Object inputElement)
    {
      return (TableModel[])inputElement;      
    }

    public void dispose()
    {

    }

    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.deleteObserver(this);
          }

        }

      }
      
     
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.addObserver(this);
          }
        }

      }

    }

    @Override
    public void update(Observable arg0, Object arg1)
    {

      if (tableViewer != null)
      {
        tableViewer.refresh();

      }
    }
  }

  /**
   * Erzeugt ein TabellenModell fr den Inhalt der Tabellen
   * 
   * @param table
   * @return TabellenModell
   */
  private TableModel[] createModel1(Table table)
  {    
    TableModel[] elements = new TableModel[7];

    //Layout festlegen
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.grabExcessVerticalSpace = true; 
    
    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.verticalAlignment = GridData.END;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;
    
    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.FILL;
    gd2.grabExcessHorizontalSpace = true;
    
    //zum Ausfllen der Zelle
    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;
    
    /*
     * Comboboxen erstellen ...
     * ... Status bercksichtigen
     */
    Composite comp0 = new Composite( table, SWT.NONE );
    comp0.setBackground( table.getBackground() );
    comp0.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp0.setLayoutData( gd0 );
    comp0.setLayout( new GridLayout() );
    Label label0 = new Label( comp0, SWT.NONE );
    label0.setText( _itemStatusBeruecksichtigen );
    
    Composite comp1 = new Composite( table, SWT.NONE );
    comp1.setBackground( table.getBackground() );
    comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp1.setLayoutData( gd1 );
    comp1.setLayout( new GridLayout() );
    Combo combo1 = new Combo( comp1, SWT.BORDER );
    combo1.setLayoutData( gdTableItem );
    
    elements[0] = new TableModel( comp0, comp1 );

    /*
     * ... Aspekt Verwendung
     */
    Composite comp2 = new Composite( table, SWT.NONE );
    comp2.setBackground( table.getBackground() );
    comp2.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp2.setLayoutData( gd0 );
    comp2.setLayout( new GridLayout() );
    Label label2 = new Label( comp2, SWT.NONE );
    label2.setText( _itemAspektVerwendung );
    
    Composite comp3 = new Composite( table, SWT.NONE );
    comp3.setBackground( table.getBackground() );
    comp3.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp3.setLayoutData( gd1 );
    comp3.setLayout( new GridLayout() );
    Combo combo3 = new Combo( comp3, SWT.BORDER );
    combo3.setLayoutData( gdTableItem );
    
    elements[1] = new TableModel( comp2, comp3 );

    /*
     * ... Semiautomatik
     */
    Composite comp4 = new Composite( table, SWT.NONE );
    comp4.setBackground( table.getBackground() );
    comp4.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp4.setLayoutData( gd0 );
    comp4.setLayout( new GridLayout() );
    Label label4 = new Label( comp4, SWT.NONE );
    label4.setText( _itemSemiAutomatik );
    
    Composite comp5 = new Composite( table, SWT.NONE );
    comp5.setBackground( table.getBackground() );
    comp5.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp5.setLayoutData( gd1 );
    comp5.setLayout( new GridLayout() );
    Combo combo5 = new Combo( comp5, SWT.BORDER );
    combo5.setLayoutData( gdTableItem );
    
    elements[2] = new TableModel( comp4, comp5 );

    /*
     * ... Bearbeitung
     */
    Composite comp6 = new Composite( table, SWT.NONE );
    comp6.setBackground( table.getBackground() );
    comp6.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp6.setLayoutData( gd0 );
    comp6.setLayout( new GridLayout() );
    Label label6 = new Label( comp6, SWT.NONE );
    label6.setText( _itemBearbeitung );
    
    Composite comp7 = new Composite( table, SWT.NONE );
    comp7.setBackground( table.getBackground() );
    comp7.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp7.setLayoutData( gd1 );
    comp7.setLayout( new GridLayout() );
    Combo combo7 = new Combo( comp7, SWT.BORDER );
    combo7.setLayoutData( gdTableItem );
    
    elements[3] = new TableModel( comp6, comp7 );

    /*
     * Zeiteingaben erstellen
     */
    comp8 = new Composite( table, SWT.NONE );
    comp8.setBackground( table.getBackground() );
    comp8.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp8.setLayoutData( gd0 );
    comp8.setLayout( new GridLayout() );
    Label label8 = new Label( comp8, SWT.NONE );
    label8.setText( _itemTMinEin );
    
    comp9 = new Composite(table, SWT.NONE);
    comp9.setBackground(table.getBackground());
    comp9.setBackgroundMode(SWT.INHERIT_DEFAULT);
    comp9.setLayoutData( gd2 );
    comp9.setLayout( new FillLayout() );
    new MyCompositeZeitDauer(comp9, SWT.NONE, new Date().getTime(), true);
    
    elements[4] = new TableModel( comp8, comp9 );
        
    Composite comp10 = new Composite( table, SWT.NONE );
    comp10.setBackground( table.getBackground() );
    comp10.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp10.setLayoutData( gd0 );
    comp10.setLayout( new GridLayout() );
    Label label10 = new Label( comp10, SWT.NONE );
    label10.setText( _itemTMinAus );
    
    Composite comp11 = new Composite(table, SWT.NONE);
    comp11.setBackground(table.getBackground());
    comp11.setBackgroundMode(SWT.INHERIT_DEFAULT);
    comp11.setLayoutData( gd2 );
    comp11.setLayout( new FillLayout() );
    new MyCompositeZeitDauer(comp11, SWT.NONE, new Date().getTime(), true);
    
    elements[5] = new TableModel( comp10, comp11 );   
    
    Composite comp12 = new Composite( table, SWT.NONE );
    comp12.setBackground( table.getBackground() );
    comp12.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp12.setLayoutData( gd0 );
    comp12.setLayout( new GridLayout() );
    Label label12 = new Label( comp12, SWT.NONE );
    label12.setText( _itemTMinStoer );
    
    Composite comp13 = new Composite(table, SWT.NONE);
    comp13.setBackground(table.getBackground());
    comp13.setBackgroundMode(SWT.INHERIT_DEFAULT);
    comp13.setLayoutData( gd2 );
    comp13.setLayout( new FillLayout() );
    new MyCompositeZeitDauer(comp13, SWT.NONE, new Date().getTime(), true);
    
    elements[6] = new TableModel( comp12, comp13 );
   
    return elements;
  }

  /**
   * Anzahl der vorhandenen Gruppen
   * @return
   */
  public int getAnzahlGruppen()
  {
    return 1;
  }

  /**
   * Anzahl der vorhandenen Items je Gruppe
   * 
   * @param index   Index der Gruppe
   * @return
   */
  public int getAnzahlItems(int index)
  {
    return 7;
  }
  
  public TableModel[] getItems(int index)
  {
    return (TableModel[])_tvAnzeige.getInput();
  }
  
//  /**
//   * Anzahl der vorhandenen TabelViewer
//   * @return
//   */
//  public int getAnzahlTvAnzeige()
//  {
//    return _tvAnzeige.length;
//  }
//  
  public TableViewer getTvAnzeige()
  {
    return _tvAnzeige;
  }
  
  public void setTvAnzeige( TableViewer tvSet )
  {
    setTvAnzeige( _tvAnzeige, tvSet );
  }
  
  
  private void setTvAnzeige( TableViewer tvAnzeige, TableViewer tvSet )
  {
    TableModel[] modelSet = (TableModel[]) tvSet.getInput();
    TableModel[] models1 = (TableModel[]) tvAnzeige.getInput();

    if ( models1.length == modelSet.length )
    {
      for (TableModel tm : models1)
      {
        Label lb = (Label) tm.getAttribut().getChildren()[0];

        String wert = "";        
        for (TableModel tmSet : modelSet)
        {
          Label lbSet = (Label) tmSet.getAttribut().getChildren()[0];

          
          if ( lb.getText().equals( lbSet.getText() ) )
          {
            wert = tmSet.getSelectedWert();      
          }
        }
        tm.setSelectedWert( wert);
      }
    }
  }

  private class MyCellLabelProvider extends CellLabelProvider
  {

    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

      if (item.getData("EDITOR") != null)
      {
        TableEditor editor = (TableEditor)item.getData("EDITOR");
        editor.dispose();
      }

      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getAttribut();
          break;
        case 1:
          comp = model.getWert();
          break;
        default:
      }
      
      TableEditor editor = new TableEditor(item.getParent());
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);

    }
  }
 
}
