/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.seitenstreifenfreigabeereigniszeit;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;

import de.bsvrz.buv.plugin.ursacheneinheit.tools.AuswahlObjekte;
import de.bsvrz.buv.plugin.ursacheneinheit.tools.Tools;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;

/**
 * Darstellung des Fensters zur Parameternderung mit den Messquerschnitten und den Algorithmus spezifischen
 * Parametern
 * @author ObertM
 */
public class CompositeParameterGesamt extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Algorithmus spezifische Parameter
   */
  private ResultData _resultDataAlg;

  /**
   * Liste mit den mglichen Ereignistypen
   */
  private List<String> _listeMoeglicheEreignisTypen;

  /**
   * Liste mit den mglichen Systemkalendereintrge
   */
  private List<String> _listeMoeglicheSystemKalenderEintraege;

  /**
   * Hauptcomposite
   */
  private Composite _composite;

  /**
   * Composite mit den Buttons zum Verschieben der Messquerschnitte
   */
  private Composite _composite1 = null;

	/**
	 * Compoiste mit den Algorithmus spezifischen Parametern
	 */
	private CompositeParameterAlgorithmus _compositeAlg;

	private Group _groupParameter;

	private AuswahlObjekte _auswahlEreignisTypen;

	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
	 * @param listMoeglicheEreignisTypen Liste mit den mglichen Ereignistypen
	 * @param listMoeglicheSystemKalenderEintraege Liste mit den mglichen Systemkalendereintrgen
	 */
  CompositeParameterGesamt(Composite parent, int style, ResultData resultDataAlg, List<String> listMoeglicheEreignisTypen, List<String> listMoeglicheSystemKalenderEintraege)
  {
    super(parent, style);
    
    _resultDataAlg = resultDataAlg;
    
    _listeMoeglicheEreignisTypen = listMoeglicheEreignisTypen;
    _listeMoeglicheSystemKalenderEintraege = listMoeglicheSystemKalenderEintraege;
    
    initialize();
  }
  
	/**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

  	_auswahlEreignisTypen  = new AuswahlObjekte(_composite, "Ereignistypen", 110);

    createCompositeTab();
    
  	_compositeAlg = null;

    updateData ();
  }

  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   */
  public synchronized void update (ResultData resultDataAlg)
  {
    _resultDataAlg = resultDataAlg;

    updateData ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
    if (_resultDataAlg != null)
    {
    	updateDataEreignisTypen();
    	updataDataAlgorithmus ();
    }
  }
  
  /**
   * Updatefunktion fr die Algorithmus spezifischen Parameter
   */
  private void updataDataAlgorithmus()
  {
    Composite c = (Composite) _groupParameter;
    Control[] children = c.getChildren();
    CompositeParameterAlgorithmus compPar = null;

    Data data = null;
    if (_resultDataAlg.hasData())
      data = _resultDataAlg.getData();
    
    if ( children.length == 0 )
    {
      compPar = new CompositeParameterAlgorithmus ( c, SWT.None, _resultDataAlg, data, _listeMoeglicheSystemKalenderEintraege );
      _compositeAlg  = compPar;
      
      compPar.layout( true );
      c.layout( true );
      
      compPar.getTvParameter().refresh();
    }
    else
    {
      compPar = (CompositeParameterAlgorithmus) children[0];
      compPar.update ( _resultDataAlg, data );
      
      compPar.layout( true );
      c.layout( true );

      compPar.getTvParameter().refresh();
    }
  }

  /**
   * Updatefunktion ber die Messquerschnitte
   */
	private void updateDataEreignisTypen()
  {
		List<String> angewaehlteObjekte = new ArrayList<String>();
		
    if ( _resultDataAlg != null)
    {
      if ( _resultDataAlg.hasData() )
      {
      	Data data = _resultDataAlg.getData();
      	
        Array arrayMQ = data.getArray( "EreignisTyp" );
        for (int k = 0; k < arrayMQ.getLength(); k++)
        {
          String text = "";
          ReferenceValue rV = arrayMQ.getReferenceValue( k );
          if ( rV != null )
            text = Tools.getName(rV.getSystemObject());
          
          angewaehlteObjekte.add(text);
        }
      }
    }
    
    _auswahlEreignisTypen.setAngewaehlteObjekte(_listeMoeglicheEreignisTypen, angewaehlteObjekte);
  }

  /**
   * Methode zum Anlegen des Composite fr die Tabelle
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;

    _groupParameter = new Group(_composite, SWT.NONE);
    _groupParameter.setText("Parameter");
    _groupParameter.setLayout(new GridLayout(1, false));
    _groupParameter.setLayoutData(gridDataGroup);
  }

  /**
   * Methode liefert die Liste der angewhlten Messquerschnitte
   * @return Liste der angewhlten Messquerschnitte
   */
  public org.eclipse.swt.widgets.List getList2()
  {
    return _auswahlEreignisTypen.getZugeordneteObjekte();
  }

  /**
   * Methode liefert die einzelnen Composite fr die Algorithmus spezifischen Parameter zurck
   * @return Composites
   */
  public CompositeParameterAlgorithmus getCompositeAlg()
  {
  	return _compositeAlg;
  }
  
}
