/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.stau;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;

/**
 * Darstellung des Fensters zur Parameternderung mit den Messquerschnitten und den Algorithmus spezifischen
 * Parametern
 * @author ObertM
 */
public class CompositeParameterGesamt extends Composite
{
	/**
	 * Schalter fr Debugausgaben
	 */
  private static final boolean _isDebug = false;
  
  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Messquerschnitte
   */
  private ResultData _resultDataMq;

  /**
   * Ergebnisdaten vom DaV fr die Attributgruppe Algorithmus spezifische Parameter
   */
  private ResultData[] _resultDataAlg;

  /**
   * Liste mit den mglichen Messquerschnitte
   */
  private List<String> _listeMoeglicherObjekte;
  
  /**
   * Hauptcomposite
   */
  private Composite _composite;

  /**
   * Comparator fr die Messquerschnitte
   */
  private MyComparator _comp = new MyComparator();
  
  /**
   * Gruppe mit den Messquerschnitten
   */
  private Group _groupMessQuerschnitte;

  /**
   * Compoiste mit den Messquerschnitten
   */
  private Composite _compositeMessQuerschnitte = null;

  /**
   * Composite mit den Buttons zum Verschieben der Messquerschnitte
   */
  private Composite _composite1 = null;

  /**
   * Button zum Hinzufgen eines Messquerschnitts
   */
  private Button _addButton = null;

  /**
   * Button zum Entfernen eines Messquerschnitts
   */
  private Button _subButton = null;

  /**
   * Composite mit den Reitern
   */
  private CTabFolder _tabFolder = null;

  /**
   * Liste mit den unterschiedlichen Parametergruppen
   */
	private ArrayList<String> _list;

	/**
	 * Compoiste mit den Algorithmus spezifischen Parametern
	 */
	private CompositeParameterStauBelegProFahrstreifen[] _compositeAlg;

	/**
	 * Merker ob die Versorgung der Messquerschnitte angeboten wird oder nicht
	 */
	private boolean _auswahlMQ;

	private Group _groupParameter;

	private CompositeParameterMessQuerschnitte _compositeParameterMQ;

	private CompositeParameterStauBelegung _compositeParameterStauBelegung;

	private CompositeParameterStauGeschwindigkeit _compositeParameterStauGeschwindigkeit;

	private CompositeParameterStauStoerfall _compositeParameterStauStoerfall;

	private CompositeParameterStauASDA _compositeParameterStauASDA;

	private CompositeParameterStauMARZ _compositeParameterStauMARZ;

  private String _textMQ    = "Zuordnung MQ";
  private String _textStau1 = "Staukriterium 1";
  private String _textStau2 = "Staukriterium 2";
  private String _textStau3 = "Staukriterium 3";
  private String _textStau4 = "Staukriterium 4";
  private String _textStau5 = "Staukriterium 5";

	/**
	 * Konstruktor der Klasse
	 * @param parent Aufrufendes Composite
	 * @param style Stil
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
	 * @param listMoeglicheMessQuerschnitte Liste mit den mglichen Messquerschnitten
	 * @param auswahlMQ Merker ob die Versorgung der Messquerschnitte angeboten wird oder nicht
	 */
  CompositeParameterGesamt(Composite parent, int style, ResultData[] resultDataAlg, List<String> listMoeglicheMessQuerschnitte, boolean auswahlMQ)
  {
    super(parent, style);
    
    this._resultDataAlg = resultDataAlg;
    this._auswahlMQ = auswahlMQ;
    
    this._listeMoeglicherObjekte = listMoeglicheMessQuerschnitte;
    initialize();
  }
  
	/**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    createCompositeTab();
    
    /*
     * Reiter anlegen fr die unterschiedlichen Parameter
     */
    _list = new ArrayList<String>();
    
    if (_auswahlMQ)
      _list.add (_textMQ);
    
    _list.add (_textStau1);
    _list.add (_textStau2);
    _list.add (_textStau3);
    _list.add (_textStau4);
    _list.add (_textStau5);
   
    erzeugeTabItems (_list);
    _tabFolder.setSelection(0);

    updateData ();
  }

  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
	 * @param resultDataMq Ergebnis Messquerschnitte
	 * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   */
  public synchronized void update (ResultData[] resultDataAlg)
  {
    this._resultDataAlg = resultDataAlg;

    updateData ();
  }
  
  /**
   * Updatefunktion zur bernahme von neuen Daten aus dem DaV
   * @param resultDataAlg Ergebnis Algorithmusspezifische Parameter
   */
  public synchronized void updateDataAlg (ResultData[] resultDataAlg)
  {
    this._resultDataAlg = resultDataAlg;

    updataDataAlgorithmus ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updateData ()
  {
  	System.out.println("CompositeParameterGesamt.updateData()");
  	
    if (_resultDataAlg [0] != null)
	  {
	  	updataMessQuerschnitte(_resultDataAlg [0]);
	  }

    updataDataAlgorithmus ();
  }
  
  /**
   * Updatefunktion zum Aktualisieren der Daten
   */
  private void updataDataAlgorithmus ()
  {
    if ((_resultDataAlg [1] != null) || (_resultDataAlg [4] != null))
    {
      updateStauBelegung (_resultDataAlg [1], _resultDataAlg [4]);
    }

    if ((_resultDataAlg [2] != null) || (_resultDataAlg [4] != null))
    {
      updateStauGeschwindigkeit(_resultDataAlg [2], _resultDataAlg [4]);
    }

    if ((_resultDataAlg [3] != null) || (_resultDataAlg [4] != null))
    {
      updateStauStoerfall (_resultDataAlg [3], _resultDataAlg [4]);
    }

    if (_resultDataAlg [4] != null)
    {
      updateStauMARZ ( _resultDataAlg [4]);
      updateStauASDA ( _resultDataAlg [4]);
    }
  }

  /**
   * Methode zum Aktualisieren der Parameter "Zuordnung Messquerschnitt"
   * @param resultData Ergebnisdaten aus dem DaV
   */
  private void updataMessQuerschnitte (ResultData resultData)
  {
    /*
     * Reiter ermitteln
     */

    CTabItem item = null;
		CTabItem[] items = _tabFolder.getItems();
		for (CTabItem cti : items)
			if (cti.getText().equals(_textMQ))
				item = cti;
		
//    CTabItem item = items [0];
      
     if ( item != null )
     {
        Composite c = (Composite) item.getControl();

        if (_compositeParameterMQ == null)
        {
          _compositeParameterMQ = new CompositeParameterMessQuerschnitte( c, SWT.None, resultData, _listeMoeglicherObjekte );
          _compositeParameterMQ.layout( true );
          
          c.layout( true );
        }
        else
        {
        	_compositeParameterMQ.update ( resultData );
        	_compositeParameterMQ.layout( true );
        	
          c.layout( true );
        }
     }
  }

  /**
   * Methode zum Aktualisieren der Parameter "Stau ASDA-Foto"
   * @param resultDataBewertung Ergebnisdaten aus dem DaV
   */
  private void updateStauASDA (ResultData resultDataBewertung)
  {
    /*
     * Reiter ermmitten
     */

    CTabItem item = null;
		CTabItem[] items = _tabFolder.getItems();
		for (CTabItem cti : items)
			if (cti.getText().equals(_textStau5))
				item = cti;

//		CTabItem[] items = _tabFolder.getItems();
//    CTabItem item = items [5];
      
     if ( item != null )
     {
        Composite c = (Composite) item.getControl();

        if (_compositeParameterStauASDA == null)
        {
        	_compositeParameterStauASDA = new CompositeParameterStauASDA ( c, SWT.None, resultDataBewertung );
        	_compositeParameterStauASDA.layout( true );
          
          c.layout( true );
        }
        else
        {
        	_compositeParameterStauASDA.update ( resultDataBewertung );
        	_compositeParameterStauASDA.layout( true );
        	
          c.layout( true );
        }
     }
  }

  /**
   * Methode zum Aktualisieren der Parameter "Stau MARZ"
   * @param resultDataBewertung Ergebnisdaten aus dem DaV
   */
  private void updateStauMARZ (ResultData resultDataBewertung)
  {
    /*
     * Reiter ermmitten
     */

    CTabItem item = null;
		CTabItem[] items = _tabFolder.getItems();
		for (CTabItem cti : items)
			if (cti.getText().equals(_textStau4))
				item = cti;

//		CTabItem[] items = _tabFolder.getItems();
//    CTabItem item = items [4];
      
     if ( item != null )
     {
        Composite c = (Composite) item.getControl();

        if (_compositeParameterStauMARZ == null)
        {
        	_compositeParameterStauMARZ = new CompositeParameterStauMARZ ( c, SWT.None, resultDataBewertung );
        	_compositeParameterStauMARZ.layout( true );
          
          c.layout( true );
        }
        else
        {
        	_compositeParameterStauMARZ.update ( resultDataBewertung );
        	_compositeParameterStauMARZ.layout( true );
        	
          c.layout( true );
        }
     }
  }


  /**
   * Methode zum Aktualisieren der Parameter "Stau Belegung"
   * @param resultDataAlgorithmus Ergebnisdaten aus dem DaV
   * @param resultDataAlg Ergebnisdaten aus dem DaV
   */
  private void updateStauBelegung (ResultData resultDataAlgorithmus, ResultData resultDataBewertung)
  {
    /*
     * Reiter ermmitten
     */

    CTabItem item = null;
		CTabItem[] items = _tabFolder.getItems();
		for (CTabItem cti : items)
			if (cti.getText().equals(_textStau1))
				item = cti;

//		CTabItem[] items = _tabFolder.getItems();
//    CTabItem item = items [1];
      
     if ( item != null )
     {
        Composite c = (Composite) item.getControl();

        if (_compositeParameterStauBelegung == null)
        {
        	_compositeParameterStauBelegung = new CompositeParameterStauBelegung ( c, SWT.None, resultDataAlgorithmus, resultDataBewertung );
        	_compositeParameterStauBelegung.layout( true );
          
          c.layout( true );
        }
        else
        {
        	_compositeParameterStauBelegung.update ( resultDataAlgorithmus, resultDataBewertung );
        	_compositeParameterStauBelegung.layout( true );
        	
          c.layout( true );
        }
     }
  }

  /**
   * Methode zum Aktualisieren der Parameter "Stau Geschwindigkeit"
   * @param resultDataAlgorithmus Ergebnisdaten aus dem DaV
   * @param resultDataBewertung Ergebnisdaten aus dem DaV
   */
  private void updateStauGeschwindigkeit (ResultData resultDataAlgorithmus, ResultData resultDataBewertung)
  {
    /*
     * Reiter ermmitten
     */

    CTabItem item = null;
		CTabItem[] items = _tabFolder.getItems();
		for (CTabItem cti : items)
			if (cti.getText().equals(_textStau2))
				item = cti;

//		CTabItem[] items = _tabFolder.getItems();
//    CTabItem item = items [2];
      
     if ( item != null )
     {
        Composite c = (Composite) item.getControl();

        if (_compositeParameterStauGeschwindigkeit == null)
        {
        	_compositeParameterStauGeschwindigkeit = new CompositeParameterStauGeschwindigkeit( c, SWT.None, resultDataAlgorithmus, resultDataBewertung );
        	_compositeParameterStauGeschwindigkeit.layout( true );
          
          c.layout( true );
        }
        else
        {
        	_compositeParameterStauGeschwindigkeit.update ( resultDataAlgorithmus, resultDataBewertung );
        	_compositeParameterStauGeschwindigkeit.layout( true );
        	
          c.layout( true );
        }
     }
  }

  /**
   * Methode zum Aktualisieren der Parameter "Stau Strfall"
   * @param resultDataAlgorithmus Ergebnisdaten aus dem DaV
   * @param resultDataBewertung Ergebnisdaten aus dem DaV
   */
  private void updateStauStoerfall (ResultData resultDataAlgorithmus, ResultData resultDataBewertung)
  {
    /*
     * Reiter ermmitten
     */

    CTabItem item = null;
		CTabItem[] items = _tabFolder.getItems();
		for (CTabItem cti : items)
			if (cti.getText().equals(_textStau3))
				item = cti;

//		CTabItem[] items = _tabFolder.getItems();
//    CTabItem item = items [3];
      
     if ( item != null )
     {
        Composite c = (Composite) item.getControl();

        if (_compositeParameterStauStoerfall == null)
        {
        	_compositeParameterStauStoerfall = new CompositeParameterStauStoerfall ( c, SWT.None, resultDataAlgorithmus, resultDataBewertung );
        	_compositeParameterStauStoerfall.layout( true );
          
          c.layout( true );
        }
        else
        {
        	_compositeParameterStauStoerfall.update ( resultDataAlgorithmus, resultDataBewertung );
        	_compositeParameterStauStoerfall.layout( true );
        	
          c.layout( true );
        }
     }
  }


  /**
   * Interne Klasse zum Vergleich
   * 
   * @author Krause
   * 
   */
  private class MyComparator implements Comparator<String>
  {

    @Override
    public int compare(String o1, String o2)
    {
      int ret = 0;
      
      //String-Sortierung
      if ( o1 != null )
        ret = o1.compareTo( o2 );
     
//      //Integer-Sortierung
//      Integer i1 = new Integer(o1);
//      Integer i2 = new Integer(o2);
//
//      if (i1 < i2)
//        ret = -1;
//      else if (i1 == i2)
//        ret = 0;
//      else if (i1 > i2)
//        ret = 1;
      
      return ret;
      
    }

  }

  /**
   * Methode zum Anlegen des Composite fr die Tabelle
   */
  private void createCompositeTab()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;
    gridDataGroup.grabExcessVerticalSpace = true;
    gridDataGroup.verticalAlignment = GridData.FILL;

  	GridData gridDataTabFolder = new GridData();
    gridDataTabFolder.horizontalAlignment = GridData.FILL;
    gridDataTabFolder.grabExcessHorizontalSpace = true;
    gridDataTabFolder.grabExcessVerticalSpace = true;
    gridDataTabFolder.verticalAlignment = GridData.FILL;

    _groupParameter = new Group(_composite, SWT.NONE);
    _groupParameter.setText("Parameter");
    _groupParameter.setLayout(new GridLayout(1, false));
    _groupParameter.setLayoutData(gridDataGroup);
  
    _tabFolder = new CTabFolder(_groupParameter, SWT.NONE);
    _tabFolder.setLayout(new GridLayout());
    _tabFolder.setLayoutData(gridDataTabFolder);
    _tabFolder.addSelectionListener(new SelectionListener() {

			@Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      @Override
      public void widgetSelected(SelectionEvent e)
      {
        CTabFolder folder = (CTabFolder)e.getSource();        
        CTabItem itemSel = folder.getSelection();
        
        Composite cSel = (Composite)itemSel.getControl();
        
        /*
         * Damit die Tabellen in den einzelnen Parameterdialogen richtig dargestellt werden 
         * (in der richtigen Hhe der Zeilen) muss beim Anwhlen eines Reiters mit so einer Tabelle
         * die Tabelle aktualisiert werden.
         */
        if (((Composite)cSel).getChildren().length > 0)
        {
          if (itemSel.getText().equals(_list.get(1)))
          	if (_compositeParameterStauBelegung != null)
          	  _compositeParameterStauBelegung.refreshTabelle();
          
          if (itemSel.getText().equals(_list.get(2)))
          	if (_compositeParameterStauGeschwindigkeit != null)
          	  _compositeParameterStauGeschwindigkeit.refreshTabelle();

          if (itemSel.getText().equals(_list.get(3)))
          	if (_compositeParameterStauStoerfall != null)
          	  _compositeParameterStauStoerfall.refreshTabelle();
        }
      }
    });
  }

  /**
   * Reiter fr die Liste der Objekte erstellen
   * 
   * @param list
   */
  public void erzeugeTabItems( List<String> list )
  {
  	for (int i=0; i<list.size(); ++i)
  	{
      String s = list.get(i);
     
      CTabItem tabItem = new CTabItem( _tabFolder, SWT.NONE );

      String tabName = s;
      tabItem.setText( tabName );
      tabItem.setData( tabName, s );

      GridData gridData2 = new GridData();
      gridData2.horizontalAlignment = GridData.FILL;
      gridData2.grabExcessHorizontalSpace = true;
      gridData2.grabExcessVerticalSpace = true;
      gridData2.verticalAlignment = GridData.FILL;

      Composite c1 = new Composite( _tabFolder, SWT.NONE );
      c1.setLayout( new GridLayout() );
      c1.setLayoutData( gridData2 );

      tabItem.setControl( c1 );
    }
  }

  /**
   * Methode liefert die einzelnen Composite fr die Algorithmus spezifischen Parameter zurck
   * @return Composites
   */
  public CompositeParameterStauBelegProFahrstreifen[] getCompositeAlg()
  {
  	return _compositeAlg;
  }

  /**
   * Methode liefert das Composite zum Versorgen der Parameter "Zuordnung Messquerschnitte" 
   * @return Composite
   */
	public CompositeParameterMessQuerschnitte getCompositeMessQuerschnitte()
  {
	  return _compositeParameterMQ;
  }

	/**
   * Methode liefert das Composite zum Versorgen der Parameter "Stau Belegung" 
   * @return Composite
   */
	public CompositeParameterStauBelegung getCompositeParameterStauBelegung ()
  {
	  return _compositeParameterStauBelegung;
  }

	/**
   * Methode liefert compositeParameterStauGeschwindigkeit zurck
   * @return compositeParameterStauGeschwindigkeit
   */
  public CompositeParameterStauGeschwindigkeit getCompositeParameterStauGeschwindigkeit()
  {
  	return _compositeParameterStauGeschwindigkeit;
  }

	/**
   * Methode liefert compositeParameterStauStoerfall zurck
   * @return compositeParameterStauStoerfall
   */
  public CompositeParameterStauStoerfall getCompositeParameterStauStoerfall()
  {
  	return _compositeParameterStauStoerfall;
  }

  /**
   * Methode schreibt die eingestellten Daten in eine Attributgruppe vom Typ 
   * UrsachenEinheitStaukriterium (atg.sswSbaUeStaukriterium)
   * @param data Datenfeld
   */
	public void erzeugeAttributGruppeBewertung(Data data)
  {
		Data d1 = data.getItem("Staukriterium1");
		_compositeParameterStauBelegung.getBewertung().erzeugeAttributGruppe(d1);
		
		Data d2 = data.getItem("Staukriterium2");
		_compositeParameterStauGeschwindigkeit.getBewertung().erzeugeAttributGruppe(d2);

		Data d3 = data.getItem("Staukriterium3");
		_compositeParameterStauStoerfall.getBewertung().erzeugeAttributGruppe(d3);
		
		Data d4 = data.getItem("Staukriterium4");
		Data d4Stau = d4.getItem("Staukriterium");
		Data d4VS = d4.getItem("Verkehrsstufe");
		
		_compositeParameterStauMARZ.getBewertung().erzeugeAttributGruppe(d4Stau);
		_compositeParameterStauMARZ.getVerkehrsstufen().erzeugeAttributGruppe(d4VS);
		
		Data d5 = data.getItem("Staukriterium5");
		_compositeParameterStauASDA.getBewertung().erzeugeAttributGruppe(d5);
  }
	
  /**
   * Dispose
   */
  @Override
  public void dispose ()
  {
  	if (_compositeParameterStauBelegung != null && !_compositeParameterStauBelegung.isDisposed())
  		_compositeParameterStauBelegung.dispose();

  	if (_compositeParameterStauGeschwindigkeit != null && !_compositeParameterStauGeschwindigkeit.isDisposed())
  		_compositeParameterStauGeschwindigkeit.dispose();

  	if (_compositeParameterStauStoerfall != null && !_compositeParameterStauStoerfall.isDisposed())
  		_compositeParameterStauStoerfall.dispose();

  	if (_compositeParameterStauASDA != null && !_compositeParameterStauASDA.isDisposed())
  		_compositeParameterStauASDA.dispose();

  	if (_compositeParameterStauMARZ != null && !_compositeParameterStauMARZ.isDisposed())
  		_compositeParameterStauMARZ.dispose();
  	
  	super.dispose();
  }
}
