/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.verwaltung;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Definitionsdialog fr SystemObjekte. Die Klasse liefert ein Eingabefeld, in dem ein oder mehrere
 * SystemObjekte bestimmt werden knnen. <br>
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-11</li>
 * </ul>
 * 
 * @author Krause
 * 
 */
public class SystemObjektDefinitionsDialog extends Dialog
{
  private String _bezEingabe = null;
  
  private String _objektTyp = null;
    
  private Shell _shell = null;

  private Composite _compositeTop = null;

  private Composite _compositeBottom = null;

  private Button buttonOk = null;

  private Button buttonAbbrechen = null;

  private Label _label = null;

  private Combo _combo = null;

  private ComboViewer _comboViewer;

  /**
   * Speichert die in diesem Dialog erfassten Daten.
   */
  private EintragSystemObjekt _eintragSystemObjekt = null;

  
  /**
   * Konstruktor, liefert eine neue Instanz der Klasse {@link SystemObjektDefinitionsDialog}.
   * 
   * @param parent
   *          Eltern-Shell, das fr die Darstellung benutzt werden soll.
   */
  public SystemObjektDefinitionsDialog(Shell parent)
  {
    super(parent);

  }
  
  /**
   * ffnet den Dialog {@link SystemObjektDefinitionsDialog} zum Editieren/Anlegen eines neuen regulren Ausdrucks.
   * 
   * @return Regulrer Ausdruck als String.
   */
  public EintragSystemObjekt oeffne()
  {
    return oeffne("Definition rumliche Gltigkeit (Infrastrukturobjekt)", "Netzbestandteil", "typ.netzBestandTeil"); 
  }
  
  /**
   * ffnet den Dialog {@link SystemObjektDefinitionsDialog} zum Editieren/Anlegen eines neuen regulren Ausdrucks.
   * 
   * @param titel
   *          Titel des Dialoges         
   * @param bezEingabe
   *          Bezeichnung des Eingabefeldes 
   * @param objektTyp
   *          Objekttyp der SystemObjekte, die angezeigt werden sollen 
   *          
   * @return Regulrer Ausdruck als String.
   */
  public EintragSystemObjekt oeffne(String titel, String bezEingabe, String objektTyp)
  {
    _bezEingabe = bezEingabe;
    _objektTyp = objektTyp;
    
    // _shell = new Shell();
    _shell = new Shell(getParent(), SWT.TITLE | SWT.APPLICATION_MODAL | SWT.CLOSE);
    _shell.setText(titel);

    _shell.setLayout(new GridLayout());
    createCompositeTop();
    createCompositeBottom();
    _shell.setSize(new Point(436, 103));
    _shell.open();

    holeSystemObjekte();

    Display display = getParent().getDisplay();

    while (!_shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return _eintragSystemObjekt;

  }

//  /**
//   * Generiert die Elemente des Dialogs {@link SystemObjektDefinitionsDialog}.
//   * 
//   */
//  private void generiereGui()
//  {
//  }

  /**
   * bernimmt die nderungen am regulren Ausdruck und schliet den Dialog.
   * 
   */
  private void aktionOk()
  {

    if (_eintragSystemObjekt != null)
      _shell.dispose();
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
      mb.setText("Fehler");
      mb.setMessage("Es muss ein " + _bezEingabe + " ausgewhlt werden!");
      mb.open();
      return;
    }
  }

  /**
   * Verwirft die nderungen an den raeumlichen Gueltigkeiten und schliet den Dialog.
   * 
   */
  private void aktionAbbrechen()
  {
    _eintragSystemObjekt = null;
    _shell.dispose();
  }

  /**
   * This method initializes _compositeTop
   * 
   */
  private void createCompositeTop()
  {
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;
    _compositeTop = new Composite(_shell, SWT.NONE);
    _compositeTop.setLayoutData(gridData);
    _compositeTop.setLayout(gridLayout);
    _label = new Label(_compositeTop, SWT.NONE);
    _label.setText(_bezEingabe + ":");
    createCombo();
  }

  /**
   * This method initializes _compositeBottom
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.justify = false;
    rowLayout.wrap = true;
    rowLayout.pack = false;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.CENTER;
    gridData1.grabExcessHorizontalSpace = false;
    gridData1.verticalAlignment = GridData.CENTER;
    _compositeBottom = new Composite(_shell, SWT.NONE);
    _compositeBottom.setLayoutData(gridData1);
    _compositeBottom.setLayout(rowLayout);
    buttonOk = new Button(_compositeBottom, SWT.NONE);
    buttonOk.setText("Ok");
    buttonOk.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionOk();
      }

    });
    buttonAbbrechen = new Button(_compositeBottom, SWT.NONE);
    buttonAbbrechen.setText("Abbrechen");
    buttonAbbrechen.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionAbbrechen();
      }

    });
  }

  /**
   * This method initializes _combo
   * 
   */
  private void createCombo()
  {
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.verticalAlignment = GridData.CENTER;
    _combo = new Combo(_compositeTop, SWT.READ_ONLY);
    _combo.setLayoutData(gridData2);
    _combo.addModifyListener(new ModifyListener() {

      @Override
      public void modifyText(ModifyEvent e)
      {
        IStructuredSelection structuredSelection = (IStructuredSelection)_comboViewer.getSelection();

        Object obj = structuredSelection.getFirstElement();

        if (obj instanceof SystemObject)
        {
          SystemObject so = (SystemObject)obj;
          ConfigurationObject co = (ConfigurationObject)so;

          _eintragSystemObjekt = new EintragSystemObjekt(co.getName(), co.getInfo().getShortInfo(), so);

        }

      }
    });

    _comboViewer = new ComboViewer(_combo);
    _comboViewer.setLabelProvider(new ILabelProvider() {
      @Override
      public Image getImage(Object element)
      {
        return null;
      }

      @Override
      public String getText(Object element)
      {
        return ((SystemObject)element).getName();
      }

      @Override
      public void addListener(ILabelProviderListener listener)
      {
        // TODO Auto-generated method stub
      }

      @Override
      public void dispose()
      {
        // TODO Auto-generated method stub
      }

      @Override
      public boolean isLabelProperty(Object element, String property)
      {
        return false;
      }

      @Override
      public void removeListener(ILabelProviderListener listener)
      {

      }

    });

  }

  /**
   * Holt die SystemObjekte zum Objekttyp
   */
  private void holeSystemObjekte()
  {
    ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    List<SystemObject> list = connection.getDataModel().getType(_objektTyp).getElements();
    
    Map<String, SystemObject> treeMap = new TreeMap<String, SystemObject>();
    for (SystemObject so : list)
    {
      treeMap.put(so.getName(), so);
    }

    List<SystemObject> sortList = new ArrayList<SystemObject>();
    sortList.addAll(treeMap.values());
    
    for (SystemObject so : sortList)
    {
      _comboViewer.add(so);
    }
  }

}
