/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.buv.plugin.ursacheneinheit.verwaltung;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.ursacheneinheit.Activator;
import de.bsvrz.buv.plugin.ursacheneinheit.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.debug.Debug;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog;

/**
 * ber diesen Definitionsdialog muss ein UrsachenEinheitEintrag neu angelegt werden knnen.
 * 
 * <br>
 * <br>
 * Die Klasse erfllt die technische Anforderungen des Segments Bedienung und Visualisierung (BUV): <br>
 * <ul>
 * <li>TAnf-Vi-BW-C3-10</li>
 * <li>TAnf-Vi-BW-C3-11</li>
 * </ul>
 * 
 * @author Krause
 * 
 */
public class UrsachenEinheitDefinitionsdialog extends Dialog
{
  /**
   * Der Debugger
   */
  private Debug _debug = Debug.getLogger();

  /**
   * Speicher fr die UraschenEinheit.
   */
  private EintragUrsachenEinheit _eintragUrsachenEinheit = null;

  private Shell shell = null;

  private Composite _composite = null;

  private Text _textName = null;

  private Combo _comboObjTyp = null;
  
  private ComboViewer _comboObjTypViewer;
  
  private Composite _compositeBottom = null;

  private Button buttonOk = null;

  private Button buttonAbbrechen = null;

  private Button buttonHilfe = null;

  private DaVKommunikation _daVKommunikation = DaVKommunikation.getInstanz();

  private Boolean _bearbeiten = null;

  private Boolean _kopieren = false;

  private String _name = null;
  
  private SystemObjectType _sotSelected = null;

	private String _nameTyp = null;

	private EintragUrsachenEinheit _original;

  /**
   * Konstruktor fr eine neue Instanz der Klasse {@link UrsachenEinheitDefinitionsdialog}
   * 
   * @param parent
   *          Eltern-Shell, die fr die Darstellung benutzt werden soll.
   */
  public UrsachenEinheitDefinitionsdialog(Shell parent)
  {
    super(parent);
  }

  /**
   * ffnet den Dialog {@link UrsachenEinheitDefinitionsdialog} zum Anlegen eines Eintrags.
   * 
   * @return Liefert die erzeugte UrsachenEinheit als Instanz der Klasse {@link Dialog}.
   */
  public EintragUrsachenEinheit oeffne()
  {
    _bearbeiten = false;

    Shell parent = getParent();
//    shell = new Shell(parent, SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN | SWT.MAX | SWT.CLOSE);
    shell = new Shell();
    
    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( shell, Activator.PLUGIN_ID + ".definitionUrsachenEinheit" );
    
    shell.setText("Definition Automatikprogramm");
    shell.setLayout(new GridLayout());
    createComposite();
    createComboObjTyp();
    
//    createGroupDefaultSchaltgruende();
//    createGroupSchaltgruende();
//    createGroupStrategien();
    
    createCompositeBottom();

//    shell.setSize(new Point(495, 679));
    shell.setSize(new Point(495, 150));

    shell.open();

    Display display = parent.getDisplay();

    while (!shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return _eintragUrsachenEinheit;
  }

  /**
   * ffnet den Dialog {@link UrsachenEinheitDefinitionsdialog} zum Bearbeiten eines bereits erzeugten
   * Eintrags.
   * 
   * @param eintragUrsachenEinheit
   *          UrsachenEinheit, die bearbeitet werden soll.
   * @param kopieren
   *          Information, ob eine Kopie bearbeitet wird.
   * @return Liefert <code>true</code>, falls die UrsachenEinheit bearbeitet wurde, sonst <code>false</code>. Wenn
   *         <code>true</code> geliefert wird, kann die Verwaltungssicht ({@link VerwaltungsSicht})
   *         die Darstellung der Liste aktualisieren.
   */
  public EintragUrsachenEinheit oeffne(EintragUrsachenEinheit eintragUrsachenEinheit, boolean kopieren)
  {
    _kopieren = kopieren;

    _bearbeiten = true;

    _original = eintragUrsachenEinheit;
    
    _eintragUrsachenEinheit = eintragUrsachenEinheit;
   
    Shell parent = getParent();
    shell = new Shell(parent, SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN | SWT.MAX | SWT.CLOSE);
    
    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( shell, Activator.PLUGIN_ID + ".definitionUrsachenEinheit" );
       
    // shell = new Shell();
    shell.setText("Definition UrsachenEinheit Automatikprogramm");
    shell.setLayout(new GridLayout());
    createComposite();
    createComboObjTyp();
    
//    createGroupDefaultSchaltgruende();
//    createGroupSchaltgruende();
//    createGroupStrategien();
    
    createCompositeBottom();
    shell.setSize(new Point(495, 150));
    _textName.setText(_eintragUrsachenEinheit.getName());
    if ( _eintragUrsachenEinheit.getObjTyp() != null)
      _comboObjTypViewer.setSelection( new StructuredSelection( _eintragUrsachenEinheit.getObjTyp() ));

//    _compDefaultSchaltgruende.setListeEintraege( _eintragUrsachenEinheit.getListeEintragDefaultSchaltgruende() );
//    _compSchaltgruende.setListeEintraege( _eintragUrsachenEinheit.getListeEintragSchaltgruende() );
//    _compStrategie.setListeEintraege( _eintragUrsachenEinheit.getListeEintragStrategien() );
//    
//    _compDefaultSchaltgruende.getTableViewer().setInput( _eintragUrsachenEinheit.getListeEintragDefaultSchaltgruende() );
//    _compSchaltgruende.getTableViewer().setInput( _eintragUrsachenEinheit.getListeEintragSchaltgruende() );
//    _compStrategie.getTableViewer().setInput( _eintragUrsachenEinheit.getListeEintragStrategien() );

    shell.open();

    Display display = parent.getDisplay();

    while ( !shell.isDisposed() )
      if ( !display.readAndDispatch() )
        display.sleep();

    return _eintragUrsachenEinheit;
  }

	public EintragUrsachenEinheit oeffne(String nameTyp)
  {
    _bearbeiten = false;
    
    _nameTyp  = nameTyp;

    Shell parent = getParent();
//    shell = new Shell(parent, SWT.TITLE | SWT.APPLICATION_MODAL | SWT.MIN | SWT.MAX | SWT.CLOSE);
    shell = new Shell();
    
    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( shell, Activator.PLUGIN_ID + ".definitionUrsachenEinheit" );
    
    shell.setText("Definition Automatikprogramm");
    shell.setLayout(new GridLayout());
    createComposite();
    createComboObjTyp();
    
//    createGroupDefaultSchaltgruende();
//    createGroupSchaltgruende();
//    createGroupStrategien();
    
    createCompositeBottom();

//    shell.setSize(new Point(495, 679));
    shell.setSize(new Point(495, 150));

    shell.open();

    Display display = parent.getDisplay();

    while (!shell.isDisposed())
      if (!display.readAndDispatch())
        display.sleep();

    return _eintragUrsachenEinheit;
  }

  /**
   * bernimmt die aktuellen Einstellungen. Dabei wird geprft,
   * <ul>
   * <li>ob die bentigten Definitionen eingegeben wurden</li>
   * <li>dass ein Name eingegeben wurde</li>
   * </ul>
   * 
   */
  private void aktionOk()
  {
    EintragUrsachenEinheit eintragUrsachenEinheit = new EintragUrsachenEinheit();

    if ( _name != null && _name.length() > 0 )
    {
      if ( !_bearbeiten || _kopieren )
      {
        String pidName = _name.substring( 0, _name.length() ).replaceAll(" ", "");
        String praefix = ProjektSpezifischeEinstellungen.getInstance().getPraefixUrsachenEinheit();
        
        eintragUrsachenEinheit.setPid(praefix + pidName );
      }
      else
        eintragUrsachenEinheit.setPid(_eintragUrsachenEinheit.getPid());
      
      eintragUrsachenEinheit.setName(_name);
    }
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK | SWT.CANCEL);
      mb.setText("Fehler Definition");
      mb.setMessage("Es muss ein Name angegeben werden!");
      mb.open();
      return;
    }

    if ( _sotSelected != null )
    {
      eintragUrsachenEinheit.setObjTyp( _sotSelected );
    }
    else
    {
      MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK | SWT.CANCEL);
      mb.setText("Fehler Definition");
      mb.setMessage("Es muss ein Objekttyp ausgewhlt werden!");
      mb.open();
      return;      
    }
    
//    eintragUrsachenEinheit.setListeEintragDefaultSchaltgruende( _compDefaultSchaltgruende.getListeEintraege() );
//    eintragUrsachenEinheit.setListeEintragSchaltgruende( _compSchaltgruende.getListeEintraege() );
//    eintragUrsachenEinheit.setListeEintragStrategien( _compStrategie.getListeEintraege() );
    
    _eintragUrsachenEinheit = eintragUrsachenEinheit;

    boolean eingabenGueltig = true;
    
    if (eingabenGueltig)
    {
      
      if (!_bearbeiten || _kopieren)
      {  
        if (!_daVKommunikation.isObjektVorhanden(_eintragUrsachenEinheit.getPid()))
        {
        	/*
        	 * Prfen ob Konfigurationsbereich fr die Ursacheneinheit bereits definiert ist
        	 */
          String pidTyp = _eintragUrsachenEinheit.getPidObjTyp();
          
          ParaVerwaltungDynamischerObjekte v = ParaVerwaltungDynamischerObjekte.getInstance();
          
          List<ResultData> paraDynObjekte = null;
          if (!v.isKonfigurationsBereichDefiniert(pidTyp))
          {
          	v.erzeugeZuordnungZuDefaultBereich(pidTyp);
          	paraDynObjekte = v.getDatenFuerDaV();
          }

          /*
           * Urlasserdialog
           */
          UrlasserDialog urlasserDialog = new UrlasserDialog(shell);
          if ( (urlasserDialog.anzeigen(paraDynObjekte)).getEingabeOk() )
          {
            _daVKommunikation.erzeugeUrsachenEinheitEintrag(_eintragUrsachenEinheit);
            
            if (_kopieren)
            {
            	UrsachenEinheitenKopieren kopieren = new UrsachenEinheitenKopieren();
            	
            	kopieren.kopiereParameter (_original, _eintragUrsachenEinheit);
            }
          }
          else
          {
          	/*
          	 * Falsche Zugangsdaten
          	 * --> Zuordnung zum Defaultbereich intern wieder lschen da dieser ja nicht an den DaV 
          	 *     bertragen wurde. 
          	 */
          	if (paraDynObjekte != null)
          		v.loescheZuordnungZuDefaultBereich(pidTyp);
          }
        }
        else
        {  
          MessageBox mb = new MessageBox(new Shell(Display.getCurrent()), SWT.ICON_INFORMATION | SWT.OK);
          mb.setText("Fehler Definition");
          mb.setMessage("EintragUrsachenEinheit schon vorhanden: " + _eintragUrsachenEinheit.getPid());
          mb.open();
          return;       
        }
      }

      shell.dispose();

    }
  
  }

  /**
   * Schliet den Dialog {@link UrsachenEinheitDefinitionsdialog} ohne bernahme der Einstellungen.
   * 
   */
  private void aktionAbbrechen()
  {
    _eintragUrsachenEinheit = null;
    shell.dispose();
  }

  /**
   * Zeigt die Hilfe zum Dialog {@link UrsachenEinheitDefinitionsdialog} an.
   * 
   */
  private void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".definitionUrsachenEinheit";
    _debug.fine("ContextID: " + contextId);
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }

  /**
   * This method initializes _composite
   * 
   */
  private void createComposite()
  {
    GridData gridData21 = new GridData();
    gridData21.horizontalAlignment = GridData.FILL;
    gridData21.verticalAlignment = GridData.CENTER;
    GridData gridData11 = new GridData();
    gridData11.horizontalAlignment = GridData.FILL;
    gridData11.grabExcessHorizontalSpace = true;
    gridData11.verticalSpan = 3;
    gridData11.verticalAlignment = GridData.FILL;
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.verticalAlignment = GridData.CENTER;
    GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 2;
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.CENTER;
    _composite = new Composite(shell, SWT.NONE);
    _composite.setLayoutData(gridData);
    _composite.setLayout(gridLayout);
    
    //Fr die F1 Taste
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _composite, Activator.PLUGIN_ID + ".automatikprogrammDefinition" );
     
    Label labelName = new Label(_composite, SWT.NONE);
    labelName.setText("Name");
    _textName = new Text(_composite, SWT.BORDER);
    _textName.setLayoutData(gridData1);
    _textName.addModifyListener(new ModifyListener() {

      @Override
      public void modifyText(ModifyEvent e)
      {
        _name = _textName.getText();

      }
    });
    
  }

  /**
   * This method initializes _compositeBottom
   * 
   */
  private void createCompositeBottom()
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    _compositeBottom = new Composite(shell, SWT.NONE);
    _compositeBottom.setLayoutData(gridData3);
    _compositeBottom.setLayout(rowLayout);
    buttonOk = new Button(_compositeBottom, SWT.NONE);
    buttonOk.setText("Ok");
    buttonOk.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {        
        super.mouseDown(e);
        aktionOk();

      }

    });

    buttonAbbrechen = new Button(_compositeBottom, SWT.NONE);
    buttonAbbrechen.setText("Abbrechen");
    buttonAbbrechen.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionAbbrechen();
      }

    });
    buttonHilfe = new Button(_compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHilfe();
      }

    });
  }

  /**
   * This method initializes _combo
   * 
   */
  private void createComboObjTyp()
  {

    Label labelUe = new Label(_composite, SWT.NONE);
    labelUe.setText( "Objekttyp" );
    
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.verticalAlignment = GridData.CENTER;
    _comboObjTyp = new Combo(_composite, SWT.READ_ONLY);
    
    _comboObjTyp.setLayoutData(gridData2);
    _comboObjTyp.addModifyListener(new ModifyListener() {

      @Override
      public void modifyText(ModifyEvent e)
      {
        IStructuredSelection structuredSelection = (IStructuredSelection)_comboObjTypViewer.getSelection();

        Object obj = structuredSelection.getFirstElement();

        if (obj instanceof SystemObjectType)
        {
          _sotSelected = (SystemObjectType)obj;
        }
      }
    });

    _comboObjTyp.setEnabled(!_kopieren);
    
    _comboObjTypViewer = new ComboViewer(_comboObjTyp);
    _comboObjTypViewer.setContentProvider( new ArrayContentProvider() );
    _comboObjTypViewer.setLabelProvider(new ILabelProvider() {
      @Override
      public Image getImage(Object element)
      {
        return null;
      }

      @Override
      public String getText(Object element)
      {
        return ((SystemObjectType)element).getName();
      }

      @Override
      public void addListener(ILabelProviderListener listener)
      {
        // TODO Auto-generated method stub
      }

      @Override
      public void dispose()
      {
        // TODO Auto-generated method stub
      }

      @Override
      public boolean isLabelProperty(Object element, String property)
      {
        return false;
      }

      @Override
      public void removeListener(ILabelProviderListener listener)
      {

      }

    });
    
    holeSystemObjekte();

  }

  /**
   * Holt die SystemObjekte zum Objekttyp
   */
  private void holeSystemObjekte()
  {
    ClientDavInterface connection = DaVKommunikation.getInstanz().holeDaVVerbindung();
    List<SystemObjectType> liste = connection.getDataModel().getType("typ.sswSbaUrsachenEinheit").getSubTypes();

    List<SystemObjectType> listeTmp = new ArrayList<SystemObjectType>();

    for (SystemObjectType sot : liste)
    	bestimmeObjektTypen (sot, listeTmp);
    
    Map<String, SystemObjectType> treeMap = new TreeMap<String, SystemObjectType>();
    for (SystemObjectType sot : listeTmp)
    {
    	if (verwende (sot))
        treeMap.put(sot.getName(), sot);
    }
    
//    for ( SystemObjectType sot : liste )
//    {
//      if ( sot.getSubTypes().size() == 0 )
//      {
//      	if (verwende (sot))
//      	{
//      		System.out
//              .println("UrsachenEinheitDefinitionsdialog.holeSystemObjekte() 1 " + sot.getName());
//          treeMap.put(sot.getName(), sot);
//      	}
//      }
//      else
//      {
//        List<SystemObjectType> subTypes = sot.getSubTypes();
//        for ( SystemObjectType  subSot : subTypes )
//        {
//        	if (verwende (subSot))
//        	{
//        		System.out
//               .println("UrsachenEinheitDefinitionsdialog.holeSystemObjekte() 2 " + subSot.getName());
//        		
//            treeMap.put(subSot.getName(), subSot);
//        	}
//        }
//      }
//    }
//
//    System.out.println("UrsachenEinheitDefinitionsdialog.holeSystemObjekte()" + liste.size() + " - " + listeTmp.size());

    List<SystemObjectType> sortList = new ArrayList<SystemObjectType>();
    
    if (_nameTyp == null)
      sortList.addAll(treeMap.values());
    else
    {
    	for (SystemObjectType sot : treeMap.values())
    		if (sot.getName().equals(_nameTyp))
    			sortList.add (sot);
    }

    _comboObjTypViewer.setInput( sortList );

    if (_nameTyp != null)
    {
    	_comboObjTyp.select(0);
    	_comboObjTyp.setEnabled(false);
    }
  }
  
  /**
   * Methode bestimmt rekursiv die vorhandenen Objekttypen
   * @param sot ausgehendes Objekt
   * @param liste Liste in die die Objekte eingetragen werden
   */
  private void bestimmeObjektTypen(SystemObjectType sot, List<SystemObjectType> liste)
  {
    if ( sot.getSubTypes().size() == 0 )
    {
    	liste.add(sot);
    }
    else
    {
      for (SystemObjectType subTyp : sot.getSubTypes())
      	bestimmeObjektTypen(subTyp, liste);
    }
  }

	/**
   * Methode prft ob ein Ursacheneinheitentyp verwendet wird
   * @param sot Systemobjekttyp
   * @return Info ob Typ verwendet wird
   */
  private boolean verwende(SystemObjectType sot)
  {
  	String[] pidUrsachenEinheiten = NichtZuBetrachtendeUrsachenEinheiten.getPidUrsachenEinheiten();
  	
  	boolean verwenden = true;
  	for (String pid : pidUrsachenEinheiten)
  	  if (sot.getPid().equals(pid))
    		verwenden = false;
  	
  	return verwenden;
  }
}
