package de.bsvrz.sba.plugin.ausgabefenster.treeviewer;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.core.runtime.Status;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.ausgabefenster.Activator;

/**
 * Mit Hilfe dieser Klasse lsst sich die {@link ViewAusgabefenster} mehrfach
 * ffnen. Sie wird eindeutig ber die ID der View und einer SecondaryID,
 * bestehend aus
 * <ul>
 * <li>Pid der Attributgruppe</li>
 * <li>Pid des Aspekts</li>
 * <li>Nummer der Simulationsvariante</li>
 * <li>Komma separierte Liste der Pids der Systemobjekte</li>
 * </ul>
 * 
 * identifiziert. Diese wird als MD5-Hashwert verwendet.
 * 
 * @author HofmannJ
 * 
 */
public class OpenViewAusgabefenster
{
	/**
	 * Primre ID des zu ffnenden Views
	 */
	private static final String _viewID = "de.bsvrz.sba.plugin.ausgabefenster.treeviewer.ViewAusgabefenster";

	/**
	 * Array fr die HEX-Konvertierung der MD5-Prfsumme
	 */
	private static final char[] _digits = { '0', '1', '2', '3', '4', '5', '6',
			'7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };

	/**
	 * Liefert den Zugriff die View {@link ViewAusgabefenster} und stellt diese
	 * dar.
	 * 
	 * @param pidAtg
	 *          Pid der Attributgruppe
	 * @param pidAsp
	 *          Pid des Aspekts
	 * @param simVar
	 *          Nummer der Simulationsvariante
	 * @param so
	 *          Liste der darzustellenden SystemObjekte
	 * @return Liefert die Instanz der geffneten View {@link ViewAusgabefenster}.
	 */
	public static ViewAusgabefenster openView(String pidAtg, String pidAsp,
			short simVar, LinkedList<SystemObject> so)
	{
		ViewAusgabefenster re = null;

		String secId = "";
		secId = pidAtg + ":" + pidAsp + ":" + simVar + ":";

		Iterator<SystemObject> it = so.iterator();
		while (it.hasNext())
		{
			SystemObject systemObject = it.next();
			secId = secId + systemObject.getPid() + ",";
		}

		secId = secId.substring(0, secId.length() - 1);

		String secIdHex = md5sum(secId);

		IWorkbenchWindow window = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow();

		try
		{
			window.getActivePage().showView(_viewID, secIdHex,
					IWorkbenchPage.VIEW_VISIBLE);

			IViewReference reference = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getActivePage().findViewReference(
							_viewID, secIdHex);
			IViewPart view = reference.getView(true);
			if (view instanceof ViewAusgabefenster)
			{
				re = (ViewAusgabefenster) view;
				window.getActivePage().activate(re);
			}

		} catch (PartInitException e)
		{
			String msg = "Die ViewAusgabefenster (ID: " + _viewID + ", SecId: "
					+ secIdHex + ") konnte nicht geffnet werden!.";

			Activator.getDefault().getLog().log(
					new Status(Status.WARNING, Activator.PLUGIN_ID, msg, e));
		}

		return re;
	}

	private static String hexEncode(byte[] bytes)
	{
		StringBuffer s = new StringBuffer(bytes.length * 2);
		for (int i = 0; i < bytes.length; i++)
		{
			byte b = bytes[i];
			s.append(_digits[(b & 0xf0) >> 4]);
			s.append(_digits[b & 0x0f]);
		}
		return s.toString();
	}

	/**
	 * Liefwert den MD5-Hashwert eines {@link String}s.
	 * 
	 * @param wert
	 *          {@link String}
	 * @return MD5-Hashwert des {@link String}s.
	 */
	private static String md5sum(String wert)
	{
		String re = "";

		try
		{
			MessageDigest md = MessageDigest.getInstance("MD5");
			byte[] digest = md.digest(wert.getBytes());
			re = hexEncode(digest);

		} catch (NoSuchAlgorithmException e)
		{
			e.printStackTrace();
		}

		return re;
	}

}
