package de.bsvrz.sba.plugin.ausgabefenster.treeviewer;

import java.util.LinkedList;

import org.eclipse.jface.action.Action;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.actions.parameter.DefaultParameterIds;
import de.bsvrz.sba.plugin.actions.parameter.IParameter;
import de.bsvrz.sba.plugin.actions.parameter.IParameterAction;
import de.bsvrz.sba.plugin.actions.parameter.ParameterRegistry;
import de.bsvrz.sba.plugin.ausgabefenster.Activator;

public class ParameterActionOpenViewAusgabefenster implements IParameterAction
{
	private static final String ID = Activator.PLUGIN_ID + ".OpenAusgabefenster";

	private ParameterRegistry _registry = new ParameterRegistry();

	public static final String vSysObjs = DefaultParameterIds.pidSystemObjects;
	public static final String vPidAtg = DefaultParameterIds.pidAtg;
	public static final String vPidAsp = DefaultParameterIds.pidAsp;
	public static final String vSimVar = DefaultParameterIds.simVar;

	private Action _action = null;

	public ParameterActionOpenViewAusgabefenster()
	{
		// Parameter pidsSystemObjects = new Parameter(
		// vSysObjs,
		// null,
		// "Kommaseparierte Liste von Pids der SystemObjekte, kein Leerzeichen vor
		// und nach den Kommas.");
		// Parameter pidAtg = new Parameter(vPidAtg, null, "Pid der
		// Attributgruppe");
		// Parameter pidAsp = new Parameter(vPidAsp, null, "Pid des Aspekts");
		// Parameter simVar = new Parameter(vSimVar, null,
		// "Simulationsvariante, muss nach short konvertierbar sein.");

		Parameter pidsSystemObjects = new Parameter(
				vSysObjs,
				"mq.MQ_1.010_AB_NO_R",
				"Kommaseparierte Liste von Pids der SystemObjekte, kein Leerzeichen vor und nach den Kommas.");
		Parameter pidAtg = new Parameter(vPidAtg, "atg.strfallZustand",
				"Pid der Attributgruppe");
		Parameter pidAsp = new Parameter(vPidAsp, "asp.strfallVerfahrenMARZ",
				"Pid des Aspekts");
		Parameter simVar = new Parameter(vSimVar, "0",
				"Simulationsvariante, muss nach short konvertierbar sein.");

		_registry.setParameter(pidsSystemObjects);
		_registry.setParameter(pidAtg);
		_registry.setParameter(pidAsp);
		_registry.setParameter(simVar);

		_action = new Action()
		{
			public void run()
			{
				actionOpenAusgabefenster();
			}
		};

		_action.setId(ID);
		_action.setText("ffne Ausgabefenster");
	}

	private void actionOpenAusgabefenster()
	{
		DaVVerbindung daVVerbindung = DaVVerbindung.getInstanz();
		if (daVVerbindung.istDaVVerbindungAktiv())
		{
			String pidSystemObjects = null;
			String pidAtg = null;
			String pidAsp = null;
			short simVar = -1;

			if (_registry != null)
			{
				pidSystemObjects = _registry.getParameterValueAsString(vSysObjs);
				pidAtg = _registry.getParameterValueAsString(vPidAtg);
				pidAsp = _registry.getParameterValueAsString(vPidAsp);
				simVar = _registry.getParameterValueAsShort(vSimVar);
			}

			String out = "";
			out = out + "SystemObjects: " + pidSystemObjects + "\n";
			out = out + "ATG:           " + pidAtg + "\n";
			out = out + "ASP:           " + pidAsp + "\n";
			out = out + "SimVar:        " + simVar + "\n";

			System.out.println(out);

			if ((pidSystemObjects != null) && (pidAtg != null) && (pidAsp != null)
					&& (simVar >= 0))
			{
				/*
				 * SystmObjekte aufbereiten
				 */
				DataModel dataModel = daVVerbindung.getDaVVerbindung().getDataModel();
				LinkedList<SystemObject> systemObjekte = new LinkedList<SystemObject>();
				String[] split = pidSystemObjects.split(",");
				for (String pidSysObj : split)
				{
					if (pidSysObj != null)
					{
						SystemObject systemObject = dataModel.getObject(pidSysObj);
						if (systemObject != null)
						{
							systemObjekte.add(systemObject);
						}
					}
				}

				if (systemObjekte.size() > 0)
				{
					ViewAusgabefenster openView = OpenViewAusgabefenster.openView(pidAtg,
							pidAsp, simVar, systemObjekte);
					openView.darstellungAufbauen(systemObjekte, pidAtg, pidAsp, simVar);
				}

			}
			else
			{
				System.out.println("NEIN!!!");
			}
		}
	}

	@Override
	public String getParameterActionId()
	{
		return ID;
	}

	public IParameter getParameter(String id)
	{
		IParameter re = null;

		synchronized (_registry)
		{
			re = _registry.getParameter(id);
		}

		return re;
	}

	public void setParameters(LinkedList<IParameter> parameters)
	{
		synchronized (_registry)
		{
			_registry.setParameters(parameters);
		}
	}

	private class Parameter implements IParameter
	{
		private String _id = null;
		private Object _value = null;
		private String _info = null;

		public Parameter(String id, Object value, String info)
		{
			this._id = id;
			this._value = value;
			this._info = info;
		}

		@Override
		public String getId()
		{
			return _id;
		}

		@Override
		public String getParameterInfo()
		{
			return _info;
		}

		@Override
		public Object getValue()
		{
			return _value;
		}

		@Override
		public void setValue(Object value)
		{
			this._value = value;
		}

	}

	@Override
	public LinkedList<String> getParameterIds()
	{
		LinkedList<String> re = null;

		if (_registry != null)
		{
			synchronized (_registry)
			{
				re = _registry.getParameterIds();
			}
		}

		return re;
	}

	@Override
	public boolean runParameterAction(LinkedList<IParameter> parameters)
	{
		boolean re = false;

		synchronized (this)
		{
			synchronized (_registry)
			{
				if (parameters != null)
				{
					setParameters(parameters);
					re = true;
				}

				_action.run();
			}
		}

		return re;
	}

	@Override
	public Action getParameterAction()
	{
		return _action;
	}

	@Override
	public void clearParameters()
	{
		synchronized (_registry)
		{
			_registry.clearParameters();
		}
	}

	@Override
	public LinkedList<IParameter> getParameters()
	{
		LinkedList<IParameter> re = null;
		synchronized (_registry)
		{
			re = _registry.getParameters();
		}
		return re;
	}

}
