package de.bsvrz.sba.plugin.ausgabefenster.treeviewer;

import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.TreeEvent;
import org.eclipse.swt.events.TreeListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.dav.daf.main.config.AttributeType;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.ausgabefenster.Activator;

/**
 * Ausgabefenster fr die Baumdarstellung von Attributgruppen.
 * 
 * @author HofmannJ
 * 
 */
public class ViewAusgabefenster extends ViewPart implements ClientReceiverInterface
{
  /**
   * ID dieser View
   */
  public static final String ID = "de.bsvrz.sba.plugin.ausgabefenster.treeviewer.ViewAusgabefenster";

  /**
   * Initiale Breite der Tree-Spalte
   */
  public static final int _breiteTreeSpalte = 250;

  /**
   * Initiale Breite der sonstigen Spalten des Trees
   */
  public static final int _breiteSonstigeSpalten = 200;

  /**
   * Basiscomposite fr die Darstellung
   */
  private Composite _top = null;

  /**
   * Info-Text
   */
  private Text _textInfo = null;

  /**
   * Tree
   */
  private Tree _tree = null;

  /**
   * Treeviewer
   */
  private TreeViewer _treeViewer = null;

  /**
   * Liste der zu aktualisierenden Knoten
   */
  private LinkedList<Node> _updateListe = new LinkedList<Node>();

  /**
   * Datenmodell des DaV
   */
  private DataModel _dataModel = null;

  /**
   * Datenbeschreibung des DaV
   */
  private DataDescription _dd = null;

  /**
   * Verbindung zum DaV
   */
  private ClientDavInterface _connection = null;

  /**
   * Merker fr aktive Anmeldung auf Daten
   */
  private boolean _datenAngemeldet = false;

  /**
   * Liste der darzustellenden Systemobjekte
   */
  private List<SystemObject> _objekte = null;

  /**
   * Pid der darzustellenden ATG, notwendig fr Memento
   */
  private String _atgPid = null;

  /**
   * Pid des Aspekts, notwendig fr Memento
   */
  private String _aspPid = null;

  /**
   * SimVar
   */
  private short _simVar = 0;

  /**
   * Liste der PIDs der SystemObjekte, notwendig fr Memento
   */
  private LinkedList<String> _pidsSystemObjekte = new LinkedList<String>();

  /**
   * Attributgruppe
   */
  private AttributeGroup _atg;

  /**
   * Aspekt
   */
  private Aspect _asp;

  /**
   * Attributkennzeichnung im Pfad
   */
  private static final String ATTRIBUT = "1";

  /**
   * Attributlistenkennzeichnung im Pfad
   */
  private static final String ATTRIBUTLISTE = "2";

  /**
   * Anzahl von Spalten
   */
  private int _anzahlSpalten = 0;

  /**
   * Namen der Spalten bzw. Spaltenelemente
   */
  private String[] _elementNames = null;

  /**
   * Wurzelknoten
   */
  private Node _wurzel = null;

  /**
   * {@link ImageRegistry} zur Speicherung der im Baum verwendeten Grafiken
   */
  private ImageRegistry _ir = new ImageRegistry(PlatformUI.getWorkbench().getDisplay());

  /**
   * Dieses Flag ist <code>true</code>, solange es im {@link TreeViewer} noch keinen initialen Input gab. Danach wird
   * dieses Flag auf <code>false</code> gesetzt. Dies hat zur Folge, dass nur noch Updates auf den {@link TreeViewer}
   * durchgefhrt werden.
   */
  private boolean _noFirstInput = true;

  /**
   * Aktion zum Kollabieren der Selektion im Baum
   */
  private Action _actionCollapseSel = null;

  /**
   * Aktion zum Expandieren der Selektion im Baum
   */
  private Action _actionExpandSel = null;

  /**
   * Aktion zum Kollabieren des gesamten Baumes.
   */
  private Action _actionCollapseAll = null;

  /**
   * Aktion zum Expandieren des gesammten Baumes
   */
  private Action _actionExpandAll = null;

  /**
   * Flag, zeigt an ob die View schon mal geffnet wurde.
   */
  private boolean _viewWurdeSchonMalGeoffnet = false;

  /**
   * Memento
   */
  private IMemento _memento = null;

  @Override
  public void createPartControl(Composite parent)
  {
    _connection = DaVVerbindung.getInstanz().getDaVVerbindung();

    /*
     * Image Registry fllen
     */
    _ir.put("folder_closed", Activator.getImageDescriptor("icons/tree/folder_closed.png"));
    _ir.put("folder_open", Activator.getImageDescriptor("icons/tree/folder_open.png"));
    _ir.put("grau", Activator.getImageDescriptor("icons/tree/grau_16x16.png"));
    _ir.put("collapseall", Activator.getImageDescriptor("icons/tree/collapseall_16x16.png"));
    _ir.put("expandall", Activator.getImageDescriptor("icons/tree/expandall_16x16.png"));
    _ir.put("collapse", Activator.getImageDescriptor("icons/tree/collapse_16x16.png"));
    _ir.put("expand", Activator.getImageDescriptor("icons/tree/expand_16x16.png"));

    _top = new Composite(parent, SWT.NONE);
    _top.setLayout(new GridLayout());

    GridData gridDataText = new GridData();
    gridDataText.horizontalAlignment = GridData.FILL;
    gridDataText.grabExcessHorizontalSpace = true;

    _textInfo = new Text(_top, SWT.BORDER);
    _textInfo.setEditable(false);
    _textInfo.setLayoutData(gridDataText);

    GridData gridDataTree = new GridData();
    gridDataTree.horizontalAlignment = GridData.FILL;
    gridDataTree.grabExcessHorizontalSpace = true;
    gridDataTree.grabExcessVerticalSpace = true;
    gridDataTree.verticalAlignment = GridData.FILL;

    _tree = new Tree(_top, SWT.MULTI | SWT.FULL_SELECTION);
    _tree.setLayoutData(gridDataTree);

    /**
     * Initiale Spalte fr den Baum.
     */
    TreeColumn tc1 = new TreeColumn(_tree, SWT.NONE);
    tc1.setText("Bezeichnung");
    tc1.setWidth(_breiteTreeSpalte);

    _tree.setHeaderVisible(true);
    _tree.setLinesVisible(true);

    _treeViewer = new TreeViewer(_tree);
    _treeViewer.setContentProvider(new ContentProvider());
    _treeViewer.setLabelProvider(new LabelProvider());

    /**
     * Tree Listener auf dem Baum zur Aktualisierung der open/closed Icons
     */
    _tree.addTreeListener(new TreeListener() {

      @Override
      public void treeCollapsed(TreeEvent e)
      {
        TreeItem ti = (TreeItem)e.item;
        ti.setImage(_ir.get("folder_closed"));
      }

      @Override
      public void treeExpanded(TreeEvent e)
      {
        TreeItem ti = (TreeItem)e.item;
        ti.setImage(_ir.get("folder_open"));
      }

    });

    // darstellungAufbauen();
    erzeugeActions();
    registerActions();
    verankereKontextMenu();

    /*
     * Alten Status mittels Memento herstellen, sofern vorhanden
     */
    restoreState();

    /*
     * Verweis auf die Hilfe an die View binden
     */
    PlatformUI.getWorkbench().getHelpSystem().setHelp(_top, "de.bsvrz.sba.plugin.ausgabefenster.ausgabefenster");
  }

  /**
   * Wird der TreeViewer Disposed, mss eine Abmeldung von der Attributgruppe erfolgen
   */
  public void dispose()
  {
    datenAbmelden();
    super.dispose();
  }

  /**
   * Registriert die Actions fr die View {@link ToolBar}
   */
  private void registerActions()
  {
    IToolBarManager tb = this.getViewSite().getActionBars().getToolBarManager();

    /*
     * Selektion Kollabieren
     */
    tb.add(_actionCollapseSel);

    /*
     * Selektion Expandieren
     */
    tb.add(_actionExpandSel);

    /*
     * Alles Kollabieren
     */

    tb.add(_actionCollapseAll);

    /*
     * Alles Expandieren
     */
    tb.add(_actionExpandAll);

  }

  /**
   * Erzeugt die Actions
   */
  private void erzeugeActions()
  {
    /*
     * Selektion Kollabieren
     */
    if (_actionCollapseSel == null)
    {
      _actionCollapseSel = new Action() {
        public void run()
        {
          actionCollapseActualSelection();
        }
      };
      _actionCollapseSel.setText("Collapse Selection");
      _actionCollapseSel.setText("Selektion Kollabieren");
      _actionCollapseSel.setToolTipText("Kollabiert die aktuelle Selektion.");
      _actionCollapseSel.setImageDescriptor(_ir.getDescriptor("collapse"));
      _actionCollapseSel.setId(ID + ".action.collapseSelection");
    }

    /*
     * Selektion expandieren
     */
    if (_actionExpandSel == null)
    {
      _actionExpandSel = new Action() {
        public void run()
        {
          actionExpandActualSelection();
        }
      };
      _actionExpandSel.setText("Selektion Expandieren");
      _actionExpandSel.setToolTipText("Expandiert die aktuelle Selektion.");
      _actionExpandSel.setImageDescriptor(_ir.getDescriptor("expand"));
      _actionExpandSel.setId(ID + ".action.expandSelection");
    }

    /*
     * Alles Kollabieren
     */
    if (_actionCollapseAll == null)
    {
      _actionCollapseAll = new Action() {
        public void run()
        {
          actionCollapseAll();
        }
      };
      _actionCollapseAll.setText("Alles Kollabieren");
      _actionCollapseAll.setToolTipText("Kollabiert den kompletten Baum.");
      _actionCollapseAll.setImageDescriptor(_ir.getDescriptor("collapseall"));
      _actionCollapseAll.setId(ID + ".action.collapseAll");
    }

    /*
     * Alles Expandieren
     */
    if (_actionExpandAll == null)
    {
      _actionExpandAll = new Action() {
        public void run()
        {
          actionExpandAll();
        }
      };
      _actionExpandAll.setText("Alles Expandieren");
      _actionExpandAll.setToolTipText("Expandiert den kompletten Baum.");
      _actionExpandAll.setImageDescriptor(_ir.getDescriptor("expandall"));
      _actionExpandAll.setId(ID + ".action.expandAll");
    }
  }

  /**
   * Kollabiert die aktuelle Selektion im Baum
   */
  private void actionCollapseActualSelection()
  {
    if (_treeViewer != null)
    {
      LinkedList<Node> expandedNodes = new LinkedList<Node>();

      ISelection selection = _treeViewer.getSelection();
      if (selection instanceof StructuredSelection)
      {
        StructuredSelection ss = (StructuredSelection)selection;

        @SuppressWarnings("unchecked")
        Iterator itExpNodes = ss.iterator();
        while (itExpNodes.hasNext())
        {
          Object obj = itExpNodes.next();

          if (obj instanceof Node)
          {
            Node node = (Node)obj;
            if (node.hasChildren())
            {
              if (!expandedNodes.contains(node))
              {
                expandedNodes.add(node);
              }
            }
            else if (node.hasParent())
            {
              Node parentNode = node.getParent();
              if (!expandedNodes.contains(parentNode))
              {
                expandedNodes.add(parentNode);
              }
            }
          }
        }
      }

      if (expandedNodes.size() > 0)
      {
        _tree.setRedraw(false);
        Iterator<Node> it = expandedNodes.iterator();
        while (it.hasNext())
        {
          Node node = it.next();
          _treeViewer.collapseToLevel(node, AbstractTreeViewer.ALL_LEVELS);
        }
        doRefresh();
        _tree.setRedraw(true);
      }

    }
  }

  /**
   * Expandiert die aktuelle Selektion im Baum
   */
  private void actionExpandActualSelection()
  {
    if (_treeViewer != null)
    {
      LinkedList<Node> collapsedNodes = new LinkedList<Node>();

      ISelection selection = _treeViewer.getSelection();
      if (selection instanceof StructuredSelection)
      {
        StructuredSelection ss = (StructuredSelection)selection;

        @SuppressWarnings("unchecked")
        Iterator itColBest = ss.iterator();
        while (itColBest.hasNext())
        {
          Object obj = itColBest.next();

          if (obj instanceof Node)
          {
            Node node = (Node)obj;
            if (node.hasChildren())
            {
              collapsedNodes.add(node);
            }
          }
        }
      }

      if (collapsedNodes.size() > 0)
      {
        _tree.setRedraw(false);
        Iterator<Node> it = collapsedNodes.iterator();
        while (it.hasNext())
        {
          Node colNode = it.next();
          _treeViewer.expandToLevel(colNode, AbstractTreeViewer.ALL_LEVELS);
        }
        doRefresh();
        _tree.setRedraw(true);
      }
    }
  }

  /**
   * Expandiert alles
   */
  private void actionExpandAll()
  {
    _tree.setRedraw(false);
    _treeViewer.expandAll();
    doRefresh();
    _tree.setRedraw(true);
  }

  /**
   * Kollabiert alles
   */
  private void actionCollapseAll()
  {
    _tree.setRedraw(false);
    _treeViewer.collapseAll();
    doRefresh();
    _tree.setRedraw(true);
  }

  /**
   * Verankert das Kontextmen
   * 
   */
  private void verankereKontextMenu()
  {
    MenuManager menuMgr = new MenuManager("#PopupMenu");
    menuMgr.setRemoveAllWhenShown(true);
    menuMgr.addMenuListener(new IMenuListener() {
      public void menuAboutToShow(IMenuManager manager)
      {
        fuelleKontextMenu(manager);
      }
    });

    Menu menu = menuMgr.createContextMenu(_treeViewer.getControl());
    _treeViewer.getControl().setMenu(menu);
    getSite().registerContextMenu(menuMgr, _treeViewer);
  }

  /**
   * Fllt das Kontextmen
   * 
   * @param menuManager
   *          Menmanager des Kontextmens
   */
  private void fuelleKontextMenu(IMenuManager menuManager)
  {
    menuManager.add(_actionExpandAll);
    menuManager.add(_actionCollapseAll);

    ISelection selection = _treeViewer.getSelection();
    if (selection instanceof StructuredSelection)
    {
      StructuredSelection ss = (StructuredSelection)selection;

      if (ss.size() == 1)
      {
        if (ss.getFirstElement() instanceof Node)
        {
          Node node = (Node)ss.getFirstElement();
          if (node.hasChildren())
          {
            boolean expandedState = _treeViewer.getExpandedState(node);
            if (expandedState)
            {
              menuManager.add(_actionCollapseSel);
            }
            else
            {
              menuManager.add(_actionExpandSel);
            }
          }
        }
      }
      else if (ss.size() > 1)
      {
        boolean collapse = false;
        boolean expand = false;

        @SuppressWarnings("unchecked")
        Iterator itKindBestimmen = ss.iterator();
        while (itKindBestimmen.hasNext())
        {
          Object obj = itKindBestimmen.next();
          if (obj instanceof Node)
          {
            Node node = (Node)obj;
            if (node.hasChildren())
            {
              if (_treeViewer.getExpandedState(node))
              {
                collapse = true;
              }
              else
              {
                expand = true;
              }
            }
          }
        }

        if (collapse)
        {
          menuManager.add(_actionCollapseSel);
        }

        if (expand)
        {
          menuManager.add(_actionExpandSel);
        }

      }

    }

  }

  /**
   * Setzt den Text im Info-Feld
   * 
   * @param pidAtg
   *          Pid der Attributgruppe
   * @param pidAsp
   *          Pid des Aspekts
   * @param simVar
   *          Simulationsvariante
   * @param listSystemObjects
   *          Liste der Systemobjekte
   * @param online
   *          Gibt an, ob wir uns im Online oder Offlinemodus befinden.
   */
  private void setTextInfoFeld(String pidAtg, String pidAsp, short simVar, LinkedList<String> listSystemObjects,
      boolean online)
  {
    String text = "Atg: " + pidAtg + " Aspekt: " + pidAsp + " SimVar: " + simVar;

    _textInfo.setText(text);
  }

  /**
   * Baut die Darstellung auf.
   * 
   * @param systemObjekte
   *          Liste der zu bercksichtigenden {@link SystemObject}s.
   * @param atgPid
   *          Pid der Attributgruppe
   * @param aspPid
   *          Pid des Aspekts
   * @param simVar
   *          Simulationsvariante
   */
  public void darstellungAufbauen(List<SystemObject> systemObjekte, String atgPid, String aspPid, short simVar)
  {
    /*
     * Infos fr Memento holen.
     */
    if (systemObjekte != null)
    {
      if (systemObjekte.size() > 0)
      {
        String title = "Ausgabefenster: ";

        Iterator<SystemObject> itSO = systemObjekte.iterator();
        while (itSO.hasNext())
        {
          SystemObject systemObject = itSO.next();
          if (systemObject != null)
          {
            title = title + systemObject.getPid();
            if (itSO.hasNext())
            {
              title = title + ", ";
            }

            if (!_pidsSystemObjekte.contains(systemObject.getPid()))
            {
              _pidsSystemObjekte.add(systemObject.getPid());
            }
          }
        }

        if (title.length() > 0)
        {
          this.setPartName(title);
        }
      }
    }

    setTextInfoFeld(atgPid, aspPid, simVar, _pidsSystemObjekte, true);

    this._atgPid = atgPid;
    this._aspPid = aspPid;
    this._simVar = simVar;

    if (!_viewWurdeSchonMalGeoffnet)
    {
      erzeugeDarstellung(systemObjekte, atgPid, aspPid, simVar);

      _treeViewer.refresh();

      _viewWurdeSchonMalGeoffnet = true;
    }

  }

  /**
   * Erzeugt intern die Darstellung
   * 
   * @param objekte
   *          Liste der Systemobjekte.
   * @param atgPid
   *          Pid der Attributgruppe
   * @param aspPid
   *          Pid des Aspekts
   * @param simVar
   *          Simulationsvariante
   */
  private void erzeugeDarstellung(List<SystemObject> objekte, String atgPid, String aspPid, short simVar)
  {
    this._dataModel = _connection.getDataModel();
    this._objekte = objekte;
    this._atg = _dataModel.getAttributeGroup(atgPid);
    this._asp = _dataModel.getAspect(aspPid);

    for (int i = 0; i < objekte.size(); i++)
    {
      TreeColumn tc = new TreeColumn(_tree, SWT.NONE);
      tc.setText(objekte.get(i).getPid());
      tc.setWidth(_breiteSonstigeSpalten);
    }

    if (simVar > 0)
    {
      _dd = new DataDescription(_atg, _asp, simVar);
    }
    else
    {
      _dd = new DataDescription(_atg, _asp);
    }

    erzeugeWurzel(_atg, objekte);

    datenAnmelden();
  }

  /**
   * Erzeugt die Wurzel des Baumes.
   * 
   * @param atg
   *          Attributgruppe
   * @param objekte
   *          Liste der Systemobjekte
   * @return Liefert den Wurzelknoten zurck.
   */
  private Node erzeugeWurzel(AttributeGroup atg, List<SystemObject> objekte)
  {
    _anzahlSpalten = objekte.size();

    // Spaltenbezeichnungen

    _elementNames = new String[1 + _anzahlSpalten];
    _elementNames[0] = "Bezeichnung";

    for (int i = 0, j = 1; i < _anzahlSpalten; i++)
    {
      SystemObject systemObject = objekte.get(i);
      if (systemObject != null)
      {
        _elementNames[j] = systemObject.getNameOrPidOrId();
      }

      j++;
    }

    // Root (Zeitpunkt)

    Object[] spalten = new Object[1 + _anzahlSpalten];
    spalten[0] = "Zeitpunkt";

    for (int i = 0, j = 1; i < _anzahlSpalten; i++)
    {
      spalten[j] = objekte.get(i).getNameOrPidOrId();
      j++;
    }

    Node wurzel = new Node();
    wurzel.setObject("Zeitpunkt", 0);
    wurzel.addSpalten(spalten);
    wurzel.setPfad("getDate");
    _updateListe.add(wurzel);

    // analyseAtg(atg, wurzel, "", _anzahlSpalten);

    Node topWurzel = new Node();
    topWurzel.addChild(wurzel);

    analyseAtg(atg, topWurzel, "", _anzahlSpalten);

    this._wurzel = topWurzel;

    return wurzel;
  }

  /**
   * ContentProvider fr die Baumdarstellung.
   * 
   * @author HofmannJ
   * 
   */
  private class ContentProvider implements ITreeContentProvider
  {

    @Override
    public Object[] getChildren(Object parentElement)
    {
      Object[] re = null;

      if (parentElement instanceof Node)
      {
        Node elternKnoten = (Node)parentElement;
        if (elternKnoten.getChildren() != null)
        {
          if (elternKnoten.getChildren().size() > 0)
          {
            re = elternKnoten.getChildren().toArray();
          }
        }
      }

      return re;
    }

    @Override
    public Object getParent(Object element)
    {
      Object re = null;

      if (element instanceof Node)
      {
        Node knoten = (Node)element;
        re = knoten.getParent();
      }

      return re;
    }

    @Override
    public boolean hasChildren(Object element)
    {
      boolean re = false;

      if (element != null)
      {
        if (getChildren(element) != null)
        {
          re = getChildren(element).length > 0;
        }
      }

      return re;
    }

    @Override
    public Object[] getElements(Object inputElement)
    {
      return getChildren(inputElement);
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
    }

  }

  /**
   * LabelProvider fr die Baumdarstellung.
   * 
   * @author HofmannJ
   * 
   */
  private class LabelProvider implements ITableLabelProvider
  {

    @Override
    public Image getColumnImage(Object element, int columnIndex)
    {
      Image re = null;

      if (columnIndex == 0)
      {
        if (element instanceof Node)
        {
          Node node = (Node)element;
          if (node.hasChildren())
          {

            boolean expandedState = _treeViewer.getExpandedState(element);
            if (expandedState)
            {
              re = _ir.get("folder_open");
            }
            else
            {
              re = _ir.get("folder_closed");
            }

          }
          else
          {
            re = _ir.get("grau");
          }
        }
      }

      return re;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.viewers.ITableLabelProvider#getColumnText(java.lang.Object, int)
     */
    @Override
    public String getColumnText(Object element, int columnIndex)
    {
      String re = "";

      if (element instanceof Node)
      {
        Node node = (Node)element;

        Object object = node.getObject(columnIndex);
        if (object != null)
        {
          re = object.toString();
        }
      }

      return re;
    }

    @Override
    public void addListener(ILabelProviderListener listener)
    {
    }

    @Override
    public void dispose()
    {
    }

    @Override
    public boolean isLabelProperty(Object element, String property)
    {
      return false;
    }

    @Override
    public void removeListener(ILabelProviderListener listener)
    {
    }

  }

  /**
   * Fhrt die Analyse der Attributgruppe durch.
   * 
   * @param atg
   *          Attributgruppe.
   * @param parent
   *          Elterknoten.
   * @param pfad
   *          Pfad.
   * @param anzahlSpalten
   *          Anzahl der Spalten.
   */
  private void analyseAtg(AttributeGroup atg, Node parent, String pfad, int anzahlSpalten)
  {
    List<Attribute> att = atg.getAttributes();

    Node child;

    Object[] spalten;

    for (int i = 0; i < att.size(); i++)
    {
      String pfad1 = new String(pfad);

      Attribute att2 = att.get(i);

      AttributeType att3 = att2.getAttributeType();

//      System.out.println("Liste Array att2 " + att2.isArray());

      if (att3.getType().getPid().equals("typ.attributListenDefinition"))
      {
        pfad1 += "2" + att2.getName();

        if (attributWertVorhanden(att3.getPid())) // Attributliste hat Attribut
        // "Wert"
        {
          String text = new String(pfad1 + ":1Wert");

          int j = 0;

          spalten = new Object[1 + anzahlSpalten];

          if (att2.isArray())
            spalten[j++] = att2.getName() + "[]";
          else
            spalten[j++] = att2.getName();

          for (int k = 0; k < anzahlSpalten; k++)
          {
            spalten[j++] = "-";
          }

          child = new Node();
          child.setObject(att2.getName(), 0);
          child.addSpalten(spalten);
          parent.addChild(child);
          child.setPfad(text.toString());

          _updateListe.add(child);
        }
        else
        {
          int j = 0;

          spalten = new Object[1 + anzahlSpalten];

          if (att2.isArray())
            spalten[j++] = att2.getName() + "[]";
          else
            spalten[j++] = att2.getName();

          for (int k = 0; k < anzahlSpalten; k++)
          {
            spalten[j++] = "";
          }

          child = new Node();
          child.setObject(att2.getName(), 0);
          child.addSpalten(spalten);
          parent.addChild(child);
        }

        rekursivAtl("  ", att3.getPid(), child, pfad1, anzahlSpalten);
      }
      else
      {
        if (att2.isArray())
        {
          if (att2.isCountLimited())
          {
            child = new Node();
            child.addSpalten(new Object[] { att2.getName(), "" });
            parent.addChild(child);

            for (int j = 0; j < att2.getMaxCount(); j++)
            {
              Node subChild = new Node();
              subChild.addSpalten(new Object[] { "[" + String.valueOf(j + 1) + "]", "-" });
              child.addChild(subChild);
            }

          }
          else
          {
            // System.out.println(" Array mit variabler Gre");
          }
        }
        else
        {
          pfad1 += "1" + att2.getName();

          // System.out.println("Pfad: " + pfad1);

          String text = new String(pfad1);

          int j = 0;

          spalten = new Object[1 + anzahlSpalten];
          spalten[j++] = att2.getName();

          for (int k = 0; k < anzahlSpalten; k++)
          {
            spalten[j++] = "-";
          }

          child = new Node();
          child.setObject(att2.getName(), 0);
          child.addSpalten(spalten);
          parent.addChild(child);
          child.setPfad(text.toString());
          _updateListe.add(child);
        }
      }
    }
  }

  /**
   * Fhrt die rekursive Analsyse der Attributgruppe durch.
   * 
   * @param rand
   * @param pid
   * @param parent
   * @param pfad
   * @param anzahlSpalten
   */
  private void rekursivAtl(String rand, String pid, Node parent, String pfad, int anzahlSpalten)
  {
    AttributeListDefinition atl = (AttributeListDefinition)_dataModel.getObject(pid);

    List<Attribute> att = atl.getAttributes();

    Object[] spalten;

    for (int i = 0; i < att.size(); i++)
    {
      String pfad1 = new String(pfad);

      Attribute att2 = att.get(i);

      AttributeType att3 = att2.getAttributeType();

      Node child;

      if (att3.getType().getPid().equals("typ.attributListenDefinition"))
      {
        pfad1 += ":2" + att2.getName();

        if (attributWertVorhanden(att3.getPid())) // Attributliste hat Attribut
        // "Wert"
        {
          String text = new String(pfad1 + ":1Wert");

          int j = 0;

          spalten = new Object[1 + anzahlSpalten];

          if (att2.isArray())
            spalten[j++] = att2.getName() + "[]";
          else
            spalten[j++] = att2.getName();

          for (int k = 0; k < anzahlSpalten; k++)
          {
            spalten[j++] = "-";
          }

          child = new Node();
          child.setObject(att2.getName(), 0);
          child.addSpalten(spalten);
          parent.addChild(child);
          child.setPfad(text.toString());
          _updateListe.add(child);
        }
        else
        {
          int j = 0;

          spalten = new Object[1 + anzahlSpalten];
          spalten[j++] = att2.getName();

          for (int k = 0; k < anzahlSpalten; k++)
          {
            spalten[j++] = "";
          }

          child = new Node();
          child.setObject(att2.getName(), 0);
          child.addSpalten(spalten);
          parent.addChild(child);
        }

        rekursivAtl(rand + "  ", att3.getPid(), child, pfad1, anzahlSpalten);
      }
      else
      {
        if (att2.isArray())
        {
          if (att2.isCountLimited())
          {
            child = new Node();
            child.addSpalten(new Object[] { att2.getName(), "" });
            parent.addChild(child);

            for (int j = 0; j < att2.getMaxCount(); j++)
            {
              Node subChild = new Node();
              subChild.addSpalten(new Object[] { "[" + String.valueOf(i + 1) + "]", "-" });
              child.addChild(subChild);
            }
          }
          else
          {
            // System.out.println(" Array mit variabler Gre");
          }
        }
        else
        {
          pfad1 += ":1" + att2.getName();

          // System.out.println("Pfad: " + pfad1);

          String text = new String(pfad1);

          int j = 0;

          spalten = new Object[1 + anzahlSpalten];
          spalten[j++] = att2.getName();

          for (int k = 0; k < anzahlSpalten; k++)
          {
            spalten[j++] = "-";
          }

          child = new Node();
          child.setObject(att2.getName(), 0);
          child.addSpalten(spalten);
          parent.addChild(child);
          child.setPfad(text.toString());
          _updateListe.add(child);
        }
      }
    }
  }

  /**
   * Prft, ob ein Attributwert vorhanden ist.
   * 
   * @param pid
   * @return
   */
  private boolean attributWertVorhanden(String pid)
  {
    boolean vorhanden = false;

    AttributeListDefinition atl = (AttributeListDefinition)_dataModel.getObject(pid);

    List<Attribute> att = atl.getAttributes();

    if (att.size() > 0)
    {
      Attribute att2 = att.get(0);

      if (att2.getName().equals("Wert"))
      {
        vorhanden = true;
      }
    }

    return vorhanden;
  }

  /**
   * Knotenklasse fr die Baumdarstellung.
   * 
   * @author HofmannJ
   * 
   */
  private class Node
  {
    /**
     * Eltern Knoten
     */
    private Node _parent = null;

    /**
     * Kinder
     */
    private LinkedList<Node> _children = new LinkedList<Node>();

    /**
     * Eltemente fr spaltenweise Darstellung
     */
    private LinkedList<Object> _elemente = new LinkedList<Object>();

    private Object[] _objects = new Object[_anzahlSpalten + 1];

    /**
     * Pfad zu diesem {@link Node}
     */
    private String _attributPfad = null;

    /**
     * Konstruktor, parameterlos.
     */
    public Node()
    {
    }

    /**
     * Liefert das {@link Object} dieses Knotens.
     * 
     * @return {@link Object} dieses Knotens
     */
    public Object[] getObjects()
    {
      return _objects;
    }

    /**
     * Setzt das {@link Object} dieses Knotens.
     * 
     * @param object
     *          Zu setzendes Knoten-{@link Object}.
     */
    public void setObject(Object[] objects)
    {
      this._objects = objects;
    }

    public Object getObject(int index)
    {
      Object re = null;

      if (index < _objects.length)
      {
        re = _objects[index];
      }

      return re;
    }

    /**
     * Setzt das Objekt.
     * 
     * @param object
     *          Zu setzendes Objekt
     * @param index
     *          Index
     */
    public void setObject(Object object, int index)
    {
      if (index < _objects.length)
      {
        _objects[index] = object;
      }
    }

    /**
     * Liefert den Pfad
     * 
     * @return
     */
    public String getAttributPfad()
    {
      return _attributPfad;
    }

    /**
     * Setzt den Pfad
     * 
     * @param attributPfad
     */
    public void setPfad(String attributPfad)
    {
      this._attributPfad = attributPfad;
    }

    /**
     * Liefert den Eltern-Knoten.
     * 
     * @return Eltern-Knoten oder <code>null</code>, wenn kein Eltern-Knoten gesetzt wurde.
     */
    public Node getParent()
    {
      return _parent;
    }

    /**
     * Setzt den Eltern-Knoten.
     * 
     * @param parent
     *          Eltern-Knoten als Instanz der Klasse {@link Node}.
     */
    public void setParent(Node parent)
    {
      this._parent = parent;
    }

    /**
     * Prft, ob dieser {@link Node} ber einen Parent-{@link Node} verfgt.
     * 
     * @return Liefer <code>true</code> wenn ein Parent-{@link Node} von diesem {@link Node} existiert, sonst
     *         <code>false</code>.
     */
    public boolean hasParent()
    {
      return (_parent != null);
    }

    /**
     * Liefert die Liste aller Kind-Knoten.
     * 
     * @return Liste aller Kind-Knoten
     */
    public LinkedList<Node> getChildren()
    {
      return _children;
    }

    public boolean hasChildren()
    {
      if (_children.size() > 0)
      {
        return true;
      }
      else
      {
        return false;
      }
    }

    /**
     * Fgt ein Kind-Knoten zur Liste der Kind-Knoten hinzu.
     * 
     * @param child
     *          Hinzuzufgender Kind-Knoten als Instanz der Klasse {@link Node}.
     */
    public void addChild(Node child)
    {
      if (!_children.contains(child))
      {
        _children.add(child);
        child.setParent(this);
      }
    }

    /**
     * Entfernt einen Kindknoten, sofern vorhanden, von der Liste der Kind-Knoten.
     * 
     * @param child
     *          Zu entfernender Kindknoten.
     */
    public void removeChild(Node child)
    {
      if (_children.contains(child))
      {
        _children.remove(child);
        child.setParent(null);
      }
    }

    /**
     * Liefert eine Liste der Spaltenelemente dieses Knotens.
     * 
     * @return Liste der Spaltenelemente dieses Knotens mit Instanzen vom Typ {@link Object}.
     */
    public LinkedList<Object> getSpalten()
    {
      return _elemente;
    }

    /**
     * Fgt ein Element zur Liste der Spaltenelemente hinzu, sofern noch nicht vorhanden.
     * 
     * @param element
     *          Hinzuzufgendes Element als Instanz von {@link Object}.
     */
    public void addSpalte(Object element)
    {
      if (!_elemente.contains(element))
      {
        _elemente.add(element);
      }
    }

    public void addSpalten(Object[] elements)
    {
      for (Object object : elements)
      {
        _elemente.add(object);
      }
    }

    /**
     * Entfernt ein Element von der Liste der Spaltenelemente, sofern vorhanden.
     * 
     * @param element
     *          Zu entfernendes Element.
     */
    public void removeSpalte(Object element)
    {
      if (_elemente.contains(element))
      {
        _elemente.remove(element);
      }
    }

    public String toString()
    {
      String re = "";

      re = re + "Anzahl Objekte: " + _objects.length + "\n";
      for (Object obj : _objects)
      {
        re = re + obj + " - ";
      }
      re = re + "\n";

      re = re + "Anzahl Kinder:  " + _children.size() + "\n";

      return re;
    }

    public String KinderDFS(Node node)
    {
      String re = "";

      re = re + "Kind |";

      Iterator<Node> it = _children.iterator();
      while (it.hasNext())
      {
        Node child = it.next();
        re = re + KinderDFS(child);
      }

      return re;
    }

  }

  @Override
  public void update(ResultData[] results)
  {
    int index = 2;

    for (int resultIndex = 0; resultIndex < results.length; ++resultIndex)
    {
      ResultData result = results[resultIndex];
      SystemObject object = result.getObject();

      for (int i = 0; i < _objekte.size(); i++)
      {
        if (_objekte.get(i).equals(object))
        {
          index = i + 1;
          break;
        }
      }

      AttributeGroup atg = result.getDataDescription().getAttributeGroup();

      if (atg == _atg)
      {
        if (result.hasData())
        {
          for (int j = 0; j < _updateListe.size(); j++)
          {
            Node knoten = _updateListe.get(j);
            String pfad = knoten.getAttributPfad();

            if (pfad.equals("getDate"))
            {
              String text = new Date(result.getDataTime()).toString();
              knoten.setObject(text, index);
            }
            else
            {
              String pfadElemente[] = pfad.split(":");

              Data data = result.getData();

              for (int i = 0; i < pfadElemente.length; i++)
              {
                if (pfadElemente[i].startsWith(ATTRIBUTLISTE))
                {
                  data = data.getItem(pfadElemente[i].substring(1));
                }

                if (pfadElemente[i].startsWith(ATTRIBUT))
                {
                  String sub = pfadElemente[i].substring(1);                  
//                  System.out.println("sub: " + data.getAttributeType());                  
                  try
                  {
                    String text = data.getTextValue(sub).getText().toString();
                    knoten.setObject(text, index);
                  }
                  catch (NumberFormatException e)
                  {
                    // TODO Tue nichts, weil erst noch das Verhalten zur Darstellung eins Feldes von Attributlisten
                    // implementiert werden muss!!!
                  }
                }
              }
            }
          }
        }
        else
        {
          String textKeineDaten = "-";

          if (!result.isSourceAvailable())
          {
            textKeineDaten = "keine Daten (keine Quelle)";
          }
          else
          {
            textKeineDaten = "keine Daten";
          }

          for (int j = 0; j < _updateListe.size(); j++)
          {
            Node knoten = _updateListe.get(j);
            String pfad = knoten.getAttributPfad();

            if (pfad.equals("getDate"))
            {
              String text = new Date(result.getDataTime()).toString();
              knoten.setObject(text, index);
            }
            else
            {
              knoten.setObject(textKeineDaten, index);
            }
          }
        }
      }
    }

    doRefresh();
  }

  /**
   * Fhrt den Refresh des Baumes durch.
   */
  private void doRefresh()
  {
    Runnable runner = new Runnable() {

      @Override
      public void run()
      {
        if (_wurzel != null)
        {
          if (_top != null && !_top.isDisposed())
          {
            if (_noFirstInput)
            {
              _tree.setRedraw(false);
              _treeViewer.setInput(_wurzel);
              _treeViewer.expandAll();
              _noFirstInput = false;
              _tree.setRedraw(true);
            }
            else
            {
              _treeViewer.refresh();
            }
          }
        }

      }
    };
    PlatformUI.getWorkbench().getDisplay().asyncExec(runner);
  }

  /**
   * Daten anmelden, wenn nicht angemeldet.<br>
   */
  private void datenAnmelden()
  {
    if (!_datenAngemeldet)
    {
      // System.out.println("Melde Daten an.");

      _datenAngemeldet = true;

      if (!_dd.getAspect().getPid().equals("asp.eigenschaften"))
      {
        _connection.subscribeReceiver(this, _objekte, _dd, ReceiveOptions.delayed(), ReceiverRole.receiver());
      }else{
        ResultData[] ress = new ResultData[_objekte.size()];
        for (int i = 0; i < _objekte.size(); i++)
        {
          SystemObject so = _objekte.get(i);
          Data data = so.getConfigurationData(_dd.getAttributeGroup());
          ress[i] = new ResultData(so, _dd, new Date().getTime(), data);          
        }
        update(ress);
      }
    }
  }

  /**
   * Daten abmelden, wenn angemeldet.<br>
   */
  private void datenAbmelden()
  {
    if (_datenAngemeldet)
    {
      _datenAngemeldet = false;

      // System.out.println("Melde Daten ab.");

      if (!_dd.getAspect().getPid().equals("asp.eigenschaften"))
      {
        _connection.unsubscribeReceiver(this, _objekte, _dd);
      }
    }
  }

  public void saveState(IMemento memento)
  {
    if (_atgPid != null)
    {
      memento.putString("pitAtg", _atgPid);
    }

    if (_aspPid != null)
    {
      memento.putString("pidAsp", _aspPid);
    }

    if (_pidsSystemObjekte != null)
    {
      if (_pidsSystemObjekte.size() > 0)
      {
        memento.putInteger("AnzahlObjekte", _pidsSystemObjekte.size());
        for (int i = 0; i < _pidsSystemObjekte.size(); i++)
        {
          memento.putString("pidObj" + i, _pidsSystemObjekte.get(i));
        }

      }
    }

  }

  /**
   * Stellt den Status wieder her, der im _memento abgespeichert wurde.
   */
  private void restoreState()
  {
    if (_memento != null)
    {
      this._atgPid = _memento.getString("pitAtg");
      this._aspPid = _memento.getString("pidAsp");

      String simVar = _memento.getString("simVar");
      if (simVar != null)
      {
        try
        {
          this._simVar = Short.valueOf(simVar);
        }
        catch (NumberFormatException nfe)
        {

        }
      }

      Integer anzahlObjekte = _memento.getInteger("AnzahlObjekte");

      String msg = "restoreState: Atg=" + _atgPid + " Asp=" + _aspPid + " Anzahl Objekte=" + anzahlObjekte + ".";

      // System.out.println(msg);

      if ((_atgPid != null) && (_aspPid != null) && (anzahlObjekte != null))
      {
        for (int i = 0; i < anzahlObjekte.intValue(); i++)
        {
          String pidObj = _memento.getString("pidObj" + i);
          // System.out.println("PidObj: " + pidObj);
          if (pidObj != null)
          {
            _pidsSystemObjekte.add(pidObj);
          }
        }

        /*
         * Wenn alle notwendigen Daten vorhanden sind und eine Verbindung zum DaV besteht, dann die Darstellung aus dem
         * DaV laden.
         */
        if ((_pidsSystemObjekte.size() > 0) && (_connection != null))
        {
          DataModel dataModel = _connection.getDataModel();

          LinkedList<SystemObject> systemObjekte = new LinkedList<SystemObject>();
          Iterator<String> it = _pidsSystemObjekte.iterator();
          while (it.hasNext())
          {
            String pidObj = it.next();
            SystemObject so = dataModel.getObject(pidObj);
            if (so != null)
            {
              systemObjekte.add(so);
            }
          }

          if (systemObjekte.size() > 0)
          {
            darstellungAufbauen(systemObjekte, _atgPid, _aspPid, _simVar);
          }

        }
        else
        {
          // Offline
          setTextInfoFeld(_atgPid, _aspPid, _simVar, _pidsSystemObjekte, true);
        }

      }
    }
  }

  public void init(IViewSite site, IMemento memento) throws PartInitException
  {
    this._memento = memento;
    init(site);

  }

  @Override
  public void setFocus()
  {
    if (_top != null)
    {
      _top.setFocus();
    }

    doRefresh();
  }

}
