/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.dynobj.configurationAreas.compsites;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.List;

import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.DaVPreLader;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/*******************************************************************************
 * Composite zur Auswahl eines Objekttypes. Wird mit Hilfe des
 * {@link DaVPreLader}s gefllt. <br>
 * Abfrage:
 * connection.getDataModel().getType("typ.dynamischesObjekt").getSubTypes();
 * Diese Abfrage wird so lange ausgefhrt, bis nur noch leere Listen
 * herauskommen!.
 * 
 * @author HofmannJ
 * 
 */
public class CompositeObjectTypeChooser extends Composite
{
	/**
	 * Verbindung zum DaV
	 */
	private ClientDavInterface _connection = null;

	/**
	 * Liste zur Darstellung der Objekttypen
	 */
	private List _listObjTypes;

	/**
	 * {@link ListViewer} fr die Objekttypen.
	 */
	private ListViewer _lvObjTypes;

	/**
	 * Liste der {@link DynamicObjectType}s.
	 */
	private LinkedList<SystemObjectType> _dynObjTypes = null;

	/**
	 * Konstuktor
	 * 
	 * @param parent
	 *          Eltern {@link Composite}, auf dessen Basis die Darstellung
	 *          erfolgen soll.
	 * @param style
	 *          SWT-Stil
	 * @param connection
	 *          Verbindung zum DaV
	 */
	public CompositeObjectTypeChooser(Composite parent, int style,
			ClientDavInterface connection,
			LinkedList<SystemObjectType> dynObjectTypes)
	{
		super(parent, style);
		this._connection = connection;
		this._dynObjTypes = dynObjectTypes;
		createContent();
	}

	/**
	 * Erzeugt die Darstellung.
	 */
	private void createContent()
	{
		_listObjTypes = new List(this, SWT.NONE | SWT.FULL_SELECTION | SWT.SINGLE
				| SWT.H_SCROLL | SWT.V_SCROLL);
		_lvObjTypes = new ListViewer(_listObjTypes);
		_lvObjTypes.setSorter(new ViewerSorter());
		_lvObjTypes.setContentProvider(new CP());

		if (_dynObjTypes != null)
		{
			_lvObjTypes.setInput(_dynObjTypes);
		}

		/*
		 * Layout
		 */
		GridLayout gl = new GridLayout(1, false);
		this.setLayout(gl);

		GridData gdList = new GridData();
		gdList.grabExcessHorizontalSpace = true;
		gdList.horizontalAlignment = GridData.FILL;
		gdList.grabExcessVerticalSpace = true;
		gdList.verticalAlignment = GridData.FILL;
		_listObjTypes.setLayoutData(gdList);
	}

	/**
	 * Liefert die Liste der selektierten ObjektTypen zurck.
	 * 
	 * @return Liste der selektierten ObjektTypen
	 */
	public LinkedList<SystemObjectType> getSelectedObjectTypes()
	{
		LinkedList<SystemObjectType> re = new LinkedList<SystemObjectType>();

		ISelection selection = _lvObjTypes.getSelection();
		if (selection instanceof StructuredSelection)
		{
			StructuredSelection ss = (StructuredSelection) selection;
			@SuppressWarnings("unchecked")
			Iterator it = ss.iterator();
			while (it.hasNext())
			{
				Object object = it.next();
				if (object instanceof SystemObjectType)
				{
					SystemObjectType sot = (SystemObjectType) object;
					re.add(sot);
				}
			}
		}

		return re;
	}

	/**
	 * Liefert die Liste der Pids der selektierten {@link SystemObjectType}s
	 * zurck.
	 * 
	 * @return Liste der Pids der selektierten {@link SystemObjectType}s
	 */
	public LinkedList<String> getSelectedObjectTypesPids()
	{
		LinkedList<String> re = new LinkedList<String>();

		ISelection selection = _lvObjTypes.getSelection();
		if (selection instanceof StructuredSelection)
		{
			StructuredSelection ss = (StructuredSelection) selection;
			@SuppressWarnings("unchecked")
			Iterator it = ss.iterator();
			while (it.hasNext())
			{
				Object object = it.next();
				if (object instanceof SystemObjectType)
				{
					SystemObjectType sot = (SystemObjectType) object;
					re.add(sot.getPid());
				}
			}
		}

		return re;
	}

	/**
	 * Setzt die zu selektierenden {@link SystemObjectType}s.
	 * 
	 * @param types
	 *          Zu selektierenden {@link SystemObjectType}s
	 */
	public void setSelectedObjectTypes(LinkedList<SystemObjectType> types)
	{
		if ((types != null) && (types.size() > 0))
		{
			StructuredSelection ss = new StructuredSelection(types);
			_lvObjTypes.setSelection(ss);
		}
	}

	/**
	 * Setzt die zu selektierenden {@link SystemObjectType}s an hand Ihrer PIDs
	 * 
	 * @param typePids
	 *          Liste von PIDs der zu selektierenden {@link SystemObjectType}s.
	 */
	public void setSelectedObjectTypesPids(LinkedList<String> typePids)
	{
		LinkedList<SystemObjectType> types = new LinkedList<SystemObjectType>();

		if ((_connection != null) && (typePids != null))
		{
			DataModel model = _connection.getDataModel();
			Iterator<String> it = typePids.iterator();
			while (it.hasNext())
			{
				String pidType = it.next();
				try
				{
					SystemObjectType type = model.getType(pidType);
					if (type != null)
					{
						types.add(type);
					}
				}
				catch (IllegalArgumentException iae)
				{
					// iae.printStackTrace();
				}
			}
		}

		if (types.size() > 0)
		{
			setSelectedObjectTypes(types);
		}
	}

	/**
	 * ContentProvider
	 * 
	 * @author HofmannJ
	 * 
	 */
	private class CP implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = new Object[] { "Kein Element verfgbar!" };

//			if (inputElement instanceof GtmDatenBasis)
//			{
//				GtmDatenBasis datenBasis = (GtmDatenBasis) inputElement;
//				re = datenBasis.getSystemObjectTypes().toArray();
//			}

			if (inputElement instanceof LinkedList)
			{
				LinkedList<Object> ll = (LinkedList<Object>) inputElement;
				LinkedList<SystemObjectType> dynObjTypes = new LinkedList<SystemObjectType>();
				Iterator<Object> it = ll.iterator();
				while (it.hasNext())
				{
					Object obj = it.next();
					if (obj instanceof SystemObjectType)
					{
						dynObjTypes.add((SystemObjectType) obj);
					}
				}
				
				if(dynObjTypes.size() > 0)
				{
					re = dynObjTypes.toArray();
				}

			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	/**
	 * Setzt die Editierbarkeit der Liste zur Darstellung der verfgbaren
	 * Objekttypen.
	 * 
	 * @param editable
	 *          Wenn <code>true</code>, dann ist die Liste editierbar, bei
	 *          <code>false</code> nicht.
	 */
	public void setEditable(boolean editable)
	{
		if (editable)
		{
			_listObjTypes.setEnabled(true);
		}
		else
		{
			_listObjTypes.setEnabled(false);
		}
	}

	/**
	 * Prft, ob die Liste zur Darstellung von Objekttypen editierbar ist oder
	 * nicht.
	 * 
	 * @return Wenn <code>true</code>, dann ist die Liste editierbar, bei
	 *         <code>false</code> nicht.
	 */
	public boolean isEditable()
	{
		return _listObjTypes.isEnabled();
	}

}
