/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.kontextmenu;


import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List; 
import java.util.Map;
//import java.util.Map;
//import java.util.TreeMap;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.Separator;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.actions.parameter.IParameterAction;
import de.bsvrz.sba.plugin.actions.parameter.verwaltung.ParameterActionRegistry;
import de.bsvrz.sba.plugin.actions.parameter.verwaltung.ParameterActionRegistryElement;
import de.bsvrz.sba.plugin.navigator.Activator;
import de.bsvrz.sba.plugin.navigator.kontextmenu.MenuAktion.AKTION;
import de.bsvrz.sba.plugin.navigator.navigatorModell.NavigatorElement.KATEGORIE;

/**
 * Klasse fr den Inhalt eines Kontextmens abhngig von der Kategorie
 * 
 * @author Krause
 * 
 */
public class KontextMenuInhalt
{
  /**
   * Debug-Flag
   */
  private static final boolean m_debug = false;

  /**
   * NavigatorID fr die Identifizierung der gltigen Aktionen (Plug-in's)
   */
  private static final String m_navigatorId = "NAVIGATORID";
  
  /**
   * Datei mit der Aktionen des Kontextmens
   */
  private static final String m_inputFilePathAktionen = "XML/aktions_typen.xml";

  /**
   * Datei mit der Baumstruktur des Kontextmens
   */
  private static final String m_inputFilePathKontextMenu = "XML/kontextmenu.xml";

  /**
   * Mens zu den Kategorien der Baumknoten
   */
  private static final HashMap<KATEGORIE, String> KATEGORIE_MENU = new HashMap<KATEGORIE, String>();
  static
  {
//    KATEGORIE_MENU.put( KATEGORIE.Anzeigequerschnitte, "typ.baumAqDarst" );
    KATEGORIE_MENU.put( KATEGORIE.Anzeigequerschnitt, "typ.baumAqDarst" );
//    KATEGORIE_MENU.put( KATEGORIE.Anzeige, "typ.baumAqDarst" );
//    KATEGORIE_MENU.put( KATEGORIE.KonfigBereich, "typ.baumMq" );
//    KATEGORIE_MENU.put( KATEGORIE.Messquerschnitte, "typ.baumMq" );
    KATEGORIE_MENU.put( KATEGORIE.Messquerschnitt, "typ.baumMq" );
    KATEGORIE_MENU.put( KATEGORIE.Fahrspur, "typ.baumFs" );
    KATEGORIE_MENU.put( KATEGORIE.Kalender, "typ.baumSystemKalenderEintrag");
    KATEGORIE_MENU.put( KATEGORIE.Ereignis, "typ.baumEreignis");
    KATEGORIE_MENU.put( KATEGORIE.Ereignistyp, "typ.baumEreignisTyp");
    KATEGORIE_MENU.put( KATEGORIE.UrsachenEinheit, "typ.baumUrsachenEinheit");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenMessStelle, "typ.baumUmfeldDatenMessStelle");

    KATEGORIE_MENU.put( KATEGORIE.Datenendgeraet, "typ.baumDe");
    KATEGORIE_MENU.put( KATEGORIE.DatenendgeraetDe128, "typ.baumDe128");
    KATEGORIE_MENU.put( KATEGORIE.DatenendgeraetDeAxl, "typ.baumDeAxl");
    KATEGORIE_MENU.put( KATEGORIE.DatenendgeraetDeLve, "typ.baumDeLve");
    KATEGORIE_MENU.put( KATEGORIE.DatenendgeraetDeSys, "typ.baumDeSys");
    KATEGORIE_MENU.put( KATEGORIE.DatenendgeraetDeUfd, "typ.baumDeUfd");
    KATEGORIE_MENU.put( KATEGORIE.DatenendgeraetDeVlt, "typ.baumDeVlt");
    KATEGORIE_MENU.put( KATEGORIE.DatenendgeraetDeWww, "typ.baumDeWww");
    KATEGORIE_MENU.put( KATEGORIE.DatenendgeraetDeWzg, "typ.baumDeWzg");

    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorHelligkeit, "typ.baumUmfeldDatenSensorHelligkeit");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorFahrBahnOberflaechenTemperatur, "typ.baumUmfeldDatenSensorFahrBahnOberFlaechenTemperatur");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorFahrBahnOberflaechenZustand, "typ.baumUmfeldDatenSensorFahrBahnOberFlaechenZustand");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorGefrierTemperatur, "typ.baumUmfeldDatenSensorGefrierTemperatur");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorLuftTemperatur, "typ.baumUmfeldDatenSensorLuftTemperatur");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorNiederschlagsart, "typ.baumUmfeldDatenSensorNiederschlagsart");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorNiederschlagsintensitaet, "typ.baumUmfeldDatenSensorNiederschlagsIntensitaet");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorRelativeLuftFeuchte, "typ.baumUmfeldDatenSensorRelativeLuftFeuchte");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorRestSalz, "typ.baumUmfeldDatenSensorRestSalz");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorSichtWeite, "typ.baumUmfeldDatenSensorSichtWeite");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorTaupunktTemperatur, "typ.baumUmfeldDatenSensorTaupunktTemperatur");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorTemperaturInTiefe1, "typ.baumUmfeldDatenSensorTemperaturInTiefe1");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorTemperaturInTiefe3, "typ.baumUmfeldDatenSensorTemperaturInTiefe3");
    KATEGORIE_MENU.put( KATEGORIE.UmfeldDatenSensorWasserFilmDicke, "typ.baumUmfeldDatenSensorWasserFilmDicke");

  }
  
  /**
   * Liste der gltigen Aktionen (gltige Plug-in's)
   */
  private static HashMap<String, IParameterAction> m_listeparActions = new HashMap<String, IParameterAction>();
  
  /**
   * NavigatorAktionen mit ID
   */
  private static HashMap<String, MenuAktion> NAVAKTION = new HashMap<String, MenuAktion>();

  /**
   * NavigatorMens mit ID
   */
  private static HashMap<String, MenuElement> MENU = new HashMap<String, MenuElement>();
 

  /**
   * Initialisierung erfolgte
   */
  private static boolean m_init = false;
  
  /**
   * Root-Element zum Kontextmen
   */
  private MenuElement m_menu_element;
  
  /**
   * Liste mit den Aktionen und Untermens zum Kontextmen
   */
  private LinkedList<Object> m_kontextMenu;

  /**
   * Liste der anzuzeigende SystemObjekte
   */
  private LinkedList<SystemObject> m_listeObjekte;

  /**
   * Konstruktor fr den Inhalt eines Kontextmens
   * 
   * @param kategorie
   *          Kategorie zur Unterscheidung der verschiedenen Kontextmens
   * @param kategorieParent
   *          Kategorie der Elternteils
   * @param listeObjekte
   *          Liste der anzuzeigenden Systemobjekte
   */
  public KontextMenuInhalt( KATEGORIE kategorie, KATEGORIE kategorieParent, LinkedList<SystemObject> listeObjekte )
  {
    // Menstrukturen einmalig aufbauen
    if ( !m_init )
      init();

    // Erstellen des darzustellenden Kontextmens 
    //mit bertragung der Liste der anzuzeigenden Systemobjekte auf die Aktionen
    erstelleKontextMenu( kategorie, kategorieParent, listeObjekte );

  }

  /**
   * Erstellen des darzustellenden Kontextmens 
   * mit bertragung der Liste der anzuzeigenden Systemobjekte auf die Aktionen
   * 
   * @param kategorie
   *          Kategorie zur Unterscheidung der verschiedenen Kontextmens
   * @param kategorieParent
   *          Kategorie der Elternteils
   * @param listeObjekte
   *          Liste der anzuzeigenden Systemobjekte
   */
  private void erstelleKontextMenu( KATEGORIE kategorie, KATEGORIE kategorieParent, LinkedList<SystemObject> listeObjekte )
  {
    // Liste der anzuzeigenden Systemobjekte
    m_kontextMenu = new LinkedList<Object>();
    if ( listeObjekte != null )
      m_listeObjekte = listeObjekte;
    else
      m_listeObjekte = new LinkedList<SystemObject>();

    // beim Konfigurationsbereich muss das Kontextmen des Elternteils verwendet
    // werden
    if ( kategorie == KATEGORIE.KonfigBereich )
      kategorie = kategorieParent;

    if ( m_debug )
      System.out.println( "Kategorie: " + kategorie );

    // Kontextmen und dessen Kinder zur Kategorie ermitteln
    String menu_id = KATEGORIE_MENU.get( kategorie );
    m_menu_element = MENU.get( menu_id );
    if ( m_menu_element == null )
      return;

    LinkedList<Object> children = m_menu_element.getChildren();
    if ( children.size() == 0 )
      return;

    // Liste der anzuzeigenden Systemobjekte auf die Aktionen des Kontextmens
    // bertragen
    for (Iterator iterator = children.iterator(); iterator.hasNext();)
    {
      Object object = (Object) iterator.next();
      if ( object instanceof MenuElement )
      {
        LinkedList<Object> submenuChildren = ((MenuElement) object).getChildren();
        for (Iterator iterator2 = submenuChildren.iterator(); iterator2.hasNext();)
        {
          Object object2 = (Object) iterator2.next();
          if ( object2 instanceof MenuAktion )
          {
            MenuAktion navAktion = (MenuAktion) object2;
            navAktion.setObjekte( m_listeObjekte );
          }
        }

      }
      else if ( object instanceof MenuAktion )
      {
        MenuAktion navAktion = (MenuAktion) object;
        navAktion.setObjekte( m_listeObjekte );
      }
    }
    m_kontextMenu.addAll( children );
  }

  /**
   * Baumstrukturen der Kontextmens aufbauen
   * 
   * @return m_init true
   */
  private void init()
  {   
    // Aktionen zu den Kontextmens anlegen
    Element elementAktionen = leseRootElement( m_inputFilePathAktionen );

    //keine XML-Datei gefunden
    if ( elementAktionen != null )
      createAktionsTypen( elementAktionen );

    // Menstrukturen der Kontextmens aufbauen
    Element elementKontextMenu = leseRootElement( m_inputFilePathKontextMenu );

    //keine XML-Datei gefunden
    if ( elementKontextMenu != null )
      createMenues( elementKontextMenu );

    m_init = true;
  }

  /**
   * Anlegen der Aktionstypen
   * 
   * @param element
   *          Wurzelelement der Aktionstypen aus einer XML-Datei
   */
  private void createAktionsTypen( Element element )
  {
    //gltige Aktionen (Plug-in's) ermitteln
    ParameterActionRegistry paRegistry = ParameterActionRegistry.getInstance();
    LinkedList<ParameterActionRegistryElement> parActionElements = paRegistry.getParameterActionElements( m_navigatorId );
    for (Iterator iterator = parActionElements.iterator(); iterator.hasNext();)
    {
      ParameterActionRegistryElement parActionElement = (ParameterActionRegistryElement) iterator.next();
      IParameterAction parameterAction = parActionElement.getParameterAction();
      m_listeparActions.put( parameterAction.getParameterActionId(), parameterAction );
      if (m_debug)
      {
        System.out.println("AktionId:" + parameterAction.getParameterActionId() + ":");
        System.out.println("Aktion:" + m_listeparActions.get(parameterAction.getParameterActionId()));
      }

    }

    //Aktionen an Hand der gltigen Plug-in's anlegen
    List children = element.getChildren();

    if ( children == null )
      return;

    for (Iterator iterator = children.iterator(); iterator.hasNext();)
    {
      Element child = (Element) iterator.next();
      String id = child.getAttributeValue( "aktionsTypId" );
      
      //auszufhrende Aktion ermitteln
      AKTION aktionTodo = null;
      IParameterAction parameterAction = null;
      
      String attributeAktion = child.getAttributeValue( "aktion" );
      if (attributeAktion != null && attributeAktion != "" )
      {
        //Plug-in fr diese Aktion vorhanden?
        parameterAction = m_listeparActions.get( attributeAktion );
        //statische Aktion vorhanden?
        if ( parameterAction == null )
        {
          try
          {
            if (m_debug)
              System.out.println("Aktion: " + attributeAktion + " ParaAction: " + parameterAction);
            
            aktionTodo = AKTION.valueOf( attributeAktion );            
          }
          catch (Exception e)
          {
            for (Iterator iterator2 = m_listeparActions.entrySet().iterator(); iterator2.hasNext();)
            {
              Map.Entry<String, IParameterAction> object = (Map.Entry<String, IParameterAction>) iterator2.next();
              System.out.println("AktionId:" + object.getKey() + ":");
              System.out.println("Aktion:" + object.getValue());              
            }
          }

        }
      }
      else
        aktionTodo = AKTION.Sysout;
 
      //Navigatoraktion anlegen
      if (aktionTodo != null || parameterAction != null)
        NAVAKTION.put( id, new MenuAktion( id, child.getAttributeValue( "name" ), null, null, aktionTodo, parameterAction,
          child.getAttributeValue( "atg" ), child.getAttributeValue( "asp" ), null ) );
      
    }
  }

  /**
   * Anlegen der Mens mit den Items
   * 
   * @param element
   *          Wurzelelement/Elternelement der Mens/Items aus einer XML-Datei
   */
  private void createMenues( Element element )
  {
    MenuElement menu = MENU.get( element.getAttributeValue( "id" ) );
    List children = element.getChildren();

    if ( children == null )
      return;

    for (Iterator iterator = children.iterator(); iterator.hasNext();)
    {
      Element child = (Element) iterator.next();
      
      if ( child.getName().equals( "menu" ) )
      {
        String id = child.getAttributeValue( "id" );
        MenuElement submenu = MENU.get( child.getAttributeValue( "id" ) );
        //Falls Submen noch nicht vorhanden und Name angegeben ==> Submen anlegen
        if ( submenu == null && child.getAttributeValue( "name" ) != null )
        {
          submenu = new MenuElement( menu, id, child.getAttributeValue( "name" ) );
          MENU.put( id, submenu );
        }

        if ( menu != null && submenu != null ) // das Wurzelelement wird hierbei nicht bercksichtigt
        {
          menu.add( submenu );
        }
        
        if ( child.getChildren().size() > 0 )
          createMenues( child );
      }
      else if ( child.getName().equals( "item" ) )
      {
        MenuAktion action = NAVAKTION.get( child.getAttributeValue( "aktionsTypId" ) );
        boolean defaultElement = child.getAttributeValue( "default" ).equals( "true" );
        if ( menu != null && action != null )
        {
          menu.add( action, defaultElement );
      }
      }
      else if ( child.getName().equals( "separator" ) )
      {
        menu.add( new Separator() );
      }

    }
  }

  /**
   * Methode liest das Rootelement aus einer XML-Datei aus. Vorher wird geprft,
   * ob die Datei berhaupt existiert und ob sie der DTD Beschreibung
   * entspricht.
   * 
   * @param datei
   *          Name der Datei die in eine XML-Struktur eingelesen werden soll mit
   *          absoluter und relativer Pfad-Angabe
   * @return Element RootElement der XML-Struktur oder null im Fehlerfall
   */
  private Element leseRootElement( String datei )
  {
    URL url = null;

    // Datei mit absolutem oder relativem Pfad ?

    if ( datei.matches( "[a-zA-Z]:.+" ) )
    {
      // Prfen ob Datei berhaupt existiert

      if ( !(new File( datei )).exists() )
      {
        System.out.println( "Datei \"" + datei + "\" nicht gefunden !" );
        return null;
      }
    }
    else
    {
      url = Activator.getDefault().getBundle().getResource( datei );
      if ( url == null )
      {
        System.out.println( "Datei \"" + datei + "\" nicht gefunden !" );
        return null;
      }
      else
        datei = url.toExternalForm();
    }

    // Einlesen der Datei in eine XML-Struktur

    boolean validate = true; // Validierung gegen die jeweilige DTD

    SAXBuilder builder = new SAXBuilder( validate );
    builder.setEntityResolver( new EntityResolver()
    {
      public InputSource resolveEntity( String publicId, String systemId ) throws SAXException, IOException
      {        
        if ( (systemId != null) )
        {
          return new InputSource( systemId );
        }

        return null;
      }
    } );

    Document jdomDokument = null;

    try
    {
      jdomDokument = builder.build( datei );
    }
    catch (JDOMException e)
    {
      System.out.println( e.getMessage() );
      return null;
    }
    catch (IOException e)
    {
      System.out.println( e.getMessage() );
      return null;
    }

    // Rootelement aus eingelesener Datei bestimmen

    Element rootElement = jdomDokument.getRootElement();

    return rootElement;
  }

  /**
   * Fgt weitere anzuzeigende Objekte hinzu
   */
  public void addObjekte( LinkedList<SystemObject> listeObjekte )
  {
    m_listeObjekte.addAll( listeObjekte );
  }

  /**
   * Liefert die Liste der Aktionen und Untermens
   */
  public LinkedList<Object> getKontextListe()
  {
    return m_kontextMenu;
  }
  
  /**
   * Fhrt die <code>run</code>-Methode der default {@link Action} aus.
   * 
   * @return Liefert <code>true</code>, wenn die <code>run</code>-Methode ausgefhrt werden konnte, sonst
   *         <code>false</code> (z.B. wenn keine {@link Action} default gesetzt wurde!).
   */
  public boolean selektierteAktionAusfuehren()
  {
    boolean erfolgreich = false;

    //Default Aktion aus dem Menelement holen
    if (m_menu_element != null)
    {
      Action selektierterEintrag = m_menu_element.getDefaultChild();
      
      if ( selektierterEintrag != null)
      {
        //Default Aktion ausfhren
        selektierterEintrag.run();
        erfolgreich = true;
      }
    }

    return erfolgreich;
  }

}
