/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.kontextmenu;


import java.util.LinkedList;

import org.eclipse.jface.action.Action;

import de.bsvrz.buv.rw.basislib.simulationsvariante.Simulationsvariante;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.actions.parameter.DefaultParameterIds;
import de.bsvrz.sba.plugin.actions.parameter.IParameterAction;
import de.bsvrz.sba.plugin.actions.parameter.ParameterRegistry;

/**
 * Klasse um Aktionen anzulegen
 * 
 * @author Krause
 * 
 */

public class MenuAktion extends Action
{

  /**
   * mgliche statische Aktionen
   */
  public static enum AKTION
    {
      Sysout
    };

  /**
   * ID
   */
  private String m_id;

  /**
   * Name
   */
  private String m_name;

  /**
   * Pfadname der das Bild ab dem Plugin-Verzeichnis identifiziert
   */
  private String m_bildPfad;

  /**
   * Tooltip
   */
  private String m_tooltip;

  /**
   * auszufhrende statische Aktion
   */
  private AKTION m_aktion;

  /**
   * auszufhrende dynamische Aktion
   */
  private IParameterAction m_parameterAction;

  /**
   * Pid der Attributgruppe
   */
  private String m_pidAtg;

  /**
   * Pid des Aspekts
   */
  private String m_pidAsp;

  /**
   * Liste der Systemobjekte
   */
  private LinkedList<SystemObject> m_listeObjekte;

  /**
   * Konstruktor um eine Aktion anzulegen
   * 
   * @param id
   *          ID der Aktion
   * @param name
   *          anzuzeigender Name der Aktion
   * @param bildPfad
   *          BildPfad zur Aktion
   * @param tooltip
   *          anzuzeigender Tooltip
   * @param aktion
   *          auszufhrende statische Aktion
   * @param aktionDyn
   *          auszufhrende dynamische Aktion
   * @param pidAtg
   *          Pid der Attributgruppe
   * @param pidAsp
   *          Pid des Aspekts
   * @param listeObjekte
   *          Liste der Systemobjekte
   */
  public MenuAktion( String id, String name, String bildPfad, String tooltip, AKTION aktion, 
      IParameterAction parameterAction, String pidAtg, String pidAsp, LinkedList<SystemObject> listeObjekte )
  {
    super( name );

    m_id = id;
    m_name = name;
    m_bildPfad = bildPfad;
    m_tooltip = tooltip;
    m_aktion = aktion;
    m_parameterAction = parameterAction;
    m_pidAtg = pidAtg;
    m_pidAsp = pidAsp;
    m_listeObjekte = listeObjekte;

    super.setToolTipText( m_tooltip );

    //wird vorerst nicht bentigt:
//    if ( bildPfad != null && NavigatorView.getMyImageRegistry().getImage( m_bildPfad ) != null )
//      super.setImageDescriptor( NavigatorView.getMyImageRegistry().getDescriptor( m_bildPfad ) );

  }

  public void run()
  {

    if ( m_aktion == null && m_parameterAction != null )
    {
      short sim = 0;
      sim = (short) Simulationsvariante.getInstanz().getSimulationsvariante();

      ParameterRegistry parameterRegistry = new ParameterRegistry();
      parameterRegistry.setParameter( new DefaultParameter( DefaultParameterIds.pidSystemObjects, m_listeObjekte ) );
      parameterRegistry.setParameter( new DefaultParameter( DefaultParameterIds.pidAtg, m_pidAtg ) );
      parameterRegistry.setParameter( new DefaultParameter( DefaultParameterIds.pidAsp, m_pidAsp ) );
      parameterRegistry.setParameter( new DefaultParameter( DefaultParameterIds.simVar, "" + sim ) );

      m_parameterAction.runParameterAction( parameterRegistry.getParameters() );
    }
    else
    {

      switch (m_aktion)
        {
        case Sysout:
        {
          System.out.println( "Aktion <" + m_name + "> noch nicht implementiert! Dynamische Aktion in aktions_typen.xml: " + m_parameterAction);
          break;
        }

        default:
          break;
        }
    }
  }

  /**
   * Setzt die zugeordneten Systemobjekte
   * 
   * @param listeObjekte
   *          Liste der Systemobjekte
   */
  public void setObjekte( LinkedList<SystemObject> listeObjekte )
  {
    m_listeObjekte = listeObjekte;
  }

}
