/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.navigator.tools;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;

import de.bsvrz.sba.plugin.navigator.Activator;

public class Tools
{

	/**
	 * Liest einen String aus einer Datei.
	 * 
	 * @param path
	 *          Pfad zur Datei
	 * @return Liefert den String aus der angegebenen Datei oder <code>null</code>,
	 *         wenn die Datei nicht gefunden wurde.
	 */
	public static String readStringFromFile(String path)
	{
		String re = null;

		File file = new File(path);
		if (file.exists())
		{
			try
			{
				StringBuffer sb = new StringBuffer();
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line;
				try
				{
					while ((line = br.readLine()) != null)
					{
						sb.append(line);
						sb.append("\n");
					}
					re = sb.toString();
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}

			}
			catch (FileNotFoundException e)
			{
				e.printStackTrace();
			}
		}

		return re;
	}

	/**
	 * Liest einen String aus einer Textdatei im Plug-In.
	 * 
	 * @param path
	 *          Pfad innerhalb des Plug-Ins.
	 * @return String der entsprechenden Textdatei oder <code>null</code>, wenn
	 *         die Datei nicht gefunden wurde.
	 */
	public static String readStringFromPluginFile(String path)
	{
		String re = null;

		URL url = Activator.getDefault().getBundle().getResource(path);

		if (url != null)
		{
			String file = url.toExternalForm();
			try
			{
				StringBuffer sb = new StringBuffer();
				FileReader fr = new FileReader(file);
				BufferedReader br = new BufferedReader(fr);
				String line;
				try
				{
					while ((line = br.readLine()) != null)
					{
						sb.append(line);
						sb.append("\n");
					}
					re = sb.toString();
				}
				catch (IOException e)
				{
					e.printStackTrace();
				}

			}
			catch (FileNotFoundException e)
			{
				e.printStackTrace();
			}
		}

		return re;
	}

	/**
	 * Schreibt einen String in eine Datei.
	 * 
	 * @param path
	 *          Pfad zur Datei, die geschrieben werden soll.
	 * @param overWrite
	 *          Wenn <code>true</code>, dann wird eine bereits vorhandene Datei
	 *          berschrieben.
	 * @param writeString
	 *          String, der geschrieben werden soll.
	 * @throws IllegalArgumentException
	 *           Wird geworfen, wenn die Datei bereits existiert und overWrit==<code>false</code>
	 *           ist.
	 */
	public static void writeStringToFile(String path, boolean overWrite,
			String writeString) throws IllegalArgumentException
	{
		File file = new File(path);

		if (file.exists() && overWrite)
		{
			try
			{
				PrintWriter pw = new PrintWriter(file);
				pw.print(writeString);
				pw.close();
			}
			catch (FileNotFoundException e)
			{
				e.printStackTrace();
			}
		}
		else if (!file.exists())
		{
			try
			{
				PrintWriter pw = new PrintWriter(file);
				pw.print(writeString);
				pw.close();
			}
			catch (FileNotFoundException e)
			{
				e.printStackTrace();
			}
		}
		else
		{
			throw new IllegalArgumentException("Die Datei " + path
					+ " existiert und ueberschreiben == flase");
		}
	}
}
