/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.daten;

/**
 * Klasse definiert ber ein Enum die Fahrtrichtung
 * @author Dambach Werke GmbH
 * @author Matthias Obert
 */
public enum FahrtRichtung
{
  /**
   * Richtung Stuttgart
   */
  STUTTGART ("Stuttgart", "DD", 0),
  
  /**
   * Richtung Tbingen
   */
  TUEBINGEN ("Tbingen", "MD", 1),
  
  ;
  
  /**
   * Kodierung im DaV
   */
  private int m_kodierung;
  
  /**
   * Bezeichnung
   */
  private String m_name;

  /**
   * Krzel
   */
  private String m_kuerzel;
  
  /**
   * 
   * @param name
   * @param kodierung
   */
  private FahrtRichtung (String name, String kuerzel, int kodierung)
  {
  	m_name = name;
  	
  	m_kuerzel = kuerzel;
  	
    m_kodierung = kodierung;
  }

  /**
   * Methode liefert die Variable m_kodierung zurck
   * @return Variable m_kodierung
   */
  public int getKodierung()
  {
    return m_kodierung;
  }
  
  /**
   * Methode liefert die Richtung zu einer bergebenen Kodierung
   * @param kodierung Kodierung
   * @return Richtung
   */
  public FahrtRichtung getRichtung (int kodierung)
  {
    for (FahrtRichtung r : FahrtRichtung.values())
    {
      if (r.getKodierung() == kodierung)
        return r;
    }
    
    return null;
  }

  /**
   * Methode liefert die Richtung zu einem bestimmten Namen
   * @param nameRichtung Name
   * @return Richtung
   */
	public static FahrtRichtung getRichtung(String nameRichtung)
  {
		for (FahrtRichtung r : FahrtRichtung.values())
			if (r.getName().equals(nameRichtung))
				return r;
		
    return null;
  }

	/**
   * Methode liefert name zurck
   * @return name
   */
  public String getName()
  {
  	return m_name;
  }

	/**
   * Methode setzt die Variable name
   * @param name zu bernehmender Wert fr name
   */
  public void setName(String name)
  {
  	m_name = name;
  }

	/**
   * Methode setzt die Variable kodierung
   * @param kodierung zu bernehmender Wert fr kodierung
   */
  public void setKodierung(int kodierung)
  {
  	m_kodierung = kodierung;
  }

	/**
   * Methode liefert kuerzel zurck
   * @return kuerzel
   */
  public String getKuerzel()
  {
  	return m_kuerzel;
  }

	/**
   * Methode setzt die Variable kuerzel
   * @param kuerzel zu bernehmender Wert fr kuerzel
   */
  public void setKuerzel(String kuerzel)
  {
  	m_kuerzel = kuerzel;
  }
}
