package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.abhaengigkeiten;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse speichert die Abhngigkeiten einer Anzeige
 * @author Dambach Werke GmbH
 * @author Matthias Obert
 */
public class Abhaengigkeiten
{
  /**
   * Hauptanzeige
   */
  private SystemObject m_hauptAnzeige;
  
  /**
   * Abhngige Anzeige
   */
  private SystemObject m_abhaengigeAnzeige;
  
  /**
   * Parameterloser Konstruktor der Klasse
   */
  public Abhaengigkeiten()
  {
  }

  /**
   * Konstruktor der Klasse
   * @param hauptAnzeige Hausptanzeige
   * @param abhaengigeAnzeige Abhngige Anzeige
   */
  public Abhaengigkeiten(SystemObject hauptAnzeige, SystemObject abhaengigeAnzeige)
  {
    m_hauptAnzeige = hauptAnzeige;
    m_abhaengigeAnzeige = abhaengigeAnzeige;
  }

  /**
   * Methode liefert die Variable m_hauptAnzeige zurck
   * @return Variable m_hauptAnzeige
   */
  public SystemObject getHauptAnzeige()
  {
    return m_hauptAnzeige;
  }

  /**
   * Methode setzt die Variable m_hauptAnzeige
   * @param hauptAnzeige Wert auf den die Variable m_hauptAnzeige gesetzt werden soll
   */
  public void setHauptAnzeige( SystemObject hauptAnzeige )
  {
    m_hauptAnzeige = hauptAnzeige;
  }

  /**
   * Methode liefert die Variable m_abhaengigeAnzeige zurck
   * @return Variable m_abhaengigeAnzeige
   */
  public SystemObject getAbhaengigeAnzeige()
  {
    return m_abhaengigeAnzeige;
  }

  /**
   * Methode setzt die Variable m_abhaengigeAnzeige
   * @param abhaengigeAnzeige Wert auf den die Variable m_abhaengigeAnzeige gesetzt werden soll
   */
  public void setAbhaengigeAnzeige( SystemObject abhaengigeAnzeige )
  {
    m_abhaengigeAnzeige = abhaengigeAnzeige;
  }

  /**
   * Methode zum Auswerten der vom DaV empfangenen Attributgruppe 'atl.sswSbaAbhngigkeiten'
   * @param data vom DaV empfangene Attributgruppe
   */
  public void auswerteAttributGruppe( Data data )
  {
    m_hauptAnzeige      = data.getReferenceValue( "HauptAnzeige" ).getSystemObject();
    m_abhaengigeAnzeige = data.getReferenceValue( "AbhngigeAnzeige" ).getSystemObject();
  }

	/* (non-Javadoc)
   * @see java.lang.Object#toString()
   */
  @Override
  public String toString()
  {
  	String s = "";
  	
  	s += "Hauptanzeige: ";
  	if (getHauptAnzeige() != null)
  		s += getHauptAnzeige().getName();
  	else
  		s += "null";

  	s += " - abhngige Anzeige: ";
  	if (getAbhaengigeAnzeige() != null)
  		s += getAbhaengigeAnzeige().getName();
  	else
  		s += "null";

	  return s;
  }
  
  
}
