package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.aufhebungen.aufhebung;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausschaltregel.FuellRegel;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausschaltregel.RegelAusschaltRegel;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausschaltregel.RegelAusschaltRegelNeu;

/**
 * Klasse enthlt ein Feld (Liste) mit Parametern
 * @author       Dambach Werke GmbH
 */
public class ListeMitRegeln
{
  /**
   * Liste mit den Parametern
   */
  private Vector<RegelAufhebungNeu> m_parameter = new Vector<RegelAufhebungNeu> ();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public ListeMitRegeln ()
  {
  }
  
  /**
   * Methode liefert die Variable parameter zurck
   * @return Variable parameter
   */
  public Vector<RegelAufhebungNeu> getParameter()
  {
    Vector<RegelAufhebungNeu> v = new Vector<RegelAufhebungNeu>();
    synchronized (m_parameter)
    {
      for (RegelAufhebungNeu r : m_parameter)
        v.add( r );
    }
    return v;
  }

  /**
   * Methode setzt die Variable parameter
   * @param parameter Wert fr parameter
   */
  public void setParameter(Vector<RegelAufhebungNeu> parameter)
  {
    synchronized (m_parameter)
    {
      m_parameter = parameter;
    } 
  }

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaAufhebungProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    Array a = data.getArray( "Regeln" );
    int anzahl = a.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        RegelAufhebung r = new RegelAufhebung ();
        r.auswerteAttributgruppe( a.getItem( x ), schaltgrundAllgemein  );
        
        m_parameter.addAll ( umwandleParameter (r) );
      }
    }
  }

  /**
   * Methode wandelt einen Parametersatz in das neue Format um
   * @param regel Regel die umgewandelt werden soll
   * @return Vektor mit Regeln im neuen Format
   */
  public Vector<RegelAufhebungNeu> umwandleParameter ( RegelAufhebung regel )
  {
  	Vector<RegelAufhebungNeu> v = new Vector<RegelAufhebungNeu>();
  	
  	Vector<WVZInhalt> xyz = regel.getZuPruefendeInhalte();
  	WVZInhalt a = regel.getAngefordeterInhalt();
		Aspect schaltgrund = regel.getSchaltgrundRegel();
		long t1 = regel.getT1();
		long t2 = regel.getT2();
  	
  	for (WVZInhalt inhalt : xyz)
  	{
  		v.add(new RegelAufhebungNeu(schaltgrund, inhalt, a, t1, t2));
  	}
  	
  	return v;
  }


  /**
   * Methode fgt eine Regel hinzu
   * @param zuPruefendeInhalte Zu betrachtende Anzeigeinhalte
   * @param angefordeterInhalt Zu schaltende Aufhebung
   * @param t1 Zeitpunkt relativ zum Auslser ab dem die Aufhebung geschaltet werden soll
   * @param t2 Zeitpunkt relativ zum Auslser bis zu dem die Aufhebung geschaltet werden soll
   * @param schaltgrund Schaltgrund unter dem diese Regel publiziert wird
   */
  public void addRegel (WVZInhalt zuPruefenderInhalt,
                        WVZInhalt angefordeterInhalt,
                        long t1,
                        long t2,
                        Aspect schaltgrund) 
  {
    synchronized (m_parameter)
    {
      RegelAufhebungNeu r = new RegelAufhebungNeu ();
      
      r.setAnzeigenInhalt(zuPruefenderInhalt);
      r.setAngefordeterInhalt(angefordeterInhalt);
      r.setT1( t1 );
      r.setT2( t2 );
      r.setSchaltgrundRegel( schaltgrund );
      
      m_parameter.add( r );
    }
  }

  /**
   * Methode liefert den Inahlt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      s += "Regeln:\n";
      for (RegelAufhebungNeu r : m_parameter)
        s += r.toString() + "\n";
    }
    
    return s;
  }
}
