package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.aufhebungen.aufhebung;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt eine Regel fr die Regelklasse "Aufhebungen", Regel "Aufhebung".<br>  
 * @author Dambach Werke GmbH
 */
public class RegelAufhebung
{
  /**
   * Regelspezifischer Schaltgrund
   */
  private Aspect m_schaltgrundRegel;
  
  /**
   * Liste mit den zu prfenden Inhalten (in der Regel: ["X", "Y", "Z", ])
   */
  private Vector<WVZInhalt> m_zuPruefendeInhalte = new Vector<WVZInhalt>();
  
  /**
   * Angeforderter Inhalt (in der Regel: "A")
   */
  private WVZInhalt m_angefordeterInhalt = new WVZInhalt();
  
  /**
   * Zeitpunkt t1 (relativer Offset zu t0)
   */
  private long m_t1;
  
  /**
   * Zeitpunkt t2 (relativer Offset zu t0)
   */
  private long m_t2;
  
  /**
   * Parameterloser Konstruktor der Klasse
   */
  public RegelAufhebung ()
  {
  }

  /**
   * Methode zum Auswerten einer Regel der Attributliste 'atl.sswSbaRegelAufhebung'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    // Schaltgrundregel
    
    m_schaltgrundRegel = (Aspect) data.getReferenceValue( "SchaltgrundRegel" ).getSystemObject();
    
    if (m_schaltgrundRegel == null)
      m_schaltgrundRegel = schaltgrundAllgemein;
    
    // Anfordern ab

    m_t1 = 0;
    if (data.getUnscaledValue( "AnfordernAb" ).longValue() > 0)
      m_t1 = data.getUnscaledValue( "AnfordernAb" ).longValue();
    m_t1 *= 1000;  // In Millisekunden
    
    // Anfordern bis

    m_t2 = 0;
    if (data.getUnscaledValue( "AnfordernBis" ).longValue() > 0)
      m_t2 = data.getUnscaledValue( "AnfordernBis" ).longValue();
    m_t2 *= 1000; // In Millisekunden

    // Anzeigeninhalt
    
    Array a = data.getArray( "AnzeigeInhalt" );
    m_zuPruefendeInhalte.clear();
    int anzahl = a.getLength();
    
    for (int x = 0; x < anzahl; ++x)
    {
      Data d1 = a.getItem( x );
      WVZInhalt inhalt = new WVZInhalt();
      inhalt.auswerteAttributgruppe( d1 );
      
      m_zuPruefendeInhalte.add( inhalt );
    }
    
    // Aufhebung
    
    Data anzeigenInhalt = data.getItem( "Aufhebung" );
    m_angefordeterInhalt.auswerteAttributgruppe( anzeigenInhalt );
  }

  /**
   * Methode liefert die Variable m_schaltgrundRegel zurck
   * @return Variable m_schaltgrundRegel
   */
  public Aspect getSchaltgrundRegel()
  {
    return m_schaltgrundRegel;
  }

  /**
   * Methode setzt die Variable m_schaltgrundRegel
   * @param schaltgrundRegel Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
   */
  public void setSchaltgrundRegel( Aspect schaltgrundRegel ) 
  {
    m_schaltgrundRegel = schaltgrundRegel;
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";

    s += "Zu prfende Bildinhalte:\n";
    for (WVZInhalt wi : m_zuPruefendeInhalte)
      System.out.println("- " + Tools.getWVZInhalt( wi ) + "\n");

    s += "A = " + Tools.getWVZInhalt( m_angefordeterInhalt ) + "\n";
    
    s += "Von: " + getT1() + " Sekunden \n";
    s += "Bis: " + getT2() + " Sekunden \n";
    return s;
  }

  /**
   * Methode liefert die Variable m_zuPruefendeInhalte zurck
   * @return Variable m_zuPruefendeInhalte
   */
  public Vector<WVZInhalt> getZuPruefendeInhalte()
  {
    return m_zuPruefendeInhalte;
  }

  /**
   * Methode setzt die Variable m_zuPruefendeInhalte
   * @param zuPruefendeInhalte Wert auf den die Variable m_zuPruefendeInhalte gesetzt werden soll
   */
  public void setZuPruefendeInhalte( Vector<WVZInhalt> zuPruefendeInhalte )
  {
    m_zuPruefendeInhalte = zuPruefendeInhalte;
  }

  /**
   * Methode liefert die Variable m_angefordeterInhalt zurck
   * @return Variable m_angefordeterInhalt
   */
  public WVZInhalt getAngefordeterInhalt()
  {
    return m_angefordeterInhalt;
  }

  /**
   * Methode setzt die Variable m_angefordeterInhalt
   * @param angefordeterInhalt Wert auf den die Variable m_angefordeterInhalt gesetzt werden soll
   */
  public void setAngefordeterInhalt( WVZInhalt angefordeterInhalt )
  {
    m_angefordeterInhalt = angefordeterInhalt;
  }

  /**
   * Methode liefert die Variable m_t1 zurck
   * @return Variable m_t1
   */
  public long getT1()
  {
    return m_t1;
  }

  /**
   * Methode setzt die Variable m_t1
   * @param t1 Wert auf den die Variable m_t1 gesetzt werden soll
   */
  public void setT1( long t1 )
  {
    m_t1 = t1;
  }

  /**
   * Methode liefert die Variable m_t2 zurck
   * @return Variable m_t2
   */
  public long getT2()
  {
    return m_t2;
  }

  /**
   * Methode setzt die Variable m_t2
   * @param t2 Wert auf den die Variable m_t2 gesetzt werden soll
   */
  public void setT2( long t2 )
  {
    m_t2 = t2;
  }

  /**
   * Methode fgt einen zu prfenden Inhalt hinzu
   * @param inhalt zu prfender Inhalt
   */
  public void addZuPruefenderAnzeigenInhalt (WVZInhalt inhalt)
  {
  	m_zuPruefendeInhalte.add(inhalt);
  }
}
