package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallandereszeichen;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;

/**
 * Klasse enthlt ein Feld (Liste) mit Parametern
 * @author       Swarco Traffic Systems GmbH
 */
public class ListeMitRegeln
{
  /**
   * Liste mit den Parametern
   */
  private Vector<RegelAusfallAnderesZeichen> m_parameter = new Vector<RegelAusfallAnderesZeichen> ();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public ListeMitRegeln ()
  {
  }
  
  /**
   * Methode liefert die Variable parameter zurck
   * @return Variable parameter
   */
  public Vector<RegelAusfallAnderesZeichen> getParameter()
  {
    Vector<RegelAusfallAnderesZeichen> v = new Vector<RegelAusfallAnderesZeichen>();
    synchronized (m_parameter)
    {
      for (RegelAusfallAnderesZeichen r : m_parameter)
        v.add( r );
    }
    return v;
  }

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaAusfallGleichesZeichenProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    Array a = data.getArray( "Regeln" );
    int anzahl = a.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        RegelAusfallAnderesZeichen r = new RegelAusfallAnderesZeichen ();
        r.auswerteAttributgruppe( a.getItem( x ), schaltgrundAllgemein );
        
        m_parameter.add( r );
      }
    }
  }

  /**
   * Methode fgt eine Regel hinzu
   * @param richtung Richtung in die die Regel arbeitet
   * @param anzeigeInhalt zu prfender Anzeigeinhalt ("X")
   * @param angeforderterAnzeigeInhalt angeforderter Anzeigeinhalt ("aus")
   * @param auswahlAnzeige Welche Anzeige soll geschaltet werden
   * @param pruefeAndereZeichen Sollen die anderen Zeichen geprft werden
   * @param anzeigeTypZusaetzlich zustzlicher Anzeigetyp ("Typ A")
   * @param auswahlAnzeigeQuerschnitt Welche Anzeigequerschnitte sollen geschaltet werden
   * @param angeforderterAnzeigenInhaltZusaetzlich angeforderter Anzeigeninhalte ("X", "Y", "Z");
   * @param anzeigeTypVerschiebung Anzeigetyp Verschiebung ("Typ B")
   * @param angeforderterAnzeigenInhaltVerschiebung angeforderter Anzeigeninhalte ("X", "Y", "Z");
   * @param ebene Ebene bis zu der verschoben wird
   * @param schaltgrund Schaltgrund unter dem die Regel arbeitet
   */
  public void addRegel (ArbeitsRichtungVN richtung,
                        WVZInhalt anzeigeInhalt, 
                        WVZInhalt angeforderterAnzeigeInhalt,
                        SelektionAnzeige auswahlAnzeige,
                        boolean pruefeAndereZeichen,
                        SystemObject anzeigeTypZusaetzlich,
                        SelektionAnzeigeQuerschnitt auswahlAnzeigeQuerschnitt,
                        Vector<WVZInhalt> angeforderterAnzeigenInhaltZusaetzlich,
                        SystemObject anzeigeTypVerschiebung,
                        Vector<WVZInhalt> angeforderterAnzeigenInhaltVerschiebung,
                        int ebene,
                        Aspect schaltgrund )
  {
    synchronized (m_parameter)
    {
      RegelAusfallAnderesZeichen r = new RegelAusfallAnderesZeichen ();
      
      r.setRichtung( richtung );
      r.setSchaltgrundRegel( schaltgrund );
      r.setAnzeigenInhalt( anzeigeInhalt );
      r.setAngeforderterAnzeigenInhalt( angeforderterAnzeigeInhalt );
      r.setAuswahlAnzeige( auswahlAnzeige );
      r.setAnzeigeTypZusaetzlich( anzeigeTypZusaetzlich );
      r.setAuswahlAnzeigeQuerschnitt( auswahlAnzeigeQuerschnitt );
      r.setAngeforderterAnzeigenInhaltZusaetzlich( angeforderterAnzeigenInhaltZusaetzlich );
      r.setAnzeigeTypVerschiebung( anzeigeTypVerschiebung );
      r.setEbene( ebene );
      r.setAngeforderterAnzeigenInhaltVerschiebung( angeforderterAnzeigenInhaltVerschiebung );
      
      r.setPruefeAndereZeichen( pruefeAndereZeichen );
      
      m_parameter.add( r );
    }
  }

  /**
   * Methode liefert den Inahlt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      s += "Regeln:\n";
      for (RegelAusfallAnderesZeichen r : m_parameter)
        s += r.toString() + "\n";
    }
    
    return s;
  }

  /**
   * Methode zum bernehmen der Parameter
   * @param parameter zu bernehmende Parameter
   */
  public void setParameter(Vector<RegelAusfallAnderesZeichen> parameter)
  {
    synchronized (m_parameter)
    {
      m_parameter.clear();
      
      for (RegelAusfallAnderesZeichen r : parameter)
         m_parameter.add(r);
    } 
  }

  /**
   * Methode zum Hinzufgen einer Regel
   * @param regel Regel die hinzugefgt werden soll
   */
	public void addRegel(RegelAusfallAnderesZeichen regel)
  {
		synchronized (m_parameter)
    {
	    m_parameter.add(regel); 
    }
  }
}
