package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausfallgleicheszeichen;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt eine Regel fr die Regelklasse "Ausfallbehandlung", 
 * Regel "Ausfall Gleiches Zeichen".<br> 
 * @author Dambach-Werke GmbH
 * @author Matthias Obert
 */
public class RegelAusfallGleichesZeichen
{
  /**
   * Regelspezifischer Schaltgrund
   */
  private Aspect m_schaltgrundRegel;
  
  /**
   * Anzeigeninhalt (in der Regel: "X")
   */
  private WVZInhalt  m_anzeigenInhalt = new WVZInhalt();
  
  /**
   * angeforderter Anzeigeninhalt (in der Regel: "Y")
   */
  private WVZInhalt m_angeforderterAnzeigenInhalt = new WVZInhalt();

  /**
   * Sollen die anderen Anzeigen an diesem AQ berprft werden ?
   */
  private boolean m_pruefeAndereZeichen;
  
  /**
   * Parameterloser Konstruktor der Klasse
   */
  public RegelAusfallGleichesZeichen ()
  {
  }
  
  public RegelAusfallGleichesZeichen (Aspect schaltgrund, WVZInhalt x, WVZInhalt y,  boolean pruefeAndereZeichen)
  {
  	m_schaltgrundRegel = schaltgrund;
  	
  	m_anzeigenInhalt = x;
  	
  	m_angeforderterAnzeigenInhalt = y;
  	
  	m_pruefeAndereZeichen = pruefeAndereZeichen;
  }

	/**
   * Methode liefert die Variable m_anzeigenInhalt zurck
   * @return Variable m_anzeigenInhalt
   */
  public WVZInhalt getAnzeigenInhalt()
  {
    return m_anzeigenInhalt;
  }

  /**
   * Methode setzt die Variable m_anzeigenInhalt
   * @param anzeigenInhalt Wert auf den die Variable m_anzeigenInhalt gesetzt werden soll
   */
  public void setAnzeigenInhalt( WVZInhalt anzeigenInhalt )
  {
    m_anzeigenInhalt = anzeigenInhalt;
  }

  /**
   * Methode liefert die Variable m_angeforderterAnzeigenInhalt zurck
   * @return Variable m_angeforderterAnzeigenInhalt
   */
  public WVZInhalt getAngeforderterAnzeigenInhalt()
  {
    return m_angeforderterAnzeigenInhalt;
  }

  /**
   * Methode setzt die Variable m_angeforderterAnzeigenInhalt
   * @param angeforderterAnzeigenInhalt Wert auf den die Variable m_angeforderterAnzeigenInhalt gesetzt werden soll
   */
  public void setAngeforderterAnzeigenInhalt( WVZInhalt angeforderterAnzeigenInhalt )
  {
    m_angeforderterAnzeigenInhalt = angeforderterAnzeigenInhalt;
  }
  
  /**
   * Methode zum Auswerten einer Regel der Attributliste 'atl.sswSbaRegelAusfallGleichesZeichen'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    m_schaltgrundRegel = (Aspect) data.getReferenceValue( "SchaltgrundRegel" ).getSystemObject();
    
    String jaNein = data.getItem( "PruefeAndereZeichen" ).asTextValue().getText();
    m_pruefeAndereZeichen = jaNein.equalsIgnoreCase( "Ja" );
    
    if (m_schaltgrundRegel == null)
      m_schaltgrundRegel = schaltgrundAllgemein;
    
    Data anzeigenInhalt = data.getItem( "Anzeigeinhalt" );
    m_anzeigenInhalt.auswerteAttributgruppe( anzeigenInhalt );
    
    Data angeforderterAnzeigenInhalt = data.getItem( "AngeforderterAnzeigeInhalt" );
    m_angeforderterAnzeigenInhalt.auswerteAttributgruppe( angeforderterAnzeigenInhalt );
  }

  /**
   * Methode liefert die Variable m_schaltgrundRegel zurck
   * @return Variable m_schaltgrundRegel
   */
  public Aspect getSchaltgrundRegel()
  {
    return m_schaltgrundRegel;
  }

  /**
   * Methode setzt die Variable m_schaltgrundRegel
   * @param schaltgrundRegel Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
   */
  public void setSchaltgrundRegel( Aspect schaltgrundRegel ) 
  {
    m_schaltgrundRegel = schaltgrundRegel;
  }
  
  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";

    s += "X = " + Tools.getWVZInhalt( m_anzeigenInhalt ) + "\n";
    s += "Y = " + Tools.getWVZInhalt( m_angeforderterAnzeigenInhalt ) + "\n";
    s += "Andere Zeichen prfen: " + isPruefeAndereZeichen() + "\n";
    
    return s;
  }

  /**
   * Methode liefert die Variable m_pruefeAndereZeichen zurck
   * @return Variable m_pruefeAndereZeichen
   */
  public boolean isPruefeAndereZeichen()
  {
    return m_pruefeAndereZeichen;
  }

  /**
   * Methode setzt die Variable m_pruefeAndereZeichen
   * @param pruefeAndereZeichen Wert auf den die Variable m_pruefeAndereZeichen gesetzt werden soll
   */
  public void setPruefeAndereZeichen( boolean pruefeAndereZeichen )
  {
    m_pruefeAndereZeichen = pruefeAndereZeichen;
  }
}
