package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.ausfallbehandlung.ausschaltregel;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt eine Regel fr die Regelklasse "Ausschaltregel", 
 * Regel "Ausschaltregel".<br> 
 * @author Dambach-Werke GmbH
 * @author Matthias Obert
 */
public class RegelAusschaltRegel
{
  /**
   * Regelspezifischer Schaltgrund
   */
  private Aspect m_schaltgrundRegel;
  
  /**
   * Anzeigeninhalt (in der Regel: "X")
   */
  private WVZInhalt  m_anzeigenInhalt = new WVZInhalt();

  /**
   * Regeln fr das Fllen der Vorgnger und Nachfolger in Querrichtung.
   */
  private Vector<FuellRegel> m_fuellRegeln = new Vector<FuellRegel>();

  /**
   * Methode liefert die Variable m_schaltgrundRegel zurck
   * @return Variable m_schaltgrundRegel
   */
  public Aspect getSchaltgrundRegel()
  {
    return m_schaltgrundRegel;
  }

  /**
   * Methode setzt die Variable m_schaltgrundRegel
   * @param schaltgrundRegel Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
   */
  public void setSchaltgrundRegel( Aspect schaltgrundRegel )
  {
    m_schaltgrundRegel = schaltgrundRegel;
  }

  /**
   * Methode liefert die Variable m_anzeigenInhalt zurck
   * @return Variable m_anzeigenInhalt
   */
  public WVZInhalt getAnzeigenInhalt()
  {
    return m_anzeigenInhalt;
  }

  /**
   * Methode setzt die Variable m_anzeigenInhalt
   * @param anzeigenInhalt Wert auf den die Variable m_anzeigenInhalt gesetzt werden soll
   */
  public void setAnzeigenInhalt( WVZInhalt anzeigenInhalt )
  {
    m_anzeigenInhalt = anzeigenInhalt;
  }

  /**
   * Methode liefert die Variable m_fuellRegeln zurck
   * @return Variable m_fuellRegeln
   */
  public Vector<FuellRegel> getFuellRegeln()
  {
    return m_fuellRegeln;
  }

  /**
   * Methode setzt die Variable m_fuellRegeln
   * @param fuellRegeln Wert auf den die Variable m_fuellRegeln gesetzt werden soll
   */
  public void setFuellRegeln( Vector<FuellRegel> fuellRegeln )
  {
    m_fuellRegeln = fuellRegeln;
  }

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public RegelAusschaltRegel ()
  {
  }
  
  /**
   * Methode zum Auswerten einer Regel der Attributliste 'atl.sswSbaRegelAusschaltRegel'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    // Try/Catch notwendig bis Item in DaV aufgenommen ist
    
    try
    {
      m_schaltgrundRegel = (Aspect) data.getReferenceValue( "SchaltgrundRegel" ).getSystemObject();
    }
    catch (Exception e) 
    {
    }
    
    if (m_schaltgrundRegel == null)
      m_schaltgrundRegel = schaltgrundAllgemein;
    
    Data anzeigenInhalt = data.getItem( "Anzeigeinhalt" );
    m_anzeigenInhalt.auswerteAttributgruppe( anzeigenInhalt );
    
    Array a = data.getArray( "FllRegeln" );
    int anzahl = a.getLength();
    
    m_fuellRegeln.clear();
    for (int x = 0; x < anzahl; ++x)
    {
      Data d = a.getItem( x );
      FuellRegel fr = new FuellRegel ();
      fr.auswerteAttributgruppe (d);
      
      m_fuellRegeln.add( fr );
    }
  }

  /**
   * Methode liefert den Inhalt der Klasse als String
   */
  public String toString ()
  {
    String s = "";

    s += "X = " + Tools.getWVZInhalt( m_anzeigenInhalt ) + "\n";
    s += "Regeln:\n";
    for (FuellRegel fr : m_fuellRegeln)
      s += fr.toString () + "\n";
    
    return s;
  }

  /**
   * Methode fgt ein Fllregel hinzu
   * @param anzeigeTyp Anzeigetyp
   * @param angeforderterInhalt Angeforderter Inhalt
   * @param prioritaet Prioritt
   */
	public void addregel(SystemObject anzeigeTyp, WVZInhalt angeforderterInhalt, int prioritaet)
  {
		m_fuellRegeln.add(new FuellRegel(anzeigeTyp, angeforderterInhalt, prioritaet));
  }
}
