package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.monotonieKreuzPfeil;

import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;

/**
 * Klasse enthlt ein Feld (Liste) mit Parametern
 * @author       Swarco Traffic Systems GmbH
 */
public class ListeMitRegeln
{
  /**
   * Liste mit den Parametern
   */
  private Vector<RegelMonotonieKreuzPfeil> m_parameter = new Vector<RegelMonotonieKreuzPfeil> ();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public ListeMitRegeln ()
  {
  }
  
  public void addRegel( RegelMonotonieKreuzPfeil r )
  {
    synchronized (m_parameter)
    {
      m_parameter.add( r ); 
    }
  }

  /**
   * Methode liefert die Variable parameter zurck
   * @return Variable parameter
   */
  public Vector<RegelMonotonieKreuzPfeil> getParameter()
  {
    Vector<RegelMonotonieKreuzPfeil> v = new Vector<RegelMonotonieKreuzPfeil>();
    synchronized (m_parameter)
    {
      for (RegelMonotonieKreuzPfeil r : m_parameter)
        v.add( r );
    }
    return v;
  }

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaMonotonieKreuzPfeilProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    Array a = data.getArray( "Regeln" );
    int anzahl = a.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        RegelMonotonieKreuzPfeil r = new RegelMonotonieKreuzPfeil ();
        r.auswerteAttributgruppe( a.getItem( x), schaltgrundAllgemein  );
        
        m_parameter.add( r );
      }
    }
  }
  
  /**
   * Methode fgt eine Regel hinzu
   * @param x Anzeigeinhalt der aktuell betrachteten Anzeige 
   * @param y Anzeigeinhalt des Vorgnger/Nachfolgers
   * @param z Angeforderter Anzeigeinhalt
   * @param richtungLaengs Lngsrichtung in der die Regel arbeitet
   * @param richtungQuer Querrichtung in der die Regel arbeitet
   * @param schaltgrund Schaltgrund unter dem diese Regel publiziert wird
   */
  public void addRegel (List<WVZInhalt> x, List<WVZInhalt> y, List<WVZInhalt> z, ArbeitsRichtungVN richtungLaengs, ArbeitsRichtungVN richtungQuer, Aspect schaltgrund )
  {
    synchronized (m_parameter)
    {
      RegelMonotonieKreuzPfeil r = new RegelMonotonieKreuzPfeil ();
      
      r.setSchaltgrundRegel( schaltgrund );
      r.setAnzeigenInhalt( x );
      r.setAnzeigenInhaltVorgaengerNachfolgerLaengs( y );
      r.setAnzeigenInhaltVorgaengerNachfolgerQuer( z );
      r.setRichtungLaengs( richtungLaengs );
      r.setRichtungQuer( richtungQuer );
      
      m_parameter.add( r );
    }
  }

  /**
   * Methode liefert den Inahlt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      s += "Regeln:\n";
      for (RegelMonotonieKreuzPfeil r : m_parameter)
        s += r.toString() + "\n";
    }
    
    return s;
  }

  /**
   * Methode setzt die Variable parameter
   * @param parameter Wert fr parameter
   */
	public void setParameter(Vector<RegelMonotonieKreuzPfeil> parameter)
  {
    synchronized (m_parameter)
    {
      m_parameter.clear();
      
      for (RegelMonotonieKreuzPfeil r : parameter)
         m_parameter.add(r);
    } 
  }
}
