package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.monotonieKreuzPfeil;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse enthlt eine Regel fr die Regelklasse
 * "Lngsabgleichs- und Querabgleichsregeln", Regel "Monotonie Kreuz/Pfeil"
 * 
 * @author Swarco Traffic Systems GmbH
 */
public class RegelMonotonieKreuzPfeil
{
	/**
	 * Regelspezifischer Schaltgrund
	 */
	private Aspect m_schaltgrundRegel;

	/**
	 * arbeitet Regel in Lngsrichtung in Vorgnger- oder Nachfolgerrichtung
	 */
	private ArbeitsRichtungVN m_richtungLaengs = ArbeitsRichtungVN.NACHFOLGER;

	/**
	 * arbeitet Regel in Querrichtung in Vorgnger- oder Nachfolgerrichtung
	 */
	private ArbeitsRichtungVN m_richtungQuer = ArbeitsRichtungVN.NACHFOLGER;

	/**
	 * Anzeigeninhalt (in der Regel: "X")
	 */
	private List<WVZInhalt> m_anzeigenInhalt = new ArrayList<WVZInhalt>();

	/**
	 * Anzeigeninhalt Vorgnger/Nachfolger (in der Regel: "Y")
	 */
	private List<WVZInhalt> m_anzeigenInhaltVorgaengerNachfolgerLaengs = new ArrayList<WVZInhalt>();

	/**
	 * Anzeigeninhalt Vorgnger/Nachfolger (in der Regel: "Z")
	 */
	private List<WVZInhalt> m_anzeigenInhaltVorgaengerNachfolgerQuer = new ArrayList<WVZInhalt>();

	/**
	 * Parameterloser Konstruktor der Klasse
	 */
	public RegelMonotonieKreuzPfeil()
	{
	}

	/**
	 * Konstruktor der Klasse
	 * @param schaltgrund Regelspezifischer Schaltgrund
	 * @param richtungLaengs arbeitet Regel in Lngsrichtung in Vorgnger- oder Nachfolgerrichtung
	 * @param richtungQuer arbeitet Regel in Querrichtung in Vorgnger- oder Nachfolgerrichtung
	 * @param anzeigenInhalt Anzeigeninhalt (in der Regel: "X")
	 * @param anzeigenInhaltVorgaengerNachfolgerLaengs Anzeigeninhalt Vorgnger/Nachfolger (in der Regel: "Y")
	 * @param anzeigenInhaltVorgaengerNachfolgerQuer Anzeigeninhalt Vorgnger/Nachfolger (in der Regel: "Z")
	 */
	public RegelMonotonieKreuzPfeil(Aspect schaltgrund,
	    ArbeitsRichtungVN richtungLaengs, ArbeitsRichtungVN richtungQuer,
	    List<WVZInhalt> anzeigenInhalt,
	    List<WVZInhalt> anzeigenInhaltVorgaengerNachfolgerLaengs,
	    List<WVZInhalt> anzeigenInhaltVorgaengerNachfolgerQuer)
	{
		m_schaltgrundRegel = schaltgrund;
		m_richtungLaengs = richtungLaengs;
		m_richtungQuer = richtungQuer;
		m_anzeigenInhalt = anzeigenInhalt;
		m_anzeigenInhaltVorgaengerNachfolgerLaengs = anzeigenInhaltVorgaengerNachfolgerLaengs;
		m_anzeigenInhaltVorgaengerNachfolgerQuer = anzeigenInhaltVorgaengerNachfolgerQuer;
	}

	/**
	 * Methode prft ob die Regel bei der Lngsrichtung in Vorgngerrichtung
	 * arbeitet
	 * 
	 * @return Regel arbeitet in Vorgngerrichtung
	 */
	public boolean isRichtungVorgaengerLaengs()
	{
		return m_richtungLaengs == ArbeitsRichtungVN.VORGAENGER;
	}

	/**
	 * Methode prft ob die Regel bei der Lngsrichtung in Vorgngerrichtung
	 * arbeitet
	 * 
	 * @return Regel arbeitet in Vorgngerrichtung
	 */
	public boolean isRichtungVorgaengerQuer()
	{
		return m_richtungQuer == ArbeitsRichtungVN.VORGAENGER;
	}

	/**
	 * Methode zum Auswerten einer Regel der Attributliste
	 * 'atl.sswSbaRegelMonotonieKreuzPfeil'
	 * 
	 * @param data
	 *          Daten vom DaV
	 */
	public void auswerteAttributgruppe(Data data, Aspect schaltgrundAllgemein)
	{
		m_schaltgrundRegel = (Aspect) data.getReferenceValue("SchaltgrundRegel")
		    .getSystemObject();

		if (m_schaltgrundRegel == null)
			m_schaltgrundRegel = schaltgrundAllgemein;

		String vorgaengerNachfolgerLaengs = data
		    .getItem("VorgngerNachfolgerLngs").asTextValue().getText();
		if (vorgaengerNachfolgerLaengs.equals("Vorgnger"))
			setRichtungLaengs(ArbeitsRichtungVN.VORGAENGER);
		else
			setRichtungLaengs(ArbeitsRichtungVN.NACHFOLGER);

		String vorgaengerNachfolgerQuer = data.getItem("VorgngerNachfolgerQuer")
		    .asTextValue().getText();
		if (vorgaengerNachfolgerQuer.equals("Vorgnger"))
			setRichtungQuer(ArbeitsRichtungVN.VORGAENGER);
		else
			setRichtungQuer(ArbeitsRichtungVN.NACHFOLGER);

		m_anzeigenInhalt.clear();
		Array a1 = data.getArray("AnzeigeinhaltAktuell");
		for (int i = 0; i < a1.getLength(); ++i)
		{
			WVZInhalt wi = new WVZInhalt();
			wi.auswerteAttributgruppe(a1.getItem(i));
			m_anzeigenInhalt.add(wi);
		}

		m_anzeigenInhaltVorgaengerNachfolgerLaengs.clear();
		Array a2 = data.getArray("AnzeigeinhaltLngs");
		for (int i = 0; i < a2.getLength(); ++i)
		{
			WVZInhalt wi = new WVZInhalt();
			wi.auswerteAttributgruppe(a2.getItem(i));
			m_anzeigenInhaltVorgaengerNachfolgerLaengs.add(wi);
		}

		m_anzeigenInhaltVorgaengerNachfolgerQuer.clear();
		Array a3 = data.getArray("AnzeigeinhaltQuer");
		for (int i = 0; i < a3.getLength(); ++i)
		{
			WVZInhalt wi = new WVZInhalt();
			wi.auswerteAttributgruppe(a3.getItem(i));
			m_anzeigenInhaltVorgaengerNachfolgerQuer.add(wi);
		}
	}

	/**
	 * Methode liefert die Variable m_schaltgrundRegel zurck
	 * 
	 * @return Variable m_schaltgrundRegel
	 */
	public Aspect getSchaltgrundRegel()
	{
		return m_schaltgrundRegel;
	}

	/**
	 * Methode setzt die Variable m_schaltgrundRegel
	 * 
	 * @param schaltgrundRegel
	 *          Wert auf den die Variable m_schaltgrundRegel gesetzt werden soll
	 */
	public void setSchaltgrundRegel(Aspect schaltgrundRegel)
	{
		m_schaltgrundRegel = schaltgrundRegel;
	}

	/**
	 * Methode liefert den Inhalt der Klasse als String
	 */
	public String toString()
	{
		String s = "";

		String s1 = "";
		if (getSchaltgrundRegel() != null)
			s1 = getSchaltgrundRegel().toString();

		s += "Schaltgrund:   " + s1 + "\n";
		s += "Lngsrichtung: " + m_richtungLaengs + "\n";
		s += "Querrichtung:  " + m_richtungQuer + "\n";
		s += "X = \n";
		for (WVZInhalt wi : m_anzeigenInhalt)
			s += "-" + Tools.getWVZInhalt(wi) + "\n";
		s += "\n";

		s += "Y = \n";
		for (WVZInhalt wi : m_anzeigenInhaltVorgaengerNachfolgerLaengs)
			s += "-" + Tools.getWVZInhalt(wi) + "\n";
		s += "\n";

		s += "Z = \n";
		for (WVZInhalt wi : m_anzeigenInhaltVorgaengerNachfolgerQuer)
			s += "-" + Tools.getWVZInhalt(wi) + "\n";
		s += "\n";

		return s;
	}

	/**
	 * Methode liefert die Variable m_richtungLaengs zurck
	 * 
	 * @return Variable m_richtungLaengs
	 */
	public ArbeitsRichtungVN getRichtungLaengs()
	{
		return m_richtungLaengs;
	}

	/**
	 * Methode setzt die Variable m_richtungLaengs
	 * 
	 * @param richtungLaengs
	 *          Wert auf den die Variable m_richtungLaengs gesetzt werden soll
	 */
	public void setRichtungLaengs(ArbeitsRichtungVN richtungLaengs)
	{
		m_richtungLaengs = richtungLaengs;
	}

	/**
	 * Methode liefert die Variable m_richtungQuer zurck
	 * 
	 * @return Variable m_richtungQuer
	 */
	public ArbeitsRichtungVN getRichtungQuer()
	{
		return m_richtungQuer;
	}

	/**
	 * Methode setzt die Variable m_richtungQuer
	 * 
	 * @param richtungQuer
	 *          Wert auf den die Variable m_richtungQuer gesetzt werden soll
	 */
	public void setRichtungQuer(ArbeitsRichtungVN richtungQuer)
	{
		m_richtungQuer = richtungQuer;
	}

	/**
	 * Methode liefert die Variable m_anzeigenInhalt zurck
	 * 
	 * @return Variable m_anzeigenInhalt
	 */
	public List<WVZInhalt> getAnzeigenInhalt()
	{
		return m_anzeigenInhalt;
	}

	/**
	 * Methode setzt die Variable m_anzeigenInhalt
	 * 
	 * @param anzeigenInhalt
	 *          Wert auf den die Variable m_anzeigenInhalt gesetzt werden soll
	 */
	public void setAnzeigenInhalt(List<WVZInhalt> anzeigenInhalt)
	{
		m_anzeigenInhalt = anzeigenInhalt;
	}

	/**
	 * Methode liefert die Variable m_anzeigenInhaltVorgaengerNachfolgerLaengs
	 * zurck
	 * 
	 * @return Variable m_anzeigenInhaltVorgaengerNachfolgerLaengs
	 */
	public List<WVZInhalt> getAnzeigenInhaltVorgaengerNachfolgerLaengs()
	{
		return m_anzeigenInhaltVorgaengerNachfolgerLaengs;
	}

	/**
	 * Methode setzt die Variable m_anzeigenInhaltVorgaengerNachfolgerLaengs
	 * 
	 * @param anzeigenInhaltVorgaengerNachfolgerLaengs
	 *          Wert auf den die Variable
	 *          m_anzeigenInhaltVorgaengerNachfolgerLaengs gesetzt werden soll
	 */
	public void setAnzeigenInhaltVorgaengerNachfolgerLaengs(
	    List<WVZInhalt> anzeigenInhaltVorgaengerNachfolgerLaengs)
	{
		m_anzeigenInhaltVorgaengerNachfolgerLaengs = anzeigenInhaltVorgaengerNachfolgerLaengs;
	}

	/**
	 * Methode liefert die Variable m_anzeigenInhaltVorgaengerNachfolgerQuer
	 * zurck
	 * 
	 * @return Variable m_anzeigenInhaltVorgaengerNachfolgerQuer
	 */
	public List<WVZInhalt> getAnzeigenInhaltVorgaengerNachfolgerQuer()
	{
		return m_anzeigenInhaltVorgaengerNachfolgerQuer;
	}

	/**
	 * Methode setzt die Variable m_anzeigenInhaltVorgaengerNachfolgerQuer
	 * 
	 * @param anzeigenInhaltVorgaengerNachfolgerQuer
	 *          Wert auf den die Variable m_anzeigenInhaltVorgaengerNachfolgerQuer
	 *          gesetzt werden soll
	 */
	public void setAnzeigenInhaltVorgaengerNachfolgerQuer(
	    List<WVZInhalt> anzeigenInhaltVorgaengerNachfolgerQuer)
	{
		m_anzeigenInhaltVorgaengerNachfolgerQuer = anzeigenInhaltVorgaengerNachfolgerQuer;
	}
}
