package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;
import java.util.Vector;

import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften.UniversalEigenschaften;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Fenster zum Versorgen der Algorithmus abhngigen Parameter
 * @author ObertM
 */
public class CompositeParameterEigenschaft11 extends Composite
{
  /**
   * ParameterElemente des DAV, die als Spinner angezeigt werden, die Namen mssen identisch sein
   * mit den Namen der Items der Attributgruppe im DaV 
   */
  private static final String[] _parItemSpin = 
  {
  };
  
  /**
   * ParameterElemente des DAV, die als ComboBox angezeigt werden, die Namen mssen identisch sein
   * mit den Namen der Items der Attributgruppe im DaV 
   */
  private static final String[] _parItemCombo = 
  {
  	"Kriterium",
  };

  /**
   * Gesmatcomposite
   */
  private Composite _composite;

  /**
   * Composite (Gruppe) fr die Parameter
   */
  private Group _groupParameter;

  /**
   * Composite mit der Tabelle
   */
  private Composite _compositeTabelle = null;

  /**
   * Compoiste fr die Tabelle
   */
  private Composite _composite3 = null;

  /**
   * Table Viewer
   */
  private TableViewer _tvParameter;;

  /**
   * Spinner
   */
  private Spinner[] _spinnerPar = new Spinner [100];

  /**
   * ComboBoxen
   */
  private Combo[] _comboTyp = new Combo [100];

  /**
   * ComboBoxen
   */
  private Combo[] _comboVergleich = new Combo [100];

  /**
   * Universal Regel
   */
	private RegelUniversal _aktuell;

	/**
	 * Tabellenmodell
	 */
	private TableModel[] _tableModel = new TableModel [0];

	private List<SystemObject> _listMoeglicheWZGTypen;

  /**
   * Konstruktor der Klasse
   * @param parent Aufrufendes Composite
   * @param style Stil
   * @param result Ergebnis vom Dav
   * @param data Datenfeld innerhalb des Ergebnisses welches ausgewertet werden soll
   */
  CompositeParameterEigenschaft11(Composite parent, int style, RegelUniversal regel)
  {
    super(parent, style);
    
    _aktuell = regel;
    
    _listMoeglicheWZGTypen = Tools.getSystemObjekte("typ.sswSbaAnzeigeTyp");

    initialize();
  }
  
  /**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.heightHint = -1;
    gridData1.widthHint = -1;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;
    setLayout(new GridLayout(1, false));
    setLayoutData(gridData1);
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.heightHint = -1;
    gridData2.widthHint = -1;
    gridData2.horizontalIndent = 0;
    gridData2.verticalAlignment = GridData.FILL;

    _composite = new Composite(this, SWT.NONE);
    _composite.setLayoutData(gridData2);
    _composite.setLayout(gridLayout);

    erzeugeKommentar();
    	
    createGroupParameter();

    updateData();
  }

  /**
   * Methode blendet einen Kommentar zu dem Parametersatz ein
   */
  private void erzeugeKommentar ()
  {
  	String kommentar = "Universalregel Eigenschaft 11:\n\n" + 
  	                   "Anzahl Fahrstreifen AQ [Erste Anzeige der Reihe] ist \"n\" = ja/nein/beliebig\n\n";
  	
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    
    Group groupKommentar = new Group (_composite, SWT.NONE);
    groupKommentar.setLayout(new FillLayout());
    groupKommentar.setLayoutData(gridData1);

//    groupKommentar.setText("Erklrung");
    
    Label text = new Label (groupKommentar, SWT.NONE);
    text.setText(kommentar);
  }

  /**
   * Updatefunktion fr Aufruf von aussen
   * @param result Ergebnis vom Dav
   * @param data Datenfeld innerhalb des Ergebnisses welches ausgewertet werden soll
   */
  public void update (RegelUniversal regel)
  {
    _aktuell = regel;
    
    updateData();
    
  }

  private int bestimmeAnzahlEingabeFelder ()
  {
    return _aktuell.getEigenschaften().size();	
  }
  
  /**
   * Updatefunktion
   */
  private void updateData ()
  {
    Table table = _tvParameter.getTable();
    TableModel[] input = createModel(table);
    _tvParameter.setInput(input);

  	Vector<UniversalEigenschaften> eigenschaften = _aktuell.getEigenschaften();
  	
	  for (int i=0; i<eigenschaften.size(); ++i)
	  {
	  	JaNeinBeliebig jnb = eigenschaften.get(i).getEigenschaftAnzahlFahrstreifen().getVergleich();
			Combo cb = (Combo) _comboVergleich [i];
		  for (int k = 0; k < cb.getItemCount(); k++)
		  {
		    if (jnb.getKennung().equals(cb.getItem( k )))
		      cb.select(k);      
		  }
		  
	  	int anzahl = eigenschaften.get(i).getEigenschaftAnzahlFahrstreifen().getAnzahlFahrstreifen();
	  	Spinner sp = _spinnerPar [i];
	  	sp.setSelection(anzahl);
	  }	  	

    getTvParameter().refresh();
    layout(true);
  }

  /**
   * Methode zum Anlegen des Composite mit den Parametern
   */
  private void createGroupParameter()
  {
    FillLayout fillLayout2 = new FillLayout();
    fillLayout2.type = org.eclipse.swt.SWT.VERTICAL;
    GridData gridData2 = new GridData();
    gridData2.horizontalAlignment = GridData.FILL;
    gridData2.grabExcessHorizontalSpace = true;
    gridData2.grabExcessVerticalSpace = true;
    gridData2.verticalAlignment = GridData.FILL;
    
    _groupParameter = new Group(_composite, SWT.NONE);
    _groupParameter.setText("Parameter");
    _groupParameter.setLayout(fillLayout2);
    _groupParameter.setLayoutData(gridData2);
    
    //GridLayout mit Anzahl Spalten
    GridLayout gridLayout = new GridLayout();
    gridLayout.makeColumnsEqualWidth = false;
    gridLayout.numColumns = 1;
    gridLayout.marginHeight = 5;
    gridLayout.marginWidth = 5;
    
    _compositeTabelle = new Composite(_groupParameter, SWT.NONE);
    _compositeTabelle.setLayout(gridLayout);

    erzeugeTabelle();
  }

  /**
   * Tabelle fr die Parameter erzeugen
   */
  private void erzeugeTabelle()
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessHorizontalSpace = true;
    gridData.grabExcessVerticalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    FillLayout fillLayout = new FillLayout();
    fillLayout.type = org.eclipse.swt.SWT.HORIZONTAL; 

    _composite3 = new Composite(_compositeTabelle, SWT.None);
    _composite3.setLayout(fillLayout);
    _composite3.setLayoutData( gridData );
    
    _tvParameter = new TableViewer(_composite3, SWT.HIDE_SELECTION | SWT.H_SCROLL | SWT.V_SCROLL);
    _tvParameter.setContentProvider(new TableContentProvider());

    TableViewerColumn column1 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column1.getColumn().setWidth(100);
    column1.getColumn().setText("Anzeige");
    column1.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column2 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column2.getColumn().setWidth(100);
    column2.getColumn().setText("Anzahl Fahrstreifen");
    column2.setLabelProvider(new MyCellLabelProvider());

    TableViewerColumn column3 = new TableViewerColumn(_tvParameter, SWT.NONE);
    column3.getColumn().setWidth(100);
    column3.getColumn().setText("Vergleich");
    column3.setLabelProvider(new MyCellLabelProvider());

    /*
     * Zeilenhhe anpassen, nachdem die Daten eingegangen sind
     */
    _tvParameter.getTable().addListener(SWT.MeasureItem, new Listener() 
    {
      @Override
      public void handleEvent(Event event)
      {
        event.height = 29;
      }
    });

    _tvParameter.getTable().setLinesVisible(true);
    _tvParameter.getTable().setHeaderVisible(true);
  }

  /**
   * Contentprovider fr die Tabelle
   * @author ObertM
   *
   */
  private class TableContentProvider implements IStructuredContentProvider, Observer
  {
    private TableViewer tableViewer;

    /*
     * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
     */
    public Object[] getElements(Object inputElement)
    {
      return (TableModel[])inputElement;
    }

    /*
     * @see org.eclipse.jface.viewers.IContentProvider#dispose()
     */
    public void dispose()
    {

    }

    /*
     * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object,
     *      java.lang.Object)
     */
    public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
    {
      tableViewer = (TableViewer)viewer;

      if (oldInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;

        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.deleteObserver(this);
          }

        }

      }
      if (newInput != null)
      {
        TableModel[] tms = (TableModel[])newInput;
        if (tms != null)
        {
          for (TableModel tm : tms)
          {
            tm.addObserver(this);
          }
        }

      }

    }

    @Override
    public void update(Observable arg0, Object arg1)
    {
      if (tableViewer != null)
      {
        tableViewer.refresh();

      }
    }
  }

  /**
   * Tabellenmodell fr die Parameter erzeugen
   * @param table Tabelle
   * @return TabellenModell
   */
  private TableModel[] createModel( Table table )
  {
  	int benoetigt = bestimmeAnzahlEingabeFelder();
  	
  	if (_tableModel != null)
	  	if (benoetigt == _tableModel.length)
	  		return _tableModel;
  	
		/*
		 * Tabelle erweitern
		 */
		if (_tableModel.length < benoetigt)
		{
		  Vector<TableModel> v = new Vector<TableModel>();
		  
		  for (int i=0; i<_tableModel.length; ++i)
		  	v.add(_tableModel [i]);
		  
		  for (int i=_tableModel.length; i<benoetigt; ++i)
	  		v.add (erzeugeVergleichsOperatorInTabelle (table, i));
		  
		  _tableModel = new TableModel [benoetigt];
		  for (int i=0; i<v.size(); ++i)
		  	_tableModel [i] = v.get(i);
		}
		
		/*
		 * Tabelle reduzieren
		 */
		else if (_tableModel.length > benoetigt)
		{
			/*
			 * Nicht mehr bentigte Elemente lschen
			 */
			
			for (int i = _tableModel.length - 1; i > benoetigt - 1; i--)
			{
				_tableModel [i].getEinheit().dispose();
				_tableModel [i].getAttribut().dispose();
				_tableModel [i].getBezeichnung().dispose();
			}
				
		  Vector<TableModel> v = new Vector<TableModel>();
		  for (int i=0; i<benoetigt; ++i)
		  	v.add(_tableModel [i]);

		  _tableModel = new TableModel [benoetigt];
		  for (int i=0; i<v.size(); ++i)
		  	_tableModel [i] = v.get(i);
		}

    return _tableModel;
  }

  /**
   * Methode erzeugt das Tabellenmodell fr den Vergleichsoperator
   * @param table Tabelle
   * @return Tabellemodell
   */
	private TableModel erzeugeVergleichsOperatorInTabelle (Table table, int index)
  {
    /*
     * Ja/Nein/Beliebig
     */
    List<String> jaNeinBeliebig = new ArrayList<String>();
    for (JaNeinBeliebig jnb : JaNeinBeliebig.values())
    	jaNeinBeliebig.add (jnb.getKennung());

    /*
     * ComboBoxen erzeugen
     */
    return createComboElement(table, index, "Anzeige " + (index+1), jaNeinBeliebig, Tools.bestimmeObjektNamen(_listMoeglicheWZGTypen, true) );
  }
  
  /**
   * ein Parameter-Element des TabellenModells erzeugen
   * 
   * @param table           Tabelle
   * @param index           Index der ComboBox
   * @param item            Bezeichnung des Parameters
   * @param einheit         Einheit des Parameters
   * @param jaNeinBeliebig  mgliche Werte der ComboBox
   * @return
   */
  private TableModel createComboElement( Table table, int index, String item, List<String> jaNeinBeliebig, List<String> typen )
  {
  	/*
  	 * Composite 0: Beschriftung 
  	 */
    GridData gd0 = new GridData();
    gd0.horizontalAlignment = GridData.CENTER;
    gd0.grabExcessHorizontalSpace = true;
    gd0.verticalAlignment = GridData.CENTER;
    gd0.grabExcessVerticalSpace = true;
    
    Composite comp0 = new Composite(table, SWT.NONE);
    comp0.setBackground(table.getBackground());
    comp0.setBackgroundMode(SWT.INHERIT_DEFAULT);
    comp0.setLayoutData(gd0);
    comp0.setLayout(new GridLayout());
    
    Label label = new Label(comp0, SWT.NONE);
    label.setText( item );

    /*
     * Composite 1: Combobox Art des Vorgngers/Nachfolgers
     */
    GridData gd1 = new GridData();
    gd1.horizontalAlignment = GridData.CENTER;
    gd1.grabExcessHorizontalSpace = true;
    gd1.grabExcessVerticalSpace = true;
    gd1.verticalAlignment = GridData.END;

    GridData gdTableItem = new GridData();
    gdTableItem.horizontalAlignment = GridData.FILL;
    gdTableItem.grabExcessHorizontalSpace = true;
    gdTableItem.verticalAlignment = GridData.CENTER;

    Composite comp1 = new Composite( table, SWT.NONE );
    comp1.setBackground( table.getBackground() );
    comp1.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp1.setLayoutData( gd1 );
    comp1.setLayout( new GridLayout() );
    
    _spinnerPar[index] = new Spinner(comp1, SWT.BORDER);
    
    // zum Ausfllen der Zelle
    GridData gdSpinner = new GridData();
    gdSpinner.horizontalAlignment = GridData.FILL;
    gdSpinner.grabExcessHorizontalSpace = true;
    gdSpinner.verticalAlignment = GridData.CENTER;
    _spinnerPar[index].setLayoutData(gdSpinner);

    _comboTyp[index] = new Combo( comp1, SWT.BORDER );
    _comboTyp[index].setLayoutData( gdTableItem );
    
    String[] t = new String [typen.size()];
    for (int i=0; i<typen.size(); ++i)
    	t[i] = typen.get(i);
    	
    _comboTyp[index].setItems(t);
    _comboTyp[index].select(0);

    /*
     *  Composite 2: Vergleichsoperator
     */
    
    GridData gd2 = new GridData();
    gd2.horizontalAlignment = GridData.CENTER;
    gd2.grabExcessHorizontalSpace = true;
    gd2.grabExcessVerticalSpace = true;
    gd2.verticalAlignment = GridData.END;

    GridData gdTableItem2 = new GridData();
    gdTableItem2.horizontalAlignment = GridData.FILL;
    gdTableItem2.grabExcessHorizontalSpace = true;
    gdTableItem2.verticalAlignment = GridData.CENTER;

    Composite comp2 = new Composite( table, SWT.NONE );
    comp2.setBackground( table.getBackground() );
    comp2.setBackgroundMode( SWT.INHERIT_DEFAULT );
    comp2.setLayoutData( gd1 );
    comp2.setLayout( new GridLayout() );
    
    _comboVergleich[index] = new Combo( comp2, SWT.BORDER );
    _comboVergleich[index].setLayoutData( gdTableItem2 );
    
    String[] t2 = new String [jaNeinBeliebig.size()];
    for (int i=0; i<jaNeinBeliebig.size(); ++i)
    	t2[i] = jaNeinBeliebig.get(i);
    	
    _comboVergleich[index].setItems(t2);
    _comboVergleich[index].select(0);
    
    return new TableModel(comp0, comp1, comp2);
  }


  /**
   * Methode liefert den TableViewer
   * @return TableViewer
   */
  public TableViewer getTvParameter()
  {
    return _tvParameter;
  }

  /**
   * Methode liefert den Spinner zu einem Index
   * @param index Index des Spinners
   * @return Spinner
   */
  public Spinner getSpinner(int index)
  {
    return _spinnerPar [index];
  }

  /**
   * Methode liefert die ComboBox zu einem Index
   * @param index Index des Spinners
   * @return Spinner
   */
  public Combo getComboBox (int index)
  {
    return _comboTyp [index];
  }

  /**
   * Methode liefert den Namen eines Items eines Spinners zu einem Index
   * @param index Index
   * @return Name des Items
   */
  public String getItemSpinnerName (int index)
  {
  	return _parItemSpin [index];
  }

  /**
   * Methode liefert den Namen eines Items einer ComboBox zu einem Index
   * @param index Index
   * @return Name des Items
   */
  public String getItemComboBoxName (int index)
  {
  	return _parItemCombo [index];
  }

  /**
   * Labelprovider
   * @author ObertM
   */
  private class MyCellLabelProvider extends CellLabelProvider
  {
    @Override
    public void update(ViewerCell cell)
    {
      TableItem item = (TableItem)cell.getItem();

      if (item.getData("EDITOR") != null)
      {
        TableEditor editor = (TableEditor)item.getData("EDITOR");
        editor.dispose();
      }


      int cnt = cell.getColumnIndex();

      TableModel model = (TableModel)item.getData();
      Composite comp = null;
      switch (cnt)
      {
        case 0:
          comp = model.getBezeichnung();
          break;
        case 1:
          comp = model.getAttribut();
          break;
        case 2:
          comp = model.getEinheit();
          break;
        default:
      }
      TableEditor editor = new TableEditor(item.getParent());
      
      editor.grabHorizontal = true;
      editor.setEditor(comp, item, cnt);
    }
  }
  
  /**
   * Methode liefert die Anzahl der verwendeten Items
   * @return Anzahl 
   */
  public int getAnzahlSpinnerItems ()
  {
  	return _parItemSpin.length;
  }

  /**
   * Methode liefert die Anzahl der verwendeten Items
   * @return Anzahl 
   */
  public int getAnzahlComboBoxItems ()
  {
  	return _parItemCombo.length;
  }

  /**
   * Methode refresht die Tabelle
   */
	public void refresh()
  {
		if (_tvParameter != null)
		{
//	    Table table = _tvParameter.getTable();
//      TableModel[] input = createModel(table);
//      
//      updateData();
//      
//      _tvParameter.setInput(input);
	    _tvParameter.refresh();
		}
  }

	/**
	 * Methode zum bernehmen der eingestellten Daten
	 */
	public void uebernehmeDaten ()
  {
    Vector<UniversalEigenschaften> eigenschaften = _aktuell.getEigenschaften();
    
    for (int i=0; i<eigenschaften.size(); ++i)
    {
    	if (_comboVergleich [i] != null)
    	{
	      Combo cb = _comboVergleich [i];
		    String selected = cb.getItem( cb.getSelectionIndex() );
		    JaNeinBeliebig jnb = JaNeinBeliebig.getWert(selected);
	
		    Spinner sp = _spinnerPar [i];
		    int anzahlFahrstreifen = sp.getSelection();
		    
		    eigenschaften.get(i).getEigenschaftAnzahlFahrstreifen().setVergleich(jnb);
		    eigenschaften.get(i).getEigenschaftAnzahlFahrstreifen().setAnzahlFahrstreifen(anzahlFahrstreifen);
    	}
    	else
    	{
		    eigenschaften.get(i).getEigenschaftAnzahlFahrstreifen().setVergleich(JaNeinBeliebig.BELIEBIG);
		    eigenschaften.get(i).getEigenschaftAnzahlFahrstreifen().setAnzahlFahrstreifen(0);
    	}
    }
  }
}
