/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.JaNeinBeliebig;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse realisiert folgende Eigenschaft der Universalregel:<br>
 * Verweist auf Fahrstreifen "4712" = ja/nein/beliegig
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 */
public class UniversalEigenschaft8
{
  /**
   * Variable zum Speichern des Fahrstreifens 
   */
  private SystemObject m_fahrstreifen;
  
  /**
   * Vergleichsoperator  
   */
  private JaNeinBeliebig m_vergleich = JaNeinBeliebig.BELIEBIG;

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public UniversalEigenschaft8 ()
  {
  }

  /**
   * Methode zum Auswerten der Attributliste 'atl.sswSbaRegelUniversalEigenschaft8'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe( Data data )
  {
    // Kriterium
    
    String jaNeinBeliebig = data.getItem( "Kriterium" ).asTextValue().getText();
    if (jaNeinBeliebig.equals( "Ja" ))
      m_vergleich = JaNeinBeliebig.JA;
    else if (jaNeinBeliebig.equals(  "Nein" ))
      m_vergleich = JaNeinBeliebig.NEIN;
    else
      m_vergleich = JaNeinBeliebig.BELIEBIG;
    
    // Fahrstreifentyp
    
    m_fahrstreifen = data.getReferenceValue( "Fahrstreifen" ).getSystemObject();
  }


  /**
   * Methode liefert die Variable m_vergleich zurck
   * @return Variable m_vergleich
   */
  public JaNeinBeliebig getVergleich()
  {
    return m_vergleich;
  }

  /**
   * Methode setzt die Variable m_vergleich
   * @param vergleich Wert auf den die Variable m_vergleich gesetzt werden soll
   */
  public void setVergleich( JaNeinBeliebig vergleich )
  {
    m_vergleich = vergleich;
  }

  /**
   * Methode liefert die Variable m_fahrstreifen zurck
   * @return Variable m_fahrstreifen
   */
  public SystemObject getFahrstreifen()
  {
    return m_fahrstreifen;
  }

  /**
   * Methode setzt die Variable m_fahrstreifen
   * @param fahrstreifen Wert auf den die Variable m_fahrstreifen gesetzt werden soll
   */
  public void setFahrstreifen( SystemObject fahrstreifen )
  {
    m_fahrstreifen = fahrstreifen;
  }
  
  /**
   * Methode gibt den Inhalt der Klasse als String aus
   */
  @Override
  public String toString()
  {
    String s = "Verweist auf Fahrstreifen " + Tools.getObjektName( m_fahrstreifen ) + " = " + m_vergleich + "\n";
    return s;
  }

}
