/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.universal.eigenschaften;

import de.bsvrz.dav.daf.main.Data;

/**
 * Zusammenfassung der Eigenschaften einer Regel
 * @author Swarco Traffic Systems GmbH
 * @author Matthias Obert
 */
public class UniversalEigenschaften
{
  /**
   * Eigenschaft: Anzeigeninhalt ist "X" = ja/nein/beliebig
   */
  private UniversalEigenschaft1 m_eigenschaftAnzeigenInhalt = new UniversalEigenschaft1();
  
  /**
   * Eigenschaft: Anzeigeninhalt des Vorintervalls ist "Y" = ja/nein/beliebig
   */
  private UniversalEigenschaft2 m_eigenschaftAnzeigenInhaltVorIntervall = new UniversalEigenschaft2();
  
  /**
   * Eigenschaft: hat keinen Vorgaenger = ja/nein/beliebig
   */
  private UniversalEigenschaft3 m_eigenschaftVorgaenger = new UniversalEigenschaft3();

  /**
   * Eigenschaft: hat keinen Nachfolger = ja/nein/beliebig
   */
  private UniversalEigenschaft4 m_eigenschaftNachfolger = new UniversalEigenschaft4();

  /**
   * Eigenschaft: ist vom WZG Typ "A" = ja/nein/beliebig
   */
  private UniversalEigenschaft5 m_eigenschaftWzgTyp = new UniversalEigenschaft5();

  /**
   * Eigenschaft: ist die Anzeige "4711" = ja/nein/beliebig
   */
  private UniversalEigenschaft6 m_eigenschaftAnzeige = new UniversalEigenschaft6();

  /**
   * Eigenschaft: verweist auf Fahrstreifentyp "F" = ja/nein/beliebig
   */
  private UniversalEigenschaft7 m_eigenschaftFahrstreifenTyp = new UniversalEigenschaft7();

  /**
   * Eigenschaft: verweist auf die Fahrstreifen "4712" = ja/nein/beliebig
   */
  private UniversalEigenschaft8 m_eigenschaftFahrstreifen = new UniversalEigenschaft8();

  /**
   * Eigenschaft: Anzeige ausgefallen = ja/nein/beliebig
   */
  private UniversalEigenschaft9 m_eigenschaftAnzeigeAusgefallen = new UniversalEigenschaft9();

  /**
   * Eigenschaft: WVZ Inhalt in ["U","V","W"] ausgefallen = ja/nein/beliebig
   */
  private UniversalEigenschaft10 m_eigenschaftInhaltAusgefallen = new UniversalEigenschaft10();

  /**
   * Eigenschaft: Anzahl Fahrstreifen AQ[ErsteAnzeigeDerReihe] ist "n" = ja/nein/beliebig
   */
  private UniversalEigenschaft11 m_eigenschaftAnzahlFahrstreifen = new UniversalEigenschaft11();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public UniversalEigenschaften ()
  {
  }

  /**
   * Methode zum Auswerten der Attributliste 'atl.sswSbaRegelUniversalEigenschaften'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe( Data data )
  {
    // Eigenschaft: Anzeigeinhalt ist "X" = ja/nein/beliebig
    
    Data anzeigeInhalt = data.getItem( "AnzeigeInhalt" );
    m_eigenschaftAnzeigenInhalt = new UniversalEigenschaft1 ();
    m_eigenschaftAnzeigenInhalt.auswerteAttributgruppe (anzeigeInhalt);
    
    // Eigenschaft: Anzeigeinhalt des Vorintervalls ist "Y" = ja/nein/beliebig
    
    Data anzeigeInhaltVorIntervall = data.getItem( "AnzeigeInhaltVorIntervall" );
    m_eigenschaftAnzeigenInhaltVorIntervall = new UniversalEigenschaft2 ();
    m_eigenschaftAnzeigenInhaltVorIntervall.auswerteAttributgruppe (anzeigeInhaltVorIntervall);
    
    // Eigenschaft: hat keinen Vorgnger = ja/nein/beliebig
    
    Data vorgaenger = data.getItem( "Vorgnger" );
    m_eigenschaftVorgaenger = new UniversalEigenschaft3 ();
    m_eigenschaftVorgaenger.auswerteAttributgruppe( vorgaenger );
    
    // Eigenschaft: hat keinen Nachfolger = ja/nein/beliebig
    
    Data nachfolger = data.getItem( "Nachfolger" );
    m_eigenschaftNachfolger = new UniversalEigenschaft4 ();
    m_eigenschaftNachfolger.auswerteAttributgruppe( nachfolger );

    // Eigenschaft: ist vom WZGTyp "A" = ja/nein/beliebig
    
    Data wzgTyp = data.getItem( "WZGTyp" );
    m_eigenschaftWzgTyp = new UniversalEigenschaft5 ();
    m_eigenschaftWzgTyp.auswerteAttributgruppe( wzgTyp );
    
    // Eigenschaft: ist die Anzeige "4711" = ja/nein/beliebig

    Data anzeige = data.getItem( "Anzeige" );
    m_eigenschaftAnzeige = new UniversalEigenschaft6 ();
    m_eigenschaftAnzeige.auswerteAttributgruppe( anzeige );

    // Eigenschaft: Verweist auf Fahrstreifentyp "F" = ja/nein/beliebig

    Data fahrstreifenTyp = data.getItem( "FahrstreifenTyp" );
    m_eigenschaftFahrstreifenTyp = new UniversalEigenschaft7 ();
    m_eigenschaftFahrstreifenTyp.auswerteAttributgruppe( fahrstreifenTyp );

    // Eigenschaft: Verweist auf Fahrstreifen "4712" = ja/nein/beliebig

    Data fahrstreifen = data.getItem( "Fahrstreifen" );
    m_eigenschaftFahrstreifen = new UniversalEigenschaft8 ();
    m_eigenschaftFahrstreifen.auswerteAttributgruppe( fahrstreifen );

    // Eigenschaft: Anzeige ausgefallen = ja/nein/beliebig
    
    Data anzeigeAusgefallen = data.getItem( "AusfallAnzeige" );
    m_eigenschaftAnzeigeAusgefallen = new UniversalEigenschaft9 ();
    m_eigenschaftAnzeigeAusgefallen.auswerteAttributgruppe( anzeigeAusgefallen );

    // Eigenschaft: WVZ Inhalt in ["U", "V", "W"] ausgefallen = ja/nein/beliebig
    
    Data ausfallWVZInhalt = data.getItem( "AusfallWVZInhalt" );
    m_eigenschaftInhaltAusgefallen = new UniversalEigenschaft10 ();
    m_eigenschaftInhaltAusgefallen.auswerteAttributgruppe( ausfallWVZInhalt );

    // Eigenschaft: Anzahl Fahrstreifen AQ = n = ja/nein/beliebig
    
    Data anzahlFahrstreifen = data.getItem( "AnzahlFahrstreifen" );
    m_eigenschaftAnzahlFahrstreifen = new UniversalEigenschaft11 ();
    m_eigenschaftAnzahlFahrstreifen.auswerteAttributgruppe( anzahlFahrstreifen );
  }

  /**
   * Methode gibt den Inhalt der Klasse als String aus
   */
  @Override
  public String toString()
  {
    String s = "";
    
    s += m_eigenschaftAnzeigenInhalt.toString();
    s += m_eigenschaftAnzeigenInhaltVorIntervall.toString();
    s += m_eigenschaftVorgaenger.toString();
    s += m_eigenschaftNachfolger.toString();
    s += m_eigenschaftWzgTyp.toString();
    s += m_eigenschaftAnzeige.toString();
    s += m_eigenschaftFahrstreifenTyp.toString();
    s += m_eigenschaftFahrstreifen.toString();
    s += m_eigenschaftAnzeigeAusgefallen.toString();
    s += m_eigenschaftInhaltAusgefallen.toString();
    s += m_eigenschaftAnzahlFahrstreifen.toString();
    
    return s;
  }

  /**
   * Methode liefert die Variable m_eigenschaftAnzeigenInhalt zurck
   * @return Variable m_eigenschaftAnzeigenInhalt
   */
  public UniversalEigenschaft1 getEigenschaftAnzeigenInhalt()
  {
    return m_eigenschaftAnzeigenInhalt;
  }

  /**
   * Methode setzt die Variable m_eigenschaftAnzeigenInhalt
   * @param eigenschaftAnzeigenInhalt Wert auf den die Variable m_eigenschaftAnzeigenInhalt gesetzt werden soll
   */
  public void setEigenschaftAnzeigenInhalt( UniversalEigenschaft1 eigenschaftAnzeigenInhalt )
  {
    m_eigenschaftAnzeigenInhalt = eigenschaftAnzeigenInhalt;
  }

  /**
   * Methode liefert die Variable m_eigenschaftAnzeigenInhaltVorIntervall zurck
   * @return Variable m_eigenschaftAnzeigenInhaltVorIntervall
   */
  public UniversalEigenschaft2 getEigenschaftAnzeigenInhaltVorIntervall()
  {
    return m_eigenschaftAnzeigenInhaltVorIntervall;
  }

  /**
   * Methode setzt die Variable m_eigenschaftAnzeigenInhaltVorIntervall
   * @param eigenschaftAnzeigenInhaltVorIntervall Wert auf den die Variable m_eigenschaftAnzeigenInhaltVorIntervall gesetzt werden soll
   */
  public void setEigenschaftAnzeigenInhaltVorIntervall( UniversalEigenschaft2 eigenschaftAnzeigenInhaltVorIntervall )
  {
    m_eigenschaftAnzeigenInhaltVorIntervall = eigenschaftAnzeigenInhaltVorIntervall;
  }

  /**
   * Methode liefert die Variable m_eigenschaftVorgaenger zurck
   * @return Variable m_eigenschaftVorgaenger
   */
  public UniversalEigenschaft3 getEigenschaftVorgaenger()
  {
    return m_eigenschaftVorgaenger;
  }

  /**
   * Methode setzt die Variable m_eigenschaftVorgaenger
   * @param eigenschaftVorgaenger Wert auf den die Variable m_eigenschaftVorgaenger gesetzt werden soll
   */
  public void setEigenschaftVorgaenger( UniversalEigenschaft3 eigenschaftVorgaenger )
  {
    m_eigenschaftVorgaenger = eigenschaftVorgaenger;
  }

  /**
   * Methode liefert die Variable m_eigenschaftNachfolger zurck
   * @return Variable m_eigenschaftNachfolger
   */
  public UniversalEigenschaft4 getEigenschaftNachfolger()
  {
    return m_eigenschaftNachfolger;
  }

  /**
   * Methode setzt die Variable m_eigenschaftNachfolger
   * @param eigenschaftNachfolger Wert auf den die Variable m_eigenschaftNachfolger gesetzt werden soll
   */
  public void setEigenschaftNachfolger( UniversalEigenschaft4 eigenschaftNachfolger )
  {
    m_eigenschaftNachfolger = eigenschaftNachfolger;
  }

  /**
   * Methode liefert die Variable m_eigenschaftWzgTyp zurck
   * @return Variable m_eigenschaftWzgTyp
   */
  public UniversalEigenschaft5 getEigenschaftWzgTyp()
  {
    return m_eigenschaftWzgTyp;
  }

  /**
   * Methode setzt die Variable m_eigenschaftWzgTyp
   * @param eigenschaftWzgTyp Wert auf den die Variable m_eigenschaftWzgTyp gesetzt werden soll
   */
  public void setEigenschaftWzgTyp( UniversalEigenschaft5 eigenschaftWzgTyp )
  {
    m_eigenschaftWzgTyp = eigenschaftWzgTyp;
  }

  /**
   * Methode liefert die Variable m_eigenschaftAnzeige zurck
   * @return Variable m_eigenschaftAnzeige
   */
  public UniversalEigenschaft6 getEigenschaftAnzeige()
  {
    return m_eigenschaftAnzeige;
  }

  /**
   * Methode setzt die Variable m_eigenschaftAnzeige
   * @param eigenschaftAnzeige Wert auf den die Variable m_eigenschaftAnzeige gesetzt werden soll
   */
  public void setEigenschaftAnzeige( UniversalEigenschaft6 eigenschaftAnzeige )
  {
    m_eigenschaftAnzeige = eigenschaftAnzeige;
  }

  /**
   * Methode liefert die Variable m_eigenschaftFahrstreifenTyp zurck
   * @return Variable m_eigenschaftFahrstreifenTyp
   */
  public UniversalEigenschaft7 getEigenschaftFahrstreifenTyp()
  {
    return m_eigenschaftFahrstreifenTyp;
  }

  /**
   * Methode setzt die Variable m_eigenschaftFahrstreifenTyp
   * @param eigenschaftFahrstreifenTyp Wert auf den die Variable m_eigenschaftFahrstreifenTyp gesetzt werden soll
   */
  public void setEigenschaftFahrstreifenTyp( UniversalEigenschaft7 eigenschaftFahrstreifenTyp )
  {
    m_eigenschaftFahrstreifenTyp = eigenschaftFahrstreifenTyp;
  }

  /**
   * Methode liefert die Variable m_eigenschaftFahrstreifen zurck
   * @return Variable m_eigenschaftFahrstreifen
   */
  public UniversalEigenschaft8 getEigenschaftFahrstreifen()
  {
    return m_eigenschaftFahrstreifen;
  }

  /**
   * Methode setzt die Variable m_eigenschaftFahrstreifen
   * @param eigenschaftFahrstreifen Wert auf den die Variable m_eigenschaftFahrstreifen gesetzt werden soll
   */
  public void setEigenschaftFahrstreifen( UniversalEigenschaft8 eigenschaftFahrstreifen )
  {
    m_eigenschaftFahrstreifen = eigenschaftFahrstreifen;
  }

  /**
   * Methode liefert die Variable m_eigenschaftAnzeigeAusgefallen zurck
   * @return Variable m_eigenschaftAnzeigeAusgefallen
   */
  public UniversalEigenschaft9 getEigenschaftAnzeigeAusgefallen()
  {
    return m_eigenschaftAnzeigeAusgefallen;
  }

  /**
   * Methode setzt die Variable m_eigenschaftAnzeigeAusgefallen
   * @param eigenschaftAnzeigeAusgefallen Wert auf den die Variable m_eigenschaftAnzeigeAusgefallen gesetzt werden soll
   */
  public void setEigenschaftAnzeigeAusgefallen( UniversalEigenschaft9 eigenschaftAnzeigeAusgefallen )
  {
    m_eigenschaftAnzeigeAusgefallen = eigenschaftAnzeigeAusgefallen;
  }

  /**
   * Methode liefert die Variable m_eigenschaftInhaltAusgefallen zurck
   * @return Variable m_eigenschaftInhaltAusgefallen
   */
  public UniversalEigenschaft10 getEigenschaftInhaltAusgefallen()
  {
    return m_eigenschaftInhaltAusgefallen;
  }

  /**
   * Methode setzt die Variable m_eigenschaftInhaltAusgefallen
   * @param eigenschaftInhaltAusgefallen Wert auf den die Variable m_eigenschaftInhaltAusgefallen gesetzt werden soll
   */
  public void setEigenschaftInhaltAusgefallen( UniversalEigenschaft10 eigenschaftInhaltAusgefallen )
  {
    m_eigenschaftInhaltAusgefallen = eigenschaftInhaltAusgefallen;
  }

  /**
   * Methode liefert die Variable m_eigenschaftAnzahlFahrstreifen zurck
   * @return Variable m_eigenschaftAnzahlFahrstreifen
   */
  public UniversalEigenschaft11 getEigenschaftAnzahlFahrstreifen()
  {
    return m_eigenschaftAnzahlFahrstreifen;
  }

  /**
   * Methode setzt die Variable m_eigenschaftAnzahlFahrstreifen
   * @param eigenschaftAnzahlFahrstreifen Wert auf den die Variable m_eigenschaftAnzahlFahrstreifen gesetzt werden soll
   */
  public void setEigenschaftAnzahlFahrstreifen( UniversalEigenschaft11 eigenschaftAnzahlFahrstreifen )
  {
    m_eigenschaftAnzahlFahrstreifen = eigenschaftAnzahlFahrstreifen;
  }
}
