package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.prio;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.Map.Entry;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.laengsquerabgleich.monotonie.RegelMonotonie;

/**
 * Klasse zum speichern der Regeln die fr die Anlage gelten
 * @author Dambach Werke GmbH
 * @author Matthias Obert
 */
class ListeMitRegeln
{
	Vector<RegelPriorisierungNeu> _parameter = new Vector<RegelPriorisierungNeu> ();

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaRegelPriorisierungProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data)
  {
    Array a = data.getArray( "Prioritten" );
    int anzahl = a.getLength();
    
    synchronized (_parameter)
    {
      _parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        Data d1 = a.getItem( x );
        
        WVZInhalt inhalt = new WVZInhalt();
        inhalt.auswerteAttributgruppe( d1.getItem( "Zeichenfolge" ) );
        
        RegelPriorisierung r = new RegelPriorisierung ();
        r.auswerteAttributGruppe ( d1.getItem( "Prioritten" ) );
        
        _parameter.addAll(umwandleParameter(inhalt, r));
      }
    }
  }

  /**
   * Methode wandelt einen Parametersatz in das neue Format um
   * @param inhalt WVZ Inhalt
   * @param regel Regel
   * @return Vektor mit Regeln im neuen Format
   */
  public Vector<RegelPriorisierungNeu> umwandleParameter (WVZInhalt inhalt, RegelPriorisierung regel)
  {
  	Vector<RegelPriorisierungNeu> v = new Vector<RegelPriorisierungNeu>();
  	
    RegelPriorisierungNeu rpn = new RegelPriorisierungNeu (inhalt, regel.getPrioritaet());
    v.add(rpn);
    
    HashMap<Aspect, Integer> prioProAspekt = regel.getPrioritaetAspekte();
    
    Iterator<Entry<Aspect, Integer>> it1 = prioProAspekt.entrySet().iterator();
    while (it1.hasNext())
    {
    	Entry<Aspect, Integer> entry1 = it1.next();
    	
    	Aspect schaltgrund = entry1.getKey();
    	Integer prio = entry1.getValue();
    	
    	rpn = new RegelPriorisierungNeu(inhalt, prio, schaltgrund);
    	v.add(rpn);
    }
    
  	return v;
  }

  /**
   * Methode zum Ausgeben der Klasse als String
   * @return Klasse als String
   */
  public String toString ()
  {
    String s = "Parameter Anlage:\n";
    s +=       "=================\n";
    
    synchronized (_parameter)
    {
    	for (RegelPriorisierungNeu rpn : _parameter)
    	{
    		s +=  "WVZ-Inhalt: " + rpn.getWvzInhalt().toString() + "\n";
    		
    		if (rpn.getSchaltgrund() != null)
    		  s +=  "Aspekt    : " + rpn.getSchaltgrund().getPid() + "\n";
    		else
    		  s +=  "Aspekt    : null\n";
    		
    		s += "Prio:       " + rpn.getPrioritaet() + "\n";
    	}
    }
    
    return s;
  }

  /**
   * Methode fgt eine Regel mit einer Priorittenzuordnung der Klasse hinzu
   * @param inhalt
   * @param prio
   * @param schaltgrund
   */
  public void addRegel (WVZInhalt inhalt, int prio, Aspect schaltgrund)
  {
    synchronized (_parameter)
    {
    	RegelPriorisierungNeu rpn = new RegelPriorisierungNeu(inhalt, prio, schaltgrund);
    	_parameter.add(rpn);
    }
  }

  /**
   * Methode liefert die Variable m_regeln zurck
   * @return Variable m_regeln
   */
  public HashMap<WVZInhalt, RegelPriorisierung> getRegeln()
  {
  	HashMap<String, RegelPriorisierung> regelnAufBasisVonPid = new HashMap<String, RegelPriorisierung>();
  	
  	synchronized (_parameter)
    {
      RegelPriorisierung regel = null;
      
      for (RegelPriorisierungNeu rpn : _parameter)
      {
        WVZInhalt inhalt = rpn.getWvzInhalt();
        Aspect schaltgrund = rpn.getSchaltgrund();
        int prio = rpn.getPrioritaet();
      
				if (regelnAufBasisVonPid.containsKey( inhalt.getPid() ))
		    {
		      regel = regelnAufBasisVonPid.get( inhalt.getPid() );
		    }
		    else
		    {
		      regel = new RegelPriorisierung ();
		      regelnAufBasisVonPid.put( inhalt.getPid(), regel);
		    }
		    
		    if (schaltgrund == null)
		    {
		      regel.setPrioritaet(prio);
		    }
		    else
		    {
		    	regel.addAspekt (schaltgrund, prio);
		    	regel.getPrioritaetAspekte().put(schaltgrund, prio);
		    }
      }
    }

  	HashMap<WVZInhalt, RegelPriorisierung> regelnAufBasiVonWVZInhalt = new HashMap<WVZInhalt, RegelPriorisierung>();

  	Iterator<Entry<String, RegelPriorisierung>> it = regelnAufBasisVonPid.entrySet().iterator();
  	while (it.hasNext())
  	{
  	  Entry<String, RegelPriorisierung> entry = it.next();
  	  
  	  String pid = entry.getKey();
  	  RegelPriorisierung rp = entry.getValue();
  	  
  	  WVZInhalt inhalt = new WVZInhalt(pid);
  	  
  	  regelnAufBasiVonWVZInhalt.put(inhalt, rp);
  	}
  	
  	return regelnAufBasiVonWVZInhalt;
  }

  /**
   * Methode setzt die Variable m_regeln
   * @param regeln Wert auf den die Variable m_regeln gesetzt werden soll
   */
  public void setParameter( Vector<RegelPriorisierungNeu> regeln )
  {
  	synchronized (_parameter)
    {
      _parameter.clear();
      
      for (RegelPriorisierungNeu rpn : regeln)
      	_parameter.add(rpn);
    }
  }

  /**
   * Methode liefert die Parameter als Vector
   * @return Parameter als Vector
   */
  public Vector<RegelPriorisierungNeu> getParameter ()
  {
//  	Vector<RegelPriorisierungNeu> v = new Vector<RegelPriorisierungNeu>();
//  	synchronized (_parameter)
//    {
//  		for (RegelPriorisierungNeu rpn : _parameter)
//  			v.add(rpn);
//    }
//  	
  	return getParameterSortiert();
  }
  
  private Vector<RegelPriorisierungNeu> getParameterSortiert ()
  {
  	Vector<RegelPriorisierungNeu> v = new Vector<RegelPriorisierungNeu>();
  	synchronized (_parameter)
    {
  		int max = -1;
  		int min = Integer.MAX_VALUE;
  		
  		for (RegelPriorisierungNeu rpn : _parameter)
  		{
  			if (rpn.getPrioritaet() > max)
  				max = rpn.getPrioritaet();

  			if (rpn.getPrioritaet() < min)
  				min = rpn.getPrioritaet();
  		}


  		for (int prio=max; prio>=min; prio--)
  		{
    		for (RegelPriorisierungNeu rpn : _parameter)
    			if (rpn.getPrioritaet() == prio)
    				v.add(rpn);
  		}
    }
  	
  	return v;
  	
  }
  
}
