package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.steuerungskern;

import java.util.TreeMap;
import java.util.Vector;

import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.List;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;

/**
 * Darstellung des Fensters zur Parameternderung ohne ButtonLeiste
 * 
 * @author Krause
 *
 */
public class CompositeAnzeigen extends Composite
{
	/**
	 * Mgliche Anzeigen
	 */
	private Einheit _moeglicheAnzeigen = new Einheit ();

	/**
	 * Anzeigen die ermittelt und versendet werden sollen
	 */
	private Einheit _ermittelnUndVersenden = new Einheit ();

	/**
	 * Anzeigen die nur ermittelt aber nicht versendet werden sollen
	 */
	private Einheit _nurErmitteln = new Einheit ();

	/**
	 * Button Feld fr das Verschieben der mglichen Anzeigen zu den Anzeigen die nur ermittelt 
	 * und nicht versendet werden sollen
	 */
  private EinheitButtons _buttons1;

	/**
	 * Button Feld fr das Verschieben der mglichen Anzeigen zu den Anzeigen die ermittelt und 
	 * versendet werden sollen
	 */
  private EinheitButtons _buttons2;
  
	/**
	 * Aktuelle Parameter
	 */
	private ParaAnzeigen _parameter;
	
  /**
   * Konstruktor der Klasse
   * @param parent Parent Composite
   * @param style Stil
   * @param parameter aktuelle Parameter
   */
	CompositeAnzeigen(Composite parent, int style, ParameterSteuerungsKern parameter) 
  {
    super(parent, style);
    
    _parameter = parameter.getParaAnzeigen();
   
    initialize();
  }

	/**
   * Fhrt die Basis-Initialisierung des Composites durch
   */
  private void initialize()
  {
    GridData gridData1 = new GridData();
    gridData1.horizontalAlignment = GridData.FILL;
    gridData1.grabExcessHorizontalSpace = true;
    gridData1.grabExcessVerticalSpace = true;
    gridData1.horizontalIndent = 0;
    gridData1.verticalAlignment = GridData.FILL;

    setLayout(new GridLayout(5, false));
    setLayoutData(gridData1);
    
    bestimmeAlleAnzeigen();
    
    if (_parameter.getErmittelndUndVersenden() != null)
    {
      _ermittelnUndVersenden.clear();
      for (SystemObject so :_parameter.getErmittelndUndVersenden())
      {
      	_ermittelnUndVersenden.add(so);
      	_moeglicheAnzeigen.remove(so);
      }
      
      _nurErmitteln.clear();
      for (SystemObject so :_parameter.getNurErmitteln())
      {
      	_nurErmitteln.add(so);
      	_moeglicheAnzeigen.remove(so);
      }
    }

    createComposite(this);
  }

  /**
   * Methode bestimmt die im DaV definierten Anzeigen
   */
	private void bestimmeAlleAnzeigen()
  {
		TreeMap<String, SystemObject> sorter = new TreeMap<String, SystemObject>();
		
		java.util.List<SystemObject> objekte = Konfig.getInstance().getAnzeigen();
    for (SystemObject so : objekte)
    	sorter.put(getBezeichnung(so), so);
    
    for (SystemObject so : sorter.values())
		  _moeglicheAnzeigen.add(so);
  }

  /**
   * Methode legt ein Composite an in dem die Anzeigen versorgt werden knnen
   * @param parent Parent Composite
   */
  private void createComposite (Composite parent)
  {
    // Zeile 0
    
    GridData gdZeile0 = new GridData ();
    gdZeile0.horizontalAlignment = GridData.FILL;
    gdZeile0.verticalAlignment = GridData.FILL;
    gdZeile0.grabExcessHorizontalSpace = true;
    gdZeile0.grabExcessVerticalSpace = false;

//    Group zeile0 = new Group (parent, SWT.NONE);
//    zeile0.setText("Zeile 0");
//    zeile0.setLayout(new GridLayout (5, false));
    
    Composite zeile0 = parent;
    
    GridData gd = new GridData();
    
    gd.horizontalAlignment = GridData.BEGINNING;
    gd.verticalAlignment = GridData.BEGINNING;
    gd.grabExcessHorizontalSpace = false;
    gd.grabExcessVerticalSpace = false;
    gd.heightHint = 400;
    
//    zeile0.setLayoutData(gd);
  	
    // Zeile 0 Spalte 1
    
    GridData gdZeile0Spalte1 = new GridData ();
    gdZeile0Spalte1.horizontalAlignment = GridData.FILL;
    gdZeile0Spalte1.verticalAlignment = GridData.FILL;
    gdZeile0Spalte1.grabExcessHorizontalSpace = true;
    gdZeile0Spalte1.grabExcessVerticalSpace = true;
    gdZeile0Spalte1.minimumWidth = 200;

    Composite zeile0Spalte1 = new Composite (zeile0, SWT.BORDER);
    zeile0Spalte1.setLayout(new GridLayout(1, false));
    zeile0Spalte1.setLayoutData(gdZeile0Spalte1);

    // Zeile 0 Spalte 2

    GridData gdZeile0Spalte2 = new GridData ();
    gdZeile0Spalte2.horizontalAlignment = GridData.CENTER;
    gdZeile0Spalte2.grabExcessHorizontalSpace = false;
    gdZeile0Spalte2.minimumWidth = 80;

    Composite zeile0Spalte2 = new Composite (zeile0, SWT.BORDER);
    zeile0Spalte2.setLayout(new GridLayout (1, true));
    zeile0Spalte2.setLayoutData(gdZeile0Spalte2);

    // Zeile 0 Spalte 3

    GridData gdZeile0Spalte3 = new GridData ();
    gdZeile0Spalte3.horizontalAlignment = GridData.FILL;
    gdZeile0Spalte3.verticalAlignment = GridData.FILL;
    gdZeile0Spalte3.grabExcessHorizontalSpace = true;
    gdZeile0Spalte3.grabExcessVerticalSpace = true;
    gdZeile0Spalte3.minimumWidth = 200;

    Composite zeile0Spalte3 = new Composite (zeile0, SWT.BORDER);
    zeile0Spalte3.setLayout(new GridLayout(1, false));
    zeile0Spalte3.setLayoutData(gdZeile0Spalte3);

    // Zeile 0 Spalte 4

    GridData gdZeile0Spalte4 = new GridData ();
    gdZeile0Spalte4.horizontalAlignment = GridData.CENTER;
    gdZeile0Spalte4.grabExcessHorizontalSpace = false;
    gdZeile0Spalte4.minimumWidth = 80;

    Composite zeile0Spalte4 = new Composite (zeile0, SWT.BORDER);
    zeile0Spalte4.setLayout(new GridLayout (1, true));
    zeile0Spalte4.setLayoutData(gdZeile0Spalte4);

    // Zeile 0 Spalte 5

    GridData gdZeile0Spalte5 = new GridData ();
    gdZeile0Spalte5.horizontalAlignment = GridData.FILL;
    gdZeile0Spalte5.verticalAlignment = GridData.FILL;
    gdZeile0Spalte5.grabExcessHorizontalSpace = true;
    gdZeile0Spalte5.grabExcessVerticalSpace = true;
    gdZeile0Spalte5.minimumWidth = 200;

    Composite zeile0Spalte5 = new Composite (zeile0, SWT.BORDER);
    zeile0Spalte5.setLayout(new GridLayout(1, false));
    zeile0Spalte5.setLayoutData(gdZeile0Spalte5);

    addNurErmitteln(zeile0Spalte1);

    addKopierenButtons1(zeile0Spalte2);
    
    addMoeglicheAnzeigen(zeile0Spalte3);
    
    addKopierenButtons2(zeile0Spalte4);
      
    addErmittelnUndVersenden(zeile0Spalte5);
  }

  /**
   * Fgt die Liste mit den mglichen Funktionen hinzu
   * @param parent Composite in das die Liste eingefgt werden sollen
   */
  private void addMoeglicheAnzeigen (Composite parent)
  {
    Group group = new Group (parent, SWT.BORDER);
    group.setText("Mgliche Anzeigen");
    group.setLayout(new FillLayout(SWT.VERTICAL));
    
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;

    group.setLayoutData(gd);
   
    final List list = new List (group, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);

    _moeglicheAnzeigen.setListe(list);

    list.addSelectionListener(new SelectionListener () 
    {
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      public void widgetSelected(SelectionEvent e)
      {
      	boolean anwahlMoeglich = true;
      	
     		anwahlMoeglich = list.getSelectionCount() != 0;

     		if (_buttons1 != null)
          _buttons1.getButtonNachLinks().setEnabled(anwahlMoeglich);

     		if (_buttons2 != null)
          _buttons2.getButtonNachRechts().setEnabled(anwahlMoeglich);
      }
    });
    
    list.addMouseListener(new MouseListener()
		{
			@Override
			public void mouseUp(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDown(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDoubleClick(MouseEvent e)
			{
				if (_buttons2 != null)
					_buttons2.nachRechts ();

		}
		});
  }

  /**
   * Fgt die Liste mit den zugewiesenden Funktionen hinzu
   * @param parent Composite in das die Liste eingefgt werden sollen
   */
  private void addErmittelnUndVersenden (Composite parent)
  {
    Group group = new Group (parent, SWT.BORDER);
    group.setText("Ermitteln und Versenden");
    group.setLayout(new FillLayout(SWT.VERTICAL));
    
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;

    group.setLayoutData(gd);
   
    final List list = new List (group, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
    _ermittelnUndVersenden.setListe(list);
    
    list.addSelectionListener(new SelectionListener () 
    {
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      public void widgetSelected(SelectionEvent e)
      {
      	boolean anwahlMoeglich = true;
      	
     		anwahlMoeglich = list.getSelectionCount() != 0;
      	
     		if (_buttons2 != null)
          _buttons2.getButtonNachLinks().setEnabled(anwahlMoeglich);
      }
    });

    list.addMouseListener(new MouseListener()
		{
			@Override
			public void mouseUp(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDown(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDoubleClick(MouseEvent e)
			{
				if (_buttons2 != null)
					_buttons2.nachLinks ();
			}
		});
  }

  /**
   * Fgt die Liste mit den zugewiesenden Funktionen hinzu
   * @param parent Composite in das die Liste eingefgt werden sollen
   */
  private void addNurErmitteln (Composite parent)
  {
    Group group = new Group (parent, SWT.BORDER);
    group.setText("Nur Ermitteln");
    group.setLayout(new FillLayout(SWT.VERTICAL));
    
    GridData gd = new GridData();
    gd.horizontalAlignment = GridData.FILL;
    gd.verticalAlignment = GridData.FILL;
    gd.grabExcessHorizontalSpace = true;
    gd.grabExcessVerticalSpace = true;

    group.setLayoutData(gd);
   
    final List list = new List (group, SWT.BORDER | SWT.MULTI | SWT.V_SCROLL);
    _nurErmitteln.setListe(list);
    
    list.addSelectionListener(new SelectionListener () 
    {
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }

      public void widgetSelected(SelectionEvent e)
      {
      	boolean anwahlMoeglich = true;
      	
     		anwahlMoeglich = list.getSelectionCount() != 0;
      	
     		if (_buttons1 != null)
     		{
          _buttons1.getButtonNachRechts().setEnabled(anwahlMoeglich);
     		}
      }
    });

    list.addMouseListener(new MouseListener()
		{
			@Override
			public void mouseUp(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDown(MouseEvent e)
			{
			}
			
			@Override
			public void mouseDoubleClick(MouseEvent e)
			{
				if (_buttons1 != null)
				{
					_buttons1.nachRechts();
				}
			}
		});
  }

  /**
   * Fgt die Kopierbuttons hinzu
   * @param parent Composite in das die Buttons eingefgt werden sollen
   */
  private void addKopierenButtons1 (Composite parent)
  {
  	_buttons1 = new EinheitButtons(parent, _nurErmitteln, _moeglicheAnzeigen);
  }

  /**
   * Fgt die Kopierbuttons hinzu
   * @param parent Composite in das die Buttons eingefgt werden sollen
   */
  private void addKopierenButtons2 (Composite parent)
  {
  	_buttons2 = new EinheitButtons(parent, _moeglicheAnzeigen, _ermittelnUndVersenden);
  }


  /**
   * Methode liefert die aktuell eingestellten Parameter
   * @return aktuell eingestellten Parameter
   */
	public ParaAnzeigen getParameter()
  {
	  return leseAktuelleEinstellungen();
  }
 
	private ParaAnzeigen leseAktuelleEinstellungen ()
	{
    ParaAnzeigen para = new ParaAnzeigen();
    
    /*
     * Ermitteln und Versenden
     */
    para.setErmittelndUndVersenden(_ermittelnUndVersenden.getAnzeigen());

    /*
     * nur Ermitteln
     */
    para.setNurErmitteln(_nurErmitteln.getAnzeigen());

  	return para;
  }

	/**
	 * Methode aktualisiert die View
	 */
	public void refresh()
  {
  }
	
	/**
	 * Methode liefert eine Bezeichnung fr einer Anzeige. Die Bezeichnung setzt sich
	 * zusammen aus Name + Pid
	 * @param objekt Systemobjekt der Anzeige
	 * @return Bezeichnung 
	 */
	private String getBezeichnung (SystemObject objekt)
	{
		return objekt.getName() + " (" + objekt.getPid() + ")";
	}
	
	/**
	 * Klasse speichert die Informationen einer List mit Anzeigen
	 * @author ObertM
	 *
	 */
	private class Einheit 
	{
	  /**
	   * Liste mit den Systemobjekten der Anzeigen
	   */
	  private Vector<SystemObject> _anzeigen = new Vector<SystemObject>();  

	  /**
	   * Dazugehrende Liste
	   */
	  private List _liste;

		/**
		 * Viewer
		 */
		private ListViewer _viewer;

		/**
     * Methode liefert anzeigen zurck
     * @return anzeigen
     */
    public Vector<SystemObject> getAnzeigen()
    {
    	return _anzeigen;
    }

    /**
     * Methode lscht die Liste der Anzeigen
     */
		public void clear()
    {
			_anzeigen.clear();
    }

		/**
		 * Methode entfern eine Anzeige
		 * @param so Anzeige
		 */
		public void remove(SystemObject so)
    {
      _anzeigen.remove(so);	    
    }

		/**
		 * Methode fgt eine Anzeige hinzu
		 * @param so Anzeige
		 */
		public void add(SystemObject so)
    {
			_anzeigen.add(so);
    }

		/**
		 * Methode prft ob eine Anzeige vorhanden ist
		 * @param so Anzeige
		 * @return Ergebnis der Prfung
		 */
		public boolean contains(SystemObject so)
    {
	    return _anzeigen.contains(so);
    }

		/**
		 * Methode refreshed dev Viewer
		 */
		public void refresh()
    {
	    if (getViewer() != null)
	    	getViewer().refresh();
    }

		/**
     * Methode setzt die Variable liste
     * @param liste zu bernehmender Wert fr liste
     */
    public void setListe(List liste)
    {
    	_liste = liste;
    	
      _viewer = new ListViewer(_liste);
      
      _viewer.setContentProvider(new IStructuredContentProvider()
  		{
  			@Override
  			public void dispose()
  			{
  			}
  			
  			@Override
  			public Object[] getElements(Object inputElement)
  			{
          Vector v = (Vector)inputElement;
          return v.toArray();
  			}

  			@Override
        public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
        {
        }
  		});
      
      _viewer.setInput(_anzeigen);
      
      _viewer.setLabelProvider(new LabelProvider() 
      {
        public Image getImage(Object element) 
        {
          return null;
        }

        public String getText(Object element) 
        {
          return (getBezeichnung ((SystemObject) element));
        }
      });
      
      _viewer.setSorter(new ViewerSorter()
      {
        public int compare(Viewer viewer, Object e1, Object e2) 
        {
          return getBezeichnung ((SystemObject)e1).compareTo(getBezeichnung((SystemObject)e2));
        }
      });
    }

		/**
     * Methode liefert viewer zurck
     * @return viewer
     */
    public ListViewer getViewer()
    {
    	return _viewer;
    }
	}

	/**
	 * Klasse realisiert ein Composite mit Buttons zum Verschieben von Systemobjekte
	 * zwischen zwei Listen
	 * @author ObertM
	 *
	 */
	private class EinheitButtons
	{
		/**
		 * Button nach rechts 
		 */
		private Button _buttonNachRechts;
		
		/**
		 * Button nach links
		 */
		private Button _buttonNachLinks;
		
		/**
		 * Einheit mit Systemobjekten links
		 */
		private Einheit _links;
		
		/**
		 * Einheit mit Systemobjekten rechts
		 */
		private Einheit _rechts;
			
		/**
		 * Konstruktor der Klasse
		 * @param parent 
		 */
		public EinheitButtons (Composite parent, Einheit links, Einheit rechts)
		{
			_links = links;
			
			_rechts = rechts;
			
		  erzeugeComposite(parent);	
		}

		/**
		 * Methode verschiebt die Sytemobjekte von rechts nach links
		 */
	  public void nachLinks()
    {
	  	actionKopieren(_rechts, _links);
	  	_buttonNachLinks.setEnabled(false);
    }

	  /**
	   * Methode verschiebt die Systemobjekte von links nach rechts
	   */
		public void nachRechts()
    {
	  	actionKopieren(_links, _rechts);
	  	_buttonNachRechts.setEnabled(false);
    }

		/**
	   * Fgt die Kopierbuttons hinzu
	   * @param parent Composite in das die Buttons eingefgt werden sollen
	   */
	  public void erzeugeComposite (Composite parent)
	  {
	    Composite composite = new Composite (parent, SWT.NONE);
	    
	    _buttonNachRechts = new Button(composite, SWT.PUSH);
//	    _buttonNachRechts.setText(" Hinzufgen >>");
	    _buttonNachRechts.setText(" >> ");
	    _buttonNachRechts.setEnabled(false);
	    _buttonNachRechts.addSelectionListener(new SelectionAdapter()
	    {
	      public void widgetSelected(SelectionEvent e)
	      {
	        actionKopieren(_links, _rechts);
	        _buttonNachRechts.setEnabled(false);
	        
	        // Wenn in der Auswahl der mglichen Anzeigen eine Anzeige verschoben wird,
	        // so mssen danach beide Richtungen deaktiviert werden.
	        
	        if (EinheitButtons.this == _buttons2)
	        {
	        	if (_buttons1 != null)
	        		_buttons1.getButtonNachLinks().setEnabled(false);
	        }
	      }
	    });

	    _buttonNachLinks = new Button(composite, SWT.PUSH);
//	    _buttonNachLinks.setText(" << Entfernen ");
	    _buttonNachLinks.setText(" << ");
	    _buttonNachLinks.setEnabled(false);
	    _buttonNachLinks.addSelectionListener(new SelectionAdapter()
	    {
	      public void widgetSelected(SelectionEvent e)
	      {
	        actionKopieren(_rechts, _links);
	        _buttonNachLinks.setEnabled(false);

	        // Wenn in der Auswahl der mglichen Anzeigen eine Anzeige verschoben wird,
	        // so mssen danach beide Richtungen deaktiviert werden.

	        if (EinheitButtons.this == _buttons1)
	        {
	        	if (_buttons2 != null)
	        		_buttons2.getButtonNachRechts().setEnabled(false);
	        }

	      }
	    });

	    /*
	     * Layout
	     */
	    RowLayout rlUnten = new RowLayout(SWT.VERTICAL);
	    rlUnten.pack = false;
	    composite.setLayout(rlUnten);
	  }
	  
	  /**
	   * Kopiert die selektierten Eintrge von einer Einheit in die andere
	   * @param quelle Quelle
	   * @param ziel Ziel
	   */
	  private void actionKopieren (Einheit quelle, Einheit ziel)
	  {
	    IStructuredSelection selection = (IStructuredSelection) quelle.getViewer().getSelection();
	    StructuredSelection ss = (StructuredSelection) selection;

	  	for (int x = 0; x < ss.size(); ++x)
	    {
	      Object obj1 = ss.toArray()[x];

	      if (obj1 instanceof SystemObject)
	      {
	      	SystemObject as = (SystemObject) obj1;
	      	
	      	if (!ziel.contains(as))
	          ziel.add(as);
	        
	        quelle.remove(as);
	      }
	    }

	    ziel.refresh();  
	    quelle.refresh();
	  }

		/**
     * Methode liefert buttonNachRechts zurck
     * @return buttonNachRechts
     */
    public Button getButtonNachRechts()
    {
    	return _buttonNachRechts;
    }

		/**
     * Methode liefert buttonNachLinks zurck
     * @return buttonNachLinks
     */
    public Button getButtonNachLinks()
    {
    	return _buttonNachLinks;
    }
	}
}
