package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.steuerungskern;

import java.util.Vector;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse speichert einen Parametersatz "Anzeigen"
 * @author ObertM
 */
public class ParaAnzeigen
{
	/**
	 * Instanzen fr die der Wert gilt
	 */
  private Vector<String> _instanzen = new Vector<String>();
  
  /**
   * Liste mit den Anzeigen die ermittelt und versendet werden sollen
   */
  private Vector<SystemObject> _ermittelndUndVersenden = new Vector<SystemObject>();
  
  /**
   * Liste mit den Anzeigen die nur ermittelt werden
   */
  private Vector<SystemObject> _nurErmitteln = new Vector<SystemObject>();
  

  /**
   * Konstruktor der Klasse
   * @param instanzen Instanzen fr die der Parametersatz gilt
   * @param ermittelnUndVersenden Anzeigen die ermittelt und versendet werden sollen
   * @param nurErmitteln Anzeigen die nur ermittelt werden sollen
   */
  public ParaAnzeigen (Vector<String> instanzen, Vector<SystemObject> ermittelnUndVersenden, Vector<SystemObject> nurErmitteln )
  {
    _instanzen = instanzen;
    
    _ermittelndUndVersenden = ermittelnUndVersenden;
    
    _nurErmitteln = nurErmitteln;
  }

  /**
   * Konstruktor der Klasse
   * @param instanz Instanz fr die der Parametersatz gilt
   * @param ermittelnUndVersenden Anzeigen die ermittelt und versendet werden sollen
   * @param nurErmitteln Anzeigen die nur ermittelt werden sollen
   */
  public ParaAnzeigen (String instanz, Vector<SystemObject> ermittelnUndVersenden, Vector<SystemObject> nurErmitteln)
  {
  	_instanzen.add(instanz);
    
    _ermittelndUndVersenden = ermittelnUndVersenden;
    
    _nurErmitteln = nurErmitteln;
  }

  /**
   * Parameterloser Konstrukotr der Klasse
   */
	public ParaAnzeigen()
  {
  }

	/**
   * Methode liefert instanzen zurck
   * @return instanzen
   */
  public Vector<String> getInstanzen()
  {
  	return _instanzen;
  }

	/**
   * Methode setzt die Variable instanzen
   * @param instanzen zu bernehmender Wert fr instanzen
   */
  public void setInstanzen(Vector<String> instanzen)
  {
  	_instanzen = instanzen;
  }


  /**
   * Methode prft ob der Parametersatz von den Parameterwerten her gleich ist mit einem bergebenen
   * Parametersatz. Es wird nicht geprft ob die selben Instanzen in beiden Parameterstzen verwendet
   * werden
   * @param vgl Vergleichsdatensatz
   * @return Information ob der bergebene Parametersatz von den Werten her gleich ist
   */
	public boolean isWerteIdentisch (ParaAnzeigen vgl)
  {
		if (!isIdentisch(getErmittelndUndVersenden(), vgl.getErmittelndUndVersenden()))
			return false;

		if (!isIdentisch(getNurErmitteln(), vgl.getNurErmitteln()))
			return false;

	  return true;
  }
	
	/**
	 * Methode prft ob 2 Datenstze identisch sind
	 * @param ds1 Datensatz 1
	 * @param ds2 Datensatz 2
	 * @return Information ob identisch
	 */
	private boolean isIdentisch(Vector<SystemObject> ds1, Vector<SystemObject> ds2)
	{
		if (ds1 == null && ds2 == null)
			return true;
		
		if (ds1 == null || ds2 == null)
			return false;
		
		if (ds1.size() != ds2.size())
			return false;
		
		for (SystemObject so : ds1)
			if (!ds2.contains(so))
				return false;
		
	  return true;	
	}

	/**
   * Methode liefert ermittelndUndVersenden zurck
   * @return ermittelndUndVersenden
   */
  public Vector<SystemObject> getErmittelndUndVersenden()
  {
  	return _ermittelndUndVersenden;
  }

	/**
   * Methode setzt die Variable ermittelndUndVersenden
   * @param ermittelndUndVersenden zu bernehmender Wert fr ermittelndUndVersenden
   */
  public void setErmittelndUndVersenden(Vector<SystemObject> ermittelndUndVersenden)
  {
  	_ermittelndUndVersenden.clear();
  	for (SystemObject so : ermittelndUndVersenden)
  	  _ermittelndUndVersenden.add(so);
  }

	/**
   * Methode liefert nurErmitteln zurck
   * @return nurErmitteln
   */
  public Vector<SystemObject> getNurErmitteln()
  {
  	return _nurErmitteln;
  }

	/**
   * Methode setzt die Variable nurErmitteln
   * @param nurErmitteln zu bernehmender Wert fr nurErmitteln
   */
  public void setNurErmitteln(Vector<SystemObject> nurErmitteln)
  {
  	_nurErmitteln.clear();
  	for (SystemObject so : nurErmitteln)
  	  _nurErmitteln.add(so);
  }
}
