package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.steuerungskern;

import java.util.Vector;

/**
 * Klasse speichert einen Parametersatz "Aspekte"
 * @author ObertM
 */
public class ParaAspekte
{
	/**
	 * Instanzen fr die der Wert gilt
	 */
  private Vector<String> _instanzen = new Vector<String>();
  
  /**
   * Aspekte fr die der Steuerungskern Manahmenanforderungen empfngt und verarbeitet.
   */
  private Vector<String> _aspekteEingang = new Vector<String>();
  
  /**
   * Aspekt unter dem der Steuerungskern die berechneten Ergebnisse publiziert.
   */
  private String _aspektAusgang;

  /**
   * Aspekt unter dem der Steuerungskern die Diagnosedaten publiziert (opional), wenn null
   * dann werden keine Diagnosedaten versendet.
   */
  private String _aspektDiagnosedaten;

  /**
   * Konstruktor der Klasse
   * @param instanzen Instanzen fr die der Parametersatz gilt
   * @param eingang Eingangsaspekte
   * @param ausgang Ausgangsaspekt
   * @param diagnose Diagnoseaspekt
   */
  public ParaAspekte (Vector<String> instanzen, Vector<String> eingang, String ausgang, String diagnose )
  {
    _instanzen = instanzen;
    
    _aspekteEingang = eingang;
    
    _aspektAusgang = ausgang;
    
    _aspektDiagnosedaten = diagnose;
  }

  /**
   * Konstruktor der Klasse
   * @param instanz Instanz fr die der Parametersatz gilt
   * @param eingang Eingangsaspekte
   * @param ausgang Ausgangsaspekt
   * @param diagnose Diagnoseaspekt
   */
  public ParaAspekte (String instanz, Vector<String> eingang, String ausgang, String diagnose )
  {
  	_instanzen.add(instanz);
    
    _aspekteEingang = eingang;
    
    _aspektAusgang = ausgang;
    
    _aspektDiagnosedaten = diagnose;
  }

  /**
   * Parameterloser Konstrukotr der Klasse
   */
	public ParaAspekte()
  {
  }

	/**
   * Methode liefert instanzen zurck
   * @return instanzen
   */
  public Vector<String> getInstanzen()
  {
  	return _instanzen;
  }

	/**
   * Methode setzt die Variable instanzen
   * @param instanzen zu bernehmender Wert fr instanzen
   */
  public void setInstanzen(Vector<String> instanzen)
  {
  	_instanzen = instanzen;
  }

	/**
   * Methode liefert aspekteEingang zurck
   * @return aspekteEingang
   */
  public Vector<String> getAspekteEingang()
  {
  	return _aspekteEingang;
  }

	/**
   * Methode setzt die Variable aspekteEingang
   * @param aspekteEingang zu bernehmender Wert fr aspekteEingang
   */
  public void setAspekteEingang(Vector<String> aspekteEingang)
  {
  	_aspekteEingang.clear();
  	for (String as : aspekteEingang)
  	  _aspekteEingang.add (as);
  }

	/**
   * Methode liefert aspektAusgang zurck
   * @return aspektAusgang
   */
  public String getAspektAusgang()
  {
  	return _aspektAusgang;
  }

	/**
   * Methode setzt die Variable aspektAusgang
   * @param aspektAusgang zu bernehmender Wert fr aspektAusgang
   */
  public void setAspektAusgang(String aspektAusgang)
  {
  	_aspektAusgang = aspektAusgang;
  }

	/**
   * Methode liefert aspektDiagnosedaten zurck
   * @return aspektDiagnosedaten
   */
  public String getAspektDiagnosedaten()
  {
  	return _aspektDiagnosedaten;
  }

	/**
   * Methode setzt die Variable aspektDiagnosedaten
   * @param aspektDiagnosedaten zu bernehmender Wert fr aspektDiagnosedaten
   */
  public void setAspektDiagnosedaten(String aspektDiagnosedaten)
  {
  	_aspektDiagnosedaten = aspektDiagnosedaten;
  }

  /**
   * Methode prft ob der Parametersatz von den Parameterwerten her gleich ist mit einem bergebenen
   * Parametersatz. Es wird nicht geprft ob die selben Instanzen in beiden Parameterstzen verwendet
   * werden
   * @param vgl Vergleichsdatensatz
   * @return Information ob der bergebene Parametersatz von den Werten her gleich ist
   */
	public boolean isWerteIdentisch (ParaAspekte vgl)
  {
		if (!isIdentisch(getAspekteEingang(), vgl.getAspekteEingang()))
			return false;

		if (getAspektAusgang() != vgl.getAspektAusgang())
			return false;

		if (getAspektDiagnosedaten() != vgl.getAspektDiagnosedaten())
			return false;

	  return true;
  }
	
	/**
	 * Methode prft ob 2 Datenstze identisch sind
	 * @param ds1 Datensatz 1
	 * @param ds2 Datensatz 2
	 * @return Information ob identisch
	 */
	private boolean isIdentisch(Vector<String> ds1, Vector<String> ds2)
	{
		if (ds1 == null && ds2 == null)
			return true;
		
		if (ds1 == null || ds2 == null)
			return false;
		
		if (ds1.size() != ds2.size())
			return false;
		
		for (String a : ds1)
			if (!ds2.contains(a))
				return false;
		
	  return true;	
	}
}
