package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.verschiebungen.verschieben;

import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.WVZInhalt;
import de.bsvrz.sba.plugin.parameter.steuerungskern.daten.Richtung.ArbeitsRichtungVN;

/**
 * Klasse enthlt ein Feld (Liste) mit Parametern
 * @author       Dambach Werke GmbH
 */
public class ListeMitRegeln
{
  /**
   * Liste mit den Parametern
   */
  private Vector<RegelVerschieben> m_parameter = new Vector<RegelVerschieben> ();

  /**
   * Parameterloser Konstruktor der Klasse
   */
  public ListeMitRegeln ()
  {
  }
  
  /**
   * Methode liefert die Variable parameter zurck
   * @return Variable parameter
   */
  public Vector<RegelVerschieben> getParameter()
  {
    Vector<RegelVerschieben> v = new Vector<RegelVerschieben>();
    synchronized (m_parameter)
    {
      for (RegelVerschieben r : m_parameter)
        v.add( r );
    }
    return v;
  }

  /**
   * Methode setzt die Variable parameter
   * @param parameter Wert fr parameter
   */
  public void setParameter(Vector<RegelVerschieben> parameter)
  {
    synchronized (m_parameter)
    {
      m_parameter = parameter;
    } 
  }

  /**
   * Methode zum Auswerten der Regeln der Attributliste 'atl.sswSbaTrichterungProAnzeigeTyp'
   * @param data Daten vom DaV
   */
  public void auswerteAttributgruppe (Data data, Aspect schaltgrundAllgemein)
  {
    Array a = data.getArray( "Regeln" );
    int anzahl = a.getLength();
    
    synchronized (m_parameter)
    {
      m_parameter.clear();
      for (int x = 0; x < anzahl; ++x)
      {
        RegelVerschieben r = new RegelVerschieben ();
        r.auswerteAttributgruppe( a.getItem( x ), schaltgrundAllgemein );
        
        m_parameter.add( r );
      }
    }
  }

  /**
   * Methode fgt eine Regel hinzu
   * @param richtung Arbeitet Regel in Vorgnger- oder Nachfolgerrichtung
   * @param schaltgruende zu bercksichtigende Schaltgrnde
   * @param ebene Ebende in der Baumstruktur der Vorgnger/Nachfolgerbeziehung bis zu der verschoben werden soll
   * @param anzeigeTyp WZGTyp des Vorgngers/Nachfolgers in Lngsrichtung
   * @param abhaengigeAnzeigenVerschieben Sollen abhngige Anzeigen mit verschoben werden
   * @param anzeigenInhalt zu betrachtender Anzeigeinhalt
   * @param schaltgrund Schaltgrund unter dem diese Regel publiziert wird
   */
  public void addRegel (ArbeitsRichtungVN richtung,
                        List<Aspect> schaltgruende,
                        int ebene,
                        SystemObject anzeigeTyp,
                        boolean abhaengigeAnzeigenVerschieben,
                        WVZInhalt anzeigenInhalt,
                        Aspect schaltgrund) 
  {
    synchronized (m_parameter)
    {
      RegelVerschieben r = new RegelVerschieben ();
      
      r.setRichtung( richtung );
      r.setSchaltgruende( schaltgruende );
      r.setEbene( ebene );
      r.setAnzeigeTyp( anzeigeTyp );
      r.setAbhaengigeAnzeigenVerschieben( abhaengigeAnzeigenVerschieben );
      r.setAnzeigenInhalt( anzeigenInhalt );
      r.setSchaltgrundRegel( schaltgrund );
      
      m_parameter.add( r );
    }
  }

  /**
   * Methode liefert den Inahlt der Klasse als String
   */
  public String toString ()
  {
    String s = "";
    
    synchronized (m_parameter)
    {
      s += "Regeln:\n";
      for (RegelVerschieben r : m_parameter)
        s += r.toString() + "\n";
    }
    
    return s;
  }
}
