package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.vorgaengerNachfolger;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;

/**
 * Klasse erzeugt ein Composite mit denen es mglich ist ein Objekt zuzuordnen
 * @author ObertM
 */
public class AuswahlObjekt
{
	/**
	 * Parent Composite
	 */
  private Composite _parent;

  /**
   * Textuelle Beschreibung der Objekte
   */
	private String _kennung;

	/**
	 * ComboBox zur Auswahl eines Objekts
	 */
	private Combo _combo;

  /**
   * Verwaltung der Listener
   */
  private PropertyChangeSupport _pcs = new PropertyChangeSupport (this);

	private Group _groupObjekte;

	/**
	 * Konstruktor der Klasse
	 * @param parent parent Composite
	 * @param kennung Kennung die bei der Auswahl eingeblendet wird (z.B. Messquerschnitte)
	 */
  public AuswahlObjekt (Composite parent, String kennung)
  {
  	_parent = parent;
  	
  	_kennung = kennung;
  	
  	erzeugeEingabeElemente();
  }
	
	/**
	 * Methode erzeugt die Eingabeelemente
	 */
  private void erzeugeEingabeElemente()
  {
    GridData gridDataGroup = new GridData();
    gridDataGroup.horizontalAlignment = GridData.FILL;
    gridDataGroup.grabExcessHorizontalSpace = true;

    _groupObjekte = new Group(_parent, SWT.NONE);
    _groupObjekte.setText(_kennung);
    _groupObjekte.setLayout(new GridLayout(1, false));
    _groupObjekte.setLayoutData(gridDataGroup);
    
    GridData gridDataComboBox = new GridData();
    gridDataComboBox.horizontalAlignment = GridData.FILL;
    gridDataComboBox.grabExcessHorizontalSpace = true;
    gridDataComboBox.verticalAlignment = GridData.CENTER;
    
    _combo = new Combo( _groupObjekte, SWT.BORDER );
    _combo.setLayoutData( gridDataComboBox );
    _combo.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent e)
			{
				benachrichtigeListener();
			}
			
			@Override
			public void widgetDefaultSelected(SelectionEvent e)
			{
			}
		});
  }

  /**
   * Methode definiert die mglichen Objekte
   * @param moeglicheObjekte mgliche Objekte
   */
  public void setMoeglicheObjekte (List<String> moeglicheObjekte)
  {
	  String[] t = new String [moeglicheObjekte.size()];
	  for (int i=0; i<moeglicheObjekte.size(); ++i)
	  	t[i] = moeglicheObjekte.get(i);
	  	
	  _combo.removeAll();
	  _combo.setItems(t);
	  _combo.select(0);
  }

  /**
   * Methode setzt das angewhlte Objekt
   * @param moeglicheObjekte Liste mit den mglichen objekte
   * @param angewaehltesObjekt angewhltes Objekt
   */
  public void setAngewaehltesObjekt (List<String> moeglicheObjekte, String angewaehltesObjekt)
  {
  	/*
  	 * Prfen ob das angewhlte Objekt auch in der Liste der mglichen Objekte vorhanden ist.
  	 * Wenn nicht wird es hinzugefgt 
  	 */
  	
  	boolean vorhanden = false;
		for (String s1 : moeglicheObjekte)
  		if (s1.equals(angewaehltesObjekt))
  			vorhanden = true;
		
		if (!vorhanden)
			moeglicheObjekte.add(angewaehltesObjekt);
  	
  	setMoeglicheObjekte(moeglicheObjekte);
  	
    for (int k = 0; k < _combo.getItemCount(); k++)
    {
      if (angewaehltesObjekt.equals(_combo.getItem( k )))
        _combo.select(k);      
    }
  }

  /**
   * Methode liefert das angewhlter Objekt
   * @return angewhltes Objekt
   */
  public String getAusgewaehltesObjekt ()
  {
  	if (_combo.getSelectionIndex() >= 0)
      return _combo.getItem( _combo.getSelectionIndex() );
  	
  	return null;
  }
  
  /**
   * Methode fgt einen Listener der Klasse hinzu
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void addPropertyChangeListener (PropertyChangeListener pcl)
  {
    _pcs.addPropertyChangeListener( pcl );  
    benachrichtigeListener();
  }
  
  /**
   * Methode entfernt einen Listener aus der Klasse
   * @param pcl Listern der Klasse PropertyChangeListener
   */
  public void removePropertyChangeListener (PropertyChangeListener pcl)
  {
    _pcs.removePropertyChangeListener( pcl );    
  }

  /**
   * Methode benachrichtigt die Listener
   */
  private void benachrichtigeListener ()
  {
    _pcs.firePropertyChange( "Selektion", null, null );
  }

	public void setAngewaehltesObjekt(String angewaehltesObjekt)
  {
    for (int k = 0; k < _combo.getItemCount(); k++)
    {
      if (angewaehltesObjekt.equals(_combo.getItem( k )))
        _combo.select(k);      
    }
  }

	public void setVisible (boolean visible)
	{
    _groupObjekte.setVisible(visible);		
	}
	
	public void dispose ()
	{
		try
		{
      _combo.dispose();
      _groupObjekte.dispose();
		}
		catch (Exception e) 
		{
			e.printStackTrace();
		}
	}

	public void setEnable(boolean enabled)
  {
		_groupObjekte.setEnabled(enabled);
		Control[] children = _groupObjekte.getChildren();
		
		for (Control c : children)
			c.setEnabled(enabled);
  }
}
