/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.vorgaengerNachfolger;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.sba.plugin.parameter.steuerungskern.dav.DaVKommunikationSBA;
import de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen.ProjektSpezifischeEinstellungen;

/**
 * Klasse verwaltet die Parameter Vorgaenger/Nachfolger
 * @author ObertM
 *
 */
public class VerwaltungParaVorgaengerNachfolger
{
	/**
	 * Einzige Instanz der Klasse
	 */
  private static VerwaltungParaVorgaengerNachfolger _instance;

	/**
   * Pid der Attributgruppe
   */
  private String _pidAtg = "atg.sswSbaVorgngerNachfolgerBeziehung";
  
  /**
   * Parametersatz
   */
  private ParameterVorgaengerNachfolger _parameter = new ParameterVorgaengerNachfolger();
  
  /**
   * Pid der SBA
   */
	private String _pidSBA;

  /**
   * Privater Konstruktor der Klasse
   */
  private VerwaltungParaVorgaengerNachfolger ()
  {
  	_pidSBA = ProjektSpezifischeEinstellungen.getPidSba();

    /*
     * Daten lesen ber DaVKommunikation
     */
  	final DaVKommunikationSBA kom = DaVKommunikationSBA.getInstanz();
  	kom.addDatenArt(_pidSBA, _pidAtg, "asp.parameterSoll", ReceiverRole.receiver());
  	
  	kom.addPropertyChangeListener(new PropertyChangeListener()
		{
			@Override
			public void propertyChange(PropertyChangeEvent evt)
			{
				if (evt.getPropertyName().equals(_pidAtg))
				{
					ResultData resultData = kom.getData(_pidAtg);
					if (resultData != null)
						_parameter.auswerteAttributgruppe(resultData);
				}
			}
		});

		ResultData resultData = kom.getData(_pidAtg);
		if (resultData != null)
			_parameter.auswerteAttributgruppe(resultData);
  }
  
  /**
   * Methode liefert die Singleton Instanz der Klasse
   * @return Singleton Instanz der Klasse
   */
  public static VerwaltungParaVorgaengerNachfolger getInstance ()
  {
    if (_instance == null)
    {
      synchronized (VerwaltungParaVorgaengerNachfolger.class)
      {
        if (_instance == null)
        {
          _instance = new VerwaltungParaVorgaengerNachfolger ();
        }
      }
    }
    
    return _instance;
  }

  /**
   * Auswerten der vom DaV empfangenen Attributgruppen
   */
  public void auswerteAttributgruppen( ResultData[] results )
  {
    int anzahl = results.length;

    // Daten aus Eingangsqueue bearbeiten      
    
    for (int i = 0; i < anzahl; i++)
    {
      ResultData dat = results[i];
  
      // Parameter Iterationen
      
      if (dat.getDataDescription().getAttributeGroup().getPid().equals(_pidAtg))
        _parameter.auswerteAttributgruppe (dat);

    } // for (int i = 0; i < arg0.length; i++)
  }

	/**
   * Methode liefert parameter zurck
   * @return parameter
   */
  public ParameterVorgaengerNachfolger getParameter()
  {
  	return _parameter;
  }

	/**
   * Methode setzt die Variable parameter
   * @param parameter zu bernehmender Wert fr parameter
   */
  public void setParameter(ParameterVorgaengerNachfolger parameter)
  {
  	_parameter = parameter;
  }
}
