package de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungMoeglicheAnzeigeQuerschnitte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.abhaengigkeiten.Abhaengigkeiten;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;

/**
 * Klasse definiert die Defaulteinstellungen fr die A2. Diese Einstellungen werden nur
 * verwendet, wenn ber den Datenverteiler keine anderen Parameter versorgt sind
 * (atg.sswSbaAnzeigeTypMitWVZInhalt).
 * 
 * @author ObertM
 * 
 */
public class DefaultEinstellungenA2 implements IProjektEinstellungen
{
	/**
	 * Prfix mit der die Pid der AnzeigeTypen (typ.sswSbaAnzeigeTyp) beginnen
	 */
	private static final String _praefixAnzeigeTypen = "sba.anzeigeTyp.A2";

	/**
	 * Prfix mit der die Pid der WvzInhalte (typ.wvzInhalt) beginnen
	 */
	private static final String _praefixWvzInhalte = "wvzInhalt.uz.a2";

	/**
	 * Prfix mit der die Pid der WvzInhalte typ.anzeigeQuerschnitt) beginnen
	 */
	private static final String _praefixAnzeigeQuerschnitt = "aq.A2";

	/**
	 * Methode liefert eine Vector mit den im Projekt zu verwendenden Anzeigetypen
	 * @return Liste mit Anzeigetypen
	 */
	public Vector<AnzeigeTypEigenschaften> getDefaultAnzeigeTypen()
	{
		Vector<AnzeigeTypEigenschaften> l = new Vector<AnzeigeTypEigenschaften>();

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_A,
		    _praefixAnzeigeTypen + ".TypA", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_WZG_A)));

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_B,
		    _praefixAnzeigeTypen + ".TypB", _praefixWvzInhalte + ".Dunkel",
        getWvzInhalte(EAnzeigeTypen.TYP_WZG_B)));

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_WZG_C,
		    _praefixAnzeigeTypen + ".TypC", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_WZG_C)));

		l.add(new AnzeigeTypEigenschaften(EAnzeigeTypen.TYP_BAG,
		    _praefixAnzeigeTypen + ".TypBAG", _praefixWvzInhalte + ".Dunkel",
		    getWvzInhalte(EAnzeigeTypen.TYP_BAG)));

		return l;
	};

	/**
	 * Methode liefert die Pids der zu verwendenden WvzInhalte (typ.wvzInhalt) in
	 * Abhngigkeit des Anzeigetyps.
	 * 
	 * @param typ
	 *          Anzeigetyp
	 * @return Liste mit Pids
	 */
	private List<String> getWvzInhalte(EAnzeigeTypen typ)
	{
		String praefix = _praefixWvzInhalte;

		List<String> list = new ArrayList<String>();

		if (typ == EAnzeigeTypen.TYP_WZG_A)
		{
			list.add(praefix + ".Dunkel");
			list.add(praefix + ".tlsFahrstreifenGesperrt");
			list.add(praefix + ".tlsFahrstreifenWechselnNachLinks");
			list.add(praefix + ".tlsFahrstreifenWechselnNachRechts");
			list.add(praefix + ".tlsZulaessigeHoechst40");
			list.add(praefix + ".tlsZulaessigeHoechst60");
			list.add(praefix + ".tlsZulaessigeHoechst80");
			list.add(praefix + ".tlsZulaessigeHoechst100");
			list.add(praefix + ".tlsZulaessigeHoechst120");
			
			list.add(praefix + ".tlsEndeAllerStreckenverbote");
		} 
		else if (typ == EAnzeigeTypen.TYP_WZG_B)
		{
			list.add(praefix + ".Dunkel");
			list.add(praefix + ".tlsGefahrenstelle");
			list.add(praefix + ".tlsSchleudergefahr");
			list.add(praefix + ".tlsBaustelle");
			list.add(praefix + ".tlsStau");
			list.add(praefix + ".tlsUeberholverbotAllgemein");
			list.add(praefix + ".tlsUeberholverbotLkw");
			list.add(praefix + ".tlsEndeUeberholverbotAllgemein");
			list.add(praefix + ".tlsEndeUeberholverbotLkw");
			list.add(praefix + ".tlsNurLkwUndKraftomnibus");
		} 
		else if (typ == EAnzeigeTypen.TYP_WZG_C)
		{
			list.add(praefix + ".Dunkel");
			list.add(praefix + ".tlsStau");
			list.add(praefix + ".tlsNebel");
			list.add(praefix + ".tlsUnfall");
			list.add(praefix + ".tls500Meter");
			list.add(praefix + ".tls1000Meter");
			list.add(praefix + ".tls1500Meter");
			list.add(praefix + ".tls2000Meter");
			list.add(praefix + ".tls7Komma5Tonnen");
		}

		else if (typ == EAnzeigeTypen.TYP_BAG)
		{
			list.add(praefix + ".Dunkel");
			list.add(praefix + ".tlsNurLkwUndKraftomnibus");
		}
		
		return list;
	}

	@Override
	public String getPidBeibehalten()
	{
		return _praefixWvzInhalte + ".Beibehalten";
	}

	@Override
	public String getPidBeliebig()
	{
		return _praefixWvzInhalte + ".Beliebig";
	}

	@Override
	public String getPidDunkel()
	{
		return _praefixWvzInhalte + ".Dunkel";
	}

	@Override
  public String getPidDontCare()
  {
		return _praefixWvzInhalte + ".DontCare";
  }
	
	@Override
  public HashMap<String, EAnzeigeQuerschnittTyp> getDefaultAnzeigeQuerschnitte ()
  {
  	HashMap<String, EAnzeigeQuerschnittTyp> zuordnungen = new HashMap<String, EAnzeigeQuerschnittTyp>();

  	// Liste aller Anzeigequerschnitte
  	
  	List<SystemObject> objekte = Tools.getSystemObjekte("typ.anzeigeQuerschnitt");
  	for (SystemObject so : objekte)
  		if (so.getPid().startsWith(_praefixAnzeigeQuerschnitt))
  			zuordnungen.put(so.getPid(), EAnzeigeQuerschnittTyp.A14_TYP1);

  	return zuordnungen;
  }

	@Override
  public Vector<Abhaengigkeiten> getDefaultAbhaengigkeiten(SystemObject soAQ)
  {
  	EAnzeigeQuerschnittTyp typ = VerwaltungMoeglicheAnzeigeQuerschnitte.getAnzeigeQuerschnittTyp(soAQ);
  	
  	SystemObject[] anzeigen = Tools.getSystemObjekteAnzeigen(soAQ);
  	
		Vector<Abhaengigkeiten> v = new Vector<Abhaengigkeiten>();

  	if ((typ == EAnzeigeQuerschnittTyp.A2_TYP_ABC ))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [1], anzeigen [2])); // B/C
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A2_TYPBAG) ||
  			(typ == EAnzeigeQuerschnittTyp.A2_TYP_AB))
  	{
  		// keine B/C Kombination
  	}
  	
  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP1 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP6 ) ||
   			(typ == EAnzeigeQuerschnittTyp.A8_TYP6d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [4], anzeigen [6])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [3], anzeigen [5])); // B/C rechts
  	}
  	
  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP2 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP7 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP7d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [6], anzeigen [9])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [5], anzeigen [8])); // B/C mitte
  		v.add (new Abhaengigkeiten(anzeigen [4], anzeigen [7])); // B/C rechts
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP3) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP8))
  	{
  		// keine B/C Kombination
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP4) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP9))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [5], anzeigen [7])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [4], anzeigen [6])); // B/C rechts
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP5  ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP10 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP10d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [ 8], anzeigen [12])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [ 7], anzeigen [11])); // B/C mitte links
  		v.add (new Abhaengigkeiten(anzeigen [ 6], anzeigen [10])); // B/C mitte rechts
  		v.add (new Abhaengigkeiten(anzeigen [ 5], anzeigen [ 9])); // B/C rechts
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP11 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP11d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [5], anzeigen [4])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [7], anzeigen [6])); // B/C rechts
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP12 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP12d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [ 7], anzeigen [10])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [ 6], anzeigen [ 9])); // B/C mitte
  		v.add (new Abhaengigkeiten(anzeigen [ 5], anzeigen [ 8])); // B/C rechts
  	}

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP13 )
  	{
  		v.add (new Abhaengigkeiten(anzeigen [ 2], anzeigen [ 3])); // B/C 
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP14 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP14d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [ 6], anzeigen [ 9])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [ 5], anzeigen [ 8])); // B/C mitte
  		v.add (new Abhaengigkeiten(anzeigen [ 4], anzeigen [ 7])); // B/C rechts
  	}

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP15 )
  	{
  		v.add (new Abhaengigkeiten(anzeigen [ 6], anzeigen [ 9])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [ 5], anzeigen [ 8])); // B/C mitte
  		v.add (new Abhaengigkeiten(anzeigen [ 4], anzeigen [ 7])); // B/C rechts
  	}

  	return v;
  }

	@Override
  public SystemObject[] getDefaultQuerBeziehung(SystemObject soAQ)
  {
  	EAnzeigeQuerschnittTyp typ = VerwaltungMoeglicheAnzeigeQuerschnitte.getAnzeigeQuerschnittTyp(soAQ);
  	
  	SystemObject[] anzeigen = Tools.getSystemObjekteAnzeigen(soAQ);

  	Vector<SystemObject> reihenfolge = new Vector<SystemObject>(); 
  	   
  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP1 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP6 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP6d))
  	{
  		reihenfolge.add (anzeigen [2]); // A links
  		reihenfolge.add (anzeigen [4]); // B links
  		reihenfolge.add (anzeigen [6]); // C links
  		reihenfolge.add (anzeigen [1]); // A mitte
  		reihenfolge.add (anzeigen [3]); // B rechts
  		reihenfolge.add (anzeigen [5]); // C rechts
  		reihenfolge.add (anzeigen [0]); // A rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP6d)
    		reihenfolge.add (anzeigen [7]); // D2
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP2 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP7 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP7d))
  	{
  		reihenfolge.add (anzeigen [3]); // A links
  		reihenfolge.add (anzeigen [6]); // B links
  		reihenfolge.add (anzeigen [9]); // C links
  		reihenfolge.add (anzeigen [2]); // A mitte links
  		reihenfolge.add (anzeigen [5]); // B mitte
  		reihenfolge.add (anzeigen [8]); // C mitte
  		reihenfolge.add (anzeigen [1]); // A mitte rechts
  		reihenfolge.add (anzeigen [4]); // B rechts
  		reihenfolge.add (anzeigen [7]); // C rechts
  		reihenfolge.add (anzeigen [0]); // A rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP7d)
    		reihenfolge.add (anzeigen [10]); // D2
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP3) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP8))
  	{
  		reihenfolge.add (anzeigen [0]); // A
  	}

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP4) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP9))
  	{
  		reihenfolge.add (anzeigen [3]); // A links
  		reihenfolge.add (anzeigen [5]); // B links
  		reihenfolge.add (anzeigen [7]); // C links
  		reihenfolge.add (anzeigen [2]); // A mitte links
  		
  		reihenfolge.add (anzeigen [1]); // A mitte rechts
  		reihenfolge.add (anzeigen [4]); // B rechts
  		reihenfolge.add (anzeigen [6]); // C rechts
  		reihenfolge.add (anzeigen [0]); // A rechts
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP5  ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP10 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP10d))
  	{
  		reihenfolge.add (anzeigen [ 4]); // A links
  		reihenfolge.add (anzeigen [ 8]); // B links
  		reihenfolge.add (anzeigen [12]); // C links
  		reihenfolge.add (anzeigen [ 3]); // A mitte links
  		reihenfolge.add (anzeigen [ 7]); // B mitte links
  		reihenfolge.add (anzeigen [11]); // C mitte links
  		reihenfolge.add (anzeigen [ 2]); // A mitte 
  		reihenfolge.add (anzeigen [ 6]); // B mitte rechts
  		reihenfolge.add (anzeigen [10]); // C mitte rechts
  		reihenfolge.add (anzeigen [ 1]); // A mitte rechts
  		reihenfolge.add (anzeigen [ 5]); // B rechts
  		reihenfolge.add (anzeigen [ 9]); // C rechts
  		reihenfolge.add (anzeigen [ 0]); // A rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP10d)
    		reihenfolge.add (anzeigen [13]); // D2
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP11 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP11d))
  	{
  		reihenfolge.add (anzeigen [ 3]); // A links
  		reihenfolge.add (anzeigen [ 5]); // B links
  		reihenfolge.add (anzeigen [ 7]); // C links
  		reihenfolge.add (anzeigen [ 2]); // A mitte links
  		reihenfolge.add (anzeigen [ 4]); // B rechts
  		reihenfolge.add (anzeigen [ 6]); // C rechts
  		reihenfolge.add (anzeigen [ 1]); // A mitte rechts 
  		reihenfolge.add (anzeigen [ 0]); // A rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP11d)
    		reihenfolge.add (anzeigen [8]); // D2
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP12 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP12d))
  	{
  		reihenfolge.add (anzeigen [ 4]); // A links
  		reihenfolge.add (anzeigen [ 7]); // B links
  		reihenfolge.add (anzeigen [10]); // C links
  		reihenfolge.add (anzeigen [ 3]); // A mitte links
  		reihenfolge.add (anzeigen [ 6]); // B mitte
  		reihenfolge.add (anzeigen [ 9]); // C mitte
  		reihenfolge.add (anzeigen [ 2]); // A mitte
  		reihenfolge.add (anzeigen [ 1]); // A mitte rechts
  		reihenfolge.add (anzeigen [ 5]); // B rechts
  		reihenfolge.add (anzeigen [ 8]); // C rechts
  		reihenfolge.add (anzeigen [ 0]); // A rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP12d)
    		reihenfolge.add (anzeigen [11]); // D2
    };

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP13)
  	{
  		reihenfolge.add (anzeigen [1]); // A links
  		reihenfolge.add (anzeigen [2]); // B 
  		reihenfolge.add (anzeigen [3]); // C 
  		reihenfolge.add (anzeigen [0]); // A mitte
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP14 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP14d))
  	{
  		reihenfolge.add (anzeigen [12]); // TAQ links
  		reihenfolge.add (anzeigen [ 3]); // A links
  		reihenfolge.add (anzeigen [ 6]); // B links
  		reihenfolge.add (anzeigen [ 9]); // C links
  		reihenfolge.add (anzeigen [ 2]); // A mitte links
  		reihenfolge.add (anzeigen [ 5]); // B mitte
  		reihenfolge.add (anzeigen [ 8]); // C mitte
  		reihenfolge.add (anzeigen [ 1]); // A mitte rechts
  		reihenfolge.add (anzeigen [ 4]); // B rechts
  		reihenfolge.add (anzeigen [ 7]); // C rechts
  		reihenfolge.add (anzeigen [ 0]); // A rechts
  		reihenfolge.add (anzeigen [10]); // D
  		reihenfolge.add (anzeigen [11]); // TAQ rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP14d)
    		reihenfolge.add (anzeigen [13]); // D2
    };

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP15 ) 
  	{
  		reihenfolge.add (anzeigen [14]); // Blinker links
  		reihenfolge.add (anzeigen [12]); // TAQ links
  		reihenfolge.add (anzeigen [ 3]); // A links
  		reihenfolge.add (anzeigen [ 6]); // B links
  		reihenfolge.add (anzeigen [ 9]); // C links
  		reihenfolge.add (anzeigen [ 2]); // A mitte links
  		reihenfolge.add (anzeigen [ 5]); // B mitte
  		reihenfolge.add (anzeigen [ 8]); // C mitte
  		reihenfolge.add (anzeigen [ 1]); // A mitte rechts
  		reihenfolge.add (anzeigen [ 4]); // B rechts
  		reihenfolge.add (anzeigen [ 7]); // C rechts
  		reihenfolge.add (anzeigen [ 0]); // A rechts
  		reihenfolge.add (anzeigen [10]); // D
  		reihenfolge.add (anzeigen [13]); // Blinker rechts
  		reihenfolge.add (anzeigen [11]); // TAQ rechts
    };

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP16 ) 
  	{
  		reihenfolge.add (anzeigen [ 2]); // Blinker links
  		reihenfolge.add (anzeigen [ 4]); // TAQ links
  		reihenfolge.add (anzeigen [ 1]); // Blinker mitte
  		reihenfolge.add (anzeigen [ 5]); // WAQ
  		reihenfolge.add (anzeigen [ 0]); // Blinker rechts
  		reihenfolge.add (anzeigen [ 3]); // TAQ rechts
    };

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP17 ) 
  	{
  		reihenfolge.add (anzeigen [ 0]); // Blinker 
  		reihenfolge.add (anzeigen [ 1]); // WAQ
    };

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP18 ) 
  	{
  		reihenfolge.add (anzeigen [ 1]); // Blinker links 
  		reihenfolge.add (anzeigen [ 3]); // TAQ links
  		reihenfolge.add (anzeigen [ 0]); // Blinker rechts
  		reihenfolge.add (anzeigen [ 2]); // TAQ rechts
    };

  	if (typ == EAnzeigeQuerschnittTyp.A8_TYP19 ) 
  	{
  		reihenfolge.add (anzeigen [ 1]); // BAQ links 
  		reihenfolge.add (anzeigen [ 0]); // BAQ rechts
    };

    return reihenfolge.toArray(new SystemObject[reihenfolge.size()]);
  }
}
