package de.bsvrz.sba.plugin.parameter.steuerungskern.projektspezifischeEinstellungen;

import java.util.HashMap;
import java.util.Vector;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.VerwaltungMoeglicheAnzeigeQuerschnitte;
import de.bsvrz.sba.plugin.parameter.steuerungskern.parameter.abhaengigkeiten.Abhaengigkeiten;
import de.bsvrz.sba.plugin.parameter.steuerungskern.tools.Tools;


/**
 * Klasse definiert die Defaulteinstellungen fr die A81. Diese Einstellungen werden nur
 * verwendet, wenn ber den Datenverteiler keine anderen Parameter versorgt sind
 * (atg.sswSbaAnzeigeTypMitWVZInhalt).
 * 
 * @author ObertM
 * 
 */
public class DefaultEinstellungenA81 implements IProjektEinstellungen
{
	/**
	 * Methode liefert eine Vector mit den im Projekt zu verwendenden Anzeigetypen
	 * @return Liste mit Anzeigetypen
	 */
	public Vector<AnzeigeTypEigenschaften> getDefaultAnzeigeTypen()
	{
		Vector<AnzeigeTypEigenschaften> l = new Vector<AnzeigeTypEigenschaften>();
		return l;
	};

	@Override
  public String getPidBeibehalten()
  {
	  return null;
  }

	@Override
  public String getPidBeliebig()
  {
	  return null;
  }

	@Override
  public String getPidDunkel()
  {
	  return null;
  }

	@Override
  public String getPidDontCare()
  {
		return null;
  }

	@Override
  public HashMap<String, EAnzeigeQuerschnittTyp> getDefaultAnzeigeQuerschnitte ()
  {
  	HashMap<String, EAnzeigeQuerschnittTyp> zuordnungen = new HashMap<String, EAnzeigeQuerschnittTyp>();
  	return zuordnungen;
  }

	@Override
  public Vector<Abhaengigkeiten> getDefaultAbhaengigkeiten(SystemObject soAQ)
  {
  	EAnzeigeQuerschnittTyp typ = VerwaltungMoeglicheAnzeigeQuerschnitte.getAnzeigeQuerschnittTyp(soAQ);
  	
  	SystemObject[] anzeigen = Tools.getSystemObjekteAnzeigen(soAQ);
  	
		Vector<Abhaengigkeiten> v = new Vector<Abhaengigkeiten>();

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP6 ) ||
   			(typ == EAnzeigeQuerschnittTyp.A8_TYP6d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [4], anzeigen [6])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [3], anzeigen [5])); // B/C rechts
  	}
  	
  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP7 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP7d))
  	{
  		v.add (new Abhaengigkeiten(anzeigen [6], anzeigen [9])); // B/C links
  		v.add (new Abhaengigkeiten(anzeigen [5], anzeigen [8])); // B/C mitte
  		v.add (new Abhaengigkeiten(anzeigen [4], anzeigen [7])); // B/C rechts
  	}

  	return v;
  }

	@Override
  public SystemObject[] getDefaultQuerBeziehung(SystemObject soAQ)
  {
  	EAnzeigeQuerschnittTyp typ = VerwaltungMoeglicheAnzeigeQuerschnitte.getAnzeigeQuerschnittTyp(soAQ);
  	
  	SystemObject[] anzeigen = Tools.getSystemObjekteAnzeigen(soAQ);

  	Vector<SystemObject> reihenfolge = new Vector<SystemObject>(); 
  	   
  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP6 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP6d))
  	{
  		reihenfolge.add (anzeigen [2]); // A links
  		reihenfolge.add (anzeigen [4]); // B links
  		reihenfolge.add (anzeigen [6]); // C links
  		reihenfolge.add (anzeigen [1]); // A mitte
  		reihenfolge.add (anzeigen [3]); // B rechts
  		reihenfolge.add (anzeigen [5]); // C rechts
  		reihenfolge.add (anzeigen [0]); // A rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP6d)
    		reihenfolge.add (anzeigen [7]); // D2
    };

  	if ((typ == EAnzeigeQuerschnittTyp.A8_TYP7 ) ||
  			(typ == EAnzeigeQuerschnittTyp.A8_TYP7d))
  	{
  		reihenfolge.add (anzeigen [3]); // A links
  		reihenfolge.add (anzeigen [6]); // B links
  		reihenfolge.add (anzeigen [9]); // C links
  		reihenfolge.add (anzeigen [2]); // A mitte links
  		reihenfolge.add (anzeigen [5]); // B mitte
  		reihenfolge.add (anzeigen [8]); // C mitte
  		reihenfolge.add (anzeigen [1]); // A mitte rechts
  		reihenfolge.add (anzeigen [4]); // B rechts
  		reihenfolge.add (anzeigen [7]); // C rechts
  		reihenfolge.add (anzeigen [0]); // A rechts
  		
  		if (typ == EAnzeigeQuerschnittTyp.A8_TYP7d)
    		reihenfolge.add (anzeigen [10]); // D2
    };

    return reihenfolge.toArray(new SystemObject[reihenfolge.size()]);
  }
}
