package de.bsvrz.sba.plugin.parameter.steuerungskern.view.allgemein;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.WorkbenchPage;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.sba.plugin.parameter.steuerungskern.Activator;
import de.bsvrz.sba.plugin.parameter.steuerungskern.konfig.Konfig;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * View zur Auswahl der Parameter<br>
 * - anlagenspezifische Parameter<br>
 * - anzeigequerschnit spezifische Parameter<br>
 * - anzeigenspezifische Parameter<br>
 */
public class ViewAuswahlParameter extends ViewPart 
{
  public static String ID = "de.bsvrz.sba.plugin.parameter.steuerungskern.view.allgemien.ViewAuswahlParameter";

  /**
   * Debugger
   */
  private Debug _debug = Debug.getLogger();
  
  /**
   * Parent Composite
   */
  private Composite _parent;
  
  /**
   * Oberes Composite
   */
  private Composite compositeTop;

  /**
   * Unteres Composite
   */
  private Composite compositeBottom;

  /**
   * Button Abbrechen
   */
  private Button buttonAbbrechen;

  /**
   * Button Hilfe
   */
  private Button buttonHilfe;

  /**
   * Pid der SBA, wird verwendet als Kennung fr die anlagenspezifischen Parameter
   */
  private String _pidSBA = null;

  /**
   * Sekundr ID des Views
   */
	private String _secondViewId;

	/**
	 * Button Parameter fr Anlage
	 */
	private Button _buttonAnlage;

	/**
	 * Button Parameter fr Anzeigequerschnitt
	 */
	private Button _buttonAnzeigeQuerschnitt;

	/**
	 * Button fr Parameter Anzeige
	 */
	private Button _buttonAnzeige;
  
  @Override
  public void createPartControl(Composite parent)
  {
    _parent = parent;
    
    _pidSBA = Konfig.getInstance().getPidSBA();

    //Fr die F1 Taste
    
    PlatformUI.getWorkbench().getHelpSystem().setHelp( _parent, Activator.PLUGIN_ID + ".auswahl" );

    _parent.setLayout(new GridLayout());
    
    this.setPartName("Parameter " + _secondViewId);

    createCompositeTop(_parent);
    createCompositeBottom(_parent);
  }

  @Override
  public void setFocus()
  {
    if (_parent != null)
      _parent.setFocus();
  }

  /**
   * Methode erzeugt das obere Composite
   * @param parent Parent Composite
   */
  private void createCompositeTop (Composite parent)
  {
    GridData gridData = new GridData();
    gridData.horizontalAlignment = GridData.FILL;
    gridData.grabExcessVerticalSpace = true;
    gridData.grabExcessHorizontalSpace = true;
    gridData.verticalAlignment = GridData.FILL;

    compositeTop = new Composite(parent, SWT.NONE);
    compositeTop.setLayout(new GridLayout());
    compositeTop.setLayoutData(gridData);

    GridData gdButton = new GridData();
    gdButton.horizontalAlignment = GridData.CENTER;
    gdButton.grabExcessHorizontalSpace = true;
    gdButton.minimumWidth = 300;

    /*
     * Button anlagespezifische Parameter
     */
    _buttonAnlage = new Button(compositeTop, SWT.NONE);
    _buttonAnlage.setText("Anlagen spezifische Parameter");
    _buttonAnlage.setLayoutData(gdButton);
    _buttonAnlage.addSelectionListener(new SelectionListener()
    {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionParameterAnlage ();
      }

      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }
    });
    
    /*
     * Button anzeigequerschnittspezifische Parameter
     */
    _buttonAnzeigeQuerschnitt = new Button(compositeTop, SWT.NONE);
    _buttonAnzeigeQuerschnitt.setText("Anzeigequerschnitt spezifische Parameter");
    _buttonAnzeigeQuerschnitt.setLayoutData(gdButton);
    _buttonAnzeigeQuerschnitt.addSelectionListener(new SelectionListener()
    {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionParameterAnzeigeQuerschnitt();
      }
      
      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }
    });
    
    /*
     * Parameter anlagespezifische Parameter
     */
    
    _buttonAnzeige = new Button(compositeTop, SWT.NONE);
    _buttonAnzeige.setText("Anzeigen spezifische Parameter");
    _buttonAnzeige.setLayoutData(gdButton);
    _buttonAnzeige.addSelectionListener(new SelectionListener()
    {
      @Override
      public void widgetSelected(SelectionEvent e)
      {
        aktionParameterAnzeige();
      }
      
      @Override
      public void widgetDefaultSelected(SelectionEvent e)
      {
      }
    });
    
  }

  /**
   * Methode ffnet die View zum Versorgen der Anlagenspezifischen Parameter
   */
  private void aktionParameterAnlage() 
  {
  	System.out.println("ViewAuswahlParameter.aktionParameterAnlage() " +_secondViewId);
  	/*
  	 * Sonderfall allgemeine Verriegelungsdmatrix, hier wird noch nach Typen unterschieden
  	 */
  	if (_secondViewId.equals(OpenVersorgungsViews._allgemeineVerriegelungsMatrix))
  		OpenViewAuswahlAnzeigeQuerschnittTypen.openView(_secondViewId);
  	else
  	  OpenVersorgungsViews.openView (_secondViewId, _pidSBA);
  }

  /**
   * Methode ffnet die View zum Versorgen der Anzeigequerschnitt spezifischen Parameter
   */
  private void aktionParameterAnzeigeQuerschnitt ()
  {
  	OpenViewAuswahlAnzeigeQuerschnitte.openView(_secondViewId);
  }

  /**
   * Methode ffnet die View zum Versorgen der Anzeigen spezifischen Parameter
   */
  private void aktionParameterAnzeige ()
  {
  	OpenViewAuswahlAnzeigen.openView(_secondViewId);
  }


  /**
   * Methode erzeugt das Composite mit den Buttons
   * @param parent Parent Composite
   */
  private void createCompositeBottom(Composite parent)
  {
    RowLayout rowLayout = new RowLayout();
    rowLayout.pack = false;
    GridData gridData3 = new GridData();
    gridData3.horizontalAlignment = GridData.END;
    gridData3.grabExcessHorizontalSpace = true;
    gridData3.verticalAlignment = GridData.CENTER;
    
    compositeBottom = new Composite(parent, SWT.NONE);
    compositeBottom.setLayoutData(gridData3);
    compositeBottom.setLayout(rowLayout);
    
    buttonAbbrechen = new Button(compositeBottom, SWT.NONE);
    buttonAbbrechen.setText("Abbrechen");
    buttonAbbrechen.addMouseListener(new MouseAdapter() 
    {
      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionAbbrechen();
      }

    });
    
    buttonHilfe = new Button(compositeBottom, SWT.NONE);
    buttonHilfe.setText("Hilfe");
    buttonHilfe.addMouseListener(new MouseAdapter() {

      @Override
      public void mouseDown(MouseEvent e)
      {
        super.mouseDown(e);
        aktionHilfe();
      }

    });
  }

  protected void aktionAbbrechen()
  {
  	try
  	{
      PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().hideView(this);
  	}
  	catch (Exception e) 
  	{
		}
  }

  /**
   * Zeigt die Hilfe zum Dialog an.
   * 
   */
  protected void aktionHilfe()
  {
    String contextId = Activator.PLUGIN_ID + ".auswahl";
    _debug.fine("ContextID: " + contextId);
    
    PlatformUI.getWorkbench().getHelpSystem().displayHelp(contextId);
  }
  
  @Override
  public void init( IViewSite site, IMemento memento ) throws PartInitException
  {
    _secondViewId = site.getSecondaryId();
    super.init( site, memento );
  }

  /**
   * Methode zum Enablen/Disablen der einzelnen Buttons (je nach Parameterart)
   * @param auswahl Auswahl welche Buttons angeboten werden (Veroderung von {@link PA}
   */
  public void personalisiereView (int auswahl)
  {
	  _buttonAnlage.setEnabled( (auswahl & PA.ANLAGE) == PA.ANLAGE );
	  _buttonAnzeigeQuerschnitt.setEnabled( (auswahl & PA.QUERSCHNITT) == PA.QUERSCHNITT );
	  _buttonAnzeige.setEnabled( (auswahl & PA.ANZEIGE) == PA.ANZEIGE );
  }
}
