/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.parametereditor.editor;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.progress.IProgressConstants;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.sba.plugin.parametereditor.Activator;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog.UrlasserDialogInfo;

/**
 * Diese Klasse liefert eine View, in der sich Parameter bearbeiten lassen. Die
 * SystemObjekte werden in einem TabFolder dargestellt. Mit Hilfe der Methode
 * {@link ViewParameterEditor#bearbeiteKapseln(LinkedList)} wird die
 * Beschreibung der zu Bearbeitenden Daten bergeben.
 * 
 * @author HofmannJ
 * 
 */
public class ViewParameterEditor extends ViewPart
{
	/**
	 * ID dieser View
	 */
	public static final String ID = "de.bsvrz.sba.plugin.parametereditor.editor.ViewParameterEditor";
	private Composite _top = null;
	private CompositeTabbedParameterEditor _editor = null;
	private Composite _compositeKontrolle = null;
	private Button _buttonUebernehmen = null;
	private Button _buttonUebernehmenSchliessen = null;
	private Button _buttonAbbrechen = null;

	/**
	 * Memento fr die Wiederherstellung
	 */
	private IMemento _memento = null;

	/**
	 * IDs der verwendeten Kapseln
	 */
	private LinkedList<String> _kapselIDs = new LinkedList<String>();

	/**
	 * Zeigt an, ob die Klasse schon gefllt wurde. Dies ist Editor nur 1x
	 * mglich!
	 */
	private boolean _wurdeSchonGefuellt = false;
	
	private boolean _editable = true;


	@Override
	public void createPartControl(Composite parent)
	{
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 1;
		_top = new Composite(parent, SWT.NONE);
		createComposite();
		_top.setLayout(gridLayout);
		createCompositeKontrolle();

		restoreState();

		// setPartName("Blubb");
	}

	@Override
	public void setFocus()
	{
	}

	/**
	 * This method initializes composite
	 * 
	 */
	private void createComposite()
	{
		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;

		_editor = new CompositeTabbedParameterEditor(_top, SWT.NONE, DaVVerbindung
				.getInstanz().getDaVVerbindung());

		_editor.setLayoutData(gridData);
	}

	/**
	 * This method initializes _compositeKontrolle
	 * 
	 */
	private void createCompositeKontrolle()
	{
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.END;
		gridData1.verticalAlignment = GridData.CENTER;
		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		_compositeKontrolle = new Composite(_top, SWT.NONE);
		_compositeKontrolle.setLayout(rowLayout);
		_compositeKontrolle.setLayoutData(gridData1);
		_buttonUebernehmen = new Button(_compositeKontrolle, SWT.NONE);
		_buttonUebernehmen.setText("bernehmen");
		_buttonUebernehmen.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e)
			{
				aktionOk();
			}
		});

		_buttonUebernehmenSchliessen = new Button(_compositeKontrolle, SWT.NONE);
		_buttonUebernehmenSchliessen.setText("bernehmen + Schlieen");
		_buttonUebernehmenSchliessen.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e)
			{
				aktionOkSchliessen();
			}
		});

		_buttonAbbrechen = new Button(_compositeKontrolle, SWT.NONE);
		_buttonAbbrechen.setText("Abbrechen");
		_buttonAbbrechen
				.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter()
				{
					public void widgetSelected(org.eclipse.swt.events.SelectionEvent e)
					{
						aktionAbbrechen();
					}
				});
	}

	/**
	 * Speichert die nderungen des Parametereditors, die View bleibt geffnet.
	 */
	private void aktionOk()
	{
		if (_editor != null)
		{
			// _editor.sichereEditoren();
			allesInEinemRutschMitUrlasser();
		}
	}

	/**
	 * Speichert alle nderungen des Parametereditors. Es wird der Urlasserdialog
	 * hochgebracht.
	 */
	private void allesInEinemRutschMitUrlasser()
	{
		if (_editor != null)
		{
			LinkedList<ParameterEditorKapsel> sichern = _editor
					.getKapselnZumSichern();

			LinkedList<ResultData> rsds = new LinkedList<ResultData>();
			Iterator<ParameterEditorKapsel> itRDErz = sichern.iterator();
			while (itRDErz.hasNext())
			{
				DaVKommunikation dav = new DaVKommunikation(
						(ClientDavConnection) DaVVerbindung.getInstanz().getDaVVerbindung());
				ResultData resultData = dav.erzeugeResultData(itRDErz.next());
				if (resultData != null)
				{
					rsds.add(resultData);
				}
			}

			if (rsds.size() > 0)
			{
				UrlasserDialog uld = new UrlasserDialog(PlatformUI.getWorkbench()
						.getDisplay().getActiveShell());
				UrlasserDialogInfo info = uld.anzeigen(rsds);
				if (info.getFehlerInfo() != null)
				{
					if (info.getFehlerInfo().length() > 0)
					{
						String msg = "Die Daten konnten nicht versandt werden. Grund: "
								+ info.getFehlerInfo();
						MessageBox mb = new MessageBox(PlatformUI.getWorkbench()
								.getDisplay().getActiveShell(), SWT.ICON_WARNING | SWT.OK);
						mb.setText("Daten nicht versendet!");
						mb.setMessage(msg);
					}
				}
			}

		}
	}

	/**
	 * Speichert die nderungen des Parametereditors, die View wird danach
	 * geschlossen.
	 */
	private void aktionOkSchliessen()
	{
		aktionOk();
		aktionAbbrechen();
	}

	/**
	 * Schliet die View, ohne die nderungen des Parametereditors zu speichern.
	 */
	private void aktionAbbrechen()
	{
		PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage()
				.hideView(this);
	}

	public void dispose()
	{
		super.dispose();
	}

	/**
	 * Initialisiert den ParameterEditor mit einer Liste von Kapseln. Diese
	 * Initialisierung wird nur 1x durchgefhrt. Das Flag
	 * {@link ViewParameterEditor#_wurdeSchonGefuellt} ist fr die Steuerung
	 * zustndig.
	 * 
	 * @param kaspelListe
	 *          Liste von Kapseln zur Initialisierung.
	 * @param editable
	 *          Gibt an, ob die Felder des Editors editierbar sein sollen (<code>true</code>)
	 *          oder nicht (<code>false</code>).
	 */
	public void bearbeiteKapseln(LinkedList<ParameterEditorKapsel> kaspelListe)
	{
		if (!_wurdeSchonGefuellt)
		{
			if (_editor != null)
			{
				Iterator<ParameterEditorKapsel> it = kaspelListe.iterator();
				while (it.hasNext())
				{
					ParameterEditorKapsel kapsel = it.next();

					/*
					 * ID sichern
					 */
					_kapselIDs.add(kapsel.getId());

					if (kapsel != null)
					{
						_editor.setEditable(_editable);
						_editor.erzeugeEditor(kapsel);
					}
				}
			}

			if (_kapselIDs.size() > 0)
			{
				_wurdeSchonGefuellt = true;
			}

		}
	}

	/**
	 * Initialisiert den ParameterEditor mit einer Liste von Kapseln. Diese
	 * Initialisierung wird nur 1x durchgefhrt. Das Flag
	 * {@link ViewParameterEditor#_wurdeSchonGefuellt} ist fr die Steuerung
	 * zustndig.
	 * 
	 * @param kapselListe
	 *          Liste von Kapseln zur Initialisierung.
	 *          
	 */
	public void bearbeiteKapselnJob(LinkedList<ParameterEditorKapsel> kapselListe)
	{
		Iterator<ParameterEditorKapsel> itKapseln = kapselListe.iterator();
		String titleText = "Parametereditor: ";
		while (itKapseln.hasNext())
		{
			ParameterEditorKapsel kapsel = itKapseln.next();
			String pidSystemObjekt = kapsel.getPidSystemObjekt();
			titleText = titleText + pidSystemObjekt;
			if (itKapseln.hasNext())
			{
				titleText = titleText + ", ";
			}
		}
		setPartName(titleText);

		/*
		 * Job vorbereiten
		 */
		final int ticks = kapselListe.size();
		final LinkedList<ParameterEditorKapsel> liste = kapselListe;

		if (kapselListe.size() > 0)
		{
			if (!_wurdeSchonGefuellt)
			{
				Job job = new Job("Parameter einlesen")
				{

					@Override
					protected IStatus run(IProgressMonitor monitor)
					{
						setProperty(IProgressConstants.ICON_PROPERTY, Activator
								.getImageDescriptor("icons/sonstiges/text_editor_16x16.png"));

						monitor.beginTask("Lese Parameter ein...", ticks);
						try
						{
							if (_editor != null)
							{

								Iterator<ParameterEditorKapsel> it = liste.iterator();

								while (it.hasNext())
								{
									if (monitor.isCanceled())
									{
										return Status.CANCEL_STATUS;
									}

									final ParameterEditorKapsel kapsel = it.next();

									/*
									 * ID sichern
									 */
									_kapselIDs.add(kapsel.getId());

									if (kapsel != null)
									{
										final Runnable run = new Runnable()
										{

											@Override
											public void run()
											{
												_editor.setEditable(_editable);
												_editor.erzeugeEditor(kapsel);
											}
										};

										PlatformUI.getWorkbench().getDisplay().syncExec(run);
									}

									if ((!_wurdeSchonGefuellt) && (_kapselIDs.size() > 0))
									{
										_wurdeSchonGefuellt = true;
									}

									monitor.worked(1);
								}
							}
						}
						finally
						{
							monitor.done();
						}

						return Status.OK_STATUS;
					}

				};
				job.schedule();

			}
		}
	}

	public void init(IViewSite site, IMemento memento) throws PartInitException
	{
		this._memento = memento;
		init(site);
	}

	public void saveState(IMemento memento)
	{
		String idsKombiniert = "";
		Iterator<String> it = _kapselIDs.iterator();

		while (it.hasNext())
		{
			String id = it.next();
			idsKombiniert = idsKombiniert + id + ";";
		}

		if (idsKombiniert.length() > 0)
		{
			idsKombiniert = idsKombiniert.substring(0, idsKombiniert.length() - 1);
			memento.putString("ids", idsKombiniert);
		}

		if (_editor != null)
		{
			memento.putInteger("selectedTab", _editor.getSelectedTab());
		}
		
		/*
		 * Editierbar?
		 */
		if(_editable)
		{
			memento.putString("editable", "true");
		}
		else
		{
			memento.putString("editable", "false");
		}
	}

	/**
	 * Stellt aus einem Memento die View mit den aktuellen Daten wieder her. Ist
	 * keine Verbindung zum DaV vorhanden, so ist dies nicht mglich.
	 */
	private void restoreState()
	{
		if (_memento != null)
		{
			/*
			 * Editierbar?
			 */			
			String editable = _memento.getString("editable");
			_editable = new Boolean(editable).booleanValue();
			
			/*
			 * Kapseln
			 */
			LinkedList<ParameterEditorKapsel> kapselListe = new LinkedList<ParameterEditorKapsel>();

			String idsKombiniert = _memento.getString("ids");
			if (idsKombiniert.length() > 0)
			{
				String[] split = idsKombiniert.split(";");
				for (String id : split)
				{
					ParameterEditorKapsel kapsel = ParameterEditorKapsel
							.baueKapselAusId(id);
					kapselListe.add(kapsel);
				}
			}

			if (kapselListe.size() > 0)
			{
				bearbeiteKapselnJob(kapselListe);
			}

			if (_editor != null)
			{
				Integer idxSelectedTab = _memento.getInteger("selectedTab");
				if (idxSelectedTab != null)
				{
					_editor.setSelektedTab(idxSelectedTab);
				}
			}
		}
	}
	
	
	protected void setEditable(boolean editable)
	{
		this._editable = editable;
	}
	
	protected boolean isEditable()
	{
		return _editable;
	}
	
}
