/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.parametereditor.gtm;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmListenElementObjekt;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

public class GtmDatenBasisTest
{
//	 private static final String _ip = "192.0.1.116";
//	 private static final int _port = 8085;
	private static final String _ip = "127.0.0.1";
	private static final int _port = 8083;
	private static final String _benutzer = "Tester";
	private static final String _passwort = "geheim";
	private ClientDavConnection _connection = null;
	
	
	public static void main(String[] params)
	{
		GtmDatenBasisTest test = new GtmDatenBasisTest();
	}
	
	public GtmDatenBasisTest()
	{
		davVerbinden();
		
		testA();
		
		davTrennen();
	}
	
	public void testA()
	{
		GtmDatenBasis datenBasis = new GtmDatenBasis(_connection, false, null);
		
		DataModel dataModel = _connection.getDataModel();
		
		/*
		 * SystemObjekte nach Typen bestimmen
		 */
		SystemObjectType type = dataModel.getType("typ.messQuerschnitt");
		LinkedList<SystemObjectType> types = new LinkedList<SystemObjectType>();
		types.add(type);
		
		datenBasis.setTypes(types);
//		datenBasis.setAll();
		
		List<GtmListenElementObjekt> systemObjects = datenBasis.getSystemObjects();
		System.out.println("Anzahl SystemObjekte: " + systemObjects.size());
		Iterator<GtmListenElementObjekt> it = systemObjects.iterator();
		while(it.hasNext())
		{
			GtmListenElementObjekt leo = it.next();
			System.out.println("SO: " + leo.getSystemObjekt());
		}
		
		/*
		 * Atg nach Typen bestimmen
		 */
		List<AttributeGroup> atgs = datenBasis.getAtg(datenBasis.getSystemObjectTypes());
		System.out.println("Anzahl Attributgruppen: " + atgs.size());
		
		
		/*
		 * Asp nach Atgs bestimmen
		 */
		List<Aspect> asps = datenBasis.getAsp(atgs);
		System.out.println("Anzahl Aspekte: " + asps.size());
		
		
		
		
		
	}
	
	
	
	
	
	private void davVerbinden()
	{
		try
		{
			_connection = new ClientDavConnection();
			// IP setzen
			_connection.getClientDavParameters().setDavCommunicationAddress(_ip);

			// Port setzen
			_connection.getClientDavParameters().setDavCommunicationSubAddress(_port);

			try
			{
				_connection.connect();

				try
				{
					_connection.login(_benutzer, _passwort);
				} catch (InconsistentLoginException e)
				{
					e.printStackTrace();
				}

			} catch (CommunicationError e)
			{
				e.printStackTrace();
			} catch (ConnectionException e)
			{
				e.printStackTrace();
			}

		} catch (MissingParameterException e)
		{
			e.printStackTrace();
		}
	}

	private void davTrennen()
	{
		if (_connection != null)
		{
			_connection.disconnect(false, "Kein Fehler");
		}
	}
}
