/*
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */
package de.bsvrz.sba.plugin.parametereditor.gtm;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Display;

import de.bsvrz.buv.rw.basislib.datenidentifikation.GtmListeComposite;
import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

public class GtmTest
{
	// private static final String _ip = "192.0.1.116";
	// private static final int _port = 8085;
//	 private static final String _ip = "192.0.1.117";
//	 private static final int _port = 8085;
	private static final String _ip = "127.0.0.1";
	private static final int _port = 8083;
	private static final String _benutzer = "Tester";
	private static final String _passwort = "geheim";
	private static ClientDavConnection _connection = null;

	private Shell _shell = null;
	private GtmListeComposite _compGtmListe;

	/**
	 * @param args
	 */
	public static void main(String[] args)
	{
		// TODO Auto-generated method stub
		/*
		 * Before this is run, be sure to set up the launch configuration
		 * (Arguments->VM Arguments) for the correct SWT library path in order to
		 * run with the SWT dlls. The dlls are located in the SWT plugin jar. For
		 * example, on Windows the Eclipse SWT 3.1 plugin jar is:
		 * installation_directory\plugins\org.eclipse.swt.win32_3.1.0.jar
		 */

		davVerbinden();

		Display display = Display.getDefault();
		GtmTest thisClass = new GtmTest();
		thisClass.createShell();
		thisClass._shell.open();

		while (!thisClass._shell.isDisposed())
		{
			if (!display.readAndDispatch())
				display.sleep();
		}
		display.dispose();
		davTrennen();
	}

	/**
	 * This method initializes sShell
	 */
	private void createShell()
	{
		_shell = new Shell();
		_shell.setText("Shell");
		_shell.setSize(new Point(800, 600));
		_shell.setLayout(new GridLayout());

		GtmDatenBasis datenBasis = new GtmDatenBasis(_connection, false, null);

		DataModel dataModel = _connection.getDataModel();

		/*
		 * SystemObjekte nach Typen bestimmen
		 */
		SystemObjectType type = dataModel.getType("typ.messQuerschnitt");
		SystemObjectType type2 = dataModel.getType("typ.fahrStreifen");
		SystemObjectType type3 = dataModel.getType("typ.werteZustand");
		
		LinkedList<SystemObjectType> types = new LinkedList<SystemObjectType>();
		types.add(type);
		types.add(type2);
		types.add(type3);
		


		 datenBasis.setTypes(types);
//		datenBasis.setAll();

		Composite compositeOben = new Composite(_shell, SWT.NONE);
		Composite compositeUnten = new Composite(_shell, SWT.NONE);
		
		_compGtmListe = new GtmListeComposite(compositeOben, SWT.NONE, datenBasis, 4);
	
		Button buttonOk = new Button(compositeUnten, SWT.NONE);
		buttonOk.setText("OK");
		
		Button buttonAbbrechen = new Button(compositeUnten, SWT.NONE);
		buttonAbbrechen.setText("Abbrechen");		
		
		buttonOk.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{
				LinkedList<SystemObject> selektierteSystemObjekte = _compGtmListe.getSelektierteSystemObjekte();
				Iterator<SystemObject> it = selektierteSystemObjekte.iterator();
				while(it.hasNext())
				{
					System.out.println("=>" + it.next());
				}
			}
		});
		
		buttonAbbrechen.addSelectionListener(new SelectionAdapter()
		{
			public void widgetSelected(SelectionEvent e)
			{				
			}
		});
		
		
		
		
		
		
		
		

		GridData gridData = new GridData();
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.verticalAlignment = GridData.FILL;
		compositeOben.setLayout(new GridLayout());
		compositeOben.setLayoutData(gridData);

		RowLayout rowLayout = new RowLayout();
		rowLayout.pack = false;
		GridData gridData1 = new GridData();
		gridData1.horizontalAlignment = GridData.END;
		gridData1.grabExcessHorizontalSpace = true;
		gridData1.verticalAlignment = GridData.CENTER;
		compositeUnten.setLayoutData(gridData1);
		compositeUnten.setLayout(rowLayout);

		GridData gd = new GridData();
		gd.grabExcessHorizontalSpace = true;
		gd.horizontalAlignment = GridData.FILL;
		gd.grabExcessVerticalSpace = true;
		gd.verticalAlignment = GridData.FILL;
		_compGtmListe.setLayoutData(gd);

	}

	private static void davVerbinden()
	{
		try
		{
			_connection = new ClientDavConnection();
			// IP setzen
			_connection.getClientDavParameters().setDavCommunicationAddress(_ip);

			// Port setzen
			_connection.getClientDavParameters().setDavCommunicationSubAddress(_port);

			try
			{
				_connection.connect();

				try
				{
					_connection.login(_benutzer, _passwort);
				}
				catch (InconsistentLoginException e)
				{
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			}
			catch (CommunicationError e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			catch (ConnectionException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}
		catch (MissingParameterException e)
		{
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private static void davTrennen()
	{
		if (_connection != null)
		{
			_connection.disconnect(false, "Kein Fehler");
		}
	}

}
