/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.ui.PlatformUI;

//import de.bsvrz.buv.rw.basislib.login.UrlasserDialog;
//import de.bsvrz.buv.rw.basislib.login.UrlasserDialog.UrlasserDialogInfo;
//import de.bsvrz.buv.rw.basislib.login.UrlasserDialog.UrlasserDialogInfo;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog;
import de.koelnloevenich.dif.basislib.urlasserdialog.UrlasserDialog.UrlasserDialogInfo;

/**
 * Sendet Reaktionen auf den Workflow<br>
 * Empfngt die Stati der einzelnen Abschnitte.<br>
 * Kommunikation mit der ATG <code>atg.tsfAbschnittBefehl</code>. <br>
 * <br>
 * <b>Lesen:</b><br>
 * <ul>
 * <li>TYP: <code>typ.tsfAbschnitt</code></li>
 * <li>ATG: <code>atg.tsfAbschnittBefehl</code></li>
 * <li>ASP: <code>atg.tsfAbschnittBefehl</code></li>
 * </ul>
 * 
 * <br>
 * <b>Schreiben:</b><br>
 * <ul>
 * <li>TYP: <code>typ.tsfAbschnitt</code></li>
 * <li>ATG: <code>atg.tsfAbschnittBefehl</code></li>
 * <li>ASP: <code>asp.tsfVorgabe</code></li>
 * </ul>
 * 
 * 
 * @author HofmannJ
 * 
 */
public class DaVKomTSFAbschnittBefehl implements ClientReceiverInterface,
		ClientSenderInterface
{
	private static DaVKomTSFAbschnittBefehl _instance = null;
	
	public static final SimpleDateFormat SDF = new SimpleDateFormat(
	"dd.MM.yyyy HH:mm:ss");

	/**
	 * Verbindung DaV
	 */
	private ClientDavInterface _connection = null;

	/**
	 * {@link DataDescription} fr die Lese-Anmeldung auf die Schaltempfehlungen
	 */
	private DataDescription _ddLeseAnmeldungASPVorgabeZustand = null;

	/**
	 * {@link SystemObject}s, auf welche die Leseanmeldung auf die
	 * Schaltempfehlungen erfolgte.
	 */
	private LinkedList<SystemObject> _lSoLeseAnmeldungASPVorgabeZustand = new LinkedList<SystemObject>();

	/**
	 * {@link DataDescription} fr die Lese-Anmeldungen auf den aktuellen
	 * Schaltzustand.
	 */
	private DataDescription _ddLeseAnmeldungASPZustand = null;

	/**
	 * {@link SystemObject}s, auf welche die Leseanmeldung auf den aktuellen
	 * Schaltzustand erfolgte.
	 */
	private LinkedList<SystemObject> _lSoLeseAnmeldungASPZustand = new LinkedList<SystemObject>();

	/**
	 * Modell fr die Daten der GUI
	 */
	private GUIModelTSFAbschnittBefehl _guiModelTSFAbschnittBefehl = null;

	private DaVKomTSFAbschnittBefehl()
	{
		initKommunikation();
	}

	private DaVKomTSFAbschnittBefehl(ClientDavInterface con)
	{
		this._connection = con;
		initKommunikation();
	}

	public static DaVKomTSFAbschnittBefehl getInstance()
	{
		if (_instance == null)
		{
			synchronized (DaVKomTSFAbschnittBefehl.class)
			{
				if (_instance == null)
				{
					_instance = new DaVKomTSFAbschnittBefehl();
				}
			}
		}

		return _instance;
	}

	public static DaVKomTSFAbschnittBefehl getInstance(ClientDavInterface con)
	{
		if (_instance == null)
		{
			synchronized (DaVKomTSFAbschnittBefehl.class)
			{
				if (_instance == null)
				{
					_instance = new DaVKomTSFAbschnittBefehl(con);
				}
			}
		}

		return _instance;
	}

	private void initKommunikation()
	{
		_guiModelTSFAbschnittBefehl = new GUIModelTSFAbschnittBefehl();

		if (_connection == null)
		{
			this._connection = DaVVerbindung.getInstanz().getDaVVerbindung();
		}

		if (_connection != null)
		{
			anmeldenEmpfang();
		} else
		{
			System.err
					.println("Es ist keine Kommunikation mglich, dar keine Verbindung zum DaV existent ist.");
		}
	}

	private void anmeldenEmpfang()
	{
		DataModel dataModel = _connection.getDataModel();

		/*
		 * Hren auf Schaltempfehlungen unter dem Aspekt asp.tsfVorgabeZustand.
		 */
		if (_ddLeseAnmeldungASPVorgabeZustand == null)
		{
			SystemObjectType sot = dataModel.getType(Defs.TYP_TSF_ABSCHNITT);
			if (sot != null)
			{
				List<SystemObject> lSOTs = sot.getObjects();

				if (lSOTs.size() > 0)
				{
					AttributeGroup atg = dataModel
							.getAttributeGroup(Defs.ATG_TSF_ABSCHNITT_BEFEHL);
					Aspect asp = dataModel.getAspect(Defs.ASP_TSF_VORGABE_ZUSTAND);

					if ((atg != null) && (asp != null))
					{
						DataDescription dd = new DataDescription(atg, asp);
						_ddLeseAnmeldungASPVorgabeZustand = dd;
						_lSoLeseAnmeldungASPVorgabeZustand.addAll(lSOTs);

						_connection.subscribeReceiver(this, lSOTs, dd, ReceiveOptions
								.normal(), ReceiverRole.receiver());

					} else
					{
						System.err.println("ATG und ASP konnten nicht bestimmt werden.");
					}
				} else
				{
					System.err.println("Es muss mindestens ein Objekt vom Typ "
							+ Defs.TYP_TSF_ABSCHNITT
							+ " existieren. Dies ist z.Zt. nicht der Fall!");
				}

			} else
			{
				System.err.println("Es konnten z.Zt. keine Objekte vom Typ "
						+ Defs.TYP_TSF_ABSCHNITT
						+ " bestimmt werden. Es ist keine Empfangsanmeldung mglich.");
			}
		} else
		{
			System.err
					.println("Es wurde bereits eine Leseanmeldung unter dem Aspekt "
							+ Defs.ASP_TSF_VORGABE_ZUSTAND + " durchgefhrt.");

		}

		/*
		 * Anmelden auf den aktuellen Schaltzustand unter dem Aspekt atg.tfsZustand
		 */
		if (_ddLeseAnmeldungASPZustand == null)
		{
			SystemObjectType sot = dataModel.getType(Defs.TYP_TSF_ABSCHNITT);
			if (sot != null)
			{
				List<SystemObject> lSOTs = sot.getObjects();

				if (lSOTs.size() > 0)
				{
					AttributeGroup atg = dataModel
							.getAttributeGroup(Defs.ATG_TSF_ABSCHNITT_BEFEHL);
					Aspect asp = dataModel.getAspect(Defs.ASP_TSF_ZUSTAND);

					if ((atg != null) && (asp != null))
					{
						DataDescription dd = new DataDescription(atg, asp);
						_ddLeseAnmeldungASPZustand = dd;
						_lSoLeseAnmeldungASPZustand.addAll(lSOTs);

						_connection.subscribeReceiver(this, lSOTs, dd, ReceiveOptions
								.normal(), ReceiverRole.receiver());

					} else
					{
						System.err.println("ATG und ASP konnten nicht bestimmt werden.");
					}
				} else
				{
					System.err.println("Es muss mindestens ein Objekt vom Typ "
							+ Defs.TYP_TSF_ABSCHNITT
							+ " existieren. Dies ist z.Zt. nicht der Fall!");
				}

			} else
			{
				System.err.println("Es konnten z.Zt. keine Objekte vom Typ "
						+ Defs.TYP_TSF_ABSCHNITT
						+ " bestimmt werden. Es ist keine Empfangsanmeldung mglich.");
			}
		}
	}

	private void abmeldenEmpfang()
	{
		if (_connection != null)
		{
			if (_ddLeseAnmeldungASPVorgabeZustand != null)
			{
				if (_lSoLeseAnmeldungASPVorgabeZustand.size() > 0)
				{
					_connection.unsubscribeReceiver(this,
							_lSoLeseAnmeldungASPVorgabeZustand,
							_ddLeseAnmeldungASPVorgabeZustand);

					this._lSoLeseAnmeldungASPVorgabeZustand.clear();
					this._ddLeseAnmeldungASPVorgabeZustand = null;
				}
			}

			if (_ddLeseAnmeldungASPZustand != null)
			{
				if (_lSoLeseAnmeldungASPZustand.size() > 0)
				{
					_connection.unsubscribeReceiver(this, _lSoLeseAnmeldungASPZustand,
							_ddLeseAnmeldungASPZustand);

					this._lSoLeseAnmeldungASPZustand.clear();
					this._ddLeseAnmeldungASPZustand = null;
				}
			}
		}
	}

	@Override
	public void update(ResultData[] results)
	{
		if ((results != null) && (results.length > 0))
		{
			for (ResultData resultData : results)
			{
				/*
				 * Schaltempfhelungen
				 */
				if (resultData.getDataDescription().equals(
						_ddLeseAnmeldungASPVorgabeZustand))
				{
					if (resultData.hasData())
					{
						SystemObject so = resultData.getObject();
						Data data = resultData.getData();
						if (data != null)
						{
							String id = data.getTextValue("ID").getText();
							
							if (id.contains("#")){
								String[] split = id.split("#");
								if (split.length > 0)
									id = split[0];									      
							}

							int wertAktion = (int) data.getUnscaledValue("Aktion").getState()
									.getValue();
							E_TSFBefehl aktion = E_TSFBefehl.getBefehl(wertAktion);

							int wertZustand = (int) data.getUnscaledValue("Zustand")
									.getState().getValue();
							E_TSFProgrammZustand zustand = E_TSFProgrammZustand
									.getBefehl(wertZustand);

							M_TSFAbschnittBefehl abschnittBefehl = new M_TSFAbschnittBefehl(
									id, so, aktion, zustand);

							getModel().put(
									resultData.getDataDescription().getAspect().getPid(),
									abschnittBefehl);
						}
					} else
					{
						/*
						 * Keine Daten vorhanden, dann aus dem Modell entfernen.
						 */
						getModel().remove(
								resultData.getDataDescription().getAspect().getPid(),
								resultData.getObject().getPid());
					}
				}

				/*
				 * Aktueller Zustand
				 */
				if (resultData.getDataDescription().equals(_ddLeseAnmeldungASPZustand))
				{
					if (resultData.hasData())
					{
						SystemObject so = resultData.getObject();
						Data data = resultData.getData();
						if (data != null)
						{
							String id = data.getTextValue("ID").getText();

							int wertAktion = (int) data.getUnscaledValue("Aktion").getState()
									.getValue();
							E_TSFBefehl aktion = E_TSFBefehl.getBefehl(wertAktion);

							int wertZustand = (int) data.getUnscaledValue("Zustand")
									.getState().getValue();
							E_TSFProgrammZustand zustand = E_TSFProgrammZustand
									.getBefehl(wertZustand);

							M_TSFAbschnittBefehl abschnittBefehl = new M_TSFAbschnittBefehl(
									id, so, aktion, zustand);

							getModel().put(
									resultData.getDataDescription().getAspect().getPid(),
									abschnittBefehl);
						}
					} else
					{
						getModel().remove(
								resultData.getDataDescription().getAspect().getPid(),
								resultData.getObject().getPid());
					}
				}

			}
		}
	}

	public void dispose()
	{
		abmeldenEmpfang();
	}

	/**
	 * Versendet einen Befehl (ATG <code>atg.tsfAbschnittBefehl</code>) unter dem
	 * Aspekt <code>asp.tsfVorgabe</code>.
	 * 
	 * @param befehl
	 *          Der zu versendende Befehl.
	 * @return Liefert einen {@link WrapperULDInfo} mit einen sonstigen Info-Strin
	 *         {@link String} zurck mit einer Fehlerinfo, falls der Befehl nicht
	 *         versendet werden konnte. Konnte der Befehl versendet werden, wird
	 *         <code>null</code> zurckgeliefert.
	 */
	public WrapperULDInfo sendeAbschnittBefehl(M_TSFAbschnittBefehl befehl)
	{
		WrapperULDInfo uldInfofehlerInfo = null;

		if (befehl != null)
		{
			LinkedList<M_TSFAbschnittBefehl> lBefehle = new LinkedList<M_TSFAbschnittBefehl>();
			lBefehle.add(befehl);

			String berechtigung = null;
			E_TSFBefehl aktion = befehl.getAktion();
//			if (aktion == E_TSFBefehl.Freigabe)
//			{
//				berechtigung = Berechtigungen.freigeben;
//			} else if (aktion == E_TSFBefehl.Schnellfreigabe)
//			{
//				berechtigung = Berechtigungen.freigeben;
//			} else if (aktion == E_TSFBefehl.Schnellsperrung)
//			{
//				berechtigung = Berechtigungen.schnellSperren;
//			} else if (aktion == E_TSFBefehl.Sperrung)
//			{
//				berechtigung = Berechtigungen.sperren;
//			}

			uldInfofehlerInfo = sendeAbschnittsBefehle(lBefehle, "schalten");
		}

		return uldInfofehlerInfo;
	}

	/**
	 * Versendet Befehle (ATG <code>atg.tsfAbschnittBefehl</code>) unter dem
	 * Aspekt <code>asp.tsfVorgabe</code>.
	 * 
	 * @param lBefehle
	 *          Die zu versendenden Befehle.
	 * @return Liefert einen {@link WrapperULDInfo} mit einen sonstigen Info-Strin
	 *         {@link String} zurck mit einer Fehlerinfo, falls der Befehl nicht
	 *         versendet werden konnte. Konnte der Befehl versendet werden, wird
	 *         <code>null</code> zurckgeliefert.
	 */
	private WrapperULDInfo sendeAbschnittsBefehle(
			LinkedList<M_TSFAbschnittBefehl> lBefehle, String berechtigung)
	{
		WrapperULDInfo wrapperULDInfo = null;
		String sonstigeInfos = null;

		
		System.out.println("DaVKomTSFAbschnittBefehl.sendeAbschnittsBefehle(): " + lBefehle.size() + " -> " + berechtigung);
		
		if ((lBefehle != null) && (lBefehle.size() > 0))
		{
			LinkedList<ResultData> lRDs = new LinkedList<ResultData>();

			DataModel dataModel = _connection.getDataModel();
			AttributeGroup atg = dataModel
					.getAttributeGroup(Defs.ATG_TSF_ABSCHNITT_BEFEHL);
			Aspect asp = dataModel.getAspect(Defs.ASP_TSF_VORGABE);
			for (M_TSFAbschnittBefehl befehl : lBefehle)
			{
				if ((befehl != null) && (befehl.getSo() != null))
				{
					Data data = _connection.createData(atg);
					// System.out.println("DATA=>" + data);
					// System.out.println("\t>>>BEFHEL AKTIOPN " + befehl.getAktion());
					
					long l = System.currentTimeMillis();
					
					data.getTextValue("ID").setText(Defs.getClientId() + "#" + SDF.format(new Date(l)));
					data.getUnscaledValue("Aktion").set(befehl.getAktion().getWert());
					data.getUnscaledValue("Zustand").set(befehl.getZustand().getWert());
					DataDescription desc = new DataDescription(atg, asp);
					ResultData rd = new ResultData(befehl.getSo(), desc, l, data);
					lRDs.add(rd);
				}
			}

			if (lRDs.size() > 0)
			{
//				if (berechtigung == null)
//				{
//					berechtigung = Berechtigungen.schalten;
//				}

				// UrlasserDialog uld = new UrlasserDialog(PlatformUI.getWorkbench()
				// .getDisplay().getActiveShell());
				// UrlasserDialogInfo uldInfo = uld.anzeigen(lRDs, berechtigung);

				UldRunner uldRunner = new UldRunner(lRDs, berechtigung);
				UrlasserDialogInfo uldInfo = uldRunner.getUldInfo();

				sonstigeInfos = uldInfo.getFehlerInfo();
				if (!uldInfo.getEingabeOk())
				{
					if (sonstigeInfos != null)
					{
						sonstigeInfos = sonstigeInfos + " ABBRECHEN";
					} else
					{
						sonstigeInfos = "ABBRECHEN";
					}
				}

				wrapperULDInfo = new WrapperULDInfo(uldInfo, sonstigeInfos);
			}

		}

		return wrapperULDInfo;
	}

	private class UldRunner
	{
		private UrlasserDialog _uld = null;
//		private UrlasserDialogInfo _uldInfo = null;
		private UrlasserDialogInfo _uldInfo;
		private LinkedList<ResultData> _lRDs = null;
		private String _berechtigung = null;

		public UldRunner(LinkedList<ResultData> lRDs, String berechtigung)
		{

			this._lRDs = lRDs;
			this._berechtigung = berechtigung;
			doRun();
		}

		private void doRun()
		{
			Runnable run = new Runnable()
			{
				

        public void run()
				{
					_uld = new UrlasserDialog(PlatformUI.getWorkbench().getDisplay()
							.getActiveShell());
					Iterator<ResultData> it = _lRDs.iterator();
					while(it.hasNext())
					{
						ResultData rd = it.next();
					}
					if (Defs.BERECHTIGUNGEN_IGNORIEREN)
					{
						_uldInfo = _uld.anzeigen(_lRDs);
					} else
					{
						_uldInfo = _uld.anzeigen(_lRDs);
					}
				}
			};
			PlatformUI.getWorkbench().getDisplay().syncExec(run);
		}

		public UrlasserDialogInfo getUldInfo()
		{
			return _uldInfo;
		}
	}

	@Override
	public void dataRequest(SystemObject object, DataDescription dataDescription,
			byte state)
	{
	}

	@Override
	public boolean isRequestSupported(SystemObject object,
			DataDescription dataDescription)
	{
		return false;
	}

	/**
	 * Liefert das Modell mit den TSFAbschnittBehlen fr die verwendung durch
	 * GUI-Dialoge.
	 * 
	 * @return
	 */
	public GUIModelTSFAbschnittBefehl getModel()
	{
		return _guiModelTSFAbschnittBefehl;
	}

}
