/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.LinkedList;
import java.util.Set;
import java.util.TreeMap;

/**
 * Informiert Dialoge/Views ber die von {@link DaVKomTSFAbschnittBefehl}
 * gelieferten Informationen.
 * 
 * @author HofmannJ
 * 
 */
public class GUIModelTSFAbschnittBefehl
{

	public static final String PROP_INIT = "PROP_INIT";
	public static final String PROP_ADDED = "PROP_ADDED";
	public static final String PROP_REMOVED = "PROP_REMOVED";
	private PropertyChangeSupport _pcs = new PropertyChangeSupport(this);

	/**
	 * 1.Key = Aspekt der Daten, zweiter Key = Pid des SystemObjekts des
	 * Abschnitts.
	 */
	private TreeMap<String, TreeMap<String, M_TSFAbschnittBefehl>> _map = new TreeMap<String, TreeMap<String, M_TSFAbschnittBefehl>>();

	public void put(String pidAsp, M_TSFAbschnittBefehl abschnittBefehl)
	{
		if ((pidAsp != null) && (abschnittBefehl != null))
		{
			if (abschnittBefehl.getSo() != null)
			{
				M_TSFAbschnittBefehl oldValue = null;
				M_TSFAbschnittBefehl newValue = abschnittBefehl;

				synchronized (_map)
				{
					if (!_map.containsKey(pidAsp))
					{
						_map.put(pidAsp, new TreeMap<String, M_TSFAbschnittBefehl>());
					}

					TreeMap<String, M_TSFAbschnittBefehl> subMap = _map.get(pidAsp);
					subMap.put(abschnittBefehl.getSo().getPid(), abschnittBefehl);
				}

				// Listener benachrichtigen.
				WrapperListenerBenachrichtigung listenerBenachrichtigungOld = null;
				WrapperListenerBenachrichtigung listenerBenachrichtigungNew = null;

				if (oldValue != null)
				{
					listenerBenachrichtigungOld = new WrapperListenerBenachrichtigung(
							pidAsp, oldValue);
				}

				if (newValue != null)
				{
					listenerBenachrichtigungNew = new WrapperListenerBenachrichtigung(
							pidAsp, newValue);
				}

				_pcs.firePropertyChange(PROP_ADDED, listenerBenachrichtigungOld,
						listenerBenachrichtigungNew);

			} else
			{
				System.err
						.println("Das SystemObjekt des M_TSFAbschnittBefehl ist null.");
			}

		} else
		{
			System.err
					.println("Die PID des Aspekts oder der M_TSFAbschnittBefehl ist null.");
		}
	}

	public M_TSFAbschnittBefehl remove(String pidAsp, String pidAbschnitt)
	{
		M_TSFAbschnittBefehl removed = null;

		if (pidAbschnitt != null)
		{
			synchronized (_map)
			{
				TreeMap<String, M_TSFAbschnittBefehl> subMap = _map.get(pidAsp);

				if (subMap != null)
				{
					removed = subMap.remove(pidAbschnitt);
					if (subMap.isEmpty())
					{
						_map.remove(pidAsp);
					}
				}

			}
		}

		/*
		 * Listener ber entferntes Element benachrichtigen.
		 */
		if (removed != null)
		{
			WrapperListenerBenachrichtigung listenerBenachrichtigungRemoved = new WrapperListenerBenachrichtigung(
					pidAsp, removed);

			_pcs.firePropertyChange(PROP_REMOVED, listenerBenachrichtigungRemoved,
					null);
		}

		return removed;
	}

	private String[] getAspektPids()
	{
		LinkedList<String> lPidsAsp = new LinkedList<String>();

		synchronized (_map)
		{
			Set<String> keySet = _map.keySet();
			lPidsAsp.addAll(keySet);
		}

		return lPidsAsp.toArray(new String[lPidsAsp.size()]);
	}

	private M_TSFAbschnittBefehl[] getAbschnitte(String pidAsp)
	{
		LinkedList<M_TSFAbschnittBefehl> lAbsBef = new LinkedList<M_TSFAbschnittBefehl>();

		if (pidAsp != null)
		{
			synchronized (_map)
			{
				TreeMap<String, M_TSFAbschnittBefehl> subMap = _map.get(pidAsp);
				if (subMap != null)
				{
					lAbsBef.addAll(subMap.values());
				}
			}
		}

		return lAbsBef.toArray(new M_TSFAbschnittBefehl[lAbsBef.size()]);
	}

	/**
	 * Es handelt sich bei dieser Klasse um eine Art Wrapper.
	 * 
	 * @author HofmannJ
	 * 
	 */
	public class WrapperListenerBenachrichtigung
	{
		/**
		 * Aspekt unter dem die Daten empfangen wurden.
		 */
		private String _pidAsp = null;

		/**
		 * Befehl fr den jeweiligen Abschnitt.
		 */
		private M_TSFAbschnittBefehl _abschnittBefehl = null;

		/**
		 * 
		 * @param pidAsp
		 *          Aspekt unter dem die Daten empfangen wurden.
		 * @param abschnittBefehl
		 *          Befehl fr den jeweiligen Abschnitt
		 */
		public WrapperListenerBenachrichtigung(String pidAsp,
				M_TSFAbschnittBefehl abschnittBefehl)
		{
			setPidAsp(pidAsp);
			setAbschnittBefehl(abschnittBefehl);
		}

		public String getPidAsp()
		{
			return _pidAsp;
		}

		private void setPidAsp(String pidAsp)
		{
			_pidAsp = pidAsp;
		}

		public M_TSFAbschnittBefehl getAbschnittBefehl()
		{
			return _abschnittBefehl;
		}

		private void setAbschnittBefehl(M_TSFAbschnittBefehl abschnittBefehl)
		{
			_abschnittBefehl = abschnittBefehl;
		}

	}

	public void addListener(PropertyChangeListener pcl)
	{
		_pcs.addPropertyChangeListener(pcl);

		LinkedList<WrapperListenerBenachrichtigung> lListenerBenachrichtigungen = new LinkedList<WrapperListenerBenachrichtigung>();

		String[] aAspektPids = getAspektPids();
		for (String pidAsp : aAspektPids)
		{
			M_TSFAbschnittBefehl[] aAbschnitte = getAbschnitte(pidAsp);
			for (M_TSFAbschnittBefehl abschnittBefehl : aAbschnitte)
			{
				WrapperListenerBenachrichtigung listenerBenachrichtigung = new WrapperListenerBenachrichtigung(
						pidAsp, abschnittBefehl);
				lListenerBenachrichtigungen.add(listenerBenachrichtigung);
			}
		}

		if (lListenerBenachrichtigungen.size() > 0)
		{
			WrapperListenerBenachrichtigung[] aListenerBenachrichtigungen = lListenerBenachrichtigungen
					.toArray(new WrapperListenerBenachrichtigung[lListenerBenachrichtigungen
							.size()]);

			pcl.propertyChange(new PropertyChangeEvent(this, PROP_INIT, null,
					aListenerBenachrichtigungen));
		}
	}

	public void removeListener(PropertyChangeListener pcl)
	{
		_pcs.removePropertyChangeListener(pcl);
	}

}
