/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.HashMap;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Modellklasse fr den Zugriff auf die Diagnose-Daten der TSF-Abschnitte.
 * 
 * @author HofmannJ
 * 
 */
public class GUIModelTSFAbschnittDiagnoseDaten
{
	public static final String PROP_INIT = "INIT";
	public static final String PROP_ADD = "ADD";
	public static final String PROP_REMOVED = "PROP_REMOVED";

	/**
	 * Fr Listener, die ber alles informiert werden mchten.
	 */
	private PropertyChangeSupport _pcs = new PropertyChangeSupport(this);

	/**
	 * Key = SystemObjekt vom Typ <code>typ.tsfAbschnitt</code>, Val=
	 * {@link M_TSFAbschnittDiagnoseDaten}.
	 */
	private HashMap<SystemObject, M_TSFAbschnittDiagnoseDaten> _map = new HashMap<SystemObject, M_TSFAbschnittDiagnoseDaten>();

	/**
	 * Fgt {@link M_TSFAbschnittDiagnoseDaten} hinzu, alle angemeldeten Listener
	 * werden hierber informiert.
	 * 
	 * @param daten
	 *          Hinzuzufgende {@link M_TSFAbschnittDiagnoseDaten}.
	 */
	protected void put(M_TSFAbschnittDiagnoseDaten daten)
	{
		if ((daten != null) && (daten.getSo() != null))
		{
			M_TSFAbschnittDiagnoseDaten oldVal = null;

			synchronized (_map)
			{
				oldVal = _map.get(daten.getSo());
				_map.put(daten.getSo(), daten);
			}

			_pcs.firePropertyChange(PROP_ADD, oldVal, daten);
		}
	}

	/**
	 * Entfernt {@link M_TSFAbschnittDiagnoseDaten} fr einen bestimmten
	 * Abschnitt. Alle angemeldeten Listener werden hierber informiert.
	 * 
	 * @param soTsfAbschnitt
	 *          {@link SystemObject} des TSF-Abschnitts, dessen
	 *          {@link M_TSFAbschnittDiagnoseDaten} entfernt werden sollen.
	 */
	protected void remove(SystemObject soTsfAbschnitt)
	{
		if (soTsfAbschnitt != null)
		{
			M_TSFAbschnittDiagnoseDaten oldVal = null;
			synchronized (_map)
			{
				oldVal = _map.remove(soTsfAbschnitt);
			}

			_pcs.firePropertyChange(PROP_REMOVED, oldVal, null);
		}
	}

	/**
	 * Liefert alle {@link M_TSFAbschnittDiagnoseDaten}, die hier bisher
	 * registriert wurden.
	 * 
	 * @return Alle {@link M_TSFAbschnittDiagnoseDaten} als Array. Array kann leer
	 *         sein.
	 */
	protected M_TSFAbschnittDiagnoseDaten[] getAlleTsfAbschnittDiagnoseDaten()
	{
		M_TSFAbschnittDiagnoseDaten[] re = null;

		synchronized (_map)
		{
			re = _map.values().toArray(
					new M_TSFAbschnittDiagnoseDaten[_map.values().size()]);
		}

		return re;
	}

	/**
	 * Fgt einen Listener hinzu, der ber alle
	 * {@link M_TSFAbschnittDiagnoseDaten} dieses Modells auf dem laufenden
	 * gehalten werden.
	 * 
	 * @param pcl
	 *          Hinzuzufgender Listener.
	 */
	public void addListener(PropertyChangeListener pcl)
	{
		if (pcl != null)
		{
			_pcs.addPropertyChangeListener(pcl);

			PropertyChangeEvent pce = new PropertyChangeEvent(this, PROP_INIT, null,
					getAlleTsfAbschnittDiagnoseDaten());
			pcl.propertyChange(pce);
		}
	}

	/**
	 * Entfernt einen Listener, der nicht mehr ber die
	 * {@link M_TSFAbschnittDiagnoseDaten} dieses Modells auf dem laufenden
	 * gehalten werden soll.
	 * 
	 * @param pcl
	 *          Zu entfernender Listener.
	 */
	public void removeListener(PropertyChangeListener pcl)
	{
		if (pcl != null)
		{
			_pcs.removePropertyChangeListener(pcl);
		}
	}
}
