/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Auffordern des GUI zum Weiterschalten der TSF fr einen Abschnitt.
 * Reprsentiert die Daten der Attributgruppe <code>atg.tsfWeiterSchalten</code>
 * .
 * 
 * @author HofmannJ
 * 
 */
public class M_TSFWeiterSchalten implements Comparable<M_TSFWeiterSchalten>
{
	/**
	 * SystemObjekt, auf welches sich diese Instanz bezieht. Es sollte sich
	 * hierbei um den jeweiligen Abschnitt handeln.
	 */
	private SystemObject _so;

	/**
	 * Zustand des Dialogs zum Weiterschalten, Typ
	 * <code>	att.tsfWeiterSchaltenZustand</code>.
	 */
	private E_TSFWeiterschaltenZustand _zustand = null;

	/**
	 * ID der GUI die die TSF veranlasst hat.
	 */
	private String _clientId = null;

	/**
	 * Eindeutige Kennung fr den Dialog.
	 */
	private String _kennung = null;

	/**
	 * Kommentar zu der Stufe des TSF die weitergeschaltet werden soll.
	 */
	private String _kommentar = null;

	private E_TSFBefehl _programm = null;
	private long _stufe = 0l;

	public M_TSFWeiterSchalten()
	{
	}

	public M_TSFWeiterSchalten(SystemObject so)
	{
		setSo(so);
	}

	/**
	 * SystemObjekt, auf welches sich diese Instanz bezieht. Es sollte sich
	 * hierbei um den jeweiligen Abschnitt handeln.
	 * 
	 * @return SystemObjekt
	 */
	public SystemObject getSo()
	{
		return _so;
	}

	public void setSo(SystemObject so)
	{
		_so = so;
	}

	public E_TSFWeiterschaltenZustand getZustand()
	{
		return _zustand;
	}

	public void setZustand(E_TSFWeiterschaltenZustand zustand)
	{
		_zustand = zustand;
	}

	/**
	 * ID des Clients, fr den diese Weiterschalten bestimmt ist.
	 * 
	 * @return
	 */
	public String getClientId()
	{
		return _clientId;
	}

	public void setClientId(String id)
	{
		_clientId = id;
	}

	public String getKennung()
	{
		return _kennung;
	}

	public void setKennung(String kennung)
	{
		_kennung = kennung;
	}

	public String getKommentar()
	{
		return _kommentar;
	}

	public void setKommentar(String kommentar)
	{
		_kommentar = kommentar;
	}

	public E_TSFBefehl getProgramm()
	{
		return _programm;
	}

	public void setProgramm(E_TSFBefehl programm)
	{
		_programm = programm;
	}

	public long getStufe()
	{
		return _stufe;
	}

	public void setStufe(long stufe)
	{
		_stufe = stufe;
	}

	@Override
	public String toString()
	{
		StringBuffer sb = new StringBuffer();

		if (getSo() != null)
		{
			sb.append(getSo().getNameOrPidOrId());
		} else
		{
			sb.append("-");
		}

		return sb.toString();
	}

	public String toStringLong()
	{
		StringBuffer sb = new StringBuffer();

		sb.append(this.getClass().getSimpleName());
		sb.append("\n");

		/*
		 * SystemObjekt
		 */
		sb.append("\tSystemObjekt: ");
		if (getSo() != null)
		{
			sb.append(getSo());
		} else
		{
			sb.append("-");
		}
		sb.append("\n");

		/*
		 * Zustand
		 */
		sb.append("\tZustand: ");
		if (getZustand() != null)
		{
			sb.append(getZustand().toString());
		} else
		{
			sb.append("-");
		}
		sb.append("\n");

		/*
		 * ID
		 */
		sb.append("\tID: ");
		if (getClientId() != null)
		{
			sb.append(getClientId());
		} else
		{
			sb.append("-");
		}
		sb.append("\n");

		/*
		 * Kennung
		 */
		sb.append("\tKennung: ");
		if (getKennung() != null)
		{
			sb.append(getKennung());
		} else
		{
			sb.append("-");
		}
		sb.append("\n");

		/*
		 * Kommentar
		 */
		sb.append("\tKommentar: ");
		if (getKommentar() != null)
		{
			sb.append(getKommentar());
		} else
		{
			sb.append("-");
		}
		sb.append("\n");

		/*
		 * Programm
		 */
		sb.append("\tProgramm: ");
		if (getProgramm() != null)
		{
			sb.append(getProgramm());
		} else
		{
			sb.append("-");
		}
		sb.append("\n");

		/*
		 * Stufe
		 */
		sb.append("\tStufe: ");
		sb.append(getStufe());

		return sb.toString();
	}

	@Override
	public int compareTo(M_TSFWeiterSchalten o)
	{
		int re = 0;

		if (o != null)
		{
			re = this.toString().compareTo(o.toString());
		}

		return re;
	}

	public M_TSFWeiterSchalten duplicate(boolean setOwnClientId)
	{
		M_TSFWeiterSchalten re = new M_TSFWeiterSchalten();

		// SystemObjekt
		re.setSo(this.getSo());

		// Zustand
		re.setZustand(this.getZustand());

		// ClientID bei Bedarf
		if (setOwnClientId)
		{
			re.setClientId(Defs.getClientId());
		} else
		{
			if (this.getClientId() != null)
			{
				re.setClientId(new String(this.getClientId()));
			}
		}

		// Kennung
		if (this.getKennung() != null)
		{
			re.setKennung(new String(this.getKennung()));
		}

		// Kommentar
		if (this.getKommentar() != null)
		{
			re.setKommentar(new String(this.getKommentar()));
		}

		// Programm
		if (this.getProgramm() != null)
		{
			re.setProgramm(this.getProgramm());
		}

		// Stufe
		re.setStufe(this.getStufe());

		return re;
	}

}
