/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dav;

import java.util.LinkedList;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Reprsentation der TSF Workflow ATG <code>atg.tsfWorkflow</code>. Wird
 * empfangen fr Objekte vom Typ
 * <code>typ.temporaereSeitenstreifenFreigabe</code>.
 * 
 * @author HofmannJ
 * 
 */
public class M_TSFWorkflow
{
	/**
	 * Liste der TSFWorkflowAbschnitte.
	 */
	private LinkedList<M_TSFWorkflowAbschnitt> _lAbschnitte = new LinkedList<M_TSFWorkflowAbschnitt>();

	/**
	 * Objekt vom Typ <code>typ.temporaereSeitenstreifenFreigabe</code>, fr
	 * welches dieser TSFWorkflow empfangen oder gesendet wird.
	 */
	private SystemObject _so = null;

	public M_TSFWorkflow()
	{
	}

	/**
	 * 
	 * @param so
	 *          Objekt vom Typ <code>typ.temporaereSeitenstreifenFreigabe</code>,
	 *          fr welches dieser TSFWorkflow empfangen oder gesendet wird.
	 */
	public M_TSFWorkflow(SystemObject so)
	{
		this._so = so;
	}

	/**
	 * 
	 * @param aAbschnitte
	 *          Array der {@link M_TSFWorkflowAbschnitt}e.
	 */
	public M_TSFWorkflow(M_TSFWorkflowAbschnitt[] aAbschnitte)
	{
		if ((aAbschnitte != null) && (aAbschnitte.length > 0))
		{
			for (M_TSFWorkflowAbschnitt abschnitt : aAbschnitte)
			{
				if (abschnitt != null)
				{
					addAbschnitt(abschnitt);
				}
			}
		}
	}

	/**
	 * 
	 * @param so
	 *          Objekt vom Typ <code>typ.temporaereSeitenstreifenFreigabe</code>,
	 *          fr welches dieser TSFWorkflow empfangen oder gesendet wird.
	 * @param aAbschnitte
	 *          Array der {@link M_TSFWorkflowAbschnitt}e.
	 */
	public M_TSFWorkflow(SystemObject so, M_TSFWorkflowAbschnitt[] aAbschnitte)
	{
		this._so = so;

		if ((aAbschnitte != null) && (aAbschnitte.length > 0))
		{
			for (M_TSFWorkflowAbschnitt abschnitt : aAbschnitte)
			{
				if (abschnitt != null)
				{
					addAbschnitt(abschnitt);
				}
			}
		}
	}

	/**
	 * Liefert das Objekt vom Typ
	 * <code>typ.temporaereSeitenstreifenFreigabe</code>, fr welches dieser
	 * TSFWorkflow empfangen oder gesendet wird.
	 * 
	 * @return Kann <code>null</code> sein, falls das Objekt nicht gesetzt wurde.
	 */
	public SystemObject getSo()
	{
		return _so;
	}

	public M_TSFWorkflowAbschnitt[] getAbschnitte()
	{
		M_TSFWorkflowAbschnitt[] re = null;

		synchronized (_lAbschnitte)
		{
			re = _lAbschnitte
					.toArray(new M_TSFWorkflowAbschnitt[_lAbschnitte.size()]);
		}

		if (re == null)
		{
			re = new M_TSFWorkflowAbschnitt[] {};
		}

		return re;
	}

	public void addAbschnitt(M_TSFWorkflowAbschnitt abschnitt)
	{
		if (abschnitt != null)
		{
			synchronized (_lAbschnitte)
			{
				_lAbschnitte.add(abschnitt);
			}

			abschnitt.setTsfWorkflow(this);
		}
	}

	/**
	 * 
	 * @param abschnitt
	 * @return Liefert <code>true</code> wenn der {@link M_TSFWorkflowAbschnitt}
	 *         entfernt werden konnte, sonst <code>false</code>.
	 */
	public boolean removeAbschnitt(M_TSFWorkflowAbschnitt abschnitt)
	{
		boolean removed = false;

		if (abschnitt != null)
		{
			synchronized (_lAbschnitte)
			{
				removed = _lAbschnitte.remove(abschnitt);
			}
		}

		return removed;
	}

	@Override
	public String toString()
	{
		StringBuffer sb = new StringBuffer();

		sb.append("\n===START===\n");
		sb.append("SystemObjekt TSF: ");
		if (getSo() != null)
		{
			sb.append(getSo());
		} else
		{
			sb.append("-");
		}
		sb.append("\n");

		sb.append("Anzahl Abschnitte: ");
		M_TSFWorkflowAbschnitt[] abschnitte = getAbschnitte();
		if ((abschnitte != null) && (abschnitte.length > 0))
		{
			sb.append(abschnitte.length);
			sb.append("\n");

			for (int i = 0; i < abschnitte.length; i++)
			{
				sb.append("  Abschnit #");
				sb.append(i);
				sb.append(": ");
				if (abschnitte[i] != null)
				{
					sb.append(abschnitte[i].toString());
				} else
				{
					sb.append("-");
				}
				sb.append("\n");
			}

		} else
		{
			sb.append("0");
		}

		sb.append("===ENDE===\n");
		return sb.toString();
	}
	
	public String toStringShort()
	{
		StringBuffer sb = new StringBuffer();

		sb.append("SystemObjekt TSF: ");
		if (getSo() != null)
		{
			sb.append(getSo());
		} else
		{
			sb.append("-");
		}
		
		M_TSFWorkflowAbschnitt[] abschnitte = getAbschnitte();
		if ((abschnitte != null) && (abschnitte.length > 0))
		{
			sb.append(", Anz. Abs.=");
			sb.append(abschnitte.length);
			sb.append(", ");

			for (int i = 0; i < abschnitte.length; i++)
			{
				sb.append(" #");
				sb.append(i);
				sb.append("=");
				if (abschnitte[i] != null)
				{
					sb.append(abschnitte[i].getAbschnitt().getPidOrNameOrId());
				} else
				{
					sb.append("-");
				}

				if((i+1) < abschnitte.length)
				{
					sb.append(",");
				}
			}

		} else
		{
			sb.append(", Keine zugeordneten Abschnitte.");
		}

		return sb.toString();
	}

	public M_TSFWorkflow duplicate()
	{
		M_TSFWorkflow neu = new M_TSFWorkflow(this.getSo());

		M_TSFWorkflowAbschnitt[] abschnitteOriginal = this.getAbschnitte();
		for (M_TSFWorkflowAbschnitt abschnittOrg : abschnitteOriginal)
		{
			neu.addAbschnitt(abschnittOrg.duplicate());
		}

		return neu;
	}

	/**
	 * Prft ob von allen {@link M_TSFWorkflowAbschnitt}en die jeweiligen
	 * Programme geschaltet sind.
	 * 
	 * @return
	 */
	public boolean sindAlleZustaendeErreicht()
	{
		boolean re = true;

		M_TSFWorkflowAbschnitt[] aAbschnitte = getAbschnitte();
		if (aAbschnitte != null)
		{
			for (M_TSFWorkflowAbschnitt wfAbs : aAbschnitte)
			{
				if (wfAbs != null)
				{
					boolean wfAbschnittZustandErreicht = wfAbs.isZustandErreicht();

					if (!wfAbschnittZustandErreicht)
					{
						re = false;
						break;
					}
				}
			}
		}
		return re;
	}
	
	/**
	 * Prft ob von allen {@link M_TSFWorkflowAbschnitt}en die jeweiligen
	 * Programme geschaltet sind.
	 * 
	 * @return
	 */
	public boolean sindAlleZustaendeInBearbeitung()
	{
	  boolean re = true;
	  
	  M_TSFWorkflowAbschnitt[] aAbschnitte = getAbschnitte();
	  if (aAbschnitte != null)
	  {
	    for (M_TSFWorkflowAbschnitt wfAbs : aAbschnitte)
	    {
	      if (wfAbs != null)
	      {
	        boolean wfAbschnittZustandErreicht = wfAbs.isZustandInBearbeitung();
	        
	        if (!wfAbschnittZustandErreicht)
	        {
	          re = false;
	          break;
	        }
	      }
	    }
	  }
	  return re;
	}

	public boolean isGleich(M_TSFWorkflow o)
	{
		boolean gleich = false;

		if (o != null)
		{
			if (o.getSo() == this.getSo())
			{
				M_TSFWorkflowAbschnitt[] oAbschnitte = o.getAbschnitte();
				M_TSFWorkflowAbschnitt[] tAbschnitte = this.getAbschnitte();

				if (oAbschnitte.length == tAbschnitte.length)
				{
					boolean abschnitteGleich = true;
					for (int i = 0; i < tAbschnitte.length; i++)
					{
						M_TSFWorkflowAbschnitt oABS = oAbschnitte[i];
						M_TSFWorkflowAbschnitt tABS = tAbschnitte[i];

						if (!tABS.isGleich(oABS))
						{
							abschnitteGleich = false;
						}
					}

					gleich = abschnitteGleich;
				}
			}
		}

		return gleich;
	}
}
