/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.awt.Toolkit;
import java.util.Vector;

import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.sts.buv.plugin.workflow.tsf.Activator;
import de.sts.buv.plugin.workflow.tsf.DefsFonts;
import de.sts.buv.plugin.workflow.tsf.ImageGetter;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFAbschnitte;
import de.sts.buv.plugin.workflow.tsf.dav.Defs;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFAbschnitte;

/**
 * Dialog zum Schalten der Abschnitte eines Bereichs.
 * 
 * @author HofmannJ
 * 
 */
public class DialogTSFBereichSchalten extends Dialog
{
	/**
	 * DEF-Breite
	 */
	private static final int DEF_WIDTH = 600; // 650;

	/**
	 * DEF-Hhe
	 */
	private static final int DEF_HEIGHT = 450; // 400;

	/**
	 * Image links
	 */
	private static Image imageLinks;

	/**
	 * Imgae rechts
	 */
	private static Image _imageRechts;

	/**
	 * Shell
	 */
	private Shell _shell;

	/**
	 * Die zu schaltenden Abschnitte eines Bereichs.
	 */
	private M_TSFAbschnitte _tsfAbschnitte;

	private Composite _compAbschnitt;

	private Composite _compTitel;

	private Group _grpAbschnitte;

	private TableViewer _tv;

	private Composite _compCtrl;
	private Button _bSchalten;

	private Composite _compAuswahl;

	private Text _tZustand;

	private ComboViewer _cvMoeglicheZustaende;

	private TSFBereichSchalten _tsfBereichSchalten;

	private Label _lAbschnitt;

	private Composite _compInfoTxt;

	private Label _lInfoTxt;

	public DialogTSFBereichSchalten(Shell parent)
	{
		super(parent);
	}

	/**
	 * ffnet den Dialog.
	 * 
	 * @param pidTsf
	 *          Pid einer TFS vom Typ
	 *          <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 * @return
	 */
	public Object oeffne(String pidTsf)
	{
		Object re = null;

		if (pidTsf != null)
		{
			M_TSFAbschnitte tsfAbschnitte = DaVKomTSFAbschnitte.getInstance()
					.getMTSFabschnitte(pidTsf);

			Vector<SystemObject> aktiveAbschnitte = new Vector<SystemObject>();
			for (SystemObject so : tsfAbschnitte.getAbschnitte())
			{
				if (Defs.pruefeObSteuerungFuerAbschnittInBearbeitung(so.getPid()))
					aktiveAbschnitte.add(so);
			}

			if (aktiveAbschnitte.size() > 0)
			{
				Shell shell = ShellOnTop.getShell(getParent(), SWT.TITLE | SWT.BORDER);

				shell.addDisposeListener(new DisposeListener()
				{
					@Override
					public void widgetDisposed(DisposeEvent e)
					{
						dispose();
					}
				});

				MessageBox mb = new MessageBox(shell, SWT.OK | SWT.ICON_WARNING);
				mb.setText("Bereich schalten");

				StringBuffer sbMsg = new StringBuffer();
				sbMsg
						.append("Folgende Abschnitte des Bereichs befinden sich zur Zeit\n");
				sbMsg.append("in einer aktiven Schaltung:\n\n");

				for (SystemObject so : aktiveAbschnitte)
					sbMsg.append("-" + so.getNameOrPidOrId() + "\n");

				sbMsg
						.append("\nEine Schaltung des Bereichs ist daher im Moment nicht mglich !");
				mb.setMessage(sbMsg.toString());

				mb.open();
			} else
				re = oeffne(tsfAbschnitte);
		}

		return re;
	}

	/**
	 * ffnet den Dialog.
	 * 
	 * @param soTsf
	 *          {@link SystemObject} einer TFS vom Typ
	 *          <code>typ.temporaereSeitenstreifenFreigabe</code>.
	 * @return
	 */
	public Object oeffne(SystemObject soTsf)
	{
		Object re = null;

		if (soTsf != null)
		{
			M_TSFAbschnitte tsfAbschnitte = DaVKomTSFAbschnitte.getInstance()
					.getMTSFabschnitte(soTsf);
			re = oeffne(tsfAbschnitte);
		}

		return re;
	}

	public Object oeffne(M_TSFAbschnitte tsfAbschnitte)
	{
		if (tsfAbschnitte != null)
		{
			this._tsfAbschnitte = tsfAbschnitte;

			this._tsfBereichSchalten = new TSFBereichSchalten(_tsfAbschnitte);

			this._shell = ShellOnTop.getShell(getParent(), SWT.TITLE | SWT.BORDER);

			this._shell.addDisposeListener(new DisposeListener()
			{
				@Override
				public void widgetDisposed(DisposeEvent e)
				{
					dispose();
				}
			});

			/*
			 * Darstellung aufbauen
			 */
			initGui(_shell);

			/*
			 * Zentrieren
			 */

			// Grsse Bildschirm
			Toolkit test = java.awt.Toolkit.getDefaultToolkit();

			/*
			 * Auflsung durch 2 subtrahiert durch die Hlfte der in der Shell
			 * angegebenen Horizontalen, damit der Dialog mittig am Bildschirm
			 * erscheint
			 */
			int x = test.getScreenSize().width / 2 - DEF_WIDTH / 2;

			/*
			 * Auflsung durch 2 subtrahiert durch die Hlfte der in der Shell
			 * angegebenen Verzikalen
			 */
			int y = test.getScreenSize().height / 2 - DEF_HEIGHT / 2;

			_shell.setBounds(x, y, DEF_WIDTH, DEF_HEIGHT);

			_shell.open();

			Display display = getParent().getDisplay();
			while (!_shell.isDisposed())
				if (!display.readAndDispatch())
					display.sleep();

		}

		return null;
	}

	private void initGui(Shell parent)
	{
		parent.setText("Temporre Seitenstreifenfreigabe Bereich schalten");
		
		 //Fr die F1 Taste
	    PlatformUI.getWorkbench().getHelpSystem().setHelp( parent, Activator.PLUGIN_ID + ".tsfBereichSchalten" );

		// getCompTitel(parent);
		getCompositeAbschnitt(parent);
		getCompositeInfotext(parent);

		getGrpAbs(parent);
		getCompAuswahl(parent);
		getCompCtrl(parent);

		/*
		 * Layout
		 */
		parent.setLayout(new GridLayout(1, false));

		GridData gdLAbs = new GridData();
		gdLAbs.horizontalSpan = 1;
		gdLAbs.grabExcessHorizontalSpace = true;
		gdLAbs.horizontalAlignment = GridData.FILL;
		getCompositeAbschnitt(parent).setLayoutData(gdLAbs);

		GridData gdCompInfoText = new GridData();
		gdCompInfoText.grabExcessHorizontalSpace = true;
		gdCompInfoText.grabExcessVerticalSpace = true;
		gdCompInfoText.horizontalAlignment = GridData.FILL;
		gdCompInfoText.verticalAlignment = GridData.FILL;
		getCompositeInfotext(parent).setLayoutData(gdCompInfoText);

		// GridData gdCompTitel = new GridData();
		// gdCompTitel.grabExcessHorizontalSpace = true;
		// gdCompTitel.horizontalAlignment = GridData.FILL;
		// getCompTitel(parent).setLayoutData(gdCompTitel);

		GridData gdGrpAbs = new GridData();
		gdGrpAbs.grabExcessHorizontalSpace = true;
		gdGrpAbs.horizontalAlignment = GridData.FILL;
		gdGrpAbs.minimumHeight = 100;
		getGrpAbs(parent).setLayoutData(gdGrpAbs);

		GridData gdCompAuswahl = new GridData();
		gdCompAuswahl.grabExcessHorizontalSpace = true;
		gdCompAuswahl.grabExcessVerticalSpace = true;
		gdCompAuswahl.horizontalAlignment = GridData.FILL;
		gdCompAuswahl.verticalAlignment = GridData.FILL;
		// Setzen fr Content
		getCompAuswahl(parent).setLayoutData(gdCompAuswahl);

		GridData gdCompCtrl = new GridData();
		gdCompCtrl.grabExcessHorizontalSpace = true;
		gdCompCtrl.horizontalAlignment = GridData.CENTER;
		getCompCtrl(parent).setLayoutData(gdCompCtrl);
	}

	private Composite getCompositeAbschnitt(Composite parent)
	{
		if (_compAbschnitt == null)
		{
			_compAbschnitt = new Composite(parent, SWT.BORDER);

			_lAbschnitt = new Label(_compAbschnitt, SWT.NONE);
//			Font fontAbschnitt = new Font(null, "Arial", 12, SWT.BOLD);
			_lAbschnitt.setFont(DefsFonts.FONT_ARIAL_12_PKT_BOLD);
			_lAbschnitt.setText("Bereich: ");

			_compAbschnitt.setLayout(new GridLayout(1, false));

			GridData gdLAbschnitt = new GridData();
			gdLAbschnitt.grabExcessHorizontalSpace = true;
			gdLAbschnitt.horizontalAlignment = GridData.CENTER;
			_lAbschnitt.setLayoutData(gdLAbschnitt);

			SystemObject soBereich = getTsfAbschnitte().getSo();

			if (soBereich != null)
			{
				_lAbschnitt.setText(soBereich.getNameOrPidOrId());
			} else
			{
				_lAbschnitt.setText("Bereich: KEIN BEREICH GESETZT");
			}
		}

		return _compAbschnitt;
	}

	private Composite getCompositeInfotext(Composite parent)
	{
		if (_compInfoTxt == null)
		{
			_compInfoTxt = new Composite(parent, SWT.BORDER);

			Label lFrag1 = new Label(_compInfoTxt, SWT.None);
			// lFrag1.setImage(getAchtungImage());
			lFrag1.setImage(getImageLinks());

			_lInfoTxt = new Label(_compInfoTxt, SWT.CENTER);
//			Font fontInfoTxt = new Font(null, "Arial", 18, SWT.BOLD);
			_lInfoTxt.setFont(DefsFonts.FONT_ARIAL_18_PKT_BOLD);
			// _lInfoTxt.setText("Ist der Abschnitt frei und\nkann geschalten werden?");
			_lInfoTxt.setText("Temporre\nSeitenstreifenfreigabe\nschalten");

			Label lFrag2 = new Label(_compInfoTxt, SWT.None);
			// lFrag2.setImage(getAchtungImage());
			lFrag2.setImage(getImageRechts());

			_compInfoTxt.setLayout(new GridLayout(3, false));

			GridData gdLFrag1 = new GridData();
			gdLFrag1.grabExcessHorizontalSpace = true;
			gdLFrag1.grabExcessVerticalSpace = true;
			gdLFrag1.horizontalAlignment = GridData.CENTER;
			gdLFrag1.verticalAlignment = GridData.CENTER;
			lFrag1.setLayoutData(gdLFrag1);

			GridData gdLInfoTxt = new GridData();
			gdLInfoTxt.grabExcessHorizontalSpace = true;
			gdLInfoTxt.horizontalAlignment = GridData.CENTER;
			_lInfoTxt.setLayoutData(gdLInfoTxt);

			GridData gdLFrag2 = new GridData();
			gdLFrag2.grabExcessHorizontalSpace = true;
			gdLFrag2.grabExcessVerticalSpace = true;
			gdLFrag2.horizontalAlignment = GridData.CENTER;
			gdLFrag2.verticalAlignment = GridData.CENTER;
			lFrag2.setLayoutData(gdLFrag2);
		}

		return _compInfoTxt;
	}

	private Composite getCompTitel(Composite parent)
	{
		if (_compTitel == null)
		{
			_compTitel = new Composite(parent, SWT.BORDER);

			// Image Links
			Label l1 = new Label(_compTitel, SWT.NONE);
			l1.setImage(ImageGetter.getImage("icons/system-shutdown_64x64.png"));

			// Abschnitt
			Label lBereich = new Label(_compTitel, SWT.NONE);
//			Font fLA = new Font(null, "Arial", 14, SWT.BOLD);
			lBereich.setFont(DefsFonts.FONT_ARIAL_14_PKT_BOLD);

			if (getTsfAbschnitte() != null)
			{
				SystemObject soBereich = getTsfAbschnitte().getSo();

				if (soBereich != null)
				{
					lBereich.setText("Bereich: " + soBereich.getNameOrPidOrId());
				} else
				{
					lBereich.setText("Bereich: KEIN BEREICH GESETZT");
				}
			}

			// Image rechts
			Label l2 = new Label(_compTitel, SWT.NONE);
			l2.setImage(ImageGetter.getImage("icons/system-shutdown_64x64.png"));

			/*
			 * Layout
			 */
			_compTitel.setLayout(new GridLayout(3, false));

			GridData gdLBereich = new GridData();
			gdLBereich.grabExcessHorizontalSpace = true;
			gdLBereich.horizontalAlignment = GridData.CENTER;
			lBereich.setLayoutData(gdLBereich);
		}

		return _compTitel;
	}

	private Group getGrpAbs(Composite parent)
	{
		if ((_grpAbschnitte == null) && (parent != null))
		{
			_grpAbschnitte = new Group(parent, SWT.NONE);
			_grpAbschnitte.setText("Abschnitte");

			Table table = new Table(_grpAbschnitte, SWT.FULL_SELECTION | SWT.SINGLE
					| SWT.H_SCROLL | SWT.V_SCROLL);
			table.setHeaderVisible(true);
			table.setLinesVisible(true);

			_tv = new TableViewer(table);
			_tv.setContentProvider(new CP());
			_tv.setLabelProvider(new LP());
			_tv.setInput(getTsfAbschnitte());

			TableColumn tcAbschnitt = new TableColumn(_tv.getTable(), SWT.LEFT);
			tcAbschnitt.setText("Abschnitt");
			tcAbschnitt.setWidth(300);

			_grpAbschnitte.setLayout(new FillLayout());
		}

		return _grpAbschnitte;
	}

	private Composite getCompAuswahl(Composite parent)
	{
		if (_compAuswahl == null)
		{
			_compAuswahl = new Composite(parent, SWT.BORDER);

			Label lZustand = new Label(_compAuswahl, SWT.NONE);
			lZustand.setText("Zustand: ");

			_tZustand = new Text(_compAuswahl, SWT.BORDER);
			_tZustand.setEditable(false);

			Label lMoeglicheZustaende = new Label(_compAuswahl, SWT.NONE);
			lMoeglicheZustaende.setText("Mgliche Zustnde: ");

			Combo comboMoeglicheZustaende = new Combo(_compAuswahl, SWT.NONE);
			_cvMoeglicheZustaende = new ComboViewer(comboMoeglicheZustaende);
			_cvMoeglicheZustaende.setContentProvider(new ComboMoeglicheZustaendeCP());

			/*
			 * MO 02.09.11 Button Schalten erst dann aktivieren, wenn auch ein
			 * Programm angewhlt wurde
			 */

			comboMoeglicheZustaende.addSelectionListener(new SelectionListener()
			{
				@Override
				public void widgetSelected(SelectionEvent e)
				{
					ISelection selection = _cvMoeglicheZustaende.getSelection();
					if (selection instanceof StructuredSelection)
					{
						StructuredSelection ss = (StructuredSelection) selection;
						Object firstElement = ss.getFirstElement();
						if ((firstElement != null) && (firstElement instanceof E_TSFBefehl))
						{
							E_TSFBefehl befehl = (E_TSFBefehl) firstElement;

							if (befehl != null)
								_bSchalten.setEnabled(true);
						}
					}
				}

				@Override
				public void widgetDefaultSelected(SelectionEvent e)
				{
					// TODO Auto-generated method stub

				}
			});

			TSFBereichSchalten tsfBereichSchalten = getTsfBereichSchalten();
			if (tsfBereichSchalten != null)
			{
				// /*
				// * Aktuelles Prg. des letzten Abschnitts bestimmen
				// */
				// E_TSFBefehl aktProgrammLetzerAbschnitt =
				// tsfBereichSchalten.bestimmeAktProgrammLetzerAbschnitt();
				// if (aktProgrammLetzerAbschnitt != null)
				// {
				// _tZusatand.setText(aktProgrammLetzerAbschnitt.toString());
				// }

				/*
				 * MO 02.09.11 Das Bestimmen des aktuellen Zustandes erfolgt nicht auf
				 * Basis des letzten Abschnitts sonder auf Basis aller Abschnitte. Nur
				 * dann ist es mglich im Falle dass ein Abschnitt Frei der andere aber
				 * gesperrt ist, beide auf Frei bzw. Gesperrt zu setzen
				 */

				/*
				 * Aktuelles Prg. des Bereichs bestimmen
				 */
				String aktProgrammLetzerAbschnitt = tsfBereichSchalten
						.bestimmeAktProgrammBereich();

				if (aktProgrammLetzerAbschnitt != null)
				{
					Color rot = new Color(_tZustand.getDisplay(), 255, 0, 0);
					Color gruen = new Color(_tZustand.getDisplay(), 0, 128, 0);

					_tZustand.setText(aktProgrammLetzerAbschnitt);

					if ((aktProgrammLetzerAbschnitt.equals(E_TSFBefehl.Freigabe.name()))
							|| (aktProgrammLetzerAbschnitt
									.equals(E_TSFBefehl.FreigabeArgoscan.name())))
					{
						_tZustand.setForeground(gruen);
					}

					if (aktProgrammLetzerAbschnitt.equals(E_TSFBefehl.Sperrung.name()))
					{
						_tZustand.setForeground(rot);
					}
				} else
				{
					logge(
							"Das aktuell geschaltete Programm lsst sich fr den Dialog Bereich Schalten nicht bestimmen, da noch kein Programm geschaltet wurde.",
							Status.INFO);
					_tZustand.setText("-");
				}

				/*
				 * MO 02.09.11 Schaltbare Prg. fr die Combobox bestimmen.
				 */
				// E_TSFBefehl[] moeglicheSchaltbareProgramme =
				// tsfBereichSchalten.bestimmeSchaltbareProgramme();
				E_TSFBefehl[] moeglicheSchaltbareProgramme = tsfBereichSchalten
						.bestimmeSchaltbareProgramme(aktProgrammLetzerAbschnitt);

				_cvMoeglicheZustaende.setInput(moeglicheSchaltbareProgramme);

				/*
				 * MO 05.09.11
				 */
				if (moeglicheSchaltbareProgramme.length > 0)
				{
					_cvMoeglicheZustaende.setSelection(new StructuredSelection(
							moeglicheSchaltbareProgramme[0]));
					// _bSchalten.setEnabled(true);
				}

			}

			/*
			 * Layout
			 */
			_compAuswahl.setLayout(new GridLayout(2, false));

			GridData gdTZustand = new GridData();
			gdTZustand.grabExcessHorizontalSpace = true;
			gdTZustand.horizontalAlignment = GridData.FILL;
			_tZustand.setLayoutData(gdTZustand);

			GridData gdCZustand = new GridData();
			gdCZustand.grabExcessHorizontalSpace = true;
			gdCZustand.horizontalAlignment = GridData.FILL;
			comboMoeglicheZustaende.setLayoutData(gdCZustand);

		}

		return _compAuswahl;
	}

	private void logge(String msg, int serv)
	{
		ILog log = Activator.getDefault().getLog();
		log.log(new Status(serv, Activator.PLUGIN_ID, msg));
	}

	private Composite getCompCtrl(Composite parent)
	{
		if (_compCtrl == null)
		{
			_compCtrl = new Composite(parent, SWT.NONE);

			_bSchalten = new Button(_compCtrl, SWT.PUSH);
			_bSchalten.setText("Schalten");
			// _bSchalten.setEnabled(false);

			_bSchalten.addSelectionListener(new SelectionAdapter()
			{
				@Override
				public void widgetSelected(SelectionEvent e)
				{
					aktionSchalten();
				}
			});

			// Button bAbbrechen = new Button(_compCtrl, SWT.PUSH);
			// bAbbrechen.setText("TSF Abbrechen");
			// bAbbrechen.addSelectionListener(new SelectionAdapter()
			// {
			// @Override
			// public void widgetSelected(SelectionEvent e)
			// {
			// aktionAbbrechen();
			// }
			// });

			Button bSchliessen = new Button(_compCtrl, SWT.PUSH);
			bSchliessen.setText("Schlieen");
			bSchliessen.addSelectionListener(new SelectionAdapter()
			{

				@Override
				public void widgetSelected(SelectionEvent e)
				{
					/*
					 * MO 02.09.11 Wenn kein Job zu TSFBereichSchalten aktiv ist, muss
					 * dieses noch zustzlich disposed werden.
					 */
					TSFBereichSchalten tsfBereichSchalten = getTsfBereichSchalten();
					if ((tsfBereichSchalten != null)
							&& (!tsfBereichSchalten.istJobAktiv()))
					{
						tsfBereichSchalten.dispose();
					}

					if (_shell != null)
					{
						_shell.dispose();
					}
				}

			});

			/*
			 * Layout
			 */
			_compCtrl.setLayout(new RowLayout());
		}

		return _compCtrl;
	}

	private void aktionSchalten()
	{
		TSFBereichSchalten tsfBereichSchalten = getTsfBereichSchalten();
		if (tsfBereichSchalten != null)
		{
			// Zu schaltendes Programm bestimmen
			ISelection selection = _cvMoeglicheZustaende.getSelection();
			if (selection instanceof StructuredSelection)
			{
				StructuredSelection ss = (StructuredSelection) selection;
				Object firstElement = ss.getFirstElement();
				if ((firstElement != null) && (firstElement instanceof E_TSFBefehl))
				{
					E_TSFBefehl befehl = (E_TSFBefehl) firstElement;
					tsfBereichSchalten.doStartProgramm(befehl);
				}
			}
		}

		closeDialog();
	}

	public M_TSFAbschnitte getTsfAbschnitte()
	{
		return _tsfAbschnitte;
	}

	/**
	 * Liefert die Instanz des Prozesses zum Schalten vom Typ
	 * {@link TSFBereichSchalten}.
	 * 
	 * @return
	 */
	public TSFBereichSchalten getTsfBereichSchalten()
	{
		return _tsfBereichSchalten;
	}

	/**
	 * Fhrt Aufrumarbeiten durch.
	 */
	public void dispose()
	{
		/*
		 * Wenn kein Job zu TSFBereichSchalten aktiv ist, muss dieses noch
		 * zustzlich disposed werden.
		 */
		TSFBereichSchalten tsfBereichSchalten = getTsfBereichSchalten();
		if ((tsfBereichSchalten != null) && (!tsfBereichSchalten.istJobAktiv()))
		{
			/*
			 * MO 02.09.11 Der Aufruf von Dispose bewirkt, dass der Thread beendet
			 * wird, daher dispose nicht aufrufen, wenn der Ablauf als Thread
			 * realisiert wird
			 */
			if (Activator._starteAlsJob)
				tsfBereichSchalten.dispose();
		}

		closeDialog();
	}

	public void closeDialog()
	{
		if (_shell != null)
		{
			_shell.dispose();
		}
	}

	private class CP implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if ((inputElement != null) && (inputElement instanceof M_TSFAbschnitte))
			{
				M_TSFAbschnitte tsfAbschnitte = (M_TSFAbschnitte) inputElement;

				if (tsfAbschnitte != null)
				{
					SystemObject[] soAbs = tsfAbschnitte.getAbschnitte();
					re = soAbs;
				}
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	private class LP implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			return null;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = null;

			if ((element != null) && (element instanceof SystemObject))
			{
				SystemObject so = (SystemObject) element;
				re = so.getNameOrPidOrId();
			}

			if (re == null)
			{
				re = "-";
			}
			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	private class ComboMoeglicheZustaendeCP implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;
			if (inputElement != null)
			{
				if (inputElement instanceof E_TSFBefehl[])
				{
					re = (E_TSFBefehl[]) inputElement;
				} else if (inputElement instanceof TSFBereichSchalten)
				{
					TSFBereichSchalten tsfBereichSchalten = (TSFBereichSchalten) inputElement;
					re = tsfBereichSchalten.bestimmeSchaltbareProgramme();
				}
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	private static Image getImageLinks()
	{
		if (imageLinks == null)
		{
			imageLinks = Activator.getImageDescriptor("icons/tsf_links.png")
					.createImage();
		}

		return imageLinks;
	}

	private static Image getImageRechts()
	{
		if (_imageRechts == null)
		{
			_imageRechts = Activator.getImageDescriptor("icons/tsf_rechts.png")
					.createImage();
		}

		return _imageRechts;
	}

}
