/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

/**
 * Interface welches von wiedervorlegbaren Elementen implementiert werden
 * sollte.
 * 
 * @author HofmannJ
 * 
 */
public interface IWiedervorlage
{
	/**
	 * Liefert die ID des wiedervorzulegenden Elementes.
	 * 
	 * @return ID des wiedervorzulegenden Elementes.
	 */
	public String getId();

	/**
	 * Liefert die Beschreibung zum wiedervorzulegenden Elment.
	 * 
	 * @return Beschreibung zum wiedervorzulegenden Elment.
	 */
	public String getBeschreibung();

	/**
	 * Liefert den Timeout zum Wiedervorzulegenden Element.
	 * 
	 * @return Timeout zum Wiedervorzulegenden Element.
	 */
	public long getTimeout();

	/**
	 * Liefert den Startzeitpunkt, zu dem das "Warten" auf die Wiedervorlage
	 * gestartet wurde.
	 * 
	 * @return Startzeitpunkt.
	 */
	public Long getStartZeitpunkt();

	/**
	 * Lst eine vorzeigtige Wiedervorlage des Elementes aus.
	 */
	public void vorzeitigeWiedervorlage();
}
