/*******************************************************************************
 * This file is part of Rahmenwerk.
 * 
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 * 
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 *  
 *  
 * 
 * Copyright (c) 2012 SWARCO TRAFFIC SYSTEMS GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 * SWARCO TRAFFIC SYSTEMS GmbH - initial API and implementation
 * 
 * 
 * 
 * Contact Information:
 * SWARCO TRAFFIC SYSTEMS GmbH
 * Adolf-Dambach-Strae 
 * D-76571 Gaggenau
 * Phone: +49-7225-64-03
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 ******************************************************************************/
package de.sts.buv.plugin.workflow.tsf.dlg;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Collection;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.TreeMap;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bsvrz.buv.rw.basislib.seiteneinstellungen.Zusammelstellung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.sts.buv.plugin.workflow.tsf.Activator;
import de.sts.buv.plugin.workflow.tsf.ImageGetter;
import de.sts.buv.plugin.workflow.tsf.dav.DaVKomTSFSteuerung;
import de.sts.buv.plugin.workflow.tsf.dav.Defs;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFBefehl;
import de.sts.buv.plugin.workflow.tsf.dav.E_TSFProgrammZustand;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFSteuerung;
import de.sts.buv.plugin.workflow.tsf.dav.I_TSFZustand;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFZustandArgoscan;
import de.sts.buv.plugin.workflow.tsf.dav.M_TSFZustandSteuerung;
import de.sts.buv.plugin.workflow.tsf.dav.GUIModelTSFSteuerung.ZustandSammlung;

/**
 * View mit der bersicht der Steuerung des Workflows.
 * 
 * @author HofmannJ
 * 
 */
public class ViewUebersichtSteuerung extends ViewPart implements
		PropertyChangeListener
{
	public static final String ID = "de.sts.buv.plugin.workflow.tsf.dlg.ViewUebersichtSteuerung";

	private static final String[] COL_NAMES = new String[] { "Abschnitt",
			"Zeitstempel", "Zielprogramm", "Zustand Programm",
			"Aktive Bereichsschaltung", "Zustand Steuerung", "Stufe",
//			"Zustand Argoscan", "Makro Ereignis", "Zustand Argoscan" };
			"Auslser","Benutzer", "Ursache"};

	private static final int IDX_COL_ABSCHNITT = 0;
	private static final int IDX_COL_ZEITSTEMPEL = 1;
	private static final int IDX_COL_ZIELPROGRAMM = 2;
	private static final int IDX_COL_ZUSTAND_PRG = 3;
	private static final int IDX_COL_AKT_BEREICHSSCHALTUNG = 4;
	private static final int IDX_COL_ZUSTAND_STRG = 5;
	private static final int IDX_COL_ZUSTAND_STRG_STUFE = 6;
//	private static final int IDX_COL_ZUSTAND_ARGOSCAN = 7;
	private static final int IDX_COL_AUSLOESER = 7;
	private static final int IDX_COL_BENUTZER = 8;
	private static final int IDX_COL_URSCHE = 9;

	private TreeMap<String, ZustandSammlung> _map = new TreeMap<String, ZustandSammlung>();

	private TableViewer _tv;

	private TreeMap<String, GeneratorCompositeZustand> _mapCompositeZustaende = new TreeMap<String, GeneratorCompositeZustand>();

	private GeneratorCompositeWiedervorlagen _generatorCompositeWFSchaltempfehlungenUebersicht;

	private Action _aAbschnittSchalten;
	
	private Composite _parent;

	@Override
	public void createPartControl(Composite parent)
	{
		_parent = parent;
		
		 //Fr die F1 Taste
	    PlatformUI.getWorkbench().getHelpSystem().setHelp( parent, Activator.PLUGIN_ID + ".tsfUebersichtSteuerung" );
		
		CTabFolder folder = new CTabFolder(parent, SWT.BORDER);

		/*
		 * Allgemeine Tabelle
		 */
		CTabItem itemUebersicht = new CTabItem(folder, SWT.NONE);
		itemUebersicht.setText("bersicht");
		// Composite tableParent = new Composite(folder, SWT.NONE);
		// tableParent.setLayout(new FillLayout());
		// // tableParent.setBackground(new Color(null, 255, 0, 0));
		// itemUebersicht.setControl(tableParent);
		// createTable(tableParent);

		Composite comp1stFolderTop = new Composite(folder, SWT.NONE);
		itemUebersicht.setControl(comp1stFolderTop);
		comp1stFolderTop.setLayout(new GridLayout(1, false));

		Group grpOben = new Group(comp1stFolderTop, SWT.NONE);
		grpOben.setText("Zustand Steuerung");
		createTable(grpOben);

		Group grpUnten = new Group(comp1stFolderTop, SWT.NONE);
		grpUnten.setText("Anliegende Wiedervorlagen");

		grpOben.setLayout(new FillLayout());
		GridData gdGrpOben = new GridData();
		gdGrpOben.grabExcessHorizontalSpace = true;
		gdGrpOben.grabExcessVerticalSpace = true;
		gdGrpOben.horizontalAlignment = GridData.FILL;
		gdGrpOben.verticalAlignment = GridData.FILL;
		grpOben.setLayoutData(gdGrpOben);

		GridData gdGrpUnten = new GridData();
		gdGrpUnten.grabExcessHorizontalSpace = true;
		gdGrpUnten.grabExcessVerticalSpace = true;
		gdGrpUnten.horizontalAlignment = GridData.FILL;
		gdGrpUnten.verticalAlignment = GridData.FILL;
		grpUnten.setLayoutData(gdGrpUnten);

		_generatorCompositeWFSchaltempfehlungenUebersicht = new GeneratorCompositeWiedervorlagen(
				grpUnten, this);

		/*
		 * Spezialisiert fr alle Elemente vom Typ typ.tsfAbschnitt.
		 */
		ClientDavInterface con = DaVVerbindung.getInstanz().getDaVVerbindung();
		if (con != null)
		{
			SystemObjectType sot = con.getDataModel().getType(Defs.TYP_TSF_ABSCHNITT);
			if (sot != null)
			{
				TreeMap<String, SystemObject> tm = new TreeMap<String, SystemObject>();
				List<SystemObject> lObjs = sot.getObjects();
				for (SystemObject so : lObjs)
				{
					tm.put(so.getPid(), so);
				}

				if (tm.size() > 0)
				{
					for (SystemObject so : tm.values())
					{
						GeneratorCompositeZustand gen = new GeneratorCompositeZustand(so);
						synchronized (_mapCompositeZustaende)
						{
							_mapCompositeZustaende.put(gen.getSO().getPid(), gen);
						}
						Composite comp = gen.getComp(folder);
						CTabItem item = new CTabItem(folder, SWT.NONE);
						item.setText(so.getName());
						item.setControl(comp);
						gen.setTabItem(item);

					}
				}
			}
		}

		folder.setSelection(itemUebersicht);

		listenerAnmelden();
	}

	private void createTable(Composite parent)
	{
		Table table = new Table(parent, SWT.FULL_SELECTION | SWT.SINGLE
				| SWT.H_SCROLL | SWT.V_SCROLL);
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		_tv = new TableViewer(table);
		_tv.setContentProvider(new CPSteuerung());
		_tv.setLabelProvider(new LPSteuerung());
		
		ColumnViewerToolTipSupport.enableFor(_tv, ToolTip.NO_RECREATE);
//    FancyToolTipSupport.enableFor(_tv, ToolTip.NO_RECREATE);

		TableColumn tcAbschnitt = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcAbschnitt.setText(COL_NAMES[IDX_COL_ABSCHNITT]);
		tcAbschnitt.setWidth(300);

		TableColumn tcZeitstempel = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcZeitstempel.setText(COL_NAMES[IDX_COL_ZEITSTEMPEL]);
		tcZeitstempel.setWidth(150);

		TableColumn tcZustandPrg = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcZustandPrg.setText(COL_NAMES[IDX_COL_ZIELPROGRAMM]);
		tcZustandPrg.setWidth(150);

		TableColumn tcZustandPRG = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcZustandPRG.setText(COL_NAMES[IDX_COL_ZUSTAND_PRG]);
		tcZustandPRG.setWidth(150);

		TableColumn tcAktBerSchaltung = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcAktBerSchaltung.setText(COL_NAMES[IDX_COL_AKT_BEREICHSSCHALTUNG]);
		tcAktBerSchaltung.setWidth(150);

		TableColumn tcZustandStrg = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcZustandStrg.setText(COL_NAMES[IDX_COL_ZUSTAND_STRG]);
		tcZustandStrg.setWidth(150);

		TableColumn tcStrgStufe = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcStrgStufe.setText(COL_NAMES[IDX_COL_ZUSTAND_STRG_STUFE]);
		tcStrgStufe.setWidth(100);

//		TableColumn tcZustandArgoscan = new TableColumn(_tv.getTable(), SWT.LEFT);
//		tcZustandArgoscan.setText(COL_NAMES[IDX_COL_ZUSTAND_ARGOSCAN]);
//		tcZustandArgoscan.setWidth(200);
		
		TableColumn tcAusloeser = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcAusloeser.setText(COL_NAMES[IDX_COL_AUSLOESER]);
	  tcAusloeser.setWidth(100);		
		TableViewerColumn col = new TableViewerColumn(_tv, tcAusloeser);
		col.setLabelProvider(new CellLabelProvider() {
//		  @Override
//		  public void update(ViewerCell cell) {
//		    cell.setText(((Person) cell.getElement()).getLastName());
//		  }

		  @Override
		  public String getToolTipText(Object element) {
		    return bestimmeToolTipTextAusloeser((ZustandSammlung)element);
//		    return "<html><body>Tooltip (" + bestimmeToolTipTextAusloeser((ZustandSammlung)element) + ")<br /><a href='http://www.bestsolution.at' target='_NEW'>www.bestsolution.at</a></body></html>";
		  }

		  @Override
		  public Point getToolTipShift(Object object) {
		    return new Point(5, 5);
		  }

		  @Override
		  public int getToolTipDisplayDelayTime(Object object) {
		    return 100; //msec
		  }

		  @Override
		  public int getToolTipTimeDisplayed(Object object) {
		    return 5000; //msec
		  }

      @Override
      public void update(ViewerCell cell)
      {
        cell.setText(bestimmeTextAusloeser((ZustandSammlung)cell.getElement()));
      }

		}); 
		
		TableColumn tcBenutzer = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcBenutzer.setText(COL_NAMES[IDX_COL_BENUTZER]);
		tcBenutzer.setWidth(100);
		
		TableColumn tcUrsache = new TableColumn(_tv.getTable(), SWT.LEFT);
		tcUrsache.setText(COL_NAMES[IDX_COL_URSCHE]);
		tcUrsache.setWidth(150);

		verankereKontextMenu();

		table.addMouseListener(new MouseAdapter()
		{

			@Override
			public void mouseDoubleClick(MouseEvent e)
			{
				if (e.button == 1)
				{
					Action actionAbschnittSchalten = getActionAbschnittSchalten();
					actionAbschnittSchalten.run();
				}
			}

		});
	}

	private void updateTable()
	{
		LinkedList<ZustandSammlung> lZustandSteuerungSammlungen = new LinkedList<ZustandSammlung>();

		synchronized (_map)
		{
			Collection<ZustandSammlung> values = _map.values();
			for (ZustandSammlung zustandSteuerungSammlung : values)
			{
				lZustandSteuerungSammlungen.add(zustandSteuerungSammlung);
			}
		}

		final ZustandSammlung[] aSammlungen = lZustandSteuerungSammlungen
				.toArray(new ZustandSammlung[lZustandSteuerungSammlungen.size()]);

		if (_tv != null)
		{
			Runnable run = new Runnable()
			{
				@Override
				public void run()
				{
					_tv.setInput(aSammlungen);
				}
			};

			PlatformUI.getWorkbench().getDisplay().asyncExec(run);
		}
	}

//	@Override
//	public void setFocus()
//	{
//	}
	
	@Override
	public void setFocus()
	{
	  //Fr die F1 Taste
	  if (_parent != null)
	    _parent.setFocus();
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt)
	{
		if (evt != null)
		{		 
		  
//		  System.out.println("ViewUebersichtSteuerung.propertyChange(): " + evt.getPropertyName() + " -> " + evt.getNewValue().getClass().getSimpleName());
		  
			if ((evt.getSource() != null)
					&& (evt.getSource() instanceof TSFBereichSchaltenVerwaltung))
			{
//			  System.out.println("--->propertyChange(1)");
				if ((evt.getPropertyName() != null)
						&& ((evt.getPropertyName()
								.equals(TSFBereichSchaltenVerwaltung.PROP_BEREICH_SCHALTEN_ADDED)) || (evt
								.getPropertyName()
								.equals(TSFBereichSchaltenVerwaltung.PROP_BEREICH_SCHALTEN_REMOVED)))){
//          System.out.println("--->propertyChange(00)");
				  if (_tv != null)
				  {
				    Runnable run = new Runnable()
				    {
				      
				      @Override
				      public void run()
				      {
				        _tv.refresh();
				      }
				    };
				    
				    PlatformUI.getWorkbench().getDisplay().asyncExec(run);
				  }
				}else{
//				  System.out.println("--->propertyChange(01)");
				}
			} else
			{
				if ((evt.getPropertyName().equals(GUIModelTSFSteuerung.PROP_INIT))
						&& (evt.getNewValue() instanceof ZustandSammlung[]))
				{
//	        System.out.println("--->propertyChange(2)");
	        
					ZustandSammlung[] aSammlungen = (ZustandSammlung[]) evt.getNewValue();
//					if (aSammlungen.length > 0)
					{
					  for (ZustandSammlung sammlung : aSammlungen)
					  {
					    if (sammlung.getSoTfsAbschnitt() != null)
					    {
					      synchronized (_map)
					      {
					        _map.put(sammlung.getSoTfsAbschnitt().getPid(), sammlung);
					      }
					    }
					  }					  
					}
					updateTable();
				} 				 
				else if ((evt.getPropertyName().equals(GUIModelTSFSteuerung.PROP_ADD))
						&& (evt.getNewValue() instanceof ZustandSammlung))
				{
//				  System.out.println("--->propertyChange(3)");
					ZustandSammlung sammlung = (ZustandSammlung) evt.getNewValue();
					if (sammlung.getSoTfsAbschnitt() != null)
					{
						synchronized (_map)
						{
							_map.put(sammlung.getSoTfsAbschnitt().getPid(), sammlung);							
						}

						updateTable();
					}
				}else{
//				  System.out.println("--->propertyChange(4)");				  
				}
			}
		}
	}

	private void listenerAnmelden()
	{
		DaVKomTSFSteuerung.getInstance().getModelZustand().addListenerAllgemein(
				this); //
		// DaVKomTSFSteuerung.getInstance().getModelVorgabe().addListenerAllgemein(this);

		TSFBereichSchaltenVerwaltung.getInstance().addListenerBereichschalten(this);
	}

	private void listenerAbmelden()
	{
		// DaVKomTSFSteuerung.getInstance().getModelVorgabe().removeListenerAllgemein(this);
		DaVKomTSFSteuerung.getInstance().getModelZustand().removeListenerAllgemein(
				this);//

		TSFBereichSchaltenVerwaltung.getInstance().removeListenerBereichschalten(
				this);
	}

	private GeneratorCompositeZustand[] getGeneratoren()
	{
		LinkedList<GeneratorCompositeZustand> liste = new LinkedList<GeneratorCompositeZustand>();

		synchronized (_mapCompositeZustaende)
		{
			Collection<GeneratorCompositeZustand> values = _mapCompositeZustaende
					.values();
			for (GeneratorCompositeZustand generatorCompositeZustand : values)
			{
				liste.add(generatorCompositeZustand);
			}
		}

		return liste.toArray(new GeneratorCompositeZustand[liste.size()]);
	}

	@Override
	public void dispose()
	{
		super.dispose();

		listenerAbmelden();

		/*
		 * Listener fr aller Generatoren abmelden
		 */
		GeneratorCompositeZustand[] aGeneratoren = getGeneratoren();
		for (GeneratorCompositeZustand gen : aGeneratoren)
		{
			gen.dispose();
		}

		if (_generatorCompositeWFSchaltempfehlungenUebersicht != null)
		{
			_generatorCompositeWFSchaltempfehlungenUebersicht.dispose();
		}
	}

	private class CPSteuerung implements IStructuredContentProvider
	{

		@Override
		public Object[] getElements(Object inputElement)
		{
			Object[] re = null;

			if ((inputElement != null) && (inputElement instanceof ZustandSammlung[]))
			{
				ZustandSammlung[] sammlung = (ZustandSammlung[]) inputElement;
				re = sammlung;
			}

			return re;
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput)
		{
		}

	}

	private class LPSteuerung implements ITableLabelProvider
	{

		@Override
		public Image getColumnImage(Object element, int columnIndex)
		{
			Image re = null;

			if ((element != null) && (element instanceof ZustandSammlung))
			{
				ZustandSammlung sammlung = (ZustandSammlung) element;

				switch (columnIndex)
				{
				case IDX_COL_ABSCHNITT:
					re = bestimmeImageSpalte1(sammlung);
					break;
				case IDX_COL_ZUSTAND_PRG:
					// if (sammlung.getProgrammZustand() != null)
					// {
					//
					// if (sammlung.getProgrammZustand() ==
					// E_TSFProgrammZustand.Geschaltet)
					// {
					// re = ImageGetter.getImage("icons/apply_16x16.png");
					// } else if (sammlung.getProgrammZustand() ==
					// E_TSFProgrammZustand.In_Bearbeitung)
					// {
					// re = ImageGetter.getImage("icons/remove_16x16.png");
					// }
					// }
					break;
				}
			}

			return re;
		}

		@Override
		public String getColumnText(Object element, int columnIndex)
		{
			String re = "";

			if ((element != null) && (element instanceof ZustandSammlung))
			{
				ZustandSammlung sammlung = (ZustandSammlung) element;

				switch (columnIndex)
				{
				case IDX_COL_ABSCHNITT:
					if (sammlung.getSoTfsAbschnitt() != null)
					{
						re = sammlung.getSoTfsAbschnitt().getNameOrPidOrId();
					}
					break;
				case IDX_COL_ZEITSTEMPEL:
					I_TSFZustand[] aZustaendeSteuerung = sammlung.getZustaendeSteuerung();
										
//					System.out.println("Zustaende: " + aZustaendeSteuerung.length);
					
					
					
					if ((aZustaendeSteuerung != null) && (aZustaendeSteuerung.length > 0))
					{
					  I_TSFZustand tsfZustandSteuerung = aZustaendeSteuerung[aZustaendeSteuerung.length - 1];					  
					  if (tsfZustandSteuerung != null){
					    re = Defs.SDF.format(tsfZustandSteuerung.getTimestamp());
					  }
					  else
            {
              re = "-";
            }
//					  long l = 0l;
//					  for (I_TSFZustand iTSFZustandSteuerung : aZustaendeSteuerung)
//            {
////					    I_TSFZustand tsfZustandSteuerung = aZustaendeSteuerung[aZustaendeSteuerung.length - 1];
//					    Date timestamp = iTSFZustandSteuerung.getTimestamp();					     
//					    if (timestamp.getTime() > l){
//					      l = timestamp.getTime();
//					    }
//              
//            }
//					  if (l > 0)
//					  {
//					    re = Defs.SDF.format(new Date(l));
//					  } else
//					  {
//					    re = "-";
//					  }
					} else
					{
						re = "-";
					}
					break;
				case IDX_COL_ZIELPROGRAMM:
					if (sammlung.getZielProgramm() != null)
					{
						re = sammlung.getZielProgramm().toString();
					}
					break;
				case IDX_COL_ZUSTAND_PRG:
					if (sammlung.getProgrammZustand() != null)
					{
						re = sammlung.getProgrammZustand().toString();
					}
					break;
				case IDX_COL_AKT_BEREICHSSCHALTUNG:
					boolean aktiveBereichsSchaltung = TSFBereichSchaltenVerwaltung
							.getInstance().isAbschnittInAktiverBereichsschaltung(
									sammlung.getSoTfsAbschnitt().getPid());
					if (aktiveBereichsSchaltung)
					{
						re = "aktiv";
					} else
					{
						re = "inaktiv";
					}
					break;
				case IDX_COL_ZUSTAND_STRG:
					// M_TSFZustandSteuerung[] zustaendeSteuerung = sammlung
					// .getZustaendeSteuerung();
					// if (zustaendeSteuerung != null)
					// {
					// if (zustaendeSteuerung.length > 0)
					// {
					// M_TSFZustandSteuerung zustandSteuerung =
					// zustaendeSteuerung[zustaendeSteuerung.length - 1];
					// if ((zustandSteuerung != null)
					// && (zustandSteuerung.getZustand() != null))
					// {
					// re = zustandSteuerung.getZustand().getText();
					// }
					// }
					// // zustandSteuerung.getStufe() //TODO JH
					// }

					re = bestimmeTextSpalteZustandSteuerung(sammlung);

					break;
				case IDX_COL_ZUSTAND_STRG_STUFE:
					// M_TSFZustandSteuerung[] zustaendeSteuerungStufe = sammlung
					// .getZustaendeSteuerung();
					// if (zustaendeSteuerungStufe != null)
					// {
					// if ((zustaendeSteuerungStufe.length - 1) >= 0)
					// {
					// M_TSFZustandSteuerung zustandStrg =
					// zustaendeSteuerungStufe[zustaendeSteuerungStufe.length - 1];
					// if (zustandStrg != null)
					// {
					// re = "" + zustandStrg.getStufe();
					// }
					// }
					// }
					re = bestimmeTextSpalteSteuerungStufe(sammlung);
					break;
//				case IDX_COL_ZUSTAND_ARGOSCAN:
//					re = bestimmeTextSpalteZustandArgoscan(sammlung);
//					break;
				case IDX_COL_AUSLOESER:
				  re = bestimmeTextAusloeser(sammlung);
				  break;
				case IDX_COL_BENUTZER:
				  re = bestimmeTextBenutzer(sammlung);
				  break;
				case IDX_COL_URSCHE:
				  re = bestimmeTextUrsache(sammlung);
				  break;
				}
			    
			}

			return re;
		}

		@Override
		public void addListener(ILabelProviderListener listener)
		{
		}

		@Override
		public void dispose()
		{
		}

		@Override
		public boolean isLabelProperty(Object element, String property)
		{
			return false;
		}

		@Override
		public void removeListener(ILabelProviderListener listener)
		{
		}

	}

	/**
	 * Bestimmt das kombinierte {@link Image} fr die erste Spalte.
	 * 
	 * @param sammlung
	 *          {@link ZustandSammlung}, fr die das {@link Image} der ersten
	 *          Spalte bestimmt werden soll.
	 * @return {@link Image} fr die erste Spalte oder <code>null</code>, falls
	 *         nicht bestimmbar.
	 */
	private static Image bestimmeImageSpalte1(ZustandSammlung sammlung)
	{
		Image re = null;

		if (sammlung != null)
		{
			if ((sammlung.getProgrammZustand() != null)
					&& (sammlung.getProgrammZustand() == E_TSFProgrammZustand.In_Bearbeitung))
			{
				re = ImageGetter.getImage("icons/exec_16x16.png");
			} else if (sammlung.getZielProgramm() != null)
			{
				if ((sammlung.getZielProgramm() == E_TSFBefehl.Freigabe)
						|| (sammlung.getZielProgramm() == E_TSFBefehl.FreigabeArgoscan)
						|| (sammlung.getZielProgramm() == E_TSFBefehl.Schnellfreigabe))
				{
					re = ImageGetter.getImage("icons/pfeil_gruen_unten_16x16.png");
				} else if ((sammlung.getZielProgramm() == E_TSFBefehl.Sperrung)
						|| (sammlung.getZielProgramm() == E_TSFBefehl.Schnellsperrung))
				{
					re = ImageGetter.getImage("icons/x_16x16.png");
				}
			}
		}

		return re;
	}

	/**
	 * Bestimmt den Text zur Darstellung in der Spalte
	 * <code>Zustand Steuerung</code>. Dieser wird nur dargestellt, wenn das
	 * Programm noch nicht geschaltet ist.
	 * 
	 * @param sammlung
	 * @return
	 */
	private static String bestimmeTextSpalteZustandSteuerung(
			ZustandSammlung sammlung)
	{
		String re = null;

		if (sammlung != null)
		{
			if ((sammlung.getProgrammZustand() != null)
					&& (sammlung.getProgrammZustand() != E_TSFProgrammZustand.Geschaltet))
			{

				I_TSFZustand[] aZustaende = sammlung.getZustaendeSteuerung();
				if (aZustaende != null)
				{
					if (aZustaende.length > 0)
					{

						for (int i = aZustaende.length - 1; i >= 0; i--)
						{
							I_TSFZustand zustand = aZustaende[i];

							if (zustand != null)
							{
								if (zustand instanceof M_TSFZustandSteuerung)
								{
									M_TSFZustandSteuerung zustandSteuerung = (M_TSFZustandSteuerung) zustand;
									if (zustandSteuerung.getZustand() != null)
									{
										re = zustandSteuerung.getZustand().getText();
									}
									break;
								}
							}
						}

					}
				}
			}
		}

		if (re == null)
		{
			re = "";
		}

		return re;
	}

	private static String bestimmeTextSpalteZustandArgoscan(
			ZustandSammlung sammlung)
	{
		String re = null;

		if (sammlung != null)
		{			
			if (sammlung.getProgrammZustand() != null)
			{
				I_TSFZustand[] aZustaende = sammlung.getZustaendeSteuerung();
				if ((aZustaende != null) && (aZustaende.length > 0))
				{
					for (int i = aZustaende.length - 1; i >= 0; i--)
					{
						I_TSFZustand zustand = aZustaende[i];

						if ((zustand != null) && (zustand instanceof M_TSFZustandArgoscan))
						{
							M_TSFZustandArgoscan zustandArgoscan = (M_TSFZustandArgoscan) zustand;
							if (zustandArgoscan.getZustand() != null)
							{
								re = zustandArgoscan.getZustand().getText()
										+ ", Makro/Ereignis=" + zustandArgoscan.getMakroEreignis();
							}

							break;
						}
					}
				}
			}
		}

		if (re == null)
		{
			re = "";
		}

		return re;
	}
	
	private static String bestimmeTextUrsache(
      ZustandSammlung sammlung)
  {
    String re = null;

    if (sammlung != null)
    {     
      if (sammlung.getProgrammZustand() != null)
      {
        
        SystemObject so = sammlung.getSoTfsAbschnitt();
        ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
        if (connection != null){
          AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaTsfAbschnittBefehl");
          Aspect asp = connection.getDataModel().getAspect("asp.sswSbaTsfVorgabeZustand");
          ResultData result = connection.getData(so, new DataDescription(atg, asp), 1000L);
          if (result != null && result.hasData()){
            re = result.getData().getItem("Urlasser").getItem("Ursache").asTextValue().getValueText();
          }
        }
      }
    }

    if (re == null)
    {
      re = "";
    }

    return re;
  }
	
	private static String bestimmeTextBenutzer(
	    ZustandSammlung sammlung)
	{
	  String re = null;
	  
	  if (sammlung != null)
	  {     
	    if (sammlung.getProgrammZustand() != null)
	    {
	      
	      SystemObject so = sammlung.getSoTfsAbschnitt();
	      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
	      if (connection != null){
	        AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaTsfAbschnittBefehl");
	        Aspect asp = connection.getDataModel().getAspect("asp.sswSbaTsfVorgabeZustand");
	        ResultData result = connection.getData(so, new DataDescription(atg, asp), 1000L);
	        if (result != null && result.hasData()){
	          ReferenceValue ref = result.getData().getItem("Urlasser").getItem("BenutzerReferenz").asReferenceValue();
	          if (ref != null)
            {
              SystemObject systemObject = ref.getSystemObject();
              if (systemObject != null)
                re = systemObject.getNameOrPidOrId();
            }
	        }
	      }
	    }
	  }
	  
	  if (re == null)
	  {
	    re = "";
	  }
	  
	  return re;
	}
	
	private static String bestimmeTextAusloeser(
	    ZustandSammlung sammlung)
	{
	  String re = null;
	  
	  if (sammlung != null)
	  {     
	    if (sammlung.getProgrammZustand() != null)
	    {	      
	      SystemObject so = sammlung.getSoTfsAbschnitt();
	      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
	      if (connection != null){
	        SystemObject ue = DaVKomTSFSteuerung.holeUrsachenEinheitVonAbschnitt(so, connection);	        
	        AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaTsfUeInputAnalyse");
	        Aspect asp = connection.getDataModel().getAspect("asp.analyse");
	        
	        
	        ResultData result = connection.getData(ue, new DataDescription(atg, asp), 1000L);
	        if (result != null && result.hasData()){
	          Array array = result.getData().getItem("Ursacheneinheiten").asArray();	          
	          
	          re = "inaktiv";	          
	          
	          for (int i = 0; i < array.getLength(); i++)
            {
	            String input = array.getItem(i).getItem("alsInputVerwenden").asTextValue().getValueText();
              String aktiv = array.getItem(i).getItem("aktiv").asTextValue().getValueText();              
              if (aktiv.equals("Ja") && input.equals("Ja")){
                re = "aktiv";
                break;
              }             
            }	          
	        }
	      }
	    }
	  }
	  
	  if (re == null)
	  {
	    re = "";
	  }
	  
	  return re;
	}
	
	private static String bestimmeToolTipTextAusloeser(
	    ZustandSammlung sammlung)
	{
	  String re = null;
	  
	  if (sammlung != null)
	  {     
	    if (sammlung.getProgrammZustand() != null)
	    {
	      
	      SystemObject so = sammlung.getSoTfsAbschnitt();
	      ClientDavInterface connection = DaVVerbindung.getInstanz().getDaVVerbindung();
	      if (connection != null){
	        SystemObject ue = DaVKomTSFSteuerung.holeUrsachenEinheitVonAbschnitt(so, connection);	        
	        AttributeGroup atg = connection.getDataModel().getAttributeGroup("atg.sswSbaTsfUeInputAnalyse");
	        Aspect asp = connection.getDataModel().getAspect("asp.analyse");
	        ResultData result = connection.getData(ue, new DataDescription(atg, asp), 1000L);
	        
	         
	        if (result != null && result.hasData()){
	          Array array = result.getData().getItem("Ursacheneinheiten").asArray();
	          
	          re = "Auslsende UrsachenEinheit(en): \n";
	          
            for (int i = 0; i < array.getLength(); i++)
            {
              String name = array.getItem(i).getItem("Ursacheneinheit").asReferenceValue().getSystemObject().getName();
              String aktiv = array.getItem(i).getItem("aktiv").asTextValue().getValueText();
              String tmp = array.getItem(i).getItem("alsInputVerwenden").asTextValue().getValueText();
              String inputNeu = tmp.equals("Nein") ? "nicht verwendet" : "verwendet";
              re += "  " + name + " : " + aktiv + " (" + inputNeu + ")";
              if (i < array.getLength()-1)
                re += "\n";
            }
	        }
	      }
	    }
	  }
	  
	  if (re == null)
	  {
	    re = "";
	  }
	  
	  return re;
	}

	/**
	 * Bestimmt den Text fr die Spalte <b>Steuerung Stufe.</b> Dieser wird nur
	 * dargestellt, wenn das jeweilige Programm noch nicht geschaltet ist.
	 * 
	 * @param sammlung
	 * @return
	 */
	private static String bestimmeTextSpalteSteuerungStufe(
			ZustandSammlung sammlung)
	{
		String re = null;

		if (sammlung != null)
		{
			if ((sammlung.getProgrammZustand() != null)
					&& (sammlung.getProgrammZustand() != E_TSFProgrammZustand.Geschaltet))
			{

				I_TSFZustand[] aZustaende = sammlung.getZustaendeSteuerung();
				if (aZustaende != null)
				{
					if ((aZustaende.length - 1) >= 0)
					{
						I_TSFZustand zustand = aZustaende[aZustaende.length - 1];

						if (zustand instanceof M_TSFZustandSteuerung)
						{
							M_TSFZustandSteuerung zustandSteuerung = (M_TSFZustandSteuerung) zustand;
							if (zustandSteuerung != null)
							{
								re = "" + zustandSteuerung.getStufe();
							}

							// TODO prfen ob fr anderes notwendig
						}
					}
				}
			}
		}

		if (re == null)
		{
			re = "";
		}

		return re;
	}

	/**
	 * Verankert das Kontextmen an der Tabelle mit der bersicht der offenen
	 * Schaltempfehlungen.
	 * 
	 */
	private void verankereKontextMenu()
	{
		if (_tv != null)
		{
			MenuManager menuMgr = new MenuManager("#PopupMenu");
			menuMgr.setRemoveAllWhenShown(true);
			menuMgr.addMenuListener(new IMenuListener()
			{
				public void menuAboutToShow(IMenuManager manager)
				{
					fuelleKontextMenu(manager);
				}
			});

			Menu menu = menuMgr.createContextMenu(_tv.getControl());
			_tv.getControl().setMenu(menu);

			this.getSite().registerContextMenu(menuMgr, _tv);

		}
	}

	/**
	 * Fllt das Kontextmen.
	 * 
	 * @param manager
	 *          {@link IMenuManager}, der fr die Darstellung des Kontextmens
	 *          zustndig ist.
	 */
	private void fuelleKontextMenu(IMenuManager manager)
	{

		ISelection selection = _tv.getSelection();
		if (selection instanceof StructuredSelection)
		{
			StructuredSelection ss = (StructuredSelection) selection;
			if (ss.size() > 0)
			{
				manager.add(getActionAbschnittSchalten());

			}
		}

		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private Action getActionAbschnittSchalten()
	{
		if (_aAbschnittSchalten == null)
		{
			_aAbschnittSchalten = new Action("Abschnitt &schalten")
			{

				@Override
				public void run()
				{
					// System.out.println("...Abschnitt schalten...");

					ISelection selection = _tv.getSelection();
					if (selection instanceof StructuredSelection)
					{
						StructuredSelection ss = (StructuredSelection) selection;
						Object firstElement = ss.getFirstElement();
						if ((firstElement != null)
								&& (firstElement instanceof ZustandSammlung))
						{
							ZustandSammlung sammlung = (ZustandSammlung) firstElement;
							SystemObject soAbschnitt = sammlung.getSoTfsAbschnitt();
							if (soAbschnitt.getPid() != null)
							{
								DialogSchaltenGuiManager.getInstance().openDialogSchalten(
										soAbschnitt.getPid());
							}
						}
					}
				}

			};
			_aAbschnittSchalten.setId(ViewUebersichtSteuerung.class.getName()
					+ ".ActionAbschnittSchalten");
			_aAbschnittSchalten.setImageDescriptor(ImageGetter
					.getImageDescriptor("icons/system-shutdown_16x16.png"));
		}
		return _aAbschnittSchalten;
	}
	
	
	 private static class FancyToolTipSupport extends ColumnViewerToolTipSupport {

	    protected FancyToolTipSupport(ColumnViewer viewer, int style,
	        boolean manualActivation) {
	      super(viewer, style, manualActivation);
	    }
	    
	    
	    protected Composite createToolTipContentArea(Event event,
	        Composite parent) {
	      Composite comp = new Composite(parent,SWT.NONE);
	      GridLayout l = new GridLayout(1,false);
	      l.horizontalSpacing=0;
	      l.marginWidth=0;
	      l.marginHeight=0;
	      l.verticalSpacing=0;
	      
	      comp.setLayout(l);
	      Browser browser = new Browser(comp,SWT.BORDER);
	      browser.setText(getText(event));
	      browser.setLayoutData(new GridData(200,150));
	      
	      
	      return comp;
	    }

	    public boolean isHideOnMouseDown() {
	      return false;
	    }


	    public static final void enableFor(ColumnViewer viewer, int style) {
	      new FancyToolTipSupport(viewer,style,false);
	    }
	  }
}
